/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package org.spectrim.automaticprocessing;

import org.spectrim.imagegraphicobjects.Point3D;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.IOException;
import java.nio.file.FileVisitResult;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.SimpleFileVisitor;
import java.nio.file.attribute.BasicFileAttributes;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;
import java.util.Vector;
import java.util.concurrent.ConcurrentHashMap;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.JOptionPane;
import javax.swing.SwingWorker;
import org.dcm4che2.data.DicomObject;
import org.spectrim.io.dicom.DicomHandler;
import org.spectrim.io.dicom.DicomHeader;
import org.spectrim.batchprocessing.BatchProcessing;
import org.spectrim.settings.ProcessingSettings;
import org.spectrim.settings.SettingsGUI;
import org.spectrim.settings.SettingsGUIListener;
import org.spectrim.settings.processing.PreprocessingAction;
import org.spectrim.settings.processing.ProcessingAction;
import org.spectrim.settings.processing.QuantificationAction;
import org.spectrim.settings.processing.QuantificationTypeSelectionFrame;

/**
 * This class processes the files which need to be processed as defined by the user.
 * @author user
 */
public class AutomaticProcessingFileHandlerWorker extends SwingWorker<Integer, String> 
{

    private final AutomaticProcessingFileHandler parent;
    private final ArrayList<String[]> worklist = new ArrayList<>( );
    private boolean isPaused = false;
    private final Map<File, List<String>> spectroscopyFilesToProcess = new ConcurrentHashMap<>( );
    private Vector<ActionListener> m_actionListeners = new Vector<ActionListener>( );
    
    public final static int IS_PROCESSING = 0;
    public final static int IS_PERFORMED  = 1;
    
    
    public AutomaticProcessingFileHandlerWorker( AutomaticProcessingFileHandler parent ) 
    {
        this.parent = parent;
    }

    public final void pause( ) 
    {
        if ( !isPaused && !isDone( ) ) 
        {
            isPaused = true;
            firePropertyChange( "paused", false, true );
        }
    }

    public final void resume( ) 
    {
        if ( isPaused && !isDone( ) ) 
        {
            isPaused = false;
            firePropertyChange( "paused", true, false );
        }

    }

    public final void addSpectroscopyFileToProcess( File newfile, List<String> refUIDS ) 
    {
        spectroscopyFilesToProcess.put( newfile, refUIDS );
    }

    public final boolean isPaused( ) 
    {
        return isPaused;
    }

    
    /**
     * This method performs all processing requested by the user in separate
     * threads.
     * @return
     * @throws Exception 
     */
    @Override
    protected Integer doInBackground( ) throws Exception 
    {
        while ( !spectroscopyFilesToProcess.isEmpty( ) ) 
        {
            if ( isPaused( ) ) 
            {
                Thread.sleep( 200 );
            } 
            else 
            {
                for ( File file : spectroscopyFilesToProcess.keySet( ) ) 
                {
                    final Map<String, String> instanceUIDsAvailable = new HashMap<>( );
                    File studyFolder = file.getParentFile( ).getParentFile( );
                    try {
                        Files.walkFileTree( studyFolder.toPath( ), new SimpleFileVisitor<Path>( ) 
                        {
                            @Override
                            public FileVisitResult visitFile( Path file, BasicFileAttributes attrs ) throws IOException 
                            {
                                if ( file.toFile( ).toString( ).endsWith( ".dcm" ) || file.toFile( ).toString( ).endsWith( ".DCM" )
                                  || file.toFile( ).toString( ).endsWith( ".ima" ) || file.toFile( ).toString( ).endsWith( ".IMA" ) 
                                  || file.toFile( ).toString( ).endsWith( ".cdm" ) ) 
                                {
                                    DicomObject dcmObj = DicomHandler.getDicomObject( file.toFile( ).toString( ) );
                                    if ( dcmObj.contains( Integer.parseInt( "00080018", 16 ) ) ) 
                                    {
                                        instanceUIDsAvailable.put( dcmObj.getString( Integer.parseInt( "00080018", 16 ) ), file.toFile( ).toString( ) );
                                    }
                                }
                                return FileVisitResult.CONTINUE;
                            }
                        });
                    } catch ( IOException ex ) 
                    {
                        Logger.getLogger( AutomaticProcessingFileHandler.class.getName( ) ).log( Level.SEVERE, null, ex );
                    }
                    ArrayList<String> fileRefUIDs = (ArrayList<String>) spectroscopyFilesToProcess.get( file );
                    boolean allImagesAvailable = true;
                    for ( String uid : fileRefUIDs ) 
                    {
                        if ( !instanceUIDsAvailable.containsKey( uid ) ) 
                        {
                            allImagesAvailable = false;
                            break;
                        }
                    }
                    if ( allImagesAvailable ) 
                    {
                        //TO DO: process data
                        spectroscopyFilesToProcess.remove( file );
                        String[] filesToProcess = new String[]{ file.toString( ), 
                                                                instanceUIDsAvailable.get( fileRefUIDs.get( 0 ) ),
                                                                instanceUIDsAvailable.get( fileRefUIDs.get( 1 ) ), 
                                                                instanceUIDsAvailable.get( fileRefUIDs.get( 2 ) ) };
                        worklist.add( filesToProcess );
                    }
                }
                while ( !worklist.isEmpty( ) ) 
                {
                    if ( BatchProcessing.isBusy( ) ) 
                    {
                        Thread.sleep( 200 );
                    } 
                    else 
                    {
                        String[] filesToProcess = worklist.get( 0 );
                        DicomObject obj0 = DicomHandler.getDicomObject( filesToProcess[0] );
                        DicomHeader header = DicomHandler.getHeader( obj0 );
                        DicomHandler.readSiemensData( obj0, header );

                        double vx = Double.valueOf( header.get( "sSpecPara.sVoI.sNormal.dSag" ).Value );
                        double vy = Double.valueOf( header.get( "sSpecPara.sVoI.sNormal.dCor" ).Value );
                        double vz = Double.valueOf( header.get( "sSpecPara.sVoI.sNormal.dTra" ).Value );
                        Point3D p0 = new Point3D( vx, vy, vz );

                        DicomObject obj1 = DicomHandler.getDicomObject( filesToProcess[1] );
                        DicomObject obj2 = DicomHandler.getDicomObject( filesToProcess[2] );
                        DicomObject obj3 = DicomHandler.getDicomObject( filesToProcess[3] );

                        Point3D p1 = DicomHandler.getNormalVector( obj1 );
                        Point3D p2 = DicomHandler.getNormalVector( obj2 );
                        Point3D p3 = DicomHandler.getNormalVector( obj3 );

                        p0.normalize( );
                        p1.normalize( );
                        p2.normalize( );
                        p3.normalize( );

                        double iprod1 = Point3D.dotProduct( p0, p1 );
                        double iprod2 = Point3D.dotProduct( p0, p2 );
                        double iprod3 = Point3D.dotProduct( p0, p3 );

                        TreeMap<Double, String> mapAux = new TreeMap<>( );
                        mapAux.put( iprod1, filesToProcess[1] );
                        mapAux.put( iprod2, filesToProcess[2] );
                        mapAux.put( iprod3, filesToProcess[3] );
                        ArrayList<Double> ordered = new ArrayList<>( mapAux.keySet( ) );

                        BatchProcessing.addOpenSpectroscopyFileCommand( filesToProcess[0] );
                        BatchProcessing.addOpenImageFileCommand( mapAux.get( ordered.get( 2 ) ) );
                        BatchProcessing.addOpenReferencesCommand( mapAux.get( ordered.get( 2 ) ), 
                                                                  mapAux.get( ordered.get( 1 ) ), 
                                                                  mapAux.get( ordered.get( 0 ) ) );
                        
                        BatchProcessing.addSelectionCommand( BatchProcessing.SELECT_EXCITEDVOL );

                        double B0 = Double.valueOf( header.get( "sProtConsistencyInfo.flNominalB0" ).Value );
                        if ( B0 == 0 ) 
                        {
                            B0 = Double.valueOf( header.get( "sTXSPEC.flKDynMagnitudeMax" ).Value );
                        }
                        double TE = Double.valueOf( header.get( "alTE[0]" ).Value ) / 1000;
                        BatchProcessing.addAutoPreprocessingCommand( B0, TE );
                        
                        QuantificationAction qAct = SettingsGUI.getInstance( null ).getSelectedQuantificationAction( );
                        
                        BatchProcessing.addQuantification( qAct );
                        
                        BatchProcessing.addAutoPhasing( 0.0f, 4.5f );
                        File mrsFile = new File( filesToProcess[0] );
                        String reportFile = mrsFile.getParent( ) + System.getProperty( "file.separator" ) + "stdReport.png";
                        BatchProcessing.addCreateReportCommand( reportFile, 0, null );
                        BatchProcessing.run( );
                        worklist.remove( 0 );
                    }
                }
            }
        }
        return 1;
    }
        
    @Override
    protected synchronized void done() {
        super.done();
        parent.workerFinished();
        notifyActionListeners( this, IS_PERFORMED, "Processing performed" );
    }

        
    /**
     * Adds an ActionListener to the list of listeners
     * @param listener
    */
    public void addActionListener( ActionListener listener )
    {
        m_actionListeners.add( listener );
    }
    
    /**
     * Notifies actionEvent to all registered listeners
     */
    private void notifyActionListeners( Object obj, int eventType, String actionCommand ) 
    {
        Iterator it = m_actionListeners.iterator( );
        while ( it.hasNext( ) ) 
        {
            ((ActionListener)it.next( ) ).actionPerformed(new ActionEvent(obj, eventType, actionCommand));
        }
    }
}
