/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package org.spectrim.batchprocessing;

import java.util.ArrayList;
import javax.swing.SwingWorker;

/**
 *
 * @author user
 */
class BatchProcessingWorker extends SwingWorker<Void, String> 
{

    private boolean isPaused = false;
    ArrayList<String> commands = new ArrayList<>();
    private static final String separator = "€€€€####";

    //File handling commands
    private static final String OPEN_BCKGND_IMG = "OPEN_IMG";
    private static final String OPEN_SPECTROSCOPY = "OPEN_MRS";
    private static final String OPEN_REFERENCES = "OPEN_REFS";

    //Other
    private static final String CREATE_REPORT = "REPORT";
    private static final String SAVE_RESULTS = "SAVE_RES";
    private static final String SAVE_METMAPS = "SAVE_METMAPS";
    private static final String SELECTION = "SELECT";

    //Preprocessing commands
    private static final String APODIZATION             = "APO";
    private static final String FREQSHIFT               = "FREQSHIFT";
    private static final String AUTOPHASE               = "AUTOPHASE";
    private static final String WATERREMOVAL_HLSVDPro   = "WATERREMOVAL_HLSVDPro";
    private static final String WATERREMOVAL_HLSVD64    = "WATERREMOVAL_HLSVD64";
    private static final String WATERREMOVAL_TDFDFIT    = "WATERREMOVAL_TDFDFIT";
    private static final String QUEST                   = "QUEST";
    private static final String TDFDFIT                 = "TDFDFIT";
    private static final String QA                      = "QA";
    private static final String BASELINE                = "BASELINE";
    
    private static final String AMPLITUDE_OFFSET_REMOVAL_FD = "AMPLITUDE_OFFSET_REMOVAL_FD";
    private static final String AMPLITUDE_OFFSET_REMOVAL_TD = "AMPLITUDE_OFFSET_REMOVAL_TD";
    
    private final BatchProcessingTaskExecutor taskExecutor;

    public BatchProcessingWorker( BatchProcessingTaskExecutor instance ) 
    {
        taskExecutor = instance;
    }

    public void addOpenImageFileCommand( String file ) 
    {
        commands.add( OPEN_BCKGND_IMG + separator + file );
    }

    public void addOpenSpectroscopyFileCommand( String file ) 
    {
        commands.add( OPEN_SPECTROSCOPY + separator + file );
    }

    public void addOpenReferencesCommand( String file1, String file2, String file3 ) 
    {
        commands.add( OPEN_REFERENCES + separator + file1 + separator + file2 + separator + file3 );
    }

    public void addCreateReportCommand(String outputFolder, int outputFileType, String customReportFile) 
    {
        commands.add( CREATE_REPORT + separator + outputFolder + separator + outputFileType + separator + customReportFile );
    }

    public void addSaveResultsCommand(String outputFolder) 
    {
        commands.add( SAVE_RESULTS + separator + outputFolder );
    }

    public void addSaveMetaboliteMapsCommand( String outputFolder, boolean profileCorrection ) 
    {
        commands.add( SAVE_METMAPS + separator + outputFolder + separator + ( profileCorrection?"1":"0" ) );
    }

    public void addSelectionCommand( int selectMode ) 
    {
        commands.add( SELECTION + separator + selectMode );
    }

    public void addAutoPreprocessingCommand( double B0, double TE ) 
    {
//DEFINE WITH HANS NEW IMPLEMENTATION
//        ArrayList<PreProcessingOptionsGroup> preprocGroups = PreProcessingSettings.getInstance().getPreProcessingGroups();
//        PreProcessingOptionsGroup preproGroup = null;
//        for (int i = 0; i < preprocGroups.size(); i++) {
////            PreProcessingOptionsGroup p = preprocGroups.get(i);
////            if (p.getTE() == TE && Math.abs(p.getFieldStrength()-B0)<0.1) {
////                preproGroup = p;
////                break;
////            }
//        }
//        if (preproGroup != null) {
//            int[] order = preproGroup.getPanelOrder();
//
//            // add the different preprocessing steps depending on the choice
//            for (int i = 0; i < order.length; i++) {
//                for (int j = 0; j < order.length; j++) {
//                    if (i == order[j]) {
//                        switch (j) {
//                            case (0): {
//                                if (preproGroup.isAppodisationUse()) {
//                                    addApodization(preproGroup.getAppodisationType(), preproGroup.getAppodisationLineBroadning());
//                                }
//                                break;
//                            }
//                            case (1): {
//                                if (preproGroup.isQualityUse()) {
//                                    addQualityFiltering(preproGroup.getQualityRFile(), preproGroup.getQualityThreshold());
//                                }
//                                break;
//                            }
//                            case (2): {
//                                if (preproGroup.isBaselineUse()) {
//                                    addBaselineCorrection(preproGroup.getBaselineNR());
//                                }
//                                break;
//                            }
//                            case (3): {
//                                if (preproGroup.isWaterRemovalUse()) {
//                                    addWaterPeakRemoval(preproGroup.getWaterRemovalMin(), preproGroup.getWaterRemovalMax(), preproGroup.getWaterRemovalNumberOfPeaks());
//                                }
//                                break;
//                            }
//                            case (4): {
//                                if (preproGroup.isAutoPhasingUse()) {
//                                    addAutoPhasing((float) preproGroup.getAutoPhasingMinPPm(), (float) preproGroup.getAutoPhasingMaxPPm());
//                                }
//                                break;
//                            }
//                            case (5): {
//                                if (preproGroup.isFreqShiftUse()) {
//                                    int[] multiplicities = new int[preproGroup.getFreqShiftSettings().get(3).length];
//                                    int k = 0;
//                                    for (double val : preproGroup.getFreqShiftSettings().get(3)) {
//                                        multiplicities[k++] = (int) val;
//                                    }
//                                    addFrequencyShiftCorrection(preproGroup.getFreqShiftSettings().get(0), preproGroup.getFreqShiftSettings().get(1), preproGroup.getFreqShiftSettings().get(2), multiplicities, preproGroup.getFreqShiftSNR(), preproGroup.getFreqShiftMinPPM(), preproGroup.getFreqShiftMaxPPM(), preproGroup.getFreqShiftMaxLocalShift());
//                                }
//                                break;
//                            }
//                        }
//                        break;
//                    }
//                }
//            }
//        } else {
//            taskExecutor.publish("No valid pre-processing group found for " + B0 + "T and TE=" + TE);
//        }
    }

    public void addApodization( int apoMethd, double factor ) 
    {
        commands.add( "APO" + separator + apoMethd + separator + factor );
    }

    public void addFrequencyShiftCorrection( double[] minvals, double[] maxvals, 
                                             double[] targets, int[] multiplicities, 
                                             double SNRLimit, double minPPM, 
                                             double maxPPM, double maxLocalShift ) 
    {
        String cmd = "FREQSHIFT" + separator + minvals.length;
        for ( int i = 0; i < minvals.length; i++ ) 
        {
            cmd += separator + minvals[i];
        }
        for ( int i = 0; i < minvals.length; i++ ) 
        {
            cmd += separator + maxvals[i];
        }
        for ( int i = 0; i < minvals.length; i++ ) 
        {
            cmd += separator + targets[i];
        }
        for ( int i = 0; i < minvals.length; i++ ) 
        {
            cmd += separator + multiplicities[i];
        }
        cmd += separator + SNRLimit;
        cmd += separator + minPPM;
        cmd += separator + maxPPM;
        cmd += separator + maxLocalShift;

        commands.add( cmd );
    }

    public void addAutoPhasing( float minppm, float maxppm ) 
    {
        commands.add( "AUTOPHASE" + separator + minppm + separator + maxppm );
    }
    
    public void addWaterPeakRemovalHLSVDPro( double min, double max, int nrpeaks ) 
    {
        commands.add( "WATERREMOVAL_HLSVDPro" + separator + min + separator + max + separator + nrpeaks );
    }

    public void addWaterPeakRemovalHLSVD64( double min, double max, int nrpeaks ) 
    {
        commands.add( "WATERREMOVAL_HLSVD64" + separator + min + separator + max + separator + nrpeaks );
    }

    public void addWaterPeakRemovalTDFDFit( String tdfdfitWaterModelName ) 
    {
        commands.add( "WATERREMOVAL_TDFDFIT" + separator +  tdfdfitWaterModelName );
    }
    
    void addAmplitudeOffsetRemovalFD( float minAmplitudeOffsetFD, float maxAmplitudeOffsetFD ) 
    {
        commands.add( "AMPLITUDE_OFFSET_REMOVAL_FD" + 
                      separator +  minAmplitudeOffsetFD + 
                      separator +  maxAmplitudeOffsetFD );
    }

    void addAmplitudeOffsetRemovalTD(float minAmplitudeOffsetTD, float maxAmplitudeOffsetTD ) 
    {
        commands.add( "AMPLITUDE_OFFSET_REMOVAL_TD" + separator + 
                      minAmplitudeOffsetTD + separator +  
                      maxAmplitudeOffsetTD );
    }
    
    public void addQuantification( ) 
    {
        commands.add( "QUEST" );
    }

    public void addQuantificationTDFDFit(  String tdfdfitModelName, String tdfdfitPathString ) 
    {
        commands.add( "TDFDFIT" + separator + tdfdfitPathString + separator + tdfdfitModelName );
    }
    
    
    public void addQualityFiltering( String file, double threshold ) 
    {
        commands.add( "QA" + separator + file + separator + threshold );
    }

    public void addBaselineCorrection( int nrPoints ) 
    {
        commands.add( "BASELINE" + separator + nrPoints );
    }

    public final void pause( ) 
    {
        if ( !isPaused && !isDone( ) ) 
        {
            isPaused = true;
            firePropertyChange( "paused", false, true );
        }
    }

    public final void resume( ) 
    {
        if ( isPaused && !isDone( ) ) 
        {
            isPaused = false;
            firePropertyChange( "paused", true, false );
        }
    }

    public final boolean isPaused( ) 
    {
        return isPaused;
    }

    @Override
    protected synchronized Void doInBackground( ) throws Exception 
    {
        int nrcmds = commands.size( );
        int i = 0;
        while ( !commands.isEmpty( ) ) 
        {
            if ( isPaused( ) ) 
            {
                Thread.sleep( 200 );
            } 
            else 
            {
                i++;
                String cmd = commands.get( 0 );
                commands.remove( 0 );
                String[] parts = cmd.split( separator );   
                switch ( parts[0] ) 
                {
                    case OPEN_BCKGND_IMG:
                                            pause( );
                                            taskExecutor.printTxt( i + "/" + nrcmds + " Opening Background Image..." );
                                            taskExecutor.openFile( parts[1] );
                                            break;
                    case OPEN_SPECTROSCOPY:
                                            pause( );
                                            taskExecutor.printTxt( i + "/" + nrcmds + " Opening Spectroscopy File..." );
                                            taskExecutor.openFile( parts[1] );
                                            break;
                    case OPEN_REFERENCES:
                                            taskExecutor.printTxt( i + "/" + nrcmds + " Opening Reference Images..." );
                                            taskExecutor.openReferenceImages(parts[1], parts[2], parts[3]);
                                            break;
                    case APODIZATION:
                                            System.out.println( "Applying : " + APODIZATION );
                                            taskExecutor.printTxt( i + "/" + nrcmds + " Performing Apodization..." );
                                            int apodizationMethod = Integer.valueOf( parts[1] );
                                            double factor = Double.valueOf( parts[2] );
                                            if ( apodizationMethod == BatchProcessing.APO_EXP ) 
                                            {
                                                taskExecutor.apodizeExponentialSelection( factor, false );
                                            } 
                                            else if ( apodizationMethod == BatchProcessing.APO_GAUSS ) 
                                            {
                                                taskExecutor.apodizeGaussianSelection( factor, false );
                                            }
                                            break;
                    case FREQSHIFT:
                                            System.out.println( "Applying : " + FREQSHIFT );
                                            pause( );
                                            taskExecutor.printTxt( i + "/" + nrcmds + " Performing Frequency Shift Correction..." );
                                            int size = Integer.valueOf( parts[1] );
                                            double[] freqShiftMin = new double[size];
                                            double[] freqShiftMax = new double[size];
                                            double[] freqShiftTarget = new double[size];
                                            int[] freqShiftMultiplicity = new int[size];
                                            double freqShiftSNRLimit = Double.valueOf( parts[2 + 4 * size] );
                                            double freqShiftMinPPM = Double.valueOf( parts[3 + 4 * size] );
                                            double freqShiftMaxPPM = Double.valueOf( parts[4 + 4 * size] );
                                            double freqShiftMaxLocalShift = Double.valueOf( parts[5 + 4 * size] );

                                            for (int j = 0; j < size; j++) 
                                            {
                                                freqShiftMin[j] = Double.valueOf( parts[2 + j] );
                                                freqShiftMax[j] = Double.valueOf( parts[2 + j + size] );
                                                freqShiftTarget[j] = Double.valueOf( parts[2 + j + 2 * size] );
                                                freqShiftMultiplicity[j] = Integer.valueOf( parts[2 + j + 3 * size] );
                                            }

                                            taskExecutor.calcPPMshiftSelection( freqShiftMin, freqShiftMax, 
                                                                                freqShiftTarget, freqShiftMultiplicity, freqShiftSNRLimit,
                                                                                freqShiftMinPPM, freqShiftMaxPPM, 
                                                                                freqShiftMaxLocalShift, false );

                                            break;
                    case AUTOPHASE:         System.out.println( "Applying : " + AUTOPHASE );
                                            taskExecutor.printTxt( i + "/" + nrcmds + " Performing Auto-Phasing..." );
                                            float autoPhaseMinPPM = Float.valueOf( parts[1] );
                                            float autoPhaseMaxPPM = Float.valueOf( parts[2] );
                                            taskExecutor.autoPhaseSelection( autoPhaseMinPPM, autoPhaseMaxPPM, false );
                                            break;
                                            
                    case WATERREMOVAL_HLSVDPro:
                                            System.out.println( "Applying : " + WATERREMOVAL_HLSVDPro );
                                            pause( );
                                            taskExecutor.printTxt( i + "/" + nrcmds + " Performing Water Peak Removal..." );
                                            double lowerLimit_HLSVDPro = Double.valueOf( parts[1] );
                                            double upperLimit_HLSVDPro = Double.valueOf( parts[2] );
                                            int peaks_HLSVDPro = Integer.valueOf( parts[3] );
                                            taskExecutor.removeWaterHLSVDPro( lowerLimit_HLSVDPro, upperLimit_HLSVDPro, peaks_HLSVDPro );
                                            break;
                    
                    case WATERREMOVAL_HLSVD64:
                                            System.out.println( "Applying : " + WATERREMOVAL_HLSVD64 );
                                            pause( );
                                            taskExecutor.printTxt( i + "/" + nrcmds + " Performing Water Peak Removal..." );
                                            double lowerLimit = Double.valueOf( parts[1] );
                                            double upperLimit = Double.valueOf( parts[2] );
                                            int peaks = Integer.valueOf( parts[3] );
                                            taskExecutor.removeWaterHLSVD64( lowerLimit, upperLimit, peaks );
                                            break;
                        
                    case WATERREMOVAL_TDFDFIT:
                                            System.out.println( "Applying : " + WATERREMOVAL_TDFDFIT );
                                            pause( );
                                            taskExecutor.printTxt( i + "/" + nrcmds + " Performing Water Peak Removal using TDFDFit..." );
                                            String waterModelName = parts[1];
                                            taskExecutor.removeWaterTDFDFit( waterModelName );
                                            break;
                                            
                    case  AMPLITUDE_OFFSET_REMOVAL_FD:
                                            System.out.println( "Applying : " + AMPLITUDE_OFFSET_REMOVAL_FD );
                                            taskExecutor.printTxt( i + "/" + nrcmds + " Performing amplitude offset removal in FD ..." );
                                            double lowerLimitFD = Double.valueOf( parts[1] );
                                            double upperLimitFD = Double.valueOf( parts[2] );
                                            taskExecutor.removeAmplitudeOffsetFD( lowerLimitFD, upperLimitFD );
                                            break;
                                            
                    case  AMPLITUDE_OFFSET_REMOVAL_TD:
                                            System.out.println( "Applying : " + AMPLITUDE_OFFSET_REMOVAL_TD );
                                            taskExecutor.printTxt( i + "/" + nrcmds + " Performing amplitude offset removal in TD ..." );
                                            double lowerLimitTD = Double.valueOf( parts[1] );
                                            double upperLimitTD = Double.valueOf( parts[2] );
                                            taskExecutor.removeAmplitudeOffsetTD( lowerLimitTD, upperLimitTD );
                                            break;
                                            
                    case QUEST:
                                            System.out.println( "Applying : " + QUEST );
                                            pause( );
                                            taskExecutor.printTxt( i + "/" + nrcmds + " Performing Quantification (QUEST)..." );
                                            taskExecutor.quantifySpectra( );
                                            break;
                                            
                                            
                    case TDFDFIT:           System.out.println( "" );
                                            System.out.println( "Applying : " + TDFDFIT );
                                            pause( );
                                            taskExecutor.printTxt( i + "/" + nrcmds + " Performing Quantification (TDFDFIT)..." );
                                            String tdfdfitModelToUse = parts[1] + System.getProperty( "file.separator" ) + parts[2];
                                            taskExecutor.quantifyWithTDFDFitInAction( tdfdfitModelToUse );
                                            break;
                                            
                    case QA:                System.out.println( "Applying : " + QA );
                                            taskExecutor.printTxt( i + "/" + nrcmds + " Performing Quality Filtering..." );
                                            taskExecutor.assessQuality( parts[1] );
                                            double qualityThreshold = Double.valueOf( parts[2] );
                                            int nrSelected = taskExecutor.qualitySelection( qualityThreshold );
                                            if ( nrSelected == 0 ) 
                                            {
                                                taskExecutor.printTxt( "No spectra with QScores higher than " + qualityThreshold );
                                                commands.clear( );
                                            }
                                            break;
                    case BASELINE:
                                            taskExecutor.printTxt( i + "/" + nrcmds + " Performing Baseline Correction..." );
                                            taskExecutor.baselineCorrection( Integer.valueOf( parts[1] ) );
                                            break;
                    case SELECTION:
                                            taskExecutor.printTxt( i + "/" + nrcmds + " Performing voxel selection..." );
                                            switch ( Integer.valueOf( parts[1] ) ) 
                                            {
                                                case BatchProcessing.SELECT_ALL:
                                                    taskExecutor.selectAllVoxels( );
                                                    break;
                                                case BatchProcessing.SELECT_EXCITEDVOL:
                                                    taskExecutor.selectExcitedVoxels( );
                                                    break;
                                                case BatchProcessing.SELECT_INNEREXCITEDVOL:
                                                    taskExecutor.selectInnerExcitedVoxels( );
                                            }
                                            break;
                    case CREATE_REPORT:
                                            taskExecutor.printTxt( i + "/" + nrcmds + " Creating Report..." );
                                            taskExecutor.createReport( parts[1], Integer.valueOf( parts[2] ), parts[3] );
                                            break;
                    case SAVE_RESULTS:
                                            taskExecutor.printTxt( i + "/" + nrcmds + " Saving Results..." );
                                            taskExecutor.saveResults( parts[1] );
                                            break;
                    case SAVE_METMAPS:
                                            taskExecutor.printTxt( i + "/" + nrcmds + " Saving Metabolite Maps..." );
                                            taskExecutor.saveMetaboliteMapsMAT( parts[1], parts[2].trim( ).matches( "1" ) );
                                            break;
                }
            }
        }
        while ( isPaused( ) ) 
        {
            Thread.sleep( 200 );
        }
        return null;
    }

    @Override
    protected synchronized void done( ) 
    {
        super.done( );
        this.commands.clear( );
        BatchProcessing.workerFinished( );
    }
    
}