/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package org.spectrim.decisionforest.data;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;
import java.util.concurrent.ThreadLocalRandom;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 *
 * @author nunopbarros
 */
public class DataSet implements java.io.Serializable {

    public static Exception IncoherentVariableNumberException;
    public int nrvar;
    private HashMap<Integer, DataVect> data = new HashMap<>();
    private HashSet<Integer> outofbag = new HashSet<>();
    private ArrayList<Double> varMin = new ArrayList<>();
    private ArrayList<Double> varMax = new ArrayList<>();

    public DataSet() {
    }

    ;

    private DataSet(int nvar, HashMap<Integer, DataVect> data, HashSet<Integer> oob, ArrayList<Double> min, ArrayList<Double> max) {
        nrvar = nvar;
        Iterator<Integer> iter = data.keySet().iterator();
        while (iter.hasNext()) {
            Integer pos = iter.next();
            this.data.put(pos, data.get(pos).cloneVect());
        }
        this.outofbag = (HashSet<Integer>) oob.clone();
        this.varMax = max;
        this.varMin = min;
    }

    public int size() {
        return data.size();
    }

    public synchronized DataVect get(int i) {
        return data.get(i);
    }

    public void clear() {
        data.clear();
    }

    public boolean containsKey(Integer pos) {
        return data.containsKey(pos);
    }

    @Override
    public DataSet clone() {
        return new DataSet(nrvar, data, outofbag, varMin, varMax);
    }

    public void importtraindata(String file, int nrSkippedCols, boolean append) throws Exception {
        BufferedReader reader;
        String line;
        int linesize;
        DataVect datavec = new DataVect();
        String dataclass = null;
        double[] datavalues;
        int nrvar = 0;
        int linenr;
        if (append) {
            linenr = data.size();
        } else {
            data.clear();
            linenr = 0;
        }

//    System.out.println("Reading file... ("+file+"). Information found:");
        try {
            reader = new BufferedReader(new FileReader(file));
            while ((line = reader.readLine()) != null) {
                String[] parts = line.split("\\s");
                linesize = parts.length;
                if (linesize > 0) {
                    if (!parts[0].startsWith("*")) {
                        if (nrSkippedCols > 0) {
                            linesize = linesize - nrSkippedCols;
                            String[] partsaux = new String[parts.length - nrSkippedCols];
                            for (int i = 0; i < partsaux.length; i++) {
                                partsaux[i] = parts[i + nrSkippedCols];
                            }
                            parts = partsaux;
                        }
                        if (nrvar == 0) {
                            nrvar = linesize - 1;
                            if (!append || varMin.isEmpty()) {
                                for (int k = 0; k < nrvar; k++) {
                                    varMin.add(Double.POSITIVE_INFINITY);
                                    varMax.add(Double.NEGATIVE_INFINITY);
                                }
                            }
                        } else if (nrvar != (linesize - 1)) {
                            throw IncoherentVariableNumberException;
                        }
                        datavalues = new double[nrvar];
                        for (int i = 0; i < linesize; i++) {

                            if (!parts[i].isEmpty()) {

                                if (i == 0) {
//                        System.out.print("Classe "+parts[i]);
                                    dataclass = parts[i];
                                } else {
                                    if (i == (linesize - 1)) {
//                        System.out.println(" var"+i+" "+parts[i]);
                                        datavalues[i - 1] = Double.valueOf(parts[i]);
                                        datavec.setVect(dataclass, datavalues);
                                        data.put(linenr, (DataVect) datavec.cloneVect());
                                        linenr++;
                                    } else {
//                        System.out.print(" var"+i+" "+parts[i]);
                                        datavalues[i - 1] = Double.valueOf(parts[i]);
                                    }
                                    if (datavalues[i - 1] > varMax.get(i - 1)) {
                                        varMax.set(i - 1, datavalues[i - 1]);
                                    }
                                    if (datavalues[i - 1] < varMin.get(i - 1)) {
                                        varMin.set(i - 1, datavalues[i - 1]);
                                    }
                                }
                            }
                        }
                    }
                }
            }
        } catch (FileNotFoundException ex) {
            Logger.getLogger(DataSet.class.getName()).log(Level.SEVERE, null, ex);
        } catch (IOException ex) {
            Logger.getLogger(DataSet.class.getName()).log(Level.SEVERE, null, ex);
        }
        this.nrvar = nrvar;
    }

    public void importtestdata(String file, int nrSkippedCols) throws Exception {
        BufferedReader reader;
        String line;
        int linesize;
        DataVect datavec = new DataVect();
        String dataclass;
        double[] datavalues;
        int nrvar = 0;
        int linenr = 0;

//    System.out.println("Reading file... ("+file+"). Information found:");
        try {
            reader = new BufferedReader(new FileReader(file));
            while ((line = reader.readLine()) != null) {
                String[] parts = line.split("\\s");
                linesize = parts.length;
                if (linesize > 0) {
                    if (!parts[0].startsWith("*")) {
                        if (nrSkippedCols > 0) {
                            linesize = linesize - nrSkippedCols;
                            String[] partsaux = new String[parts.length - nrSkippedCols];
                            for (int i = 0; i < partsaux.length; i++) {
                                partsaux[i] = parts[i + nrSkippedCols];
                            }
                            parts = partsaux;
                        }
                        if (nrvar == 0) {
                            nrvar = linesize;
                            for (int k = 0; k < nrvar; k++) {
                                varMin.add(Double.POSITIVE_INFINITY);
                                varMax.add(Double.NEGATIVE_INFINITY);
                            }
                        } else if (nrvar != linesize && !parts[0].isEmpty()) {
                            throw IncoherentVariableNumberException;
                        }
                        datavalues = new double[nrvar];
                        for (int i = 0; i < linesize; i++) {
                            if (!parts[i].isEmpty()) {
                                if (i == (linesize - 1)) {
//                        System.out.println(" var"+(i+1)+" "+parts[i]);
                                    datavalues[i] = Double.valueOf(parts[i]);
                                    dataclass = "Unknown";
                                    datavec.setVect(dataclass, datavalues);
                                    data.put(linenr, (DataVect) datavec.cloneVect());
                                    linenr++;
                                } else {
//                        System.out.print(" var"+(i+1)+" "+parts[i]);
                                    datavalues[i] = Double.valueOf(parts[i]);
                                }

                            }
                            if (datavalues[i] > varMax.get(i)) {
                                varMax.set(i, datavalues[i]);
                            }
                            if (datavalues[i] < varMin.get(i)) {
                                varMin.set(i, datavalues[i]);
                            }
                        }
                    }
                }
            }
        } catch (FileNotFoundException ex) {
            Logger.getLogger(DataSet.class.getName()).log(Level.SEVERE, null, ex);
        } catch (IOException ex) {
            Logger.getLogger(DataSet.class.getName()).log(Level.SEVERE, null, ex);
        }
        this.nrvar = nrvar;
    }

    public void importTestData(ArrayList<double[]> data) {
        this.nrvar = data.get(0).length;
        for (int i = 0; i < data.size(); i++) {
            this.data.put(i, new DataVect("Unknown", data.get(i)));
        }
    }

    public ArrayList<Integer> bootstrap(int n) {
        int randn;
        ThreadLocalRandom randgen = ThreadLocalRandom.current();
        ArrayList<Integer> dataElems = new ArrayList<>();
        outofbag.clear();
        for (int i = 0; i < this.size(); i++) {
            this.outofbag.add(i);
        }
        for (int i = 0; i < n; i++) {
            randn = randgen.nextInt(this.data.size());
            dataElems.add(randn);
            outofbag.remove(new Integer(randn));
        }
        return dataElems;
    }

    public ArrayList<Integer> getOutOfBagElems() {
        if (outofbag.isEmpty()) {
            System.out.println("OOB set is empty!");
        }
        ArrayList<Integer> res = new ArrayList<>(outofbag);
        return res;
    }

    public int getOutOfBagSize() {
        return outofbag.size();
    }

    public void permutateVariable(int varpos) {
        int randn;
        ThreadLocalRandom randgen = ThreadLocalRandom.current();
        for (int i = 0; i < this.size(); i++) {
            randn = randgen.nextInt(this.data.size());
            double val = this.get(i).variables[varpos];
            this.get(i).variables[varpos] = this.get(randn).variables[varpos];
            this.get(randn).variables[varpos] = val;
        }
    }

    public void writeToFile(String newfile) {
        BufferedWriter writer = null;
        try {
            File file = new File(newfile);
            if (file.exists()) {
                file.delete();
            }
            writer = new BufferedWriter(new FileWriter(newfile, true));
            for (int i = 0; i < data.size(); i++) {
                DataVect vector = data.get(i);
                writer.append(i + "\t" + vector.klasse);
                for (int j = 0; j < vector.variables.length; j++) {
                    writer.append("\t" + vector.variables[j]);
                }
                writer.append(System.getProperty("line.separator"));
            }
        } catch (IOException ex) {
            Logger.getLogger(DataSet.class.getName()).log(Level.SEVERE, null, ex);
        } finally {
            try {
                writer.close();
            } catch (IOException ex) {
                Logger.getLogger(DataSet.class.getName()).log(Level.SEVERE, null, ex);
            }
        }
    }

    public ArrayList<Double> getVarMax() {
        return varMax;
    }

    public ArrayList<Double> getVarMin() {
        return varMin;
    }

    public void normalize() {
        normalize((ArrayList<Double>) varMin.clone(), (ArrayList<Double>) varMax.clone());
    }

    public void normalize(ArrayList<Double> min, ArrayList<Double> max) {
        varMin.clear();
        varMax.clear();
        for (int k = 0; k < nrvar; k++) {
            varMin.add(Double.POSITIVE_INFINITY);
            varMax.add(Double.NEGATIVE_INFINITY);
        }
        for (Map.Entry<Integer, DataVect> entry : data.entrySet()) {
            double var[] = entry.getValue().variables;
            for (int i = 0; i < this.nrvar; i++) {
                var[i] = (var[i] - min.get(i)) * 100 / (max.get(i) - min.get(i));
                if (var[i] > varMax.get(i)) {
                    varMax.set(i, var[i]);
                }
                if (var[i] < varMin.get(i)) {
                    varMin.set(i, var[i]);
                }
            }
        }
    }
}
