/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package org.spectrim.decisionforest.structure;

import com.mongodb.BasicDBObject;
import java.util.ArrayList;
import java.util.TreeMap;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.spectrim.decisionforest.data.DataSet;
import org.spectrim.decisionforest.exceptions.DimSizeBiggerThanDatasetSize;
import org.spectrim.decisionforest.multithreading.TreeTester;
import org.spectrim.decisionforest.multithreading.TreeTrainer;
import org.spectrim.decisionforest.results.ResultSet;

/**
 *
 * @author nunopbarros
 */
public class Tree implements java.io.Serializable {

    public int maxdepth;
    public int minsize;
    public int nchilds;
    public Node root;
    public int nodenum;
    public Forest parent;
    private int splitCount;
    private int maxSplits;
    public int nrVars;
    public int nrVarsNode;
    public TreeMap<Integer,Node> nodes2train = new TreeMap<>();

    Tree(Forest forest, int maxdepth, int nchilds, int nrvarnode, int nrvardata, int minsize, int maxSplits) {
//        ArrayList<Integer> nonusedvar = new ArrayList<>();

        this.maxdepth = maxdepth;
        this.nchilds = nchilds;
        this.nodenum = 1;
        this.minsize = minsize;
        this.parent = forest;
        this.maxSplits = maxSplits;
        this.nrVars = nrvardata;
        this.nrVarsNode = nrvarnode;
        
//        for (int i = 0; i < nrvardata; i++) {
//            nonusedvar.add(i);
//        }
        try {
            this.root = new Node(this, 0, 0);
//            grow(this.root, nonusedvar, nrvarnode);
        } catch (DimSizeBiggerThanDatasetSize ex) {
            Logger.getLogger(Tree.class.getName()).log(Level.SEVERE, null, ex);
        }
    }

    public Tree() {
    }
    
    public void addNode2Train(Node node){
        this.nodes2train.put(node.level*1000000+node.id, node);
    }
    
    public void incrementSplitCounter(){
        splitCount++;
//        System.out.println("Nr splits:"+splitCount);
    }
    
    public boolean isMaxSplitReached(){
        return splitCount==maxSplits;
    }

    public BasicDBObject save() {

        BasicDBObject tree = new BasicDBObject().
                append("maxdepth", this.maxdepth).
                append("nchilds", this.nchilds).
                append("nodenum", this.nodenum);

        tree.append("tree", root.save());

        return tree;
    }

//    private void grow(Node nodeparent, ArrayList<Integer> nonusedvar, int nrvarnode) throws dimSizeBiggerThanDatasetSize {
//        if (nodeparent.getLevel() < this.maxdepth) {
//            for (int i = 0; i < this.nchilds; i++) {
//                this.nodenum = nodeparent.addChild(this.nodenum, nonusedvar, nrvarnode);
//                grow(nodeparent.getChild(i), nonusedvar, nrvarnode);
//            }
//        }
//    }

    public void startTrainingThread(DataSet traindata, int nriter, int nrsamples, int baggingsize, ArrayList<ResultSet> resultsOOB, int weaklearner, boolean optimizeWeights) {
        (new Thread(new TreeTrainer(this,traindata, nriter, nrsamples, baggingsize, resultsOOB, weaklearner, optimizeWeights))).start();
    }

    public void startTestingThread(DataSet testdata, double minreliab) {
        (new Thread(new TreeTester(this,testdata, minreliab))).start();
    }
    
    public void informTrainedTree() {
        parent.informTrainedTree();
    }
    
    public void trainTree(int nriter, int nrsamples, int weaklearner, boolean optimizeWeights){       
        trainNode(this.root,nriter,nrsamples,weaklearner,optimizeWeights);
        while(!this.nodes2train.isEmpty()){
            trainNode(nodes2train.get(nodes2train.firstKey()),nriter,nrsamples,weaklearner,optimizeWeights);
            nodes2train.remove(nodes2train.firstKey());
        }
    }
    
    public void trainNode(Node node, int nriter, int nrsamples,int weaklearner, boolean optimizeWeights) {
        if (node.getLevel() <= this.maxdepth) {
            node.train(nriter, nrsamples, minsize,weaklearner,optimizeWeights);
            node.dataelem.clear();
//            if (node.hasChildren()) {
//                for (int i = 0; i < node.nrchildren; i++) {
//                    if (node.getChild(i).hasDataElems()) {
//                        train(node.getChild(i), dataset, nriter, nrsamples);
//                    }
//                }
//            }
        }
    }

    public void test(ArrayList<Integer> dataelem, DataSet testdata, ResultSet results, double minreliab) {
        root.setDataElems((ArrayList<Integer>) dataelem.clone(),false);
        if (root.getLevel() <= this.maxdepth) {
            root.setDataSet(testdata);
            root.test(results, minreliab);
//            root.clearDataElems();
            if (root.hasChildren()) {
                for (int i = 0; i < root.getNrChildren(); i++) {
                    test(root.getChild(i), results, minreliab);
                }
            }
        }
//        removeData();
    }

    public void test(Node node, ResultSet results, double minreliab) {

        if (node.getLevel() <= this.maxdepth) {
            node.test(results, minreliab);
//            node.clearDataElems();
            if (node.hasChildren()) {
                for (int i = 0; i < node.getNrChildren(); i++) {
                    test(node.getChild(i), results, minreliab);
                }
            }
        }
    }
    
    public void reportResults(ResultSet results){
        parent.reportResults(results);
    }
    
    public void removeData() {
        root.dataelem.clear();
        if (root.hasChildren()) {
            for (int i = 0; i < root.getNrChildren(); i++) {
                removeData(root.getChild(i));
            }
        }
    }

    private void removeData(Node node) {
        node.dataelem.clear();
        if (node.hasChildren()) {
            for (int i = 0; i < node.getNrChildren(); i++) {
                removeData(node.getChild(i));
            }
        }
    }
}
