/*
 * Dicom3DImageSeriesData.java
 *
 * Created on May 3, 2006, 4:52 PM
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package org.spectrim.dicom;

import org.spectrim.imagegraphicobjects.Point3D;
import org.spectrim.lib.Dialogs;
import org.spectrim.lib.DualProgressBar;
import org.spectrim.lib.SwingWorker;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.LinkedList;
import javax.swing.JFrame;
 
/**
 *
 * @author hans
 */
public class Dicom3DImageSeriesData extends DicomImageSeriesData
{
    private JFrame m_parent;
    private DualProgressBar m_dualProgressBar;
    
    /** 
     * Creates a new instance of Dicom3DImageSeriesData. 
     * @param parent
     * @param fileReferences
     */
    public Dicom3DImageSeriesData( JFrame parent, LinkedList<String> fileReferences )
    {
        super( fileReferences );
        m_parent = parent;
    }

    /**
     * Creates a new instance of Dicom3DImageSeriesData.
     * @param parent
     * @param fileReferences
     * @param fmriActivationImageOverlay
     * @param dtiFiberImageOverlays
     * @param dtiColorOverlay
     */
    public Dicom3DImageSeriesData( JFrame parent, LinkedList<String> fileReferences,
                                   boolean[][][] fmriActivationImageOverlay,
                                   boolean[][][] dtiFiberImageOverlays,
                                   Color[][][] dtiColorOverlay )
    {
        super( fileReferences );
        m_parent                        = parent;
        m_fmriActivationImageOverlays   = fmriActivationImageOverlay;
        m_dtiFiberImageOverlays         = dtiFiberImageOverlays;
        m_dtiColorOverlay               = dtiColorOverlay;
    }


    /**
     * Sets the DTI fiber overlays.
     * @param dtiFiberImageOverlays
     * @param colors
     */
    public void setDTIFiberOverlays( boolean[][][] dtiFiberImageOverlays, Color[][][] colors )
    {

//        m_dtiFiberImageOverlays       = dtiFiberImageOverlays;
//        m_dtiColorOverlay             = colors;
        
        // define a new object if the current object == null
        // else merge the new overlays with the already existant overlay
        // the last defined dti fiber information is decisive
        if ( ( m_dtiFiberImageOverlays == null ) && ( m_dtiColorOverlay == null ) )
        {
            m_dtiFiberImageOverlays = dtiFiberImageOverlays;
            m_dtiColorOverlay = colors;
        }
        else
        {
            for ( int i = 0; i < dtiFiberImageOverlays.length; i++ )
            {
                for ( int j = 0; j < m_3DMatrixSize; j++ )
                {
                    for ( int k = 0; k < m_3DMatrixSize; k++ )
                    {
                        if ( dtiFiberImageOverlays[i][j][k] == true )
                        {
                            m_dtiFiberImageOverlays[i][j][k] =  dtiFiberImageOverlays[i][j][k];
                            m_dtiColorOverlay[i][j][k] = colors[i][j][k];
                        }
                    }
                }  
            }
        }
        
        create3DOverlayImageDataOnly( );

    }

    /**
     * Sets the fMRI activation overlays.
     * @param activationOverlays
     */
    public void setfMRIActiovationOverlays( boolean[][][] activationOverlays )
    {
        m_fmriActivationImageOverlays = activationOverlays;
        create3DOverlayImageDataOnly( );
    }


    /**
     * Creates (loads) the Dicom3DImageSeriesData object.
     */
    public void create3DDataInSwingworkerThread( )
    {
        m_dualProgressBar = new DualProgressBar( m_parent, 100, 100, new Dimension( 900, 150 ), false );
        m_dualProgressBar.setTitle( "Initializing 3D slicer data"  );

        m_dualProgressBar.setUpperProgress( 0 );
        m_dualProgressBar.setUpperLabel( "        Total progress " );
        m_dualProgressBar.setUpperProgressVisible( false );
        m_dualProgressBar.setLowerLabel( "<HTML><BR>  Loading image number : " +  0 + "</HTML>" );


       final SwingWorker worker = new SwingWorker( )
       {
            @Override
            public Object construct( )
            {
                try
                {
                    // segment the image
                    create3DImageData( );
                }
                catch ( OutOfMemoryError e )
                {
                    Dialogs d = new Dialogs( );
                    Dialogs.messageBox( m_parent, "<HTML> Could note create 3D dataset <BR> Out of memmory </HTML>" );
                    notifyActionListeners( THREE_D_VIEWER_DATASET_LOAD_ERROR );
                }
                return null;
            }

            // runs in the event dispatching thread
            @Override
            public void finished( )
            {
                 // After calculation set enabled again
                 m_dualProgressBar.setVisible( false );
                 notifyActionListeners( THREE_D_VIEWER_DATASET_LOADED );
            }

        };
        worker.start( );
    }




    /**
     * Creates 3D image data.
     */ 
    public void create3DImageData( )
    {
        // sets the member that indicates that the image series is  ordered
        
        if ( m_imageSeriesIsOrdered == false )
        {
            this.orderBySlicePosition( );
        }
        
        // free memory!!
        this.m_imageSeries.clear( );
        
        m_imageSeries = null;
         
        int cnt = 0;
        
        int offset_1_InPoints = 0;
        int offset_2_InPoints = 0;
        int offset_3_InPoints = 0;
        
        m_dicomHeaderInformation = null;
        m_siemensSpecificData = null;
        
        // get a reference to all the keys of the sub map containing the series
        // info only
        Iterator iter = m_orderedImageSeries.keySet( ).iterator( );

         int totalNumber = m_orderedImageSeries.size( );


        int[] outputImage = null;
        int[] image = null;
        int cntr=0;
        while ( iter.hasNext( ) )
        {

            m_dualProgressBar.setLowerProgress( (int)( 100.0 * (double)cntr / totalNumber ) );
            m_dualProgressBar.setLowerLabel( "<HTML><BR>   Loading image number : " +  (cntr++) + "</HTML>"  );

            String pos = (String)iter.next( );
            
            DicomImageData dicom_data = m_orderedImageSeries.get( pos );
            
            FileInfo finfo = dicom_data.getHeader( );
            m_sliceThickness_12 = new Double( finfo.getSliceThickness( ) ).floatValue( );
            LinkedHashMap fSiemensSpecificData = dicom_data.getSiemensSpecificDataFromFile( );
            
            // int[] outputImage;
            
            if ( cnt == 0 ) 
            {
                m_points_dir_1 = dicom_data.getHeader( ).getImageWidth( );
                m_points_dir_2 = dicom_data.getHeader( ).getImageHeight( );
                m_points_dir_3 = m_orderedImageSeries.size( );
                
                m_imageSeriesPatientName = finfo.getPatientName( );
                
                if ( ( m_points_dir_1 >= m_points_dir_2 ) && ( m_points_dir_1 >= m_points_dir_3  ) )
                {
                    m_3DMatrixSize = m_points_dir_1;
                }
                
                if ( ( m_points_dir_2 >= m_points_dir_1 ) && ( m_points_dir_2 >= m_points_dir_3  ) )
                {
                    m_3DMatrixSize = m_points_dir_2;
                }
                
                if ( ( m_points_dir_3 >= m_points_dir_1 ) && ( m_points_dir_3 >= m_points_dir_2  ) )
                {
                    m_3DMatrixSize = m_points_dir_3;
                }
                
                m_dicomHeaderInformation = new FileInfo[ m_3DMatrixSize ];
                m_siemensSpecificData = new LinkedHashMap[ m_3DMatrixSize ];
                
                if ( image == null )
                {
                    image = new int[m_points_dir_1 * m_points_dir_2];
                }
                
                if ( outputImage == null )
                {
                    outputImage = new int[m_3DMatrixSize * m_3DMatrixSize];
                }
                
                m_3dImageData_1_2 = new int[ m_3DMatrixSize * m_3DMatrixSize * m_3DMatrixSize ];
               
                offset_1_InPoints = ( m_3DMatrixSize - m_points_dir_1 ) / 2;
                offset_2_InPoints = ( m_3DMatrixSize - m_points_dir_2 ) / 2;
                offset_3_InPoints = ( m_3DMatrixSize - m_points_dir_3 ) / 2;

                // First ensure that for all images (also the identical=0 images
                // there exists a default header information
                for ( int i = 0; i < m_3DMatrixSize; i++ )
                {
                    m_dicomHeaderInformation[i] = finfo;
                    m_siemensSpecificData[i] = fSiemensSpecificData;
                }

                if ( m_fmriActivationImageOverlays != null )
                {
                    m_fmriActivationImageOverlays_12 = new boolean[m_3DMatrixSize][m_3DMatrixSize][m_3DMatrixSize];
                    m_fmriActivationImageOverlays_13 = new boolean[m_3DMatrixSize][m_3DMatrixSize][m_3DMatrixSize];
                    m_fmriActivationImageOverlays_23 = new boolean[m_3DMatrixSize][m_3DMatrixSize][m_3DMatrixSize];

                    for ( int i = offset_3_InPoints; i < m_3DMatrixSize - offset_3_InPoints - 1; i++ )
                    {
                        for ( int j = offset_2_InPoints;  j < m_3DMatrixSize - offset_2_InPoints - 1; j++ )
                        {
                            for ( int k = offset_1_InPoints;  k < m_3DMatrixSize - offset_1_InPoints - 1; k++ )
                            {
                                m_fmriActivationImageOverlays_12[i][j][k] = m_fmriActivationImageOverlays[i-offset_3_InPoints][j-offset_2_InPoints][k-offset_1_InPoints];
                            }
                        }
                    }

                    
                    for ( int i = 0; i < m_3DMatrixSize - 1; i++ )
                    {
                        for ( int j = 0;  j < m_3DMatrixSize - 1; j++ )
                        {
                            for ( int k = 0;  k < m_3DMatrixSize - 1; k++ )
                            {
                                m_fmriActivationImageOverlays_13[i][j][k] = m_fmriActivationImageOverlays_12[k][i][j];
                            }
                        }
                    }

                    boolean[][][] transpose = new boolean[m_3DMatrixSize][m_3DMatrixSize][m_3DMatrixSize];

                    for ( int i = 0; i < m_3DMatrixSize - 1; i++ )
                    {
                        for ( int j = 0;  j < m_3DMatrixSize - 1; j++ )
                        {
                            for ( int k = 0;  k < m_3DMatrixSize - 1; k++ )
                            {
                                transpose[i][j][k] = m_fmriActivationImageOverlays_13[i][k][j];
                            }
                        }
                    }

                    for ( int i = 0; i < m_3DMatrixSize - 1; i++ )
                    {
                        for ( int j = 0;  j < m_3DMatrixSize - 1; j++ )
                        {
                            for ( int k = 0;  k < m_3DMatrixSize - 1; k++ )
                            {
                                m_fmriActivationImageOverlays_13[i][j][k] = transpose[i][j][k];
                            }
                        }
                    }


                    for ( int i = 0; i < m_3DMatrixSize - 1; i++ )
                    {
                        for ( int j = 0;  j < m_3DMatrixSize - 1; j++ )
                        {
                            for ( int k = 0;  k < m_3DMatrixSize - 1; k++ )
                            {
                                m_fmriActivationImageOverlays_23[i][j][k] = m_fmriActivationImageOverlays_12[j][k][i];
                            }
                        }
                    }
                }

                if ( m_dtiFiberImageOverlays != null )
                {
                    m_dtiFiberImageOverlays_12 = new boolean[m_3DMatrixSize][m_3DMatrixSize][m_3DMatrixSize];
                    m_dtiFiberImageOverlays_13 = new boolean[m_3DMatrixSize][m_3DMatrixSize][m_3DMatrixSize];
                    m_dtiFiberImageOverlays_23 = new boolean[m_3DMatrixSize][m_3DMatrixSize][m_3DMatrixSize];

                    for ( int i = offset_3_InPoints; i < m_3DMatrixSize - offset_3_InPoints - 1; i++ )
                    {
                        for ( int j = offset_2_InPoints;  j < m_3DMatrixSize - offset_2_InPoints - 1; j++ )
                        {
                            for ( int k = offset_1_InPoints;  k < m_3DMatrixSize - offset_1_InPoints - 1; k++ )
                            {
                                m_dtiFiberImageOverlays_12[i][j][k] = m_dtiFiberImageOverlays[i-offset_3_InPoints][j-offset_2_InPoints][k-offset_1_InPoints];
                            }
                        }
                    }


                    for ( int i = 0; i < m_3DMatrixSize - 1; i++ )
                    {
                        for ( int j = 0;  j < m_3DMatrixSize - 1; j++ )
                        {
                            for ( int k = 0;  k < m_3DMatrixSize - 1; k++ )
                            {
                                m_dtiFiberImageOverlays_13[i][j][k] = m_dtiFiberImageOverlays_12[k][i][j];
                            }
                        }
                    }

                    boolean[][][] transpose = new boolean[m_3DMatrixSize][m_3DMatrixSize][m_3DMatrixSize];

                    for ( int i = 0; i < m_3DMatrixSize - 1; i++ )
                    {
                        for ( int j = 0;  j < m_3DMatrixSize - 1; j++ )
                        {
                            for ( int k = 0;  k < m_3DMatrixSize - 1; k++ )
                            {
                                transpose[i][j][k] = m_dtiFiberImageOverlays_13[i][k][j];
                            }
                        }
                    }

                    for ( int i = 0; i < m_3DMatrixSize - 1; i++ )
                    {
                        for ( int j = 0;  j < m_3DMatrixSize - 1; j++ )
                        {
                            for ( int k = 0;  k < m_3DMatrixSize - 1; k++ )
                            {
                                m_dtiFiberImageOverlays_13[i][j][k] = transpose[i][j][k];
                            }
                        }
                    }


                    for ( int i = 0; i < m_3DMatrixSize - 1; i++ )
                    {
                        for ( int j = 0;  j < m_3DMatrixSize - 1; j++ )
                        {
                            for ( int k = 0;  k < m_3DMatrixSize - 1; k++ )
                            {
                                m_dtiFiberImageOverlays_23[i][j][k] = m_dtiFiberImageOverlays_12[j][k][i];
                            }
                        }
                    }
                }


                if ( this.m_dtiColorOverlay != null )
                {
                    m_dtiColorOverlay_12 = new Color[m_3DMatrixSize][m_3DMatrixSize][m_3DMatrixSize];
                    m_dtiColorOverlay_13 = new Color[m_3DMatrixSize][m_3DMatrixSize][m_3DMatrixSize];
                    m_dtiColorOverlay_23 = new Color[m_3DMatrixSize][m_3DMatrixSize][m_3DMatrixSize];

                    for ( int i = offset_3_InPoints; i < m_3DMatrixSize - offset_3_InPoints - 1; i++ )
                    {
                        for ( int j = offset_2_InPoints;  j < m_3DMatrixSize - offset_2_InPoints - 1; j++ )
                        {
                            for ( int k = offset_1_InPoints;  k < m_3DMatrixSize - offset_1_InPoints - 1; k++ )
                            {
                                m_dtiColorOverlay_12[i][j][k] = m_dtiColorOverlay[i-offset_3_InPoints][j-offset_2_InPoints][k-offset_1_InPoints];
                            }
                        }
                    }


                    for ( int i = 0; i < m_3DMatrixSize - 1; i++ )
                    {
                        for ( int j = 0;  j < m_3DMatrixSize - 1; j++ )
                        {
                            for ( int k = 0;  k < m_3DMatrixSize - 1; k++ )
                            {
                                m_dtiColorOverlay_13[i][j][k] = m_dtiColorOverlay_12[k][i][j];
                            }
                        }
                    }

                    Color[][][] transposeC = new Color[m_3DMatrixSize][m_3DMatrixSize][m_3DMatrixSize];

                    for ( int i = 0; i < m_3DMatrixSize - 1; i++ )
                    {
                        for ( int j = 0;  j < m_3DMatrixSize - 1; j++ )
                        {
                            for ( int k = 0;  k < m_3DMatrixSize - 1; k++ )
                            {
                                transposeC[i][j][k] = m_dtiColorOverlay_13[i][k][j];
                            }
                        }
                    }

                    for ( int i = 0; i < m_3DMatrixSize - 1; i++ )
                    {
                        for ( int j = 0;  j < m_3DMatrixSize - 1; j++ )
                        {
                            for ( int k = 0;  k < m_3DMatrixSize - 1; k++ )
                            {
                                m_dtiColorOverlay_13[i][j][k] = transposeC[i][j][k];
                            }
                        }
                    }


                    for ( int i = 0; i < m_3DMatrixSize - 1; i++ )
                    {
                        for ( int j = 0;  j < m_3DMatrixSize - 1; j++ )
                        {
                            for ( int k = 0;  k < m_3DMatrixSize - 1; k++ )
                            {
                                m_dtiColorOverlay_23[i][j][k] = m_dtiColorOverlay_12[j][k][i];
                            }
                        }
                    }
                }


            }

            m_dicomHeaderInformation[ offset_3_InPoints + cnt ] =  finfo;
            m_siemensSpecificData[ offset_3_InPoints + cnt ] =  fSiemensSpecificData;

            image = dicom_data.getPixelData( );
            if ( outputImage == null )
            {
                outputImage = new int[m_3DMatrixSize * m_3DMatrixSize];
            }
            
            for ( int i = 0; i < m_points_dir_2 ; i++ )
            {
                for ( int j = 0; j < m_points_dir_1; j++ )
                {
                    outputImage[ offset_2_InPoints + j + ( ( i + offset_1_InPoints ) * m_points_dir_2 ) ] = image[ j + i * m_points_dir_1 ];
                }
            }
            
            System.arraycopy( outputImage, 0, m_3dImageData_1_2, ( offset_3_InPoints + cnt ) * m_3DMatrixSize * m_3DMatrixSize, m_3DMatrixSize * m_3DMatrixSize );
            
            //System.out.println( "counter " + cnt );
            
            cnt++;
        }
    }



    /**
     * Creates 3D image data.
     */
    public void create3DOverlayImageDataOnly( )
    {
        // sets the member that indicates that the image series is  ordered

        int cnt = 0;

        int offset_1_InPoints = 0;
        int offset_2_InPoints = 0;
        int offset_3_InPoints = 0;

        // get a reference to all the keys of the sub map containing the series
        // info only
        Iterator iter = m_orderedImageSeries.keySet( ).iterator( );

        int totalNumber = m_orderedImageSeries.size( );


        int[] outputImage = null;
        int[] image = null;
        int cntr=0;

        String pos = (String)iter.next( );

        DicomImageData dicom_data = m_orderedImageSeries.get( pos );

        FileInfo finfo = dicom_data.getHeader( );
        m_sliceThickness_12 = new Double( finfo.getSliceThickness( ) ).floatValue( );
        LinkedHashMap fSiemensSpecificData = dicom_data.getSiemensSpecificDataFromFile( );

        m_points_dir_1 = dicom_data.getHeader( ).getImageWidth( );
        m_points_dir_2 = dicom_data.getHeader( ).getImageHeight( );
        m_points_dir_3 = m_orderedImageSeries.size( );

        m_imageSeriesPatientName = finfo.getPatientName( );

        if ( ( m_points_dir_1 >= m_points_dir_2 ) && ( m_points_dir_1 >= m_points_dir_3  ) )
        {
            m_3DMatrixSize = m_points_dir_1;
        }

        if ( ( m_points_dir_2 >= m_points_dir_1 ) && ( m_points_dir_2 >= m_points_dir_3  ) )
        {
            m_3DMatrixSize = m_points_dir_2;
        }

        if ( ( m_points_dir_3 >= m_points_dir_1 ) && ( m_points_dir_3 >= m_points_dir_2  ) )
        {
            m_3DMatrixSize = m_points_dir_3;
        }

        offset_1_InPoints = ( m_3DMatrixSize - m_points_dir_1 ) / 2;
        offset_2_InPoints = ( m_3DMatrixSize - m_points_dir_2 ) / 2;
        offset_3_InPoints = ( m_3DMatrixSize - m_points_dir_3 ) / 2;


        if ( m_fmriActivationImageOverlays != null )
        {
            m_fmriActivationImageOverlays_12 = new boolean[m_3DMatrixSize][m_3DMatrixSize][m_3DMatrixSize];
            m_fmriActivationImageOverlays_13 = new boolean[m_3DMatrixSize][m_3DMatrixSize][m_3DMatrixSize];
            m_fmriActivationImageOverlays_23 = new boolean[m_3DMatrixSize][m_3DMatrixSize][m_3DMatrixSize];
            for ( int i = offset_3_InPoints; i < m_3DMatrixSize - offset_3_InPoints - 1; i++ )
            {
                for ( int j = offset_2_InPoints;  j < m_3DMatrixSize - offset_2_InPoints - 1; j++ )
                {
                    for ( int k = offset_1_InPoints;  k < m_3DMatrixSize - offset_1_InPoints - 1; k++ )
                    {
                        m_fmriActivationImageOverlays_12[i][j][k] = m_fmriActivationImageOverlays[i-offset_3_InPoints][j-offset_2_InPoints][k-offset_1_InPoints];
                    }
                }
           }


           for ( int i = 0; i < m_3DMatrixSize - 1; i++ )
           {
               for ( int j = 0;  j < m_3DMatrixSize - 1; j++ )
               {
                   for ( int k = 0;  k < m_3DMatrixSize - 1; k++ )
                   {
                       m_fmriActivationImageOverlays_13[i][j][k] = m_fmriActivationImageOverlays_12[j][i][k];
                   }
               }
           }

           boolean[][][] transpose = new boolean[m_3DMatrixSize][m_3DMatrixSize][m_3DMatrixSize];
           for ( int i = 0; i < m_3DMatrixSize - 1; i++ )
           {
               for ( int j = 0;  j < m_3DMatrixSize - 1; j++ )
               {
                   for ( int k = 0;  k < m_3DMatrixSize - 1; k++ )
                   {
                        transpose[i][j][k] = m_fmriActivationImageOverlays_13[i][k][j];
                   }
               }
           }

           for ( int i = 0; i < m_3DMatrixSize - 1; i++ )
           {
               for ( int j = 0;  j < m_3DMatrixSize - 1; j++ )
               {
                   for ( int k = 0;  k < m_3DMatrixSize - 1; k++ )
                   {
                       m_fmriActivationImageOverlays_13[i][j][k] = transpose[i][j][k];
                   }
               }
           }

           for ( int i = 0; i < m_3DMatrixSize - 1; i++ )
           {
               for ( int j = 0;  j < m_3DMatrixSize - 1; j++ )
               {
                   for ( int k = 0;  k < m_3DMatrixSize - 1; k++ )
                   {
                       m_fmriActivationImageOverlays_23[i][j][k] = m_fmriActivationImageOverlays_12[k][j][i];
                   }
               }
            }
        }

        if ( m_dtiFiberImageOverlays != null )
        {
            m_dtiFiberImageOverlays_12 = new boolean[m_3DMatrixSize][m_3DMatrixSize][m_3DMatrixSize];
            m_dtiFiberImageOverlays_13 = new boolean[m_3DMatrixSize][m_3DMatrixSize][m_3DMatrixSize];
            m_dtiFiberImageOverlays_23 = new boolean[m_3DMatrixSize][m_3DMatrixSize][m_3DMatrixSize];

            for ( int i = offset_3_InPoints; i < m_3DMatrixSize - offset_3_InPoints - 1; i++ )
            {
                for ( int j = offset_2_InPoints;  j < m_3DMatrixSize - offset_2_InPoints - 1; j++ )
                {
                    for ( int k = offset_1_InPoints;  k < m_3DMatrixSize - offset_1_InPoints - 1; k++ )
                    {
                        m_dtiFiberImageOverlays_12[i][j][k] = m_dtiFiberImageOverlays[i-offset_3_InPoints][j-offset_2_InPoints][k-offset_1_InPoints];
                    }
                }
            }


            for ( int i = 0; i < m_3DMatrixSize - 1; i++ )
            {
                for ( int j = 0;  j < m_3DMatrixSize - 1; j++ )
                {
                   for ( int k = 0;  k < m_3DMatrixSize - 1; k++ )
                   {
                        m_dtiFiberImageOverlays_13[i][j][k] = m_dtiFiberImageOverlays_12[j][i][k];
                   }
               }
            }

            boolean[][][] transpose = new boolean[m_3DMatrixSize][m_3DMatrixSize][m_3DMatrixSize];
            for ( int i = 0; i < m_3DMatrixSize - 1; i++ )
            {
                for ( int j = 0;  j < m_3DMatrixSize - 1; j++ )
                {
                    for ( int k = 0;  k < m_3DMatrixSize - 1; k++ )
                    {
                        transpose[i][j][k] = m_dtiFiberImageOverlays_13[i][k][j];
                    }
                }
            }

            for ( int i = 0; i < m_3DMatrixSize - 1; i++ )
            {
                for ( int j = 0;  j < m_3DMatrixSize - 1; j++ )
                {
                    for ( int k = 0;  k < m_3DMatrixSize - 1; k++ )
                    {
                        m_dtiFiberImageOverlays_13[i][j][k] = transpose[i][j][k];
                    }
                }
            }


            for ( int i = 0; i < m_3DMatrixSize - 1; i++ )
            {
                for ( int j = 0;  j < m_3DMatrixSize - 1; j++ )
                {
                    for ( int k = 0;  k < m_3DMatrixSize - 1; k++ )
                    {
                        m_dtiFiberImageOverlays_23[i][j][k] = m_dtiFiberImageOverlays_12[k][j][i];
                    }
                }
            }
        }


        if ( this.m_dtiColorOverlay != null )
        {
            m_dtiColorOverlay_12 = new Color[m_3DMatrixSize][m_3DMatrixSize][m_3DMatrixSize];
            m_dtiColorOverlay_13 = new Color[m_3DMatrixSize][m_3DMatrixSize][m_3DMatrixSize];
            m_dtiColorOverlay_23 = new Color[m_3DMatrixSize][m_3DMatrixSize][m_3DMatrixSize];

            for ( int i = offset_3_InPoints; i < m_3DMatrixSize - offset_3_InPoints - 1; i++ )
            {
                for ( int j = offset_2_InPoints;  j < m_3DMatrixSize - offset_2_InPoints - 1; j++ )
                {
                    for ( int k = offset_1_InPoints;  k < m_3DMatrixSize - offset_1_InPoints - 1; k++ )
                    {
                        m_dtiColorOverlay_12[i][j][k] = m_dtiColorOverlay[i-offset_3_InPoints][j-offset_2_InPoints][k-offset_1_InPoints];
                    }
                }
            }


            for ( int i = 0; i < m_3DMatrixSize - 1; i++ )
            {
                for ( int j = 0;  j < m_3DMatrixSize - 1; j++ )
                {
                    for ( int k = 0;  k < m_3DMatrixSize - 1; k++ )
                    {
                        m_dtiColorOverlay_13[i][j][k] = m_dtiColorOverlay_12[j][i][k];
                    }
                }
            }

            Color[][][] transposeC = new Color[m_3DMatrixSize][m_3DMatrixSize][m_3DMatrixSize];

            for ( int i = 0; i < m_3DMatrixSize - 1; i++ )
            {
                for ( int j = 0;  j < m_3DMatrixSize - 1; j++ )
                {
                    for ( int k = 0;  k < m_3DMatrixSize - 1; k++ )
                    {
                        transposeC[i][j][k] = m_dtiColorOverlay_13[i][k][j];
                    }
                }
            }

            for ( int i = 0; i < m_3DMatrixSize - 1; i++ )
            {
                for ( int j = 0;  j < m_3DMatrixSize - 1; j++ )
                {
                    for ( int k = 0;  k < m_3DMatrixSize - 1; k++ )
                    {
                        m_dtiColorOverlay_13[i][j][k] = transposeC[i][j][k];
                    }
                }
            }


            for ( int i = 0; i < m_3DMatrixSize - 1; i++ )
            {
                for ( int j = 0;  j < m_3DMatrixSize - 1; j++ )
                {
                    for ( int k = 0;  k < m_3DMatrixSize - 1; k++ )
                    {
                        m_dtiColorOverlay_23[i][j][k] = m_dtiColorOverlay_12[k][j][i];
                    }
                }
            }
        }
    }






    /**
     * Returns the resized fmri overlay map
     * @return
     */
    public boolean[][][] getFMRIActivationImageOverlays_12( )
    {
        return m_fmriActivationImageOverlays_12;
    }

    /**
     * Returns the resized fmri overlay map
     * @return
     */
    public boolean[][][] getFMRIActivationImageOverlays_13( )
    {
        return m_fmriActivationImageOverlays_13;
    }

    /**
     * Returns the resized fmri overlay map
     * @return
     */
    public boolean[][][] getFMRIActivationImageOverlays_23( )
    {
        return m_fmriActivationImageOverlays_23;
    }


     /**
     * Returns the resized dti fiber overlay map
     * @return
     */
    public boolean[][][] getDTIFiberImageOverlays_12( )
    {
        return m_dtiFiberImageOverlays_12;
    }

    /**
     * Returns the resized dti overlay map
     * @return
     */
    public boolean[][][] getDTIFiberImageOverlays_13( )
    {
        return m_dtiFiberImageOverlays_13;
    }

    /**
     * Returns the resized dti overlay map
     * @return
     */
    public boolean[][][] getDTIFiberImageOverlays_23( )
    {
        return m_dtiFiberImageOverlays_23;
    }


     /**
     * Returns the resized dti color fiber overlay map
     * @return
     */
    public Color[][][] getDTIColorOverlay_12( )
    {
        return m_dtiColorOverlay_12;
    }

    /**
     * Returns the resized dti color overlay map
     * @return
     */
    public Color[][][] getDTIColorOverlay_13( )
    {
        return m_dtiColorOverlay_13;
    }

    /**
     * Returns the resized dti color overlay map
     * @return
     */
    public Color[][][] getDTIColorOverlay_23( )
    {
        return m_dtiColorOverlay_23;
    }


    /**
     * Returns the image pixel data of the first acquisition.
     * @param sliceNumber
     * @return 
     */
    public int[] getImagePixelData_1_2( int sliceNumber )
    {
        int[] ima = new int[ m_3DMatrixSize * m_3DMatrixSize ];
        
        System.arraycopy( m_3dImageData_1_2, sliceNumber * m_3DMatrixSize * m_3DMatrixSize, ima, 0, m_3DMatrixSize * m_3DMatrixSize );

        return ima;
    }

    /**
     * Returns the FileInfo object of the slice having "sliceNumber".
     */
    public FileInfo getImageFileInfoObject_1_2( int sliceNumber )
    {
       FileInfo retObject;
       
       retObject = m_dicomHeaderInformation[ sliceNumber ];
       
       return retObject;
    }
 
    /**
     * Returns the FileInfo object of the slice having "sliceNumber".
     */
    public LinkedHashMap getSiemensSpecificFileInfomationObject_1_2( int sliceNumber )
    {
        LinkedHashMap siemensInfo = m_siemensSpecificData[ sliceNumber ];
        
        return siemensInfo;
    }
    
    
    /**
     * Returns the image pixel data of the first acquisition
     */
    public int[] getImagePixelData_1_3( int sliceNumber )
    {
        int [] ima = new int[ m_3DMatrixSize * m_3DMatrixSize ];
        
        for ( int j = m_3DMatrixSize-1; j >= 0; j-- )
        {
            for ( int i = 0; i < m_3DMatrixSize; i++ )
            {
                
                ima[ j + m_3DMatrixSize * i ] = m_3dImageData_1_2[ i * m_3DMatrixSize + sliceNumber + j * m_3DMatrixSize * m_3DMatrixSize ];
                
            }
        }
        
        return ima;
    }
    
    /**
     * Returns the image pixel data of the first acquisition
     */
    public int[] getImagePixelData_2_3( int sliceNumber )
    {
        int [] ima = new int[ m_3DMatrixSize * m_3DMatrixSize ];
       
        for ( int j = m_3DMatrixSize-1; j >= 0; j-- )
        {
            for ( int i = 0; i < m_3DMatrixSize; i++ )
            {
                ima[ j + m_3DMatrixSize * i ] = m_3dImageData_1_2[ sliceNumber * m_3DMatrixSize + i + j * m_3DMatrixSize * m_3DMatrixSize ];
            }
        }
        
        return ima;        
    }
    
    /**
     * Returns the matrix size of the 3D data cube;
     * There are m_3DMatrixSize*m_3DMatrixSize*m_3DMatrixSize image pixels
     */
    public int get3DMatrixSize( )
    {
        return m_3DMatrixSize;
    }
        
    /**
     * Sets the direction vector of the 3D dataset in plane components
     * The perpendicular components are computed from them
     */
    public void setDirectionVectors_1_2( Point3D dirX, Point3D dirY )
    {
        m_view_1_2_dirX = dirX;
        m_view_1_2_dirY = dirY;
        Point3D c = new Point3D( );
        
        m_normal_1_2 = c.computeNormalVector( dirX, dirY );
        
        m_view_1_3_dirX = m_normal_1_2;
        m_view_1_3_dirY = m_view_1_2_dirY;
        
        m_view_2_3_dirX = m_normal_1_2;
        m_view_2_3_dirY = m_view_1_2_dirX;
        
    }
    
    /**
     * Sets the FOV parameter values necessary to compute distances etc.
     */ 
    public void set3DFieldOfViewParameters( double fovx, double fovy, double sliceThick )
    {
        m_FOVX_12 = fovx;
        m_FOVY_12 = fovy;
        m_FOVZ_12 = m_3DMatrixSize * sliceThick;
    }
    
    
    /**
     * Get direction vector (X) of 1_2 direction
     */
    public Point3D getDirectionVector_1_2_X( )
    {
        return m_view_1_2_dirX;
    }
    
    /**
     * Get direction vector (Y) of 1_2 direction
     */
    public Point3D getDirectionVector_1_2_Y( )
    {
        return m_view_1_2_dirY;
    }

    /**
     * Get direction vector (X) of 1_3 direction
     */
    public Point3D getDirectionVector_1_3_X( )
    {
        return m_view_1_3_dirX;
    }
    
    /**
     * Get direction vector (Y) of 1_3 direction
     */
    public Point3D getDirectionVector_1_3_Y( )
    {
        return m_view_1_3_dirY;
    }

    /**
     * Get direction vector of 2_3 direction
     */
    public Point3D getDirectionVector_2_3_X( )
    {
        return m_view_2_3_dirX;
    }
    
    /**
     * Get direction vector of 2_3 direction
     */
    public Point3D getDirectionVector_2_3_Y( )
    {
        return m_view_2_3_dirY;
    }
    
    /**
     * Returns FOV_12 of the original slicestack in the X-direction
     */
    public double getFOVX_12( )
    {
        return m_FOVX_12;
    }
    
    /**
     * Returns FOV_12 of the original slicestack in the Y-direction
     */
    public double getFOVY_12( )
    {
        return m_FOVY_12;
    }

    /**
     * Returns FOV_12 of the original slicestack in the Z-direction
     * i.e. perpendicular to the slice (i.e. along the normal vector)
     */
    public double getFOVZ_12( )
    {
        return m_FOVZ_12;
    }


    /**
     * Notifies actionEvent to all registered listeners
     */
    private void notifyActionListeners( int eventType )
    {
        Iterator it = m_actionListeners.iterator( );
        while( it.hasNext( ) )
        {
            ( ( ActionListener )it.next( ) ).actionPerformed( new ActionEvent( this, eventType, "" ) );
        }
    }

    /**
     * Adds an actionlistener to the list of listeners
     */
    public void addActionListener( ActionListener e )
    {
        m_actionListeners.add( e );
    }
    
    // Attributes
    private int   m_points_dir_1;
    private int   m_points_dir_2;
    private int   m_points_dir_3;
    
    private Point3D m_view_1_2_dirX;
    private Point3D m_view_1_2_dirY;
    private Point3D m_normal_1_2;
    
    
    private Point3D m_view_1_3_dirX;
    private Point3D m_view_1_3_dirY;
    
    private Point3D m_view_2_3_dirX;
    private Point3D m_view_2_3_dirY;
    
    private double m_FOVX_12;
    private double m_FOVY_12;
    private double m_FOVZ_12;
    
    private float m_sliceThickness_12;
    
    private int   m_3DMatrixSize;
    
    private int[] m_3dImageData_1_2;
    
    private FileInfo[] m_dicomHeaderInformation;
    private LinkedHashMap[] m_siemensSpecificData;
    
    private String m_imageSeriesPatientName;

    private boolean[][][] m_fmriActivationImageOverlays;
    boolean[][][]         m_dtiFiberImageOverlays;
    Color[][][]           m_dtiColorOverlay;


    private boolean[][][] m_fmriActivationImageOverlays_12;
    private boolean[][][] m_fmriActivationImageOverlays_13;
    private boolean[][][] m_fmriActivationImageOverlays_23;

    boolean[][][]         m_dtiFiberImageOverlays_12;
    boolean[][][]         m_dtiFiberImageOverlays_13;
    boolean[][][]         m_dtiFiberImageOverlays_23;

    Color[][][]           m_dtiColorOverlay_12;
    Color[][][]           m_dtiColorOverlay_13;
    Color[][][]           m_dtiColorOverlay_23;

    private LinkedList <ActionListener> m_actionListeners = new LinkedList <ActionListener>( );

    public final static int THREE_D_VIEWER_DATASET_LOADED = 0;
    public final static int THREE_D_VIEWER_DATASET_LOAD_ERROR = -1;

    int m_offset_1_InPoints = 0;
    int m_offset_2_InPoints = 0;
    int m_offset_3_InPoints = 0;
}
