/*
 * File: $Id: DicomDecoder.java,v 1.4 2006/11/09 10:19:41 hans Exp $
 * History
 * Date: 28. Januar 2003, 07:28     Authors: Hans Slotboom Martin Buerki 
 * Changes:
 *
 * Copyright (C) 2003 - 2009 by
 * 
 * Inselspital - Department of Neuroradiology
 * CH 3010 Bern Switzerland
 */

package org.spectrim.dicom;


import java.io.*;
import java.nio.ByteBuffer;
import java.util.Properties;
import org.spectrim.lib.Convert;


// Below are the tags ordered for proper writing (after DICOM Filemeta information

//        public static final int CHARACTER_SET                  = 0x00080005;
//        public static final int SOP_CLASS_UID                  = 0x00080016;
//        public static final int SOP_INSTANCE_UID               = 0x00080018;
//        public static final int MANUFACTURER                   = 0x00080070;
//        public static final int IMAGE_MODALITY                 = 0x00080060;
//        public static final int IMAGE_TYPE                     = 0x00080008;
//        public static final int STUDY_DATE                     = 0x00080020;
//        public static final int SERIES_DATE                    = 0x00080021;
//        public static final int ACQUISITION_DATE               = 0x00080022;
//        public static final int CONTENT_DATE                   = 0x00080023;
//        public static final int STUDY_TIME                     = 0x00080030;
//        public static final int SERIES_TIME                    = 0x00080031;
//        public static final int ACQUISITION_TIME               = 0x00080032;
//        public static final int CONTENT_TIME                   = 0x00080033;
//        public static final int ACCESSION_NUMBER               = 0x00080050;
//        public static final int STUDY_DESCRIPTION              = 0x00081030;
//        public static final int SCANNER_MODEL                  = 0x00081090;
//        public static final int SERIES_DESCRIPTION             = 0x0008103e;
//
//        // Patient related
//        public static final int PATIENT_NAME                   = 0x00100010;
//        public static final int PATIENT_ID                     = 0x00100020;
//        public static final int PATIENT_BIRTHDATE              = 0x00100030;
//        public static final int PATIENT_SEX                    = 0x00100040;
//        public static final int PATIENT_AGE                    = 0x00101010;
//        public static final int PATIENT_WEIGHT                 = 0x00101030;
//
//
//        public static final int SLICE_THICKNESS                = 0x00180050;
//        public static final int TR                             = 0x00180080;
//        public static final int ECHO_TIME                      = 0x00180081;
//        public static final int IMAGE_FREQUENCY                = 0x00180084;
//        public static final int SLICE_SPACING                  = 0x00180088;
//        public static final int NUM_PHASE_ENCODES              = 0x00180089;
//        public static final int ECHO_TRAIN_LENGTH              = 0x00180091;
//        public static final int PERCENT_SAMPLING               = 0x00180093;
//        public static final int PERCENT_PHASE_FOV              = 0x00180094;
//        public static final int PIXEL_BANDWIDTH                = 0x00180095;
//        public static final int SEQUENCE_NAME                  = 0x00180024;
//        public static final int SOFTWARE_VERSION               = 0x00181020;
//        public static final int PROTOCOL_NAME                  = 0x00181030;
//        public static final int CONTRAST_BOLUS_VOLUME          = 0x00181041;
//        public static final int CONTRAST_FLOW_DURATION         = 0x00181047;
//        public static final int TRANSMIT_COIL_NAME             = 0x00181251;
//        public static final int PHASE_ENCODING_DIR             = 0x00181312;
//        public static final int FLIP_ANGLE                     = 0x00181314;
//        public static final int VARIABLE_FLIP_ANGLE_FLAG       = 0x00181315;
//        public static final int SAR                            = 0x00181316;
//        public static final int DBDT                           = 0x00181318;
//        public static final int PATIENT_POSITION               = 0x00185100;
//
//        public static final  int SERIES_INSTANCE_UID           = 0x0020000E;
//        public static final  int STUDY_INSTANCE_UID            = 0x0020000D;
//
//        public static final int STUDY_ID                       = 0x00200010;
//        public static final int SERIES_NUMBER                  = 0x00200011;
//        public static final int ACQUISITION_NUMBER             = 0x00200012;
//        public static final int INSTANCE_NUMBER                = 0x00200013;
//        public static final int IMAGE_POSITION                 = 0x00200032;
//        public static final int IMAGE_ORIENTATION              = 0x00200037;
//
//        public static final int SLICE_LOCATION                 = 0x00201041;
//
//
//        public static final  int SAMPLES_PER_PIXEL             = 0x00280002;
//        public static final  int PHOTOMETRIC_INTERPRETATION    = 0x00280004;
//        private static final int PLANAR_CONFIGURATION          = 0x00280006;
//        public static final  int NUMBER_OF_FRAMES              = 0x00280008;
//        public static final  int ROWS                          = 0x00280010;
//        public static final  int COLUMNS                       = 0x00280011;
//        public static final  int PIXEL_SPACING                 = 0x00280030;
//
//        public static final  int BITS_ALLOCATED                = 0x00280100;
//        public static final  int BITS_STORED                   = 0x00280101;
//        public static final  int HIGH_BIT                      = 0x00280102;
//        public static final  int PIXEL_REPRESENTATION          = 0x00280103;
//        public static final  int SMALLEST_PIXEL_VALUE          = 0x00280106;
//        public static final  int LARGEST_PIXEL_VALUE           = 0x00280107;
//
//        public static final  int WINDOW_CENTER                 = 0x00281050;
//        public static final  int WINDOW_WIDTH                  = 0x00281051;
//
//        public static final  int RED_PALETTE                   = 0x00281201;
//        public static final  int GREEN_PALETTE                 = 0x00281202;
//        public static final  int BLUE_PALETTE                  = 0x00281203;
//
//        // Private TAGS
//        public static final int SCANLAB_INFO_TAG               = 0x6FE00090;
//        public static final int JMRUI_INFO_TAG                 = 0x6FE00100;
//
//        public static final int  PIXEL_DATA                    = 0x7FE00010;




public class DicomDecoder
{
    public static final int GROUP_LENGTH                   = 0x00020000;
    public static final int META_INFO_VERSION              = 0x00020001;

    public static final int PIXEL_REPRESENTATION           = 0x00280103;
    public static final int MEDIA_STORAGE_SOP_CLASS_UID    = 0x00020002;
    public static final int MEDIA_STORAGE_SOP_INSTANCE_UID = 0x00020003;
    public static final int IMPLEMENTATION_CLASS_UID       = 0x00020012;
    public static final int IMPLEMENTATION_VERSION_NAME    = 0x00020013;


    public static final  int CHARACTER_SET                 = 0x00080005;
    private static final int SIEMENS_MEDCON_HEADER         = 0x00290012;

    public static final  int SCANNER_MODEL                 = 0x00081090;

    public static final  int SOP_CLASS_UID                 = 0x00080016;
    public static final  int SOP_INSTANCE_UID              = 0x00080018;
    public static final int  REFERENCED_SOP_INSTANCE_UID   = 0x00081150;
    
    public static final  int SERIES_INSTANCE_UID           = 0x0020000E;
    public static final  int STUDY_INSTANCE_UID            = 0x0020000D;
    
    public static final  int TRANSFER_SYNTAX_UID           = 0x00020010;
    public static final  int SLICE_SPACING                 = 0x00180088;
    public static final  int SAMPLES_PER_PIXEL             = 0x00280002;
    public static final  int PHOTOMETRIC_INTERPRETATION    = 0x00280004;
    private static final int PLANAR_CONFIGURATION          = 0x00280006;
    public static final  int NUMBER_OF_FRAMES              = 0x00280008;
    public static final  int ROWS                          = 0x00280010;
    public static final  int COLUMNS                       = 0x00280011;
    public static final  int PIXEL_SPACING                 = 0x00280030;
    public static final  int BITS_ALLOCATED                = 0x00280100;
    public static final  int WINDOW_CENTER                 = 0x00281050;
    public static final  int WINDOW_WIDTH                  = 0x00281051;
    public static final  int RED_PALETTE                   = 0x00281201;
    public static final  int GREEN_PALETTE                 = 0x00281202;
    public static final  int BLUE_PALETTE                  = 0x00281203;
    
    public static final int  PIXEL_DATA                    = 0x7FE00010;
    
    // Additionally added tags 
    public  static final int SLICE_LOCATION                = 0x00201041;
    public  static final int SLICE_THICKNESS               = 0x00180050;

    public static final int TR                             = 0x00180080;
    public static final int ECHO_TIME                      = 0x00180081;
    public static final int IMAGE_FREQUENCY                = 0x00180084;
    public static final int NUM_PHASE_ENCODES              = 0x00180089;
    public static final int ECHO_TRAIN_LENGTH              = 0x00180091;
    public static final int PERCENT_SAMPLING               = 0x00180093;
    public static final int PERCENT_PHASE_FOV              = 0x00180094;
    public static final int PIXEL_BANDWIDTH                = 0x00180095;
    
    public static final int SEQUENCE_NAME                  = 0x00180024;
    public static final int SOFTWARE_VERSION               = 0x00181020;
    public static final int PROTOCOL_NAME                  = 0x00181030;
   
    
    public static final int CONTRAST_BOLUS_VOLUME          = 0x00181041;
    public static final int CONTRAST_FLOW_DURATION         = 0x00181047;
    
    public static final int TRANSMIT_COIL_NAME             = 0x00181251;
    public static final int PHASE_ENCODING_DIR             = 0x00181312;
    public static final int FLIP_ANGLE                     = 0x00181314;
    public static final int VARIABLE_FLIP_ANGLE_FLAG       = 0x00181315;
    public static final int SAR                            = 0x00181316;
    public static final int DBDT                           = 0x00181318;
    public static final int PATIENT_POSITION               = 0x00185100;
    public static final int STUDY_ID                       = 0x00200010;
    public static final int SERIES_NUMBER                  = 0x00200011;
    public static final int ACQUISITION_NUMBER             = 0x00200012;
    public static final int INSTANCE_NUMBER                = 0x00200013;
    
    public static final int MANUFACTURER                   = 0x00080070;
    public static final int IMAGE_MODALITY                 = 0x00080060;
    
    public static final int IMAGE_POSITION                 = 0x00200032;
    public static final int IMAGE_ORIENTATION              = 0x00200037;
    
    public static final int BITS_STORED                    = 0x00280101;
    public static final int HIGH_BIT                       = 0x00280102;
    public static final int SMALLEST_PIXEL_VALUE           = 0x00280106;
    public static final int LARGEST_PIXEL_VALUE            = 0x00280107;
    
    public static final int PATIENT_NAME                   = 0x00100010;
    public static final int PATIENT_ID                     = 0x00100020;
    public static final int PATIENT_BIRTHDATE              = 0x00100030;
    public static final int PATIENT_SEX                    = 0x00100040;
    public static final int PATIENT_AGE                    = 0x00101010;

    public static final int PATIENT_WEIGHT                 = 0x00101030;
    public static final int CONTRAST_AGENT                 = 0x00180010;
    public static final int SCANNING_SEQUENCE              = 0x00180020;
    public static final int SEQUENCE_VARIANT               = 0x00180021;
    public static final int SCAN_OPTIONS                   = 0x00180022;
    public static final int ACQUISITION_TYPE               = 0x00180023;

    public static final int IMAGE_TYPE                     = 0x00080008;
    public static final int STUDY_DATE                     = 0x00080020;
    public static final int STUDY_DESCRIPTION              = 0x00081030;

    public static final int SERIES_DATE                    = 0x00080021;
    public static final int ACQUISITION_DATE               = 0x00080022;
    public static final int CONTENT_DATE                   = 0x00080023;
    
    public static final int STUDY_TIME                     = 0x00080030;
    public static final int SERIES_TIME                    = 0x00080031;

    public static final int SERIES_DESCRIPTION             = 0x0008103e;
    public static final int ACQUISITION_TIME               = 0x00080032;
    public static final int CONTENT_TIME                   = 0x00080033;
    public static final int ACCESSION_NUMBER               = 0x00080050;

    
    public static final int SCANLAB_INFO_TAG               = 0x6FE00090;
    public static final int JMRUI_INFO_TAG                 = 0x6FE00100;

    private static final int SPECTROSCOPIC_DATA            = 0x7FE11010;

    private static final int SPECTROSCOPIC_DATA_STANDARD_FORMAT       = 0x56000020;
    
    private static final int GE_PRIVAT_TAG                 = 0x0025101B;

    private static final int NORDIC_PIXELSCALING           = 0x00771001;

    // Angiography tags
    private static final int FLUOROSCOPY_STATION_NAME      = 0x00081010;
    private static final int IMAGE_AND_FLUOROSCOPY_AREA_DOSE_PRODUCT  = 0x0018115E;
    private static final int FLUOROSCOPY_EXPOSURE_TIME                = 0x00181150;
    
    
    private static final int FLUOROSCOPY_KVP                          = 0x00180060;
    private static final int FLUOROSCOPY_XRAY_TUBE_CURRENT            = 0x00181151;
    
    private static final int FLUOROSCOPY_ESTIMATED_RADIOGRAPH_MAGN_FACTOR = 0x00181114;
    
    private static final int FLUOROSCOPY_AVERAGE_PULSE_WIDTH              = 0x00181154;
    
    private static final int FLUOROSCOPY_DOSE                        = 0x00211005;
    private static final int FLUOROSCOPY_SKIN_DOSE_PERCENT           = 0x00211006;
    private static final int FLUOROSCOPY_SKIN_DOSE_ACCUMULATION      = 0x00211007;
    private static final int FLUOROSCOPY_SKIN_DOSE_RATE              = 0x00211008;
    private static final int FLUOROSCOPY_CONTRAST_BOLUS_AGENT        = 0x00180010;   
    private static final int BODY_PART_EXAMINED                      = 0x00180015;
    
    private static final int FLUOROSCOPY_RADIATION_SETTING           = 0x00181155;
    private static final int FLUOROSCOPY_RADIATION_MODE              = 0x0018115a;
    private static final int FLUOROSCOPY_INTENSIFIER_SIZE            = 0x00181162;
    private static final int FLUOROSCOPY_IMAGE_PIXEL_SPACING         = 0x00181164;
    private static final int FLUOROSCOPY_FOCAL_SPOTS                 = 0x00181190;
    private static final int FLUOROSCOPY_POSITIONER_PRIMARY_ANGLE    = 0x00181510;
    private static final int FLUOROSCOPY_POSITIONER_SECONDARY_ANGLE  = 0x00181511;
   
    private static final int FLUOROSCOPY_ACQUISITION_TYPE            = 0x00211000;
    private static final int FLUOROSCOPY_ACQUISITION_MODE            = 0x00211001;
    private static final int FLUOROSCOPY_FOOTSWITCH_INDEX            = 0x00211002;
    private static final int FLUOROSCOPY_CURRENT_TIME_PRODUCT        = 0x00211004;
    private static final int FLUOROSCOPY_COPPER_FILTER               = 0x0021100A;
    private static final int FLUOROSCOPY_SOURCE_TO_ISOCENTER         = 0x00211017;
    
    // CT specific DICOM tags for database
    
    // already defined element tag="0018,0060" name="KVP">140</element><element tag="0018,0090" vr="DS" vm="1" len="4"
    private static final int  CT_DataCollectionDiameter               = 0x00180090;
    private static final int  CT_ContrastBolusStartTime               = 0x00181042;
    private static final int  CT_ContrastBolusStopTime                = 0x00181043;
    private static final int  CT_ContrastBolusTotalDose               = 0x00181044;
    private static final int  CT_ContrastFlowRate                     = 0x00181046;
    private static final int  CT_ContrastFlowDuration                 = 0x00181047;
    
    private static final int  CT_ContrastBolusIngredientConcentration = 0x00181049;
    private static final int  CT_ReconstructionDiameter               = 0x00181100;
    private static final int  CT_DistanceSourceToDetector             = 0x00181110;
    private static final int  CT_DistanceSourceToPatient              = 0x00181111;
    private static final int  CT_GantryDetectorTilt                   = 0x00181120;
    private static final int  CT_TableHeight                          = 0x00181130;
    private static final int  CT_RotationDirection                    = 0x00181140;
    private static final int  CT_ExposureTime                         = 0x00181150;
    private static final int  CT_XRayTubeCurrent                      = 0x00181151;
    private static final int  CT_Exposure                             = 0x00181152;
    private static final int  CT_FilterType                           = 0x00181160;
    private static final int  CT_GeneratorPower                       = 0x00181170;
    private static final int  CT_FocalSpots                           = 0x00181190;
    private static final int  CT_DateOfLastCalibration                = 0x00181200;
    private static final int  CT_TimeOfLastCalibration                = 0x00181201;
    private static final int  CT_ConvolutionKernel                    = 0x00181210;
    private static final int  CT_SingleCollimationWidth               = 0x00189306;
    private static final int  CT_TotalCollimationWidth                = 0x00189307;
    private static final int  CT_TableSpeed                           = 0x00189309;
    private static final int  CT_TableFeedPerRotation                 = 0x00189310;
    private static final int  CT_SpiralPitchFactor                    = 0x00189311;
    private static final int  CT_DataCollectionCenterPatient          = 0x00189313;
    private static final int  CT_ReconstructionTargetCenterPatient    = 0x00189318;
    private static final int  CT_ExposureModulationType               = 0x00189323;
    private static final int  CT_EstimatedDoseSaving                  = 0x00189324;
    private static final int  CT_CTDIvol                              = 0x00189345;
    private static final int  CT_CTDIPhantomTypeCodeSequence          = 0x00189346;
    
    private static final int CT_CodeValue                             = 0x00080100;
    private static final int CT_CodingSchemeDesignator                = 0x00080102;
    private static final int CT_CodeMeaning                           = 0x00080104;
    
    private static final int CT_FeedPerRotation                       = 0x001910b0;
    
    
    public static final int  AE=0x4145, AS=0x4153, AT=0x4154, CS=0x4353, DA=0x4441, DS=0x4453, DT=0x4454,
                             FD=0x4644, FL=0x464C, IS=0x4953, LO=0x4C4F, LT=0x4C54, PN=0x504E, SH=0x5348, SL=0x534C, 
                             SS=0x5353, ST=0x5354, TM=0x544D, UI=0x5549, UL=0x554C, US=0x5553, UT=0x5554,
                             OB=0x4F42, OW=0x4F57, SQ=0x5351, UN=0x554E, QQ=0x3F3F, OF=0x4F46;
		
    private static Properties m_dictionary;

    private String              m_directory;
    private String              m_fileName;
    private InputStream         m_input_stream;
    private String              m_imageModality;
    private boolean             m_isSpectroscopyData = false;
    private static final int    m_ID_OFFSET    = 128;          // location of "m_DICM"
    private static final String m_DICM         = "DICM";
	
    private BufferedInputStream m_bufferedInputStream;
    
    private int                 m_location      = 0;
    private boolean             m_littleEndian  = true;
	
    private int                 m_elementLength;
    private int                 m_vr;                           // Value Representation
    private static final int    m_IMPLICIT_VR    = 0x2D2D;      // '--'
    
    private int                 m_previousGroup;
    private StringBuffer        m_dicomInfo = new StringBuffer( 10000 );
    private boolean             m_dicmFound;                // "m_DICM" found at offset 128
    private boolean             m_oddLocations;             // one or more tags at odd locations
    private boolean             m_bigEndianTransferSyntax = false;

    
    // Array storing the spectro byte data in case the dicom file stores 
    // spectroscopic data
    byte[]                      m_spectroscopicData;

    private boolean             m_compressedImageFormat = false;
    private boolean             m_pixelInformationReached = false;
    private boolean             m_is_SR_Data = false;
    // Changes needed to load VIDA data.
    int m_imageWidth = -1;
    int m_imageHeight = -1;
    int m_pixelBytesPerPixel = -1;
    boolean m_pixelInformationRead = false;
    int[] m_pixelData = null;
    
    /**
     * Constructor using a file in a file system as input.
     * @param directory
     * @param fileName
     */
    public DicomDecoder( String directory, String fileName ) 
    {
        m_directory = directory;
        m_fileName = fileName;
        m_input_stream = null;
        
        if ( m_dictionary == null )
        {
            DicomDictionary d = new DicomDictionary( );
            m_dictionary = d.getDictionary( );
        }
    }
    
    /**
     * Constructor using a file in a file system as input.
     * @param input_stream
     */
    public DicomDecoder( InputStream input_stream ) 
    {
        m_directory = null;
        m_fileName = null;
        
        m_input_stream = input_stream;
        
        if ( m_dictionary == null )
        {
            DicomDictionary d = new DicomDictionary( );
            m_dictionary = d.getDictionary( );
        }
    }
    
    
    /**
     * Returns the pixel data from the Dicom file (if any).
     * @return 
     */
    public int[] getPixelData( )
    {
       return m_pixelData;
    }
    
    /**
     * Reads a string from the input stream.
     */
    String getString( int length ) throws IOException 
    {
        // System.out.println( " Length (to read next) : " +  length );
        byte[] buf = new byte[length];
        int pos = 0;
        while ( pos < length )
        {
            int count = m_bufferedInputStream.read( buf, pos, length - pos );
            pos += count;
        }
        // System.out.println( "File location = " + location + " Length " +  length + " Read string = " + new String( buf ) );
        m_location += length;
        // System.out.println( "Value = " + new String( buf ) );
        return new String( buf );
    }

    /**
     * Returns a byte as int from the IO-channels.
     * @return
     * @throws java.io.IOException
     */
    int getByte( ) throws IOException 
    {
        int b = m_bufferedInputStream.read( );
        if ( b == -1 ) 
        {
            throw new IOException( "unexpected EOF" );
        }
        ++m_location;
        return b;
    }

    /**
     * Returns a short as int from the IO-channels.
     * @return
     * @throws java.io.IOException
     */
    int getShort( ) throws IOException 
    {
        int b0 = getByte( );
        int b1 = getByte( );
        if ( m_littleEndian )
        {
            return ( ( b1 << 8 ) + b0 );
        }
        else
        {
            return ( ( b0 << 8 ) + b1 );
        }
    }

    /**
     * Returns a int from the IO-stream.
     * @return
     * @throws java.io.IOException
     */
    final int getInt( ) throws IOException 
    {
        int b0 = getByte( );
        int b1 = getByte( );
        int b2 = getByte( );
        int b3 = getByte( );
        if ( m_littleEndian )
        {
            return ( ( b3 << 24 ) + ( b2 << 16 ) + ( b1 << 8 ) + b0 );
        }
        else
        {
            return ( ( b0 << 24 ) + ( b1 << 16 ) + ( b2 << 8 ) + b3 );
        }
    }

    /**
     * Reads one float typed information from the I/0-stream.
     * @return
     * @throws IOException 
     */
    final float getFloat( ) throws IOException
    {
        int b0 = getByte( );
        int b1 = getByte( );
        int b2 = getByte( );
        int b3 = getByte( );
        
        int asInt; 
        
        if ( m_littleEndian )
        {
            asInt =     (b0 & 0xFF ) 
                    | ( (b1 & 0xFF ) << 8 ) 
                    | ( (b2 & 0xFF ) << 16 ) 
                    | ( (b3 & 0xFF ) << 24 );

        }
        else
        {
            asInt =     (b3 & 0xFF ) 
                    | ( (b2 & 0xFF ) << 8 ) 
                    | ( (b1 & 0xFF ) << 16 ) 
                    | ( (b0 & 0xFF ) << 24 );

        }
        
        return Float.intBitsToFloat(asInt);
    }
    
    /**
     * Reads and returns a double from the IO-stream.
     * @return
     * @throws IOException 
     */
    final double getDouble( ) throws IOException
    {
        
        int len = 8;
        byte[] tmp = new byte[len];
        // staight forward
//        tmp[0] = (byte)getByte( );
//        tmp[1] = (byte)getByte( );
//        tmp[2] = (byte)getByte( );
//        tmp[3] = (byte)getByte( );
//        
//        tmp[4] = (byte)getByte( );
//        tmp[5] = (byte)getByte( );
//        tmp[6] = (byte)getByte( );
//        tmp[7] = (byte)getByte( );
        
        
        tmp[7] = (byte)getByte( );
        tmp[6] = (byte)getByte( );
        tmp[5] = (byte)getByte( );
        tmp[4] = (byte)getByte( );
        
        
        tmp[3] = (byte)getByte( );
        tmp[2] = (byte)getByte( );
        tmp[1] = (byte)getByte( );
        tmp[0] = (byte)getByte( );
        
        
        return ByteBuffer.wrap(tmp).getDouble();
    }
    
    /**
     * Returns a lut from IO-channel
     *
     * @param length
     * @return
     * @throws java.io.IOException
     */
    byte[] getLut( int length ) throws IOException 
    {
        if ( ( length & 1 ) != 0 ) 
        { // odd
            String dummy = getString( length );
            return null;
        }
        length /= 2;
        byte[] lut = new byte[length];
        for ( int i = 0; i < length; i++ )
        {
            lut[i] = (byte)( getShort( ) >>> 8 );
        }
        return lut;
    }

    /**
     * Returns the length of element to read from the DICOM file.
     * @return
     * @throws java.io.IOException
     */
    int getLength( ) throws IOException 
    {
        int b0 = getByte( );
        int b1 = getByte( );
        int b2 = getByte( );
        int b3 = getByte( );
		
        // We cannot know whether the VR is implicit or explicit
        // without the full DICOM Data Dictionary for public and
        // private groups.
		
        // We will assume the VR is explicit if the two bytes
        // match the known codes. It is possible that these two
        // bytes are part of a 32-bit length for an implicit VR.
		
        m_vr = ( b0 << 8 ) + b1;
	
//        if ( m_vr == UT )
//        {
//            System.out.println( "m_vr = UT" );
//        }
        
        switch ( m_vr )
        {
            case OB: case OW: case SQ: case UN: case OF: case UT:
                    // Explicit VR with 32-bit length if other two bytes are zero
                    if ( ( b2 == 0 ) || ( b3 == 0 ) ) return getInt( );
                    // Implicit VR with 32-bit length
                    m_vr = m_IMPLICIT_VR;
                    if ( m_littleEndian )
                    {
                        return ( ( b3 << 24 ) + ( b2 << 16 ) + ( b1 << 8 ) + b0 );
                    }
                    else
                    {
                        return ( ( b0 << 24 ) + ( b1 << 16 ) + ( b2 << 8 ) + b3 );
                    }
                    
            case AE: case AS: case AT: case CS: case DA: case DS: case DT:  case FD:
            case FL: case IS: case LO: case LT: case PN: case SH: case SL: case SS:
            case ST: case TM: case UI: case UL: case US: case QQ:
                    // Explicit m_vr with 16-bit length
                    if ( m_littleEndian )
                    {
                        return ( ( b3 << 8 ) + b2 );
                    }
                    else
                    {
                        return ( ( b2 << 8 ) + b3 );
                    }
            default:
                    // Implicit VR with 32-bit length...
                    m_vr = m_IMPLICIT_VR;
                    if ( m_littleEndian )
                    {
                        return ( ( b3 << 24 ) + ( b2 << 16 ) + ( b1 << 8 ) + b0 );
                    }
                    else
                    {
                        return ( ( b0 << 24 ) + ( b1 << 16 ) + ( b2 << 8 ) + b3 );
                    }
        }
    }

    /**
     * Returns the next tag from the I/O-stream.
     */
    int getNextTag( ) throws IOException 
    {
        int groupWord = getShort( );
        
        if ( groupWord == 0x0800 && m_bigEndianTransferSyntax )
        {
            m_littleEndian = false;
            groupWord = 0x0008;
        }

        int elementWord = getShort( );
        int tag = groupWord << 16 | elementWord;

        m_elementLength = getLength( );
        // System.out.println( "Element length = " + m_elementLength );


        // Hack needed to read some GE files
        // The element length must be even!
        if ( m_elementLength == 13 && !m_oddLocations )
        { 
            m_elementLength = 10;
        }
		
        // "Undefined" element length.
        // This is a sort of bracket that encloses a sequence of elements.
        if ( m_elementLength == -1 )
        {
            m_elementLength = 0;
        }

        return tag;
    }
  
    
    /**
     * Closes the buffered input stream.
     */
    public void closeInputStream( )
    {
        try
        {
            m_bufferedInputStream.close( );
        }
        catch ( IOException ex )
        {
            // Logger.getLogger(DicomDecoder.class.getName()).log(Level.SEVERE, null, ex);
            ex.printStackTrace();
            System.out.println( "IOException in DicomDecoder :: closeInputStream( ) : " );
        }
    }

    /**
     * Returns whether Dicom file contains SR data.
     * (Currently only 1.2.840.10008.5.1.4.1.1.88.33 and 1.2.840.10008.5.1.4.1.1.88.22 is handled).
     * @return 
     */
    boolean is_Dicom_SR_Data( )
    {
        return m_is_SR_Data;
    }
    
    
    /**
     * Returns whether the pixel information has successfully been read
     * from the I/0-stream.
     * @return 
     */
    public boolean getDicomImageDataRead( )
    {
        return m_pixelInformationRead;
    }
    
    
    /**
     * Returns a reference to an associated FileInfo object.
     * @return 
     * @throws java.io.IOException
     */
    public FileInfo getFileInfo( ) throws IOException 
    {
        long skipCount;
	
        FileInfo fi = new FileInfo( );
        
        int bitsAllocated = 16;
        fi.setFileFormat( FileInfo.RAW );
        fi.setFileName( m_fileName );
        fi.setDirectory( m_directory );
        fi.setImageWidth( 0 );
        fi.setImageHeight( 0 ); 
        fi.setFilePointerOffsetOfPixelData( 0 );
        fi.setIntelByteOrder( true );
        fi.setFileType( FileInfo.GRAY16_UNSIGNED );

        int planarConfiguration = 0;
        int samplesPerPixel = 1;
        String photoInterpretation = "";
        
        if ( ( m_directory != null ) && ( m_fileName != null ) )
        {               
            m_bufferedInputStream = new BufferedInputStream( new FileInputStream( m_directory + m_fileName ) );
        }
        else
        {
            m_bufferedInputStream = new BufferedInputStream( m_input_stream );
        }
        
        skipCount = (long)m_ID_OFFSET;
        while ( skipCount > 0 ) 
        { 
            skipCount -= m_bufferedInputStream.skip( skipCount );
        }
        
        m_location += m_ID_OFFSET;
		
        if ( !getString( 4 ).equals( m_DICM ) )
        { 
            
            if ( ( m_directory != null ) && ( m_fileName != null ) )
            {
                m_bufferedInputStream.close( );
                m_bufferedInputStream = new BufferedInputStream( new FileInputStream( m_directory + m_fileName ) );
            }
            else
            {   
                
                m_bufferedInputStream = new BufferedInputStream( m_input_stream );
                
            }
            m_location = 0;
        } 
        else 
        {
            m_dicmFound = true;
        }
		
        boolean inSequence = true;
        boolean m_decodingTags = true;
	
        while ( m_decodingTags )
        {
            int tag = -1;
            try
            {
                tag = getNextTag( );
            }
            catch ( IOException ioe )
            {
                System.out.println( "Warning: parsing beyond the end of the file!" );
                System.out.println( "Problematic File: " + m_fileName );
                 
                // ioe.printStackTrace( );
                m_bufferedInputStream.close( );
                m_decodingTags= false;
            }
                    
            
            if ( ( m_location == 5704 ) || ( m_location == 5716 ) )
            {
                System.out.println( "Location : " + m_location );
            }
            
            fi.setDataOffsetPointer( m_location );
            if ( ( m_location & 1 ) != 0 ) // DICOM tags must be at even locations
            {
                m_oddLocations = true;
            }
            
            String s = "null";
            
            switch ( tag ) 
            {

                    case GROUP_LENGTH:          int groupLength = getInt( );
                                                fi.setGroupLength( "" + groupLength );
                                                break;

                    case MANUFACTURER:
                                                s = getString( m_elementLength );
                                                fi.setManufacturer( s );
                                                break;

                    case CHARACTER_SET:         s = getString( m_elementLength );
                                                fi.setCharacterSet( s );
                                                break;

                    case SIEMENS_MEDCON_HEADER: //System.out.println( "SIEMENS_MEDCON_HEADER" );
                                                s = getString( m_elementLength );
                                                fi.setManufacturer( "SIEMENS" );
                                                break;
                        
                    case NORDIC_PIXELSCALING:   float f = getFloat( );
                                                fi.setNordicPixelScaling( f );
                                                break;
                        
                    case MEDIA_STORAGE_SOP_CLASS_UID:   s = getString( m_elementLength );
                                                        fi.setMediaStorageSOPClassUID( s );
                                                        break;

                    case IMAGE_AND_FLUOROSCOPY_AREA_DOSE_PRODUCT: s = getString( m_elementLength );
                                                                  //System.out.println( "IMAGE_AND_FLUOROSCOPY_AREA_DOSE_PRODUCT : " + s );
                                                                  fi.setImageAndFluoroscopyAreaDoseProduct( s );
                                                                  break;
                        
                    case FLUOROSCOPY_EXPOSURE_TIME              : s = getString( m_elementLength );
                                                                  //System.out.println( "EXPOSURE_TIME : " + s );
                                                                  fi.setFluoroscopyExposureTime( s );
                                                                  fi.setCTExposureTime( s );
                                                                  break;
    

                    case  FLUOROSCOPY_KVP                       : s = getString( m_elementLength );
                                                                  //System.out.println( "FLUOROSCOPY_KVP : " + s );
                                                                    fi.setFluoroscopyKVP( s );
                                                                    break;
                        
                        
                    case  FLUOROSCOPY_XRAY_TUBE_CURRENT        : s = getString( m_elementLength );
                                                                 //System.out.println( "XRAY_TUBE_CURRENT : " + s );
                                                                 fi.setFluoroscopyXRayTubeCurrent( s );
                                                                 fi.set_CTXRayTubeCurrent( s ); 
                                                                    
                                                                    break;
                        
                    case  FLUOROSCOPY_ESTIMATED_RADIOGRAPH_MAGN_FACTOR : s = getString( m_elementLength );
                                                                    //System.out.println( "ESTIMATED_RADIOGRAPH_MAGN_FACTOR : " + s );
                                                                    fi.setFluoroscopyEstimatedRadiographicMagnificationFactor( s );
                                                                    
                                                                    break;
    
                    case  FLUOROSCOPY_DOSE                         : if ( m_imageModality.compareTo("MR") == 0 )
                                                                     {
                                                                        s = getString( m_elementLength );
                                                                        
                                                                        fi.addUnknownSiemensProprietoryDicomTag( "0x00211005", s );
                                                                     }
                                                                     if ( m_imageModality.compareTo("XA") == 0 )
                                                                     {
                                                                        int iii = getInt( );
                                                                        // System.out.println( "DOSE : " + iii );
                                                                        fi.setFluoroscopyDose( "" + iii );
                                                                     }
                                                                     break;
                        
                    case  FLUOROSCOPY_SKIN_DOSE_PERCENT            : if ( m_imageModality.compareTo("MR") == 0 )
                                                                     {
                                                                        s = getString( m_elementLength );
                                                                        
                                                                        fi.addUnknownSiemensProprietoryDicomTag( "0x00211006", s );
                                                                     }
                                                                     if ( m_imageModality.compareTo("XA") == 0 )
                                                                     {
                                                                        int iii = getInt( );
                                                                        //System.out.println( "FLUOROSCOPY_SKIN_DOSE_PERCENT : " + iii );
                                                                        fi.setFluoroscopySkinDosePercent( "" + iii );
                                                                     }
                                                                     break;
                        
                    case  FLUOROSCOPY_SKIN_DOSE_ACCUMULATION       : if ( m_imageModality.compareTo("MR") == 0 )
                                                                     {
                                                                        s = getString( m_elementLength );
                                                                        
                                                                        fi.addUnknownSiemensProprietoryDicomTag( "0x00211007", s );
                                                                     }
                                                                     if ( m_imageModality.compareTo("XA") == 0 )
                                                                     {
                                                                        int iii = getInt( );
                                                                        // System.out.println( "SKIN_DOSE_ACCUMULATION : " + iii );
                                                                        fi.setFluoroscopySkinDoseAccumulation( "" + iii );
                                                                     }
                                                                     break;
                        
                    case  FLUOROSCOPY_SKIN_DOSE_RATE              :  if ( m_imageModality.compareTo("MR") == 0 )
                                                                     {
                                                                        s = getString( m_elementLength );
                                                                        
                                                                        fi.addUnknownSiemensProprietoryDicomTag( "0x00211008", s );
                                                                     }
                                                                     if ( m_imageModality.compareTo("XA") == 0 )
                                                                     {
                                                                        int iii = getInt( );
                                                                        // System.out.println( "SKIN_DOSE_RATE : " + iii );
                                                                        fi.setFluoroscopySkinDoseRate( "" + iii );
                                                                     }
                                                                     break;
                        
                    case FLUOROSCOPY_STATION_NAME                 :  s = getString( m_elementLength );
                                                                     // System.out.println( "FLUOROSCOPY_STATION_NAME : " + s );
                                                                     fi.setFluoroscopyStationName( s );
                                                                     break;
                    
                    case FLUOROSCOPY_CONTRAST_BOLUS_AGENT         :  s = getString( m_elementLength );
                                                                     // System.out.println( "CONTRAST_BOLUS_AGENT : " + s );
                                                                     fi.setContrastBolusAgentName( s );
                                                                     fi.setContrastAgent( s );
                                                                     break;
                                            
                    case BODY_PART_EXAMINED                       :  s = getString( m_elementLength );
                                                                     //System.out.println( "BODY_PART_EXAMINED : " + s );
                                                                     fi.setBodyPartExamined( s );                                             
                                                                     break;
                        
                        
                    case FLUOROSCOPY_AVERAGE_PULSE_WIDTH          :  s = getString( m_elementLength );
                                                                     //System.out.println( "AVERAGE_PULSE_WIDTH : " + s );
                                                                     fi.setFluoroscopyAveragePulseWidth( s );                                             
                                                                     break;
                        
                    case FLUOROSCOPY_RADIATION_SETTING           :   s = getString( m_elementLength );
                                                                     //System.out.println( "FLUOROSCOPY_RADIATION_SETTING : " + s );
                                                                     fi.setFluoroscopyRadiationSetting( s );                                             
                                                                     break;
                    case FLUOROSCOPY_RADIATION_MODE               :  s = getString( m_elementLength );
                                                                     //System.out.println( "FLUOROSCOPY_RADIATION_MODE : " + s );
                                                                     fi.setFluoroscopyRadiationMode( s );                                             
                                                                     break;
                    case FLUOROSCOPY_INTENSIFIER_SIZE             :  s = getString( m_elementLength );
                                                                     //System.out.println( "FLUOROSCOPY_INTENSIFIER_SIZE : " + s );
                                                                     fi.setFluoroscopyIntensifierSize( s );                                             
                                                                     break;
                    case FLUOROSCOPY_IMAGE_PIXEL_SPACING          :  s = getString( m_elementLength );
                                                                     //System.out.println( "FLUOROSCOPY_IMAGE_PIXEL_SPACING : " + s );
                                                                     fi.setFluoroscopyImagePixelSpacing( s );                                             
                                                                     break;
                    case FLUOROSCOPY_FOCAL_SPOTS                  :  s = getString( m_elementLength );
                                                                     //System.out.println( "FLUOROSCOPY_FOCAL_SPOTS : " + s );
                                                                     fi.setFluoroscopyFocalSpot( s );
                                                                     fi.set_CT_FocalSpot( s );
                                                                     break;
                    case FLUOROSCOPY_POSITIONER_PRIMARY_ANGLE     :  s = getString( m_elementLength );
                                                                     //System.out.println( "FLUOROSCOPY_POSITIONER_PRIMARY_ANGLE : " + s );
                                                                     fi.setFluoroscopyPositionerPrimaryAngle( s );                                             
                                                                     break;
                    case FLUOROSCOPY_POSITIONER_SECONDARY_ANGLE   :  s = getString( m_elementLength );
                                                                     //System.out.println( "FLUOROSCOPY_POSITIONER_SECONDARY_ANGLE : " + s );
                                                                     fi.setFluoroscopyPositionerSecondaryAngle( s );                                             
                                                                     break;
   
                    case FLUOROSCOPY_ACQUISITION_TYPE             :  int ii = getShort( );
                                                                     //System.out.println( "FLUOROSCOPY_ACQUISITION_TYPE : " + ii );
                                                                     fi.setFluoroscopyAcquisitionType( "" + ii );                                             
                                                                     break;
                                                                     
                                                                     // Below a hack to get multiple used private tags correctly parsed
                                                                     // ### start ###
                    case FLUOROSCOPY_ACQUISITION_MODE             :  if ( m_imageModality.compareTo("MR") == 0 )
                                                                     {
                                                                        s = getString( m_elementLength );
                                                                        
                                                                        fi.addUnknownSiemensProprietoryDicomTag( "0x00211001", s );
                                                                     }
                    
                                                                     if ( m_imageModality.compareTo("XA") == 0 )
                                                                     {
                                                                      
                                                                        ii = getShort( );
                                                                        // System.out.println( "FLUOROSCOPY_ACQUISITION_MODE : " + ii );
                                                                        fi.setFluoroscopyAcquisitionMode( "" + ii );
                                                                     }
                                                                     
                                                                     break;
                                                                     
                    case FLUOROSCOPY_FOOTSWITCH_INDEX             :  if ( m_imageModality.compareTo("MR") == 0 )
                                                                     {
                                                                        s = getString( m_elementLength );
                                                                        
                                                                        fi.addUnknownSiemensProprietoryDicomTag( "0x00211002", s );
                                                                     }
                    
                                                                     if ( m_imageModality.compareTo("XA") == 0 )
                                                                     {
                                                                        ii = getShort( );
                                                                        // System.out.println( "FLUOROSCOPY_FOOTSWITCH_INDEX : " + ii );
                                                                        fi.setFluoroscopyFootSwitchIndex( "" + ii );
                                                                     }
                                                                     
                                                                     break;
                    case FLUOROSCOPY_CURRENT_TIME_PRODUCT         :  if ( m_imageModality.compareTo("MR") == 0 )
                                                                     {
                                                                        s = getString( m_elementLength );
                                                                        
                                                                        fi.addUnknownSiemensProprietoryDicomTag( "0x00211004", s );
                                                                     }
                    
                                                                     if ( m_imageModality.compareTo("XA") == 0 )
                                                                     {
                                                                        float flt = getInt( );
                                                                        // System.out.println( "FLUOROSCOPY_CURRENT_TIME_PRODUCT : " + flt );
                                                                        fi.setFluoroscopyCurrentTimeProduct( "" + flt );                                             
                                                                     }
                                                                     break;
                                                                     
                    case FLUOROSCOPY_COPPER_FILTER                :  if ( m_imageModality.compareTo("MR") == 0 )
                                                                     {
                                                                        s = getString( m_elementLength );
                                                                        
                                                                        fi.addUnknownSiemensProprietoryDicomTag( "0x0021100A", s );
                                                                     }
                                                                     if ( m_imageModality.compareTo("XA") == 0 )
                                                                     {
                                                                        int iii = getInt( );
                                                                        // System.out.println( "FLUOROSCOPY_COPPER_FILTER : " + iii );
                                                                        fi.setFluoroscopyCopperFilter( "" + iii );                                             
                                                                     }
                                                                     break;
                                                                     
                    case FLUOROSCOPY_SOURCE_TO_ISOCENTER          :  if ( m_imageModality.compareTo("MR") == 0 )
                                                                     {
                                                                        s = getString( m_elementLength );
                                                                        
                                                                        fi.addUnknownSiemensProprietoryDicomTag( "0x00211017", s );
                                                                     }
                                                                     if ( m_imageModality.compareTo("XA") == 0 )
                                                                     {
                                                                        ii = getInt( );
                                                                        // System.out.println( "FLUOROSCOPY_SOURCE_TO_ISOCENTER : " + ii );
                                                                        fi.setFluoroscopySourceToIsoCenter( "" + ii );
                                                                     }
                                                                     break;  
                    
                    case CT_DataCollectionDiameter                  : s = getString( m_elementLength );
                                                                     fi.set_CT_DataCollectionDiameter( s );
                                                                     break;
                        
                        
                    case CT_ContrastBolusStartTime                  : s = getString( m_elementLength );
                                                                      fi.set_CT_ContrastBolusStartTime( s );
                                                                      break;
                    
                        
                    case CT_ContrastBolusStopTime                   : s = getString( m_elementLength );
                                                                      fi.set_CT_ContrastBolusStopTime( s );
                                                                      break;    
                    
                        
                    case CT_ContrastBolusTotalDose                  : s = getString( m_elementLength );
                                                                      fi.set_CT_ContrastBolusTotalDose( s );
                                                                      break;    
                        
                        
                    case CT_ContrastFlowRate                        : s = getString( m_elementLength );
                                                                      fi.set_CT_ContrastFlowRate( s );
                                                                      break;
                        
                        
                        
                    case  CT_ContrastFlowDuration                    :  s = getString( m_elementLength );
                                                                        fi.set_CT_ContrastFlowDuration( s );
                                                                        fi.setContrastFlowDuration( s );
                                                                        addInfo( tag, s  );
                                                                        break;
                                                                      
                    case  CT_ContrastBolusIngredientConcentration    :  s = getString( m_elementLength );
                                                                        fi.set_CT_ContrastBolusIngredientConcentration( s );
                                                                        break;
                        
                    case  CT_ReconstructionDiameter               :     s = getString( m_elementLength );
                                                                        fi.set_CT_ReconstructionDiameter( s );
                                                                        break;
                        
                    case  CT_DistanceSourceToDetector             :     s = getString( m_elementLength );
                                                                        fi.set_CT_DistanceSourceToDetector( s );
                                                                        break;
                    case  CT_DistanceSourceToPatient              :     s = getString( m_elementLength );
                                                                        fi.set_CT_DistanceSourceToPatient( s );
                                                                        break;
                    case  CT_GantryDetectorTilt                   :     s = getString( m_elementLength );
                                                                        fi.set_CT_GantryDetectorTilt( s );
                                                                        break;
                        
                    case  CT_TableHeight                          :     s = getString( m_elementLength );
                                                                        System.out.println( "table_height = " + s );
                                                                        fi.set_CT_TableHeight( s );
                                                                        break;
                        
                    case  CT_RotationDirection                    :     s = getString( m_elementLength );
                                                                        fi.set_CT_RotationDirection( s );
                                                                        break;
                    
                    case  CT_Exposure                             :     s = getString( m_elementLength );
                                                                        fi.set_CT_Exposure( s );
                                                                        break;
                        
                    case  CT_FilterType                           :     s = getString( m_elementLength );
                                                                        fi.set_CT_FilterType( s );
                                                                        break;
                        
                    case  CT_GeneratorPower                       :     s = getString( m_elementLength );
                                                                        fi.set_CT_GeneratorPower( s );
                                                                        break;
                    
                    case  CT_DateOfLastCalibration                :     s = getString( m_elementLength );
                                                                        fi.set_CT_DateOfLastCalibration( s );
                                                                        break;
                        
                    case  CT_TimeOfLastCalibration                :     s = getString( m_elementLength );
                                                                        fi.set_CT_TimeOfLastCalibration( s );
                                                                        break;
                        
                    case  CT_ConvolutionKernel                    :     s = getString( m_elementLength );
                                                                        fi.set_CT_ConvolutionKernel( s );
                                                                        break;
                        
                    case  CT_SingleCollimationWidth               :     double s_col = getDouble( );
                                                                        fi.set_CT_SingleCollimationWidth( s_col );
                                                                        // System.out.println( "Single collimation width : " + s_col );
                                                                        break;
                        
                    case  CT_TotalCollimationWidth                :     double t_col = getDouble( );
                                                                        fi.set_CT_TotalCollimationWidth( t_col );
                                                                        // System.out.println( "Total collimation width : " + t_col );
                                                                        break;
                        
                    case  CT_TableSpeed                           :     double t_speed = getDouble( );
                                                                        fi.set_CT_TableSpeed( t_speed );
                                                                        break;
                        
                    case  CT_TableFeedPerRotation                 :     double feed = getDouble( );
                                                                        fi.set_CT_TableFeedPerRotation( feed );
                                                                        break;
                        
                    case  CT_SpiralPitchFactor                    :     double pitch = getDouble( );
                                                                        fi.set_CT_SpiralPitchFactor( pitch );
                                                                        break;
                        
                    case  CT_DataCollectionCenterPatient          :     s = getString( m_elementLength );
                                                                        fi.set_CT_DataCollectionCenterPatient( s );
                                                                        break;
                        
                    case  CT_ReconstructionTargetCenterPatient    :     s = getString( m_elementLength );
                                                                        fi.set_CT_ReconstructionTargetCenterPatient( s );
                                                                        break;
                        
                    case  CT_ExposureModulationType               :     s = getString( m_elementLength );
                                                                        fi.set_CT_ExposureModulationType( s );
                                                                        break;
                        
                    case  CT_EstimatedDoseSaving                  :     double dose_saving = getDouble( );
                                                                        fi.set_CT_EstimatedDoseSaving( dose_saving );
                                                                        break;
                        
                    case  CT_CTDIvol                              :     double ctdiVol = getDouble( );
                                                                        fi.set_CT_CTDIvol( ctdiVol );
                                                                        break;
                        
                    case CT_CodeValue                             :     s = getString( m_elementLength );
                                                                        fi.set_CT_CodeValue( s );
                                                                        break;
                        
                    case CT_CodingSchemeDesignator                :     s = getString( m_elementLength );
                                                                        fi.set_CT_CodingSchemeDesignator( s );
                                                                        break;
                        
                    case CT_CodeMeaning                           :     s = getString( m_elementLength );
                                                                        fi.set_CT_CodeMeaning( s );
                                                                        break;
    
                    case CT_FeedPerRotation                       :     s = getString( m_elementLength );
                                                                        fi.set_CT_FeedPerRotation( s );
                                                                        break;
                        
                        
                                                                     
                    case IMPLEMENTATION_CLASS_UID: //System.out.println( "IMPLEMENTATION_CLASS_UID" );
                                                   s = getString( m_elementLength );
                                                   fi.setImplementationClassUID( s );
                                                   break;

                    case MEDIA_STORAGE_SOP_INSTANCE_UID: 
                                                // System.out.println( "MEDIA_STORAGE_SOP_INSTANCE_UID" );
                                                s = getString( m_elementLength );
                                                fi.setMediaStorageSOPInstanceUID( s );
                                                break;
                                                
                    case SOP_INSTANCE_UID:      // System.out.println( "SOP_INSTANCE_UID" );
                                                s = getString( m_elementLength );
                                                fi.setSOPInstanceUID( s );
                                                break;
                                                
                                                
                                                
                    case REFERENCED_SOP_INSTANCE_UID: s = getString( m_elementLength );
//                                                     if ( s.contains( "1.2.840.10008.3.1.2.3.3" ) == true )
//                                                     {
//                                                        System.out.println( "Philips Modality Performed Procedure Step SOP Class." );
//                                                        m_is_SR_Data = true;
//                                                        fi.set_Is_SR_Data( true );
//                                                        m_decodingTags = false;
//                                                     }
                                                     fi.setReferencedSOPClassUID( s );
                                                     break;                            
                                                
                    case SOP_CLASS_UID:         // System.out.println( "SOP_CLASS_UID" );
                                                s = getString( m_elementLength );
                                                if ( s.contains( "1.2.840.10008.5.1.4.1.1.88.33" ) == true )
                                                {
                                                    System.out.println( "SOP_CLASS_UID = 1.2.840.10008.5.1.4.1.1.88.33 -> SR data" );
                                                    m_decodingTags = false;
                                                    // This SOP_CLASS_UID is a structured report associated to MRS (Siemens)
                                                    m_is_SR_Data = true;
                                                    fi.set_Is_SR_Data( true );
                                                }
                                                
                                                if ( s.contains( "1.2.840.10008.5.1.4.1.1.88.22" ) == true )
                                                {
                                                    System.out.println( "SOP_CLASS_UID = 1.2.840.10008.5.1.4.1.1.88.22 -> Phoenix data" );
                                                    m_decodingTags = false;
                                                    // This SOP_CLASS_UID is a structured report associated to MRS (Siemens)
                                                    m_is_SR_Data = true;
                                                    fi.set_Is_SR_Data( true );
                                                }
            
                                                fi.setSOPClassUID( s );
                                                break;

                    case SERIES_INSTANCE_UID:   //System.out.println( "SERIES_INSTANCE_UID" );
                                                s = getString( m_elementLength );
                                                fi.setSeriesInstanceUID( s );
                                                break;
                                                
                    case STUDY_INSTANCE_UID:    //System.out.println( "STUDY_INSTANCE_UID" );
                                                s = getString( m_elementLength );
                                                fi.setStudyInstanceUID( s );
                                                break;

                    case STUDY_DESCRIPTION:     //System.out.println( "STUDY_DESCRIPTION" );

                                                s = getString( m_elementLength );
                                                // System.out.println( "STUDY_DESCRIPTION = " + s );
                                                fi.setStudyDescription( s );
                                                break;

                    case TRANSFER_SYNTAX_UID:   // System.out.println( "TRANSFER_SYNTAX_UID" );
                                                s = getString( m_elementLength );
//                                              //  System.out.println( "Transfer syntax uid : " + s );
                                                addInfo( tag, s );
                                                if ( s.indexOf( "1.2.4" ) > -1 || s.indexOf( "1.2.5" ) > -1 )
                                                {
                                                      // m_compressedImageFormat = true;
//                                                    m_bufferedInputStream.close( );
//                                                    String msg = "Cannot open compressed DICOM images.\n \n";
//                                                    msg += "Transfer Syntax UID = " + s;
//                                                    //return fi;
//                                                    throw new IOException( msg );
                                                }

                                                fi.setTransferSyntaxUID( s );
                                                
                                                if ( s.indexOf( "1.2.840.10008.1.2.2" ) >= 0 )
                                                {    
                                                    m_bigEndianTransferSyntax = true;
                                                }

                                                break;

                    case NUMBER_OF_FRAMES:      //System.out.println( "NUMBER_OF_FRAMES" );
                                                s = getString( m_elementLength );
                                                addInfo( tag, s );
                                                fi.setNumberOfFrames( s );
                                                break;

                    case SAMPLES_PER_PIXEL:     //System.out.println( "SAMPLES_PER_PIXEL" );
                                                samplesPerPixel = getShort( );
                                                fi.setSamplesPerPixel( "" + samplesPerPixel );
                                                addInfo( tag, fi.getSamplesPerPixel( ) );
                                                break;

                    case PHOTOMETRIC_INTERPRETATION:
                                                //System.out.println( "PHOTOMETRIC_INTERPRETATION" );
                                                photoInterpretation = getString( m_elementLength );
                                                fi.setPhotometricInterpretation( photoInterpretation );
                                                addInfo( tag, fi.getPhotometricInterpretation( ) );
                                                break;

                    case PLANAR_CONFIGURATION:
                                                //System.out.println( "PLANAR_CONFIGURATION" );
                                                planarConfiguration = getShort( );
                                                fi.setPlanarConfiguration( planarConfiguration );
                                                addInfo( tag, fi.getPlanarConfiguration( ) );
                                                break;

                    case ROWS:                  // System.out.println( "ROWS" );
                                                m_imageHeight = getShort( );
                                                fi.setImageHeight( m_imageHeight );
                                                fi.setInitialImageHeight( m_imageHeight );
                                                addInfo( tag, fi.getImageHeight( ) );
                                                break;

                    case COLUMNS:               //System.out.println( "COLUMNS" );
                                                m_imageWidth = getShort( );

                                                fi.setImageWidth( m_imageWidth );
                                                fi.setInitialImageWidth( m_imageWidth );

                                                addInfo( tag, fi.getImageWidth( ) );
                                                break;

                    case PIXEL_SPACING:         // System.out.println( "PIXEL_SPACING" );
                                                String scale = getString( m_elementLength );
                                                getSpatialScale( fi, scale );
                                                addInfo( tag, scale );
                                                break;

                    case SLICE_SPACING:         //System.out.println( "SLICE_SPACING" );
                                                String spacing = getString( m_elementLength );
                                                fi.setPixelDepth( spacing );
                                                addInfo( tag, spacing );
                                                break;

                    case BITS_ALLOCATED:        //System.out.println( "BITS_ALLOCATED" );
                                                bitsAllocated = getShort( );
                                                m_pixelBytesPerPixel = bitsAllocated/8;
                                                fi.setBitsAllocated( "" + bitsAllocated );

                                                if ( bitsAllocated == 8 )
                                                {
                                                    fi.setFileType( FileInfo.GRAY8 );
                                                }
                                                addInfo( tag, bitsAllocated );
                                                break;

                    case PIXEL_REPRESENTATION:  // System.out.println( "PIXEL_REPRESENTATION" );
                                                int pixelRepresentation = getShort( );
                                                fi.setPixelRepresentation( new String( "" + pixelRepresentation ).trim( ) );
                                                if ( pixelRepresentation == 1 )
                                                {
                                                    fi.setFileType( FileInfo.GRAY16_SIGNED );
                                                }
                                                addInfo( tag, pixelRepresentation );
                                                break;

                    case WINDOW_CENTER:         //System.out.println( "WINDOW_CENTER" );
                                                String windowCenter = getString( m_elementLength );
                                                fi.setWindowCenter( windowCenter );
                                                addInfo( tag, windowCenter );
                                                break;

                    case WINDOW_WIDTH:          //System.out.println( "WINDOW_WIDTH" );
                                                String windowWidth = getString( m_elementLength );
                                                fi.setWindowWidth( windowWidth );
                                                addInfo( tag, windowWidth );
                                                break;

                    case RED_PALETTE:           //System.out.println( "RED_PALETTE" );
                                                fi.setReds( getLut( m_elementLength ) );
                                                addInfo( tag, m_elementLength/2 );
                                                break;

                    case GREEN_PALETTE:         //System.out.println( "GREEN_PALETTE" );
                                                fi.setGreens( getLut( m_elementLength ) );
                                                addInfo( tag, m_elementLength/2 );
                                                break;

                    case BLUE_PALETTE:          //System.out.println( "BLUE_PALETTE" );
                                                fi.setBlues( getLut( m_elementLength ) );
                                                addInfo( tag, m_elementLength/2 );
                                                break;

                    case PIXEL_DATA:            // System.out.println( "PIXEL_DATA" );
                                                m_pixelInformationReached = true;
                                                // Start of image data...
                                                if ( m_elementLength != 0 )
                                                {
                                                    
                                                    fi.setFilePointerOffsetOfPixelData( m_location );
                                                    addInfo( tag, m_location );
                                                    m_decodingTags = false;
                                                    // try directly to read the image data ..
                                                    
                                                    int bDataSize = m_pixelBytesPerPixel * m_imageWidth * m_imageHeight;
                                                    // int imageSize = m_headerInfo.getImageWidth( ) * m_headerInfo.getImageHeight( );
                                                    byte[] bData = new byte[ bDataSize ];
                                                    
                                                    int b = m_bufferedInputStream.read( bData, 0, bDataSize );
                                                    if ( b == -1 ) 
                                                    {
                                                        throw new IOException( "unexpected EOF" );
                                                    }
                                                    
                                                    m_pixelData = new int[ bDataSize / 2 ];
                                                    for ( int n = 0; n < bDataSize/2; n++ )
                                                    {
                                                        // System.out.println( "Number " + n );
                                                        m_pixelData[ n ] = Convert.byte2intl( bData, n << 1 );
                                                    }
                                                    m_pixelInformationRead = true;
                                                    
                                                }
                                                else
                                                {
                                                    addInfo( tag, null );
                                                }

                                                if ( m_compressedImageFormat == true )
                                                {
                                                    m_bufferedInputStream.close( );
                                                }
                                                break;

                    case 0x7F880010:            //System.out.println( "0x7F880010" );
                                                // What is this? - RAK
                                                if ( m_elementLength != 0 )
                                                {
                                                    fi.setFilePointerOffsetOfPixelData( m_location + 4 );
                                                    m_decodingTags = false;
                                                }
                                                break;

                    case SLICE_LOCATION:        //System.out.println( "SLICE_LOCATION" );
                                                String sliceLoc = getString( m_elementLength );
                                                fi.setSliceLocation( sliceLoc );
                                                addInfo( tag, sliceLoc );
                                                break;

                    case SLICE_THICKNESS:       //System.out.println( "SLICE_THICKNESS" );
                                                String sliceThickness = getString( m_elementLength );
                                                fi.setSliceThickness( sliceThickness );
                                                addInfo( tag, fi.getSliceThickness( ) );
                                                break;

                    case TR:                    //System.out.println( "TR" );
                                                String TRep = getString( m_elementLength );
                                                fi.setTR( TRep );
                                                addInfo( tag, fi.getTR( ) );
                                                break;

                    case ECHO_TIME:             //System.out.println( "ECHO_TIME" );
                                                String TE = getString( m_elementLength );
                                                fi.setEchoTime( TE );
                                                addInfo( tag, fi.getEchoTime( ) );
                                                break;

                    case IMAGE_FREQUENCY:       //System.out.println( "IMAGE_FREQUENCY" );
                                                String image_frequency = getString( m_elementLength );
                                                fi.setImagingFrequency( image_frequency );
                                                addInfo( tag, image_frequency );
                                                break;

                    case NUM_PHASE_ENCODES:     //System.out.println( "NUM_PHASE_ENCODES" );
                                                String num_phase_enc = getString( m_elementLength );
                                                fi.setNumberOfPhaseEncodes( num_phase_enc );
                                                addInfo( tag, num_phase_enc );
                                                break;
                                                
                    case ECHO_TRAIN_LENGTH:     //System.out.println( "ECHO_TRAIN_LENGTH" );
                                                String echo_train_length = getString( m_elementLength );
                                                fi.setEchoTrainLength( echo_train_length );
                                                addInfo( tag, echo_train_length  );
                                                break;

                    case PERCENT_SAMPLING:      //System.out.println( "PERCENT_SAMPLING" );
                                                String percent_sampling = getString( m_elementLength );
                                                fi.setPercentSampling(  percent_sampling );
                                                addInfo( tag, percent_sampling  );
                                                break;

                    case PERCENT_PHASE_FOV:     //System.out.println( "PERCENT_PHASE_FOV" );
                                                String percent_phase_fov = getString( m_elementLength );
                                                fi.setPercentPhaseFOV( percent_phase_fov );
                                                addInfo( tag, percent_phase_fov  );
                                                break;

                    case PIXEL_BANDWIDTH:       //System.out.println( "PIXEL_BANDWIDTH" );
                                                String pixel_bandwidth = getString( m_elementLength );
                                                fi.setPixelBandwidth( pixel_bandwidth );
                                                addInfo( tag, pixel_bandwidth  );
                                                break;

                    case SEQUENCE_NAME:         //System.out.println( "SEQUENCE_NAME" );
                                                String sequence_name = getString( m_elementLength );
                                                fi.setSequenceName( sequence_name );
                                                addInfo( tag, sequence_name );
                                                break;

                    case SOFTWARE_VERSION:      //System.out.println( "SOFTWARE_VERSION" );
                                                String software_version = getString( m_elementLength );
                                                fi.setSoftwareVersion( software_version );
                                                addInfo( tag, software_version );
                                                break;

                    case PROTOCOL_NAME:         String protocol_name = getString( m_elementLength );
                                                fi.setProtocolName( protocol_name );
                                                addInfo( tag, protocol_name );
                                                break;

                    case CONTRAST_BOLUS_VOLUME: //System.out.println( "CONTRAST_BOLUS_VOLUME" );
                                                String contrast_bolus_vol = getString( m_elementLength );
                                                fi.setContrastBolusVolume( contrast_bolus_vol );                                            
                                                addInfo( tag, contrast_bolus_vol  );
                                                break;
                        
                    case TRANSMIT_COIL_NAME:    //System.out.println( "TRANSMIT_COIL_NAME" );
                                                String transmit_coil_name = getString( m_elementLength );
                                                fi.setTransmitCoilName( transmit_coil_name );
                                                addInfo( tag, transmit_coil_name );
                                                break;

                    case PHASE_ENCODING_DIR:    //System.out.println( "PHASE_ENCODING_DIR" );
                                                String phase_encoding_dir = getString( m_elementLength );
                                                fi.setPhaseEncodingDirection( phase_encoding_dir );
                                                addInfo( tag, phase_encoding_dir );
                                                break;

                    case FLIP_ANGLE:            //System.out.println( "FLIP_ANGLE" );
                                                String flip_angle = getString( m_elementLength );
                                                fi.setFlipAngle( flip_angle );
                                                addInfo( tag, flip_angle  );
                                                break;

                    case VARIABLE_FLIP_ANGLE_FLAG:
                                                //System.out.println( "VARIABLE_FLIP_ANGLE_FLAG" );
                                                String var_flip_angle_flag = getString( m_elementLength );
                                                fi.setVariableFlipAngleFlag( var_flip_angle_flag );
                                                addInfo( tag, var_flip_angle_flag );
                                                break;

                    case SAR:                   //System.out.println( "SAR" );
                                                String sar  = getString( m_elementLength );
                                                fi.setSAR( sar );
                                                addInfo( tag, sar  );
                                                break;

                    case DBDT:                  //System.out.println( "DBDT" );
                                                String dBdt  = getString( m_elementLength );
                                                fi.setdBdt( dBdt );
                                                addInfo( tag, dBdt  );
                                                break;

                    case PATIENT_POSITION:      //System.out.println( "PATIENT_POSITION" );
                                                String patient_position = getString( m_elementLength );
                                                fi.setPatientPosition( patient_position );
                                                addInfo( tag, patient_position );
                                                break;
                                                
                    case STUDY_ID:              //System.out.println( "STUDY_ID" );
                                                String studyID = getString( m_elementLength );
                                                fi.setStudyID( studyID );
                                                addInfo( tag, fi.getStudyID( ) );
                                                break;

                    case SERIES_NUMBER:         //System.out.println( "SERIES_NUMBER" );
                                                String seriesNumber = getString( m_elementLength );
                                                fi.setSeriesNumber( seriesNumber );
                                                addInfo( tag, fi.getSeriesNumber( ).trim( ) );
                                                break;
                                                
                    case ACQUISITION_NUMBER:    //System.out.println( "ACQUISITION_NUMBER" );
                                                fi.setAcquisitionNumber( getString( m_elementLength ) );
                                                addInfo( tag, fi.getAcquisitionNumber( ).trim( ) );
                                                break;

                    case INSTANCE_NUMBER:       //System.out.println( "INSTANCE_NUMBER" );
                                                String instanceNumber = getString( m_elementLength );
                                                fi.setInstanceNumber( instanceNumber );
                                                addInfo( tag, fi.getInstanceNumber( ).trim( ) );
                                                break;

                    case IMAGE_POSITION:        //System.out.println( "IMAGE_POSITION" );
                                                String image_position = getString( m_elementLength );
                                                fi.setImagePositionPatient( image_position );
                                                addInfo( tag, image_position );
                                                break;

                    case IMAGE_ORIENTATION:     //System.out.println( "IMAGE_ORIENTATION" );
                                                String image_orientation = getString( m_elementLength );
                                                fi.setImageOrientationPatient( image_orientation );
                                                addInfo( tag, image_orientation );
                                                break;

                    case BITS_STORED:           //System.out.println( "BITS_STORED" );
                                                fi.setBitsStored( "" + getShort( ) );
                                                addInfo( tag, fi.getBitsStored( ) );
                                                break;
                                                
                    case HIGH_BIT:              //System.out.println( "HIGH_BIT" );
                                                fi.setHighBit( "" + getShort( ) );
                                                addInfo( tag, fi.getBitsStored( ) );
                                                break;

                    case SMALLEST_PIXEL_VALUE:  //System.out.println( "SMALLEST_PIXEL_VALUE" );
                                                fi.setSmallestPixelValue( "" + getShort( ) );
                                                addInfo( tag, fi.getSmallestPixelValue( ) );
                                                break;

                    case LARGEST_PIXEL_VALUE:   //System.out.println( "LARGEST_PIXEL_VALUE" );
                                                fi.setLargestPixelValue( "" + getShort( ) );
                                                addInfo( tag, fi.getLargestPixelValue( ) );
                                                break;

                    case PATIENT_NAME:          //System.out.println( "PATIENT_NAME" );
                                                String patient_name = getString( m_elementLength );
                                                fi.setPatientName( patient_name );
                                                addInfo( tag, patient_name );
                                                break;

                    case PATIENT_ID:            //System.out.println( "PATIENT_ID" );
                                                String patient_id = getString( m_elementLength );
                                                fi.setPatientID( patient_id );
                                                addInfo( tag, patient_id );
                                                break;

                    case PATIENT_BIRTHDATE:     //System.out.println( "PATIENT_BIRTHDATE" );
                                                String patient_birthdate = getString( m_elementLength );
                                                fi.setPatientBirthDate( patient_birthdate );
                                                addInfo( tag, patient_birthdate );
                                                break;

                    case PATIENT_SEX:           //System.out.println( "PATIENT_SEX" );
                                                String patient_sex = getString( m_elementLength );
                                                fi.setPatientSex( patient_sex );
                                                addInfo( tag, patient_sex );
                                                break;

                    case PATIENT_AGE:           //System.out.println( "PATIENT_AGE" );
                                                String patient_age = getString( m_elementLength );
                                                fi.setPatientAge( patient_age );
                                                addInfo( tag, patient_age );
                                                break;

                    case PATIENT_WEIGHT:        //System.out.println( "PATIENT_WEIGHT" );
                                                String patient_weight = getString( m_elementLength );
                                                fi.setPatientWeight( patient_weight );
                                                addInfo( tag, patient_weight );
                                                break;

                    case SCANNING_SEQUENCE:     //System.out.println( "SCANNING_SEQUENCE" );
                                                String scanning_sequence = getString( m_elementLength );
                                                fi.setScanningSequence( scanning_sequence );
                                                addInfo( tag, scanning_sequence );
                                                break;

                    case SEQUENCE_VARIANT:      //System.out.println( "SEQUENCE_VARIANT" );
                                                String sequence_variant = getString( m_elementLength );
                                                fi.setSequenceVariant( sequence_variant );
                                                addInfo( tag, sequence_variant );
                                                break;

                    case SCAN_OPTIONS:          //System.out.println( "SCAN_OPTIONS" );
                                                String scan_options = getString( m_elementLength );
                                                fi.setScanOptions( scan_options );
                                                addInfo( tag, scan_options );
                                                break;

                    case ACQUISITION_TYPE:      //System.out.println( "ACQUISITION_TYPE" );
                                                String acq_type = getString( m_elementLength );
                                                fi.setAcquisitionType( acq_type );
                                                addInfo( tag, acq_type );
                                                break;
                                            
                    case IMAGE_TYPE:            //System.out.println( "IMAGE_TYPE" );
                                                String image_type = getString( m_elementLength );
                                                fi.setImageType( image_type );
                                                addInfo( tag, image_type );
                                                break;

                    case STUDY_DATE:            //System.out.println( "STUDY_DATE" );
                                                String study_date = getString( m_elementLength );
                                                fi.setStudyDate( study_date );
                                                addInfo( tag, study_date );
                                                break;

                    case SERIES_DATE:           //System.out.println( "SERIES_DATE" );
                                                String series_date = getString( m_elementLength );
                                                fi.setSeriesDate( series_date );
                                                addInfo( tag, series_date );
                                                break;
                                                
                    case ACQUISITION_DATE:      //System.out.println( "ACQUISITION_DATE" );
                                                String acq_date = getString( m_elementLength );
                                                fi.setAcquisitionDate( acq_date );
                                                addInfo( tag, acq_date );
                                                break;

                    case CONTENT_DATE:          //System.out.println( "CONTENT_DATE" );
                                                String content_date = getString( m_elementLength );
                                                fi.setContentDate( content_date );
                                                addInfo( tag, content_date );
                                                break;
                                            
                    case STUDY_TIME:            //System.out.println( "STUDY_TIME" );
                                                String studyTime = getString( m_elementLength );
                                                fi.setStudyTime( studyTime );
                                                addInfo( tag, studyTime );
                                                break;

                    case SERIES_TIME:           //System.out.println( "SERIES_TIME" );
                                                String seriesTime = getString( m_elementLength );
                                                fi.setSeriesTime( seriesTime );
                                                addInfo( tag, fi.getSeriesTime( ) );
                                                break;

                    case ACQUISITION_TIME:      //System.out.println( "ACQUISITION_TIME" );
                                                String acquisitionTime = getString( m_elementLength );
                                                fi.setAcquisitionTime( acquisitionTime );
                                                addInfo( tag, fi.getAcquisitionTime( ) );
                                                break;

                    case CONTENT_TIME:          //System.out.println( "CONTENT_TIME" );
                                                fi.setContentTime( getString( m_elementLength ) );
                                                addInfo( tag, fi.getContentTime( ) );
                                                break;

                    case ACCESSION_NUMBER:      //System.out.println( "ACCESSION_NUMBER" );
                                                String accession_number = getString( m_elementLength );
                                                fi.setAccessionNumber( accession_number );
                                                addInfo( tag, accession_number );
                                                break;
                                               
                    case SERIES_DESCRIPTION:    // System.out.println( "SERIES_DESCRIPTION" );
                                                String series_description = getString( m_elementLength );
                                                fi.setSeriesDescription( series_description );
                                                addInfo( tag, fi.getSeriesDescription( ) );
                                                break;
                                               
                    case SPECTROSCOPIC_DATA:    // System.out.println( "SPECTROSCOPIC_DATA" );
                                                // Siemens proprietory data
                                                getSpectroscopicDataFromFile( m_elementLength );
                                                fi.setNonImageData( m_spectroscopicData );
                                                fi.setSpectrum( );
                                                m_isSpectroscopyData = true;
                                                m_spectroscopicData = null;
                                                m_decodingTags = false;
                                                break;
                                                
                    case SPECTROSCOPIC_DATA_STANDARD_FORMAT:  
                                                System.out.println( "Spectroscopy standard Dicom format." );
                                                // Siemens VIDA spectro data.
                                                m_isSpectroscopyData = true;
                                                getSpectroscopicDataFromFile( m_elementLength );
                                                fi.setNonImageData( m_spectroscopicData );
                                                fi.setSpectrum( );
                                                // m_spectroscopicData = null;
                                                m_decodingTags = false;
                                                break;
                                               
                    case IMAGE_MODALITY:        // System.out.println( "IMAGE_MODALITY" );
                                                m_imageModality = getString( m_elementLength );
                                                fi.setImageModality( m_imageModality );
                                                addInfo( tag, fi.getImageModality( ) );
                                                break;

//                    case GE_PRIVAT_TAG:       getSpectroscopicDataFromFile( m_elementLength );
//                                              fi.setNonImageData( m_spectroscopicData );
//                                              fi.setSpectrum( );
//                                              m_spectroscopicData = null;
//                                              decodingTags = false;
//                                              break;

                    case SCANLAB_INFO_TAG:      // System.out.println( "SCAN_INFO_TAG" );
                                                String SCANInfo = getString( m_elementLength );
                                                fi.setSCANInfo( SCANInfo );
                                                addInfo( tag, fi.getSCANInfo( ) );
                                                break;

                    case JMRUI_INFO_TAG:        // System.out.println( "JMRUI_INFO_TAG" );
                                                String JMRUI_Info = getString( m_elementLength );
                                                fi.setJMRUIInfo( JMRUI_Info );
                                                addInfo( tag, fi.getJMRUIInfo( ) );
                                                break;

                    case SCANNER_MODEL:
                                                String scannerModelName = getString( m_elementLength );
                                                fi.setScannerModelName( scannerModelName );
                                                addInfo( tag, fi.getScannerModelName( ) );
                                                break;
                             
                    default:
                                                // System.out.println( "default" );
                                                // Not used, skip over it...
                                                // System.out.println( "tag skipped ..." + m_elementLength );
                                                addInfo( tag, null );
            }

            if ( ( m_compressedImageFormat == true ) && ( m_pixelInformationReached == true ) )
            {
                break;
            }
            
           /// System.out.println( "tag value : " + s );
            
        } // while(decodingTags)

        if ( fi.getFileType( ) == FileInfo.GRAY8 ) 
        {
            if ( fi.getReds( ) != null && fi.getGreens( ) != null && fi.getBlues( ) != null
                 && fi.getReds( ).length == fi.getGreens( ).length
                 && fi.getReds( ).length == fi.getBlues( ).length )
            {
                    fi.setFileType( FileInfo.COLOR8 );
                    fi.setLutSize( fi.getReds( ).length );
            }
        }

        if ( samplesPerPixel == 3 && photoInterpretation.startsWith( "RGB" ) ) 
        {
            if ( planarConfiguration == 0 )
            {
                fi.setFileType( FileInfo.RGB );
            }
            else if ( planarConfiguration == 1 )
            {
                fi.setFileType( FileInfo.RGB_PLANAR );
            }
        } 
        else if ( photoInterpretation.endsWith( "1 " ) )
        {
            fi.setWhiteIsZero( true );
        }

        if ( !m_littleEndian )
        {
            fi.setIntelByteOrder( false );
        }	
	
        m_bufferedInputStream.close( );
        m_pixelInformationRead = true;
        return fi;
    }
	
    
    /**
     * Reads the spectroscopic data from the Dicom file.
     */
    private void getSpectroscopicDataFromFile( int length ) throws IOException
    {
        // Read the spectroscopic data into a byte array
        m_spectroscopicData = new byte[length];
        
        int count = m_bufferedInputStream.read( m_spectroscopicData, 0, length );
        
        m_location += length;
    }


    /**
     * Returns the DICOM info string.
     * @return
     */
    String getDicomInfo( ) 
    {
        return new String( m_dicomInfo );
    }



    /**
     *
     * Adds the header info tag.
     *
     * @param tag
     * @param value
     * @throws java.io.IOException
     */
    void addInfo( int tag, String value ) throws IOException 
    {
        String info = getHeaderInfo( tag, value );
            
        if ( info != null ) 
        {
            int group = tag >>> 16;
            if ( group != m_previousGroup )
            { 
                m_dicomInfo.append( "\n" );
            }
            m_previousGroup = group;
            m_dicomInfo.append( Tools.tag2hex( tag ) + info + "\n" );
        }
    }

    /**
     *
     * Adds the header info tag.
     *
     * @param tag
     * @param value
     * @throws java.io.IOException
     */
    void addInfo( int tag, int value ) throws IOException 
    {
        addInfo( tag, Integer.toString( value ) );
    }

    String getHeaderInfo( int tag, String value ) throws IOException 
    {
        String key = i2hex( tag );
        //while ( key.length() < 8 )
        //	key = '0' + key;
        String id = (String)m_dictionary.get( key );
        if ( id != null ) 
        {
            if ( m_vr == m_IMPLICIT_VR && id != null )
            {
                m_vr = ( id.charAt( 0 ) << 8 ) + id.charAt( 1 );
            }
            id = id.substring( 2 );
        }
        if ( value != null )
        {
            return id + ": " + value;
        }
        switch ( m_vr )
        {
            case AE: 
            case AS: 
            case AT:
            case CS: 
            case DA: 
            case DS:
            case DT:
            case IS:
            case LO: 
            case LT:
            case PN: 
            case SH:
            case ST:
            case TM:
            case UI:
           
                        value = getString( m_elementLength );
                        // System.out.println( "### Tag : " + tag + " === Value : " + value + "  element length : " + m_elementLength );
			break;
            case US:
                        if ( m_elementLength == 2 )
                        {
                            value = Integer.toString( getShort( ) );
                        }
                        else
                        {
                            value = "";
                            int n = m_elementLength / 2;
                            for ( int i = 0; i < n; i++ )
                            {
                                value += Integer.toString( getShort( ) ) + " ";
                            }
                        }
                        break;
                        
            case UT:    
                        value = getString( m_elementLength );
                        // System.out.println( "### Tag : " + tag + " === Value : " + value + "  element length : " + m_elementLength );
                        m_location += m_elementLength;
                        
			break;
                        
            default:	
                        long skipCount = (long)m_elementLength;
                        while ( skipCount > 0 ) 
                        { 
                            skipCount -= m_bufferedInputStream.skip( skipCount );
                        }
                        m_location += m_elementLength;
                        value = "";
        }
        if ( id == null )
        {
            return null;
        }
        else
        {
            return id + ": " + value;
        }
    }

    static char[] buf8 = new char[8];
	
    /** 
     * Converts an int to an 8 byte hex string.
     */
    String i2hex( int i ) 
    {
        for ( int pos = 7; pos >= 0; pos-- ) 
        {
            buf8[pos] = Tools.hexDigits[i&0xf];
            i >>>= 4;
        }
        return new String( buf8 );
    }

    /**
     * Sets the pixel spacing attributes in th FileInfo object.
     * @param fi
     * @param scale
     */
    void getSpatialScale( FileInfo fi, String scale ) 
    {
        fi.setPixelSpacing( scale );
        
        String xscale = null;
        String yscale = null;

        if ( ( scale != null ) && ( scale.compareTo( "" ) != 0 ) )
        {
            int i = scale.indexOf( '\\' );

            if ( i > 0 )
            {
                xscale = scale.substring( 0, i );
                yscale = scale.substring( i + 1 );
            }

            if ( ( xscale == null ) || ( yscale == null ) )
            {
                fi.setPixelWidth( "N.A." );
                fi.setPixelHeight( "N.A." );
                fi.setUnit( "mm" );
            }

            if ( new Double( xscale ).doubleValue( ) != 0.0 &&
                 new Double( yscale ).doubleValue( ) != 0.0 )
            {
                fi.setPixelWidth( xscale );
                fi.setPixelHeight( yscale );
                fi.setUnit( "mm" );
            }
        }
        else
        {
            fi.setPixelWidth( "N.A." );
            fi.setPixelHeight( "N.A." );
            fi.setUnit( "mm" );
        }
    }

    /**
     * Returns the m_dicmFound boolean object.
     * @return
     */
    boolean dicmFound( ) 
    {
    	return m_dicmFound;
    }

    /**
     * Returns whether the decoded Dicom file contains spectroscopy information.
     * @return 
     */
    boolean isSpectroscopyData() {
        return m_isSpectroscopyData;
    }
}
