/*
 * Image3DSlicerDefaults.java
 *
 * Created on May 3, 2006, 2:57 PM
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

/*
 * File: $Id: DicomDefaults.java,v 1.2 2006/05/09 11:25:55 hans Exp $
 * History
 * Date: 02.06.2004, 13:06  Authors: Hans Slotboom  
 *
 * Changes:
 *
 * Copyright (C) 2003 by
 * 
 * Inselspital - Department of Neuroradiology
 * CH 3010 Bern Switzerland
 */

package org.spectrim.dicom;

import java.util.Properties;

import java.io.BufferedInputStream;
import java.io.FileInputStream;
import java.io.InputStream;

import java.io.BufferedOutputStream;
import java.io.FileOutputStream;
import java.io.OutputStream;

import java.io.IOException;

/**
 * Singleton class that contains all user preferred settings
 * @author  hans
 */
public class DicomDefaults extends Properties
{
    
    /** 
     * Creates a new instance of PerfusionDefaults 
     */
    private DicomDefaults( ) 
    {
        setProperty( DICOM_MOUNT_PATH, System.getProperty( "user.home" ) );        
        setProperty( DEFAULT_IMAGE_MODALITY_TYPE, IMAGE_MODALITY_MR );
    }
    
    /**
     * Returns the one and onnly instance of MRFittingDefaults
     */
    public static DicomDefaults getInstance( )
    {
        if ( theInstance == null )
        {
            theInstance = new DicomDefaults( );
        }
        return theInstance;
    }

    /**
     * Loads the user preferences file
     */ 
    public void load( )
    {
        String userPreferenceFileName = System.getProperty( "user.dir" ) + System.getProperty( "file.separator" ) + "DicomDefaults.ini";
        try
        {
            InputStream is = new BufferedInputStream( new FileInputStream( userPreferenceFileName ) );
        
            // now the ini file must exist with default contents, so load it
            
            this.load( is );
            is.close( );
        }
        catch ( IOException e )
        {
            try
            {
                OutputStream os = new BufferedOutputStream( new FileOutputStream( userPreferenceFileName ) );
                this.save( os, "// This is a generated file: do not edit!!" );
                os.close( );
            }
            catch ( IOException ioe )
            { 
                // should hopefully never occur
            }
        }
        
        // now the ini file must exist with default contents, so load it
        try
        {
            InputStream is = new BufferedInputStream( new FileInputStream( userPreferenceFileName ) );
            this.load( is );
            is.close( );
        }
        catch ( IOException e )
        {
            // should never occur
        }
    }
    
   /**
     * Saves the user preferences file
     */ 
    public void save( )
    {
        try
        {
            String userPreferenceFileName = System.getProperty( "user.dir" ) + System.getProperty( "file.separator" ) + "DicomDefaults.ini";
            OutputStream out = new BufferedOutputStream( new FileOutputStream( userPreferenceFileName ) );
            this.save( out, "//This is a generated file : do not edit" );
        }
        catch ( IOException e )
        {
            // should not occur
        }
    }
    // static private attribute is the instance itself (Singleton!)
    private static  DicomDefaults theInstance = null;
    
    public final static String DICOM_MOUNT_PATH = "Dicom mount path";
    public final static String DEFAULT_IMAGE_MODALITY_TYPE = "Image modality";
    
    public final static String IMAGE_MODALITY_MR = "MR";
    public final static String IMAGE_MODALITY_CT = "CT";
    public final static String IMAGE_MODALITY_OTHER = "Other";
    
    // Must be declarated somehow
    private static final long serialVersionUID = 3487455845819293L;
}
