/*
 * File: $Id: DicomFileSelectionControl.java,v 1.1.1.1 2006/04/19 09:38:47 hans Exp $
 * History
 * Date: 12. April 2003, 13:43 Hans Slotboom  Changes:
 *
 * Copyright (C) 2003 by
 * 
 * Inselspital - Department of Neuroradiology
 * CH 3010 Bern Switzerland
 */
package org.spectrim.dicom;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.event.MouseListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;

import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.JTree;
import javax.swing.tree.TreePath;
import javax.swing.tree.TreeSelectionModel;
import javax.swing.JPopupMenu;
import javax.swing.JMenuItem;

import javax.swing.JScrollPane;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.table.DefaultTableModel;

import javax.swing.BoxLayout;
import javax.swing.BorderFactory;

import java.util.TreeMap;

import java.util.LinkedList;
import java.util.Iterator;
import java.util.Collection;
import java.util.Enumeration;
import java.util.Hashtable;
import javax.swing.ImageIcon;
import javax.swing.tree.DefaultTreeCellRenderer;
import javax.swing.tree.TreeNode;
import org.spectrim.patientbrowser.ModalityIdentifier;
import org.spectrim.patientbrowser.OrientationIdentifier;
import org.spectrim.patientbrowser.Patient;
import org.spectrim.patientbrowser.PatientList;
import org.spectrim.patientbrowser.Series;
import org.spectrim.patientbrowser.Study;

/**
 *
 * Control allowing the use to select a complete image series or some file of a
 * image series.
 *
 * @author hans
 */
public class DicomFileSelectionControl extends JPanel 
{

    /**
     * Creates a new instance of DicomFileSelectionControl.
     */
    public DicomFileSelectionControl() 
    {
        m_treePanel = new JPanel();
        m_treePanel.setLayout(new BoxLayout(m_treePanel, BoxLayout.X_AXIS));
        m_treePanel.setBorder(BorderFactory.createEmptyBorder(3, 3, 3, 3));
        createNewTreeControl();
        m_tablePanel = new JPanel();
//        m_tablePanel.setLayout(new BoxLayout(m_tablePanel, BoxLayout.X_AXIS));
        m_tablePanel.setLayout(new BorderLayout());
        m_tablePanel.setBorder(BorderFactory.createEmptyBorder(3, 3, 3, 3));

//        Object[][] cellData = { { "", "", "", "", "", "", "" } };
//        String[] columnNames = { "<HTML><B>File name</B></HTML>",
//                                 "<HTML><B>Type</B></HTML>",
//                                 "<HTML><B>Instance number</B></HTML>",
//                                 "<HTML><B>Slice Position [mm]</B></HTML>",
//                                 "<HTML><B>FOV [mm]x[mm]</B></HTML>",
//                                 "<HTML><B>TE [ms]</B></HTML>",
//                                 "<HTML><B>TR [ms]</B></HTML>"
//                               };
//        
//        m_dicomImageSeriesTable = new JTable( cellData, columnNames );
        m_dicomImageSeriesTable = new DicomFileSelectionTable();

        // JTree does not provide a JScrollPane automatically so we have to 
        // add it ourselves
        m_treeScrollPane = new JScrollPane();
//        m_treeScrollPane.setMinimumSize(new Dimension(200, 200));
//        m_treeScrollPane.setPreferredSize(new Dimension(200, 200));
        m_treeScrollPane.getViewport().setView(m_dicomDirectoryTree);

        m_treePanel.add(m_treeScrollPane);

        // JList does not provide a scroll list automatically so we have to 
        // add it ourselves.
        m_listScrollPane = new JScrollPane();
        m_listScrollPane.getViewport().setView(m_dicomImageSeriesTable);
//        m_listScrollPane.setPreferredSize(new Dimension(600, 200));

        m_tablePanel.add(m_listScrollPane);

//        setLayout(new BoxLayout(this, BoxLayout.X_AXIS));
        setLayout( new BorderLayout( ) );

//        setMinimumSize(new Dimension(810, 210));
//        setPreferredSize(new Dimension(810, 210));
        setBorder( BorderFactory.createCompoundBorder(
                   BorderFactory.createLoweredBevelBorder( ),
                   BorderFactory.createTitledBorder( "Dicom file/series selector" ) ) );

        add( m_treePanel );
        add( m_tablePanel );
        // check for system and define separator
        m_separator = File.separator;

    }

    /**
     * Sets the series names present in the selected DICOM directory.
     * @param dicomDir
     */
    public void setDicomDir( DicomDirectory dicomDir ) 
    {
        m_dicomDir = dicomDir;
        setDicomDirectoryTree( );
        m_directoryIsMounted = true;
    }

    /**
     * Redefines the selection behavior of the JTree
     * @param multiSelectable
     */
    public void setTreeMultiSelectable( boolean multiSelectable ) 
    {
        m_dicomDirectoryTree.getSelectionModel( ).setSelectionMode( TreeSelectionModel.CONTIGUOUS_TREE_SELECTION );
    }

    /**
     * Creates the tree control.
     */
    private void createNewTreeControl( ) 
    {
        m_treeRoot = new DefaultMutableTreeNode("<html>Mounted Dicom Directory <font color=\"blue\">(Right-click to select directory to mount)</font></html>");

        m_dicomDirectoryTree = null;
        m_dicomDirectoryTree = new JTree(m_treeRoot);
        // define new renderer which changes the favicon in the treeview
        //System.out.println(System.getProperty("user.dir"));
        m_dicomDirectoryTree.setCellRenderer(new DefaultTreeCellRenderer() {
            //private Icon saveIcon = UIManager.getIcon("OptionPane.informationIcon");
            @Override
            public Component getTreeCellRendererComponent(JTree tree,
                    Object value, boolean selected, boolean expanded,
                    boolean isLeaf, int row, boolean focused) {
                Component c = super.getTreeCellRendererComponent(tree, value,
                        selected, expanded, isLeaf, row, focused);
                // get node argument
                Object nodeObj = ((DefaultMutableTreeNode) value).getUserObject();
                // check if node argument is of type series and then change the favicon accordingly
                if (isLeaf && nodeObj instanceof SerieType) {
                    setText(((SerieType) nodeObj).getName());
                    if (((SerieType) nodeObj).getSpectrumBool()) {
                        setIcon(new ImageIcon("images" + m_separator + "icons" + m_separator + "spectrum.png"));
                    } else {
                        setIcon(new ImageIcon("images" + m_separator + "icons" + m_separator + "brain.png"));
                    }
                }
                return c;
            }
        });
        m_dicomDirectoryTree.getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);

        // Adds a mouse listener for the 
        m_treeMouseListener = new MouseAdapter() {
            @Override
            public void mousePressed(MouseEvent e) {
                int selRow = m_dicomDirectoryTree.getRowForLocation(e.getX(), e.getY());
                TreePath selPath = m_dicomDirectoryTree.getPathForLocation(e.getX(), e.getY());

                if (selPath != null) {
                    DefaultMutableTreeNode node = (DefaultMutableTreeNode) selPath.getLastPathComponent();

                    if ((selRow != -1) && (!node.isRoot())) {
                        if (e.getClickCount() == 1) {
                            updateDicomImageSeriesList(selRow, selPath);
                            notifyListeners(TREE_ENTRY_SELECTED_EVENT, TREE_ENTRY_SELECTED_CMD);
                        } else if (e.getClickCount() == 2) {
                            notifyListeners(TREE_ENTRY_DOUBLE_CLICKED_EVENT, TREE_ENTRY_DBL_CLICKED_CMD);
                        }
                    }
                }

                if (e.isPopupTrigger()) {
                    if (m_directoryIsMounted == true) {
                        // Enable all other popup menu elements
                        for (int i = 0; i < m_popupMenu.getComponentCount(); i++) {
                            if (m_popupMenu.getComponent(i) instanceof JMenuItem) {
                                ((JMenuItem) m_popupMenu.getComponent(i)).setEnabled(true);
                            }
                        }
                    }
                    m_popupMenu.show((JComponent) e.getSource(), e.getX(), e.getY());
                }
            }
            // called whenever the right mouse button is pressed

            @Override
            public void mouseReleased(MouseEvent e) {
                if (e.isPopupTrigger()) {
                    if (m_directoryIsMounted == true) {
                        // Enable all other popup menu elements
                        for (int i = 0; i < m_popupMenu.getComponentCount(); i++) {
                            if (m_popupMenu.getComponent(i) instanceof JMenuItem) {
                                ((JMenuItem) m_popupMenu.getComponent(i)).setEnabled(true);
                            }
                        }
                    }
                    m_popupMenu.show((JComponent) e.getSource(), e.getX(), e.getY());
                }
            }
        };

        m_dicomDirectoryTree.addMouseListener(m_treeMouseListener);
    }

    /**
     * Sets the series names present in the selected DCIOM directory.
     */
    public void clearDicomDir() {
        if (m_seriesNames != null) {
            m_seriesNames.clear();
        }

        Object[][] cellData = {{"", "", "", "", "", "", ""}};

        m_cellData = cellData;

        String[] columnNames = {"<HTML><B>File name</B></HTML>",
            "<HTML><B>Type</B></HTML>",
            "<HTML><B>Instance number</B></HTML>",
            "<HTML><B>Slice Position [mm]</B><HTML>",
            "<HTML><B>FOV [mm]x[mm]</B><HTML>",
            "<HTML><B>TE [ms]</B><HTML>",
            "<HTML><B>TR [ms]</B><HTML>"
        };

        m_tableModel = new DefaultTableModel(cellData, columnNames);

        if (m_dicomImageSeriesTable != null && m_tableModel != null) {
            m_dicomImageSeriesTable.setModel(m_tableModel);
        }

        if (m_dicomImageSeriesTable != null) {
            m_dicomImageSeriesTable.updateUI();
        }
        if (m_treeRoot != null) {
            m_treeRoot.removeAllChildren();
        }
        if (m_dicomDirectoryTree != null) {
            m_dicomDirectoryTree.updateUI();
        }

    }

    /**
     * Adds elements to the DICOM study -> seriesN tree.
     */
    private void setDicomDirectoryTree() 
    {
        
        m_seriesNames = m_dicomDir.getSeriesDescriptions( );

        Hashtable<String, FileInfo> dicominfo = m_dicomDir.getDicomHeaderInformation( );

        Iterator<FileInfo> iter = dicominfo.values( ).iterator( );

        patientList = new PatientList( );
        
        while ( iter.hasNext( ) ) 
        {
            FileInfo info = iter.next( );
            if ( patientList.hasPatient( info.getPatientName( ) + info.getPatientBirthDate( ) ) ) 
            {
                Patient patient = patientList.getPatient( info.getPatientName( ) + info.getPatientBirthDate( ) );
                if ( patient.hasStudy( info.getStudyDate( ) + info.getStudyDescription( ) ) ) 
                {
                    Study study = patient.getStudy( info.getStudyDate( ) + info.getStudyDescription( ) );
                    if ( study.hasSeries( info.getSeriesInstanceUID( ) ) ) 
                    {
                        Series series = study.getSeries( info.getSeriesInstanceUID( ) );
                        series.addFile( info.getDirectory( ) + m_separator + info.getFileName( ) );
                    } 
                    else 
                    {
                        Series series;
                        if ( info.getSeriesDescription( ) != null )
                        {
                            series = new Series( info.getSeriesDescription( ), 
                                                 info.getSeriesNumber(), 
                                                 info.getSeriesInstanceUID(), 
                                                 info.getSeriesDate(), 
                                                 ModalityIdentifier.getModality( info.getSeriesDescription( ).split( "-" )[0] ), 
                                                 info.isSpectrum( ), 
                                                 OrientationIdentifier.getOrientation(info.getImageOrientationPatient( ) ) );
                        }
                        else
                        {
                            String alterNativeSeriesDescription = info.getSequenceName( ).replace( '*', ' ' );
                            series = new Series( alterNativeSeriesDescription, 
                                                 info.getSeriesNumber( ), 
                                                 info.getSeriesInstanceUID( ), 
                                                 info.getSeriesDate( ), 
                                                 ModalityIdentifier.getModality( alterNativeSeriesDescription ), 
                                                 info.isSpectrum( ), 
                                                 OrientationIdentifier.getOrientation( info.getImageOrientationPatient( ) ) );
                        }
                        study.addSeries( series );
                        series.addFile( info.getDirectory( ) + m_separator + info.getFileName( ) );
                    }
                } 
                else 
                {
                    Study study = new Study( info.getStudyDescription( ), 
                                             info.getStudyInstanceUID(), 
                                             info.getStudyDate( ) );
                    patient.addStudy(study);
                    
                    Series series;
                    if ( info.getSeriesDescription( ) != null )
                    {
                        series = new Series( info.getSeriesDescription( ), 
                                             info.getSeriesNumber( ), 
                                             info.getSeriesInstanceUID( ), 
                                             info.getSeriesDate( ), 
                                             ModalityIdentifier.getModality( info.getSeriesDescription( ).split( "-" )[0] ), 
                                             info.isSpectrum( ), 
                                             OrientationIdentifier.getOrientation( info.getImageOrientationPatient( ) ) );
                    }
                    else
                    {
                        String alterNativeSeriesDescription = info.getSequenceName( ).replace( '*', ' ' );
                        series = new Series( alterNativeSeriesDescription, 
                                             info.getSeriesNumber( ), 
                                             info.getSeriesInstanceUID( ), 
                                             info.getSeriesDate( ), 
                                             ModalityIdentifier.getModality( alterNativeSeriesDescription ), 
                                             info.isSpectrum( ), 
                                             OrientationIdentifier.getOrientation( info.getImageOrientationPatient( ) ) );
                    }
                    study.addSeries( series );
                    series.addFile( info.getDirectory( ) + m_separator + info.getFileName( ) );
                }
            } 
            else 
            {
                Patient patient = new Patient( info.getPatientName( ), 
                                               info.getPatientID( ), 
                                               info.getPatientBirthDate( ) );
                patientList.addPatient( patient );
                Study study = new Study( info.getStudyDescription( ), 
                                         info.getStudyInstanceUID( ), 
                                         info.getStudyDate( ) );
                patient.addStudy( study );
                Series series;
                if ( info.getSeriesDescription( ) != null )
                {
                    series = new Series( info.getSeriesDescription( ), 
                                         info.getSeriesNumber( ), 
                                         info.getSeriesInstanceUID( ), 
                                         info.getSeriesDate( ), 
                                         ModalityIdentifier.getModality( info.getSeriesDescription( ).split( "-" )[0] ), 
                                         info.isSpectrum( ), 
                                         OrientationIdentifier.getOrientation( info.getImageOrientationPatient( ) ) );
                }
                else
                {
                    String alterNativeSeriesDescription = info.getSequenceName( ).replace( '*', ' ' );
                    series = new Series( alterNativeSeriesDescription, info.getSeriesNumber( ), 
                                         info.getSeriesInstanceUID( ), 
                                         info.getSeriesDate( ), 
                                         ModalityIdentifier.getModality( alterNativeSeriesDescription ), 
                                         info.isSpectrum( ), 
                                         OrientationIdentifier.getOrientation( info.getImageOrientationPatient( ) ) );
                }
                study.addSeries( series );
                series.addFile( info.getDirectory( ) + m_separator + info.getFileName( ) );
            }
        }

        Iterator<Patient> patients = patientList.getPatients( ).iterator( );
        while ( patients.hasNext( ) ) 
        {
            Patient patientaux = patients.next( );
            DefaultMutableTreeNode patientnode = new DefaultMutableTreeNode( patientaux.getName( ) + "-" + patientaux.getBirthDate( ) );
            Iterator<Study> studies = patientaux.getStudies( ).iterator( );
            while ( studies.hasNext( ) ) 
            {
                Study studyaux = studies.next( );
                DefaultMutableTreeNode studynode = new DefaultMutableTreeNode( studyaux.getDate( ) + "-" + studyaux.getDescription( ) );
                //Iterator<Series> seriesl = studyaux.getSeries().iterator();
                Iterator<Series> seriesl = studyaux.getSortedBySeriesNumberSeries( ).iterator( );
                
                while ( seriesl.hasNext( ) ) 
                {
                    Series seriesaux = seriesl.next( );
                    String seriesNr = seriesaux.getSeriesNr( ).trim( );
  
//                    if (Integer.valueOf(seriesNr) < 10) {
//                        seriesNr = "000" + seriesNr;
//                    } else if (Integer.valueOf(seriesNr) < 100) {
//                        seriesNr = "00" + seriesNr;
//                    } else if (Integer.valueOf(seriesNr) < 1000) {
//                        seriesNr = "0" + seriesNr;
//                    }
                    
                    DefaultMutableTreeNode seriesnode = new DefaultMutableTreeNode( new SerieType( seriesNr + "-" + seriesaux.getDescription( ), seriesaux.isSpectroscopy( ) ) );
                    studyaux.setSeriesHashcode( seriesnode.hashCode( ), seriesaux );
                    studynode.add( seriesnode );
                }
                patientnode.add(studynode );
            }
            m_treeRoot.add( patientnode );
        }

        m_dicomDirectoryTree.updateUI( );

        expandAll( m_dicomDirectoryTree, true );

    }

    /**
     * If expand is true, expands all nodes in the tree; Otherwise, collapses
     * all nodes in the tree.
     * @param tree
     * @param expand
     */
    public void expandAll(JTree tree, boolean expand) {
        TreeNode root = (TreeNode) tree.getModel().getRoot();

        // Traverse tree from root
        expandAll(tree, new TreePath(root), expand);
    }

    /**
     * Does the actual expansion of the list
     *
     * @param tree
     * @param parent
     * @param expand
     */
    private void expandAll(JTree tree, TreePath parent, boolean expand) {
        // Traverse children
        TreeNode node = (TreeNode) parent.getLastPathComponent();
        if (node.getChildCount() >= 0) {
            for (Enumeration e = node.children(); e.hasMoreElements();) {
                TreeNode n = (TreeNode) e.nextElement();
                TreePath path = parent.pathByAddingChild(n);
                expandAll(tree, path, expand);
            }
        }

        // Expansion or collapse must be done bottom-up
        if (expand) {
            tree.expandPath(parent);
        } else {
            tree.collapsePath(parent);
        }
    }

    /**
     * Updates the JList component that indicates the images in the selected
     * Image series.
     */
    private void updateDicomImageSeriesList(int selRow, TreePath selPath) {
        final TreePath selpath = selPath;
//        m_dicomImageSeriesTable.setEnabled(false);
//        SwingWorker worker = new SwingWorker<Integer, Void>() {
//            @Override
//            public Integer doInBackground() {
        DefaultMutableTreeNode dmtn = (DefaultMutableTreeNode) selpath.getLastPathComponent();

//        if (!dmtn.isRoot()) {
//            m_selectedNodeName = selpath.getLastPathComponent().toString();
//            selpath.getLastPathComponent();
//            int beginSeriesNameIndex = m_selectedNodeName.indexOf(" ");
//            m_selectedSeriesNumber = new Integer(m_selectedNodeName.substring(0, beginSeriesNameIndex));
//            // eliminate the - sign
//            m_selectedNodeName = m_selectedNodeName.substring(beginSeriesNameIndex + 3);
//
////            System.out.println( "Selected Series Name : " + m_selectedNodeName );
////            System.out.println( "Selected Series Number : " + m_selectedSeriesNumber.intValue( ) );
//
//            m_fileSelectedSeriesFileReferences = m_dicomDir.getImageFileReferencesOfSeriesNumber(m_selectedSeriesNumber.intValue());
        if (selPath.getPathCount() == 4) {
//            int beginSeriesNameIndex = selPath.getPathComponent(3).toString().indexOf("-");
//            m_selectedSeriesNumber = new Integer(selPath.getPathComponent(3).toString().substring(0, beginSeriesNameIndex).trim());

            String patientKey = "";
            if (patientList.getPatient(selPath.getPathComponent(1).toString().replace("-", "")) == null) {
                String rawPatientKey = selPath.getPathComponent(1).toString();
                int index = rawPatientKey.lastIndexOf("-");
                patientKey = rawPatientKey.substring(0, index) + rawPatientKey.substring(index + 1);
            } else {
                patientKey = selPath.getPathComponent(1).toString().replace("-", "");
            }
            
            m_selectedSeriesNumber = selPath.getPathComponent(3).hashCode();
            Series currentser = patientList.getPatient(patientKey).getStudy(selPath.getPathComponent(2).toString().replaceFirst("-", "")).getSeries(m_selectedSeriesNumber);        
            
//            m_selectedSeriesNumber = selPath.getPathComponent(3).hashCode();
//            String study = selPath.getPathComponent(2).toString();  
//            study = study.replaceFirst("-", "" );
//       
//          
//            
//            
//            Series currentser = patientList.getPatient(patientKey).getStudy( study ).getSeries(m_selectedSeriesNumber);
//            
//            currentser = patientList.getPatient(patientKey).getStudy(selPath.getPathComponent(2).toString().replace("-", "")).getSeries(m_selectedSeriesNumber);
//            
//            currentser = patientList.getPatient(patientKey).getStudy(selPath.getPathComponent(2).toString()).getSeries(m_selectedSeriesNumber);
//            currentser = patientList.getPatient(selPath.getPathComponent(1).toString().replace("-", "")).getStudy(selPath.getPathComponent(2).toString().replace("-", "")).getSeries(m_selectedSeriesNumber);
            
            m_fileSelectedSeriesFileReferences = currentser.getFiles( );
            
            m_dicomImageSeriesTable.setListData( m_fileSelectedSeriesFileReferences, patientList );
        } 
        else 
        {
            if ( m_fileSelectedSeriesFileReferences != null ) 
            {
                m_fileSelectedSeriesFileReferences.clear( );
                m_dicomImageSeriesTable.setListData( m_fileSelectedSeriesFileReferences, patientList );
            }
        }
//                return 1;
//            }
//
//            @Override
//            public void done() {
////                m_dicomImageSeriesTable.setEnabled(true);
//            }
//        };
//        worker.execute();

    }

    /**
     * Returns the selected series name of the selected tree node.
     * @return 
     */
    public String getSelectedSeriesName() {
        return m_selectedNodeName;
    }

    /**
     * Returns the selected series name of the selected tree node.
     * @return 
     */
    public int getSelectedSeriesNumber() {
        return m_selectedSeriesNumber.intValue();
    }

//    /**
//     * Sets the cell values of the list. Since it is a public method it can be 
//     * called from a other object
//     */
//    public void setListData( TreeSet<String> fileReferences ) 
//    {
//        
//        m_fileSelectedSeriesFileReferences = fileReferences;
//        
//        m_cellData = new Object[ m_fileSelectedSeriesFileReferences.size( ) ][ 7 ];
//                
//        // reimplement the constructor code
//        int counter = 0;
//        Iterator it = fileReferences.iterator( );
//        while ( it.hasNext( ) )
//        {
//            String fileName = (String)it.next( );
//            DicomImageData data = new DicomImageData( fileName );
//            FileInfo finfo = data.getHeader( );
//            
//            m_cellData[counter][0] = fileName;
//            
//            if ( finfo.isImageData( ) == true )
//            {
//                m_cellData[counter][1] = "Image";
//                m_isImageSeries = true;
//            }
//            else
//            {
//                m_cellData[counter][1] = "Spectrum";
//                m_isImageSeries = false;
//            }
//            
//            m_cellData[counter][2] = new Integer( finfo.getInstanceNumber( ) );
//            m_cellData[counter][3] = new Double( finfo.getSlicePosition( ) ).toString( );
//            String fov = new Double( finfo.getPixelWidth( ) * finfo.getImageWidth( ) ).toString( );
//            fov += " x " + ( finfo.getPixelHeight( ) * finfo.getImageHeight( ) );
//            m_cellData[counter][4] = fov;
//            m_cellData[counter][5] = new Integer( finfo.getEchoTime( ) ).toString( );
//            m_cellData[counter][6] = new Integer( finfo.getTR( ) ).toString( );
//            counter++;
//        }
//         
//        String[] columnNames = { "<HTML><B>File name</B></HTML>",
//                                 "<HTML><B>Type</B></HTML>",
//                                 "<HTML><B>Instance number</B></HTML>",
//                                 "<HTML><B>Slice Position [mm]</B><HTML>",
//                                 "<HTML><B>FOV [mm]x[mm]</B><HTML>",
//                                 "<HTML><B>TE [ms]</B><HTML>",
//                                 "<HTML><B>TR [ms]</B><HTML>"
//                                };
//                                 
//        m_tableModel = new DefaultTableModel( m_cellData, columnNames );
//        
//        m_dicomImageSeriesTable.setModel( m_tableModel );
//        
//        m_dicomImageSeriesTable.selectAll( );
//        
//        m_dicomImageSeriesTable.updateUI( );
//    }
    /**
     * Returns the file references of all selected files from the table
     *
     * @return
     */
    public LinkedList<String> getSelectedFileReferencesFromTable() {
        return m_dicomImageSeriesTable.getSelectedFileReferencesFromTable();
    }

    // Notify all listeners that a change has taken place in this object
    protected void notifyListeners(int evType, String cmd) {
        Iterator it = m_listeners.iterator();
        // notify registered listeners
        while (it.hasNext()) {
            ((ActionListener) it.next()).actionPerformed(new ActionEvent(this, evType, cmd));
        }
    }

    /**
     * Adds an ActionListener to the list of action listeners who are notified
     * when something happens in this component.
     * @param e
     */
    public void addActionListener(ActionListener e) {
        m_listeners.add(e);
    }

    /**
     * Returns the TreeSet contain the file references in the selected series.
     * @return 
     */
    public LinkedList<String> getSelectedSeriesFileReferences() {
        return m_dicomImageSeriesTable.getSelectedSeriesFileReferences();
    }

    /**
     * returns whether the selected series contains image data or not.
     * @return 
     */
    public boolean isImageSeries() {
        return m_dicomImageSeriesTable.isImageSeries();
    }

    /**
     * Returns the number of different slice positions in the selected series.
     * @return 
     */
    public int getNumberOfDifferentSlicePositions() {
        return m_dicomImageSeriesTable.getNumberOfDifferentSlicePositions();
    }

    /**
     * Returns the number of series in the selected Dicom directory having the
     * same series description "seriesDescription".
     * @param seriesDescription
     * @return 
     */
    public int getNumberOfSeries(String seriesDescription) {
        int numberOfSeries = 0;
        m_seriesNames = m_dicomDir.getSeriesDescriptions();

        Collection smap = m_seriesNames.keySet();
        Iterator it = smap.iterator();

        for (int i = 0; i < m_seriesNames.size(); i++) {
            Integer intje = (Integer) it.next();
            if (((String) m_seriesNames.get(intje)).compareTo(seriesDescription) == 0) {
                numberOfSeries++;
            }
        }

        return numberOfSeries;
    }

    /**
     * Returns an array of all selected series numbers.
     * @return 
     */
    public int[] getSelectedSeriesNumbers() {
        TreePath[] paths = this.m_dicomDirectoryTree.getSelectionPaths();
        int[] selSeriesNumbers = new int[paths.length];
        for (int i = 0; i < paths.length; i++) {
            String str = paths[i].getLastPathComponent().toString();
            int beginSeriesNameIndex = str.indexOf(" ");
            selSeriesNumbers[i] = (new Integer(str.substring(0, beginSeriesNameIndex))).intValue();
        }

        return selSeriesNumbers;
    }

    public PatientList getPatientList() {
        return patientList;
    }

    public DicomFileSelectionTable getDicomFileSelectionTable() {
        return m_dicomImageSeriesTable;
    }

    // Member variables
    // Listeners to textfield changes
    private LinkedList<ActionListener> m_listeners = new LinkedList<ActionListener>();
    private String m_selectedNodeName;
    private Integer m_selectedSeriesNumber;
    private String m_separator;
    // Two sub panels to put the treeview and the listview upon
    JPanel m_treePanel;
    JPanel m_tablePanel;
    // The JTree contains the list of series belonging to the selected directory
    JTree m_dicomDirectoryTree;
    JScrollPane m_treeScrollPane;
    // Contains the list of images belonging to a series study 
    DicomFileSelectionTable m_dicomImageSeriesTable;
    DefaultTableModel m_tableModel;
    // Object to store the list items of the image series data into
    Object[][] m_cellData;
    JScrollPane m_listScrollPane;
    // TreeMap containing the names of the series present in the mounted 
    // directory
    TreeMap m_seriesNames;
    DefaultMutableTreeNode m_treeRoot;
    // Data structure containing all file informations of the mounted directory
    DicomDirectory m_dicomDir;
    protected LinkedList<String> m_fileSelectedSeriesFileReferences;
    MouseListener m_treeMouseListener;
    // variable indicating whether series contains image data or else
    boolean m_isImageSeries;
    // variable to indicate that a file system is mounted
    protected boolean m_directoryIsMounted = false;
    protected JPopupMenu m_popupMenu;
    public final static int TREE_ENTRY_SELECTED_EVENT = 0;
    public final static int TREE_ENTRY_DOUBLE_CLICKED_EVENT = 1;
    public final static String TREE_ENTRY_SELECTED_CMD = "SERIES_TREE_SELECTED_CMD";
    public final static String TREE_ENTRY_DBL_CLICKED_CMD = "SERIES_TREE_DBL_CLICKED_CMD";
    // For proper serialization
    private static final long serialVersionUID = 9982128799688883L;
    private PatientList patientList = new PatientList();
}

// define class for serietye, only usefull for favicon
class SerieType {

    private String name;
    private boolean isSpectrum;

    SerieType(String name, boolean isSpectrum) {
        this.name = name;
        this.isSpectrum = isSpectrum;
    }

    public String getName() {
        return name;
    }

    public boolean getSpectrumBool() {
        return isSpectrum;
    }
}
