/*
 * File: $Id: DicomNetworkNodes.java,v 1.6 2009/15/05 13:28:39 hans Exp $
 * History
 * Date: 15.05.2009  Authors: Hans Slotboom
 *
 * Changes:
 *
 * Copyright (C) 2009 by
 *
 * Inselspital - Department of Neuroradiology
 * CH 3010 Bern Switzerland
 */



/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package org.spectrim.dicom;

import java.util.LinkedList;

/**
 *
 * Container class containing the list of valid Dicom network node
 *
 * Singleton class
 *
 * Implementation is based on functionality offered by dcm4che2 release 2.0.19
 *
 * @author slotboom
 */
public class DicomNetworkNodes
{
    /**
     * Default constructor
     */
    private DicomNetworkNodes( )
    {
        m_dicomNodes = new LinkedList<DicomNetworkNode>( );
    }


     /**
     * Returns the one and onnly instance of DicomNetworkNodeDefaults
     */
    public static DicomNetworkNodes getInstance( )
    {
        if ( theInstance == null )
        {
            theInstance = new DicomNetworkNodes( );
        }

        return theInstance;
    }

    /**
     * Adds a dicom network node
     */
    public void setRemoteDicomNetworkNode( int nodeNumber, String nodeName, String aetitle, String ipAdress, int portNumber )
    {
        if ( m_dicomNodes.size( ) == 0 )
        {
            for ( int i = 0; i < 5; i++ )
            {
                m_dicomNodes.add( new DicomNetworkNode( ) );
            }
        }

        m_dicomNodes.set( nodeNumber, new DicomNetworkNode( nodeName, aetitle, ipAdress, portNumber ) );
    }

    /**
     * Adds a dicom network node
     */
    public void setLocalHostDicomNetworkNode( String nodeName, String receive_aetitle, String transmit_aetitle,
                                              String ipaddress, int receivePortNumber, int transmitPortNumber )
    {
        if ( m_dicomNodes.size( ) == 0 )
        {
            for ( int i = 0; i < 5; i++ )
            {
                m_dicomNodes.add( new DicomNetworkNode( ) );
            }
        }

        m_dicomNodes.set( 0, new DicomNetworkNode( nodeName, receive_aetitle, transmit_aetitle,
                                                   ipaddress, receivePortNumber, transmitPortNumber ) );
    }


    /**
     * Clears the list of DicomNetworkNodes
     */
    public void clear( )
    {
        m_dicomNodes.clear( );
    }

    /**
     * Returns node number 'number
     */
    public DicomNetworkNode getLocalHostDicomNetworkNode( )
    {
        
        clear( );
        
        DicomNetworkNodeDefaults def = DicomNetworkNodeDefaults.getInstance( );

        m_selectedRemoteDicomNetworkNode = ( new Integer( def.getProperty( DicomNetworkNodeDefaults.SELECTED_REMOTE_DICOM_HOST_NUMBER ) ) ).intValue( );


        setLocalHostDicomNetworkNode( def.getProperty( DicomNetworkNodeDefaults.LOCAL_HOST_NAME ),
                                      def.getProperty( DicomNetworkNodeDefaults.LOCAL_HOST_DICOM_RECEIVE_AETITLE  ),
                                      def.getProperty( DicomNetworkNodeDefaults.LOCAL_HOST_DICOM_TRANSMIT_AETITLE ),
                                      def.getProperty( DicomNetworkNodeDefaults.LOCAL_HOST_IP_ADDRESS ),
                                      ( new Integer( def.getProperty( DicomNetworkNodeDefaults.LOCAL_HOST_DICOM_RECEIVE_PORT ) ) ).intValue( ),
                                      ( new Integer( def.getProperty( DicomNetworkNodeDefaults.LOCAL_HOST_DICOM_TRANSMIT_PORT ) ) ).intValue( ) );

        setRemoteDicomNetworkNode( 1, def.getProperty( DicomNetworkNodeDefaults.REMOTE_DICOM_HOST_0_NAME ),
                                      def.getProperty( DicomNetworkNodeDefaults.REMOTE_DICOM_HOST_0_DICOM_RECEIVE_AETITLE  ),
                                      def.getProperty( DicomNetworkNodeDefaults.REMOTE_DICOM_HOST_0_IP_ADDRESS ),
                                      ( new Integer( def.getProperty( DicomNetworkNodeDefaults.REMOTE_DICOM_HOST_0_DICOM_RECEIVE_PORT ) ) ).intValue( ) );

        setRemoteDicomNetworkNode( 2, def.getProperty( DicomNetworkNodeDefaults.REMOTE_DICOM_HOST_1_NAME ),
                                      def.getProperty( DicomNetworkNodeDefaults.REMOTE_DICOM_HOST_1_DICOM_RECEIVE_AETITLE  ),
                                      def.getProperty( DicomNetworkNodeDefaults.REMOTE_DICOM_HOST_1_IP_ADDRESS ),
                                      ( new Integer( def.getProperty( DicomNetworkNodeDefaults.REMOTE_DICOM_HOST_1_DICOM_RECEIVE_PORT ) ) ).intValue( ) );

        setRemoteDicomNetworkNode( 3, def.getProperty( DicomNetworkNodeDefaults.REMOTE_DICOM_HOST_2_NAME ),
                                      def.getProperty( DicomNetworkNodeDefaults.REMOTE_DICOM_HOST_2_DICOM_RECEIVE_AETITLE  ),
                                      def.getProperty( DicomNetworkNodeDefaults.REMOTE_DICOM_HOST_2_IP_ADDRESS ),
                                      ( new Integer( def.getProperty( DicomNetworkNodeDefaults.REMOTE_DICOM_HOST_2_DICOM_RECEIVE_PORT ) ) ).intValue( ) );

        setRemoteDicomNetworkNode( 4, def.getProperty( DicomNetworkNodeDefaults.REMOTE_DICOM_HOST_3_NAME ),
                                      def.getProperty( DicomNetworkNodeDefaults.REMOTE_DICOM_HOST_3_DICOM_RECEIVE_AETITLE  ),
                                      def.getProperty( DicomNetworkNodeDefaults.REMOTE_DICOM_HOST_3_IP_ADDRESS ),
                                      ( new Integer( def.getProperty( DicomNetworkNodeDefaults.REMOTE_DICOM_HOST_3_DICOM_RECEIVE_PORT ) ) ).intValue( ) );


        return m_dicomNodes.get( 0 );
    }

    /**
     * Returns DicomNetworkNode-network node object belonging to "number"
     */
    public DicomNetworkNode getRemoteDicomNetworkNode( int number )
    {
        clear( );

        DicomNetworkNodeDefaults def = DicomNetworkNodeDefaults.getInstance( );

        m_selectedRemoteDicomNetworkNode = ( new Integer( def.getProperty( DicomNetworkNodeDefaults.SELECTED_REMOTE_DICOM_HOST_NUMBER ) ) ).intValue( );

        setLocalHostDicomNetworkNode( def.getProperty( DicomNetworkNodeDefaults.LOCAL_HOST_NAME ),
                                      def.getProperty( DicomNetworkNodeDefaults.LOCAL_HOST_DICOM_RECEIVE_AETITLE  ),
                                      def.getProperty( DicomNetworkNodeDefaults.LOCAL_HOST_DICOM_TRANSMIT_AETITLE ),
                                      def.getProperty( DicomNetworkNodeDefaults.LOCAL_HOST_IP_ADDRESS ),
                                      ( new Integer( def.getProperty( DicomNetworkNodeDefaults.LOCAL_HOST_DICOM_RECEIVE_PORT ) ) ).intValue( ),
                                      ( new Integer( def.getProperty( DicomNetworkNodeDefaults.LOCAL_HOST_DICOM_TRANSMIT_PORT ) ) ).intValue( ) );

        setRemoteDicomNetworkNode( 1, def.getProperty( DicomNetworkNodeDefaults.REMOTE_DICOM_HOST_0_NAME ),
                                      def.getProperty( DicomNetworkNodeDefaults.REMOTE_DICOM_HOST_0_DICOM_RECEIVE_AETITLE  ),
                                      def.getProperty( DicomNetworkNodeDefaults.REMOTE_DICOM_HOST_0_IP_ADDRESS ),
                                      ( new Integer( def.getProperty( DicomNetworkNodeDefaults.REMOTE_DICOM_HOST_0_DICOM_RECEIVE_PORT ) ) ).intValue( ) );

        setRemoteDicomNetworkNode( 2, def.getProperty( DicomNetworkNodeDefaults.REMOTE_DICOM_HOST_1_NAME ),
                                      def.getProperty( DicomNetworkNodeDefaults.REMOTE_DICOM_HOST_1_DICOM_RECEIVE_AETITLE  ),
                                      def.getProperty( DicomNetworkNodeDefaults.REMOTE_DICOM_HOST_1_IP_ADDRESS ),
                                      ( new Integer( def.getProperty( DicomNetworkNodeDefaults.REMOTE_DICOM_HOST_1_DICOM_RECEIVE_PORT ) ) ).intValue( ) );

        setRemoteDicomNetworkNode( 3, def.getProperty( DicomNetworkNodeDefaults.REMOTE_DICOM_HOST_2_NAME ),
                                      def.getProperty( DicomNetworkNodeDefaults.REMOTE_DICOM_HOST_2_DICOM_RECEIVE_AETITLE  ),
                                      def.getProperty( DicomNetworkNodeDefaults.REMOTE_DICOM_HOST_2_IP_ADDRESS ),
                                      ( new Integer( def.getProperty( DicomNetworkNodeDefaults.REMOTE_DICOM_HOST_2_DICOM_RECEIVE_PORT ) ) ).intValue( ) );

        setRemoteDicomNetworkNode( 4, def.getProperty( DicomNetworkNodeDefaults.REMOTE_DICOM_HOST_3_NAME ),
                                      def.getProperty( DicomNetworkNodeDefaults.REMOTE_DICOM_HOST_3_DICOM_RECEIVE_AETITLE  ),
                                      def.getProperty( DicomNetworkNodeDefaults.REMOTE_DICOM_HOST_3_IP_ADDRESS ),
                                      ( new Integer( def.getProperty( DicomNetworkNodeDefaults.REMOTE_DICOM_HOST_3_DICOM_RECEIVE_PORT ) ) ).intValue( ) );

        return m_dicomNodes.get( number + 1 );
    }

    /**
     * Sets the user selected remote dicom node
     * @param selectedRemoteDicomNetworkNode
     */
    public void setSelectedRemoteDicomNode( int selectedRemoteDicomNetworkNode )
    {
        m_selectedRemoteDicomNetworkNode = selectedRemoteDicomNetworkNode;
    }


    /**
     * Returns the user selected remote dicom node
     * @return selectedRemoteDicomNetworkNode
     */
    public int  getSelectedRemoteDicomNodeNumber( )
    {
        DicomNetworkNodeDefaults def = DicomNetworkNodeDefaults.getInstance( );
        m_selectedRemoteDicomNetworkNode = ( new Integer( def.getProperty( DicomNetworkNodeDefaults.SELECTED_REMOTE_DICOM_HOST_NUMBER ) ) ).intValue( );

        return m_selectedRemoteDicomNetworkNode;
    }


    // attributes
    private LinkedList<DicomNetworkNode> m_dicomNodes = new LinkedList<DicomNetworkNode>( );
    private int m_selectedRemoteDicomNetworkNode = -1;

    // static private attribute is the instance itself (Singleton!)
    private static  DicomNetworkNodes theInstance = null;

}
