/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package org.spectrim.dicom;

import java.io.IOException;
import java.io.InputStream;


import java.io.File;

import java.io.RandomAccessFile;

/**
 * Helper class that enables the user to make changes to the Dicom tags. 
 * @author slotboom
 */
public class DicomTagChanger
{

    /**
     * Constructs a new object of DicomTagChanger
     * @param pathFileName
     */
    public DicomTagChanger( String pathFileName )
    {
        m_pathFileName = pathFileName;
    }


    /**
     * Changes the value of XML tag belonging to the SOP instance UID.
     * @param tagID
     * @param newValue
     * @throws java.lang.Exception
     */
    public void changeDicomTaginXMLFile( String tagID, String newValue ) throws Exception
    {

        // parse the file ..
        RandomAccessFile xmlFile = new RandomAccessFile( m_xmlFilePath, "rwd" );

        String xmlFileOutStringPath = m_xmlFilePath.replaceFirst( ".xml", "_tmp.xml" );

        RandomAccessFile xmlFileOut = new RandomAccessFile( xmlFileOutStringPath, "rwd" );

        long pos = xmlFile.getFilePointer( );
        String readLine = xmlFile.readLine( );
        String outputLine = "";

        while ( readLine != null )
        {
            if ( readLine.contains( tagID ) )
            {
                int lenPosition = readLine.indexOf( "len=" );
                outputLine = readLine.substring( 0, lenPosition + 4 );
                outputLine = outputLine +  '"' + newValue.length( ) + '"' + ">" + newValue +"</attr>";

                xmlFileOut.writeBytes( outputLine + "\n" );

            }
            else
            {
                xmlFileOut.writeBytes( ( readLine + "\n") );
            }

            pos = xmlFile.getFilePointer( );
            readLine = xmlFile.readLine( );
        }

        // close the files
        xmlFile.close( );
        xmlFileOut.close( );

        // remove the old xml
        File remove = new File( m_xmlFilePath );
        remove.delete( );

        // rename temporary xml to original xml
        File file = new File( xmlFileOutStringPath );
        file.renameTo( new File( m_xmlFilePath ) );

    }

    /**
     * Deletes temporary xml file from file system.
     */
    public void clean( )
    {
        File remove = new File( m_xmlFilePath );
        remove.delete( );
    }

    /**
     * Performs a conversion from Dicom format to the XML format.
     * @return 
     */
    public boolean convertDicom2XML( )
    {

        String workingDir = System.getProperty( "user.dir" );

        int returnValue;
        String cmd = "";
        
        m_xmlFilePath = m_pathFileName + ".xml";    

        if ( System.getProperty( "os.name" ).toLowerCase( ).contains( "linux" ) == true )
        {
            cmd = workingDir + "/lib/dcm2xml -C " + m_pathFileName + " -o " + m_xmlFilePath + "\n";
        }

        if ( System.getProperty( "os.name" ).toLowerCase( ).contains( "windows" ) == true )
        {
            cmd = workingDir + "\\lib\\dcm2xml.bat -C " + m_pathFileName + " -o " + m_xmlFilePath + "\n";
            cmd = generateProperWindowsCommand( cmd );
        }

        System.out.println( "Processing following system command : " + cmd );

        try
        {
            Process p = Runtime.getRuntime( ).exec( cmd );

            InputStream in = p.getInputStream( );
            int c;
            while ( ( c = in.read( ) ) != -1 )
            {
                System.out.print( (char)c ) ;
            }

            returnValue = p.waitFor( );
            in.close( );
        }
        catch ( IOException ioX )
        {
            // not handled yet
            System.out.println( "IOException occurred" );
            return false;
        }
        catch ( InterruptedException intX )
        {
            // not handled yet
            System.out.println( "InterruptedException occurred" );
            return false;
        }

        // remove the original dicom..
        File file = new File( m_pathFileName );
        if ( file.exists( ) )
        {
            file.delete( );
        }
        
        return true;
    }


    /**
     * Performs a conversion from the XML format to the Dicom format.
     * @return 
     */
    public boolean convertXML2Dicom( )
    {

        String workingDir = System.getProperty( "user.dir" );

        int returnValue;
        String cmd = "";

         m_xmlFilePath = m_pathFileName + ".xml";


        if ( System.getProperty( "os.name" ).toLowerCase( ).contains( "linux" ) == true )
        {
            cmd = workingDir + "/lib/xml2dcm -x " + m_xmlFilePath + " -o " + m_pathFileName + "\n";
        }

        if ( System.getProperty( "os.name" ).toLowerCase( ).contains( "windows" ) == true )
        {
            cmd = workingDir + "\\lib\\xml2dcm.bat -x " + m_xmlFilePath + " -o " + m_pathFileName + "\n";
            cmd = generateProperWindowsCommand( cmd );
        }

        System.out.println( "Processing following system command : " + cmd );

        try
        {
            Process p = Runtime.getRuntime( ).exec( cmd );

            InputStream in = p.getInputStream( );
            int c;
            while ( ( c = in.read( ) ) != -1 )
            {
                System.out.print( (char)c ) ;
            }

            returnValue = p.waitFor( );
            in.close( );
        }
        catch ( IOException ioX )
        {
            // Not handled yet
            System.out.println( "IOException occurred" );
            return false;
        }
        catch ( InterruptedException intX )
        {
            // Not handled yet
            System.out.println( "InterruptedException occurred" );
            return false;
        }

        return true;
    }

    /**
     * Replaces all blanks by "\  " in order to be interpreted
     * correctly by the Windows operating system
     * @param cmd
     * @return
     */
    private String generateProperWindowsCommand( String cmd )
    {
        String out = "";
        for ( int i = 0; i < cmd.length( ); i++ )
        {
            if ( cmd.substring( i, i + 1 ).compareTo( "\\" ) == 0 )
            {
                out += "\\" + "\\";
            }
            else
            {
                out += cmd.substring( i, i + 1 );
            }
        }
        return out;
    }

    // Attributes of DicomTagChanger

    private String m_pathFileName = "";
    private String m_xmlFilePath = "";

}
