
package org.spectrim.hlsvd;

import java.util.*;
import org.spectrim.tdfdfit.TDFDFittingDefaults;
import org.spectrim.utilities.ClientSocket;
import static org.spectrim.tdfdfit.TDFDFittingDefaults.LIB_HLSVD_PATH_AND_DLL_NAME;
import java.io.IOException;


/**
 * This class enables the spectral estimation by Hankel-Lanczos
 * Singular Value Decomposition: it calls the native Fortran code 
 * to do so.
 * @author Ron de Beer and Johannes Slotboom (c) 2018
 */


public class Compute_Native_HLSVD 
{
	
    private int len = 6;
    private String[] nat_pars;
    // private ReadWriteMRSFilesRon readwritefiles;
    
    
    double[] signal_re = new double[1024];
    double[] signal_im = new double[1024];
    
    private double[] ampl;
    private double[] freq;
    private double[] damp;
    private double[] phase;

    private double[] recon_re = new double[1024]; 
    private double[] recon_im = new double[1024]; 
    private double[] filtered_re = new double[1024]; 
    private double[] filtered_im = new double[1024]; 
    
    private int      ndp;
    private double   step;
    private double   larmor;
    private int      numberOfResonanceLines;
    private String[] str_str;
    private String[] str_int;
    private String[] str_dbl;
    private String   signal_name;
    private int      HankelMatrixSize;
    // Attribute to access the results of the HLSVD
    private HLSVDResults m_HLSVDResults;
    private String FilterMode;
    private Double ExtremeBroadLimit;
    
    static 
    {   
        TDFDFittingDefaults defaults = TDFDFittingDefaults.getInstance();
        defaults.load( );
        
        String libCallFortranPathAndFileName   = defaults.getProperty( TDFDFittingDefaults.CALL_FORTRAN_PATH_AND_DLL_NAME );
//        System.load( libCallFortranPathAndFileName  );
        
        
        String libHLSVDPathAndFileName   = defaults.getProperty( TDFDFittingDefaults.LIB_HLSVD_PATH_AND_DLL_NAME );
        System.load( libHLSVDPathAndFileName  );

        String libHLSVDPathAndFileName2   = defaults.getProperty( TDFDFittingDefaults.LIB_HLSVD_PATH_AND_DLL_NAME );
        System.load( libHLSVDPathAndFileName2  );

        
//        System.load( "C:\\Users\\johan\\Documents\\development\\java\\Netbeans\\spectrIm-QMRS\\spectrIm\\native\\target_hlsvd_native_64bits\\callfortran_hlsvd_arr.dll" );
//        System.load( "C:\\Users\\johan\\Documents\\development\\java\\Netbeans\\spectrIm-QMRS\\spectrIm\\native\\target_hlsvd_native_64bits\\libhlsvd.dll" );
    }
    
    
    
    public final native String[] callNativeHLSVD( double[] signal_re, double[] signal_im, 			
                                                  String signal_name, int ndp, double step, double larmor,
                                                  String[] str_str, String[] str_int, String[] str_dbl,
                                                  double[] ampl, double[] freq, double[] damp, double[] phase,
                                                  double[] recon_re, double[] recon_im, 
                                                  double[] filtered_re, double[] filtered_im );

    public native double[] getSignal( );

    /* 
     * Constructor Compute_Native_HLSVD. 
     */
    public Compute_Native_HLSVD( String[] pars, double[] signal_re, double[] signal_im ) 
    {
        
        numberOfResonanceLines = Integer.parseInt( pars[9] );
        
        signal_name = pars[0];
        
        ndp    = Integer.parseInt( pars[1] );
        step   = Double.parseDouble( pars[2] );
        larmor = Double.parseDouble( pars[3] );
        
        str_str = new String[len];
        str_int = new String[len];
        str_dbl = new String[len];
		
        for ( int i = 0; i < len; i++ ) 
        {
            str_str[i] = "0";
            str_int[i] = "0";
            str_dbl[i] = "0";
        }
		
        str_str[0] = pars[4];
        str_str[5] = pars[5];
		
        str_int[0] = pars[6];
        str_int[1] = pars[7];
        str_int[2] = pars[8];
        str_int[3] = pars[9];   
        str_dbl[0] = pars[10];
        str_dbl[1] = pars[11];
        
        this.HankelMatrixSize = Integer.parseInt(pars[8]);
        if (pars.length==14){
            this.FilterMode = pars[12];
            this.ExtremeBroadLimit = Double.parseDouble( pars[13] );
        }
        // First to do: give arrays the correct length
        this.signal_re = new double[ndp];
        this.signal_im = new double[ndp];
    
        this.ampl = new double[numberOfResonanceLines];
        this.freq = new double[numberOfResonanceLines];
        this.damp = new double[numberOfResonanceLines];
        this.phase = new double[numberOfResonanceLines];

        this.recon_re   = new double[ndp]; 
        this.recon_im   = new double[ndp]; 
        this.filtered_re = new double[ndp]; 
        this.filtered_im = new double[ndp]; 

        nat_pars = new String[ pars.length ];
		
        System.arraycopy( pars, 0, nat_pars, 0, pars.length );
		
//        for( int i = 0; i < pars.length; i++ ) 
//        {
//            System.out.println( "nat_pars[" + i + "] = " + nat_pars[i] ); 
//        }
		
        // generate a new test signal
        recon_re = new double[ndp];
        recon_im = new double[ndp];
        filtered_re = new double[ndp];
        filtered_im = new double[ndp];

        // Complex conjugate must be taken! otherwise positive 
        // and negative frequencies are wrong
        for ( int i = 0; i < ndp; i++ )
        {
            this.signal_re[i] = signal_re[i];
            this.signal_im[i] = signal_im[i];
        }
        
        for( int i = 0; i < numberOfResonanceLines; i++ )
        {
            ampl[i]     = 0.0;
            freq[i]     = 0.0;
            damp[i]     = 0.0;
            phase[i]    = 0.0;
        }
                
    }
    
    
    void computeHLSVDPro(double MinimumCutFreq, double MaximumCutFreq)
    {
        ClientSocket socket;
        try {
            socket = new ClientSocket(33356);
        } catch (IOException e) {
            socket = null;
            System.out.println( "ERROR connecting to socket" );
        }
        if(socket!=null){
            try{
                socket.write_str("DATA");
                socket.writeDouble(MinimumCutFreq);
                socket.writeDouble(MaximumCutFreq);
                socket.write_doubleArray(signal_re);
                socket.write_doubleArray(signal_im);
                socket.writeInt(ndp);
                socket.writeInt(numberOfResonanceLines);
                socket.writeDouble(step);
                socket.writeInt(HankelMatrixSize);
                socket.write_str("PROCESS");
                if ( FilterMode.compareTo( TDFDFittingDefaults.HLSVD_FILTER_WATER_AND_EXTREME_BROAD_LINES ) == 0 ){
                    socket.write_str("WATER+BROAD");
                    socket.writeDouble(ExtremeBroadLimit);
                }
                else if(FilterMode.compareTo( TDFDFittingDefaults.HLSVD_FILTER_EXTREME_BROAD_LINES_ONLY ) == 0 ){
                    socket.write_str("BROAD");
                    socket.writeDouble(ExtremeBroadLimit);
                }
                else{
                    socket.write_str("WATER");
                }

                freq = socket.readDoubleArray(numberOfResonanceLines);
                damp = socket.readDoubleArray(numberOfResonanceLines);
                ampl = socket.readDoubleArray(numberOfResonanceLines);
                phase = socket.readDoubleArray(numberOfResonanceLines);
                filtered_re = socket.readDoubleArray(ndp);
                filtered_im = socket.readDoubleArray(ndp);
                socket.write_str("FINISHED");
                socket.closeConnection();
            } catch (IOException e){
                socket.closeConnection();
                socket = null;
            }
        }

        m_HLSVDResults = new HLSVDResults( ndp, numberOfResonanceLines );
        
        m_HLSVDResults.setSignalRealPart( signal_re );
        m_HLSVDResults.setSignalImaginaryPart( signal_im );
        m_HLSVDResults.setSignalName( signal_name );
        m_HLSVDResults.setNumberOfDataPoints( ndp );
        m_HLSVDResults.setNumberOfResonlineLines( numberOfResonanceLines );
        m_HLSVDResults.setStepSize( step );
        m_HLSVDResults.setLarmorFrequency( larmor );
        m_HLSVDResults.setAmplitudesHLSVD( ampl );
        m_HLSVDResults.setFrequenciesHLSVD( freq );
        m_HLSVDResults.setDampingsHLSVD( damp );
        m_HLSVDResults.setPhasesHLSVD( phase );
        m_HLSVDResults.setReconstructedSignalRealPart( recon_re );
        m_HLSVDResults.setReconstructedSignalImagenaryPart( recon_im );
        m_HLSVDResults.setFilteredSignalRealPart( filtered_re );
        m_HLSVDResults.setFilteredSignalImagenaryPart( filtered_im );
    }
    
    
    /**
     * Performs the actual computation of the HLSVD.
     */
    void computeHLSVD( ) 
    {
        
        String[] ret = (String[])callNativeHLSVD( signal_re, signal_im, 
                                                  signal_name, ndp, step, larmor,
                                                  str_str, str_int, str_dbl,
                                                  ampl, freq, damp, phase,
                                                  recon_re, recon_im, 
                                                  filtered_re, filtered_im );
		
	m_HLSVDResults = new HLSVDResults( ndp, numberOfResonanceLines );
        
        m_HLSVDResults.setSignalRealPart( signal_re );
        m_HLSVDResults.setSignalImaginaryPart( signal_im );
        m_HLSVDResults.setSignalName( signal_name );
        m_HLSVDResults.setNumberOfDataPoints( ndp );
        m_HLSVDResults.setNumberOfResonlineLines( numberOfResonanceLines );
        m_HLSVDResults.setStepSize( step );
        m_HLSVDResults.setLarmorFrequency( larmor );
        m_HLSVDResults.setAmplitudesHLSVD( ampl );
        m_HLSVDResults.setFrequenciesHLSVD( freq );
        m_HLSVDResults.setDampingsHLSVD( damp );
        m_HLSVDResults.setPhasesHLSVD( phase );
        m_HLSVDResults.setReconstructedSignalRealPart( recon_re );
        m_HLSVDResults.setReconstructedSignalImagenaryPart( recon_im );
        m_HLSVDResults.setFilteredSignalRealPart( filtered_re );
        m_HLSVDResults.setFilteredSignalImagenaryPart( filtered_im );
        
        //System.out.println( " i \t ampl[i] \t freq[i] \t damp[i] \t phase  (1st time)" );
        //for( int i = 0; i < 30; i++ )
        //{
        //    System.out.println( "" + i + "\t" + ampl[i] + "\t" + freq[i] + "\t" + damp[i] + "\t" + phase[i] );
        //}
        
        //double[] xaxis = new double[signal_re.length];
        
        //for( int i = 0; i < signal_re.length; i++ )
        //{
        //    xaxis[i] = 1.0 * i;
        //}
        
//        XYPlotFrame xyPlot = new XYPlotFrame( "HLSVD Test", null );
//        xyPlot.addCurve( new Curve( xaxis, signal_re ), true );
//        xyPlot.addCurve( new Curve( xaxis, filtered_re, java.awt.Color.GREEN ), true );
//        xyPlot.addCurve( new Curve( xaxis, recon_re, java.awt.Color.RED ), true );
        
    }
    
    /**
     * Returns the results of the HLSVD call.
     * @return 
     */
    public HLSVDResults getHLSVDResults( )
    {
        return this.m_HLSVDResults;
    }
    
}
