package org.spectrim.hlsvd;

import java.awt.*;
import javax.swing.*;
import java.io.*;
import org.spectrim.data.CSIData;
import org.spectrim.spectra.Spectra;

//import mrui.MruiImpl;
//import mrui.data.Data;

public class Gui extends JInternalFrame 
{

    private static final long serialVersionUID = 1L;

    private JPanel mainpanel;
    private JPanel northpanel;
    private JPanel centerpanel;
    private JPanel southpanel;

    private JLabel[] label_tf_str;
    private JLabel[] label_tf_int;
    private JLabel[] label_tf_dbl;
    private JTextField[] tf_str;
    private JTextField[] tf_int;
    private JTextField[] tf_dbl;

    private JButton readsig;
    private JButton quit;
    private JButton runhlsvd;

    private CSIData m_csiData;

    private int len = 6;
    private String[] gui_info;

    private String work_dir;

    private String signal_name;
    private int ndp;
    private double step;
    private double larmor;

    private double[] ampl;
    private double[] freq;
    private double[] damp;

    private ReadWriteMRSFiles readwritefiles;
	
    static 
    {
	System.loadLibrary( "hlsvd" );
    }
    
    public native String[] callNativeC( String signal_name,
                                        int ndp, double step, double larmor,
                                        String[] str_str, String[] str_int, String[] str_dbl );

    public Gui( JFrame parent, CSIData csiData ) 
    {

        // super("HLSVD Plugin", true, true, true, true);

        m_csiData = csiData;

        this.setContentPane( getDisplay( ) );
        this.setBounds( 20, 20, 950, 200 );
        this.setVisible( true );

        // work_dir = mrui.getPreferences( ).lastWorkDirectory;
        work_dir = m_csiData.getFile( );
        System.out.println( "current jMRUI working directory = " + work_dir );	

    }

    private JPanel getDisplay( )
    {

        mainpanel = new JPanel(new BorderLayout());
        setVisible(true);
        setResizable(true);
        setIconifiable(true);
        setTitle("GUI HLSVD Plugin");

        northpanel  = new JPanel();
        centerpanel = new JPanel(new GridLayout(3,20,0,10));
        southpanel  = new JPanel();

        Color mycolor = new Color(240,130,240);
        northpanel.setBackground(mycolor);
        centerpanel.setBackground(Color.cyan);
        southpanel.setBackground(mycolor);

        label_tf_str = new JLabel[len];
        tf_str       = new JTextField[len];
        label_tf_int = new JLabel[len];
        tf_int       = new JTextField[len];
        label_tf_dbl = new JLabel[len];
        tf_dbl       = new JTextField[len];

        for ( int i = 0; i < len; i++ ) 
        {
            label_tf_str[i] = new JLabel("string" + i, JLabel.LEFT);
            tf_str[i] = new JTextField ("", 4);
            label_tf_int[i] = new JLabel("integer" + i, JLabel.LEFT);
            tf_int[i] = new JTextField ("", 4);
            label_tf_dbl[i] = new JLabel("double" + i, JLabel.LEFT);
            tf_dbl[i] = new JTextField ("", 4);
            label_tf_str[i].setFont(new Font("Dialog", Font.BOLD, 12));
            label_tf_int[i].setFont(new Font("Dialog", Font.BOLD, 12));
            label_tf_dbl[i].setFont(new Font("Dialog", Font.BOLD, 12));
            label_tf_str[i].setForeground(Color.blue);
            label_tf_int[i].setForeground(Color.blue);
            label_tf_dbl[i].setForeground(Color.blue);
        }

        label_tf_str[0].setText("comment");
        tf_str[0].setText("xxxx");
        //
        
        work_dir = m_csiData.getFile( );
        // work_dir = mrui.getPreferences( ).lastWorkDirectory;
        label_tf_str[len-1].setText( "work_dir" );
        tf_str[len-1].setText(work_dir);
        //		      
        label_tf_int[0].setText("ndp");
        tf_int[0].setText("xxxx");
        label_tf_int[1].setText("ndphlsvd");
        tf_int[1].setText("xxxx");
        label_tf_int[2].setText("nrow");
        tf_int[2].setText("xxxx");
        label_tf_int[3].setText("ncom");
        tf_int[3].setText("xxxx");
        label_tf_int[4].setText("nback");
        tf_int[4].setText("xxxx");
        label_tf_dbl[0].setText("fmin (PPM)");
        tf_dbl[0].setText("xxxx");			  
        label_tf_dbl[1].setText("fmax (PPM)");
        tf_dbl[1].setText("xxxx");

        tf_int[0].setEnabled(false);

        tf_int[1].addActionListener (new java.awt.event.ActionListener () 
        {
            @Override
            public void actionPerformed (java.awt.event.ActionEvent evt) 
            {
                System.out.println("Hello in ActionListener of ndphlsvd");
                String str = tf_int[1].getText();
                int ndphlsvd = Integer.parseInt(str);
                if (ndphlsvd > ndp) 
                {
                    ndphlsvd = ndp;
                    tf_int[1].setText(Integer.toString(ndphlsvd));
                    System.out.println("ndphlsvd set at " + ndp + "!");
                }
                int nrow_min = ndphlsvd/7;
                int nrow_max = ndphlsvd/2 - 1;
                int nr1 = nrow_min - 1;
                int nr2 = nrow_max + 1;
                String label = "<html>nrow GT " + nr1 + "<br>AND LT " + nr2 + "</html>";
                label_tf_int[2].setFont(new Font("Dialog", Font.BOLD, 10));
                label_tf_int[2].setText(label);
            }
        }
        );	

        tf_int[2].addActionListener( new java.awt.event.ActionListener () 
        {
            @Override
            public void actionPerformed (java.awt.event.ActionEvent evt) 
            {
                System.out.println("Hello in ActionListener of nrow");
                String str = tf_int[1].getText();
                int ndphlsvd = Integer.parseInt(str);				
                str = tf_int[2].getText();
                int nrow = Integer.parseInt(str);
                int nrow_min = ndphlsvd/7;
                int nrow_max = ndphlsvd/2 - 1;
                if (nrow < nrow_min) 
                {
                    nrow = nrow_min;
                    tf_int[2].setText(Integer.toString(nrow));
                    System.out.println("nrow set at minimal value of " + nrow_min + "!");
                }						
                if (nrow > nrow_max) 
                {
                    nrow = nrow_max;
                    tf_int[2].setText(Integer.toString(nrow));
                    System.out.println("nrow set at maximal value of " + nrow_max + "!");
                }					
                label_tf_int[3].setText("ncom LT " + nrow);
                label_tf_int[3].setFont(new Font("Dialog", Font.BOLD, 10));
            }
        }
        );			

        tf_int[3].addActionListener (new java.awt.event.ActionListener () 
        {
            @Override
            public void actionPerformed (java.awt.event.ActionEvent evt) 
            {
                System.out.println("Hello in ActionListener of ncom");				
                String str = tf_int[2].getText();
                int nrow = Integer.parseInt(str);				
                str = tf_int[3].getText();
                int ncom = Integer.parseInt(str);				
                int ncom_max = nrow - 1;
                if ( ncom > ncom_max ) 
                {
                    ncom = ncom_max;
                    tf_int[3].setText( Integer.toString( ncom ) );
                    System.out.println("ncom set at " + ncom_max + "!");
                }					
            }
        }
        );

        tf_dbl[0].addActionListener (new java.awt.event.ActionListener () 
        {
            @Override
            public void actionPerformed (java.awt.event.ActionEvent evt) 
            {
                System.out.println("Hello in ActionListener of fmin");
                String str = tf_dbl[0].getText();
                double fmin = Double.parseDouble(str);
                double fspec_min = -1000.0/(2.0*step*larmor);
                if (fmin < fspec_min) 
                {
                    fmin = fspec_min;
                    tf_dbl[0].setText(Double.toString(fmin));
                    System.out.println("fmin set at minimal value of " + fspec_min + "!");
                }						
            }
        }
        );	

        tf_dbl[1].addActionListener (new java.awt.event.ActionListener () 
        {
            @Override
            public void actionPerformed (java.awt.event.ActionEvent evt) 
            {
                System.out.println( "Hello in ActionListener of fmax" );
                String str = tf_dbl[1].getText( );
                double fmax = Double.parseDouble( str );
                double fspec_max = 1000.0/(2.0*step*larmor);
                if ( fmax > fspec_max ) 
                {
                    fmax = fspec_max;
                    tf_dbl[1].setText( Double.toString( fmax ) );
                    System.out.println("fmax set at maximal value of " + fspec_max + "!");
                }						
            }
        }
        );	

        readsig = new JButton( "Read Signal" );
        readsig.setForeground( Color.red );
        readsig.setBackground( Color.yellow );
        readsig.addActionListener( new java.awt.event.ActionListener( ) 
        {
            @Override
            public void actionPerformed (java.awt.event.ActionEvent evt) 
            {
                readsigActionPerformed (evt);
            }
        }
        );

        quit = new JButton("Quit");
        quit.setForeground(Color.red);
        quit.setBackground(Color.yellow);
        quit.addActionListener (new java.awt.event.ActionListener( ) 
        {
            @Override
            public void actionPerformed (java.awt.event.ActionEvent evt) 
            {
                quitActionPerformed (evt);
            }
        }
        );

        runhlsvd = new JButton("Run HLSVD");
        runhlsvd.setForeground(Color.red);
        runhlsvd.setBackground(Color.yellow);
        runhlsvd.setEnabled(false);
        runhlsvd.addActionListener (new java.awt.event.ActionListener( ) 
        {
            @Override
            public void actionPerformed( java.awt.event.ActionEvent evt ) 
            {
                runhlsvdActionPerformed( evt );
            }
        }
        );

        for (int i = 0; i < len; i++) 
        {
            centerpanel.add(label_tf_str[i]);
            centerpanel.add(tf_str[i]);
        }
        for (int i = 0; i < len; i++) 
        {
            centerpanel.add(label_tf_int[i]);
            centerpanel.add(tf_int[i]);
        }
        for (int i = 0; i < len; i++) 
        {
            centerpanel.add(label_tf_dbl[i]);
            centerpanel.add(tf_dbl[i]);
        }

        northpanel.add(readsig);
        southpanel.add(quit);
        southpanel.add(runhlsvd);

        mainpanel.add(northpanel,  BorderLayout.NORTH);
        mainpanel.add(centerpanel, BorderLayout.CENTER);
        mainpanel.add(southpanel,  BorderLayout.SOUTH);

        return mainpanel;
    }
	
    private void readsigActionPerformed( java.awt.event.ActionEvent evt ) 
    {
        System.out.println("Hello in readsigActionPerformed of Gui");

//        mrui.openFileAs();
//        Data data = mrui.getData();
        
        if ( m_csiData != null )
        {
            
            Spectra spectra = m_csiData.getSpectra( );
            String key = spectra.getData( ).keySet( ).iterator( ).next( );
                    
            float[] absTD  = m_csiData.getSpectra( ).getSpectrumData( key, Spectra.Domain.TD, Spectra.Mode.ABS );
            float[] dispTD = m_csiData.getSpectra( ).getSpectrumData( key, Spectra.Domain.TD, Spectra.Mode.DISP );
            signal_name = key;
            
            // signal_name = data.getNameOfFile( );
            System.out.println( "signal_name = " + signal_name );
            // Vector vec = data.getHeader();
        
            // int vecsize = vec.size( );
            // int vecsize = absTD.length;
        
//            System.out.println( "vecsize = " + vecsize );
//
//            Object[] array = vec.toArray( );
//        
//            for( int i = 0; i < vecsize; i++ )
//            {
//                System.out.println( "array[" + i + "] = " + array[i] );
//            }

            // double dndp = data.getLength();
            double dndp = absTD.length;
            ndp = (int)Math.round(dndp);
            System.out.println("ndp = " + ndp);

            // step = (Double) vec.get(0);
            step = spectra.getDwellTime( );
            System.out.println("step = " + step);
        
            // larmor = (Double) vec.get(2);
            larmor = spectra.getResonanceFrequency( );
            larmor = larmor/1000000.0;
            System.out.println("larmor = " + larmor);

            // double[][] datapoints = data.getSignal(0);

            readwritefiles = new ReadWriteMRSFiles();
            System.out.println("Writing signal.txt");
            readwritefiles.writeTextFile (work_dir + "/signal.txt", ndp, absTD, dispTD);

            gui_info = readGuiInfo(3*len);		
            for(int i=0; i < 3*len; i++) 
            {
                System.out.println("gui_info[" + i + "] = " + gui_info[i]);
            }

            tf_str[0].setText(gui_info[0]);
            tf_int[0].setText(Integer.toString(ndp));
            tf_int[1].setText(gui_info[7]);
            tf_int[2].setText(gui_info[8]);
            tf_int[3].setText(gui_info[9]);
            tf_int[4].setText(gui_info[10]);
            tf_dbl[0].setText(gui_info[12]);
            tf_dbl[1].setText(gui_info[13]);

            readsig.setEnabled(false);
            runhlsvd.setEnabled(true);

        }
    }
    
    private void quitActionPerformed(java.awt.event.ActionEvent evt) 
    {
        try 
        {
            Process p1 = Runtime.getRuntime().exec("killall gnuplot_x11");
            Process p2 = Runtime.getRuntime().exec("killall gnuplot");
            System.out.println("After exec calls");
        }
        catch(IOException e) {}
        dispose();
    }

    private void runhlsvdActionPerformed (java.awt.event.ActionEvent evt) 
    {
		
        System.out.println("Hello in runhlsvdActionPerformed of Gui");

        String[] str_str = new String[len];
        String[] str_int = new String[len];
        String[] str_dbl = new String[len];

        for (int i = 0; i < len; i++) 
        {
            str_str[i] = tf_str[i].getText();
            System.out.println("Text of tf_str[" + i + "] = " + str_str[i]);
        }
        for (int i = 0; i < len; i++) 
        {
            str_int[i] = tf_int[i].getText();
            System.out.println("Text of tf_int[" + i + "] = " + str_int[i]);
        }
        for (int i = 0; i < len; i++) 
        {
            str_dbl[i] = tf_dbl[i].getText();
            System.out.println("Text of tf_dbl[" + i + "] = " + str_dbl[i]);
        }

        String[] str_return = callNativeC( signal_name, ndp, step, larmor,
                                           str_str, str_int, str_dbl );

        String tmp_str;		
        for( int i=0; i < str_return.length; i++ ) 
        {
            tmp_str = "str_return[" + i + "] = " + str_return[i];
            System.out.println(tmp_str);
        }
        
        for(int i=0; i < len; i++) 
        {
            gui_info[i]       = tf_str[i].getText();
            gui_info[i+  len] = tf_int[i].getText();
            gui_info[i+2*len] = tf_dbl[i].getText();
        }
        writeGuiInfo(gui_info);

        tmp_str = tf_int[1].getText();
        int ndphlsvd = Integer.parseInt(tmp_str);

        tmp_str = tf_int[4].getText();	
        int nback;
	try 
        { 
            nback = Integer.parseInt(tmp_str);
        } 
        catch( NumberFormatException e ) 
        { 
            nback = 0;
            System.out.println("NumberFormatException: nback set at 0!");
	}
	    
        text2mrui(ndphlsvd, nback, false, "hlsvd_signal",          tf_str[0].getText());
        text2mrui(ndphlsvd, nback, true,  "hlsvd_recon",           tf_str[0].getText());
        text2mrui(ndphlsvd, nback, false, "hlsvd_residu",          tf_str[0].getText());
        text2mrui(ndphlsvd, nback, false, "hlsvd_sinvals",         tf_str[0].getText());
        text2mrui(ndphlsvd, nback, false, "hlsvd_signal_filtered", tf_str[0].getText());

        runhlsvd.setText( "Run finished" );
        runhlsvd.setEnabled(false);
    }
	
    /**
     * Reads the graphical user interface information.
     * @param length
     * @return
     */
    public String[] readGuiInfo( int length ) 
    {
        // String work_dir = mrui.getPreferences().lastWorkDirectory;
        work_dir = m_csiData.getFile( );
        String file_name = work_dir + "/gui_info.txt";
    	String gui_info[] = new String[length];   
    	try 
        {		
            FileReader filereader = new FileReader( file_name );
            BufferedReader bufreader = new BufferedReader( filereader );
            for(int i=0; i < length; i++) 
            {
                gui_info[i] = bufreader.readLine( );
            }
            filereader.close( );
    	}
    	catch ( IOException e ) 
        {
            
        }	
    	return gui_info;
    }
    
    public void writeGuiInfo( String[] gui_info ) 
    {
        //String work_dir = mrui.getPreferences().lastWorkDirectory;
        work_dir = m_csiData.getFile( );
        String file_name = work_dir + "/gui_info.txt";   
    	try 
        {    		
            FileWriter filewriter = new FileWriter(file_name);
            try (PrintWriter outputfile = new PrintWriter(filewriter)) {
                for (String gui_info1 : gui_info) {
                    outputfile.println(gui_info1);
                }
            }
    	}
    	catch ( IOException e )
        {
        }	
    	return;
    }
    
    public void text2mrui( int ndp, int nback, boolean back, String filename, String comment ) 
    {
    
    	// String work_dir = mrui.getPreferences( ).lastWorkDirectory;
        work_dir = m_csiData.getFile( );
    	double[][] datapoints =  new double[2][ndp+nback];
    	double[] dreturn_true =  new double[2*(ndp+nback)];
    	double[] dreturn_false = new double[2*ndp];
    	
    	if ( back == true ) 
        {
            dreturn_true = readwritefiles.readTextFile (work_dir + "/" + filename + ".txt");
            for (int i = 0; i < ndp+nback; i++) 
            {
                datapoints[0][i] = dreturn_true[i];
                datapoints[1][i] = dreturn_true[i+ndp+nback];
            }
    	}
    	if ( back == false ) 
        {
            dreturn_false = readwritefiles.readTextFile (work_dir + "/" + filename + ".txt");
            for (int i = 0; i < nback; i++) 
            {
                datapoints[0][i] = 0.0;
                datapoints[1][i] = 0.0;
            }
            for ( int i = nback; i < ndp+nback; i++ ) 
            {
                datapoints[0][i] = dreturn_false[i-nback];
                datapoints[1][i] = dreturn_false[i+ndp-nback];
            }
    	}
    	
    	double lar = 1000000.0*larmor;
    	readwritefiles.writeMruiFile( work_dir + "/" + filename + "_" + comment + ".mrui", 
                                      ndp+nback, step, lar, datapoints );
    }

}