/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package org.spectrim.hlsvd;

import org.spectrim.tdfdfit.SingleVoxelMeasuredSpectralData;
import org.spectrim.tdfdfit.TDFDFittingDefaults;

/**
 *  Enables the user to call HLSVD in a non interactive manner.
 * @author johan
 */
public class HLSVDPro 
{
    
    private String[] args = new String[]{ "testsignal", 
                                          "1024", 
                                          "0.2", 
                                          "128.0", 
                                          "x", 
                                          "C:\\Users\\johan\\Documents\\development\\java\\Netbeans\\spectrIm-QMRS\\spectrIm\\native\\target_hlsvd_native_64bits", 
                                          "1024", 
                                          "1024", 
                                          "300", 
                                          "30", 
                                          "1.0", 
                                          "3.5" };    

    int m_numberOfPointsInHLSVDPro = new Integer( TDFDFittingDefaults.getInstance( ).getProperty( TDFDFittingDefaults.HLSVD_NUMBER_OF_DATAPOINTS ) );

    /**
     * Constructs a NonInteractiveHLSVD object.
     * @param signal_re
     * @param signal_im
     * @param acqBandwidth
     * @param resonanceFrequency
     */
    public HLSVDPro( double[] signal_re, double[] signal_im, 
                     double acqBandwidth, double resonanceFrequency,
                     double ppmReference )
    {
        
        m_signal_re = new double[signal_re.length];
        m_signal_im = new double[signal_im.length];
        
        System.arraycopy( signal_re, 0, m_signal_re, 0, signal_re.length );
        System.arraycopy( signal_im, 0, m_signal_im, 0, signal_im.length );
        
        TDFDFittingDefaults defaults = TDFDFittingDefaults.getInstance( );
        defaults.load( );
        
        String tdSampleStepSize             = "" + ( 1.0 / acqBandwidth );
        
        String larmorFrequency              = "" + resonanceFrequency;
        
        String numberOfPointsInHLSVD        = defaults.getProperty( TDFDFittingDefaults.HLSVD_NUMBER_OF_DATAPOINTS );
        String hankelMatrixSize             = defaults.getProperty( TDFDFittingDefaults.HLSVD_HANKEL_MATRIX_SIZE );
        String numberOfHLSVDComponents      = defaults.getProperty( TDFDFittingDefaults.HLSVD_NUMBER_OF_SINUSOIDS );
        // we call hlsvd itself by interchanging high and low point nummers of frequencies.
        m_hlsvdMinimumFilterFrequency       = defaults.getProperty( TDFDFittingDefaults.HLSVD_MINUMUM_FILTER_FREQUENCY );
        m_hlsvdMaximumFilterFrequency       = defaults.getProperty( TDFDFittingDefaults.HLSVD_MAXIMUM_FILTER_FREQUENCY );
        
        m_extremeLineWidthThresshold        = defaults.getProperty( TDFDFittingDefaults.HLSVD_EXTREME_LINE_WIDTH_THRESSHOLD ); 
        m_hlsvdFilterMode                   = defaults.getProperty( TDFDFittingDefaults.HLSVD_FILTER_TYPE );
        m_extremeBroadWidth                 = defaults.getProperty( TDFDFittingDefaults.HLSVD_EXTREME_LINE_WIDTH_THRESSHOLD );
                
//        System.out.println( "################################################"  );
//        System.out.println( "numberOfPointsInHLSVD : " + numberOfPointsInHLSVD );
//        System.out.println( "hankelMatrixSize : " + hankelMatrixSize );
//        System.out.println( "numberOfHLSVDComponents : " + numberOfHLSVDComponents );
//        System.out.println( "m_hlsvdMinimumFilterFrequency : " + m_hlsvdMinimumFilterFrequency );
//        System.out.println( "m_hlsvdMaximumFilterFrequency : " + m_hlsvdMaximumFilterFrequency );
//        System.out.println( "m_extremeLineWidthThresshold : " + m_extremeLineWidthThresshold );
//        System.out.println( "m_hlsvdFilterMode : " + m_hlsvdFilterMode );
//        System.out.println( "################################################\n"  );
        
        
        String[] arguments = new String[14];
        
        arguments[0] = "null";
        arguments[1] = "" + signal_re.length;
        arguments[2] = "" + tdSampleStepSize; // overwrite "tdSampleStepSize" with 1: we want point units! 
        arguments[3] = "" + larmorFrequency; // overwrite "larmorFrequency" with 1: we want point units! 
        arguments[4] = "x";
        arguments[5] = "null";
        arguments[6] = "" + signal_re.length;;
        arguments[7] = "" + numberOfPointsInHLSVD;
        arguments[8] = "" + hankelMatrixSize;
        arguments[9] = "" + numberOfHLSVDComponents;
        arguments[10] = "" + m_hlsvdMinimumFilterFrequency;
        arguments[11] = "" + m_hlsvdMaximumFilterFrequency;
        arguments[12] = "" + m_hlsvdFilterMode;
        arguments[13] = "" + m_extremeBroadWidth;
        
        float totalSpectralWidthPPM = (float)( acqBandwidth * 1.0E6 / resonanceFrequency );
        double minFreqStopInHz = ( new Double( m_hlsvdMinimumFilterFrequency ) - ppmReference ) * acqBandwidth / totalSpectralWidthPPM  ;
        double maxFreqStopInHz = ( new Double( m_hlsvdMaximumFilterFrequency ) - ppmReference ) * acqBandwidth / totalSpectralWidthPPM  ;

        m_hlsvd = new Compute_Native_HLSVD( arguments, m_signal_re, m_signal_im );
        m_hlsvd.computeHLSVDPro( minFreqStopInHz, maxFreqStopInHz);
        m_hlsvdResults = m_hlsvd.getHLSVDResults( );
        
        double[] partReal_aux = new double[signal_re.length];
        double[] partImag_aux = new double[signal_re.length];

        partReal_aux = m_hlsvdResults.getFilteredSignalRealPart();
        partImag_aux = m_hlsvdResults.getFilteredSignalImagenaryPart();
        
        float[] partReal = new float[partReal_aux.length];
        float[] partImag = new float[partImag_aux.length];
        for (int i = 0 ; i < partImag_aux.length; i++)
        {
            partReal[i] = (float) partReal_aux[i];
            partImag[i] = (float) partImag_aux[i];
        }
        
        m_residualWaterResonanceSignal = new SingleVoxelMeasuredSpectralData( signal_re.length );
        m_broadBaselineResonanceSignals = new SingleVoxelMeasuredSpectralData( signal_re.length );
        m_residualWaterResonanceSignal.addTimeDomainResponse( partReal, partImag );

        float[] real_water = m_residualWaterResonanceSignal.getAbsorptionChannelTD( ); 
        float[] imag_water = m_residualWaterResonanceSignal.getDispersionChannelTD( );
        
        double[] real_part_model_water =  new double[real_water.length];
        double[] imag_part_model_water =  new double[imag_water.length];
        
        for ( int i = 0; i < real_water.length; i++ )
        {
            real_part_model_water[i] = (double)real_water[i];
            imag_part_model_water[i] = (double)imag_water[i];
        }
        
        m_hlsvdResults.setResidualWaterModelSignalRealPart(real_part_model_water);
        m_hlsvdResults.setResidualWaterModelSignalImaginaryPart(imag_part_model_water);

        
        // Subtract water signal..
        for ( int i = 0; i < signal_re.length; i++ )
        {
            signal_re[i] = signal_re[i] - real_water[i];
            signal_im[i] = signal_im[i] - imag_water[i];
        }
        
        m_hlsvdResults.setFilteredSignalRealPart( signal_re );
        m_hlsvdResults.setFilteredSignalImagenaryPart( signal_im );
    }

    /**
     * Gives calling instance access to the results of the HLSVD.
     * @return 
     */
    public HLSVDResults getHLSVDResults( )
    {
        return m_hlsvdResults;
    }
    
    // Member variables
    private final double[]          m_signal_re;
    private final double[]          m_signal_im;
    private HLSVDResults            m_hlsvdResults;
    private Compute_Native_HLSVD    m_hlsvd;
    private final String            m_hlsvdMinimumFilterFrequency;
    private final String            m_hlsvdMaximumFilterFrequency;
    private final String            m_extremeBroadWidth;
    private final String            m_extremeLineWidthThresshold;
    private final String            m_hlsvdFilterMode;
    private final SingleVoxelMeasuredSpectralData m_residualWaterResonanceSignal;
    private final SingleVoxelMeasuredSpectralData m_broadBaselineResonanceSignals;
    
}