package org.spectrim.hlsvd;

import java.util.Random;

/**
 * Test class to launch test code of native HLSVD-code
 * 
 * Written by Ron de Beer, adapted by Johannes Slotboom.
 * 
 * @author johan
 */
public class Hlsvd_j2f 
{
    
	// Attributes 
    
	private final String[] m_pars;
	private Compute_Native_HLSVD m_nat_hlsvd;

	/**
         * Constructor of Hlsvd_j2f.
         * @param args 
         */
        public Hlsvd_j2f( String[] args ) 
        {
		m_pars = new String[args.length];

		for( int i = 0; i < args.length; i++ ) 
                {
                    m_pars[i] = args[i];
		}
	}

        /**
         * Performs the running of the native code.
         */
	public void run( )
        {
            // System.out.println("==================================");
            // System.out.println("Hello in run() method of Hlsvd_j2f");
            // System.out.println("==================================");
            double[] signal_re = new double[1024];
            double[] signal_im = new double[1024];
      
            Random rand = new Random( );

            for( int i = 0 ; i < 1024 ; i++ )
            {	
                signal_re[i] = 1.0 + Math.cos( 2.0*3.141592*i*320.0/1024.0 ) + Math.cos( 2.0*3.141592*i*330.0/1024.0 ) + Math.cos( 2.0*3.141592*i*360.0/1024.0 );
                signal_im[i] = Math.sin( 2.0*3.141592*i*320.0/1024.0 ) + Math.sin( 2.0*3.141592*i*330.0/1024.0 ) + Math.sin( 2.0*3.141592*i*360.0/1024.0 );
            }

            // First time
        
            for( int i = 0; i < 1024; i++ )
            {
                signal_re[i] = signal_re[i] * Math.exp( -i / 256.0 ) + 0.1 * 2 * ( rand.nextDouble() - 0.5 );
                signal_im[i] = signal_im[i] * Math.exp( -i / 256.0 ) + 0.1 * 2 * ( rand.nextDouble() - 0.5 );
            }
            
            // Create a new HLSVD native object.
            m_nat_hlsvd = new Compute_Native_HLSVD( m_pars, signal_re, signal_im );
            System.out.println( "Call HLSVD First time." );
            m_nat_hlsvd.computeHLSVD( );
            
            System.out.println( "Call HLSVD Second time." );
            m_nat_hlsvd.computeHLSVD( );
	}

        /**
         * Entry point of this test class
         * @param args 
         */
	public static void main( String args[] ) 
        {
            
            args = new String[]{ "testsignal", 
                                 "1024", 
                                 "0.2", 
                                 "128.0", 
                                 "x", 
                                 "C:\\Users\\johan\\Documents\\development\\java\\Netbeans\\spectrIm-QMRS\\spectrIm\\native\\target_hlsvd_native_64bits", 
                                 "1024", 
                                 "1024", 
                                 "300", 
                                 "30", 
                                 "1.0", 
                                 "3.5" };
            
            //for( int i = 0; i < args.length; i++ ) 
            //{
            //	System.out.println("args[" + i + "] = " + args[i]); 
            //}
            
            new Hlsvd_j2f( args ).run( );
	}
}
