/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package org.spectrim.hlsvd;

import java.awt.Color;
import java.util.Random;
import org.spectrim.tdfdfit.MRFittingParameters;
import org.spectrim.tdfdfit.MRLineModelSpectralResponse;
import org.spectrim.tdfdfit.MRSignalProperties;
import org.spectrim.tdfdfit.SingleVoxelMeasuredSpectralData;
import org.spectrim.tdfdfit.TDFDFittingDefaults;
import org.spectrim.xyplot.Curve;
import org.spectrim.xyplot.XYPlotFrame;

/**
 *  Enables the user to call HLSVD in a non interactive manner.
 * @author johan
 */
public class NonInteractiveHLSVD 
{
    
    private String[] args = new String[]{ "testsignal", 
                                          "1024", 
                                          "0.2", 
                                          "128.0", 
                                          "x", 
                                          "C:\\Users\\johan\\Documents\\development\\java\\Netbeans\\spectrIm-QMRS\\spectrIm\\native\\target_hlsvd_native_64bits", 
                                          "1024", 
                                          "1024", 
                                          "300", 
                                          "30", 
                                          "1.0", 
                                          "3.5" };    

    int m_numberOfPointsInHLSVD = new Integer( TDFDFittingDefaults.getInstance( ).getProperty( TDFDFittingDefaults.HLSVD_NUMBER_OF_DATAPOINTS ) );

    /**
     * Constructs a NonInteractiveHLSVD object.
     * @param signal_re
     * @param signal_im
     * @param acqBandwidth
     * @param resonanceFrequency
     */
    public NonInteractiveHLSVD( double[] signal_re, double[] signal_im, 
                                double acqBandwidth, double resonanceFrequency )
    {
        
        m_signal_re = new double[signal_re.length];
        m_signal_im = new double[signal_im.length];
        
        System.arraycopy( signal_re, 0, m_signal_re, 0, signal_re.length );
        System.arraycopy( signal_im, 0, m_signal_im, 0, signal_im.length );
        
        double magnitude_td_sig_0 = Math.sqrt( m_signal_re[0] * m_signal_re[0] + m_signal_im[0] * m_signal_im[0] );
        
        TDFDFittingDefaults defaults = TDFDFittingDefaults.getInstance( );
        defaults.load( );
        
        String tdSampleStepSize             = "" + ( 1.0 / acqBandwidth );
        
        String larmorFrequency              = "" + resonanceFrequency;
        
        String numberOfPointsInHLSVD        = defaults.getProperty( TDFDFittingDefaults.HLSVD_NUMBER_OF_DATAPOINTS );
        String hankelMatrixSize             = defaults.getProperty( TDFDFittingDefaults.HLSVD_HANKEL_MATRIX_SIZE );
        String numberOfHLSVDComponents      = defaults.getProperty( TDFDFittingDefaults.HLSVD_NUMBER_OF_SINUSOIDS );
        // we call hlsvd itself by interchanging high and low point nummers of frequencies.
        m_hlsvdMinimumFilterFrequency       = defaults.getProperty( TDFDFittingDefaults.HLSVD_MINUMUM_FILTER_FREQUENCY );
        m_hlsvdMaximumFilterFrequency       = defaults.getProperty( TDFDFittingDefaults.HLSVD_MAXIMUM_FILTER_FREQUENCY );
        
        m_extremeLineWidthThresshold        = defaults.getProperty( TDFDFittingDefaults.HLSVD_EXTREME_LINE_WIDTH_THRESSHOLD ); 
        m_hlsvdFilterMode                   = defaults.getProperty( TDFDFittingDefaults.HLSVD_FILTER_TYPE );
        m_refPPMFrequency                   = new Double( defaults.getProperty( TDFDFittingDefaults. HLSVD_PPM_REFERENCE_VALUE ) ).doubleValue( );
        
        
//        System.out.println( "################################################"  );
//        System.out.println( "numberOfPointsInHLSVD : " + numberOfPointsInHLSVD );
//        System.out.println( "hankelMatrixSize : " + hankelMatrixSize );
//        System.out.println( "numberOfHLSVDComponents : " + numberOfHLSVDComponents );
//        System.out.println( "m_hlsvdMinimumFilterFrequency : " + m_hlsvdMinimumFilterFrequency );
//        System.out.println( "m_hlsvdMaximumFilterFrequency : " + m_hlsvdMaximumFilterFrequency );
//        System.out.println( "m_extremeLineWidthThresshold : " + m_extremeLineWidthThresshold );
//        System.out.println( "m_hlsvdFilterMode : " + m_hlsvdFilterMode );
//        System.out.println( "################################################\n"  );
        
        
        String[] arguments = new String[12];
        
        arguments[0] = "null";
        arguments[1] = "" + signal_re.length;
        arguments[2] = "" + tdSampleStepSize; // overwrite "tdSampleStepSize" with 1: we want point units! 
        arguments[3] = "" + larmorFrequency; // overwrite "larmorFrequency" with 1: we want point units! 
        arguments[4] = "x";
        arguments[5] = "null";
        arguments[6] = "" + signal_re.length;;
        arguments[7] = "" + numberOfPointsInHLSVD;
        arguments[8] = "" + hankelMatrixSize;
        arguments[9] = "" + numberOfHLSVDComponents;
        arguments[10] = "" + m_hlsvdMinimumFilterFrequency;
        arguments[11] = "" + m_hlsvdMaximumFilterFrequency;
        
        m_hlsvd = new Compute_Native_HLSVD( arguments, m_signal_re, m_signal_im );
        m_hlsvd.computeHLSVD( );
        m_hlsvdResults = m_hlsvd.getHLSVDResults( );
        
        double[] ampls = m_hlsvdResults.getAmplitudesHLSVD( );
        double[] freqs = m_hlsvdResults.getFrequenciesHLSVD( );
        double[] phass = m_hlsvdResults.getPhasesHLSVD( );
        double[] damps = m_hlsvdResults.getDampingsHLSVD( );


        // Make frequency units to be expressed in points
        // Make the phase parameter correct
        for ( int i = 0; i < freqs.length;i++ )
        {
            freqs[i] = freqs[i] * signal_re.length;
            phass[i] = phass[i] * 180.0 / Math.PI;
            damps[i] = damps[i] * 1000.0 / Math.PI;
        }
        
        // determine the sum of the amps
        double sumamps = 0.0;
        for ( int i = 0; i < ampls.length; i++ )
        {
            sumamps += Math.abs( ampls[i] );
        }

        // test whether standard HLSVD failed..
        if ( sumamps < 0.01 * magnitude_td_sig_0 )
        {

            System.out.println( "#### Straight HLSVD Failed!" );
            
            // HLSVD failed apparently ...
            Random ran = new Random( );

            double[] noise_re = new double[signal_re.length];
            double[] noise_im = new double[signal_re.length];
            // compute noisy signal to perturbe the signal
            
            for ( int i = 0; i < m_signal_im.length; i++ )
            {
                noise_re[i] = 0.001 * magnitude_td_sig_0 * ( ran.nextDouble( ) - 0.5 );
                noise_im[i] = 0.001 * magnitude_td_sig_0 * ( ran.nextDouble( ) - 0.5 );
            }
            
            for ( int i = 0; i < m_signal_im.length; i++ )
            {
                m_signal_re[i] = m_signal_re[i] + noise_re[i];
                m_signal_im[i] = m_signal_im[i] + noise_im[i];
            }

            m_hlsvd = new Compute_Native_HLSVD( arguments, m_signal_re, m_signal_im );
            
            m_hlsvd.computeHLSVD( );
            
            m_hlsvdResults = m_hlsvd.getHLSVDResults( );

            ampls = m_hlsvdResults.getAmplitudesHLSVD( );
            freqs = m_hlsvdResults.getFrequenciesHLSVD( );
            phass = m_hlsvdResults.getPhasesHLSVD( );
            damps = m_hlsvdResults.getDampingsHLSVD( );
            
            // Make frequency units to be expressed in points
            // Make the phase parameter correct
            for ( int i = 0; i < ampls.length; i++ )
            {
                freqs[i] = freqs[i] * signal_re.length;
                phass[i] = phass[i] * 180.0 / Math.PI;
                damps[i] = damps[i] * 1000.0 / Math.PI;
            }
            // subtract the noise source again!
            for ( int i = 0; i < m_signal_im.length; i++ )
            {
                m_signal_re[i] = m_signal_re[i] - noise_re[i];
                m_signal_im[i] = m_signal_im[i] - noise_im[i];
            }
        }
        
        // Perform the actual  filtering of the signals ...
        float[] sumTDReal = new float[signal_re.length];
        float[] sumTDImag = new float[signal_re.length];
        
        m_residualWaterResonanceSignal = new SingleVoxelMeasuredSpectralData( signal_re.length );
        m_broadBaselineResonanceSignals = new SingleVoxelMeasuredSpectralData( signal_re.length );
        
        // Compute the water line signal
        // and extreme broad base line
        for ( int i = 0; i < ampls.length; i++ )
        {
            double ampl = ampls[i];
            double freq = freqs[i];
            double phas = phass[i];
            double damp = damps[i];
            
//            float minFreqStopInPPM = new Float ( m_minimumWaterRemovalFrequencyRangeValueTextField.getText( ) );  
//            float maxFreqStopInPPM = new Float ( m_maximumWaterRemovalFrequencyRangeValueTextField.getText( ) );

            float totalSpectralWidthPPM = (float)( acqBandwidth * 1.0E6 / resonanceFrequency );
            double minFreqStopInHz = ( new Double( m_hlsvdMinimumFilterFrequency ) - m_refPPMFrequency ) * acqBandwidth / totalSpectralWidthPPM  ;
            double maxFreqStopInHz = ( new Double( m_hlsvdMaximumFilterFrequency ) - m_refPPMFrequency ) * acqBandwidth / totalSpectralWidthPPM  ;
   
            
//            System.out.println( "Filtered water component:  " + i + " - ampl[" + i + "] = " + ampl + "  freq[" + i + "] = " + freq  + 
//                                        "  phas[" + i + "] = " + phas + "  damp[" + i + "] = " + damp );
            
            if ( ( m_hlsvdFilterMode.compareTo( TDFDFittingDefaults.HLSVD_FILTER_WATER_ONLY ) == 0 ) ||
                 ( m_hlsvdFilterMode.compareTo( TDFDFittingDefaults.HLSVD_FILTER_WATER_AND_EXTREME_BROAD_LINES ) == 0 ) )
            {
            
                if ( ( freq > minFreqStopInHz ) && ( freq < maxFreqStopInHz ) )
                {

//                    System.out.println( "Filtered water component:  " + i + " - ampl[" + i + "] = " + ampl + "  freq[" + i + "] = " + freq  + 
//                                        "  phas[" + i + "] = " + phas + "  damp[" + i + "] = " + damp );

                    MRFittingParameters parameters = new MRFittingParameters( );

                    parameters.setArea( (float)ampl );
                    parameters.setOffset( (float)freq );
                    parameters.setPhase( (float)phas );
                    parameters.setLorentzianWidth( (float)Math.abs( damp ) );
                    parameters.setGaussianWidth( 0.0f );

                    MRLineModelSpectralResponse response = new MRLineModelSpectralResponse( parameters, signal_re.length );

                    float[] partReal = new float[signal_re.length];
                    partReal = response.getAbsorptionChannelTD( );

                    float[] partImag = new float[signal_re.length];
                    partImag = response.getDispersionChannelTD( );

                    partReal[0] = 2.0f * partReal[0];
                    partImag[0] = 2.0f * partImag[0];
                   
                    m_residualWaterResonanceSignal.addTimeDomainResponse( partReal, partImag );
                }
            }
                      
            if ( ( m_hlsvdFilterMode.compareTo( TDFDFittingDefaults.HLSVD_FILTER_EXTREME_BROAD_LINES_ONLY ) == 0 ) ||
                 ( m_hlsvdFilterMode.compareTo( TDFDFittingDefaults.HLSVD_FILTER_WATER_AND_EXTREME_BROAD_LINES ) == 0 ) )
            {
            
                if ( Math.abs( damp ) > new Float( m_extremeLineWidthThresshold ) )
                {
                    MRFittingParameters parameters = new MRFittingParameters( );

                    parameters.setArea( (float)ampl );
                    parameters.setOffset( (float)freq );
                    parameters.setPhase( (float)phas );
                    parameters.setLorentzianWidth( (float)Math.abs( damp ) );
                    parameters.setGaussianWidth( 0.0f );

                    MRLineModelSpectralResponse broadresponse = new MRLineModelSpectralResponse( parameters, signal_re.length );

                    float[] partReal = new float[signal_re.length];
                    partReal = broadresponse.getAbsorptionChannelTD( );

                    float[] partImag = new float[signal_re.length];
                    partImag = broadresponse.getDispersionChannelTD( );

                    partReal[0] = 2.0f * partReal[0];
                    partImag[0] = 2.0f * partImag[0];

                    m_broadBaselineResonanceSignals.addTimeDomainResponse( partReal, partImag ); 

                }
            }
        }
        
        float[] real_water = m_residualWaterResonanceSignal.getAbsorptionChannelTD( ); 
        float[] imag_water = m_residualWaterResonanceSignal.getDispersionChannelTD( );
        
        double[] real_part_model_water =  new double[real_water.length];
        double[] imag_part_model_water =  new double[imag_water.length];
        
        for ( int i = 0; i < real_water.length; i++ )
        {
            real_part_model_water[i] = (double)real_water[i];
            imag_part_model_water[i] = (double)imag_water[i];
        }
        
        m_hlsvdResults.setResidualWaterModelSignalRealPart(real_part_model_water);
        m_hlsvdResults.setResidualWaterModelSignalImaginaryPart(imag_part_model_water);
        
        float[] real_broad_baseline = m_broadBaselineResonanceSignals.getAbsorptionChannelTD( ); 
        float[] imag_broad_baseline = m_broadBaselineResonanceSignals.getDispersionChannelTD( );
        
        double[] real_part_model_broad_baseline =  new double[real_broad_baseline.length];
        double[] imag_part_model_broad_baseline =  new double[imag_broad_baseline.length];
        
        for ( int i = 0; i < real_water.length; i++ )
        {
            real_part_model_broad_baseline[i] = (double)real_broad_baseline[i];
            imag_part_model_broad_baseline[i] = (double)imag_broad_baseline[i];
        }
        
        m_hlsvdResults.setBroadBaselineModelSignalRealPart( real_part_model_broad_baseline );
        m_hlsvdResults.setBroadBaselineModelSignalImaginaryPart( imag_part_model_broad_baseline );
        
        float[] real_broad = m_broadBaselineResonanceSignals.getAbsorptionChannelTD( ); 
        float[] imag_broad = m_broadBaselineResonanceSignals.getDispersionChannelTD( );
        
        // Subtract water signal..
        for ( int i = 0; i < signal_re.length; i++ )
        {
            if ( i == 0 )
            {
                System.out.println( "water real[0] = " + real_water[0]);
            }
            signal_re[i] = signal_re[i] - real_water[i] - real_broad[i];
            signal_im[i] = signal_im[i] - imag_water[i] - imag_broad[i];
        }
        
        m_hlsvdResults.setFilteredSignalRealPart( signal_re );
        m_hlsvdResults.setFilteredSignalImagenaryPart( signal_im );
         
    }

    /**
     * Gives calling instance access to the results of the HLSVD.
     * @return 
     */
    public HLSVDResults getHLSVDResults( )
    {
        return m_hlsvdResults;
    }
    
    // Member variables
    private final double[]          m_signal_re;
    private final double[]          m_signal_im;
    private HLSVDResults            m_hlsvdResults;
    private Compute_Native_HLSVD    m_hlsvd;
    private final String            m_hlsvdMinimumFilterFrequency;
    private final String            m_hlsvdMaximumFilterFrequency;
    private final String            m_extremeLineWidthThresshold;
    private final String            m_hlsvdFilterMode;
    private final double            m_refPPMFrequency;
    
    private final SingleVoxelMeasuredSpectralData m_residualWaterResonanceSignal;
    private final SingleVoxelMeasuredSpectralData m_broadBaselineResonanceSignals;
    
}
