/*
 * File: $Id: PolygonUtils.java,v 1.1.1.1 2006/04/19 09:38:47 hans Exp $
 * History
 * Date: 19.03.2003  Authors: Hans Slotboom  
 *
 * Changes:
 *
 * Copyright (C) 2003 by
 * 
 * Inselspital - Department of Neuroradiology
 * CH 3010 Bern Switzerland
 */

package org.spectrim.imagegraphicobjects;


import java.awt.Point;
import java.awt.Polygon;

/**
 * Class that offers some utility functions related to polygons.
 *
 * @author  hans
 */
public class PolygonUtils 
{
    
    /** 
     * Creates a new instance of PolygonUtils. 
     */
    public PolygonUtils( ) 
    {
    }
    
    /**
     * Returns the starting point of the polygon
     * (method should be defined elsewhere).
     * @param field
     * @param lastLocation
     * @return 
     */
    public Point getPolygonStart( int[][] field, Point lastLocation ) 
    {
        
        //System.out.println( "polygon start : (" + lastLocation.x + "," + lastLocation.y + ")" );
        
        int w = field[0].length;
        int h = field.length;
        Point p = new Point( lastLocation );
        while ( true )
        {
            if ( p.x == w - 1 )
            {
                p.x = 0;
                p.y++;
            }
            if ( p.y == h - 1 ) break;
            if ( field[p.y][p.x + 1] < 0 ) break;
            else if ( field[p.y][p.x + 1] > 0 ) p = skipToNext( field, p );
            else p.x++;
        }
        if ( p.y == h - 1 )
        { 
            return null;
        }
        else
        { 
            return p;
        }
    }
    
    /**
     * Skips to next point.
     * @param field
     * @param p
     * @return 
     */
    public Point skipToNext( int[][] field, Point p ) 
    {
        int w = field[0].length;
        Point p1 = new Point( p.x + 1, p.y );
        while ( p1.x < w - 1 && field[p1.y][p1.x] != 0 ) p1.x++;
        return p1;
    }
    
    /**
     * Returns the border of a region.
     * @param field
     * @param startingPoint
     * @param polygonIndex
     * @return 
     */
    public RegioPolygon regionBorder( int[][] field, Point startingPoint, int polygonIndex ) 
    {

        /* the encoded step directions (0..7)
	   
            7   0   1       .--> stepX
                           |
            6   .   2       |
	                   V stepY
            5   4   3
        */
        int[] stepX = {  0,  1,  1,  1,  0, -1, -1, -1 };
        int[] stepY = { -1, -1,  0,  1,  1,  1,  0, -1 };

        Polygon pol = new Polygon( );
        int currentDirection = 0;
        int lastDirection = currentDirection;
        Point cP = new Point( startingPoint );
	
        while ( true )
        {
            int start = currentDirection + 2;
            int end = start -6;
            int newDirection = start;
            while ( newDirection >= end )
            {
                int d = ( newDirection < 0 ? newDirection + 8 : newDirection % 8 );
                if ( field[cP.y + stepY[d]][cP.x + stepX[d]] == 0 ) break;
                else
                {
                    field[cP.y + stepY[d]][cP.x + stepX[d]] = 1;
                    newDirection--;
                }
            }

            currentDirection = ( newDirection < 0 ? newDirection + 8 : newDirection % 8 );
            cP.translate( stepX[currentDirection], stepY[currentDirection] );
            if ( cP.equals( startingPoint ) ) break;
            else
            {
                if ( lastDirection!=currentDirection) pol.addPoint( cP.x, cP.y );
                lastDirection = currentDirection;
            }
        }
        return new RegioPolygon( polygonIndex, pol.xpoints, pol.ypoints, pol.npoints );
    }
    
}
