/*
 * File: $Id: RegioPolygon.java,v 1.2 2006/11/09 10:39:19 hans Exp $
 * History
 * Date: 30.11.2002  Authors: Martin Buerki / Hans Slotboom  
 *
 * Changes:
 *
 * Copyright (C) 2003 by
 * 
 * Inselspital - Department of Neuroradiology
 * CH 3010 Bern Switzerland
 */

package org.spectrim.imagegraphicobjects;


import java.awt.Color;
import java.awt.Point;
import java.awt.Polygon;
import java.awt.Rectangle;
import java.nio.ByteBuffer;
 

public class RegioPolygon extends Polygon 
{

  
    
    /**
     * Copy constructor.
     */
    public RegioPolygon( RegioPolygon polygon )
    {
        super( polygon.xpoints, polygon.ypoints, polygon.npoints );
        this.m_index = polygon.m_index;
        this.m_label = polygon.m_label;
        this.m_color = polygon.m_color;
        this.m_labelColor = polygon.m_labelColor;
    }
    
    public RegioPolygon( int index )
    {
        super( );
        this.m_index = index;
        m_label = LABEL_PREFIX + index;
        m_color = Color.red;
        m_labelColor = Color.green;
    }

    public RegioPolygon( int index, int[] x, int[] y, int n ) 
    {
        super( x, y, n );
        this.m_index = index;
        m_label = LABEL_PREFIX + index;
        m_color = Color.red;
        m_labelColor = Color.green;
    }

    /**
     * Returns the point of gravity of the polygon
     * @return
     */
    public Point getPointOfGravity( )
    {
        float meanX = 0f;
        float meanY = 0f;

        for ( int i = 0; i < xpoints.length; i++ )
        {
            meanX = meanX + xpoints[i];
            meanY = meanY + ypoints[i];
        }

        meanX = meanX / xpoints.length;
        meanY = meanY / ypoints.length;

        return new Point( (int)(meanX + 0.5), (int)( meanY + 0.5 ) );

    }

    /**
     * Sets the new point of gravity. If this points is different from the 
     * current point of gravity, this will result in a translation of the 
     * object
     * @param p
     */
    public void setTranslatePointOfGravity( Point p )
    {
        for( int i = 0; i < xpoints.length; i++ )
        {
            xpoints[i] = xpoints[i] - p.x;
            ypoints[i] = ypoints[i] - p.y;
        }
    }

    /**
     * Rotates a roi about the point of gravity
     */
    public void rotateAboutPointOfGravity( double angle )
    {
        RegioPolygon polyCopy = new RegioPolygon( this );

        Point p = polyCopy.getPointOfGravity();
        polyCopy.setTranslatePointOfGravity( p );

        double[] rotatedX = new double[polyCopy.xpoints.length];
        double[] rotatedY = new double[polyCopy.ypoints.length];

        int[] outX = new int[polyCopy.xpoints.length];
        int[] outY = new int[polyCopy.ypoints.length];

        for ( int i = 0; i < polyCopy.xpoints.length; i++ )
        {
            rotatedX[i] = polyCopy.xpoints[i] * Math.cos( 2.0 * Math.PI * angle / 360.0 ) - polyCopy.ypoints[i] * Math.sin( 2.0 * Math.PI * angle / 360.0 );
            rotatedY[i] = polyCopy.ypoints[i] * Math.cos( 2.0 * Math.PI * angle / 360.0 ) + polyCopy.xpoints[i] * Math.sin( 2.0 * Math.PI * angle / 360.0);

            outX[i] = (int)( rotatedX[i] + 0.5 );
            outY[i] = (int)( rotatedY[i] + 0.5 );
        }

        RegioPolygon polyCopy2 = new RegioPolygon( this.getIndex( ), outX, outY, outX.length );

        polyCopy2.setTranslatePointOfGravity( new Point( -p.x, -p.y ) );

        // copy rotated points back in "this"
        for ( int i = 0; i < this.xpoints.length; i++ )
        {
            xpoints[i]= polyCopy2.xpoints[i];
            ypoints[i]= polyCopy2.ypoints[i];
        }

    }

    /**
     * Verifies whether two polygons do intersect or not
     * @param poly_1  RegioPolygon 1
     * @param poly_2  RegioPolygon 2
     * @return true if interesecting, false if not
     */
    public boolean intersects( RegioPolygon poly_1, RegioPolygon poly_2 )
    {
        if ( poly_1.npoints > poly_2.npoints )
        {
            for ( int i = 0 ; i < poly_2.npoints; i++ )
            {
                if ( poly_1.contains( poly_2.xpoints[i], poly_2.ypoints[i] ) == true )
                {
                    return true;
                }
            }
        }

        if ( poly_2.npoints > poly_1.npoints )
        {
            for ( int i = 0 ; i < poly_1.npoints; i++ )
            {
                if ( poly_2.contains( poly_1.xpoints[i], poly_1.ypoints[i] ) == true )
                {
                    return true;
                }
            }
        }

        return false;
    }

    /**
     * 
     * @return
     */
    public boolean isTotallyWithin( RegioPolygon poly_1 )
    {

        for ( int i = 0 ; i < this.npoints; i++ )
        {
            if ( poly_1.contains( this.xpoints[i], this.ypoints[i] ) == false )
            {
                return false;
            }
        }

        return true;
    }

     /**
     * Verifies whether two polygons do intersect or not
     * @param poly_1  RegioPolygon 1
     * @return true if intersecting, false if not
     */
    public boolean intersects( RegioPolygon poly_1 )
    {
//        if ( poly_1.npoints > this.npoints )
//        {
            for ( int i = 0 ; i < this.npoints; i++ )
            {
                if ( poly_1.contains( xpoints[i], ypoints[i] ) == true )
                {
                    return true;
                }
            }
//        }
//        else
//        {
            for ( int i = 0 ; i < poly_1.npoints; i++ )
            {
                if ( this.contains( poly_1.xpoints[i], poly_1.ypoints[i] ) == true )
                {
                    return true;
                }
            }
//        }

        return false;
    }

    /**
     *
     * Sets the instance number of the image the polygon belongs to
     *
     * @param instanceNumber
     */
    public void setInstanceNumber( int instanceNumber )
    {
        m_instanceNumber = instanceNumber;
    }


    /**
     * Sets the index number of the polygon region
     * @param newIndex
     */
    public void setIndex( int newIndex )
    {
        boolean defaultLabel = m_label.equals( LABEL_PREFIX + m_index );
        m_index = newIndex;
        if (defaultLabel) m_label = LABEL_PREFIX + newIndex;
    }
    
    /**
     * Returns the m_index member
     */
    public int getIndex( )
    {
        return m_index;
    }
    
    public void setLabel( String newLabel ) 
    {
        m_label = newLabel;
    }
    
    /**
     * Returns the m_label string, i.e the name of Polygon.
     */
     public String getLabel( ) 
    {
        return m_label;
    }
    
    public void setLabelColor( Color c ) 
    {
        m_labelColor = c;
    }

    public void setColor( Color c ) 
    {
        m_color = c;
    }

    /**
     * Sets the scaling factor to apply.
     */
    public void setScalingFactor( double s ) 
    {
        m_scaling_factor *= s;
        for ( int k = 0; k < npoints; k++ ) 
        {
            xpoints[k] *= s;
            ypoints[k] *= s;
        }
	calculateBounds( );
    }
    
    /**
     * Sets the scaling factor to apply.
     */
    public void setScalingFactor( float s )
    {
        m_scaling_factor *= s;
	
        for ( int k = 0; k < npoints; k++ ) 
        {
            xpoints[k] = (int)( (float)xpoints[k] * s );
            ypoints[k] = (int)( (float)ypoints[k] * s );
        }
        
        calculateBounds( );
    }


    /**
     * Sets the scaling factor to apply.
     * @param s
     */
    public void setHorizontalScalingFactor( float s )
    {
        m_scaling_factor *= s;

        for ( int k = 0; k < npoints; k++ )
        {
            xpoints[k] = (int)( (float)xpoints[k] * s );
        }

        calculateBounds( );
    }


    /**
     * Sets the scaling factor to apply.
     * @param s
     */
    public void setVerticalScalingFactor( float s )
    {
        m_scaling_factor *= s;

        for ( int k = 0; k < npoints; k++ )
        {
            ypoints[k] = (int)( (float)ypoints[k] * s );
        }

        calculateBounds( );
    }


    /**
     * Returns the applied scaling factor.
     * @return 
     */
    public float getScalingFactor( ) 
    {
        return m_scaling_factor;
    }
    
    public RegioPolygon scaled( int s ) 
    {
        RegioPolygon p = new RegioPolygon( m_index, xpoints, ypoints, npoints );
        p.setScalingFactor( s );
        return p;
    }

    /**
     * Returns the scaled polygon.
     * @param s
     * @return 
     */
    public RegioPolygon scaled( float s ) 
    {
        
        RegioPolygon p = new RegioPolygon( m_index, xpoints, ypoints, npoints );
        p.setScalingFactor( s );
        
        return p;
        
    }

    /**
     * 
     * @param s
     * @return 
     */
    public RegioPolygon scaledToPointOfGravity( double s ) 
    {
        
        RegioPolygon p = new RegioPolygon( m_index, xpoints, ypoints, npoints );
        
        Point center = p.getPointOfGravity( );
        p.translate( -center.x, -center.y );
        p.setScalingFactor( s );
        p.translate( center.x, center.y );
        
        return p;
        
    }
    
    /**
     * Returns the average radius value with respect to point of gravity.
     * @return 
     */
    public float getMeanRadius( ) 
    {
        RegioPolygon p = new RegioPolygon( m_index, xpoints, ypoints, npoints );
        
        Point center = p.getPointOfGravity( );
        p.translate( -center.x, -center.y );
        
        
        float mean_radius = 0f;
        float cur_radius = 0f;
        
        for ( int i = 0; i < p.npoints; i++ )
        {
            cur_radius = cur_radius + (float)Math.sqrt( p.xpoints[i] * p.xpoints[i] + p.ypoints[i] * p.ypoints[i] );
        }
        
        mean_radius = cur_radius/p.npoints;
        
        return mean_radius;
                
    }
    
    
    /**
     * Deep copy of a RegioPolygon object.
     * @return
     */
    public RegioPolygon copy( )
    {
        RegioPolygon p = new RegioPolygon( m_index, xpoints, ypoints, npoints );
        return p;
    }

    /**
     * Returns the inner-volume to caller
     * @return
     */
    public int getInnerVolume( ) 
    {
        Rectangle b = getBounds( );
        int volume = 0;
        for ( int x = b.x; x <= b.x + b.width; x++ )
        {
            for ( int y = b.y; y <= b.y + b.height; y++ )
            {
                if ( contains( x, y ) )
                { 
                    volume++; 
                }
            }
        }
        return volume;
    }

    /**
     * Returns the inner-volume of the regio.
     * @param dims
     * @return
     */
    public double getInnerVolumeMM( double[] dims ) 
    {
        int vol = getInnerVolume( );
        return dims[0] * dims[1] * dims[2] * (double)vol;
    }

     /**
     * Appends points to an already existing RegioPolygon
     */
    public boolean isEmpty( )
    {
        Rectangle b = getBounds( );
        for ( int x = b.x; x <= b.x + b.width; x++ )
        {
            for ( int y = b.y; y <= b.y + b.height; y++)
            {
                if ( contains( x, y ) )
                {
                    return false;
                }
            }
        }
        return true;
    }

     /**
     * Computes the volume of the RegioPolygon.
     * @param size
     */
    public boolean isBiggerThan( int size ) 
    {
        Rectangle b = getBounds( );
        int volume = 0;
        for ( int x = b.x; x <= b.x+b.width; x++ )
        {
            for ( int y=b.y; y<=b.y+b.height; y++ )
            {
                if ( contains( x, y ) )
                {
                    volume++;
                    if ( volume > size )
                    {
                        return true;
                    }
                }
            }
		}
        return false;
    }


     /**
     * Replaces all points to an already existing RegioPolygon.
     * @param x
     * @param y
     * @param n
     */
    public void replaceAllPoints( int[] x, int[] y, int n ) 
    {
        int N = n;

        if ( x.length < N )
        {
            N = x.length;
        }

        if ( y.length < N )
        {
            N = y.length;
        }
        xpoints = new int[N];
        ypoints = new int[N];
        npoints = N;
        System.arraycopy( x, 0, xpoints, 0, N );
        System.arraycopy( y, 0, ypoints, 0, N );
        calculateBounds( );
    }

    /**
     * Appends points to an already existing RegioPolygon.
     * @param x
     * @param y
     * @param n
     */
    public void appendPoints( int[] x, int[] y, int n ) 
    {
        int N = n;
        if ( x.length < N )
        {
            N = x.length;
        }
        if ( y.length < N )
        {
            N = y.length;
        }
        for ( int k = 0; k < N; k++ )
        {
            addPoint( x[k], y[k] );
        }
        calculateBounds( );
    }

    /**
     *
     * Appends points to the RegioPolygon.
     *
     * @param x
     * @param y
     * @param start
     * @param end
     */
    public void appendPoints( int[] x, int[] y, int start, int end ) 
    {
        if ( start <= end )
        {
            for ( int k=start; k<=end; k++ )
            { 
                addPoint(x[k], y[k]); 
            }
        }
        else
        {
            for ( int k = start; k >= end; k-- )
            {
                addPoint( x[k], y[k] );
            }
        }
        calculateBounds( );
    }

    public void clear( )
    {
        npoints = 0;
    }
    
    /**
     * Calculates the bounds of the polygon. 
     */
    public void calculateBounds( )
    {
        int boundsMinX = Integer.MAX_VALUE;
        int boundsMinY = Integer.MAX_VALUE;
        int boundsMaxX = Integer.MIN_VALUE;
        int boundsMaxY = Integer.MIN_VALUE;
	
        for ( int i = 0; i < npoints; i++ )
        {
            int x = xpoints[i];
            boundsMinX = Math.min( boundsMinX, x );
            boundsMaxX = Math.max( boundsMaxX, x );
            int y = ypoints[i];
            boundsMinY = Math.min( boundsMinY, y );
            boundsMaxY = Math.max( boundsMaxY, y );
        }
        bounds = new Rectangle( boundsMinX, boundsMinY,
                                boundsMaxX - boundsMinX,
                                boundsMaxY - boundsMinY );
    }

    /**
     * Reduces the number of vertices, while retaining the approximate shape.
     * @param minDeviation 
     */
    public void reduceVertices( double minDeviation ) 
    {
        double phi1, phi2;
        int k;
        int n = 0;
        int x[] = new int[npoints];
        int y[] = new int[npoints];
        double dp = minDeviation*Math.PI / 180.0;
	
        phi1 = Math.atan2( (double)( xpoints[0] - xpoints[npoints-1]), (double)( ypoints[0] - ypoints[npoints-1] ) );
        phi2 = Math.atan2( (double)( xpoints[1] - xpoints[0]), (double)( ypoints[1] - ypoints[0] ) );
        
        if ( Math.abs( phi2 - phi1 ) > dp )
        {
            x[n] = xpoints[0];
            y[n] = ypoints[0];
            n++;
        }

        for ( k = 1; k < npoints - 1; k++ )
        {
            phi1 = Math.atan2( (double)(xpoints[k]-xpoints[k-1]), (double)(ypoints[k]-ypoints[k-1]));
            phi2 = Math.atan2( (double)(xpoints[k+1]-xpoints[k]), (double)(ypoints[k+1]-ypoints[k]));
            if ( Math.abs( phi2 - phi1 ) > dp)
            {
                x[ n ] = xpoints[ k ];
                y[ n ] = ypoints[ k ];
                n++;
            }
        }

        phi1 = Math.atan2( (double)(xpoints[k] - xpoints[k-1]), (double)(ypoints[k]-ypoints[k-1] ) );
        phi2 = Math.atan2( (double)(xpoints[0] - xpoints[k]), (double)(ypoints[0]-ypoints[k] ) );

        if ( Math.abs( phi2 - phi1 ) > dp )
        {
            x[ n ] = xpoints[ k ];
            y[ n ] = ypoints[ k ];
            n++;
        }

        System.arraycopy( x, 0, xpoints, 0, n );
        System.arraycopy( y, 0, ypoints, 0, n );
        npoints = n;
    }

    /**
     * Returns the nearest vertex.
     * @param x
     * @param y
     * @return 
     */
    public int[] getNearestVertex( int x, int y ) 
    {
        // returns info about the nearest vertex of the polygon to the point (x,y)
        // in the following order: { m_index, x-coordinate, y-coordinate, square distance }
        int[] nearestVertex = new int[4];
        nearestVertex[0] = 0;
        nearestVertex[1] = xpoints[0];
        nearestVertex[2] = ypoints[0];
        nearestVertex[3] = ( xpoints[ 0 ] - x ) * ( xpoints[ 0 ] - x ) +
                           ( ypoints[ 0 ] - y ) * ( ypoints[ 0 ] - y );
        for ( int k = 1; k < npoints; k++ )
        {
            int d2 = ( xpoints[ k ] - x ) * ( xpoints[ k ] - x ) +
                     ( ypoints[ k ] - y ) * ( ypoints[ k ] - y );
            if ( d2 < nearestVertex[ 3 ] )
            { 
                nearestVertex[0] = k;
                nearestVertex[1] = xpoints[k];
                nearestVertex[2] = ypoints[k];
                nearestVertex[3] = d2;
            }
        }
	
        return nearestVertex;
    }

    /**
     * Returns the RegioPolygon as InputStream.
     */ 
    public ByteBuffer regioPolygonasAsIntByteBuffer( )
    {
        
        // 4 bytes per int !
        int buffersize = 4 * 2 * npoints;
        
        ByteBuffer buf = ByteBuffer.allocate( buffersize );
        buf.hasArray( );
        
        buf.asIntBuffer( );
        
        for ( int i = 0 ; i < npoints ; i++ )
        {
            buf.putInt( xpoints[i] );
            buf.putInt( ypoints[i] );
            System.out.println(" write to db (x,y) = (" + xpoints[i]  + "," + ypoints[i] + ")" );
        }
        
        buf.hasArray( );
        
        return buf;
    }    

    /**
     * Adds a connected polygon to the RegioPolygonConnections object.
     * @param poly
     */
    public void addConnectedRegioPolygon( RegioPolygon poly )
    {

        if ( m_regionPolygonConnections == null )
        {
            m_regionPolygonConnections = new RegioPolygonConnections( m_instanceNumber, m_index );
        }

        m_regionPolygonConnections.addConnectedRegioPolygon( poly );

    }

    /**
     * Allows other objects access to the RegioPolygonConnections information.
     * @return
     */
    public RegioPolygonConnections getConnectedRegionPolyInformation( )
    {
        return m_regionPolygonConnections;
    }

    /**
     * Allows to reset the polygon information; connection 
     * information is dynamic.
     */
    public void resetConnectedRegionPolyInformation( )
    {
        m_regionPolygonConnections = null;
        m_regionPolygonConnections = new RegioPolygonConnections( );
    }
    
    /**
     * 
     * @return 
     */
    public String toJSON( ) {
        StringBuilder sb = new StringBuilder( );
        
        if( this.npoints > 0 ) {
            char bracketOpen = '[';
            char bracketClose = ']';
            char comma = ',';
            sb.append( bracketOpen );
            for( int k = 0; k < this.npoints; k++ ) {
                sb.append( bracketOpen );
                sb.append( this.xpoints[ k ] );
                sb.append( comma );
                sb.append( this.ypoints[ k ] );
                sb.append( bracketClose );
            }
            sb.append( bracketClose );
        }
        
        return sb.toString( );
    }
    
    // Members 
    public final static String LABEL_PREFIX = "region ";
    public String m_label;
    public int m_index;
    public Color m_color;
    public Color m_labelColor;
    public float m_scaling_factor = 1.0f;

    // needed for polygon connection handling
    private int m_instanceNumber;

    // region polygon connection
    private RegioPolygonConnections m_regionPolygonConnections = new RegioPolygonConnections( );

    // For proper serialization
    private static final long serialVersionUID = 1499324191289393L;
 
}