/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package org.spectrim.imagegraphicobjects;

import java.io.IOException;
import java.io.RandomAccessFile;
import java.util.LinkedHashMap;
import java.util.Vector;

/**
 * 
 * Parses a .vec file of Arto Nirkko containing a list of regions.
 *
 * File has reference fileNameAndPath;
 * 
 * @author slotboom
 */
public class RegionsInVectorFileFormatReader {

    /**
     * Creates a RegionsInVectorFileFormatReader which parses a .vec file
     * and creates RegioPolygons of the ROI defined in it
     * 
     * @param columns number of horizontal points of the images where the contours will belong to
     * @param rows number of vertical points of the images where the contours will belong to
     */
    public RegionsInVectorFileFormatReader( int columns, int rows )
    {
        m_imageCols = columns;
        m_imageRows = rows;
    }
    
    /**
     * Reads
     * @param fileNameAndPath
     * @return
     */
    public LinkedHashMap<String, RegioPolygon > parseFile( String fileNameAndPath )
    {
        int returnVal=0;
        
        try
        {
            m_f = new RandomAccessFile( fileNameAndPath, "r" );
            
            String str = m_f.readLine( );
            // check whether the file is a valid .vec-file
            if ( str.compareTo( "Ima Objects" ) == 0 )
            {
                returnVal = parseRestOfFile( );
            }
        }
        catch ( IOException e )
        {
            e.printStackTrace( );
            System.out.println( "IOException while reading " + fileNameAndPath );
            return null;
        }
        if ( returnVal == 1 )
        {
            return m_namedRegioPolygons;
        }
        else
        {
            return null;
        }
    }
    
    /**
     * Returns the LinkedHashMap containing key=<names> value= <polygon>
     * @return
     */
    public LinkedHashMap<String, RegioPolygon > getRegioPolygons( )
    {
        return m_namedRegioPolygons;
    }
    
    /**
     *  Does the parsing and constructs a list of valid RegioPolygon
     * @return -1 if file cannot be parsed due to some exception
     */
    private int parseRestOfFile( )
    {
        int qcol, qrow, index;
        String polygonName;
        try
        {
            String str = m_f.readLine( );
        
            while ( str != null )
            {
            
                if ( str.compareTo( "===========" ) == 0 )
                {
                    // do nothing
                }
            
                if ( str.contains( "$FACTOR_INTERN_SLICES" ) == true )
                {
                    m_numberOfSlicesIntern = ( new Integer( str.substring( str.indexOf( '=' ) + 1, str.length( ) ) ) ).intValue( );
                }
            
                if ( str.contains( "$FACTOR_INTERN_ROWS" ) == true )
                {
                    m_numberOfRowsIntern = ( new Integer( str.substring( str.indexOf( '=' ) + 1, str.length( ) ) ) ).intValue( );
                }
            
                if ( str.contains( "$FACTOR_INTERN_COLS" ) == true )
                {
                    m_numberOfColsIntern = (new Integer( str.substring( str.indexOf( '=' ) + 1, str.length( ) ) ) ).intValue( );
                }
            
                if ( str.contains( "$TYPE(2)" ) == true )
                {
                    m_dataType = str.substring( str.lastIndexOf( ' ' ) + 1, str.length( ) );
                }
            
                if ( str.contains( "#type" ) == true )
                {
                    m_dataTypeDescriptor_3 = str.substring( str.lastIndexOf( '\t' ) + 1, str.length( ) );
                    str = str.substring( 0, str.lastIndexOf( '\t' ) - 1 );
                    m_dataTypeDescriptor_2 = str.substring( str.lastIndexOf( '\t' ) + 1, str.length( ) );
                    str = str.substring( 0, str.lastIndexOf( '\t' ) - 1 );
                    m_dataTypeDescriptor_1 = str.substring( str.lastIndexOf( ' ' ) + 1, str.length( ) ).trim( );
                }
            
                if ( str.contains( "$DATA(2)" ) == true )
                {
                    qcol = new Integer( str.substring( str.lastIndexOf( '\t' ) + 1, str.length( ) ) ).intValue( );
                    str = str.substring( 0, str.lastIndexOf( '\t' ) - 1 );
                    qrow = new Integer( str.substring( str.lastIndexOf( '\t' ) + 1, str.length( ) ) ).intValue( ) ;
                    str = str.substring( 0, str.lastIndexOf( '\t' ) - 1 );
                    index = new Integer( str.substring( str.lastIndexOf( '\t' ) + 1, str.length( ) ) );
                
                    m_qrows.add( new Integer( qrow ) );
                    m_qcols.add( new Integer( qcol ) );
                    m_index.add( new Integer( index ) );
                }
            
            
                if ( str.contains( "$DATA(70)" ) == true )
                {
                    // retrieve the name of the polygon
                    polygonName = str.substring( str.indexOf( '"' ) + 1, str.lastIndexOf( '"' ) );
                
                    m_polygonIndices.clear( );
                    
                    while ( str.lastIndexOf( '\t' ) > str.lastIndexOf( '"' ) )
                    {
                        // parse the rest of the line
                        if ( ( str.lastIndexOf( '\t' ) + 1 ) < str.length( ) ) 
                        {
                            
                            index = new Integer( str.substring( str.lastIndexOf( '\t' ) + 1, str.length( ) ) ).intValue( );
                    
                            // add it to the vector of index values
                            m_polygonIndices.add( new Integer( index ) );
                    
                            // retrieve the new shorter string
                            if ( str.lastIndexOf( '\t' ) >  str.lastIndexOf( '"' )  )
                            {
                                str = str.substring( 0, str.lastIndexOf( '\t' ) - 1 );
                            }
                            else
                            {
                                break;
                            }
                        }
                        else
                        {
                            break;
                        }
                    }
                
                    // the line has been parsed: create RegioPolygon-object and
                    // store in into an LinkedHashMap
                    int[] x_coordinates = new int[ m_polygonIndices.size( ) ];
                    int[] y_coordinates = new int[ m_polygonIndices.size( ) ];
                
                    for ( int i = 0; i < m_polygonIndices.size( ); i++ )
                    {
                        x_coordinates[i] = (int)( ( (float)( m_qcols.get( ( m_polygonIndices.get( i ) ).intValue( ) ) ) * (float)m_imageCols ) / (float)m_numberOfColsIntern );
                        y_coordinates[i] = (int)( ( (float)( m_qrows.get( ( m_polygonIndices.get( i ) ).intValue( ) ) ) * (float)m_imageRows ) / (float)m_numberOfRowsIntern );
                    }
                    
                    RegioPolygon poly = new RegioPolygon( m_namedRegioPolygons.size( ), x_coordinates, y_coordinates, x_coordinates.length );
                    poly.setLabel( polygonName + "[" + m_namedRegioPolygons.size( ) + "]" );
                    
                    m_namedRegioPolygons.put( polygonName + "[" + m_namedRegioPolygons.size( ) + "]", poly );
                }
            
            
                // read the next line of the file
                str = m_f.readLine( );
            }
        }
        catch ( IOException e )
        {
            // inform user
            // print the stack trace if this occurs
            e.printStackTrace( );
            // inform caller method ..
            return -1;
        }
        
        // if the program gets here file could be parsed successfully: return 1
        return 1;
    }
    
    // Attributes of the file
    private int m_imageCols = -1;
    private int m_imageRows = -1;
    
    private RandomAccessFile    m_f;
    private int                 m_numberOfSlicesIntern = -1;
    private int                 m_numberOfRowsIntern = -1;
    private int                 m_numberOfColsIntern = -1;
    
    private String              m_dataType;
    
    private String              m_dataTypeDescriptor_1;
    private String              m_dataTypeDescriptor_2;
    private String              m_dataTypeDescriptor_3;
    
    private final Vector<Integer> m_qrows = new Vector<Integer>( );
    private final Vector<Integer> m_qcols = new Vector<Integer>( );
    private final Vector<Integer> m_index = new Vector<Integer>( );
    
    private final LinkedHashMap<String, RegioPolygon> m_namedRegioPolygons = new LinkedHashMap<String, RegioPolygon>( );
    
    private final Vector<Integer> m_polygonIndices = new Vector<Integer>( );
    
}
