/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package org.spectrim.imageprocessing;

import org.spectrim.dicom.DicomSliceCanvasPolyDraw;
import org.spectrim.imagegraphicobjects.RegioPolygon;
import org.spectrim.lib.ColumnSorter;
import org.spectrim.lib.EnterStringFrame;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.text.DecimalFormat;
import java.text.NumberFormat;
import java.util.Collections;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.Vector;
import javax.swing.JFrame;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.JTable;
import javax.swing.table.DefaultTableModel;

/**
 * Allows the viewing of statistical properties of regions.
 * @author slotboom
 */
public class ExtendedRegionPropertiesTable extends JTable implements ActionListener, MouseListener, KeyListener
{
    /**
     * Creates a new instance of RegionPropertiesTable.
     * @param parent
     * @param canvas
     * @param persistence
     */
    public ExtendedRegionPropertiesTable( JFrame parent, 
                                          DicomSliceCanvasPolyDraw canvas, 
                                          boolean persistence )
    {
        super( );

        m_parent = parent;
        m_persistence = persistence;

        m_dicomSliceCanvas = canvas;

        m_tableModel = new DefaultTableModel( );

        m_tableModel.addColumn( "<HTML><B>Region no.</B></HTML>" );
        m_tableModel.addColumn( "<HTML><B>Image no.</B></HTML>" );
        m_tableModel.addColumn( "<HTML><B>ROI name</B></HTML>" );
        m_tableModel.addColumn( "<HTML><B>Pixels</B></HTML>" );
        m_tableModel.addColumn( "<HTML><B>Volume</B></HTML>" );

        m_tableModel.addColumn( "<HTML><B>Mean</B></HTML>" );
        m_tableModel.addColumn( "<HTML><B>St. dev.</B></HTML>" );

        m_tableModel.addColumn( "<HTML><B>Variance</B></HTML>" );
        m_tableModel.addColumn( "<HTML><B>Var[var]</B></HTML>" );

        m_tableModel.addColumn( "<HTML><B>Skewness</B></HTML>" );
        m_tableModel.addColumn( "<HTML><B>Var[skew.]</B></HTML>" );

        m_tableModel.addColumn( "<HTML><B>Kurtosis</B></HTML>" );
        m_tableModel.addColumn( "<HTML><B>Var[kurt.]</B></HTML>" );

        m_tableModel.addColumn( "<HTML><B>Minimum</B></HTML>" );
        m_tableModel.addColumn( "<HTML><B>Maximum</B></HTML>" );
        
        setModel( m_tableModel );

        // define default behavior of the JTable
        setAutoCreateColumnsFromModel( false );
        setColumnSelectionAllowed (false );
        setRowSelectionAllowed( true );

        constructObject( );

        addKeyListener( this );

        updateROIList( );
    }


    /**
     * Does the rest of the object construction work.
     */
    private void constructObject( )
    {
        m_popup = new JPopupMenu( );

        m_deleteSelectedROIs = new JMenuItem( "Delete selected items" );
        m_deleteSelectedROIs.addActionListener( this );

        m_sortByROISize = new JMenuItem( "Sort by ROI size" );
        m_sortByROISize.addActionListener( this );

        m_sortByROIName = new JMenuItem( "Sort by ROI name" );
        m_sortByROIName.addActionListener( this );

        m_sortByAverageROIPixelValue = new JMenuItem( "Sort by average ROI pixel value" );
        m_sortByAverageROIPixelValue.addActionListener( this );

        m_sortByDeviationROIPixelValue = new JMenuItem( "Sort by deviation ROI pixel value" );
        m_sortByDeviationROIPixelValue.addActionListener( this );

        m_setPolygonsPersistentTag = new JMenuItem( "Set selected polygons persistent" );
        m_setPolygonsPersistentTag.addActionListener( this );

        m_unsetPolygonsPersistentTag = new JMenuItem( "Set selected polygons non-persistent" );
        m_unsetPolygonsPersistentTag.addActionListener( this );

        m_renameSelectedPolygons = new JMenuItem( "Rename selected polygons ..." );
        m_renameSelectedPolygons.addActionListener( this );

        m_popup.add( m_deleteSelectedROIs );

        m_popup.addSeparator( );

        m_popup.add( m_sortByROIName );
        m_popup.add( m_sortByROISize );
        m_popup.add( m_sortByAverageROIPixelValue );
        m_popup.add( m_sortByDeviationROIPixelValue );

        m_popup.addSeparator( );

        if ( m_persistence == true )
        {
            m_popup.add( m_setPolygonsPersistentTag );
            m_popup.add( m_unsetPolygonsPersistentTag );
        }

        m_popup.add( m_renameSelectedPolygons );

        add( m_popup );

        addMouseListener( this );
    }


    /**
     * Is called whenever a registered component evokes an ActionEvent.
     * @param e
     */
    @Override
    public void actionPerformed( ActionEvent e )
    {
        Object sender = e.getSource( );
        if ( sender == m_sortByROISize )
        {
            sortAllRowsBy( (DefaultTableModel)this.getModel( ), 3, true );
        }
        if ( sender == m_sortByROIName )
        {
            sortAllRowsBy( (DefaultTableModel)this.getModel( ), 2, true );
        }

        if ( sender == m_sortByAverageROIPixelValue )
        {
            sortAllRowsBy( (DefaultTableModel)this.getModel( ), 5, true );
        }

        if ( sender == m_sortByDeviationROIPixelValue )
        {
            sortAllRowsBy( (DefaultTableModel)this.getModel( ), 6, true );
        }

        if ( sender == m_setPolygonsPersistentTag )
        {
            setPersistentPolygons( );
        }

        if ( sender == m_unsetPolygonsPersistentTag )
        {
            unsetPersistentPolygons( );
        }

        if ( sender == m_renameSelectedPolygons )
        {
            renameSelectedPolygons( );
        }

        if ( sender == m_strEnterFrame )
        {
            if ( e.getID( ) == EnterStringFrame.APPLY_EVENT )
            {
                String str = m_strEnterFrame.getText( );
                applySelectedROIRenaming( str );
            }
        }

        if ( sender == m_deleteSelectedROIs )
        {
            deleteSelectedPolygons( );
        }

    }

    /**
     * Is called whenever a mouse button is pressed.
     * @param e
     */
    @Override
    public void mousePressed( MouseEvent e )
    {
        if ( e.isPopupTrigger( ) )
        {
            m_popup.show( e.getComponent( ), e.getX( ), e.getY( ) );
        }

        setSelectedROIItems( );
    }

    /**
     * Is called whenever a mouse button is clicked.
     * @param e
     */
    @Override
    public void mouseClicked( MouseEvent e )
    {
        // nothing to do yet.
    }

    /**
     * Is called whenever a mouse pointer enters
     * @param e
     */
    @Override
    public void mouseEntered( MouseEvent e )
    {
        // nothing to do yet.
    }

    /**
     * Is called whenever the mouse pointer exits
     * @param e
     */
    @Override
    public void mouseExited( MouseEvent e )
    {
        // nothing to do yet.
    }

    /**
     * Is called whenever a mouse button is released.
     * @param e
     */
    @Override
    public void mouseReleased( MouseEvent e )
    {
        if ( e.isPopupTrigger( ) )
        {
            m_popup.show( e.getComponent( ), e.getX( ), e.getY( ) );
        }
    }

    /** Regardless of sort order (ascending or descending), 
       null values always appear last; colIndex specifies a column in model.
     * @param model
     * @param colIndex
     * @param ascending
    */
    public void sortAllRowsBy( DefaultTableModel model, int colIndex, 
                               boolean ascending )
    {
        Vector data = model.getDataVector( );
        Collections.sort( data, new ColumnSorter( colIndex, ascending ) );
        model.fireTableStructureChanged( );
    }


    /**
     * This method updates the ROI list with current information.
     */
    public void updateROIList( )
    {
        int totalPixelNumber = 0;
        float totalSegmentVolume = 0f;
        // first clear the table
        int tableSize =  m_tableModel.getRowCount( );

        for ( int i = tableSize - 1 ; i >= 0; i-- )
        {
            m_tableModel.removeRow( i );
        }

        int cntr = 0;
        
        NumberFormat formatter = new DecimalFormat( "#####.#" );
        NumberFormat formatter2 = new DecimalFormat( "###.###" );
        // fill the table with new information
        for ( int i = 0; i < m_dicomSliceCanvas.getNumberOfImages( ); i++ )
        {
            int  numberOfPolygons = m_dicomSliceCanvas.getNumberOfPolygons( i );
            for ( int j = 0 ; j < numberOfPolygons; j++ )
            {
                RegioPolygon poly = m_dicomSliceCanvas.getPolygon( i, j );
                MomentsAboutTheMean moments = m_dicomSliceCanvas.getMomentsAboutTheMeanOfRegioPolygon( i, j );

                float volume = poly.getInnerVolume( ) *
                               new Float( m_dicomSliceCanvas.getDicomFileInfo( ).getSliceThickness( ) ).floatValue( ) *
                               new Float( m_dicomSliceCanvas.getDicomFileInfo( ).getPixelHeight( ) ).floatValue( ) *
                               new Float( m_dicomSliceCanvas.getDicomFileInfo( ).getPixelWidth( ) ).floatValue( );

                m_tableModel.addRow( new Object[]{ new Integer( cntr++ ),
                                                   new Integer( i ),
                                                   poly.getLabel( ),
                                                   new Integer( poly.getInnerVolume( ) ),
                                                   new Float( formatter.format( volume ) ),
                                                   new Float( formatter.format( moments.getMean( ) ) ),
                                                   new Float( formatter.format( moments.getStDev( ) ) ),
                                                   new Float( formatter.format( moments.getVariance( ) ) ),
                                                   new Float( formatter.format( moments.getVarianceOfVariance( ) ) ),
                                                   new Float( formatter2.format( moments.getSkewness( ) ) ),
                                                   new Float( formatter2.format( moments.getVarianceOfSkewness( ) ) ),
                                                   new Float( formatter2.format( moments.getKurtosis( ) ) ),
                                                   new Float( formatter2.format( moments.getVarianceOfKurtosis( ) ) ),
                                                   new Float( formatter2.format( moments.getMinimum( ) ) ),
                                                   new Float( formatter2.format( moments.getMaximum( ) ) )
                                                 }
                                    );

            }
        }
    }

    /**
     * Due to the fact that the segmentation algorithm may find and ROI
     * within an ROI, ie the case when a pixel collection is again larger
     * or smaller than the segmentation thresholds; In other words, it may
     * find inner or outer borders; In order to add pixel values correctly
     * together for eg averaging, or count volume segments together one needs
     * an algorithm that finds out whether the contour was an inner or an outer
     * border.
     */
    private int determineROISign( float value, int minThresshold, int maxThresshold )
    {
        if ( ( value <= maxThresshold ) && ( value >= minThresshold ) )
        {
            return 1;
        }
        else
        {
            return -1;
        }
    }

    /**
     * Sets the selected ROI items in the list, also selected (blue) in the
     * SliceCanvasPolyDraw.
     */
    private void setSelectedROIItems( )
    {
        int[] selectedRows = this.getSelectedRows( );
        String[] selectedROINames = new String[ selectedRows.length ];

        for ( int i = 0; i < selectedRows.length; i++ )
        {
            selectedROINames[i] = (String)getValueAt( selectedRows[i], 2 );
        }

        m_dicomSliceCanvas.selectPolygons( selectedROINames );
    }

    /**
     * Launches the rename frame.
     */
    private void renameSelectedPolygons( )
    {
        m_strEnterFrame = new EnterStringFrame( m_parent, "New regio name", "newName",
                                                          "Enter new name for selected ROI's" );
        m_strEnterFrame.addActionListener( this );
    }

    /**
     * Applies the renaming of the roi's.
     */
    private void applySelectedROIRenaming( String newName )
    {
        int[] selectedRows = this.getSelectedRows( );
        String[] selectedROINames = new String[ selectedRows.length ];

        String cntrString;

        for ( int i = 0; i < selectedRows.length; i++ )
        {
            selectedROINames[i] = (String)getValueAt( selectedRows[i], 2 );
        }

        for ( int i = 0; i < selectedRows.length; i++ )
        {
            if ( i < 10 )
            {
                cntrString = "0" + i;
            }
            else
            {
                cntrString = "" + i;
            }

            m_dicomSliceCanvas.getPolygon( selectedROINames[i] ).setLabel( newName + cntrString );
            m_tableModel.setValueAt( newName + cntrString, selectedRows[i], 2 );

        }

        m_dicomSliceCanvas.update( );
    }


    /**
     * Set the persistency tag of currently displayed regio polygons to
     * yes, ie they are persistent.
     */
    private void setPersistentPolygons( )
    {
        int[] selectedRows = this.getSelectedRows( );
        String[] selectedROINames = new String[ selectedRows.length ];

        for ( int i = 0; i < selectedRows.length; i++ )
        {
            selectedROINames[i] = (String)getValueAt( selectedRows[i], 2 );
        }

        for ( int i = 0; i < selectedRows.length; i++ )
        {
            m_persistentROIList.addLast( m_dicomSliceCanvas.getPolygon( selectedROINames[i] ) );
            m_tableModel.setValueAt( "yes", selectedRows[i], 7 );
        }
    }

    /**
     * Set the persistency tag of currently displayed regio polygons to
     * no, ie they are persistent.
     */
    private void unsetPersistentPolygons( )
    {
        int[] selectedRows = this.getSelectedRows( );
        String[] selectedROINames = new String[ selectedRows.length ];

        // list must be determined new
        m_persistentROIList.clear( );

        for ( int i = 0; i < selectedRows.length; i++ )
        {
            selectedROINames[i] = (String)getValueAt( selectedRows[i], 2 );
        }

        for ( int i = 0; i < selectedRows.length; i++ )
        {
            m_tableModel.setValueAt( "no", selectedRows[i], 7 );
        }

        // Build up the list opf remaining persistent regions
        int tableSize =  m_tableModel.getRowCount( );

        for ( int i = tableSize - 1 ; i >= 0; i-- )
        {
            if ( ( (String)m_tableModel.getValueAt( i, 7 ) ).compareTo( "yes" ) == 0 )
            {
                m_persistentROIList.addLast( m_dicomSliceCanvas.getPolygon( (String)m_tableModel.getValueAt( i, 1 ) ) );
            }
        }
    }

    /**
     * Deletes the currently selected polygons from the list.
     */
    public void deleteSelectedPolygons( )
    {
        int[] selectedRows = this.getSelectedRows( );

        String[] selectedROINames = new String[ selectedRows.length ];

        for ( int i = 0; i < selectedRows.length; i++ )
        {
            m_ROIsToDeleteNameList.addLast( (String)getValueAt( selectedRows[i], 2 ) );
        }

        for ( int i = selectedRows.length -1 ; i > -1 ; i-- )
        {
            String regionToDelete = (String) m_tableModel.getValueAt( i, 2 );
            // check deletion of only one item if in the list items are removed

            m_tableModel.removeRow( selectedRows[i] );
            notifyActionListeners( DELETE_EVENT,  regionToDelete );


        }

        m_tableModel.fireTableStructureChanged( );

    }

    /**
     * Notify all actionListeners.
     */
    private void notifyActionListeners( int eventType, String regionName )
    {
        Iterator it = m_listeners.iterator( );
        while( it.hasNext( ) )
        {
            ( ( ActionListener )it.next( ) ).actionPerformed( new ActionEvent( this, eventType, regionName ) );
        }
    }

    /**
     * Is called whenever a key is pressed
     * @param e
     */
    @Override
    public void keyPressed( KeyEvent e )
    {
        // nothing to do yet!
    }

    /**
     * Is called whenevef a key is released
     * @param e
     */
    @Override
    public void keyReleased( KeyEvent e )
    {

        if ( ( e.getKeyCode( ) == KeyEvent.VK_PAGE_UP ) || ( e.getKeyCode( ) == KeyEvent.VK_PAGE_DOWN ) ||
             ( e.getKeyCode( ) == KeyEvent.VK_DOWN ) || ( e.getKeyCode( ) == KeyEvent.VK_UP ) )
        {
            setSelectedROIItems( );
        }
    }

    /**
     * Is called whenever a key is typed
     * @param e
     */
    @Override
    public void keyTyped( KeyEvent e )
    {
        // nothing to do yet!
    }

    /**
     * Returns the linked list to the caller.
     * @return 
     */
    public LinkedList getPersistentPolygons( )
    {
        return m_persistentROIList;
    }

    /**
     * Returns the list of ROIs that needs to be delete from the list,
     * and from the DicomSliceCanvaPolyDraw.
     * @return 
     */
    public LinkedList getROIsToDeleteNameList( )
    {
        return m_ROIsToDeleteNameList;
    }

    /**
     * Adds an action listener to the list of action listeners. 
     * @param e
     */
    public void addActionListener( ActionListener e )
    {
	m_listeners.add( e );
    }

    /**
     * Sets the polygon information of all the polygons in the
     * DicomSliceCanvasPolydraw object.
     */
    public void setPolygonInformation( )
    {
        updateROIList( );
    }

    // Private attributes

    private JPopupMenu m_popup;
    private JMenuItem  m_deleteSelectedROIs;
    private JMenuItem  m_sortByROISize;
    private JMenuItem  m_sortByROIName;
    private JMenuItem  m_sortByAverageROIPixelValue;
    private JMenuItem  m_sortByDeviationROIPixelValue;
    private JMenuItem  m_setPolygonsPersistentTag;
    private JMenuItem  m_unsetPolygonsPersistentTag;
    private JMenuItem  m_renameSelectedPolygons;

    private final DefaultTableModel m_tableModel;

    private boolean m_persistence = true;

    private final DicomSliceCanvasPolyDraw m_dicomSliceCanvas;

    private final LinkedList<RegioPolygon> m_persistentROIList = new LinkedList<>( );

    private final JFrame m_parent;
    private EnterStringFrame m_strEnterFrame;

    public final  static int DELETE_EVENT = 0;
    private final LinkedList<String> m_ROIsToDeleteNameList = new LinkedList<>( );

    // Listeners to changes in the segmentation gui control
    private final LinkedList<ActionListener> m_listeners = new LinkedList<>( );

    // Must be declarated somehow
    private static final long serialVersionUID = 3487493391119393L;

}
