/*
 * File: $Id: Morphologic1DKernalOperator.java,v 1.1.1.1 2009/06/30 09:38:48 hans Exp $
 * History
 * Date: 30.06.2009  Authors: Hans Slotboom
 *
 * Changes:
 *
 * Copyright (C) 2009 by
 *
 * Inselspital - Department of Neuroradiology
 * CH 3010 Bern Switzerland
 *
 *
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package org.spectrim.imageprocessing;

/**
 *
 * @author slotboom
 */
public class Morphologic1DKernalOperator
{

    /**
     * Default constructor.
     */
    public Morphologic1DKernalOperator( )
    {
    }

    /**
     * Creates a new instance of MorphologicOperator with
     * default constructor.
     * @param type
     */
    public Morphologic1DKernalOperator( String type )
    {

        if ( type.compareTo( HORIZONTAL_OPERATOR ) == 0 )
        {
            m_type = HORIZONTAL_OPERATOR;
            m_kernelWidth  = 1;
            m_kernelHeight = 3;
        }
        else
        {
           m_type = VERTICAL_OPERATOR;
           m_kernelWidth  = 3;
           m_kernelHeight = 1;
        }

        m_kernelLength = 3;

        m_kernel = new int[ m_kernelLength ];

        m_kernel[0] = 1;
        m_kernel[1] = 1;
        m_kernel[2] = 1;

    }


    /**
     * Applies the erode function with kernel 'm_kernal' to the image 'image'
     * and returns the eroded image.
     * @param imageWidth
     * @param imageHeight
     * @param image
     * @return 
     */
    public int[] erode( int imageWidth, int imageHeight, int[] image )
    {
        int[] processedImage = new int[ image.length] ;
        if ( imageWidth * imageHeight == image.length )
        {
            for ( int i = 0; i < imageWidth; i++ )
            {
                for ( int j = 0; j < imageHeight; j++ )
                {
                    // assume that all pixel are 1 of mask and image
                    processedImage[ i * imageHeight + j ] = erodePixel( i, j, imageWidth, imageHeight, image );
                }
            }
        }
        else
        {
            System.out.println("Error: image array size does not fit with specified image sizes");
        }
        return processedImage;
    }

    /**
     * Contains the actual erosion functionality.
     */
    private int erodePixel( int i, int j, int imageWidth, int imageHeight, int[] image )
    {
        for ( int m = 0 ; m < m_kernelWidth - 1; m++ )
        {
            for ( int n = 0; n < m_kernelHeight - 1; n++ )
            {
                int ii = ( i + m - m_kernelWidth + imageWidth ) % imageWidth;
                int jj = ( j + n - m_kernelHeight + imageHeight ) % imageHeight;

                if ( m_kernel[ m * m_kernelHeight + n ] == 1 )
                {
                    // in this case the kernal != 0 so image mask must be everywhere !=0
                    // for non zero result
                    if ( image[ ii * imageWidth + jj ] == 0 )
                    {
                        return 0;
                    }
                }
            }
        }
        return image[ i * imageWidth + j ];
    }


    /**
     * Applies the erode function with kernel 'm_kernal' to the image 'image'
     * and returns the eroded image.
     * @param imageWidth
     * @param imageHeight
     * @param image
     * @return 
     */
    public boolean[][] erode( int imageWidth, int imageHeight, boolean[][] image )
    {
        boolean[][] processedImage = new boolean[imageWidth][imageHeight];


        for ( int i = 0; i < imageWidth; i++ )
        {
            for ( int j = 0; j < imageHeight; j++ )
            {
                // assume that all pixel are 1 of mask and image
                processedImage[i][j] = erodePixel( i, j, imageWidth, imageHeight, image );
            }
        }

        return processedImage;
    }

    /**
     * Contains the actual erosion functionality.
     */
    private boolean erodePixel( int i, int j, int imageWidth, int imageHeight, boolean[][] image )
    {
        for ( int m = 0 ; m < m_kernelWidth - 1; m++ )
        {
            for ( int n = 0; n < m_kernelHeight - 1; n++ )
            {
                int ii = ( i + m - m_kernelWidth + imageWidth ) % imageWidth;
                int jj = ( j + n - m_kernelHeight + imageHeight ) % imageHeight;

                if ( m_kernel[ m * m_kernelHeight + n ] == 1 )
                {
                    // in this case the kernal != 0 so image mask must be everywhere !=0
                    // for non zero result
                    if ( image[ii][jj] == false )
                    {
                        return false;
                    }
                }
            }
        }
        return image[i][j];
    }

    /**
     * Applies the dilate function with kernel 'm_kernal' to the image 'image'
     * and returns the dilated image.
     * @param imageWidth
     * @param imageHeight
     * @param image
     * @return 
     */
    public int[] dilate( int imageWidth, int imageHeight, int[] image )
    {
        int[] processedImage = new int[ image.length] ;
        if ( imageWidth * imageHeight == image.length )
        {
            for ( int i = 0; i < imageWidth; i++ )
            {
                for ( int j = 0; j < imageHeight; j++ )
                {
                    // assume that all pixel are 1 of mask and image
                    processedImage[ i * imageHeight + j ] = dilatePixel( i, j, imageWidth, imageHeight, image );
                }
            }

            if ( m_type.compareTo( HORIZONTAL_OPERATOR ) == 0 )
            {
                processedImage = shiftMatrix( processedImage, imageWidth, imageHeight, 1, 2 );
            }

            if ( m_type.compareTo( VERTICAL_OPERATOR ) == 0 )
            {
                processedImage = shiftMatrix( processedImage, imageWidth, imageHeight, 2, 1 );
            }

        }
        else
        {
            System.out.println( "Error: image array size does not fit with specified image sizes" );
        }
        return processedImage;
    }

    /**
     * Contains the actual dilate functionality.
     */
    private int dilatePixel( int i, int j, int imageWidth, int imageHeight, int[] image )
    {

        for ( int m = 0 ; m < m_kernelWidth; m++ )
        {
            for ( int n = 0; n < m_kernelHeight; n++ )
            {
                int ii = ( i + m - m_kernelWidth + imageWidth ) % imageWidth;
                int jj = ( j + n - m_kernelHeight + imageHeight ) % imageHeight;

                if ( m_kernel[ m * m_kernelHeight + n ] == 1 )
                {
                    // in this case the kernel != 0 so image mask must be everywhere !=0
                    // for non zero result
                    if ( image[ ii * imageWidth + jj ] != 0 )
                    {
                        return image[ ii * imageWidth + jj ];
                    }
                }
            }
        }
        return 0;
    }

    /**
     * Applies the dilate function with kernel 'm_kernal' to the image 'image'
     * and returns the dilated image.
     * @param imageWidth
     * @param imageHeight
     * @param image
     * @return 
     */
    public boolean[][] dilate( int imageWidth, int imageHeight, boolean[][] image )
    {

        boolean[][] processedImage = new boolean[imageWidth][imageHeight];

        for ( int i = 0; i < imageWidth; i++ )
        {
            for ( int j = 0; j < imageHeight; j++ )
            {
                // assume that all pixel are 1 of mask and image
                processedImage[i][j] = dilatePixel( i, j, imageWidth, imageHeight, image );
            }
        }

        if ( m_type.compareTo( HORIZONTAL_OPERATOR ) == 0 )
        {
            processedImage = shiftMatrix( processedImage, imageWidth, imageHeight, 1, 2 );
        }

        if ( m_type.compareTo( VERTICAL_OPERATOR ) == 0 )
        {
            processedImage = shiftMatrix( processedImage, imageWidth, imageHeight, 2, 1 );
        }

        return processedImage;
    }

    /**
     *
     * Shifts a given matrix over the given number of pixels
     *
     * @param inputImage
     * @param imageWidth
     * @param imageHeight
     * @param hor_shift
     * @param vert_shift
     * @return
     */
    private boolean[][] shiftMatrix( boolean[][] inputImage , int imageWidth, int imageHeight, int hor_shift, int vert_shift )
    {
        boolean[][] shiftedMatrix = new boolean[imageWidth][imageHeight];

        for ( int i = 0; i < imageWidth; i++ )
        {
            for ( int j = 0; j < imageHeight; j++ )
            {
                shiftedMatrix[ ( i - hor_shift + imageWidth ) % imageWidth ][ ( j - vert_shift + imageHeight ) % imageHeight ] = inputImage[i][j];
            }
        }
        return shiftedMatrix;
    }

    /**
     *
     * Shifts a given int[] matrix over the given number of pixels
     *
     * @param inputImage
     * @param imageWidth
     * @param imageHeight
     * @param hor_shift
     * @param vert_shift
     * @return
     */
    private int[] shiftMatrix( int[] inputImage , int imageWidth, int imageHeight, int hor_shift, int vert_shift )
    {
        int[] shiftedMatrix = new int[ imageWidth * imageHeight ];
        int shifted;

        for ( int i = 0; i < imageWidth; i++ )
        {
            for ( int j = 0; j < imageHeight; j++ )
            {
                shifted = ( ( ( i - hor_shift + imageWidth ) % imageWidth ) * imageHeight ) + ( ( j - vert_shift + imageHeight ) % imageHeight );
                shiftedMatrix[ shifted ] = inputImage[ i * imageHeight + j ];
            }
        }
        return shiftedMatrix;
    }


    /**
     * Contains the actual dilate functionality.
     */
    private boolean dilatePixel( int i, int j, int imageWidth,
                                 int imageHeight, boolean[][] image )
    {

        for ( int m = 0 ; m < m_kernelWidth; m++ )
        {
            for ( int n = 0; n < m_kernelHeight; n++ )
            {
                int ii = ( i + m - m_kernelWidth + imageWidth ) % imageWidth;
                int jj = ( j + n - m_kernelHeight + imageHeight ) % imageHeight;

                if ( m_kernel[ m * m_kernelHeight + n ] == 1 )
                {
                    // in this case the kernel != 0 so image mask must be everywhere !=0
                    // for non zero result
                    if ( image[ii][jj] != false )
                    {

                        return image[ii][jj];
                    }
                }
            }
        }

        return false;
    }

    /**
     * Performs the exclusive or operation on two int[] images
     * @param image1
     * @param image2
     * @return EXOR of both input images
     */
    public int[] performExorOperation( int[] image1, int[] image2 )
    {

        int[] outImage = new int[image1.length];

        for ( int i = 0; i < image1.length; i++ )
        {
            if ( ( image1[i] == 0 ) && ( image2[i] != 0 ) )
            {
                outImage[i] = 1;
            }
            if ( ( image1[i] != 0 ) && ( image2[i] == 0 ) )
            {
                outImage[i] = 1;
            }
        }

        return outImage;
    }

    /**
     * Performs the exclusive or operation on two int[] images
     * @param image1
     * @param image2
     * @param imageWidth
     * @param imageHeight
     * @return EXOR of both input images
     */
    public boolean[][] performExorOperation( boolean[][] image1, boolean[][] image2, int imageWidth, int imageHeight )
    {

        boolean[][] outImage = new boolean[imageWidth][imageHeight];

        for ( int i = 0; i < imageWidth; i++ )
        {
            for ( int j = 0; j < imageHeight; j ++ )
            {
                // assume false
                outImage[i][j] = false;

                if ( ( image1[i][j] == true ) && ( image2[i][j] == false ) )
                {
                    outImage[i][j] = true;
                }
                if ( ( image1[i][j] == false ) && ( image2[i][j] == true ) )
                {
                    outImage[i][j] = true;
                }
            }
        }

        return outImage;
    }


    /**
     *
     * Test the functionality of the erode and dilate function
     *
     * @param args
     */
    public static void main( String[] args )
    {

        Morphologic1DKernalOperator morphVert = new Morphologic1DKernalOperator( Morphologic1DKernalOperator.VERTICAL_OPERATOR );
        Morphologic1DKernalOperator morphHori = new Morphologic1DKernalOperator( Morphologic1DKernalOperator.HORIZONTAL_OPERATOR );

        //test code int
        int[] image = new int[25];
        int[] imageOut = new int[25];
        int imageWidth = 5;
        int imageHeight = 5;

        image[0]  = 0; image[1]  = 0; image[2]  = 0; image[3]  = 0; image[4]  = 0;
        image[5]  = 0; image[6]  = 0; image[7]  = 0; image[8]  = 0; image[9]  = 0;
        image[10] = 0; image[11] = 0; image[12] = 1; image[13] = 0; image[14] = 0;
        image[15] = 0; image[16] = 0; image[17] = 0; image[18] = 0; image[19] = 0;
        image[20] = 0; image[21] = 0; image[22] = 0; image[23] = 0; image[24] = 0;

        System.out.println( "Input Image" );

        for ( int i = 0; i < imageHeight; i++ )
        {
            for( int j = 0; j < imageWidth; j++ )
            {
                System.out.print( image[ i * imageWidth + j ] );
            }

            System.out.println( );
        }


        imageOut = morphVert.dilate( 5, 5, image );
        imageOut = morphHori.dilate( 5, 5, imageOut );


        imageOut = morphHori.performExorOperation( image, imageOut );

        System.out.println( "Output Image" );

        for ( int i = 0; i < imageHeight; i++ )
        {
            for( int j = 0; j < imageWidth; j++ )
            {
                System.out.print( imageOut[i*imageWidth + j] );
            }
            System.out.println( );
        }

        //test code boolean
        boolean[][] imageB = new boolean[5][5];
        boolean[][] imageOutB = new boolean[5][5];

        imageB[0][0]=false;  imageB[0][1]=false; imageB[0][2]=false; imageB[0][3]=false; imageB[0][4]=false;
        imageB[1][0]=false;  imageB[1][1]=false; imageB[1][2]=false;  imageB[1][3]=false; imageB[1][4]=false;
        imageB[2][0]=false;  imageB[2][1]=false; imageB[2][2]=true;  imageB[2][3]=false; imageB[2][4]=false;
        imageB[3][0]=false;  imageB[3][1]=false; imageB[3][2]=false;  imageB[3][3]=false; imageB[3][4]=false;
        imageB[4][0]=false;  imageB[4][1]=false; imageB[4][2]=false; imageB[4][3]=false; imageB[4][4]=false;

        for ( int i = 0; i < imageHeight; i++ )
        {
            for( int j = 0; j < imageWidth; j++ )
            {
                if ( imageB[i][j] == true )
                {
                    System.out.print( "true  " );
                }
                else
                {
                    System.out.print( "false " );
                }
            }
            System.out.println( );
        }

        imageOutB = morphVert.dilate( 5, 5, imageB );
        imageOutB = morphHori.dilate( 5, 5, imageOutB );

        imageOutB = morphHori.performExorOperation( imageOutB, imageB, imageWidth, imageHeight );

        System.out.println( "Output Image" );

        for ( int i = 0; i < imageHeight; i++ )
        {
            for( int j = 0; j < imageWidth; j++ )
            {
                if ( imageOutB[i][j] == true )
                {
                    System.out.print( "true  " );
                }
                else
                {
                    System.out.print( "false " );
                }
            }
            System.out.println( );
        }
    }


    // Attributes
    int m_kernelWidth  = 0;
    int m_kernelHeight = 0;
    int m_kernelRadius = 0;
    int m_kernelLength = 0;

    int[] m_kernel;

    private String m_type = HORIZONTAL_OPERATOR;

    public final static String HORIZONTAL_OPERATOR = "HORIZONTAL_OPERATOR";
    public final static String VERTICAL_OPERATOR = "VERTICAL_OPERATOR";
}
