/*
 * File: $Id: RegionTTester.java,v 1.1.1.1 2006/04/19 09:38:48 hans Exp $
 * History
 * Date: 03. September 2003, 16:14 Hans Slotboom  Changes:
 *
 * Copyright (C) 2004 by
 * 
 * Inselspital - Department of Neuroradiology
 * CH 3010 Bern Switzerland
 */

/*
 * RegionTTester.java
 *
 * Created on September 3, 2004, 10:00 AM
 */

package org.spectrim.imageprocessing;

import org.spectrim.imagegraphicobjects.RegioPolygon;
import org.spectrim.signalprocessing.SignalTools;
import java.awt.Rectangle;

/**
 *
 * @author  hans
 */
public class RegionTTester 
{
    
    /*
     *  main(..) method; Entry point to test the application
     */
    public static void main( String args[] ) 
    {
	System.out.println( "Total memory available to JVM = " + Runtime.getRuntime( ).totalMemory( ) );
     
        float[] x = new float[11];
        float[] y = new float[10];
        
        x[0] = 1.0f; y[0] = 1.0f; 
        x[1] = 2.0f; y[1] = 2.0f;
        x[2] = 2.0f; y[2] = 2.0f;
        x[3] = 4.0f; y[3] = 4.0f;
        x[4] = 3.0f; y[4] = 3.0f;
        x[5] = 2.0f; y[5] = 2.0f;
        x[6] = 4.0f; y[6] = 4.0f;
        x[7] = 4.0f; y[7] = 2.0f;
        x[8] = 4.0f; y[8] = 4.0f;
        x[9] = 1.0f; y[9] = 1.0f;
        x[10] = 1.0f;
        RegionTTester tst = new RegionTTester( ); 
        tst.calculateTTest( x, y );

    }
    /**
     * Default constructor
     */
    public RegionTTester( )
    {
        
    }
    
    /** 
     * Creates a new instance of RegionTTester
     * This constructor is used to calculate simply one time a
     * tTest between two regions which are located on an image "image". 
     * @param regPol_1
     * @param regPol_2
     * @param image
     * @param imageWidth
     * @param imageHeight
     * @param zoom
     * @param nordicScalingFactor
     */
    public RegionTTester( RegioPolygon regPol_1, RegioPolygon regPol_2, 
                          int[] image, 
                          int imageWidth, int imageHeight, 
                          float zoom, float nordicScalingFactor )
    {
        // scale back to original size!
        m_regPol_1 = regPol_1.scaled( 1f / zoom );
        m_regPol_2 = regPol_2.scaled( 1f / zoom );
        
        m_nordicScalingFactor = nordicScalingFactor;
        
        m_image = new int[image.length];
        System.arraycopy( image, 0, m_image, 0, image.length );
        
        m_imageWidth = imageWidth;
        m_imageHeight = imageHeight;
        m_zoom = zoom;
        
        // get the pixel values as simple arrays
        m_roi_1_as_array = getRegionPolygonAsFloatArray( m_regPol_1, nordicScalingFactor );
        m_roi_2_as_array = getRegionPolygonAsFloatArray( m_regPol_2, nordicScalingFactor );
        
        calculateTTest( m_roi_1_as_array, m_roi_2_as_array );
    }
    
    /** 
     * Creates a new instance of RegionTTester
     * This constructor is used to calculate probability maps (tTest-maps)  
     * @param tissueTypeROI
     * @param thresholdMask
     * @param image
     * @param imageWidth
     * @param imageHeight
     * @param zoom
     * @param probeSizeX
     * @param probeSizeY
     * @param permutations
     * @param numberOfProbingPixels
     * @param nordicScalingFactor
     */
    public RegionTTester( RegioPolygon tissueTypeROI, boolean[][][] thresholdMask , 
                          int[] image, 
                          int imageWidth, int imageHeight, float zoom, 
                          int probeSizeX, int probeSizeY,
                          int permutations, int numberOfProbingPixels, 
                          float nordicScalingFactor )
    {
        // scale polygon back to its original size!
        m_tissueTypeROI = tissueTypeROI.scaled( 1f / zoom );
                
        m_image = new int[image.length];
        System.arraycopy( image, 0, m_image, 0, image.length );
        
        // threshold mask limits the computation time, if set properly 
        m_thresholdMask = thresholdMask;
        
        m_imageWidth = imageWidth;
        m_imageHeight = imageHeight;
        m_zoom = zoom;
        
        m_probeSizeX = probeSizeX;
        m_probeSizeY = probeSizeY;
        
        m_permutations = permutations;
        m_numberOfProbingPixels = numberOfProbingPixels;
        
        // get the pixel values as simple arrays
        m_tissueTypeROI_as_array = getRegionPolygonAsFloatArray( m_tissueTypeROI, nordicScalingFactor );
        
        // Now initialize the random variable for this tissue typw
        m_tissueType_rv = new randomVar( );
        m_tissueType_rv.mean = SignalTools.mean( m_tissueTypeROI_as_array );
        m_tissueType_rv.sumSquares = SignalTools.sum2( m_tissueTypeROI_as_array );
        m_tissueType_rv.n = m_tissueTypeROI_as_array.length;
        m_tissueType_rv.sampleVariance = ( m_tissueType_rv.sumSquares / m_tissueType_rv.n - m_tissueType_rv.mean * m_tissueType_rv.mean ) * 
                               ( m_tissueType_rv.n / ( m_tissueType_rv.n - 1 ) );
        m_tissueType_rv.sampleVarianceNormalized = m_tissueType_rv.sampleVariance/m_tissueType_rv.n;
        m_tissueType_rv.sampleStandardDeviation = Math.sqrt( m_tissueType_rv.sampleVariance );
        
    }
    
    /**
     * Performs the tissue specific probability map (p-map) computation
     * 
     * The >type< indicates the way the probability map should be computed
     * 
     * @param imageNumber
     * @param computationType
     * @return 
     */
    public float[][] computeTissueSpecificProbabilityMaps( int imageNumber, int computationType )
    {
        m_pMap = new float[m_imageWidth][m_imageHeight];
        m_tMap = new float[m_imageWidth][m_imageHeight];
        
        // First, set up the random variable of the current tissue type
        m_tissueType_rv = new randomVar( );
        m_tissueType_rv.mean = SignalTools.mean( m_tissueTypeROI_as_array );
        m_tissueType_rv.sumSquares = SignalTools.sum2( m_tissueTypeROI_as_array );
        m_tissueType_rv.n = m_tissueTypeROI_as_array.length;
        m_tissueType_rv.sampleVariance = ( m_tissueType_rv.sumSquares / m_tissueType_rv.n - m_tissueType_rv.mean * m_tissueType_rv.mean ) * 
                               ( m_tissueType_rv.n / ( m_tissueType_rv.n - 1 ) );
        m_tissueType_rv.sampleVarianceNormalized = m_tissueType_rv.sampleVariance/m_tissueType_rv.n;
        m_tissueType_rv.sampleStandardDeviation = Math.sqrt( m_tissueType_rv.sampleVariance );
        
        m_computationType = computationType;
        
        int arrayIndex;
        
        // Do the computation for each pixel
        for ( int w = 0; w < m_imageWidth; w++ )
        {
            for ( int h = 0; h < m_imageHeight; h++ )
            {
                
                if ( m_thresholdMask[imageNumber][w][h] == true )
                {
                    arrayIndex = m_imageWidth * h + w;
                    m_probeROI_as_array = computeROIForProbing( w, h, computationType );
                    m_pMap[w][h] = calculateTTest( m_probeROI_as_array );
                    m_tMap[w][h] = m_lastComputedTvalue;
                    if ( m_pMap[w][h] != 0.0 )
                    {
                        System.out.println( "p[i][j] = " + m_pMap[w][h] );
                    }
                }
            }
        }
        
        return m_pMap;
    }
    
    /**
     * Returns the T-map containing the T-values of the last computed complete
     * T-tests.
     * @return 
     */
    public float[][] getTMap( )
    {
        return m_tMap;
    }
    
    /**
     * Compute ROI pixel values as array for statistical probing.
     * @param w
     * @param h
     * @param computationType
     * @return 
     */
    public float[] computeROIForProbing( int w, int h, int computationType )
    {
        
        if ( computationType == FULL_PROBING_ROIS )
        {
            m_probeROI_as_array = new float[ (2 * m_probeSizeX + 1) * (2 * m_probeSizeY + 1 )  ];
            
            int cntr = 0;
            int imageArrayIndex = 0;
            
            for ( int i = -m_probeSizeX ; i <= m_probeSizeX; i++ )
            {
                for ( int j = -m_probeSizeY; j <= m_probeSizeY; j++ )
                {
                    imageArrayIndex = m_imageWidth * ( h + j ) + ( w + i );
                    m_probeROI_as_array[cntr] = m_image[ imageArrayIndex ];
                    cntr++;
                }
            }
        }
        
        return m_probeROI_as_array;
    }
    
    /**
     * Returns an array containing all points within the RegioPolygon.
     */
    private float[] getRegionPolygonAsFloatArray( RegioPolygon regPol, 
                                                  float nordicScalingFactor )
    {
        int arrayLength = regPol.getInnerVolume( );
        float[] array = new float[arrayLength];
        float[] returnArray = null;
                
        if ( regPol != null )
        {
            Rectangle bb = regPol.getBounds( );
            if ( bb.x < 0 )
            {    
                bb.x = 0;
            }
            if ( bb.x + bb.width >= m_imageWidth ) 
            {
                bb.width = m_imageWidth - bb.x - 1;
            }
            if ( bb.y < 0 )
            {
                bb.y = 0;
            }
        
            if ( bb.y + bb.height >= m_imageHeight ) 
            {
                bb.height = m_imageHeight - bb.y - 1;
            }
            
            int cntr = 0;
            
            // Calculate the average of the ROI
            for ( int y = bb.y; y <= bb.y + bb.height; y++ ) 
            {
                int yoffset = y * m_imageWidth;
                for ( int x = bb.x; x <= bb.x + bb.width; x++ ) 
                {
                    if ( regPol.contains( x, y ) ) 
                    {
                        array[cntr] = (float)m_image[ x + m_imageWidth * y ] * nordicScalingFactor;
                        cntr++;
                    }
                }
            }
            
            returnArray = new float[cntr];
            
            for ( int i = 0; i < cntr; i++ )
            {
                returnArray[i] = array[i] * m_nordicScaleFactor;
            }
        }
                
        return returnArray;
    }
    
    /**
     * Routine that calculates the one sided TTest for a given array.
     * The second array must be set prior to the function call.
     * The method returns the p-value (one sided) of the test
     */
    public float calculateTTest( float[] array2 )
    {
        m_imageProbe_rv = new randomVar( );
        m_imageProbe_rv.mean = SignalTools.mean( array2 );
        m_imageProbe_rv.sumSquares = SignalTools.sum2( array2 );
        m_imageProbe_rv.n = array2.length;
        m_imageProbe_rv.sampleVariance = ( m_imageProbe_rv.sumSquares / m_imageProbe_rv.n - m_imageProbe_rv.mean * m_imageProbe_rv.mean ) * 
                               ( m_imageProbe_rv.n / ( m_imageProbe_rv.n - 1 ) );
        m_imageProbe_rv.sampleVarianceNormalized = m_imageProbe_rv.sampleVariance / m_imageProbe_rv.n;
        m_imageProbe_rv.sampleStandardDeviation = Math.sqrt( m_imageProbe_rv.sampleVariance );
        
//      double sv1 = 0;	// 1st dataset sample variance
//      double sv2 = 0;	// 2nd dataset sample variance
    
//        sample--not pop. variance is calculated below
//        sv1 = ( rv1.sumSquares / rv1.n - rv1.mean * rv1.mean ) * 
//              ( rv1.n / ( rv1.n - 1 ) );
//        sv2 = ( rv2.sumSquares / rv2.n - rv2.mean * rv2.mean ) * 
//              ( rv2.n / ( rv2.n - 1 ) );
						
        /*	
            Now it remains to be determined if the population variances can
            be pooled or not. To see this, an F-test can be applied to them.
            This decision is determined automatically, with an alpha level of
            1%, in order to not encumber the user with details.
        */
		
        double f = 0;
        double fAlpha = 0;
        SpecialFunctions sf = new SpecialFunctions( );
        
        if ( m_tissueType_rv.sampleVariance  > m_imageProbe_rv.sampleVariance ) 
        {
            f = m_tissueType_rv.sampleVariance / m_imageProbe_rv.sampleVariance;
            fAlpha = sf.fDist( m_tissueType_rv.n - 1, m_imageProbe_rv.n - 1, f );
        }
        else 
        {
            f = m_imageProbe_rv.sampleVariance / m_tissueType_rv.sampleVariance;
            fAlpha = sf.fDist( m_imageProbe_rv.n - 1, m_tissueType_rv.n - 1, f );
        }
    
        double df = 0;		// t Test degrees of freedom
        double t = 0;		// t value
        
//        String comment = new String( );
//        String comment1 = new String( );

        if ( fAlpha <= 0.005 ) 
        {
//            comment = "An F test on the sample variances indicates that they are " + newln +
//                      "probably not from the same population (the variances " + newln +
//                      "can't be pooled), at an alpha level of " + fAlpha + "." + newln +
//                      "Thus, the t-test was set up for samples with unequal varainces. "+ newln +
//                      "(The degrees of freedom were adjusted.)";
      
//            double svn1 = m_tissueType_rv.sampleVariance / m_tissueType_rv.n;
//            double svn2 = rv2.sampleVariance / rv2.n;
                     
            df = Math.pow( m_tissueType_rv.sampleVarianceNormalized + m_imageProbe_rv.sampleVarianceNormalized, 2 ) /
                ( Math.pow( m_tissueType_rv.sampleVarianceNormalized, 2 ) / ( m_tissueType_rv.n + 1 ) + Math.pow( m_imageProbe_rv.sampleVarianceNormalized, 2 ) / ( m_imageProbe_rv.n + 1 ) ) - 2;
            t = Math.abs( m_tissueType_rv.mean - m_imageProbe_rv.mean ) / Math.sqrt( ( m_tissueType_rv.sampleVariance / m_tissueType_rv.n ) + ( m_imageProbe_rv.sampleVariance / m_imageProbe_rv.n ) );
        }
        else
        {
            
//            comment = "An F test on the sample variances indicates that they could be " + newln +
//                      "from the same population, (alpha level of 0.005)." + newln +
//                      "Accordingly, the t-test was set up for samples with equal population variance.";
            
             df = m_tissueType_rv.n + m_imageProbe_rv.n - 2;
             double sp = Math.sqrt( ( ( m_tissueType_rv.n - 1 ) * m_tissueType_rv.sampleVariance + ( m_imageProbe_rv.n - 1 ) * m_imageProbe_rv.sampleVariance ) /
                                    ( m_tissueType_rv.n + m_imageProbe_rv.n - 2 ) );
             t = Math.abs( m_tissueType_rv.mean - m_imageProbe_rv.mean ) * Math.sqrt( m_tissueType_rv.n * m_imageProbe_rv.n / ( m_tissueType_rv.n + m_imageProbe_rv.n ) ) / sp;
        }
    
        double pVal = sf.stDist( df, t );
        
        m_lastComputedTvalue = (float)t;
        
        String pValComment = "" + pVal;
//    
//        if ( pVal <= 0.01 ) 
//        {
//            comment1 = "This probability indicates that there is a difference in sample means.";
//            if ( pVal <= 0.0001 ) 
//            {
//                pValComment = "< 0.0001";
//            }
//        }
//        else if ( pVal <= 0.05 ) 
//        {
//            comment1 = "This probability indicates that there may be a difference in sample means.";
//        }
//        else 
//        {
//            comment1 = "There is not a significant difference in the sample means. " + newln +
//                       "A difference could not be detected due to large variability, small sample size, " + newln +
//                        "or both. Of course, it's possible that the samples really are from the same population!";
//        }
    
        // a hack to take care of garbage data
        if ( ( m_tissueType_rv.n == 0 ) || ( m_imageProbe_rv.n == 0 ) ) 
        {
            System.out.println( "RegionTTester :: There is a problem with the input roi data." );
        }
        
        // build the report
//        char tab = (char) 9;
//        m_reportString = "Student's t-Test for Comparing Things " + newln +
//                         "Java Implementation by Bryan Lewis, Kent State University, 1997 " + newln +
//                         "This test determines if there is a significant "+ newln + "difference between "
//                         + "the averages of the two samples above. " + newln +
//                         newln +
//                         "Mean of first data set                   : " + tab + m_tissueType_rv.mean +   newln +
//                         "Standard deviation of first data set     : " + tab + m_tissueType_rv.sampleStandardDeviation +  newln +
//                         "Number of observations in the first set  : " + tab + m_tissueType_rv.n +  newln +
//                         "Mean of second data set                  : " + tab + rv2.mean +  newln +
//                         "Standard deviation of second data set    : " + tab + rv2.sampleStandardDeviation +  newln +
//                         "Number of observations in the second set : " + tab + rv2.n +  newln +
//                         newln + 
//                         "Degrees of freedom   : " + tab + df +  newln +
//                         "t Value (one-tailed) : " + tab + t +  newln +				
//                         "P(x>t)               : " + tab + pValComment +  newln +
//                         newln + comment1 +  newln + newln  + comment;
//    
//        System.out.println( m_reportString );
        
        return (float)pVal;
    }
    
    /**
     * Routine that calcultes the one sided TTest
     */
    public String calculateTTest( float[] array1, float[] array2 )
    {
        // First, set up the random variables
        randomVar rv1 = new randomVar( );
        rv1.mean = SignalTools.mean( array1 );
        rv1.sumSquares = SignalTools.sum2( array1 );
        rv1.n = array1.length;
    
        randomVar rv2 = new randomVar( );
        rv2.mean = SignalTools.mean( array2 );
        rv2.sumSquares = SignalTools.sum2( array2 );
        rv2.n = array2.length;
    
        double sv1 = 0;	// 1st dataset sample variance
        double sv2 = 0;	// 2nd dataset sample variance
    
        // sample--not pop. variance is calculated below
        sv1 = ( rv1.sumSquares / rv1.n - rv1.mean * rv1.mean ) * 
              ( rv1.n / ( rv1.n - 1 ) );
        sv2 = ( rv2.sumSquares / rv2.n - rv2.mean * rv2.mean ) * 
              ( rv2.n / ( rv2.n - 1 ) );
						
        /*	
            Now it remains to be determined if the population variances can
            be pooled or not. To see this, an F-test can be applied to them.
            This decision is determined automatically, with an alpha level of
            1%, in order to not encumber the user with details.
        */
		
        double f = 0;
        double fAlpha = 0;
        SpecialFunctions sf = new SpecialFunctions( );
        
        if ( sv1 > sv2 ) 
        {
            f = sv1 / sv2;
            fAlpha = sf.fDist( rv1.n - 1, rv2.n - 1, f );
        }
        else 
        {
            f = sv2 / sv1;
            fAlpha = sf.fDist( rv2.n - 1, rv1.n - 1, f );
        }
    
        double df = 0;		// t Test degrees of freedom
        double t = 0;		// t value
        String comment = new String( );
        String comment1 = new String( );
    
        if ( fAlpha <= 0.005 ) 
        {
            comment = "An F test on the sample variances indicates that they are " + newln +
                      "probably not from the same population (the variances " + newln +
                      "can't be pooled), at an alpha level of " + fAlpha + "." + newln +
                      "Thus, the t-test was set up for samples with unequal varainces. "+ newln +
                      "(The degrees of freedom were adjusted.)";
      
            double svn1 = sv1 / rv1.n;
            double svn2 = sv2 / rv2.n;
      
            df = Math.pow( svn1 + svn2, 2 ) /
                ( Math.pow( svn1, 2 ) / ( rv1.n + 1 ) + Math.pow( svn2, 2 ) / ( rv2.n + 1 ) )  - 2;
            t = Math.abs( rv1.mean - rv2.mean ) / Math.sqrt( sv1 / rv1.n + sv2 / rv2.n );
        }
        else
        {
            comment = "An F test on the sample variances indicates that they could be " + newln +
                      "from the same population, (alpha level of 0.005)." + newln +
                      "Accordingly, the t-test was set up for samples with equal population variance.";
             df = rv1.n + rv2.n - 2;
             double sp = Math.sqrt( ( ( rv1.n - 1 ) * sv1 + ( rv2.n - 1 ) * sv2 ) /
                                    ( rv1.n + rv2.n - 2 ) );
             t = Math.abs( rv1.mean - rv2.mean ) * Math.sqrt( rv1.n * rv2.n / ( rv1.n + rv2.n ) ) / sp;
        }
    
        double pVal = sf.stDist( df, t );
        String pValComment = "" + pVal;
        
        m_lastComputedTvalue = (float)t;
        
        if ( pVal <= 0.01 ) 
        {
            comment1 = "This probability indicates that there is a difference in sample means.";
            if ( pVal <= 0.0001 ) 
            {
                pValComment = "< 0.0001";
            }
        }
        else if ( pVal <= 0.05 ) 
        {
            comment1 = "This probability indicates that there may be a difference in sample means.";
        }
        else 
        {
            comment1 = "There is not a significant difference in the sample means. " + newln +
                       "A difference could not be detected due to large variability, small sample size, " + newln +
                        "or both. Of course, it's possible that the samples really are from the same population!";
        }
    
        // a hack to take care of garbage data
        if ( ( rv1.n == 0 ) || ( rv2.n == 0 ) ) 
        {
            comment1 = "There is a problem with the roi data. ";
        }
    
    
        // convert variance to std. deviation for report
        sv1 = Math.sqrt( sv1 );
        sv2 = Math.sqrt( sv2 );
    
    
        // build the report
        char tab = (char) 9;
        m_reportString = "Student's t-Test for Comparing Things " + newln +
                         "Java Implementation by Bryan Lewis, Kent State University, 1997 " + newln +
                         "This test determines if there is a significant "+ newln + "difference between "
                         + "the averages of the two samples above. " + newln +
                         newln +
                         "Mean of first data set                   : " + tab + rv1.mean +   newln +
                         "Standard deviation of first data set     : " + tab + sv1 +  newln +
                         "Number of observations in the first set  : " + tab + rv1.n +  newln +
                         "Mean of second data set                  : " + tab + rv2.mean +  newln +
                         "Standard deviation of second data set    : " + tab + sv2 +  newln +
                         "Number of observations in the second set : " + tab + rv2.n +  newln +
                         newln + 
                         "Degrees of freedom   : " + tab + df +  newln +
                         "t Value (one-tailed) : " + tab + t +  newln +				
                         "P(x>t)               : " + tab + pValComment +  newln +
                         newln + comment1 +  newln + newln  + comment;
    
        System.out.println( m_reportString );
        return m_reportString;
    }
    
    /**
     * Returns the mean of the region which defines the tissue contour
     */
    public float getMeanOfRegion( )
    {
        float mean = -Float.MIN_VALUE;
        
        // check whether this ROI has been defined ..
        if ( m_tissueTypeROI != null )
        {
            float[] tissueArray =  getRegionPolygonAsFloatArray( m_tissueTypeROI, m_nordicScalingFactor );
            mean = SignalTools.mean( tissueArray );
        }
        
        return mean;
    }
    
    /**
     * Returns the center value of the region which defines the tissue contour
     */
    public float getCenterValueOfRegion( )
    {
        float minimum = 0f;
        float maximum = 0f;
        
        // check whether this ROI has been defined ..
        if ( m_tissueTypeROI != null )
        {
            float[] tissueArray =  getRegionPolygonAsFloatArray( m_tissueTypeROI, m_nordicScalingFactor );
            
            minimum = SignalTools.getMinimum( tissueArray );
            maximum = SignalTools.getMaximum( tissueArray );
        }
        
        return ( minimum + maximum ) / 2;
    }
    
    /**
     * Returns the pixel range value of the region which defines 
     * the tissue contour
     */
    public float getMinimumMaximumRangeOfRegion( )
    {
        float minimum = 0f;
        float maximum = 0f;
        
        if ( m_tissueTypeROI != null )
        {
            float[] tissueArray =  getRegionPolygonAsFloatArray( m_tissueTypeROI, m_nordicScalingFactor );
            minimum = SignalTools.getMinimum( tissueArray );
            maximum = SignalTools.getMaximum( tissueArray );
        }
        
        return ( maximum - minimum );
        
    }
    
    
    
    /**
     * Returns the mean of the region which defines the tissue contour
     */
    public float getSampleVarianceOfRegion( )
    {
        
        float mean = -Float.MIN_VALUE;
        float stdev = -Float.MIN_VALUE;
        float sumSquares = 0f;
        int   n = 0;
        float sampleVariance = 0f;
        
        // check whether this ROI has been defined ..
        if ( m_tissueTypeROI != null )
        {
            float[] tissueArray =  getRegionPolygonAsFloatArray( m_tissueTypeROI, m_nordicScalingFactor );
            
            mean = SignalTools.mean( tissueArray );
            sumSquares = SignalTools.sum2( tissueArray );
            n = tissueArray.length;
            
            sampleVariance = ( sumSquares / n - mean * mean ) * ( n / ( n - 1 ) );
        }
        
        return sampleVariance;
    }
    
    /**
     * Returns the last computed T-value
     */
    public float getLastComputedTvalue( )
    {
        return m_lastComputedTvalue;
    }
    
    /**
     * Returns the report of the t-Test
     */
    public String getReportString( )
    {
        return m_reportString;
    }
    
    // variables
    
    private String newln = System.getProperty( "line.separator" );  
    private String m_reportString = "";
    
    // needed for tissue type characterization
    private RegioPolygon m_tissueTypeROI;
    
    private float[] m_tissueTypeROI_as_array;
    private float[] m_probeROI_as_array;
    
    private randomVar m_tissueType_rv = new randomVar( );
    private randomVar m_imageProbe_rv = new randomVar( );
    
    private boolean[][][] m_thresholdMask;
    
    private float[] m_roi_1_as_array;
    private float[] m_roi_2_as_array;
    private int[] m_image;
    private int m_imageWidth;
    private int m_imageHeight;

    private RegioPolygon m_regPol_1, m_regPol_2;
    private float m_zoom;

    private int m_computationType = FULL_PROBING_ROIS;

    private int m_permutations;

    private int m_numberOfProbingPixels;
    
    public final static int FULL_PROBING_ROIS = 0;

    private int m_probeSizeX;
    private int m_probeSizeY;
    
    public final static int PERMUTATED_PROBING_ROIS = 1;
    
    private float m_lastComputedTvalue;
    
    private float[][] m_pMap;
    private float[][] m_tMap;
    private float m_nordicScalingFactor;
    private float m_nordicScaleFactor;
    
}


/**
 * Little helper class
 */
class randomVar 
{
    // a generic random variable
    public double mean;
    public double sumSquares;
    public double sampleVariance;
    public double sampleVarianceNormalized;
    public double sampleStandardDeviation;
    public double n;
}
