/*
 * File: $Id: Convert.java,v 1.1.1.1 2006/04/19 09:38:48 hans Exp $
 * History
 * Date: 30.11.2002 Authors: Martin Buerki  Changes:
 *
 * Copyright (C) 2003 by
 * 
 * Inselspital - Department of Neuroradiology
 * CH 3010 Bern Switzerland
 */
package org.spectrim.lib;


/**
 * Class that handles at least following conversions:
 *           string to int
 *           double to string
 *           float to string
 *           byte to short
 *           byte to int
 *           byte to long
 *           byte to double
 *           short to byte
 */

public final class Convert 
{

    /**
     * Converts an integer to a String object
     * @param val
     * @param decimals
     * @return
     */
    public static String int2str( int val, int decimals ) 
    {
        return int2str( val, decimals, '0' );
    }

    /**
     * Converts an int of to a String object with lenght decimals and prefix
     * @param val
     * @param decimals
     * @param prefix
     * @return
     */
    public static String int2str( int val, int decimals, char prefix ) 
    {
        String pfx = String.valueOf( prefix );
        String s = "";
        int n = ( val != 0 ? (int)(Math.log( (double)val) / Math.log( 10.0 ) ) : 0 );
        for ( int k = 1; k < decimals - n; k++ )
        {
            s += pfx;
        }
        s += val;
        return s;
    }

    /**
     * Converts a float to a String
     * @param val
     * @param decimals
     * @param fraction
     * @param prefix
     * @return
     */
    public static String float2str( float val, int decimals, int fraction, char prefix ) 
    {
        String pfx = String.valueOf( prefix );
        String s = "";
        int n = ( val!=0f ? (int)( Math.log((double)val)/Math.log(10.0) ) : 0 );
        for ( int k = 1; k < decimals - n; k++ )
        {
            s += pfx;
        }
        s += Float.toString( val );
        int i = s.lastIndexOf( "." );
        if ( i==-1 )
        {
            s = s + "." + charArray( '0', fraction );
        }    
        else if ( i + fraction >= s.length( ) )
        {
            s = s + charArray( '0', i + fraction - s.length( ) + 1 );
        }
        else
        {
            s = s.substring( 0, i + fraction + 1 );
        }
        return s;
    }

    public static String charArray( char c, int length )
    {
        String s = "";
        for ( int k = 0; k < length; k++ )
        {
            s += String.valueOf( c );
        }
        return s;
    }

    public static short byte2shortl( byte[] b, int offset ) 
    {
        return (short)(    ( ( (short)b[ offset+1 ] ) << 8 )
                         | ( ( (short)b[ offset ] ) & 255 )
                      );
    }

    public static int byte2intl( byte[] b, int offset ) 
    {
        if ( ( offset < b.length ) && offset + 1 < b.length )
        {
            return (  ( ( (int)b[offset+1] ) << 8 )
                    | ( ( (int)b[offset] ) & 255 ) );
        }
        else
        {
            return -1;
        }
    }

    /**
     * Converts a short "value" to a byte array
     * @param b
     * @param index
     * @param value
     */
    public static void short2bytel( byte[] b, int index, short value ) 
    {
        b[index] = (byte)( value & 255 );
        b[index+1] = (byte)( value >> 8 );
    }

    /**
     * Converts two bytes of an array to short
     * @param b
     * @param offset
     * @return
     */
    public static short byte2shortb( byte[] b, int offset ) 
    {
        return (short)(   ( ( (short)b[offset] ) << 8 )
                        | ( ( (short)b[offset+1] ) & 255 ) );
    }

    /**
     * Converts 2 bytes of an array to a short
     * @param b
     * @param offset
     * @return
     */
    public static int byte2intb( byte[] b, int offset ) 
    {
        return ( ( ( (int)b[offset] ) << 8 )
                | ( ( (int)b[offset+1] ) & 255 ) );
    }

    /**
     * Stores two array elements of a short 'value' as a byte of an array
     * @param b
     * @param index
     * @param value
     */
    public static void short2byteb( byte[] b, int index, short value ) 
    {
        b[index+1] = (byte)( value & 255 );
        b[index] = (byte)( value >> 8 );
    }

    public static void int2wordl( byte[] b, int index, int value ) 
    {
        b[index] = (byte)( value & 0xFF );
        b[index + 1 ] = (byte)( ( value & 0xFF00 ) >> 8 );
    }

    public static void int2wordb( byte[] b, int index, int value ) 
    {
        b[index + 1] = (byte)( value & 0xFF );
        b[index] = (byte)( ( value & 0xFF00 ) >> 8 );
    }

    public static final int byte2int( byte[] b, int offs ) 
    {
        return (  ( ( (int)b[offs]) << 24 )
		  | ( ( ( (int)b[offs+1]) & 255 ) << 16 ) 
		  | ( ( ( (int)b[offs+2]) & 255 ) << 8 )
		  | ( ( (int)b[offs+3]) & 255 ) 
		);
    }

    public static final long byte2long( byte[] b, int offs ) 
    {
	return ( ( ( (long)byte2int( b, offs ) ) << 32 )
		  | ( ( (long)byte2int( b, offs ) ) & ( -1l >>> 32 ) ) 
               );
    }

    public static final double byte2double( byte[] b, int offs ) 
    {
        return ( Double.longBitsToDouble( byte2long( b, offs ) ) );
    }
    
    /**
     * Test code to convert a bytearray to a float
     */
    public static final float byte2float( byte[] b, int offs ) 
    {
        return ( Float.intBitsToFloat( byte2int( b, offs ) ) );
    }

    // int -> byte[4];
    public static final byte[] integer2ByteArray( int number )
    {
        byte[] data = new byte[4];

        // int -> byte[]
        for (int i = 0; i < 4; ++i)
        {
            int shift = i << 3; // i * 8
            data[3-i] = (byte)((number & (0xff << shift)) >>> shift);
        }
        return data;
    }

    // byte[] -> int
    public static final int byteArray2Integer( byte[] data )
    {
        int number = 0;
        for ( int i = 0; i < 4; ++i )
        {
            number |= (data[3-i] & 0xff) << (i << 3);
        }
        return number;
    }

}
