/*
 * File: $Id: DecimalTextField.java,v 1.1.1.1 2006/04/19 09:38:48 hans Exp $
 * History
 * Date: 27.05.2003, 2:27  Authors: Hans Slotboom
 *
 * Changes:
 *
 * Copyright (C) 2003 by
 *
 * Inselspital - Department of Neuroradiology
 * CH 3010 Bern Switzerland
 */

package org.spectrim.lib;

import java.awt.Dimension;
import java.awt.Component;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseWheelListener;

import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseWheelEvent;

import java.awt.event.FocusAdapter;
import java.awt.event.FocusEvent;

import java.text.DecimalFormat;

import java.util.LinkedList; 
import java.util.Iterator;

import javax.swing.JComponent;
import javax.swing.JTextField;
import javax.swing.Timer;
import javax.swing.JPopupMenu;
import javax.swing.JRadioButtonMenuItem;
import javax.swing.ButtonGroup;


/**
 * Handles the GUI behavior of the DecimalTextField object.
 * @author  hans
 */
public class DecimalTextField extends JTextField implements ActionListener, KeyListener, MouseWheelListener
{    
    
    /** 
     * Creates a new instance of DecimalTextField (default constructor). 
     */
    public DecimalTextField( )
    {
        // default formatting string:
        m_formatString = "#0.000###E0";
        m_formatter = new DecimalFormat( "#0.000###E0" );
    } 
    
    /** 
     * Creates a new instance of DecimalTextField. 
     */
    public DecimalTextField( String format )
    {
        super( );
        
        m_formatter = new DecimalFormat( format );
        addKeyListener( this );
        setPreferredSize( new Dimension( 80, 25 ) );
        setMaximumSize( new Dimension( 80, 25 ) );
        setMinimumSize( new Dimension( 80, 25 ) );
        setAlignmentX( Component.LEFT_ALIGNMENT );
        addMouseWheelListener( this );
        addPopup( );
    }

    /**
     * Activates possibility to use mouse wheel to adjust values.
     */
    public void enableGUIInteraction( boolean activate )
    {
        if ( activate == true )
        {
            addFocusListener( new FocusAdapter( )
            {
                public void focusLost( FocusEvent fe )
                {
                    String text = getText( );
                    String newText = text.replace( ',' , '.' );
                    Float value = new Float( newText );
                    String formattedValue = m_formatter.format( value.doubleValue( ) );
                    setText( formattedValue );
                    if ( m_currentFloatValue != value.floatValue( ) )
                    {

                        m_currentFloatValue = value.floatValue( );
                        notifyValueChanged( );
                    }
                    else
                    {
                        // nothing
                    } 
                }
            });
            
            addMouseWheelListener( this );
            addPopup( );
        }
        else
        {
            removeMouseWheelListener( this );
            this.removePopup( );
        }
    } 
    
    
    /** 
     * Creates a new instance of DecimalTextField. 
     * @param format
     * @param value
     * @param fine
     * @param medium
     * @param coarse
     */
    public DecimalTextField( String format, float value, 
                             float fine, float medium, 
                             float coarse )
    {
        m_formatString = format;
        m_formatter = new DecimalFormat( format );
        
        String formattedValue = m_formatter.format( (double)value );
        formattedValue.replace( ',' , '.' );
        m_currentFloatValue = value;
        
        addKeyListener( this );
        
        this.setText( formattedValue );
        
        setPreferredSize( new Dimension( 80, 25 ) );
        
        addFocusListener( new FocusAdapter( )
        {
            public void focusLost( FocusEvent fe )
            {
                String text = getText( );
                String newText = text.replace( ',' , '.' );
                Float value = new Float( newText );
                String formattedValue = m_formatter.format( value.doubleValue( ) );
                setText( formattedValue );
                if ( m_currentFloatValue != value.floatValue( ) )
                {
                    
                    m_currentFloatValue = value.floatValue( );
                    notifyValueChanged( );
                }
                else
                {
                    // nothing
                } 
            }
        });
        
        addMouseWheelListener( this );
        addPopup( );
    }
    
    
    /**
     * Removes a pop up from the decimal text field.
     */
    public void removePopup( )
    {
        m_popup = null;
    }
    
    /**
     * Adds a pop up to the decimal text field.
     */
    private void addPopup( )
    {
        m_popup = new JPopupMenu( "Parameter step size" );
        
        m_fineAdjustment = new JRadioButtonMenuItem( "Fine" );
        m_fineAdjustment.addActionListener( this );
        m_mediumAdjustment = new JRadioButtonMenuItem( "Medium" );
        m_mediumAdjustment.addActionListener( this );
        m_mediumAdjustment.setSelected( true );
        m_coarseAdjustment = new JRadioButtonMenuItem( "Coarse" );
        m_coarseAdjustment.addActionListener( this );
        m_popup.add( m_fineAdjustment );
        m_popup.add( m_mediumAdjustment );
        m_popup.add( m_coarseAdjustment );
        
        m_buttenGroup = new ButtonGroup( );
        
        m_buttenGroup.add( m_fineAdjustment );
        m_buttenGroup.add( m_mediumAdjustment );
        m_buttenGroup.add( m_coarseAdjustment );
        
        this.addMouseListener( 
            new MouseAdapter( )
            {
                @Override
                public void mouseReleased( MouseEvent e )
                {
                    if ( e.isPopupTrigger( ) )
                    {   
                        // popup depends on which node has been selected
                        m_popup.show( (JComponent)e.getSource( ), e.getX( ), e.getY( ) );
                    }
                }
            }
        );
    }
    
    
    /**
     * Sets the decimal format string of the DecimalFormat string.
     * @param format
     */
    public void setDecimalFormat( String format )
    {
       m_formatString = format;
       m_formatter = new DecimalFormat( format );  
    }  
    
    /**
     * 
     * Sets the fine adjustment value, which is the positive or negative 
     * increment if the user rotates the mouse wheel.
     * 
     * @param fineAdjustmentValue 
     */
    public void setFineAdjustmentValue( float fineAdjustmentValue )
    {
        m_fineAdjustmentValue = fineAdjustmentValue;
    }
    
    /**
     * 
     * Sets the medium adjustment value, which is the positive or negative 
     * increment if the user rotates the mouse wheel.
     * 
     * @param mediumAdjustmentValue
     */
    public void setMediumAdjustmentValue( float mediumAdjustmentValue )
    {
        m_mediumAdjustmentValue = mediumAdjustmentValue;
    }
    
    /**
     * 
     * Sets the medium adjustment value, which is the positive or negative 
     * increment if the user rotates the mouse wheel.
     * 
     * @param coarseAdjustmentValue
     */
    public void setCoarseAdjustmentValue( float coarseAdjustmentValue )
    {
        m_coarseAdjustmentValue = coarseAdjustmentValue;
    }
    
    /**
     * Sets the text formatted according to the DecimalFormatter object.
     * @param stringValue
     */ 
    public void setFormattedText( String stringValue )
    {
        Float value = new Float( stringValue );
        String formattedValue = m_formatter.format( value.doubleValue( ) );  
        formattedValue.replace( ',' , '.' );
        m_currentFloatValue = value.floatValue( );
        
        this.setText( formattedValue );
    }
    
    /**
     * Sets the text formatted according to the DecimalFormatter object.
     * @param floatValue
     */ 
    public void setFormattedText( float floatValue )
    {
        String formattedValue = m_formatter.format( floatValue );  
        formattedValue.replace( ',' , '.' );
        m_currentFloatValue = floatValue;
        
        this.setText( formattedValue );
    }
    
    /**
     * Adds an ActionListener to the list of action listeners who are notified
     * when something happens with this DecimalTextField.
     * @param e
     */
    @Override
    public void addActionListener( ActionListener e )
    {
	m_listeners.add( e );
    }
    
    /**
     * Sets the float value currently indicated by this DecimalTextField.
     * @param value
     */
    public void setFloatValue( float value )
    {
        setText( "" + m_formatter.format( value ) );
        m_currentFloatValue = value;
    }
    
    /**
     * Returns the float value currently indicated by this DecimalTextField.
     * @return 
     */
    public float getFloatValue( )
    {
        return m_currentFloatValue;
    }
    
    /**
     * Raises an actionEvent notifying registered objects that the internal state
     * of this object has changed.
     */
    private void notifyValueChanged( )
    {
        Iterator it = m_listeners.iterator( );
        while( it.hasNext( ) )
        {            
            ( ( ActionListener )it.next( ) ).actionPerformed( new ActionEvent( this, VALUE_CHANGED_EVENT, "" ) );
        }    
    }
    
    
    /**
     * Called whenever a key is pressed.
     * @param keyEvent
     */
    @Override
    public void keyPressed( KeyEvent keyEvent )
    {
        if ( keyEvent.getKeyCode( ) == KeyEvent.VK_ENTER )
        {
            String text = getText( );
            String newText = text.replace( ',' , '.' );
            Float value = new Float( newText );
            String formattedValue = m_formatter.format( value.doubleValue( ) );
            setText( formattedValue );
            if ( m_currentFloatValue != value.floatValue( ) )
            {
                m_currentFloatValue = value.floatValue( );
                notifyValueChanged( );
            }
            else
            {
                // do nothing
            }
        }
    }
    
    /**
     * Called whenever a key is pressed.
     * @param keyEvent
     */
    @Override
    public void keyReleased( KeyEvent keyEvent ) 
    {
    }
    
    /**
     * Called whenever a key is typed.
     * @param keyEvent
     */
    @Override
    public void keyTyped( KeyEvent keyEvent ) 
    {
    }
    
    /**
     * Is called whenever a registered component raises a MouseWheelEvent.
     */
    public void mouseWheelMoved( MouseWheelEvent mouseWheelEvent ) 
    {
        m_currentFloatValue = m_currentFloatValue + mouseWheelEvent.getWheelRotation( ) * m_currentAdjustmentStepValue;
        String formattedValue = m_formatter.format( (double)m_currentFloatValue );            
        setText( formattedValue );
        
        m_timer = new Timer( 300, this );
        m_timer.start( );
    }
    
    /**
     * Is called whenever a registered component fires an actionEvent.
     * @param e
     */
    @Override
    public void actionPerformed( ActionEvent e )
    {
        Object sender = e.getSource( );
       
        // the timer started on the moment the user rotated the mouse wheel
        // was timed out: now the GUI update can take place
        if ( sender == m_timer )
        {
            m_timer.stop( );
            String formattedValue = m_formatter.format( (double)m_currentFloatValue );
            setText( formattedValue );
            notifyValueChanged( );
        }
        
        if ( sender == m_fineAdjustment )
        {
            m_currentAdjustmentStepValue = m_fineAdjustmentValue;
        }
        
        if ( sender == m_mediumAdjustment )
        {
            m_currentAdjustmentStepValue = m_mediumAdjustmentValue;
        }
        
        if ( sender == m_coarseAdjustment )
        {
            m_currentAdjustmentStepValue = m_coarseAdjustmentValue;
        }
    }
    
    // Attributes
    private DecimalFormat       m_formatter;
    private String              m_formatString;
    
    private float               m_currentFloatValue;
    public final static int     VALUE_CHANGED_EVENT = 0;
    
    private LinkedList<ActionListener> m_listeners = new LinkedList<>( );
    
    private boolean             m_userRotatedMouseWheelRecently = false;
    
    // Timer needed to prevent an overload of mousewheel rotation events
    private Timer m_timer;
    
    // The user can also choose what effect a rotation should have in the numeric
    // value change: therefore a popup menu is added to the text box
    JPopupMenu                  m_popup;
    
    JRadioButtonMenuItem        m_fineAdjustment;
    JRadioButtonMenuItem        m_mediumAdjustment;
    JRadioButtonMenuItem        m_coarseAdjustment;
    
    ButtonGroup                 m_buttenGroup;
    private float               m_currentAdjustmentStepValue = 1.0f;
    
    private float               m_fineAdjustmentValue     = 0.1f;
    private float               m_mediumAdjustmentValue   = 1.0f;
    private float               m_coarseAdjustmentValue   = 10.0f;
    
    // For proper serialization
    private static final long   serialVersionUID = 1482224591285513L;
}
