/*
 * File: $Id: EnterDecimalValueFrame.java,v 1.1.1.1 2006/04/19 09:38:48 hans Exp $
 * History
 * Date: 16.10.2008 2004, 11:13   Authors: Hans Slotboom
 * Changes:
 *
 * Copyright (C) 2003 by
 * 
 * Inselspital - Department of Neuroradiology
 * CH 3010 Bern Switzerland
 */

package org.spectrim.lib;

import java.awt.BorderLayout;
import java.awt.Point;
import java.awt.Dimension;

import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

import javax.swing.Box;
import javax.swing.BorderFactory;
import javax.swing.BoxLayout;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JLabel;

import java.util.LinkedList;
import java.util.Iterator;

/**
 * Allows the user to change a decimal value interactively.
 * @author  hans
 */
public class EnterDecimalValueFrame extends JFrame implements ActionListener
{
    
    /** 
     * Creates a new instance of EnterDecimalValueFrame. 
     * @param parent
     * @param labelName
     * @param initVal
     * @param title
     * @param format
     */
    public EnterDecimalValueFrame( JFrame parent, String labelName, 
                                   float initVal, String title, String format )
    {
        
        m_parent = parent;
        m_labelName = labelName;
        m_initVal = initVal;
        m_title = title;
        m_format = format;
        createStringPanel( );
        createSelectionPanel( );
        
        m_contentPanel = new JPanel( );
        m_contentPanel.setLayout( new BorderLayout( ) );
        
        m_contentPanel.add( m_stringPanel, BorderLayout.CENTER );
        m_contentPanel.add( m_selectionPanel, BorderLayout.SOUTH );
        
        setContentPane( m_contentPanel );
        
        // add window listener necessary to handle the window closing 
        // event properly
        addWindowListener( 
            new WindowAdapter( )
            {
                @Override
                public void windowClosing( WindowEvent event )
                {
                   dispose( );
                }
            }
        );
        
        setTitle( m_title );
        
        pack( );
        
        // Put the frame nicely centered with respect to its parent.
        Point parentLoc = m_parent.getLocation( );
        Point centerParent = new Point( m_parent.getX( ) + ( m_parent.getWidth( ) / 2 ) - this.getWidth( ) / 2,
                                        m_parent.getY( ) + ( m_parent.getHeight( ) / 2 ) - this.getHeight( ) / 2 );
        setLocation( centerParent );
        
        setVisible( true );
    }
    
    /**
     * Creates the panel where the user can define its string.
     */
    private void createStringPanel( )
    {
        m_stringPanel = new JPanel( );
        m_stringPanel.setLayout( new BoxLayout( m_stringPanel, BoxLayout.X_AXIS ) );
        m_stringPanel.setBorder( BorderFactory.createEmptyBorder( 20, 10, 20, 10 ) );
        
        m_stringPanel.setPreferredSize( new Dimension( 350, 65 ) );
        m_stringPanel.setMaximumSize( new Dimension( 350, 65 ) );
        m_stringPanel.setMinimumSize( new Dimension( 350, 65 ) );
        
        m_textLabel = new JLabel( m_labelName );
        
        m_decimalTextField = new DecimalTextField( m_format, (float)m_initVal, 1f, 5f, 10f );
        
        m_decimalTextField.setPreferredSize( new Dimension( 200, 25 ) );
        m_decimalTextField.setMaximumSize( new Dimension( 200, 25 ) );
        m_decimalTextField.setMinimumSize( new Dimension( 200, 25 ) );
        
        m_decimalTextField.selectAll( );
        
        m_stringPanel.add( m_textLabel );
        m_stringPanel.add( Box.createHorizontalGlue( ) );
        m_stringPanel.add( m_decimalTextField );
    }
    
    /**
     * Create selection panel.
     */
    private void createSelectionPanel( )
    {
        
        m_applyButton = new JButton( "Apply" );
        m_applyButton.addActionListener( this );
        m_applyButton.setToolTipText( "Applies the selected zerofilling factor to the currently loaded dataset" );
        m_cancelButton = new JButton( "Cancel" );
        m_cancelButton.addActionListener( this );
        m_cancelButton.setToolTipText( "Cancels this operation without applying the selected zerofilling factor" );
        m_selectionPanel = new JPanel( );
        
        m_selectionPanel.setLayout( new BoxLayout( m_selectionPanel, BoxLayout.X_AXIS ) );
        m_selectionPanel.setBorder( BorderFactory.createEmptyBorder( 5, 5, 5, 5 ) );
        
        m_selectionPanel.setPreferredSize( new Dimension( 350, 60 ) );
        m_selectionPanel.setMinimumSize( new Dimension( 350, 60 ) );
        m_selectionPanel.setMaximumSize( new Dimension( 350, 60 ) );
        
        m_selectionPanel.add( Box.createGlue( ) );
        m_selectionPanel.add( m_applyButton );
        m_selectionPanel.add( Box.createGlue( ) );
        m_selectionPanel.add( m_cancelButton );
        m_selectionPanel.add( Box.createGlue( ) );
    }
    
    /**
     * Is called when ever a registered component raises an ActionEvent.
     * @param e
     */
    @Override
    public void actionPerformed( ActionEvent e ) 
    {
        Object sender = e.getSource( );
        Iterator it = m_actionListeners.iterator( );
        while ( it.hasNext( ) )
        {
            if ( sender == m_applyButton )
            {
               ( ( ActionListener )it.next( ) ).actionPerformed( new ActionEvent( this, APPLY_EVENT, "APPLY" ) );
            }
            if ( sender == m_cancelButton )
            {
               ( ( ActionListener )it.next( ) ).actionPerformed( new ActionEvent( this, CANCEL_EVENT, "CANCEL" ) );
            }
        }
        dispose( );
    }
    
    /**
     * Add action listener to the list of listeners.
     * @param e
     */
    public void addActionListener( ActionListener e )
    {
        m_actionListeners.add( e );
    }
    
    /**
     * Returns the current text of the text field as string.
     * @return 
     */
    public String getText( )
    {
        return m_decimalTextField.getText( );
    }
    
    // (Private) attributes
    private JPanel m_selectionPanel;
    private JPanel m_stringPanel;
    private final JPanel m_contentPanel;
    
    private JButton m_applyButton;
    private JButton m_cancelButton;
    private final JFrame  m_parent;
    
    private DecimalTextField m_decimalTextField;
    private JLabel     m_textLabel;
    
    private final String m_labelName;
    private final float  m_initVal;
    private final String m_title;
    private final String m_format;
    
    // Listeners for action events in SliceCanvasPolyDraw
    private final LinkedList<ActionListener> m_actionListeners = new LinkedList<>( );
    
    public final static int APPLY_EVENT = 0;
    public final static int CANCEL_EVENT = 1;

    // For proper serialization
    private static final long serialVersionUID = 1482224591289213L;
}
