/*
 * File: $Id: GenericFilenameFilter.java,v 1.1.1.1 2006/04/19 09:38:48 hans Exp $
 * History
 * Date: 04.02.2003  Authors: Martin Buerki  Changes:
 *
 * Copyright (C) 2003 by
 * 
 * Inselspital - Department of Neuroradiology
 * CH 3010 Bern Switzerland
 */


package org.spectrim.lib;

import java.util.StringTokenizer;

import java.io.FilenameFilter;
import java.io.File;

public final class GenericFilenameFilter implements FilenameFilter 
{
    
    private String[] tokens;
    private int numTokens;
    private boolean hasFixedPrefix;
    private boolean hasFixedSuffix;
    
    public GenericFilenameFilter( String pattern ) 
    {
	StringTokenizer sTok = new StringTokenizer( pattern, "*" );
	numTokens = sTok.countTokens( );
	tokens = new String[numTokens];
	for ( int k=0; k<numTokens; k++ ) 
        { 
            tokens[k] = new String( sTok.nextToken( ) ); 
        }
	hasFixedPrefix = !pattern.startsWith( "*" );
	hasFixedSuffix = !pattern.endsWith( "*" );
    }
    
    public String[] getTokens( )
    {
	return tokens;
    }

    public int countTokens( ) 
    {
	return numTokens;
    }

    public boolean accept( File dir, String name )
    {
	if ( hasFixedPrefix && !name.startsWith( tokens[0] ) ) 
        { 
            return false;
        }
	if ( hasFixedSuffix && !name.endsWith( tokens[numTokens-1] ) ) 
        {
            return false;
        }
	boolean ret = true;
	int k = 0;
	int currentIndex = -1;
	while ( ret && k < numTokens ) 
        {
	    int i = name.indexOf( tokens[k++] );
	    if ( i > currentIndex ) 
            {
                currentIndex = i; 
            }
	    else
            {
                ret = false;
            }
	}
	return ret;
    }
}