/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package org.spectrim.main;

/**
 * Interface necessary to retrieve metabolite features in SpectrIm as well
 * as from jMRUI.
 * @author Johannes Slotboom <johannes.slotboom at insel.ch>
 */
public interface MetaboliteFeaturesInterface 
{
   
    /**
     * Returns the feature belonging to number 'feature'.
     * @param feature
     * @return 
     */
    public double getFeature( int feature ); 
    

    /**
     * Returns the number of features to be called.
     * @return 
     */
    public int getNbFeatures( );

    /**
     * Returns the different names of the features as an array.
     * @return 
     */
    public String[] getFeatureNames( ); 
    

    /**
     * Returns the area attribute.
     * @return 
     */
    public double getArea( ); 
   
    /**
     * Sets the area attribute.
     * @param area 
     */
    public void setArea( double area ); 
   

    /**
     * Returns the amplitude attribute.
     * @return 
     */
    public double getAmplitude( ); 

    /**
     * Sets the amplitude attribute.
     * @param amplitude 
     */
    public void setAmplitude( double amplitude );

    /**
     * Returns the frequency shift attribute.
     * @return 
     */
    public double getFrqShift( );


    /**
     * Sets the frequency shift parameter.
     * @param frqShift 
     */
    public void setFrqShift( double frqShift ); 

    
    /**
     * Returns the Lorentzian damping constant.
     * @return 
     */
    public double getDamping( ); 

    /**
     * Sets the Lorentzian damping constant.
     * @param damping 
     */
    public void setDamping( double damping ); 

    /**
     * Returns the Gaussian damping constant.
     * @return 
     */
    public double getGaussianDamping( ); 

    
    /**
     * Sets the Gaussian damping constant.
     * @param gdamping 
     */
    public void setGaussianDamping( double gdamping ); 
    
    
    /**
     * Returns the CR-error of the amplitude parameter.
     * @return 
     */
    public double getStdDevArea( );

    /**
     * Sets the CR-error of the amplitude parameter.
     * @param stdDevAmpl
     */
    public void setStdDevArea( double stdDevAmpl );

    /**
     * Returns the phase parameter.
     * @return 
     */
    public double getPhase( );

    /**
     * Sets the phase parameter.
     * @param phase 
     */
    public void setPhase( double phase ); 

    
    /**
     * Returns the error parameter.
     * @return 
     */
    public double getError( );


    /**
     * Sets the error parameter.
     * @param error 
     */
    public void setError( double error );
    
}
