/*
 * File: $Id: MRPatientSpectrumDataBase.java,v 1.1.1.1 2006/04/19 09:38:48 hans Exp $
 * History
 * Date: 05.01.2004, 21:37  Authors: Hans Slotboom
 *
 * Changes:
 *
 * Copyright (C) 2003 -2004 by
 * 
 * Inselspital - Department of Neuroradiology
 * CH 3010 Bern Switzerland
 */


/*
 * MRPatientSpectrumDataBase.java
 *
 * Created on January 5, 2004, 21:37 PM
 */

package org.spectrim.patientdatabase;

import java.io.File;
import java.io.IOException;
import java.io.RandomAccessFile;

import java.util.TreeMap;
import java.util.Iterator;
import org.spectrim.tdfdfit.TDFDFittingDefaults;


/**
 *
 * @author  hans
 */
public class MRPatientSpectrumDataBase 
{
    
    /**
     * Singleton class that handles the file based spectroscopic database
     *
     * @author  hans
     */
    private MRPatientSpectrumDataBase( )
    {
        TDFDFittingDefaults fitDef = TDFDFittingDefaults.getInstance( );
        fitDef.load( );
        
        m_dataBasePath = fitDef.getProperty(TDFDFittingDefaults.PATIENT_DATABASE_ROOT_PATH );
        
        m_dataBasePath = m_dataBasePath + System.getProperty( "file.separator" ) + "spectroDBASE.bin";

        boolean exists = ( new File( m_dataBasePath ) ).exists( );

        try
        { 
            // if the database exits : load it
            if ( exists )
            {
                 RandomAccessFile dBaseReader = new RandomAccessFile( new File( m_dataBasePath ) , "rw" );
                 readDataBase( dBaseReader );
                 dBaseReader.close( );
            }
            else
            {
                // TO DO : create an empty database file
                RandomAccessFile dBaseWriter = new RandomAccessFile( new File( m_dataBasePath ) , "rw" );
                writeDataBase( dBaseWriter );
                dBaseWriter.close( );
            }
        }
        catch ( IOException e )
        {
            System.out.println( "IOException in reading spectroDBASE.bin" );
        }
    }
    
    /**
     * Writes the patient spectra database to a file
     */
    public void writeDataBase( RandomAccessFile writer )
    {
        try
        {   
            writer.writeUTF( "<MRPatientSpectrumDataBase>" );
            writeDataBaseItems( writer );
            writer.writeUTF( "</MRPatientSpectrumDataBase>" );
        }
        catch( IOException e )
        {
            System.out.println( "IOException while writing spectroDBASE.bin" );
        }
    }

    /**
     * Reads the database items from the database:
     * These items are objects stored in a TreeMap
     */
    private void writeDataBaseItems( RandomAccessFile writer )
    {
        Iterator it = m_databaseItems.keySet( ).iterator( );
        // iterate over the list of items
        while ( it.hasNext( ) )
        {
            MRPatientSpectrumDataBaseItem item = m_databaseItems.get( it.next( ) );
            item.write( writer );
        }
    }
    
    /**
     * Reads the patient spectra database to a file
     */
    public void readDataBase( RandomAccessFile reader )
    {
        String tag;
        try
        {
            // reads the first tag of the database file
            String dummy = reader.readUTF( );
            if ( dummy.compareTo( "<MRPatientSpectrumDataBase>" ) == 0 )
            {
                readDataBaseItems( reader );
            }
            else
            {
                System.out.println( "The file that was opened is not a database file" );
            }
        }
        catch ( IOException e )
        {   
            System.out.println( "IOException occured while reading database file" );
        }
    }

    /**
     * Reads the database items from the database:
     * These items are objects stored in a TreeMap
     */
    private void readDataBaseItems( RandomAccessFile reader )
    {
        try
        {
            String tag = reader.readUTF( );
                
            while ( tag.compareTo( "<MRPatientSpectrumDataBaseItem>" ) == 0 )
            {
                MRPatientSpectrumDataBaseItem item =  new MRPatientSpectrumDataBaseItem( );
                item.read( reader );
                    
                m_databaseItems.put( new Integer( m_databaseItems.size( ) + 1 ), item );
                    
                tag = reader.readUTF( );
            }
                
            if ( tag.compareTo( "</MRPatientSpectrumDataBase>" ) == 0 )
            {
                // System.out.println( "Correctly read patient database file" );
            }
        }
        catch ( IOException e )
        {
            System.out.println( "IOException occured while reading database items in patient database file" );
        }
    }

    /**
     * Returns the one and only instance of MRPatientSpectrumDataBase
     */
    public static MRPatientSpectrumDataBase getInstance( )
    {
        if ( theInstance == null )
        {
            theInstance = new MRPatientSpectrumDataBase( );
        }
        return theInstance;
    }
    
    // Member variables
    private static MRPatientSpectrumDataBase theInstance;
    
    private String m_dataBasePath;
    
    private TreeMap<Integer, MRPatientSpectrumDataBaseItem> m_databaseItems = new TreeMap<Integer, MRPatientSpectrumDataBaseItem>( );
    
}