/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package org.spectrim.registration;

import org.spectrim.imagegraphicobjects.Point3D;
import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.apache.commons.math3.linear.Array2DRowRealMatrix;
import org.apache.commons.math3.linear.LUDecomposition;
import org.apache.commons.math3.linear.RealMatrix;
import org.spectrim.geometricobjects.Cube3D;
import org.spectrim.geometricobjects.Polygon3D;

/**
 *
 * @author nuno
 */
public class AffineTransformation {

    private double[][] matrix = new double[3][4];
    private double[][] matrixInv = new double[3][4];

    public AffineTransformation(double a11, double a12, double a13, double a21, double a22, double a23, double a31, double a32, double a33, double b1, double b2, double b3) {
        matrix[0][0] = a11;
        matrix[0][1] = a12;
        matrix[0][2] = a13;
        matrix[1][0] = a21;
        matrix[1][1] = a22;
        matrix[1][2] = a23;
        matrix[2][0] = a31;
        matrix[2][1] = a32;
        matrix[2][2] = a33;
        matrix[0][3] = b1;
        matrix[1][3] = b2;
        matrix[2][3] = b3;
        computeInverse();
    }

    public AffineTransformation(String file) throws Exception {
            BufferedReader reader;
            String line;
            int linesize;

            reader = new BufferedReader(new FileReader(file));
            int ind = 0;
            while ((line = reader.readLine()) != null && ind < 3) {
                String[] parts = line.split("\\s");
                linesize = parts.length;
                if (linesize > 0) {
                    matrix[ind][0] = Double.valueOf(parts[0]);
                    matrix[ind][1] = Double.valueOf(parts[1]);
                    matrix[ind][2] = Double.valueOf(parts[2]);
                    matrix[ind][3] = Double.valueOf(parts[3]);
                }
                ind++;
            }

//            corrections
            matrix[0][2] = -matrix[0][2];
            matrix[1][2] = -matrix[1][2];
            matrix[2][0] = -matrix[2][0];
            matrix[2][1] = -matrix[2][1];

            
            matrix[0][3] = -matrix[0][3];
            matrix[1][3] = -matrix[1][3];
            
            computeInverse();
            /*catch (FileNotFoundException ex) {
            Logger.getLogger(AffineTransformation.class.getName()).log(Level.SEVERE, null, ex);
        } catch (IOException ex) {
            Logger.getLogger(AffineTransformation.class.getName()).log(Level.SEVERE, null, ex);
        }*/
    }
    
    /**
     * this function allows to add an affine transform operation which will be performed
     * after the current operation: A_new * (A_current * x)
     */
    public void addPosteriorTransformation(String file)
    {
        double[][] matrix = new double[3][4];

        try {
            BufferedReader reader;
            String line;
            int linesize;

            reader = new BufferedReader(new FileReader(file));
            int ind = 0;
            while ((line = reader.readLine()) != null && ind < 3) {
                String[] parts = line.split("\\s");
                linesize = parts.length;
                if (linesize > 0) {
                    matrix[ind][0] = Double.valueOf(parts[0]);
                    matrix[ind][1] = Double.valueOf(parts[1]);
                    matrix[ind][2] = Double.valueOf(parts[2]);
                    matrix[ind][3] = Double.valueOf(parts[3]);
                }
                ind++;
            }
            
//            corrections
            matrix[0][2] = -matrix[0][2];
            matrix[1][2] = -matrix[1][2];
            matrix[2][0] = -matrix[2][0];
            matrix[2][1] = -matrix[2][1];

            
            matrix[0][3] = -matrix[0][3];
            matrix[1][3] = -matrix[1][3];
            
            computeInverse();
        } catch (FileNotFoundException ex) {
            Logger.getLogger(AffineTransformation.class.getName()).log(Level.SEVERE, null, ex);
            return;
        } catch (IOException ex) {
            Logger.getLogger(AffineTransformation.class.getName()).log(Level.SEVERE, null, ex);
            return;
        }
        double [][] cM = this.matrix.clone();
        this.matrix[0][0] = matrix[0][0] * cM[0][0] + matrix[0][1] * cM[1][0] + matrix[0][2] * cM[2][0];
        this.matrix[0][1] = matrix[0][0] * cM[0][1] + matrix[0][1] * cM[1][1] + matrix[0][2] * cM[2][1];
        this.matrix[0][2] = matrix[0][0] * cM[0][2] + matrix[0][1] * cM[1][2] + matrix[0][2] * cM[2][2];
        this.matrix[0][3] = matrix[0][0] * cM[0][3] + matrix[0][1] * cM[1][3] + matrix[0][2] * cM[2][3] + matrix[0][3];
        
        this.matrix[1][0] = matrix[1][0] * cM[0][0] + matrix[1][1] * cM[1][0] + matrix[1][2] * cM[2][0];
        this.matrix[1][1] = matrix[1][0] * cM[0][1] + matrix[1][1] * cM[1][1] + matrix[1][2] * cM[2][1];
        this.matrix[1][2] = matrix[1][0] * cM[0][2] + matrix[1][1] * cM[1][2] + matrix[1][2] * cM[2][2];
        this.matrix[1][3] = matrix[1][0] * cM[0][3] + matrix[1][1] * cM[1][3] + matrix[1][2] * cM[2][3] + matrix[1][3];
        
        this.matrix[2][0] = matrix[2][0] * cM[0][0] + matrix[2][1] * cM[1][0] + matrix[2][2] * cM[2][0];
        this.matrix[2][1] = matrix[2][0] * cM[0][1] + matrix[2][1] * cM[1][1] + matrix[2][2] * cM[2][1];
        this.matrix[2][2] = matrix[2][0] * cM[0][2] + matrix[2][1] * cM[1][2] + matrix[2][2] * cM[2][2];
        this.matrix[2][3] = matrix[2][0] * cM[0][3] + matrix[2][1] * cM[1][3] + matrix[2][2] * cM[2][3] + matrix[2][3];
    }
    
    /**
     * this function allows to add an affine transform operation which will be performed
     * after the current operation: A_new * (A_current * x)
     */
    public void addPosteriorTransformation(AffineTransformation aff)
    {
        double[][] matrix = aff.getAffineMatrix().clone();
        double [][] cM = this.matrix.clone();
        this.matrix[0][0] = matrix[0][0] * cM[0][0] + matrix[0][1] * cM[1][0] + matrix[0][2] * cM[2][0];
        this.matrix[0][1] = matrix[0][0] * cM[0][1] + matrix[0][1] * cM[1][1] + matrix[0][2] * cM[2][1];
        this.matrix[0][2] = matrix[0][0] * cM[0][2] + matrix[0][1] * cM[1][2] + matrix[0][2] * cM[2][2];
        this.matrix[0][3] = matrix[0][0] * cM[0][3] + matrix[0][1] * cM[1][3] + matrix[0][2] * cM[2][3] + matrix[0][3];
        
        this.matrix[1][0] = matrix[1][0] * cM[0][0] + matrix[1][1] * cM[1][0] + matrix[1][2] * cM[2][0];
        this.matrix[1][1] = matrix[1][0] * cM[0][1] + matrix[1][1] * cM[1][1] + matrix[1][2] * cM[2][1];
        this.matrix[1][2] = matrix[1][0] * cM[0][2] + matrix[1][1] * cM[1][2] + matrix[1][2] * cM[2][2];
        this.matrix[1][3] = matrix[1][0] * cM[0][3] + matrix[1][1] * cM[1][3] + matrix[1][2] * cM[2][3] + matrix[1][3];
        
        this.matrix[2][0] = matrix[2][0] * cM[0][0] + matrix[2][1] * cM[1][0] + matrix[2][2] * cM[2][0];
        this.matrix[2][1] = matrix[2][0] * cM[0][1] + matrix[2][1] * cM[1][1] + matrix[2][2] * cM[2][1];
        this.matrix[2][2] = matrix[2][0] * cM[0][2] + matrix[2][1] * cM[1][2] + matrix[2][2] * cM[2][2];
        this.matrix[2][3] = matrix[2][0] * cM[0][3] + matrix[2][1] * cM[1][3] + matrix[2][2] * cM[2][3] + matrix[2][3];
    }
    
    /**
     * this function allows to add an affine transform operation which will be performed
     * before the current operation: A_current * (A_new * x)
     */
    public void addPriorTransformation(AffineTransformation aff)
    {
        double[][] cM = aff.getAffineMatrix().clone();
        double [][] matrix = this.matrix.clone();
        this.matrix[0][0] = matrix[0][0] * cM[0][0] + matrix[0][1] * cM[1][0] + matrix[0][2] * cM[2][0];
        this.matrix[0][1] = matrix[0][0] * cM[0][1] + matrix[0][1] * cM[1][1] + matrix[0][2] * cM[2][1];
        this.matrix[0][2] = matrix[0][0] * cM[0][2] + matrix[0][1] * cM[1][2] + matrix[0][2] * cM[2][2];
        this.matrix[0][3] = matrix[0][0] * cM[0][3] + matrix[0][1] * cM[1][3] + matrix[0][2] * cM[2][3] + matrix[0][3];
        
        this.matrix[1][0] = matrix[1][0] * cM[0][0] + matrix[1][1] * cM[1][0] + matrix[1][2] * cM[2][0];
        this.matrix[1][1] = matrix[1][0] * cM[0][1] + matrix[1][1] * cM[1][1] + matrix[1][2] * cM[2][1];
        this.matrix[1][2] = matrix[1][0] * cM[0][2] + matrix[1][1] * cM[1][2] + matrix[1][2] * cM[2][2];
        this.matrix[1][3] = matrix[1][0] * cM[0][3] + matrix[1][1] * cM[1][3] + matrix[1][2] * cM[2][3] + matrix[1][3];
        
        this.matrix[2][0] = matrix[2][0] * cM[0][0] + matrix[2][1] * cM[1][0] + matrix[2][2] * cM[2][0];
        this.matrix[2][1] = matrix[2][0] * cM[0][1] + matrix[2][1] * cM[1][1] + matrix[2][2] * cM[2][1];
        this.matrix[2][2] = matrix[2][0] * cM[0][2] + matrix[2][1] * cM[1][2] + matrix[2][2] * cM[2][2];
        this.matrix[2][3] = matrix[2][0] * cM[0][3] + matrix[2][1] * cM[1][3] + matrix[2][2] * cM[2][3] + matrix[2][3];
    }

    public double[][] getAffineMatrix() {
        return matrix;
    }
    
    private void computeInverse()
    {
        double[][] tmpM = new double[3][3];
        for (int i = 0; i < 3; i++) {
            for (int j = 0; j < 3; j++) {
                tmpM[i][j] = matrix[i][j];
            }
        }
        RealMatrix realMatrix = new Array2DRowRealMatrix(tmpM); // this has to be just for 3x3 matrix
        RealMatrix matrixinv = new LUDecomposition(realMatrix).getSolver().getInverse();
        for (int i = 0; i < 3; i++) {
            for (int j = 0; j < 3; j++) {
                matrixInv[i][j] = matrixinv.getData()[i][j];
            }
        }
        matrixInv[0][3] = matrixInv[0][0] * matrix[0][3] + matrixInv[0][1] * matrix[1][3] + matrixInv[0][2] * matrix[2][3];
        matrixInv[1][3] = matrixInv[1][0] * matrix[0][3] + matrixInv[1][1] * matrix[1][3] + matrixInv[1][2] * matrix[2][3];
        matrixInv[2][3] = matrixInv[2][0] * matrix[0][3] + matrixInv[2][1] * matrix[1][3] + matrixInv[2][2] * matrix[2][3];
    }

    public Point3D transform(Point3D p) {
        double p1 = matrix[0][0] * p.m_x1 + matrix[0][1] * p.m_x2 + matrix[0][2] * p.m_x3 + matrix[0][3];
        double p2 = matrix[1][0] * p.m_x1 + matrix[1][1] * p.m_x2 + matrix[1][2] * p.m_x3 + matrix[1][3];
        double p3 = matrix[2][0] * p.m_x1 + matrix[2][1] * p.m_x2 + matrix[2][2] * p.m_x3 + matrix[2][3];

        return new Point3D(p1, p2, p3);
    }
    
    public Polygon3D transform(Polygon3D p) {
        if(p != null)
        {
            Polygon3D pOut = new Polygon3D();
            for (int i = 0; i < p.size(); i++) {
                double p1 = matrix[0][0] * p.getPoint(i).m_x1 + matrix[0][1] * p.getPoint(i).m_x2 + matrix[0][2] * p.getPoint(i).m_x3 + matrix[0][3];
                double p2 = matrix[1][0] * p.getPoint(i).m_x1 + matrix[1][1] * p.getPoint(i).m_x2 + matrix[1][2] * p.getPoint(i).m_x3 + matrix[1][3];
                double p3 = matrix[2][0] * p.getPoint(i).m_x1 + matrix[2][1] * p.getPoint(i).m_x2 + matrix[2][2] * p.getPoint(i).m_x3 + matrix[2][3];
    //            System.out.println("ref point: ("+p.getPoint(i).m_x1+","+p.getPoint(i).m_x2+","+p.getPoint(i).m_x3+"), transformed: ("+p1+","+p2+","+p3+")");
                pOut.addPoint(p1, p2, p3);
            }
            return pOut;
        }
        return null;
    }
    
    public Polygon3D transformInverse(Polygon3D p) {
        Polygon3D pOut = new Polygon3D();
        for (int i = 0; i < p.size(); i++) {
            double p1 = matrixInv[0][0] * p.getPoint(i).m_x1 + matrixInv[0][1] * p.getPoint(i).m_x2 + matrixInv[0][2] * p.getPoint(i).m_x3 + matrixInv[0][3];
            double p2 = matrixInv[1][0] * p.getPoint(i).m_x1 + matrixInv[1][1] * p.getPoint(i).m_x2 + matrixInv[1][2] * p.getPoint(i).m_x3 + matrixInv[1][3];
            double p3 = matrixInv[2][0] * p.getPoint(i).m_x1 + matrixInv[2][1] * p.getPoint(i).m_x2 + matrixInv[2][2] * p.getPoint(i).m_x3 + matrixInv[2][3];
//            System.out.println("ref point: ("+p.getPoint(i).m_x1+","+p.getPoint(i).m_x2+","+p.getPoint(i).m_x3+"), transformed: ("+p1+","+p2+","+p3+")");
            pOut.addPoint(p1, p2, p3);
        }
        return pOut;
    }

    public Cube3D transform(Cube3D cube) {
        Point3D p1;
        Point3D p2;
        Point3D p3;
        Point3D p4;
        Point3D p5;
        Point3D p6;
        Point3D p7;
        Point3D p8;

        p1 = transform(cube.p1);
        p2 = transform(cube.p2);
        p3 = transform(cube.p3);
        p4 = transform(cube.p4);
        p5 = transform(cube.p5);
        p6 = transform(cube.p6);
        p7 = transform(cube.p7);
        p8 = transform(cube.p8);

        return new Cube3D(p1, p2, p3, p4, p5, p6, p7, p8);
    }

    public static void main(String[] args) {
//        AffineTransformation t = new AffineTransformation("C:\\Users\\nuno\\Desktop\\teste\\aff1406290040640.nii");
    }
}
