/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package org.spectrim.settings;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Properties;
import org.spectrim.main.SpectrImClinicalQMRSBoot;
import org.spectrim.model.GlobalModel;
import org.spectrim.settings.processing.PreprocessingAction;
import org.spectrim.settings.processing.ProcessingAction;
import org.spectrim.settings.processing.QuantificationAction;
import org.spectrim.settings.processing.ReferencePeak;
import org.spectrim.settings.processing.ResultsAction;
import org.spectrim.settings.processing.SeriesFilter;

/**
 *
 * @author nunopbarros // johannes slotboom
 */
public class ProcessingSettings {

    private final String PROPNAME_SERIES_FILT = "seriesFilter";
    private final String PROPNAME_PREPRO_ACTION = "preproAction";
    private final String PROPNAME_QUANTF_ACTION = "quantfAction";
    private final String PROPNAME_RESULTS_ACTION = "resAction";
    private final String PROPNAME_PROC_ACTION = "processAction";
    private final String PROPNAME_QUAL_SINGLE = "qualitySingleEvaluationFile";
    private final String PROPNAME_REFPEAKS_SHIFTCORR = "refPeaksShiftCorr";
    SpectrImClinicalQMRSBoot instance = SpectrImClinicalQMRSBoot.getInstance( );
    
    private final String spectrimRootDir = instance.getProperty( SpectrImClinicalQMRSBoot.SPECTRIM_CLINICAL_QMRS_BASE_PATH );  //System.getProperty("user.home") + File.separator + ".SpectrIm";
    private static ProcessingSettings theInstance = null;

    private String qualitySingleEvaluationFile;
    private ArrayList<SeriesFilter> seriesFilters = new ArrayList<>();
    private ArrayList<PreprocessingAction> preprocessingActions = new ArrayList<>();
    private ArrayList<QuantificationAction> quantificationActions = new ArrayList<>();
    private ArrayList<ResultsAction> resultsActions = new ArrayList<>();
    private ArrayList<ProcessingAction> processingActions = new ArrayList<>();
    private ArrayList<ReferencePeak> referencePeaks = new ArrayList<>();

    private Properties          properties;
    private boolean             somethingChanged = false;
    public static final String  separator1 = ">>";
    public static final String  separator2 = "::";

    private ProcessingSettings( ) 
    {
        loadParams( );
    }

    public static ProcessingSettings getInstance( ) 
    {
        if ( theInstance == null ) 
        {
            theInstance = new ProcessingSettings( );
        }
        return theInstance;
    }

    private void loadParams( ) 
    {
        // Create folder if not existent
        new File( spectrimRootDir + File.separator + "settings" ).mkdirs( );
        properties = new Properties( );
        InputStream is = null;

        // First try loading from the current directory
        try 
        {
            File f = new File( spectrimRootDir + File.separator + "settings" + File.separator + "processingSettings.properties" );
            is = new FileInputStream( f );
        } 
        catch ( Exception e ) 
        {
        }

        try 
        {
            if ( is == null ) 
            {
                // Try loading from classpath
                is = getClass( ).getResourceAsStream( "processingSettings.properties" );
            }

            // Try loading properties from the file (if found)
            properties.load( is );
        } catch ( Exception e )
        {
            System.out.print( " " );
        }

        qualitySingleEvaluationFile = properties.getProperty( this.PROPNAME_QUAL_SINGLE, System.getProperty( "user.home" ) );

        //First load all elements that compose processing actions
        for ( String key : properties.stringPropertyNames( ) ) 
        {
            if ( key.startsWith( this.PROPNAME_SERIES_FILT ) ) 
            {
                SeriesFilter filter = new SeriesFilter( properties.getProperty( key ) );
                seriesFilters.add( filter );
            } 
            else if ( key.startsWith( this.PROPNAME_PREPRO_ACTION ) ) 
            {
                PreprocessingAction action = new PreprocessingAction( properties.getProperty( key ) );
                preprocessingActions.add( action );
            } 
            else if ( key.startsWith( this.PROPNAME_QUANTF_ACTION ) ) 
            {
                QuantificationAction action = new QuantificationAction( properties.getProperty( key ) );
                quantificationActions.add( action );
            } else if (key.startsWith( this.PROPNAME_RESULTS_ACTION ) ) 
            {
                ResultsAction action = new ResultsAction( properties.getProperty( key ) );
                resultsActions.add( action );
            } else if ( key.startsWith(this.PROPNAME_REFPEAKS_SHIFTCORR ) ) 
            {
                ReferencePeak peak = new ReferencePeak( properties.getProperty( key ) );
                referencePeaks.add( peak );
            }
        }

        // Now processing actions can be loaded
        for ( String key : properties.stringPropertyNames( ) ) 
        {
            if ( key.startsWith( this.PROPNAME_PROC_ACTION ) ) 
            {
                String[] txt = properties.getProperty( key ).split( separator1 );
                ProcessingAction action = new ProcessingAction( );
                action.setName( txt[0] );
                String nameFilter   = txt[1];
                String namePrepro   = txt[2];
                String nameQuantif  = txt[3];
                String nameResults  = ""; //Compatibility with files prior to introduction of Results Actions
                if ( txt.length == 5 ) 
                {
                    nameResults = txt[4];    
                }
                for ( SeriesFilter filter : this.seriesFilters ) 
                {
                    if ( filter.getName( ).matches( nameFilter ) ) 
                    {
                        action.setSeriesFilter( filter );
                        break;
                    }
                }
                for ( PreprocessingAction actionPrepro : this.preprocessingActions ) 
                {
                    if ( actionPrepro.getName( ).matches( namePrepro ) ) 
                    {
                        action.setPreprocessingAction( actionPrepro );
                        break;
                    }
                }
                for ( QuantificationAction actionQuantif : this.quantificationActions ) 
                {
                    if ( actionQuantif.getName( ).matches( nameQuantif ) ) 
                    {
                        action.setQuantificationAction( actionQuantif );
                        break;
                    }
                }
                for ( ResultsAction actionRes : this.resultsActions ) 
                {
                    if ( actionRes.getName( ).matches( nameResults ) ) 
                    {
                        action.setResultsAction(actionRes);
                        break;
                    }
                }

                processingActions.add( action );
            }
        }
        Collections.reverse( seriesFilters );
        Collections.reverse( preprocessingActions );
        Collections.reverse( quantificationActions );
        Collections.reverse( resultsActions );
        Collections.reverse( processingActions );
        if ( referencePeaks.isEmpty( ) ) 
        {
            referencePeaks.add( new ReferencePeak( "Cho", 3.19, 3.11, 3.3, 1 ) );
            referencePeaks.add( new ReferencePeak( "Cr", 3.02, 2.8, 3.1, 1 ) );
            referencePeaks.add( new ReferencePeak( "NAA", 2.008, 1.93, 2.1, 1 ) );
            referencePeaks.add( new ReferencePeak( "Cit", 2.6, 2.5, 2.8, 2 ) );
        }
    }

    public void saveProperties( ) 
    {
        somethingChanged = true;
        properties.setProperty( this.PROPNAME_QUAL_SINGLE, qualitySingleEvaluationFile );
        for ( String key : properties.stringPropertyNames( ) ) 
        {
            if ( key.startsWith( this.PROPNAME_SERIES_FILT ) || key.startsWith( this.PROPNAME_PREPRO_ACTION )
                 || key.startsWith( this.PROPNAME_QUANTF_ACTION ) || key.startsWith( this.PROPNAME_RESULTS_ACTION )
                 || key.startsWith( this.PROPNAME_PROC_ACTION ) || key.startsWith( this.PROPNAME_REFPEAKS_SHIFTCORR ) ) 
            {
                properties.remove( key );
            }
        }

        int index = 0;
        for ( SeriesFilter filter : seriesFilters ) 
        {
            properties.setProperty( PROPNAME_SERIES_FILT + index, filter.toString( ) );
            index++;
        }
        index = 0;
        for ( PreprocessingAction action : preprocessingActions ) 
        {
            properties.setProperty(PROPNAME_PREPRO_ACTION + index, action.toString());
            index++;
        }
        index = 0;
        for ( QuantificationAction action : quantificationActions ) 
        {
            properties.setProperty( PROPNAME_QUANTF_ACTION + index, action.toString( ) );
            index++;
        }
        index = 0;
        for ( ResultsAction action : resultsActions ) 
        {
            properties.setProperty( PROPNAME_RESULTS_ACTION + index, action.toString( ) );
            index++;
        }
        index = 0;
        for ( ProcessingAction action : processingActions ) 
        {
            properties.setProperty( PROPNAME_PROC_ACTION + index, action.toString( ) );
            index++;
        }
        index = 0;
        for ( ReferencePeak peak : referencePeaks ) 
        {
            properties.setProperty( PROPNAME_REFPEAKS_SHIFTCORR + index, peak.toString( ) );
            index++;
        }

        try 
        {
            File f = new File( spectrimRootDir + File.separator + "settings" + File.separator + "processingSettings.properties" );
            OutputStream out = new FileOutputStream( f );
            properties.store( out, "SpectrIm Processing properties" );
        } 
        catch ( Exception e ) 
        {
            e.printStackTrace( );
        }
    }

    public boolean didSomethingChange( ) 
    {
        if ( somethingChanged ) 
        {
            somethingChanged = false;
            return true;
        }
        return false;
    }

    public ArrayList<ReferencePeak> getReferencePeaks( ) 
    {
        return referencePeaks;
    }

    public void setReferencePeaks( ArrayList<ReferencePeak> referencePeaks ) 
    {
        this.referencePeaks = referencePeaks;
    }

    public String getQualitySingleEvaluationFile( ) 
    {
        return qualitySingleEvaluationFile;
    }

    public void setQualitySingleEvaluationFile( String qualitySingleEvaluationFile ) 
    {
        this.qualitySingleEvaluationFile = qualitySingleEvaluationFile;
    }

    public ArrayList<SeriesFilter> getSeriesFilters( ) 
    {
        return seriesFilters;
    }

    public void setSeriesFilters( ArrayList<SeriesFilter> seriesFilters ) 
    {
        this.seriesFilters = seriesFilters;
    }

    public ArrayList<PreprocessingAction> getPreprocessingActions( ) 
    {
        return preprocessingActions;
    }

    public void setPreprocessingActions( ArrayList<PreprocessingAction> preprocessingActions ) 
    {
        this.preprocessingActions = preprocessingActions;
    }

    public ArrayList<QuantificationAction> getQuantificationActions( ) 
    {
        return quantificationActions;
    }

    public void setQuantificationActions( ArrayList<QuantificationAction> quantificationActions ) 
    {
        this.quantificationActions = quantificationActions;
    }

    public ArrayList<ResultsAction> getResultsActions( ) 
    {
        return resultsActions;
    }

    public void setResultsActions( ArrayList<ResultsAction> resultsActions ) 
    {
        this.resultsActions = resultsActions;
    }

    public ArrayList<ProcessingAction> getProcessingActions( ) 
    {
        return processingActions;
    }

    public ArrayList<ProcessingAction> getProcessingActions( GlobalModel model ) 
    {
        ArrayList<ProcessingAction> validActions = new ArrayList<>( );
        for (ProcessingAction action : processingActions) 
        {
            if ( action.getSeriesFilter( ) == null ) 
            {
                validActions.add( action );
            } 
            else if ( action.getSeriesFilter( ).evaluate( model ) )
            {
                validActions.add(action);
            }
        }
        return validActions;
    }

    public void setProcessingActions( ArrayList<ProcessingAction> processingActions ) 
    {
        this.processingActions = processingActions;
    }
    
}
