/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package org.spectrim.settings;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Properties;
import org.spectrim.main.SpectrImClinicalQMRSBoot;

/**
 *
 * @author Giezi
 */
public class RegistrationSettings 
{
    SpectrImClinicalQMRSBoot instance = SpectrImClinicalQMRSBoot.getInstance( );
    private final String spectrimRootDir = instance.getProperty( SpectrImClinicalQMRSBoot.SPECTRIM_CLINICAL_QMRS_BASE_PATH ); //System.getProperty("user.home")+File.separator+".SpectrIm";
    
    private static RegistrationSettings theInstance;
    private Properties properties;
    
    private String f3dInstallPath;
    private String aladinInstallPath;
    private String regTransformInstallPath;
    private String dicom2niiInstallPath;
    private String outputFolder;
    private String affineTransformsFolder;
    private String roiPrstFolder;
    private String roiFeaturesGTFolder;
    private String roiRModel;
    private String coregistrationLibrary;
    
    private RegistrationSettings() {
        loadProperties();
    }
    
    public static RegistrationSettings getInstance( )
    {
        if ( theInstance == null )
        {
            theInstance = new RegistrationSettings( );
        }
        return theInstance;
    }
    
    private void loadProperties()
    {
        // Create folder if not existent
        new File(spectrimRootDir+File.separator+"settings").mkdirs();
        new File(spectrimRootDir + File.separator + "affineTransforms").mkdirs();
        new File(spectrimRootDir + File.separator + "roiPrstFolder").mkdirs();
        new File(spectrimRootDir + File.separator + "roiFeaturesGTFolder").mkdirs();
        
        properties = new Properties();
        InputStream is = null;

        // First try loading from the current directory
        try {
            File f = new File(spectrimRootDir+File.separator+"settings"+File.separator+"registrationSettings.properties");
            is = new FileInputStream(f);
        } catch (Exception e) {}

        try {
            if (is == null) {
                // Try loading from classpath
                is = getClass().getResourceAsStream("registrationSettings.properties");
            }

            // Try loading properties from the file (if found)
            properties.load(is);
        } catch (Exception e) {
        }
        
        f3dInstallPath = properties.getProperty("f3dInstallPath", System.getProperty("user.home"));
        aladinInstallPath = properties.getProperty("aladinInstallPath", System.getProperty("user.home"));
        dicom2niiInstallPath = properties.getProperty("dicom2niiInstallPath", System.getProperty("user.home"));
        regTransformInstallPath = properties.getProperty("regTransformInstallPath", System.getProperty("user.home"));
        outputFolder = properties.getProperty("outputFolder", System.getProperty("user.home"));
        affineTransformsFolder = properties.getProperty("affineTransformsFolder", spectrimRootDir + File.separator + "affineTransforms");
        roiPrstFolder = properties.getProperty("roiPrstFolder", spectrimRootDir + File.separator + "roiPrstFolder");
        roiFeaturesGTFolder = properties.getProperty("roiFeaturesGTFolder", spectrimRootDir + File.separator + "roiFeaturesGTFolder");
        roiRModel = properties.getProperty("roiRModel", "");
        coregistrationLibrary = properties.getProperty("coregistrationLibrary", "");
    }
    
    public void saveProperties(){
        properties.setProperty("f3dInstallPath", f3dInstallPath);
        properties.setProperty("aladinInstallPath", aladinInstallPath);
        properties.setProperty("dicom2niiInstallPath", dicom2niiInstallPath);
        properties.setProperty("regTransformInstallPath", regTransformInstallPath);
        properties.setProperty("outputFolder", outputFolder);
        properties.setProperty("affineTransformsFolder", affineTransformsFolder);
        properties.setProperty("roiPrstFolder", roiPrstFolder);
        properties.setProperty("roiFeaturesGTFolder", roiFeaturesGTFolder);
        properties.setProperty("roiRModel", roiRModel);
        properties.setProperty("coregistrationLibrary", coregistrationLibrary);
        
        try {
            File f = new File(spectrimRootDir+File.separator+"settings"+File.separator+"registrationSettings.properties");
            OutputStream out = new FileOutputStream(f);
            properties.store(out, "SpectrIm Registration properties");
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public String getF3dInstallPath() {
        return f3dInstallPath;
    }

    public void setF3dInstallPath(String f3dInstallPath) {
        this.f3dInstallPath = f3dInstallPath;
    }

    public String getAladinInstallPath() {
        return aladinInstallPath;
    }

    public void setAladinInstallPath(String aladinInstallPath) {
        this.aladinInstallPath = aladinInstallPath;
    }

    public String getDicom2niiInstallPath() {
        return dicom2niiInstallPath;
    }

    public void setDicom2niiInstallPath(String mricronInstallPath) {
        this.dicom2niiInstallPath = mricronInstallPath;
    }

    public String getRegTransformInstallPath() {
        return regTransformInstallPath;
    }

    public void setRegTransformInstallPath(String regTransformInstallPath) {
        this.regTransformInstallPath = regTransformInstallPath;
    }

    public String getOutputFolder() {
        return outputFolder;
    }

    public void setOutputFolder(String outputFolder) {
        this.outputFolder = outputFolder;
    }

    public String getAffineTransformsFolder() {
        return affineTransformsFolder;
    }

    public void setAffineTransformsFolder(String affineTransformsFolder) {
        this.affineTransformsFolder = affineTransformsFolder;
    }

    public String getRoiPrstFolder() {
        return roiPrstFolder;
    }

    public void setRoiPrstFolder(String roiPrstFolder) {
        this.roiPrstFolder = roiPrstFolder;
    }

    public String getRoiFeaturesGTFolder() {
        return roiFeaturesGTFolder;
    }

    public void setRoiFeaturesGTFolder(String roiFeaturesGTFolder) {
        this.roiFeaturesGTFolder = roiFeaturesGTFolder;
    }

    public String getRoiRModel() {
        return roiRModel;
    }

    public void setRoiRModel(String roiRModel) {
        this.roiRModel = roiRModel;
    }
    
    public String getCoregistrationLibrary() {
        return coregistrationLibrary;
    }

    public void setCoregistrationLibrary(String coregistrationLibrary) {
        this.coregistrationLibrary = coregistrationLibrary;
    }
    
}
