/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package org.spectrim.settings.processing;

import java.util.ArrayList;
import javax.swing.JOptionPane;
import org.spectrim.batchprocessing.BatchProcessing;
import org.spectrim.model.GlobalModel;
import org.spectrim.settings.ProcessingSettings;

/**
 *
 * @author Giezi, nunopbarros, johannes slotboom
 */
public class PreprocessingAction {

    private String name = "";

    // panel order
       private int[] panelOrder = new int[]{3, 4, 1, 6, 0, 2, 5, 7, 8, 9, 10, 11, 12};

// Appodisation
    private boolean apodisationUse = false;
    private int apodisationType = BatchProcessing.APO_GAUSS;
    private double apodisationLineBroadning = 3.0;

    // Quality
    private boolean qualityUse = false;
    private String qualityRFile = "";
    private double qualityThreshold = 0.5;

    // Baseline
    private boolean baselineUse = false;
    private int baselineNR = 3;

    // Water Removal
    private boolean hlsvd64WaterRemovalUse = true;
    private double  hlsvd64WaterRemovalMin = 4.0;
    private double  hlsvd64WaterRemovalMax = 9.0;
    private int     hlsvd64WaterRemovalNumberOfPeaks = 8;

    private boolean hlsvdProWaterRemovalUse = true;
    private double  hlsvdProWaterRemovalMin = 4.0;
    private double  hlsvdProWaterRemovalMax = 9.0;
    private int     hlsvdProWaterRemovalNumberOfPeaks = 8;
    
    // Auto Phasing
    private boolean autoPhasingUse = true;
    private double  autoPhasingMinPPM = 1.5;
    private double  autoPhasingMaxPPM = 4.4;

    // Frequency Shift
    private boolean freqShiftUse = false;
    private ArrayList<ReferencePeak> freqShiftPeaks = new ArrayList<>();
    private double  freqShiftMinPPM = 0;
    private double  freqShiftMaxPPM = 4.0;
    private double  freqShiftSNR = 10;
    private double  freqShiftMaxLocalShift = 0.1;

    // zero filling
    private boolean zeroFillingUse = false;
    
    // TDFDFit water removal
    private String  tdfdfitWaterModelName;
    private boolean isTDFDFitWaterRemovalUse;
    
    private boolean isUseAmplitudeOffsetRemovalUseTD;
    private boolean isUseAmplitudeOffsetRemovalUseFD;
    
    
    private double  maxRangeFDOffsetRemoval;
    private double  minRangeFDOffsetRemoval;
    
    private double  minFDValue;
    private double  maxFDValue;
    
    private double  maxRangeTDOffsetRemoval;
    private double  minRangeTDOffsetRemoval;
    
    private double  minTDTimePoint;
    private double  maxTDTimePoint;

    /**
     * Default constructor: 
     * At least Cho and NAA reference peaks are defined.
     * TODO: should be removed, not generally applicable
     */
    public PreprocessingAction( GlobalModel model ) 
    {
        freqShiftPeaks.add( new ReferencePeak( "Cho", 3.19, 3.11, 3.3, 1 ) );
        freqShiftPeaks.add( new ReferencePeak( "NAA", 2.008, 1.93, 2.1, 1 ) );
        
        maxRangeFDOffsetRemoval = model.getMaximalPPMValueOfSpectrum( );
        minRangeFDOffsetRemoval = model.getMaximalPPMValueOfSpectrum( ) - 0.1f * model.getPPMRangeOfSpectrum( );
        
        minFDValue = model.getMinimalPPMValueOfSpectrum( );
        maxFDValue = model.getMaximalPPMValueOfSpectrum( );
        
        maxRangeTDOffsetRemoval = ( 1.0f / model.getBandwidth( ) ) * model.getCSIData( ).m_numberOfTimeSamplePoints;
        minRangeTDOffsetRemoval = ( 1.0f / model.getBandwidth( ) ) * ( model.getCSIData( ).m_numberOfTimeSamplePoints - 5f * model.getCSIData( ).m_numberOfTimeSamplePoints / 100f );
        
        minTDTimePoint = 0.0;
        maxTDTimePoint = ( 1.0f / model.getBandwidth( ) ) * model.getCSIData( ).m_numberOfTimeSamplePoints;
    
    }

    
    /**
     * Returns the minimal possible FD value in ppm.
     * @return 
     */
    public double getminFDValue( )
    {
        return minFDValue;
    }
    
    /**
     * Returns the maximum possible FD value in ppm.
     * @return 
     */
    public double getmaxFDValue( )
    {
        return maxFDValue;
    }

    /**
     * Returns the minimal possible FD value in ppm.
     * @return 
     */
    public double getminTDValue( )
    {
        return minTDTimePoint;
    }
    
    /**
     * Returns the maximum possible FD value in ppm.
     * @return 
     */
    public double getmaxTDValue( )
    {
        return maxTDTimePoint;
    }
    
    /**
     * 
     * Based on a preset string the values of the processing action are set.
     * 
     * @param preset 
     */
    public PreprocessingAction( String preset ) 
    {
        String[] grps = preset.split( ProcessingSettings.separator1 );

        String[] panelOrderString = grps[0].split( ProcessingSettings.separator2 );
        
        panelOrder = new int[panelOrderString.length];
        
        for ( int i = 0; i < panelOrderString.length; i++ ) 
        {
            panelOrder[i] = Integer.parseInt( panelOrderString[i] );
        }

        String[] headerString = grps[1].split( ProcessingSettings.separator2 );
        
        name = headerString[0];

        ///////////////////////
        
        String[] apodizationString = grps[2].split( ProcessingSettings.separator2 );
        
        if ( apodizationString.length == 3 ) 
        {
            apodisationUse              = Boolean.parseBoolean( apodizationString[0] );
            apodisationType             = Integer.parseInt( apodizationString[1] );
            apodisationLineBroadning    = Double.parseDouble( apodizationString[2] );
        } 
        else 
        {
            informOfChanges( "The apodization settings changed, please check them out" );
        }

        ///////////////////////
        
        String[] qualityString = grps[3].split( ProcessingSettings.separator2 );
        if ( qualityString.length == 3 ) 
        {
            qualityUse          = Boolean.parseBoolean( qualityString[0] );
            qualityRFile        = qualityString[1];
            qualityThreshold    = Double.parseDouble(qualityString[2]);
        } 
        else 
        {
            informOfChanges( "The quantitation settings changed, please check them out" );
        }

        ///////////////////////
        
        String[] baselineString = grps[4].split( ProcessingSettings.separator2 );
        if ( baselineString.length == 2 ) 
        {
            baselineUse = Boolean.parseBoolean(baselineString[0]);
            baselineNR = Integer.parseInt(baselineString[1]);
        } 
        else 
        {
            informOfChanges( "The baseline settings changed, please check them out" );
        }

        
        ///////////////////////
        
        String[] waterRemovalString = grps[5].split( ProcessingSettings.separator2 );
        if ( waterRemovalString.length == 4 ) 
        {
            hlsvd64WaterRemovalUse              = Boolean.parseBoolean(waterRemovalString[0]);
            hlsvd64WaterRemovalMin              = Double.parseDouble(waterRemovalString[1]);
            hlsvd64WaterRemovalMax              = Double.parseDouble(waterRemovalString[2]);
            hlsvd64WaterRemovalNumberOfPeaks    = Integer.parseInt(waterRemovalString[3]);
        } 
        else 
        {
            informOfChanges( "The water removal settings changed, please check them out" );
        }

        String[] autoPhasingString = grps[6].split( ProcessingSettings.separator2 );
        if ( autoPhasingString.length == 3 ) 
        {
            autoPhasingUse = Boolean.parseBoolean(autoPhasingString[0]);
            autoPhasingMinPPM = Double.parseDouble(autoPhasingString[1]);
            autoPhasingMaxPPM = Double.parseDouble(autoPhasingString[2]);
        } 
        else 
        {
            informOfChanges( "The autophasing settings changed, please check them out" );
        }

        ///////////////////////
        
        String[] freqShiftString = grps[7].split(ProcessingSettings.separator2);

        if ( ( freqShiftString.length - 5 ) % 5 == 0 ) 
        {
            freqShiftUse            = Boolean.parseBoolean( freqShiftString[0] );
            freqShiftSNR            = Double.parseDouble( freqShiftString[1] );
            freqShiftMinPPM         = Double.parseDouble( freqShiftString[2] );
            freqShiftMaxPPM         = Double.parseDouble( freqShiftString[3] );
            freqShiftMaxLocalShift  = Double.parseDouble( freqShiftString[4] );

            for ( int i = 0; i < ( freqShiftString.length - 5 ) / 5; i++ ) 
            {
                int index   = i * 5 + 5;
                String name = freqShiftString[index];
                double targ = Double.parseDouble(freqShiftString[index + 1]);
                double min  = Double.parseDouble(freqShiftString[index + 2]);
                double max  = Double.parseDouble(freqShiftString[index + 3]);
                int mult    = (int) Double.parseDouble(freqShiftString[index + 4]);
                this.freqShiftPeaks.add(new ReferencePeak(name, targ, min, max, mult));
            }
        } 
        else 
        {
            informOfChanges( "The frequency shift settings changed, please check them out" );
            freqShiftPeaks.add( new ReferencePeak( "Cho", 3.19, 3.11, 3.3, 1 ) );
            freqShiftPeaks.add( new ReferencePeak( "NAA", 2.008, 1.93, 2.1, 1 ) );
        }

        ///////////////////////
        
        String[] zeroFillingString = grps[8].split( ProcessingSettings.separator2 );
        if ( zeroFillingString.length == 1 ) 
        {
            zeroFillingUse = Boolean.parseBoolean( zeroFillingString[0] );
        } 
        else 
        {
            informOfChanges( "The zero filling settings changed, please check them out" );
        }
        
        ///////////////////////
        
        String[] tdfdfitWaterRemoval = grps[9].split( ProcessingSettings.separator2 );
        if ( tdfdfitWaterRemoval.length == 2 ) 
        {
            isTDFDFitWaterRemovalUse = Boolean.parseBoolean( tdfdfitWaterRemoval[0] );
            tdfdfitWaterModelName = "" + tdfdfitWaterRemoval[1];
        } 
        else 
        {
            informOfChanges( "The TFDFDFit water removal settings changed, please check them out" );
        }
        
        ///////////////////////
        
        String[] offsetAmplitudeRemovalFD = grps[10].split( ProcessingSettings.separator2 );
        if ( offsetAmplitudeRemovalFD.length == 3 ) 
        {
            this.isUseAmplitudeOffsetRemovalUseFD = Boolean.parseBoolean( offsetAmplitudeRemovalFD[0] );
            this.minRangeFDOffsetRemoval = Double.parseDouble( offsetAmplitudeRemovalFD[1] );
            this.maxRangeFDOffsetRemoval = Double.parseDouble( offsetAmplitudeRemovalFD[2] );
        } 
        else 
        {
            informOfChanges( "The amplitude offset in FD removal settings changed, please check them out" );
        }
        
        ///////////////////////
        
        String[] offsetAmplitudeRemovalTD = grps[11].split( ProcessingSettings.separator2 );
        if ( offsetAmplitudeRemovalTD.length == 3 ) 
        {
            this.isUseAmplitudeOffsetRemovalUseTD = Boolean.parseBoolean( offsetAmplitudeRemovalTD[0] );
            this.minRangeTDOffsetRemoval = Double.parseDouble( offsetAmplitudeRemovalTD[1] );
            this.maxRangeTDOffsetRemoval = Double.parseDouble( offsetAmplitudeRemovalTD[2] );
        } 
        else 
        {
            informOfChanges( "The amplitude offset in TD removal settings changed, please check them out" );
        }
        
        ///////////////////////
        
        String[] waterRemovalHLSVDProString = grps[5].split( ProcessingSettings.separator2 );
        if ( waterRemovalHLSVDProString.length == 4 ) 
        {
            hlsvdProWaterRemovalUse              = Boolean.parseBoolean(waterRemovalString[0]);
            hlsvdProWaterRemovalMin              = Double.parseDouble(waterRemovalString[1]);
            hlsvdProWaterRemovalMax              = Double.parseDouble(waterRemovalString[2]);
            hlsvdProWaterRemovalNumberOfPeaks    = Integer.parseInt(waterRemovalString[3]);
        }
        else 
        {
            informOfChanges( "The HLSVDpro water removal settings changed, please check them out" );
        }
        
    }

    private void informOfChanges( String change ) 
    {
        JOptionPane.showOptionDialog( null, change, "Settings changed", 
                                      JOptionPane.CLOSED_OPTION, 
                                      JOptionPane.INFORMATION_MESSAGE, 
                                      null, null, null);
    }

    /**
     * Clones the PreprocessingAction.
     * @return 
     */
    @Override
    public PreprocessingAction clone( ) 
    {
        return new PreprocessingAction( this.toString( ) );
    }

    /**
     * Important function that turns all the preset values by the user to one
     * string.
     * @return 
     */
    @Override
    public String toString( )
    {
        String panelOrderString = "";
        
        for ( int pn : panelOrder ) 
        {
            panelOrderString += pn + ProcessingSettings.separator2;
        }
        
        panelOrderString = panelOrderString.substring( 0, panelOrderString.length( ) - 
                                                       ProcessingSettings.separator2.length( ) );

        String headerString = name;

        String apodizationString = apodisationUse + ProcessingSettings.separator2 + 
                                   apodisationType + ProcessingSettings.separator2 + 
                                   apodisationLineBroadning;

        String qualityString     = qualityUse + ProcessingSettings.separator2 + 
                                   qualityRFile + ProcessingSettings.separator2 + 
                                   qualityThreshold;

        String baselineString    = baselineUse + ProcessingSettings.separator2 + 
                                   baselineNR;

        String waterRemovalString = hlsvd64WaterRemovalUse + 
                                    ProcessingSettings.separator2 + 
                                    hlsvd64WaterRemovalMin + 
                                    ProcessingSettings.separator2 + 
                                    hlsvd64WaterRemovalMax + 
                                    ProcessingSettings.separator2 + 
                                    hlsvd64WaterRemovalNumberOfPeaks;

        String autoPhasingString = autoPhasingUse + 
                                   ProcessingSettings.separator2 + 
                                   autoPhasingMinPPM + 
                                   ProcessingSettings.separator2 + 
                                   autoPhasingMaxPPM;

        String freqShiftString   = freqShiftUse + ProcessingSettings.separator2 + 
                                   freqShiftSNR + ProcessingSettings.separator2 + 
                                   freqShiftMinPPM + ProcessingSettings.separator2 + 
                                   freqShiftMaxPPM + ProcessingSettings.separator2 + 
                                   freqShiftMaxLocalShift;

        for ( ReferencePeak peak : this.freqShiftPeaks ) 
        {
            freqShiftString += ProcessingSettings.separator2 + peak.getName( );
            freqShiftString += ProcessingSettings.separator2 + peak.getTargetppm( );
            freqShiftString += ProcessingSettings.separator2 + peak.getMinppm( );
            freqShiftString += ProcessingSettings.separator2 + peak.getMaxppm( );
            freqShiftString += ProcessingSettings.separator2 + peak.getMultiplicity( );
        }

        String zeroFillingString = zeroFillingUse + "";

        String TDFDFitWaterRemovalString = isTDFDFitWaterRemovalUse + ProcessingSettings.separator2 + tdfdfitWaterModelName;
        
        String offsetAmplitudeRemovalTD = isUseAmplitudeOffsetRemovalUseTD + 
                                          ProcessingSettings.separator2 + 
                                          this.minRangeTDOffsetRemoval + 
                                          ProcessingSettings.separator2 + 
                                          this.maxRangeTDOffsetRemoval;
        
        String offsetAmplitudeRemovalFD = isUseAmplitudeOffsetRemovalUseFD + 
                                          ProcessingSettings.separator2 + 
                                          this.minRangeFDOffsetRemoval + 
                                          ProcessingSettings.separator2 + 
                                          this.maxRangeFDOffsetRemoval;
        
        
        return panelOrderString + ProcessingSettings.separator1 + headerString + ProcessingSettings.separator1 +
               apodizationString + ProcessingSettings.separator1 + qualityString + ProcessingSettings.separator1 +
               baselineString + ProcessingSettings.separator1 + waterRemovalString + ProcessingSettings.separator1 +
               autoPhasingString + ProcessingSettings.separator1 + freqShiftString + ProcessingSettings.separator1 + 
               zeroFillingString + ProcessingSettings.separator1 + TDFDFitWaterRemovalString + ProcessingSettings.separator1 +
               offsetAmplitudeRemovalFD + ProcessingSettings.separator1 + offsetAmplitudeRemovalTD;
    }

    /**
     * Returns the name of the PreprocessingAction.
     * @return 
     */
    public String getName( ) 
    {
        return name;
    }

    /**
     * ¨Sets the name of the PreprocessingAction.
     * @param name 
     */
    public void setName( String name )
    {
        this.name = name;
    }

    /**
     * Returns the panelOrder attribute, i.e. the order in which the sub panels 
     * should appear on the main panel. 
     * @return 
     */
    public int[] getPanelOrder( ) 
    {
        return panelOrder;
    }

    /**
     * Sets the panelOrder attribute, i.e. the order in which the sub panels 
     * should appear on the main panel. 
     * @param panelOrder
     */
    public void setPanelOrder( int[] panelOrder ) 
    {
        this.panelOrder = panelOrder;
    }

    public boolean isApodisationUse() {
        return apodisationUse;
    }

    public void setApodisationUse(boolean apodisationUse) {
        this.apodisationUse = apodisationUse;
    }

    public int getApodisationType() {
        return apodisationType;
    }

    public void setApodisationType(int apodisationType) {
        this.apodisationType = apodisationType;
    }

    public double getApodisationLineBroadning() {
        return apodisationLineBroadning;
    }

    public void setApodisationLineBroadning(double apodisationLineBroadning) {
        this.apodisationLineBroadning = apodisationLineBroadning;
    }

    public boolean isQualityUse() {
        return qualityUse;
    }

    public void setQualityUse(boolean qualityUse) {
        this.qualityUse = qualityUse;
    }

    public String getQualityRFile() {
        return qualityRFile;
    }

    public void setQualityRFile(String qualityRFile) {
        this.qualityRFile = qualityRFile;
    }

    public double getQualityThreshold() {
        return qualityThreshold;
    }

    public void setQualityThreshold(double qualityThreshold) {
        this.qualityThreshold = qualityThreshold;
    }

    public boolean isBaselineUse( ) 
    {
        return baselineUse;
    }

    public void setBaselineUse( boolean baselineUse ) 
    {
        this.baselineUse = baselineUse;
    }

    public int getBaselineNR( ) 
    {
        return baselineNR;
    }

    public void setBaselineNR( int baselineNR ) 
    {
        this.baselineNR = baselineNR;
    }

    public boolean isHLSVD64WaterRemovalUse( ) 
    {
        return hlsvd64WaterRemovalUse;
    }

    public void setHLSVD64WaterRemovalUse( boolean isHLSVD64WaterRemovalUse ) 
    {
        this.hlsvd64WaterRemovalUse = isHLSVD64WaterRemovalUse;
    }

    public double getHLSVD64WaterRemovalMin( ) 
    {
        return hlsvd64WaterRemovalMin;
    }

    public void setHLSVD64WaterRemovalMin( double waterRemovalMin ) 
    {
        this.hlsvd64WaterRemovalMin = waterRemovalMin;
    }

    public double getHLSVD64WaterRemovalMax( ) 
    {
        return hlsvd64WaterRemovalMax;
    }

    public void setHLSVD64WaterRemovalMax( double hlsvd64waterRemovalMax ) 
    {
        this.hlsvd64WaterRemovalMax = hlsvd64waterRemovalMax;
    }

    public int getHLSVD64WaterRemovalNumberOfPeaks( ) 
    {
        return hlsvd64WaterRemovalNumberOfPeaks;
    }

    public void setHLSVD64WaterRemovalNumberOfPeaks( int hlsvd64WaterRemovalNumberOfPeaks ) 
    {
        this.hlsvd64WaterRemovalNumberOfPeaks = hlsvd64WaterRemovalNumberOfPeaks;
    }

    // The same for HLSVDPro
    
        public boolean isHLSVDProWaterRemovalUse( ) 
    {
        return hlsvdProWaterRemovalUse;
    }

    public void setHLSVDProWaterRemovalUse( boolean isHLSVD64WaterRemovalUse ) 
    {
        this.hlsvdProWaterRemovalUse = isHLSVD64WaterRemovalUse;
    }

    public double getHLSVDProWaterRemovalMin( ) 
    {
        return hlsvdProWaterRemovalMin;
    }

    public void setHLSVDProWaterRemovalMin( double waterRemovalMin ) 
    {
        this.hlsvdProWaterRemovalMin = waterRemovalMin;
    }

    public double getHLSVDProWaterRemovalMax( ) 
    {
        return hlsvdProWaterRemovalMax;
    }

    public void setHLSVDProWaterRemovalMax( double hlsvd64waterRemovalMax ) 
    {
        this.hlsvdProWaterRemovalMax = hlsvd64waterRemovalMax;
    }

    public int getHLSVDProWaterRemovalNumberOfPeaks( ) 
    {
        return hlsvdProWaterRemovalNumberOfPeaks;
    }

    public void setHLSVDProWaterRemovalNumberOfPeaks( int hlsvd64WaterRemovalNumberOfPeaks ) 
    {
        this.hlsvdProWaterRemovalNumberOfPeaks = hlsvd64WaterRemovalNumberOfPeaks;
    }
    
    
    public void setTDFDFitWaterModelName( String TDFDFitWaterModelName )
    {
        this.tdfdfitWaterModelName = TDFDFitWaterModelName;
    }
    
    public String getTDFDFitWaterModelName( )
    {
        return this.tdfdfitWaterModelName;
    }
    
    public void setTDFDFitWaterRemovalUse( boolean useTDFDFitWaterRemoval )
    {
        isTDFDFitWaterRemovalUse = useTDFDFitWaterRemoval;
    }
    
    public boolean isTDFDFitWaterRemovalUse( )
    {
        return isTDFDFitWaterRemovalUse;
    }
            
    
    public boolean isAutoPhasingUse() {
        return autoPhasingUse;
    }

    public void setAutoPhasingUse(boolean autoPhasingUse) {
        this.autoPhasingUse = autoPhasingUse;
    }

    public double getAutoPhasingMinPPM() {
        return autoPhasingMinPPM;
    }

    public void setAutoPhasingMinPPM(double autoPhasingMinPPm) {
        this.autoPhasingMinPPM = autoPhasingMinPPm;
    }

    public double getAutoPhasingMaxPPM() {
        return autoPhasingMaxPPM;
    }

    public void setAutoPhasingMaxPPM(double autoPhasingMaxPPm) {
        this.autoPhasingMaxPPM = autoPhasingMaxPPm;
    }

    public boolean isFreqShiftUse() {
        return freqShiftUse;
    }

    public void setFreqShiftUse(boolean freqShiftUse) {
        this.freqShiftUse = freqShiftUse;
    }

    public ArrayList<ReferencePeak> getFreqShiftPeaks() {
        return freqShiftPeaks;
    }

    public void setFreqShiftPeaks(ArrayList<ReferencePeak> freqShiftPeaks) {
        this.freqShiftPeaks = freqShiftPeaks;
    }

    public double getFreqShiftMinPPM() {
        return freqShiftMinPPM;
    }

    public void setFreqShiftMinPPM(double freqShiftMinPPM) {
        this.freqShiftMinPPM = freqShiftMinPPM;
    }

    public double getFreqShiftMaxPPM() {
        return freqShiftMaxPPM;
    }

    public void setFreqShiftMaxPPM(double freqShiftMaxPPM) {
        this.freqShiftMaxPPM = freqShiftMaxPPM;
    }

    public double getFreqShiftSNR() {
        return freqShiftSNR;
    }

    public void setFreqShiftSNR(double freqShiftSNR) {
        this.freqShiftSNR = freqShiftSNR;
    }

    public double getFreqShiftMaxLocalShift() {
        return freqShiftMaxLocalShift;
    }

    public void setFreqShiftMaxLocalShift(double freqShiftMaxLocalShift) {
        this.freqShiftMaxLocalShift = freqShiftMaxLocalShift;
    }

    public boolean isZeroFillingUse() {
        return zeroFillingUse;
    }

    public void setZeroFillingUse(boolean zeroFillingUse) {
        this.zeroFillingUse = zeroFillingUse;
    }

    
     /**
     * Return whether amplitude offset removal has to occur in TD.
     * @return 
     */
    public boolean isUseRemoveAmplitudeOffsetFD( ) 
    {
        return isUseAmplitudeOffsetRemovalUseFD;
    }

    public double getMinFDOffsetRange( ) 
    {
        return minRangeFDOffsetRemoval;
    }

    public double getMaxFDOffsetRange( ) 
    {
        return maxRangeFDOffsetRemoval;
    }

    public void setMinFDOffsetRange( double minRangeFDOffsetRemoval ) 
    {
        this.minRangeFDOffsetRemoval = minRangeFDOffsetRemoval;
    }

    public void setMaxFDOffsetRange( double maxRangeFDOffsetRemoval ) 
    {
       this.maxRangeFDOffsetRemoval = maxRangeFDOffsetRemoval; 
    }
    
    
    /**
     * Return whether amplitude offset removal has to occur in FD.
     * @return 
     */
    public boolean isUseRemoveAmplitudeOffsetTD( ) 
    {
        return isUseAmplitudeOffsetRemovalUseTD;
    }

    public double getMinTDOffsetRange( ) 
    {
        return minRangeTDOffsetRemoval;
    }

    public double getMaxTDOffsetRange( ) 
    {
       return maxRangeTDOffsetRemoval; 
    }
    
    public void setMinTDOffsetRange( double minRangeTDOffsetRemoval ) 
    {
        this.minRangeTDOffsetRemoval = minRangeTDOffsetRemoval;
    }

    public void setMaxTDOffsetRange( double maxRangeTDOffsetRemoval ) 
    {
       this.maxRangeTDOffsetRemoval = maxRangeTDOffsetRemoval; 
    }

    void setUseRemoveAmplitudeOffsetTD( boolean isUseAmplitudeOffsetRemovalUseTD ) 
    {
        this.isUseAmplitudeOffsetRemovalUseTD =  isUseAmplitudeOffsetRemovalUseTD;
    }

    void setUseRemoveAmplitudeOffsetFD( boolean isUseAmplitudeOffsetRemovalUseFD ) 
    {
          this.isUseAmplitudeOffsetRemovalUseFD =  isUseAmplitudeOffsetRemovalUseFD;
    }
    
}
