/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package org.spectrim.settings.processing;

import java.util.ArrayList;
import org.spectrim.io.dicom.DicomHeader;
import org.spectrim.io.dicom.DicomHeaderLine;
import org.spectrim.model.GlobalModel;
import org.spectrim.settings.ProcessingSettings;
import org.spectrim.spectra.Spectra;

/**
 *
 * @author nunopbarros
 */
public class SeriesFilter {

    private String name = "";
    private String seriesDescription = "";
    private double TE = 0;
    private double TR = 0;
    private double B0 = 0;
    private String nucleus = "1H";
    private double bandwidth = 0;
    private String manufacturer = "";
    private String modelName = "";
    private double TETolerance = 1; //ms
    private double TRTolerance = 5; //ms
    private int seriesDescriptionComparisonMode = 1;
    private double bandwidthTolerance = 50; //Hz
    private double B0Tolerance = 0.3; //T
    private boolean seriesDescriptionActive = false;
    private boolean TEActive = false;
    private boolean TRActive = false;
    private boolean B0Active = false;
    private boolean nucleusActive = false;
    private boolean bandwidthActive = false;
    private boolean manufacturerActive = false;
    private boolean modelNameActive = false;

    public static final int CONTAINS_STRING = 0;
    public static final int MATCHES_STRING = 1;

    public SeriesFilter() {
    }

    public SeriesFilter(String txt) {
        String[] parts = txt.split(ProcessingSettings.separator1);
        name = parts[0];
        seriesDescription = parts[1];
        TE = Double.valueOf(parts[2]);
        TR = Double.valueOf(parts[3]);
        B0 = Double.valueOf(parts[4]);
        nucleus = parts[5];
        bandwidth = Double.valueOf(parts[6]);
        manufacturer = parts[7];
        modelName = parts[8];
        TETolerance = Double.valueOf(parts[9]);
        TRTolerance = Double.valueOf(parts[10]);
        bandwidthTolerance = Double.valueOf(parts[11]);
        B0Tolerance = Double.valueOf(parts[12]);
        seriesDescriptionActive = Integer.valueOf(parts[13]) == 1;
        TEActive = Integer.valueOf(parts[14]) == 1;
        TRActive = Integer.valueOf(parts[15]) == 1;
        B0Active = Integer.valueOf(parts[16]) == 1;
        nucleusActive = Integer.valueOf(parts[17]) == 1;
        bandwidthActive = Integer.valueOf(parts[18]) == 1;
        manufacturerActive = Integer.valueOf(parts[19]) == 1;
        modelNameActive = Integer.valueOf(parts[20]) == 1;
        seriesDescriptionComparisonMode = Integer.valueOf(parts[21]);
    }

    @Override
    public String toString() {
        String txt = name + ProcessingSettings.separator1;
        txt += seriesDescription + ProcessingSettings.separator1;
        txt += TE + ProcessingSettings.separator1;
        txt += TR + ProcessingSettings.separator1;
        txt += B0 + ProcessingSettings.separator1;
        txt += nucleus + ProcessingSettings.separator1;
        txt += bandwidth + ProcessingSettings.separator1;
        txt += manufacturer + ProcessingSettings.separator1;
        txt += modelName + ProcessingSettings.separator1;
        txt += TETolerance + ProcessingSettings.separator1;
        txt += TRTolerance + ProcessingSettings.separator1;
        txt += bandwidthTolerance + ProcessingSettings.separator1;
        txt += B0Tolerance + ProcessingSettings.separator1;
        txt += (seriesDescriptionActive ? 1 : 0) + ProcessingSettings.separator1;
        txt += (TEActive ? 1 : 0) + ProcessingSettings.separator1;
        txt += (TRActive ? 1 : 0) + ProcessingSettings.separator1;
        txt += (B0Active ? 1 : 0) + ProcessingSettings.separator1;
        txt += (nucleusActive ? 1 : 0) + ProcessingSettings.separator1;
        txt += (bandwidthActive ? 1 : 0) + ProcessingSettings.separator1;
        txt += (manufacturerActive ? 1 : 0) + ProcessingSettings.separator1;
        txt += (modelNameActive ? 1 : 0) + ProcessingSettings.separator1;
        txt += seriesDescriptionComparisonMode;
        return txt;
    }

    public SeriesFilter clone() {
        return new SeriesFilter(this.toString());
    }

    public void setName(String name) {
        this.name = name;
    }

    public void setTE(double TE) {
        this.TE = TE;
    }

    public void setTR(double TR) {
        this.TR = TR;
    }

    public void setNucleus(String nucleus) {
        this.nucleus = nucleus;
    }

    public void setBandwidth(double bandwidth) {
        this.bandwidth = bandwidth;
    }

    public void setB0(double B0) {
        this.B0 = B0;
    }

    public void setSeriesDescription(String seriesDescription) {
        this.seriesDescription = seriesDescription;
    }

    public void setVendor(String vendor) {
        this.manufacturer = vendor;
    }

    public void setModelName(String model) {
        this.modelName = model;
    }

    public void setManufacturer(String manufacturer) {
        this.manufacturer = manufacturer;
    }

    public void setTETolerance(double TETolerance) {
        this.TETolerance = TETolerance;
    }

    public void setTRTolerance(double TRTolerance) {
        this.TRTolerance = TRTolerance;
    }

    public void setBandwidthTolerance(double bandwidthTolerance) {
        this.bandwidthTolerance = bandwidthTolerance;
    }

    public void setB0Tolerance(double B0Tolerance) {
        this.B0Tolerance = B0Tolerance;
    }

    public void setSeriesDescriptionActive(boolean seriesDescriptionActive) {
        this.seriesDescriptionActive = seriesDescriptionActive;
    }

    public void setTEActive(boolean TEActive) {
        this.TEActive = TEActive;
    }

    public void setTRActive(boolean TRActive) {
        this.TRActive = TRActive;
    }

    public void setB0Active(boolean B0Active) {
        this.B0Active = B0Active;
    }

    public void setNucleusActive(boolean nucleusActive) {
        this.nucleusActive = nucleusActive;
    }

    public void setBandwidthActive(boolean bandwidthActive) {
        this.bandwidthActive = bandwidthActive;
    }

    public void setManufacturerActive(boolean manufacturerActive) {
        this.manufacturerActive = manufacturerActive;
    }

    public void setModelNameActive(boolean modelNameActive) {
        this.modelNameActive = modelNameActive;
    }

    public String getName() {
        return name;
    }

    public String getSeriesDescription() {
        return seriesDescription;
    }

    public double getTE() {
        return TE;
    }

    public double getTR() {
        return TR;
    }

    public double getB0() {
        return B0;
    }

    public String getNucleus() {
        return nucleus;
    }

    public double getBandwidth() {
        return bandwidth;
    }

    public String getManufacturer() {
        return manufacturer;
    }

    public String getModelName() {
        return modelName;
    }

    public double getTETolerance() {
        return TETolerance;
    }

    public double getTRTolerance() {
        return TRTolerance;
    }

    public double getBandwidthTolerance() {
        return bandwidthTolerance;
    }

    public double getB0Tolerance() {
        return B0Tolerance;
    }

    public boolean isSeriesDescriptionActive() {
        return seriesDescriptionActive;
    }

    public int getSeriesDescriptionComparisonMode() {
        return seriesDescriptionComparisonMode;
    }

    public void setSeriesDescriptionComparisonMode(int seriesDescriptionComparisonMode) {
        this.seriesDescriptionComparisonMode = seriesDescriptionComparisonMode;
    }

    public boolean isTEActive() {
        return TEActive;
    }

    public boolean isTRActive() {
        return TRActive;
    }

    public boolean isB0Active() {
        return B0Active;
    }

    public boolean isNucleusActive() {
        return nucleusActive;
    }

    public boolean isBandwidthActive() {
        return bandwidthActive;
    }

    public boolean isManufacturerActive() {
        return manufacturerActive;
    }

    public boolean isModelNameActive() {
        return modelNameActive;
    }

    public boolean evaluate(GlobalModel model) {
        if (TEActive && Math.abs(model.getTE() - TE) > TETolerance) {
            return false;
        }
        if (TRActive && Math.abs(model.getTR() - TR) > TRTolerance) {
            return false;
        }
        if (bandwidthActive && Math.abs(model.getBandwidth() - bandwidth) > bandwidthTolerance) {
            return false;
        }
        if (B0Active && Math.abs(model.getB0() - B0) > B0Tolerance) {
            return false;
        }
        if (seriesDescriptionActive
                && ((seriesDescriptionComparisonMode == MATCHES_STRING && !model.getSeriesDescription().trim().toLowerCase().matches(seriesDescription.trim().toLowerCase()))
                || (seriesDescriptionComparisonMode == CONTAINS_STRING && !model.getSeriesDescription().trim().toLowerCase().contains(seriesDescription.trim().toLowerCase())))) {
            return false;
        }
        if (manufacturerActive && !model.getManufacturer().trim().toLowerCase().matches(manufacturer.trim().toLowerCase())) {
            return false;
        }
        if (modelNameActive && !model.getModelName().trim().toLowerCase().matches(modelName.trim().toLowerCase())) {
            return false;
        }
        if (nucleusActive && !model.getNucleus().trim().toLowerCase().matches(nucleus.trim().toLowerCase())) {
            return false;
        }
        return true;
    }

    public boolean evaluate(DicomHeader header) {
        DicomHeaderLine lineaux = (DicomHeaderLine) header.get("(0008,0016)");
        int fileType = 0;
        if (lineaux != null) {
            if (lineaux.Value.contentEquals("1.3.12.2.1107.5.9.1")) {
                fileType = 1; //Siemens Format              
            } else if (lineaux.Value.contentEquals("1.2.840.10008.5.1.4.1.1.4.2")) {
                fileType = 2; //Standard Format
            }
            if (fileType != 0) {
                String seriesDescriptionVal = readHeaderValue(header, "(0008,103E)");
                String manufacturerVal = readHeaderValue(header, "(0008,0070)");
                String modelNameVal = readHeaderValue(header, "(0008,1090)");
                String nucleusVal = "";
                double TEVal = 0;
                double TRVal = 0;
                double bandwidthVal = 0;
                double b0Val = 0;

                if (fileType == 1) {
                    b0Val = Double.valueOf(readHeaderValue(header, "sProtConsistencyInfo.flNominalB0"));
                    if (b0Val == 0) {
                        b0Val = Double.valueOf(readHeaderValue(header, "sTXSPEC.flKDynMagnitudeMax"));
                    }
                    TEVal = Double.valueOf(readHeaderValue(header, "alTE[0]")) / 1000;
                    TRVal = Double.valueOf(readHeaderValue(header, "alTR[0]")) / 1000;

                    if (readHeaderValue(header, "sCoilSelectMeas.aTxCoilSelectData[0].tNucleus").matches("1H")
                            || readHeaderValue(header, "sCoilSelectMeas.aRxCoilSelectData[0].tNucleus").matches("1H")
                            || readHeaderValue(header, "sRXSPEC.asNucleusInfo[0].tNucleus").matches("1H")
                            || readHeaderValue(header, "asCoilSelectMeas[0].tNucleus").matches("1H")
                            || readHeaderValue(header, "sTXSPEC.asNucleusInfo[0].tNucleus").matches("1H")) {
                        nucleusVal = "1H";
                    };//Include other nucleus

                    double dwellTime = Double.valueOf(readHeaderValue(header, "sRXSPEC.alDwellTime[0]")); //nanoseconds
                    String removeOversampling = readHeaderValue(header, "sSpecPara.ucRemoveOversampling");
                    if (removeOversampling.matches("0x1")) {
                        bandwidthVal = 1000000000 / (dwellTime * 2); //Hz
                    } else {
                        bandwidthVal = 1000000000 / (dwellTime); //Hz
                    }

                } else if (fileType == 2) {
                    b0Val = Double.valueOf(readHeaderValue(header, "(0018,0087)"));
                    if (b0Val == 0) {
                        b0Val = Double.valueOf(readHeaderValue(header, "(2001,1085)"));
                    }
                    TEVal = Double.valueOf(readHeaderValue(header, "(0018,9114)->(0018,9082)"));
                    TRVal = Double.valueOf(readHeaderValue(header, "(0018,9112)->(0018,0080)"));
                    nucleusVal = readHeaderValue(header, "(0018,9100)");
                    bandwidthVal = Double.valueOf(readHeaderValue(header, "(0018,9052)")); // spectral width in Hz
                }

                if (TEActive && Math.abs(TEVal - TE) > TETolerance) {
                    return false;
                }
                if (TRActive && Math.abs(TRVal - TR) > TRTolerance) {
                    return false;
                }
                if (bandwidthActive && Math.abs(bandwidthVal - bandwidth) > bandwidthTolerance) {
                    return false;
                }
                if (B0Active && Math.abs(b0Val - B0) > B0Tolerance) {
                    return false;
                }
                if (seriesDescriptionActive
                        && ((seriesDescriptionComparisonMode == MATCHES_STRING && !seriesDescriptionVal.trim().toLowerCase().matches(seriesDescription.trim().toLowerCase()))
                        || (seriesDescriptionComparisonMode == CONTAINS_STRING && !seriesDescriptionVal.trim().toLowerCase().contains(seriesDescription.trim().toLowerCase())))) {
                    return false;
                }
                if (manufacturerActive && !manufacturerVal.trim().toLowerCase().matches(manufacturer.trim().toLowerCase())) {
                    return false;
                }
                if (modelNameActive && !modelNameVal.trim().toLowerCase().matches(modelName.trim().toLowerCase())) {
                    return false;
                }
                if (nucleusActive && !nucleusVal.trim().toLowerCase().matches(nucleus.trim().toLowerCase())) {
                    return false;
                }
                return true;
            }
        }
        return false;
    }

    protected String readHeaderValue(DicomHeader header, String tag) {
        DicomHeaderLine headerLine;
        headerLine = (DicomHeaderLine) header.get(tag);
        if (headerLine != null) {
            String val = headerLine.Value.replace("\"", "");
            return val;
        } else {
            // System.out.println("No value found for tag: " + tag);
            return "0";
        }
    }
}
