/*
 * File: $Id: ApodizationParameterDefinitionControl.java,v 1.1.1.1 2006/04/19 09:38:51 hans Exp $
 * History
 * Date: 16.05.2003, 9:54 AM  Authors: Hans Slotboom  
 *
 * Changes:
 *
 * Copyright (C) 2003 by
 * 
 * Inselspital - Department of Neuroradiology
 * CH 3010 Bern Switzerland
 */


package org.spectrim.tdfdfit;

import java.awt.BorderLayout;
import java.awt.Point;
import java.awt.Dimension;

import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

import java.util.LinkedList;
import java.util.Iterator;

import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JButton;
import javax.swing.BoxLayout;
import javax.swing.Box;
import javax.swing.BorderFactory;
import javax.swing.JRadioButton;
import javax.swing.ButtonGroup;

import org.spectrim.lib.SpinBox;


/**
 *
 * Class allowing the graphical definition of the Lorentzian and Gaussian 
 * apodization time constants for FID and halve echo signals.
 * 
 * @author  hans
 */

public class ApodizationParameterDefinitionControl extends JFrame implements ActionListener
{
    
    /**
     * Creates a new instance of ApodizationParameterDefinitionControl. 
     */
    public ApodizationParameterDefinitionControl( String title, JFrame parent, int displayMode  )  
    {
        super( );
        m_parent = parent;
        setTitle( title );
        
        m_displayMode = displayMode;
        
        // The content panel contains all subpanels
        m_contentPanel = new JPanel( );
        m_contentPanel.setLayout( new BorderLayout( ) );
        
        // Define the control panel with its inner life
        m_controlPanel = new JPanel( );
        m_controlPanel.setLayout( new BoxLayout( m_controlPanel, BoxLayout.Y_AXIS ) );
        m_controlPanel.setBorder( BorderFactory.createEmptyBorder( 10, 10, 10, 10 ) );
        m_controlPanel.setPreferredSize( new Dimension( 430, 60 ) );
        
        
        double[] stepsize = { 10.0, 100.0, 1000.0  };
        m_lorentzianTimeContstantDefiner = new SpinBox( "Lorentzian time constant", 3, stepsize, 0.0, 10000.0, "msec", 300.0, 5, "0" );
        m_lorentzianTimeContstantDefiner.setMaximumAndPreferredTitleLabelSize( new Dimension( 140, 22 ) );
        m_lorentzianTimeContstantDefiner.setMaximumAndPreferredTextFieldSize( new Dimension( 140,22 ) );
        m_lorentzianTimeContstantDefiner.addActionListener( this );
        
        m_controlPanel.add( m_lorentzianTimeContstantDefiner );
        
        m_gaussianTimeContstantDefiner = new SpinBox( "Gaussian time constant", 3, stepsize, 0.0, 10000.0, "msec", 0.0, 5, "0" );
        m_gaussianTimeContstantDefiner.setMaximumAndPreferredTitleLabelSize( new Dimension( 140, 22 ) );
        m_gaussianTimeContstantDefiner.setMaximumAndPreferredTextFieldSize( new Dimension( 140, 22 ) );
        m_gaussianTimeContstantDefiner.addActionListener( this );
        
        m_controlPanel.add( m_gaussianTimeContstantDefiner );
        
        m_contentPanel.add( m_controlPanel, BorderLayout.NORTH );
        if ( m_displayMode == MULTI_RESPONSE_DISPLAY )
        {
            createInstanceSpecificPanel( );
        
            m_contentPanel.add( m_performActionToCurrentOnlyPanel, BorderLayout.CENTER );
        }
        
        // Define the selection Panel with the Apply and Cancel button
        m_applyButton = new JButton( "Apply" );
        m_applyButton.addActionListener( this );
        m_applyButton.setToolTipText( "Applies the selected zerofilling factor to the currently loaded dataset" );
        m_cancelButton = new JButton( "Cancel" );
        m_cancelButton.addActionListener( this );
        m_cancelButton.setToolTipText( "Cancels this operation without applying the selected zerofilling factor" );
        
        m_selectionPanel = new JPanel( );
        
        m_selectionPanel.setLayout( new BoxLayout( m_selectionPanel, BoxLayout.X_AXIS ) );
        m_selectionPanel.setBorder( BorderFactory.createEmptyBorder( 5, 5, 5, 5 ) );
        m_selectionPanel.add( Box.createGlue( ) );
        m_selectionPanel.add( m_applyButton );
        m_selectionPanel.add( Box.createGlue( ) );
        m_selectionPanel.add( m_cancelButton );
        m_selectionPanel.add( Box.createGlue( ) );
        
        m_contentPanel.add( m_selectionPanel, BorderLayout.SOUTH );
        
        // add window listener necessary to handle the window closing 
        // event properly
        addWindowListener( 
            new WindowAdapter( )
            {
                @Override
                public void windowClosing( WindowEvent event )
                {
                   dispose( );
                }
            }
        );
        
        
        setContentPane( m_contentPanel );
        
        pack( );
        // Put the Dialog nicely in the middle of its parent
        Point parentLoc = m_parent.getLocation( );
        Point centerParent = new Point( m_parent.getX( ) + ( m_parent.getWidth( ) / 2 ) - this.getWidth( ) / 2,
                                        m_parent.getY( ) + ( m_parent.getHeight( ) / 2 ) - this.getHeight( ) / 2 );
        setLocation( centerParent );
        
        setVisible( true );
    }
    
    /**
     * Is called whenever a registered component raises an ActionEvent.
     * @param e
     */
    @Override
    public void actionPerformed( ActionEvent e ) 
    {
        
        Object sender = e.getSource( );
        
        if ( sender == m_gaussianTimeContstantDefiner )
        {
            Iterator it = m_listeners.iterator( );
            while( it.hasNext( ) )
            {
                ( ( ActionListener )it.next( ) ).actionPerformed( new ActionEvent( this, GAUSSIAN_TIME_CONSTANT_CHANGE_EVENT, "" ) );
            }
        }

        if ( sender == m_lorentzianTimeContstantDefiner )
        {
            Iterator it = m_listeners.iterator( );
            while( it.hasNext( ) )
            {
                ( ( ActionListener )it.next( ) ).actionPerformed( new ActionEvent( this, LORENTZIAN_TIME_CONSTANT_CHANGED_EVENT, "" ) );
            }
        }
        
        // handle responses on cancel button pressed event
        if ( sender == m_cancelButton )
        {
            Iterator it = m_listeners.iterator( );
             while( it.hasNext( ) )
             {
                ( ( ActionListener )it.next( ) ).actionPerformed( new ActionEvent( this, CANCEL_EVENT, "" ) );
             }
             dispose( );
        }
        
        // handle responses on apply button pressed event
        if ( sender == m_applyButton )
        {
            Iterator it = m_listeners.iterator( );
             while( it.hasNext( ) )
             {
                ( ( ActionListener )it.next( ) ).actionPerformed( new ActionEvent( this, ACCEPT_EVENT, "" ) );
             }
             dispose( );
        }
    }
    
    
    /**
     * Returns the selected Lorentzian TC.
     * @return 
     */
    public float getSelectedLorentzianTC( )
    {
        return (float)m_lorentzianTimeContstantDefiner.getTextFieldValue( );
    }
     
    /**
     * Returns the selected Lorentzian TC.
     * @return 
     */
    public float getSelectedGaussianTC( )
    {
        return (float)m_gaussianTimeContstantDefiner.getTextFieldValue( );
    }
    
    /**
     * Adds an ActionListener to the list of action listeners who are notified
     * when something happens within this object.
     * @param e
     */
    public void addActionListener( ActionListener e )
    {
	m_listeners.add( e );
    }

    /**
     * Creates the panel that allows the user to perform the operation to 
     * the selected instance only.
     */
    public void createInstanceSpecificPanel( )
    {
        m_performActionToCurrentOnlyPanel =  new JPanel( );
        
        m_performActionToCurrentOnlyPanel.setLayout( new BoxLayout( m_performActionToCurrentOnlyPanel, BoxLayout.X_AXIS ) );
        
        m_applyToSelectedInstanceOnly = new JRadioButton( "Selected instance only" );
        m_applyToSelectedInstanceOnly.setSelected( true );
        
        m_applyToAllInstances = new JRadioButton( "All instances" );
        m_applyToButtonGroup.add( m_applyToAllInstances );
        m_applyToButtonGroup.add( m_applyToSelectedInstanceOnly );
        m_performActionToCurrentOnlyPanel.add( Box.createHorizontalGlue( ) );
        m_performActionToCurrentOnlyPanel.add( m_applyToSelectedInstanceOnly );
        m_applyToSelectedInstanceOnly.setToolTipText( "Applies frequency shift only to the selected instance" );
        m_performActionToCurrentOnlyPanel.add( Box.createHorizontalGlue( ) );
        m_performActionToCurrentOnlyPanel.add( m_applyToAllInstances );
        m_applyToAllInstances.setToolTipText( "Applies frequency shift to all instances of the spectrum series" );
        m_performActionToCurrentOnlyPanel.add( Box.createHorizontalGlue( ) );
    }
    
    
    // Attributes
    JFrame m_parent;
    JPanel m_contentPanel;
    
    JPanel m_controlPanel;
    
    // Listeners to changes in this class
    private final LinkedList<ActionListener> m_listeners = new LinkedList<>( );

    
    JPanel  m_selectionPanel;    // Contains Apply/Cancel buttons
    JButton m_applyButton;       // When pressed, sends the apply notification to the registered listeners
    JButton m_cancelButton;      // When pressed, sends the cansel notification to the registered listeners
    
    SpinBox m_lorentzianTimeContstantDefiner;
    SpinBox m_gaussianTimeContstantDefiner;
    
    // Events that can be raised by this
    
    // Possible Events
    public final static int CANCEL_EVENT = 0;
    public final static int ACCEPT_EVENT = 1;
    
    public final static int LORENTZIAN_TIME_CONSTANT_CHANGED_EVENT = 2;
    public final static int GAUSSIAN_TIME_CONSTANT_CHANGE_EVENT = 3;
    
    
    private JPanel m_performActionToCurrentOnlyPanel;
    private JRadioButton m_applyToSelectedInstanceOnly;
    private JRadioButton m_applyToAllInstances;
    private final ButtonGroup m_applyToButtonGroup = new ButtonGroup( );

    public final static int SINGLE_RESPONSE_DISPLAY = 0;
    public final static int MULTI_RESPONSE_DISPLAY = 1;

    private int m_displayMode = SINGLE_RESPONSE_DISPLAY;
    
    // For proper serialization
    private static final long serialVersionUID = 1487495898889393L;
}
