/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package org.spectrim.tdfdfit;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.geom.Point2D;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.Vector;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.apache.commons.math3.analysis.interpolation.SplineInterpolator;
import org.apache.commons.math3.analysis.polynomials.PolynomialSplineFunction;
import org.spectrim.signalprocessing.FFT;
import org.spectrim.spectra.Spectrum;

/**
 *
 * @author johan
 */
public class BaselineRemoverSwingWorker extends javax.swing.SwingWorker<Integer, String> 
{

    // Declaration of class member variables.
    public final static int             IS_PROCESSING = 0;
    public final static int             IS_PERFORMED  = 1;
    public final static String          IS_PERFORMED_CMD  = "Baseline correction(s) performed.";
    
    private String                      m_key;
    private int                         m_threadID;
    private int                         m_samples;
    private Spectrum                    m_spec;
    private Vector<Point2D.Double>      m_splineNodes;
    private float[]                     m_xInterpolationValues;
    private long                        startComputationTime;
    private int                         n_cores;
    private float[]                     m_phaseDisplayed;
    private float[]                     m_correctedAbsorption;
    private float[]                     m_correctedDispersion;
    private float[]                     m_magnitudeDisplayed;
    private LinkedList<ActionListener>  m_listeners = new  LinkedList<>( );
    private double[]                    m_xAxis;
    private double[]                    m_y;
    
    
    public BaselineRemoverSwingWorker( int threadID, String key, int samples, Spectrum spec, 
                                       Vector<Point2D.Double> splineNodes,
                                       float[] xInterpolationValues )
    {
        m_key = key;
        m_threadID = threadID;
        m_samples = samples;
        m_spec = spec;
        m_splineNodes = splineNodes;
        m_xInterpolationValues = new float[xInterpolationValues.length];
        System.arraycopy( xInterpolationValues, 0, m_xInterpolationValues, 0, xInterpolationValues.length );
        
        // Magic code to keep the multi threadedness alive! Strange!!!
        try 
        { 
            Thread.sleep( 30 );
        } 
        catch ( InterruptedException ex ) 
        {
            Logger.getLogger( BaselineRemoverSwingWorker.class.getName( ) ).log( Level.SEVERE, null, ex );
        }
    }
    
    
    /**
     * Adds an ActionListener to the list of action listeners who are notified
     * when something happens in this component.
     * @param e
     */
    public void addActionListener( ActionListener e ) 
    {
        m_listeners.add(e);
    }
    
    /**
     * Notifies registered ActionListeners.
     * @param eventType
     * @param actionCommand
     */
    private void notifyActionListeners( int eventType, String actionCommand )
    {
        Iterator it = m_listeners.iterator( );
        while( it.hasNext( ) )
        {
            ( ( ActionListener )it.next( ) ).actionPerformed( new ActionEvent( this, eventType, actionCommand ) );
        }
    }
    
    @Override
    protected Integer doInBackground( ) throws Exception {
        
        int returnValue = baselineRemovalExecuterKernel( );
        return returnValue;
    }
    
    @Override
    protected void done( ) 
    { 
        notifyActionListeners( IS_PERFORMED, IS_PERFORMED_CMD );  
   }
    /**
     * Single threaded baseline removal code.
     * @return 
     */
    public int baselineRemovalExecuterKernel( )
    {
   
        // Determine the original phase of the signal 
        m_phaseDisplayed = new float[ m_samples ];
        m_correctedAbsorption = new float[ m_samples ];
        m_correctedDispersion = new float[ m_samples ];
        m_magnitudeDisplayed = new float[ m_samples ];
        
        // m_xInterpolationValues = new float[m_samples];
 
        
        for ( int i = 0; i < m_spec.getAbsorptionFD( ).length; i++ )
        {
            m_phaseDisplayed[i]     = (float)Math.atan2( m_spec.getAbsorptionFD( )[i], m_spec.getDispersionFD( )[i] );
            m_magnitudeDisplayed[i] = (float)Math.sqrt( m_spec.getAbsorptionFD( )[i] * m_spec.getAbsorptionFD( )[i] + m_spec.getDispersionFD( )[i] * m_spec.getDispersionFD( )[i] );
        }
        
        Vector<Point2D.Double> currentCaseSplineNodes = new Vector<Point2D.Double>( );
                            
        for ( int i = 0; i < m_splineNodes.size( ); i++ )
        {
            float xValue;
            xValue = (float)m_splineNodes.get( i ).x;
            //float yValue = getCorrespondingYValue( m_xInterpolationValues, m_magnitudeDisplayed, xValue );
            float yValue = getCorrespondingYValue( m_xInterpolationValues, m_spec.getAbsorptionFD( ), xValue );
            
            // System.out.println( "Node#[" + i + "] -> " + " xValue node " +  xValue + " yValue node " + yValue );
            currentCaseSplineNodes.add( new Point2D.Double( xValue, yValue ) ); 
        }
        
        m_xAxis = new double[ currentCaseSplineNodes.size( ) ];
        float[]  xaxisFlt = new float[ m_samples ];
        m_y = new double[ currentCaseSplineNodes.size( ) ];

        double minX = currentCaseSplineNodes.get( 0 ).x;
        double maxX = currentCaseSplineNodes.get( currentCaseSplineNodes.size( ) - 1 ).x;
                                
        for ( int i = 0; i < m_samples; i++ )
        {
            m_xInterpolationValues[i]   = (float)( minX + ( maxX - minX ) * i / ( m_samples - 1.0 ) );
            //m_xInterpolPointsFlt[i]   = (float)m_xInterpolPoints[i];
        }

        for ( int i = 0; i < currentCaseSplineNodes.size( ); i++ )
        {   
            m_xAxis[i]    = currentCaseSplineNodes.get( i ).x;
            //xaxisFlt[i] = (float)currentCaseSplineNodes.get( i ).x;
            m_y[i]        = currentCaseSplineNodes.get( i ).y;
        }
        
        doBaselineRemoval( );
        
        return 0;
    }
    
    /**
     * Method called for baseline correction ...
     * @param xValues
     * @param yValues
     * @param xValue
     * @return 
     */
    public float getCorrespondingYValue( float[] xValues, float[] yValues, float xValue )
    {
        // float[] xaxis = m_specData.getMagnitudeCurveFDPPM( ).getXvalues( );
        int index = getIndexOfFirstMatchingValue( xValue, xValues );
        
        // do not return the value of the index itsef, but average of 3 points
        float average = yValues[index];
        
        if ( ( index > 1 ) && ( index < xValues.length - 2 ) )
        {
            average += yValues[index - 1];
            average += yValues[index + 1];
            average += yValues[index - 2];
            average += yValues[index + 2];
        }
        
        if ( index == 0 )
        {
            average += yValues[1];
            average += yValues[2];
            average += yValues[3];
            average += yValues[4];
        }            
        if ( index == xValues.length - 1 )
        {
            average += yValues[xValues.length - 2];
            average += yValues[xValues.length - 3];
            average += yValues[xValues.length - 4];
            average += yValues[xValues.length - 5];
        }
        
        if ( index == 1 )
        {
            average += yValues[0];
            average += yValues[2];
            average += yValues[3];
            average += yValues[4];
        }            
        if ( index == xValues.length - 2 )
        {
            average += yValues[xValues.length - 1];
            average += yValues[xValues.length - 3];
            average += yValues[xValues.length - 4];
            average += yValues[xValues.length - 5];
        }
        return ( average / 5 );
    }
    
    /**
     * Returns the array index of the coordinate containing the maximum value.
     * @param value
     * @param y
     * @return 
     */
    public int getIndexOfFirstMatchingValue( float value, float[] y ) 
    {
	int index = 0;
	
        float current_difference = Float.MAX_VALUE;
        
	for ( int k = 0; k < y.length; k++ ) 
        {
            float difference = Math.abs( y[k] - value );            
	    if ( difference < current_difference ) 
            {
		index = k;
                current_difference = difference;
	    }
	}
	return index;
    }
    
    public void doBaselineRemoval( ) 
    {
        
        float[] spline = new float[m_samples];

        SplineInterpolator myInterpolator = new SplineInterpolator( );
        PolynomialSplineFunction polynomialSplineFunction = myInterpolator.interpolate( m_xAxis, m_y );

        for ( int k = 0; k < m_samples; k++ )
        {
            spline[k] = (float)polynomialSplineFunction.value( m_xInterpolationValues[ k ] );
        }
                                    
        float[] difference = new float[m_samples];
        for ( int i = 0; i < m_samples; i++ )
        {
            difference[i] = m_magnitudeDisplayed[i] - spline[i];
        }
                                                           
        // Restore the signal phase on the 
        // magnitude again 
        for ( int i = 0; i < m_samples; i++ )
        {
            m_correctedAbsorption[i] = (float) ( difference[i] * Math.sin( m_phaseDisplayed[i] ) );
            m_correctedDispersion[i] = (float) ( difference[i] * Math.cos( m_phaseDisplayed[i] ) );
            
        }
                      
        float[][] vaux = FD2TD( m_correctedAbsorption, m_correctedDispersion ); 

        for ( int t = 0; t < m_samples; t++ )  
        {
            m_correctedAbsorption[t] = vaux[t][0] / m_samples;
            m_correctedDispersion[t] = vaux[t][1] / m_samples;
        }
    }
    
    public float[] getCorrectedSpectrumReal( )
    {
        return m_correctedAbsorption;
    }
    
    public float[] getCorrectedSpectrumImag( )
    {
        return m_correctedDispersion;
    }
    
    public float[][] FD2TD( float[] abs, float[] disp ) 
    {
        float[][] vaux = new float[abs.length][2];

        for ( int m = 0; m < abs.length; m++ ) 
        {
            int d = (m + 1 + abs.length / 2) % abs.length;
            
            vaux[d][0] = abs[abs.length - 1 - m];
            vaux[d][1] = disp[disp.length - 1 - m];
        }
        
        FFT fft = new FFT( );
        vaux = fft.fft_1d( vaux );

        return vaux;
    }
    
    public int getThreadID( )
    {
        return m_threadID;
    }
   
    public String getKey( )
    {
        return m_key;
    }
    
}
