/*
 * File: $Id: FitModelTreePanel.java,v 1.1.1.1 2006/04/19 09:38:49 hans Exp $
 * History
 * Date: 28.05.2003 2003, 9:10    Authors: Hans Slotboom
 * Changes:
 *
 * Copyright (C) 2003 by
 * 
 * Inselspital - Department of Neuroradiology
 * CH 3010 Bern Switzerland
 */
 

package org.spectrim.tdfdfit;

import java.awt.Dimension;
import java.awt.BorderLayout;

import java.awt.event.MouseAdapter;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseEvent;

import java.io.RandomAccessFile;

import java.util.LinkedList;
import java.util.Iterator;

import javax.swing.tree.DefaultTreeModel;
import javax.swing.JPanel;

import javax.swing.BorderFactory;
import javax.swing.JScrollPane;
import javax.swing.JMenuItem;
import javax.swing.JFrame;

import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.TreeSelectionModel;

/**
 * Class handling the fit model tree related actions.
 * @author  hans
 */
public class FitModelTreePanel extends JPanel implements ActionListener, MouseListener
{
    
    /** 
     * Creates a new instance of FitModelTreePanel. 
     * @param parent
     * @param modelSpectrum
     */
    public FitModelTreePanel( JFrame parent, TDFDFitPrototypeModelSpectrumParameters modelSpectrum )
    {
        m_parent = parent;
        m_modelSpectrum = modelSpectrum;
        
        m_fitModelTreePanel = new JPanel( );

        m_fitModelTreePanel.setLayout( new BorderLayout( ) );
        m_fitModelTreePanel.setBorder( BorderFactory.createEmptyBorder( 3, 3, 3, 3 ) );
        
        m_treeRoot = new DefaultMutableTreeNode( m_modelSpectrum.toString( ) );
        
        // create the tree model
        createTreeModel( );
        
        m_fitModelTreeView = new TDFDFitModelTreeWithPopupMenu( m_treeRoot );
        m_fitModelTreeView.setEditable( true );
        m_fitModelTreeView.setShowsRootHandles( true );

        m_fitModelTreeView.getSelectionModel( ).setSelectionMode( TreeSelectionModel.SINGLE_TREE_SELECTION );
                
        m_fitModelTreeView.addMouseListener( m_treeMouseListener );
        // The MRModelSpectrum object is interested in changes in the tree model
        m_fitModelTreeView.addActionListener( m_modelSpectrum );
        
        // FitModelTreeWithPopupMenu should listen to changes in this object
        addActionListener(  m_fitModelTreeView );
        
        // create the tree model
        createTreeModel( );
        
        m_fitModelTreePanel.add( m_fitModelTreeView,BorderLayout.CENTER );
        
        // JTree does not provide a JScrollPane automatically so we have to 
        // add it ourselves
        m_treeScrollPane = new JScrollPane( JScrollPane.VERTICAL_SCROLLBAR_ALWAYS, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED );
        m_treeScrollPane.setMinimumSize( new Dimension( 540, 200 ) );
        m_treeScrollPane.setMaximumSize( new Dimension( 540, 200 ) );
        m_treeScrollPane.setPreferredSize( new Dimension( 540, 200 ) );
        m_treeScrollPane.getViewport( ).setView( m_fitModelTreeView );
        
        m_fitModelTreePanel.add( m_treeScrollPane );
        
        add( m_fitModelTreePanel );
    }

    /**
     * Creates a TreeModel and a TreeModelListener.
     */
    private void createTreeModel( )
    {
        m_defaultTreeModel = new DefaultTreeModel( m_treeRoot );
        m_defaultTreeModel.addTreeModelListener( new  FitModelTreeListener( ) );
    }
    
    /**
     * Sets the Spectral model root node. The root is String-typed.
     * @param specModelName
     */
    public void setFitModelName( String specModelName )
    {
        m_treeRoot.setUserObject( specModelName );
    }
    
    /** 
     * Notification of registered listeners.
     */
    private void notifyActionListeners( int eventType, Object obj )
    {
        Iterator it = m_actionListeners.iterator( );
        while( it.hasNext( ) )
        {
            if ( obj != null )
            {
                ( ( ActionListener )it.next( ) ).actionPerformed( new ActionEvent( obj, eventType, "" ) );
            }
        }
    }
    
    /**
     * Is called whenever a registered class raises an ActionEvent.
     * @param e 
     */
    @Override
    public void actionPerformed( ActionEvent e ) 
    {
        if ( e.getSource( ) instanceof MRParameterSpectrumModelViewer )
        {
            if ( ( e.getID( ) == MRLineModelParametersPanel.ADD_USER_DEFINED_LINE_EVENT ) &&
                 ( e.getActionCommand( ).compareTo( "add user defined line node" ) == 0 ) )
            {
                notifyActionListeners( e.getSource( ), MRLineModelParametersPanel.ADD_USER_DEFINED_LINE_EVENT, e.getActionCommand( ) );
            }        
        }
        
        if ( e.getSource( ) instanceof MRLineModelParametersPanel )
        {
            if ( ( e.getID( ) == MRLineModelParametersPanel.ADD_LINE_EVENT ) && 
                 ( e.getActionCommand( ).compareTo( "add line node" ) == 0 ) )
            {
                notifyActionListeners( this, ADD_LINE_EVENT, e.getActionCommand( ) );
            }

            if ( ( e.getID( ) == MRLineModelParametersPanel.ADD_PARAMETRIC_PATTERN_EVENT ) &&
                 ( e.getActionCommand( ).compareTo( "add parametric pattern node" ) == 0 ) )  
            {
                notifyActionListeners( this, ADD_PARAMETRIC_PATTERN_EVENT, e.getActionCommand( ) );
            }
   
            if ( ( e.getID( ) == MRLineModelParametersPanel.ADD_NUMERIC_PATTERN_EVENT ) &&
                 ( e.getActionCommand( ).compareTo( "add numeric pattern node" ) == 0 ) )
            {
                notifyActionListeners( this, ADD_NUMERIC_PATTERN_EVENT, e.getActionCommand( ) );
            }
            
            if ( ( e.getID( ) == MRLineModelParametersPanel.ADD_NUMERIC_PATTERN_EVENT ) &&
                 ( e.getActionCommand( ).compareTo( "add numeric pattern node from bin response" ) == 0 ) )
            {
                notifyActionListeners( this, ADD_NUMERIC_PATTERN_EVENT, e.getActionCommand( ) );
            }
            
            
            
            if ( ( e.getID( ) == MRLineModelParametersPanel.ADD_METABOLITE_EVENT ) &&
                 ( e.getActionCommand( ).compareTo( "add metabolite node" ) == 0 ) )
            {
                notifyActionListeners( this, ADD_METABOLITE_EVENT, e.getActionCommand( ) );
            }
            
            if ( ( e.getID( ) == MRLineModelParametersPanel.REMOVE_SPECTRAL_COMPONENT_EVENT ) && 
                 ( e.getActionCommand( ).compareTo( "delete node" ) == 0 ) )
            {
                notifyActionListeners( this, REMOVE_SPECTRAL_COMPONENT_EVENT, e.getActionCommand( ) );
            }
        }
    } 
    
    /**
     * 
     * Method called to notify registered classes of certain events.
     * 
     * @param obj
     * @param eventType
     * @param command 
     */
    private void notifyActionListeners( Object obj, int eventType,  String command )
    {
        Iterator it = m_actionListeners.iterator( );
        while( it.hasNext( ) )
        {
            ( ( ActionListener )it.next( ) ).actionPerformed( new ActionEvent( obj, eventType, command ) );
        }
    }
    
    /**
     * Called if a mouse button is clicked.
     * @param e 
     */
    @Override
    public void mouseClicked( MouseEvent e ) 
    {
    }
    
    /**
     * Called if a mouse button entered the JPanel.
     * @param e 
     */
    @Override
    public void mouseEntered( MouseEvent e )
    {
    }
    
    /**
     * Called if a mouse button exited the JPanel.
     * 
     * @param e 
     */
    @Override
    public void mouseExited( MouseEvent e ) 
    {
    }
    
    /**
     * Called if a mouse button is pressed in the JPanel.
     * 
     * @param e 
     */
    @Override
    public void mousePressed( MouseEvent e )
    {
    }
    
    /**
     * Called if a mouse button is released in the JPanel.
     * 
     * @param e 
     */
    @Override
    public void mouseReleased( MouseEvent e ) 
    {
    }
        
    /**
     * Returns the DefaultTreeModel object.
     * @return 
     */
    public DefaultTreeModel getDefaultTreeModel( )
    {
        return m_defaultTreeModel;
    }
    
    /**
     * Reads the attributes from a RandomAccesFile.
     */
    void read( RandomAccessFile in )
    {
        m_fitModelTreeView.read( in );
    }
    
    /**
     * Writes attributes to RandomAccessFile.
     */
    void write( RandomAccessFile out )
    {
        m_fitModelTreeView.write( out );
    }
    
    /**
     * Sets the most central model parameters of the model.
     * @param mrModelSpectrum 
     */
    public void setMRModelSpectrum( TDFDFitPrototypeModelSpectrumParameters mrModelSpectrum )
    {
        m_fitModelTreeView.setMRModelSpectrum( mrModelSpectrum );
    }
    
    /**
     * Adds an ActionListener to the list of action listeners of this class.
     * @param e 
     */
    public void addActionListener( ActionListener e )
    {
        m_actionListeners.add( e );
    }
    
    /**
     * Returns a reference to the model tree object.
     * @return 
     */
    public TDFDFitModelTreeWithPopupMenu getModelTree( )
    {
        return m_fitModelTreeView;
    }
    
    
    // Attributes
    private final JFrame m_parent;
    private final TDFDFitPrototypeModelSpectrumParameters m_modelSpectrum;
    
    private JMenuItem                           m_addMetaboliteMenuItem;
    private JMenuItem                           m_deleteMetaboliteMenuItem;
    
    private final DefaultMutableTreeNode        m_treeRoot;
    private DefaultTreeModel                    m_defaultTreeModel;
    private final TDFDFitModelTreeWithPopupMenu m_fitModelTreeView;
    
    private final JPanel                        m_fitModelTreePanel;
    private MouseAdapter                        m_treeMouseListener;
    private final JScrollPane                   m_treeScrollPane;

    private final LinkedList<ActionListener>    m_actionListeners = new LinkedList<ActionListener>( );
    
    public final static int                     ADD_LINE_EVENT = 0;
    public final static int                     ADD_PARAMETRIC_PATTERN_EVENT = 1;
    public final static int                     ADD_NUMERIC_PATTERN_EVENT = 2;
    public final static int                     ADD_METABOLITE_EVENT = 3;
    public final static int                     REMOVE_SPECTRAL_COMPONENT_EVENT = 4;
    
    public final static int                     CHANGE_COMPONENT_NAME_EVENT = 5;
    
    // For proper serialization
    private static final long serialVersionUID = 1482255591289213L;
}