/*
 * File: $Id: MRFittingDefaultsFrame.java,v 1.1.1.1 2006/04/19 09:38:49 hans Exp $
 * History
 * Date: 03.12.2003, 10:58  Authors: Hans Slotboom  
 *
 * Changes:
 *
 * Copyright (C) 2003 by
 * 
 * Inselspital - Department of Neuroradiology
 * CH 3010 Bern Switzerland
 */


package org.spectrim.tdfdfit;

import java.awt.Point;
import java.awt.Dimension;

import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.BoxLayout;
import javax.swing.JTabbedPane;


/**
 * 
 * Main JFrame by which the user can set important TDFDFit related 
 * default values.
 * 
 * @author  hans
 */

public class MRFittingDefaultsFrame extends JFrame 
{
    
    /** 
     * Creates a new instance of MRFittingDefaultsFrame. 
     * @param parent
     */
    public MRFittingDefaultsFrame( JFrame parent )
    {
        m_parent = parent;
        
        m_contentPanel = new JPanel( );
        m_contentPanel.setLayout( new BoxLayout( m_contentPanel, BoxLayout.Y_AXIS ) );
        // m_contentPanel.setMaximumSize( new Dimension( 800, 350 ) );
        m_contentPanel.setMinimumSize( new Dimension( 800, 350 ) );
        //m_contentPanel.setPreferredSize( new Dimension( 800, 350 ) );
        
        m_tabbedPane = new JTabbedPane( JTabbedPane.BOTTOM );
        
        m_tabbedPane.addTab( "File system settings", new TDFDFitFileSystemSettingsPanel( ) );
        
        m_tabbedPane.addTab( "Pre-Processing", new MRSpectrumPreProcessingPanel( ) );
        
        m_tabbedPane.addTab( "Patterns", new PatternDefaultsPropertyPanel( ) );
        
        m_tabbedPane.addTab( "Output file handling", new TDFDFitOutputFilesHandlingPanel( ) );
        
        m_contentPanel.add( m_tabbedPane );
        
        setContentPane( m_contentPanel );
        
        // add window listener necessary to handle the window closing 
        // event properly
        addWindowListener( 
            new WindowAdapter( )
            {
                public void windowClosing( WindowEvent event )
                {
                   dispose( );
                }
            }
        );
        
        pack( );
        
        // Put the dialog nicely in the middle of its parent
        Point centerParent = new Point( parent.getX( ) + ( parent.getWidth( ) / 2 ) - this.getWidth( ) / 2,
                                        parent.getY( ) + ( parent.getHeight( ) / 2 ) - this.getHeight( ) / 2 );
        
        setTitle( "TDFDFit related default settings" );
        
        setLocation( centerParent );
        setVisible( true );
        pack( );
    }
    // Attributes
    private final JFrame        m_parent;
    private final JPanel        m_contentPanel;
    private final JTabbedPane   m_tabbedPane;
    
    // For proper serialization
    private static final long   serialVersionUID = 1482125898889393L;
    
}
