/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package org.spectrim.tdfdfit;

import java.awt.Color;
import java.awt.Cursor;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.InputEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.geom.Point2D;
import java.util.Vector;
import javax.swing.JCheckBoxMenuItem;
import org.spectrim.xyplot.Curve;

/**
 * An XYPlot derived class that allows the easy definition of baseline 
 * @author johan
 */
public class MRSpectrumBaselineGUIPropertiesDefiner extends MRParameterSpectrumModelViewer implements ActionListener, MouseListener, MouseMotionListener
{
    public MRSpectrumBaselineGUIPropertiesDefiner( )
    {
        super( "Define baseline properties", null, null );
        setSpecializedGUIAttributes( );
    }
    
    /**
     * Adds entry point for user interactive spectrum model editing.
     */
    private void setSpecializedGUIAttributes( )
    {
        
        m_modelSpectumEditing.setEnabled( false );
        m_applyLineShapeTransform.setEnabled( false );
        m_printSpectrum.setEnabled( false );
        m_signalProcessing.setEnabled( false );
        m_manualSelectBaseLinePoints = new JCheckBoxMenuItem( "Enable manual selection of baseline points" );
        m_manualSelectBaseLinePoints.setSelected( true );
        m_manualSelectBaseLinePoints.addActionListener( this );
        m_popupmenu.add( m_manualSelectBaseLinePoints );
        // Sets the cursor shape to cross to indicate that 
        // the user needs to define points.    
        setCursor( Cursor.getPredefinedCursor( Cursor.CROSSHAIR_CURSOR ) );
        
    }
    
    /**
     * Called when the mouse button is clicked.
     * @param mouseEvent
     */
    @Override
    public void mouseClicked( MouseEvent mouseEvent ) 
    {
        if ( m_addBaselineModeActive ==  true )
        {
            if ( ( mouseEvent.getModifiers( ) & InputEvent.BUTTON1_MASK ) == InputEvent.BUTTON1_MASK )
            {
                
                if ( m_orderedSplineNodes.isEmpty( ) )
                {
                    int length = m_curves.get( 0 ).getXvalues( ).length;
                    m_orderedSplineNodes.add( new Point2D.Double( m_curves.get( 0 ).getXvalues( )[ 0 ], 
                                                                  m_curves.get( 0 ).getYvalues( )[ 0 ] ) );
                    m_orderedSplineNodes.add( new Point2D.Double( m_curves.get( 0 ).getXvalues( )[ length - 1 ], 
                                                                  m_curves.get( 0 ).getYvalues( )[ length - 1 ] ) );
                }
                
                m_mouseButtonClickedX = mouseEvent.getX( );
                m_mouseButtonClickedY = mouseEvent.getY( );
               
                Point pnt = new Point( m_mouseButtonClickedX, m_mouseButtonClickedY );
                
                // Determine the coordinates in float format of the real coordinate system
                Point2D.Double fltPoint = super.determineValuesAtCursorPosition( pnt );
                
                float[] xArrayValues = this.m_curves.get( 0 ).getXvalues( );
                float[] yArrayValues = this.m_curves.get( 0 ).getYvalues( );
                
                int arrayPosition = ( m_mouseButtonClickedX - this.getWidth( ) / 10 ) * xArrayValues.length / ( 8 * this.getWidth( ) / 10 ); 
                
                // fltPoint.y = yArrayValues[arrayPosition];
                float meanYvalue =0.0f;
                int cntr = 0;
                for ( int j=-2;j<=2;j++)
                {
                    if ( ( j > -1 ) && ( j < yArrayValues.length ) )
                    {
                        meanYvalue += yArrayValues[arrayPosition+j];
                        cntr++;
                    }
                }
                fltPoint.y = meanYvalue/cntr;
                
                float yrange = Math.abs( m_xyAutoPlotRange.getMaximumY( ) - m_xyAutoPlotRange.getMinimumY( ) );
                int yPosition = (this.getHeight( ) / 10 ) +  (int)( ( m_xyAutoPlotRange.getMaximumY( ) - fltPoint.y ) * ( 8 * getHeight( ) / 10 ) / yrange );
                
                m_splineNodesIntPointValues.add( new Point2D.Double( fltPoint.x, fltPoint.y ) );
                
                setSplineNodes( m_splineNodesIntPointValues );
                
                m_orderedSplineNodes.add( fltPoint );
                
                notifyActionListeners( SPLINE_POINT_ADDED, SPLINE_POINT_ADDED_CMD );
            }
            
            super.mouseClicked( mouseEvent );
            
        }
    }
   
    /**
     * Sets the ordered spline nodes.
     * @param orderedNodeList 
     */
    public void setOrderedSplineNodeList( Vector<Point2D.Double> orderedNodeList )
    {
        // check whether there are no double x values in the list
        // the spline algorithm does not like this ...
        int[] nodesToRemove = new int[ orderedNodeList.size( ) ];
        for ( int i = 0; i < orderedNodeList.size( ); i++ )
        {
            for ( int j = 0; j < orderedNodeList.size( ); j++ )
            {
                if ( orderedNodeList.elementAt( i ).x == orderedNodeList.elementAt( j ).x )
                {
                    nodesToRemove[i]=1;
                }
                else
                {
                     nodesToRemove[i] = 0;
                }
            }
        }
        for ( int i = nodesToRemove.length - 1 ; i > 0; i-- )
        {
            if ( nodesToRemove[i] == 1 )
            {
                // System.out.println( "node to remove : " + i );
                orderedNodeList.remove( i );
            }
        }
        
        m_orderedSplineNodes = orderedNodeList;
        setSplineNodes( m_orderedSplineNodes );
    }
    
    /**
     * Removes the ordered spline nodes. 
     */
    public void removeCurrentOrderedSplineNodeList( )
    {
        m_orderedSplineNodes.clear( );
        setSplineNodes( m_orderedSplineNodes );
    }
    
    
    public Vector<Point2D.Double> getOrderedSplineNodeList( )
    {
        
        Vector<Point2D.Double> splineNodes = new Vector<>( );
        
        for ( int i = 0; i < m_orderedSplineNodes.size( ); i++ )
        {
            splineNodes.add( m_orderedSplineNodes.get( i ) ); 
        }
        
        // now clear the data structure
        m_orderedSplineNodes.clear( );
        
        // fill with ordered values
        while ( !splineNodes.isEmpty( ) )
        {
            
            int mimimumIndex = 0;
            double minimum = Double.MAX_VALUE;
            
            for ( int i = 0; i < splineNodes.size( ); i++ )
            {
                if ( splineNodes.get(i).x < minimum )
                {
                    mimimumIndex = i;
                    minimum = splineNodes.get(i).x;
                }
            }
            
            m_orderedSplineNodes.add( splineNodes.get( mimimumIndex ) );
            splineNodes.remove( mimimumIndex );
        }    
        
        return m_orderedSplineNodes;
    }   
    
    /**
     * Called when the mouse is dragged.
     * @param mouseEvent
     */
    @Override
    public void mouseDragged( MouseEvent mouseEvent )
    {
        m_mouseDragged = true;
        super.mouseDragged( mouseEvent );
    }
    
    /**
     * Called when the mouse enters the control.
     * @param mouseEvent
     */    
    @Override
    public void mouseEntered( MouseEvent mouseEvent )
    {
        super.mouseEntered( mouseEvent );
    }
    
    /**
     * Called when the mouse exits.
     * @param mouseEvent
     */    
    @Override
    public void mouseExited( MouseEvent mouseEvent )
    {
        super.mouseExited( mouseEvent );
    }
    
    /**
     * Called when the mouse button is released.
     * @param mouseEvent
     */    
    @Override
    public void mouseMoved( MouseEvent mouseEvent )
    {
        super.mouseMoved( mouseEvent );
    }
    
     /**
     * Called whenever an ActionEvent occurs.
     */
    @Override
    public void actionPerformed( ActionEvent e )
    {   
        Object sender = e.getSource( );
        if ( sender == m_manualSelectBaseLinePoints )
        {
            if ( m_manualSelectBaseLinePoints.isSelected( ) )
            {
                m_addBaselineModeActive = true;
                this.setCursor( Cursor.getPredefinedCursor( Cursor.CROSSHAIR_CURSOR ) );
            }
            else
            {
               m_addBaselineModeActive = false;
               this.setCursor( Cursor.getPredefinedCursor( Cursor.DEFAULT_CURSOR ) );
            }
        }
    }
        
    // Atrributes to make graphical spline point selection possible
    private boolean m_addBaselineModeActive = true;
    private JCheckBoxMenuItem m_manualSelectBaseLinePoints;
    
    private Vector<Point2D.Double> m_orderedSplineNodes = new Vector<>();
    private Vector<Point2D.Double> m_splineNodesIntPointValues = new Vector<>();
    public final static int    SPLINE_POINT_ADDED     = 200;
    public final static String SPLINE_POINT_ADDED_CMD = "SPLINE_POINT_ADDED_CMD";

    
    /**
     * Removes the curve named "spline" of the display.
     */
    void removeSpline( ) 
    {
        for ( int i = 0; i < m_curves.size( ); i++ )
        {
            String name = m_curves.get( i ).getLabel( );
            if ( name.compareTo( "spline" ) == 0 )
            {
                m_curves.remove( i );
                break;
            }
        }
    }
    
    /**
     * Adds a curve named "spline" to the display.
     */
    void addSpline( Curve curve, boolean b ) 
    {
        Curve original = m_curves.get( 0 );
        
        m_curves.add( new Curve( original.getXvalues( ), 
                                 curve.getYvalues( ), 
                                 Color.BLUE, 
                                 "spline" ) );
    }
    
}
