/*
 * File: $Id: MRSpectrumModellerPanel.java,v 1.1.1.1 2006/04/19 09:38:49 hans Exp $
 * History
 * Date: 27.05.2003, 12:03  Authors: Hans Slotboom
 *
 * Changes:
 *
 * Copyright (C) 2003 by
 * 
 * Inselspital - Department of Neuroradiology
 * CH 3010 Bern Switzerland
 */
 
package org.spectrim.tdfdfit;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

import java.awt.BorderLayout;

import java.awt.Dimension;

import java.util.LinkedHashMap;

import javax.swing.JTabbedPane;
import javax.swing.JPanel;
import javax.swing.JFrame;
import javax.swing.JScrollPane;

import java.io.RandomAccessFile;

//c import SCANalyze.Dicom.MultiSpectroscopicVoxelProjectionDicomSliceCanvasPolyDraw;
import org.spectrim.dicom.DicomImageSeriesData;
import org.spectrim.dicom.FileInfo;
import org.spectrim.dicom.DicomDirectory;
import org.spectrim.lib.StatusPanel;
import org.spectrim.imagegraphicobjects.Spectroscopic3DVoxel;


//c import SCANalyze.Dicom.SpectroscopicVoxelProjectionDicomSliceCanvasPolyDraw;
//c import SCANalyze.ChemicalShiftImaging.CSIDataProcessingAndDataViewer;
//c import SCANalyze.ChemicalShiftImaging.ChemicalShiftImagingMeasuredSpectralData;

import java.util.Iterator;
import java.util.LinkedList;
import org.spectrim.model.GlobalModel;

import org.spectrim.patientdatabase.DatabaseMiningGUIControl;
import org.spectrim.patientdatabase.MRPatientSpectrumDataBaseItemViewer;
import org.spectrim.patientdatabase.MRSpectroscopyDatabaseQueryPanel;

/**
 * Main graphical user interface to control and define the quantification
 * using TDFDFit. Massively refactored in 01.2017 - 02.2017
 * @author  hans
 */
public class MRSpectrumModellerPanel extends JPanel implements ActionListener
{
    
    /** 
     * Creates a new instance of MRSpectrumModellerPanel. 
     * @param parent
     * @param ddir
     * @param showWaterReferenceDefinerFrame
     * @param model
     */
    public MRSpectrumModellerPanel( JFrame parent, DicomDirectory ddir, 
                                    boolean showWaterReferenceDefinerFrame,
                                    GlobalModel model ) 
    {
        m_parent = parent;
        m_dicomDir = ddir;
        m_globalModel = model;
        
        TDFDFitPrototypeModelSpectrumParameters modelSpectrum = TDFDFitPrototypeModelSpectrumParameters.getInstance( );
        
        setLayout( new BorderLayout( ) );
        
        m_tabbedPaneWest = new JTabbedPane( JTabbedPane.BOTTOM );
        
        
        m_lineModelParametersPanel =  new MRLineModelParametersPanel( m_parent );
        m_lineModelParametersPanel.addActionListener( this );
        
        m_tabbedPaneWest.addTab( "Spectral Model", m_lineModelParametersPanel );
        
        m_fitStrategyDefiner = new MRFitStrategyPanel( m_parent );
        
        m_tabbedPaneWest.addTab( "Fitting Strategy", m_fitStrategyDefiner );
        m_tabbedPaneWest.setEnabledAt( 1, false );
        
        m_mrSpectroscopyDatabaseQueryPanel  = new   MRSpectroscopyDatabaseQueryPanel( m_parent );
       
        m_mrSpectroscopyDatabaseQueryPanel.addActionListener( this );
        m_tabbedPaneWest.addTab( "Database" , m_mrSpectroscopyDatabaseQueryPanel );
        m_tabbedPaneWest.setEnabledAt( 2, false );
        
        m_dbaseMiningGUIControl = new DatabaseMiningGUIControl( );
        
        m_tabbedPaneWest.addTab( "Data mining", m_dbaseMiningGUIControl );
        m_tabbedPaneWest.setEnabledAt( 3, false );
        
        m_tabbedPaneWest.setSelectedIndex( 0 );
        
        add( m_tabbedPaneWest, BorderLayout.WEST );
        
        m_tabbedPane = new JTabbedPane( );
        
        m_modelSpectrumViewer = new  MRParameterSpectrumModelViewer( "Prototype model spectrum", m_parent, m_dicomDir );
        
        // The model spectrum viewer raises action events in the line model 
        // parameters panel; e.g. when the user want to add a line graphically
        m_modelSpectrumViewer.addActionListener( m_lineModelParametersPanel ); 
        
        m_modelSpectrumViewer.addActionListener( modelSpectrum );
        m_modelSpectrumViewer.addActionListener( this );
        
        m_tabbedPane.addTab( "Model Prototype", m_modelSpectrumViewer );
        m_tabbedPane.setEnabledAt( 0, true );
        m_clinicalSVSSpectrumViewer = new  MRMeasuredClinicalSVSSpectrumViewer( "Patient spectrum", m_parent, m_dicomDir, m_globalModel );
        
        m_clinicalSVSSpectrumViewer.setGlobalModel( m_globalModel );
        
        m_tabbedPane.addTab( "Clinical SVS", m_clinicalSVSSpectrumViewer );        
        m_tabbedPane.setEnabledAt( 1, true );
        
        // If the user indicates on the fit strategy definer that there is a
        // non neglectable chemical shift, than the clinical viewer should be 
        // put into a mode where the chemical shift can be determined. 
        m_fitStrategyDefiner.addActionListener( m_clinicalSVSSpectrumViewer );
        
        m_spectrumPreviewer = new  MRSingleVoxelSpectrumPreviewer( m_parent, m_dicomDir, m_globalModel );
        
        m_tabbedPane.addTab( "SVS Preview", m_spectrumPreviewer );
        m_tabbedPane.setEnabledAt( 2, true );
//        m_csiProcessorAndViewer = new CSIDataProcessingAndDataViewer( m_parent );
//        m_tabbedPane.addTab( "CSI", m_csiProcessorAndViewer );
        
        m_measuredPatientSVSSeriesViewer = new MRMeasuredPatientSpectrumSeriesViewer( "Patient spectrum series", m_parent, m_dicomDir );
        m_tabbedPane.addTab( "Clinical SVS-series", m_measuredPatientSVSSeriesViewer );
        m_tabbedPane.setEnabledAt( 3, false );
        m_measuredPatientSVSSeriesViewer.setGlobalModel( m_globalModel );
        
        // m_multiImageViewer = new MultiSpectroscopicVoxelProjectionDicomSliceCanvasPolyDraw( parent, "" );
        
        // m_multiImageViewer.setOpaque( true );
        // m_multiImageViewer.setBackground( Color.darkGray );
        
        // m_multiImageViewer.setMinimumSize( new Dimension( 512, 512 ) );
        // m_multiImageViewer.setSize( new Dimension( 512, 512 ) );
        
        // m_scrollPane = new JScrollPane( m_multiImageViewer,
        //                                JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
        //                                JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS );
        
        // m_scrollPane.setPreferredSize( new Dimension( 512, 512 ) );
        // m_scrollPane.setMaximumSize( new Dimension( 512, 512 ) );
        // m_scrollPane.setMinimumSize( new Dimension( 512, 512 ) );
        
        // m_tabbedPane.addTab( "Images", m_scrollPane );
        
        m_patientDBItemViewer = new MRPatientSpectrumDataBaseItemViewer( );
        m_patientDBItemViewer.setPreferredSize( new Dimension( 1024, 512 ) );
        m_patientDBItemViewer.setSize( new Dimension( 1024, 512 ) );
        
        m_tabbedPane.addTab( "Data Base", m_patientDBItemViewer );
        m_tabbedPane.setEnabledAt( 4, false );
        add( m_tabbedPane, BorderLayout.CENTER );
        
        // The m_modelSpectrumViewer listens to changes in the modelSpectrum
        modelSpectrum.addActionListener( m_modelSpectrumViewer );
        
        modelSpectrum.setMRLineModelParametersPanel( m_lineModelParametersPanel );
        
        setXYTitles( );
        
        setVisible( true );
        
        if ( showWaterReferenceDefinerFrame == true )
        {        
            // First thing the user has to do is to define water reference signals
            m_clinicalSVSSpectrumViewer.defineWaterReferenceSignals( );        
        }
    }

    /**
     * 
     * Sets the horizontal titles of the x-axis.
     * 
     */
    protected void setXYTitles( )
    {
        
        TDFDFittingDefaults fitdef = TDFDFittingDefaults.getInstance( );
        fitdef.load( );
        
        String displayDomain = fitdef.getProperty( TDFDFittingDefaults.DISPLAY_DOMAIN );
        String displayUnits = fitdef.getProperty( TDFDFittingDefaults.DISPLAY_UNIT );
        
        if ( displayDomain.compareTo( TDFDFittingDefaults.TIME_DOMAIN ) == 0 )
        {
            // Set the default y-titles, they never change
            m_modelSpectrumViewer.setYTitle( "a.u." );
            m_spectrumPreviewer.setYTitle( "a.u." );
            m_measuredPatientSVSSeriesViewer.setYTitle( "a.u." );
            
            if ( displayUnits.compareTo( TDFDFittingDefaults.TIME_DOMAIN_POINTS ) == 0 )
            {
                
                m_modelSpectrumViewer.setXTitle( "TD - Points" );
                m_spectrumPreviewer.setXTitle( "TD - Points" );
                m_measuredPatientSVSSeriesViewer.setXTitle( "TD - Points" );
            }
            
            if ( displayUnits.compareTo( TDFDFittingDefaults.TIME_DOMAIN_SECONDS ) == 0 )
            {
                m_modelSpectrumViewer.setXTitle( "TD - msec" );
                m_spectrumPreviewer.setXTitle( "TD - msec" );
                m_measuredPatientSVSSeriesViewer.setXTitle( "TD - msec" );
            }
        }

        if ( displayDomain.compareTo( TDFDFittingDefaults.FREQ_DOMAIN ) == 0 )
        {
            
            if ( displayUnits.compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_POINTS ) == 0 )
            {
                m_modelSpectrumViewer.setXTitle( "FD - points" );
                m_spectrumPreviewer.setXTitle( "FD - points" );
                m_measuredPatientSVSSeriesViewer.setXTitle( "FD - points" );
            }
            
            if ( displayUnits.compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_HZ ) == 0 )
            {
                m_modelSpectrumViewer.setXTitle( "FD - Hz" );
                m_spectrumPreviewer.setXTitle( "FD - Hz" );
                m_measuredPatientSVSSeriesViewer.setXTitle( "FD - Hz" );
            }

            if ( displayUnits.compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_PPM ) == 0 )
            {
                m_modelSpectrumViewer.setXTitle( "FD - ppm" );
                m_spectrumPreviewer.setXTitle( "FD - ppm" );
                m_measuredPatientSVSSeriesViewer.setXTitle( "FD - ppm" );
            }
        }
    }
    
    /**
     * This method causes an object of this class to respond to changes in
     * TFDFFittingDefaults.
     */
    public void listenToChangesInTFDFFittingDefaults( )
    {
        TDFDFittingDefaults.getInstance( ).addActionListener( this );
    }
    
    /**
     * Returns the MRLineModelParametersPanel.
     * @return 
     */
    public MRLineModelParametersPanel getMRLineModelParametersPanel( )
    {
        return m_lineModelParametersPanel;
    }
    
    /**
     * Sets the name of the fitting model.
     * @param projectName
     */
    public void setFitModelName( String projectName )
    {
        m_lineModelParametersPanel. setFitModelName( projectName );
        m_tabbedPaneWest.setEnabledAt( 1, true );
        m_fitStrategyDefiner.upDateGUI( );
    }

    /**
     * Reads the attributes from a RandomAccesFile.
     */
    private void read( RandomAccessFile in )
    {
        m_lineModelParametersPanel.read( in );
    }
    
    /**
     * Writes attributes to RandomAccessFile.
     */
    private void write( RandomAccessFile out )
    {
        m_lineModelParametersPanel.write( out );
    }
    
    /**
     * Sets the MR model spectrum in the MRLineModelParametersPanel and the
     * SingleVoxelSpectrumViewer .
     * @param modelSpectrum
     */
    public void setMRModelSpectrum( TDFDFitPrototypeModelSpectrumParameters modelSpectrum )
    {
        // set the spectrum display to visible
        m_tabbedPane.setSelectedIndex( 0 );
        
        m_lineModelParametersPanel.setMRModelSpectrum( modelSpectrum );
    }
    
    /**
     * Sets the MR model spectrum in the MRLineModelParametersPanel and the
     * SingleVoxelSpectrumViewer. 
     * @param dicomData
     * @param voxel
     */
    public void setImageSeries( DicomImageSeriesData dicomData, Spectroscopic3DVoxel voxel )
    {
        // set the spectrum display visible
//        m_tabbedPane.setSelectedIndex( 5 );
//        
//        SpectroscopicVoxelProjectionDicomSliceCanvasPolyDraw canvas = new SpectroscopicVoxelProjectionDicomSliceCanvasPolyDraw( m_parent, voxel );
//        
//        if ( m_multiImageViewer.getCurrentView( ) != null )
//        {
//            m_multiImageViewer.setCurrentView( canvas );
//        }
//        
//        m_multiImageViewer.addNewView( dicomData, new Dimension( 256, 256 ), dicomData.getImageSeriesDescription( ), voxel );
//        
//        m_multiImageViewer.setupLayout( );
//        
//        m_multiImageViewer.getCurrentView( ).displayDicomImage( 0 );
//        m_multiImageViewer.getCurrentView( ).setZoom( 1 );
//        
//        m_multiImageViewer.getCurrentView( ).drawImage( false );
//                
//        m_multiImageViewer.setupLayout( );
    }
    
    /**
     * Returns the reference on the MultiDicomSliceView
     */
//    public MultiSpectroscopicVoxelProjectionDicomSliceCanvasPolyDraw getMultiDicomSliceView( )
//    {
//       return m_multiImageViewer;
//    }
    
    
    /**
     * Sets state based on whether a non-null project object exists.
     * @param state
     */
    public void setTDFDFitModelDefined( boolean state )
    {
        m_projectDefined = state;
        m_lineModelParametersPanel.setProjectDefined( state );
    }
    
    /**
     * Sets the measured reference spectrum object in the m_modelSpectrumViewer
     * just for previewing;
     * The reference spectrum, is the spectrum that is measured in order to obtain
     * prior knowledge information.
     * @param specData
     * @param refresh
     */
    public void setMeasuredSpectrumToPreview( SingleVoxelMeasuredSpectralData specData, boolean refresh )
    {
        // set the patient spectrum display visible
        m_tabbedPane.setSelectedIndex( 2 );
        m_spectrumPreviewer.setMeasuredSpectrumToPreview( specData, refresh );    
    }
    
    /**
     * Sets the measured reference spectrum object in the m_modelSpectrumViewer
     * as definitive object;
     * The reference spectrum, is the spectrum that is measured in order to obtain
     * prior knowledge information.
     * @param specData
     * @param refresh
     */
    public void setMeasuredSpectrum( SingleVoxelMeasuredSpectralData specData, boolean refresh )
    {
        // set the model spectrum display visible
        m_tabbedPane.setSelectedIndex( 0 );
        m_modelSpectrumViewer.setMeasuredSpectrum( specData, refresh );    
    }
    
    /**
     * Sets the measured clinical spectrum object in the m_patientSpectrumViewer.
     * @param dicomFile
     * @param specData
     */
    public void setMeasuredClinicalSVSSpectrum( String dicomFile, 
                                                SingleVoxelMeasuredSpectralData specData )
    {
        // Set the patient spectrum display visible
        m_tabbedPane.setSelectedIndex( 1 );
        // Set the spectrum to preview
        m_clinicalSVSSpectrumViewer.setMeasuredClinicalSpectrum( specData );
        m_loadedSpectrumType = SVS_SPECTRUM;
        m_dicomFileRef = dicomFile;
    }
       
    /**
     * Sets the measured patient spectrum object in the m_patientSpectrumViewer.
     * @param fileName
     * @param specData
     */
    public void setMeasuredPatientSVSSpectrum( String fileName, SingleVoxelMeasuredSpectralData specData )
    {
        // set the patient spectrum display visible
        m_tabbedPane.setSelectedIndex( 1 );
        // set the spectrum to preview
        m_clinicalSVSSpectrumViewer.setMeasuredSpectrum( fileName, specData );
        m_loadedSpectrumType = SVS_SPECTRUM;
    }

    /**
     * Sets the measured patient spectrum series data object 
     * in the m_patientSpectrumViewer.
     * @param fileName
     * @param specData
     */
    public void setSVSDataSeries( String fileName, SingleVoxelSpectralDataSeries specData )
    {
        // set the svs spectrum series displayer visible
        m_tabbedPane.setSelectedIndex( 4 );
        // set the spectrum to preview
        m_measuredPatientSVSSeriesViewer.setMeasuredSpectrumSeries( fileName, specData );   
        m_loadedSpectrumType = SVS_SPECTRUM_SERIES;
    }
    
    /**
     * Sets the measured CSI spectrum object in the 
     */
//    public void setMeasuredPatientCSISpectrum( String fileName, ChemicalShiftImagingMeasuredSpectralData csiData,
//                                               DicomDirectory dicomDir, Spectroscopic3DVoxel specVoxel, CSIGrid csiGrid )
//    {
//        // set the patient spectrum display visible
//        m_tabbedPane.setSelectedIndex( 3 );
//        // set the spectrum to preview
//        m_csiProcessorAndViewer.setCSIData( fileName, csiData, dicomDir, specVoxel, csiGrid );
//    }
    
    /**
     * Sets the measured reference spectrum object in the m_patientSpectrumViewer
     * just for previewing;
     * The reference spectrum, is the spectrum that is measured in order to obtain
     * prior knowledge information.
     * @param fileName
     * @param specData
     */
    public void setMeasuredPatientWaterReferenceSpectrum( String fileName, SingleVoxelMeasuredSpectralData specData )
    {
        // set the patient spectrum display visible
        m_tabbedPane.setSelectedIndex( 1 );
        // set the spectrum to preview
        if ( m_loadedSpectrumType == SVS_SPECTRUM )
        {
            m_clinicalSVSSpectrumViewer.setMeasuredPatientWaterReferenceSpectrum( fileName, specData );
        }
        if ( m_loadedSpectrumType == SVS_SPECTRUM_SERIES )
        {
            m_measuredPatientSVSSeriesViewer.setMeasuredPatientWaterReferenceSpectrum( fileName, specData );
        }
        if ( m_loadedSpectrumType == CSI_SPECTRUM )
        {
            System.out.println( "CSI water reference not handled yet" );
        }
    }
    
    /**
     * Sets the reference to the StatusPanel to be used by the application.
     * @param statPanel
     */
    public void setStatusPanel( StatusPanel statPanel )
    {
        m_statusPanel = statPanel;
        if ( m_modelSpectrumViewer != null )
        {
            m_modelSpectrumViewer.setStatusPanel( m_statusPanel );
            m_clinicalSVSSpectrumViewer.setStatusPanel( m_statusPanel );
            m_spectrumPreviewer.setStatusPanel( m_statusPanel );
            m_measuredPatientSVSSeriesViewer.setStatusPanel( m_statusPanel );
        }
    }
    
    /**
     * Returns the measured spectrum data object.
     * @return 
     */
    public SingleVoxelMeasuredSpectralData getMeasuredModelSpectrum( )
    {
        return m_modelSpectrumViewer.getMeasuredSpectrum( );
    }
    
    
    /**
     * Called whenever a registered component raises an ActionEvent.
     * @param e
     */
    @Override
    public void actionPerformed( ActionEvent e ) 
    {
        if ( e.getSource( ) == m_lineModelParametersPanel )
        {
            if ( e.getID( ) == MRLineModelParametersPanel.DISPLAY_FREQ_UNITS_TYPE_CHANGED )
            {
                m_modelSpectrumViewer.updateForcedXYPlotRange( m_lineModelParametersPanel.getPreviouslyDisplayedFrequencyUnitType( ), 
                                                               m_lineModelParametersPanel.getCurrentlyDisplayedFrequencyUnitType( ) );
            }
        }
        
        
        if ( e.getSource( ) == TDFDFittingDefaults.getInstance( ) )
        {
            TDFDFittingDefaults defs = TDFDFittingDefaults.getInstance( );
            defs.load( );
            if ( defs.getProperty( TDFDFittingDefaults.DISPLAY_DOMAIN ).compareTo( TDFDFittingDefaults.FREQ_DOMAIN ) == 0 )
            { 
                if ( defs.getProperty( TDFDFittingDefaults.DISPLAY_UNIT ).compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_PPM ) == 0 )
                {
                    m_modelSpectrumViewer.setDisplayXReversed( true );
                }
                else
                {
                    m_modelSpectrumViewer.setDisplayXReversed( false );
                }
            }
            else
            {
                 m_modelSpectrumViewer.setDisplayXReversed( false );
            }
            setXYTitles( );
            m_modelSpectrumViewer.changeDisplayedSignal( );
        }
        
        if ( e.getActionCommand( ).compareTo( "measured spectrum changed" ) == 0 )
        {
            SingleVoxelSpectralData resp = (SingleVoxelSpectralData)e.getSource( );
            SingleVoxelMeasuredSpectralData measSpec = new SingleVoxelMeasuredSpectralData( resp.getAbsorptionChannelTD( ).length );
            
            measSpec.setSpectralDataPointsTD( resp.getAbsorptionChannelTD( ), resp.getDispersionChannelTD( ) );
            measSpec.setSiemensSpecificDicomHeaderInfo( resp.getSiemensSpecificDicomHeaderInfo( ) );
            
            m_modelSpectrumViewer.setMeasuredSpectrum( measSpec, true );     
        }
        if ( e.getSource( ) == m_mrSpectroscopyDatabaseQueryPanel )
        {
            if ( e.getID( ) == MRSpectroscopyDatabaseQueryPanel.DICOM_SERIES_SELECTED )
            {
                // forward the notification
                notifyActionListeners( e.getSource( ), MRSpectroscopyDatabaseQueryPanel.DICOM_SERIES_SELECTED,  MRSpectroscopyDatabaseQueryPanel.DICOM_SERIES_SELECTED_CMD );
            }
        }
    }
    
    /**
     * Fills in the database MRPatientSpectrumDataBaseItemViewer GUI items.
     * @param fi
     * @param ssh
     */
    public void setDataBaseItem( FileInfo fi, LinkedHashMap ssh )
    {
       m_patientDBItemViewer.updateForm( fi, ssh ); 
    }
    
    /**
     * Adds an ActionListener to the list of listeners.
     * @param listener
     */
    public void addActionListener( ActionListener listener )
    {
        m_actionListeners.add( listener );
    }
      
    /**
     * Notifies registered ActionListeners.
     */
    private void notifyActionListeners( Object obj, int eventType, String actionCommand )
    {
        Iterator it = m_actionListeners.iterator( );
        while( it.hasNext( ) )
        {
            ( ( ActionListener )it.next( ) ).actionPerformed( new ActionEvent( obj, eventType, actionCommand ) );
        }
    }
    
    /**
     * Returns a reference to the MRParameterSpectrumModelViewer.
     * @return 
     */
    public MRParameterSpectrumModelViewer getModelSpectrumViewer( ) 
    {
        return m_modelSpectrumViewer;
    }
    
     /**
     * This forces the simulation of a DecimalTextField-changed event;
     * The method is a hack to get the displaying correct after importing
     * a metabolite to the model.
     */
    void simulateDecimalTextFieldChangedEvent( ) 
    {
        m_lineModelParametersPanel.simulateDecimalTextFieldChangedEvent( );
    }
    
    // Attributes
    public JFrame                                     m_parent;
    
    private boolean                                   m_projectDefined;
    private final MRLineModelParametersPanel          m_lineModelParametersPanel;
    private final MRFitStrategyPanel                  m_fitStrategyDefiner;
    private final DatabaseMiningGUIControl            m_dbaseMiningGUIControl;
    private final MRSpectroscopyDatabaseQueryPanel    m_mrSpectroscopyDatabaseQueryPanel;
    
    // A viewer for the model development
    private final MRParameterSpectrumModelViewer      m_modelSpectrumViewer;

    // A viewer for viewing patient spectra
    private final MRMeasuredClinicalSVSSpectrumViewer m_clinicalSVSSpectrumViewer;
    
    // A viewer for viewing images of a study
    // private MultiSpectroscopicVoxelProjectionDicomSliceCanvasPolyDraw m_multiImageViewer;
    
    // A viewer for previewing single voxel spectra 
    private final MRSingleVoxelSpectrumPreviewer      m_spectrumPreviewer;
    
    // A viewer for viewing patient database item information 
    private final  MRPatientSpectrumDataBaseItemViewer m_patientDBItemViewer;
    
    // A viewer for CSI results and enbler of post processing of CSI datasets
    //c private CSIDataProcessingAndDataViewer m_csiProcessorAndViewer;
    
    // A viewer for SVS time series data
    private final MRMeasuredPatientSpectrumSeriesViewer m_measuredPatientSVSSeriesViewer;
    
    private StatusPanel m_statusPanel;
    
    // The card panel should display either an image series or a (for the time being)
    // a single voxel spectrum
    private final JTabbedPane m_tabbedPane;
    private final JTabbedPane m_tabbedPaneWest;
    
    private JScrollPane m_scrollPane;
    
    // Variables needed to select image or spectral views
    final static String IMAGE_DISPLAY               = "Image display";
    final static String SPECTRUM_DISPLAY            = "Spectrum display";
    final static String PATIENT_SPECTRUM_DISPLAY    = "Patient spectrum display";
    
    private int m_loadedSpectrumType = 0;
    
    public final static int SVS_SPECTRUM = 0;
    public final static int SVS_SPECTRUM_SERIES = 1;
    public final static int CSI_SPECTRUM = 2;
    
    private final LinkedList<ActionListener> m_actionListeners = new LinkedList<>( );
    
    // For proper serialization
    private static final long serialVersionUID = 1481295892562393L;

    private String m_dicomFileRef;
    
    private final DicomDirectory m_dicomDir;
    private GlobalModel m_globalModel;
}
