/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package org.spectrim.tdfdfit;

import java.io.BufferedReader;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.nio.ByteBuffer;
import java.util.Iterator;
import java.util.Vector;
import org.spectrim.signalprocessing.FFT;

/**
 * 
 * Container class to store the TDFDFit results into read from the .exl file:
 * This class also handles the collection of binary result files to huge
 * binary files.
 * 
 * @author Johannes Slotboom <johannes.slotboom at insel.ch>
 */
public class NumericTDFDFitResults implements Cloneable
{

    /**
     * Default constructor.
     */
    public NumericTDFDFitResults( )
    {
        // do nothing
    }
     
    public NumericTDFDFitResults( NumericTDFDFitResults res )
    {
        this.m_dataLoadedFromFile         = res.getM_dataLoadedFromFile( );
        this.m_fileReference              = res.getM_fileReference();
        this.m_commandObtainedResultsWith = res.getM_commandObtainedResultsWith( );
        this.m_typeOfFit                  = res.getM_typeOfFit( );
       
        this.m_fitQualityNumber  = res.getM_fitQualityNumber( );
        this.m_stDeviationReal   = res.getM_stDeviationReal( );
        this.m_stDeviationImag   = res.getM_stDeviationImag( );
        this.m_chiSqBefore       = res.getM_chiSqBefore( );
        this.m_chiSqAfter        = res.getM_chiSqAfter( );
        this.m_trueChiSq         = res.getM_trueChiSq( );
        this.m_lastIter          = res.getM_lastIter( );
        this.m_lastWeightingType = res.getM_lastWeightingType( );

        this.m_componentNames    = (Vector<String>)res.getComponentNames( ).clone( );
        
        Vector<TDFDFittingParameters> emptyCopyVector = new Vector<TDFDFittingParameters>( ); 
        
        Iterator it = res.getFittedParametersVector( ).iterator( );
        while ( it.hasNext( ) )
        {
            TDFDFittingParameters elements = (TDFDFittingParameters)it.next( );
            
            TDFDFittingParameters emptyCopy = new TDFDFittingParameters( elements.getComponentName( ) );
            emptyCopyVector.add( emptyCopy );
        }
        
        //this.m_fittedElements    = (Vector<TDFDFittingParameters>)res.getFittedParametersVector( ).clone( );

        m_fittedElements = emptyCopyVector; 
        
        this.m_modelRealTDSignal = res.getFittedSpectrumRealTD( ).clone( );
        this.m_modelImagTDSignal = res.getFittedSpectrumImagTD( ).clone( );
        this.m_modelRealFDSignal = res.getFittedSpectrumRealFD( ).clone( );
        this.m_modelImagFDSignal = res.getFittedSpectrumImagFD( ).clone( );
 
        this.m_preProcessedMeasuredTDRealSignal = res.getPreprocessedInputSignalRealTD( );
        this.m_preProcessedMeasuredTDImagSignal = res.getPreprocessedInputSignalImagTD( );
        this.m_preProcessedMeasuredFDRealSignal = res.getPreprocessedInputSignalRealFD( );
        this.m_preProcessedMeasuredFDImagSignal = res.getPreprocessedInputSignalImagFD( );       
        
        this.m_fittedModelOutputSpectrumFile = res.getM_fittedModelOutputSpectrumFile( );
        this.m_numberOfPoints = res.getNumberOfPoints( );

    }

    public void clearData( ) 
    {
        m_dataLoadedFromFile         = false;
        m_fileReference              = "Not defined for this voxel!";
        m_commandObtainedResultsWith = "Not defined for this voxel!";
        m_typeOfFit                  = "Not defined for this voxel!";
       
        m_fitQualityNumber  = 0.0;
        m_stDeviationReal   = 0.0;
        m_stDeviationImag   = 0.0;
        m_chiSqBefore       = 0.0;
        m_chiSqAfter        = 0.0;
        m_trueChiSq         = 0.0;
        m_lastIter          = 0;
        m_lastWeightingType = -1;

        // component names remains!
        
        
        for ( int i = 0; i < m_fittedElements.size( ); i++ )
        {
            TDFDFittingParameters par = m_fittedElements.get( i );
            
            par.setAmplitude( 0.0 );
            par.setOffset( 0.0 );
            par.setLwidth( 0.0 );
            par.setPhase( 0.0 );
            par.setGwidth( 0.0 );
            
            par.setE_Amplitude( 0.0 );
            par.setE_Offset( 0.0 );
            par.setE_Lwidth( 0.0 );
            par.setE_Phase( 0.0 );
            par.setE_Gwidth( 0.0 );
            
        }
        
        for ( int i = 0 ; i < m_modelRealTDSignal.length; i++ ) 
        {
            
            m_modelRealTDSignal[i] = 0f;
            m_modelImagTDSignal[i] = 0f;
            m_modelRealFDSignal[i] = 0f;
            m_modelImagFDSignal[i] = 0f;
            
            if ( m_preProcessedMeasuredTDRealSignal == null )
            {
                m_preProcessedMeasuredTDRealSignal = new float[m_modelRealTDSignal.length];
                m_preProcessedMeasuredTDImagSignal = new float[m_modelRealTDSignal.length];
                m_preProcessedMeasuredFDRealSignal = new float[m_modelRealTDSignal.length];
                m_preProcessedMeasuredFDImagSignal = new float[m_modelRealTDSignal.length];
            }
            
        }
        
        m_fittedModelOutputSpectrumFile = "Not defined for this voxel!";
        
    }
    
    
    public boolean getM_dataLoadedFromFile( ) 
    {
        return m_dataLoadedFromFile;
    }

    public void setM_dataLoadedFromFile( boolean m_dataLoadedFromFile ) 
    {
        this.m_dataLoadedFromFile = m_dataLoadedFromFile;
    }

    public String getM_fileReference( ) 
    {
        return m_fileReference;
    }

    public void setM_fileReference( String m_fileReference ) 
    {
        this.m_fileReference = m_fileReference;
    }

    public String getM_commandObtainedResultsWith( ) 
    {
        return m_commandObtainedResultsWith;
    }

    public void setM_commandObtainedResultsWith( String m_commandObtainedResultsWith ) 
    {
        this.m_commandObtainedResultsWith = m_commandObtainedResultsWith;
    }

    public String getM_typeOfFit( ) 
    {
        return m_typeOfFit;
    }

    public void setM_typeOfFit( String m_typeOfFit ) 
    {
        this.m_typeOfFit = m_typeOfFit;
    }

    public double getM_fitQualityNumber( ) 
    {
        return m_fitQualityNumber;
    }

    public void setM_fitQualityNumber( double m_fitQualityNumber ) 
    {
        this.m_fitQualityNumber = m_fitQualityNumber;
    }

    public double getM_stDeviationReal( )  
    {
        return m_stDeviationReal;
    }

    public void setM_stDeviationReal( double m_stDeviationReal ) 
    {
        this.m_stDeviationReal = m_stDeviationReal;
    }

    public double getM_stDeviationImag( ) 
    {
        return m_stDeviationImag;
    }

    public void setM_stDeviationImag( double m_stDeviationImag ) 
    {
        this.m_stDeviationImag = m_stDeviationImag;
    }

    public double getM_chiSqBefore( ) 
    {
        return m_chiSqBefore;
    }

    public void setM_chiSqBefore( double m_chiSqBefore ) 
    {
        this.m_chiSqBefore = m_chiSqBefore;
    }

    public double getM_chiSqAfter( ) 
    {
        return m_chiSqAfter;
    }

    public void setM_chiSqAfter( double m_chiSqAfter ) 
    {
        this.m_chiSqAfter = m_chiSqAfter;
    }

    public double getM_trueChiSq( ) 
    {
        return m_trueChiSq;
    }

    public void setM_trueChiSq( double m_trueChiSq ) 
    {
        this.m_trueChiSq = m_trueChiSq;
    }

    public int getM_lastIter( ) 
    {
        return m_lastIter;
    }

    public void setM_lastIter( int m_lastIter ) 
    {
        this.m_lastIter = m_lastIter;
    }

    public int getM_lastWeightingType( ) 
    {
        return m_lastWeightingType;
    }

    public void setM_lastWeightingType( int m_lastWeightingType ) 
    {
        this.m_lastWeightingType = m_lastWeightingType;
    }

    public Vector<String> getComponentNames( ) 
    {
        return m_componentNames;
    }

    public void setComponentNames( Vector<String> m_componentNames ) 
    {
        this.m_componentNames = m_componentNames;
    }

    public Vector<TDFDFittingParameters> getFittedElements( ) 
    {
        return m_fittedElements;
    }

    public void setFittedElements( Vector<TDFDFittingParameters> m_fittedElements ) 
    {
        this.m_fittedElements = m_fittedElements;
    }

    public float[] getPreprocessedInputSignalRealTD( ) 
    {
        return m_preProcessedMeasuredTDRealSignal;
    }

    public void setModelRealTDSignal( float[] m_realTD ) 
    {
        System.arraycopy( m_realTD, 0, m_modelRealTDSignal, 0, m_realTD.length );
    }

    public float[] getPreprocessedInputSignalImagTD( ) 
    {
        return m_preProcessedMeasuredTDImagSignal;
    }

    public void setModelImagTDSignal( float[] m_imagTD ) 
    {
        System.arraycopy( m_imagTD, 0, m_modelImagTDSignal, 0, m_imagTD.length );
    }

    public float[] getPreprocessedInputSignalRealFD( ) 
    {
        return m_preProcessedMeasuredFDRealSignal;
    }

    public void setModelRealFDSignal( float[] m_realFD ) 
    {
        System.arraycopy( m_realFD, 0, m_modelRealFDSignal, 0, m_realFD.length );
    }

    public float[] getPreprocessedInputSignalImagFD( ) 
    {
        return m_preProcessedMeasuredFDImagSignal;
    }

    public void setModelImagFDSignal( float[] m_imagFD ) 
    {
        System.arraycopy( m_imagFD, 0, m_modelImagFDSignal, 0, m_imagFD.length );
    }

    public String getM_fittedModelOutputSpectrumFile( ) 
    {
        return m_fittedModelOutputSpectrumFile;
    }

    public void setM_fittedModelOutputSpectrumFile( String m_fittedModelOutputSpectrumFile ) 
    {
        this.m_fittedModelOutputSpectrumFile = m_fittedModelOutputSpectrumFile;
    }

    public int getNumberOfPoints( ) 
    {
        return m_numberOfPoints;
    }

    public void setNumberOfPoints( int m_numberOfPoints ) 
    {
        this.m_numberOfPoints = m_numberOfPoints;
    }
    
    /**
     * Constructor using file reference.
     * @param fileReference
     */
    public NumericTDFDFitResults( String fileReference )
    {
        m_componentNames = TDFDFitPrototypeModelSpectrumParameters.getInstance( ).getComponentNameVector( );
        if ( ( new File( fileReference ) ).exists( ) == true )
        {
            m_fileReference = fileReference; 
        }
    }
    
    /**
     * Constructor indicating the file name belonging to the content of this
     * object.
     * @param fileReference
     */
    public void setResultFileName( String fileReference )
    {
        
        if ( ( new File( fileReference ) ).exists( ) == true )
        {
            m_fileReference = fileReference; 
        }
    }
    
    
    /**
     * Reads fitted spectrum file.
     * @param numberOfPoints
     * @return 
     */
    public boolean readFittedSpectrumFile( int numberOfPoints )
    {
        m_numberOfPoints = numberOfPoints;
        String m_fittedModelSpectrumFile = m_fileReference.replace( ".exl", ".fit" );
        return readTDFDFitModelOutputSpectrumFile( m_numberOfPoints, m_fittedModelSpectrumFile );
    }
    
    /**
     * Collect all files to one huge binary file spectrum file.
     * @param numberOfPoints
     * @param file_extension
     * @return 
     */
    public boolean collectSignalInformationFromFile( int numberOfPoints, String file_extension )
    {
        m_numberOfPoints = numberOfPoints;
        String m_fittedModelSpectrumFile = m_fileReference.replace( ".exl",  file_extension );
        return readTDFDFitModelOutputSpectrumFile( m_numberOfPoints, m_fittedModelSpectrumFile );
    }
         
    /**
     * 
     * @param token
     * @return 
     */
    public boolean testIsTokenADouble( String token )
    {
        boolean isADouble = true;
        
        if ( ( token.contains( "1.#QNAN0e+000" ) == true ) || ( token.contains( "-1.#IND00e+000" ) == true ) )
        {
            isADouble = false;
            System.out.println( "Number format error while parsing " + this.m_fileReference + " !" );
        }
        
        return isADouble;
    }    
    
    
    /**
     * This is a second way to read the parameters of an NumericTDFDFitResults
     * object; namely for one voxel from a file that contains all fit parameter 
     * data.
     * @param br
     * @return 
     * @throws java.io.IOException 
     */
    public boolean readNumericTDFDFitResultsFromBufferedReader( BufferedReader br ) throws IOException
    {
        int spectralComponentCounter = 0;
        String line;
        boolean retVal = false;
        
        if ( br != null )
        {

            int     iLine               = 0;
            String  key;                        // which is in fact the key
            String  command             = null;
            String  blank_line          = null;
            String  type_of_fit         = null;
            String  fitting_pars        = null;
            String  global_fitting_pars = null;

            while ( ( line = br.readLine( ) ) != null ) 
            {
                
                if ( line.length( ) > 0 )
                {

                    if  ( line.compareTo( "###" ) != 0 )
                    {
                        
                        iLine++;
                        retVal = true;
                        
                        fitting_pars = line;
                        // Pars the line of text.
                        String[] elementTokens = fitting_pars.split( "\\t" );

                        key = elementTokens[0];
                        
                        if ( m_componentNames.get( spectralComponentCounter ) == null )
                        {
                            m_componentNames.set( spectralComponentCounter, elementTokens[1] );
                        }
                      
                        // Declare a new TDFDFittingParameters object
                        TDFDFittingParameters fittingPars = new TDFDFittingParameters( m_componentNames.get( spectralComponentCounter ) );
                        
                        fittingPars.setNumber( spectralComponentCounter );
                        fittingPars.setElementType( "undefined" ); 

                        // test and set amplitude
                        if ( testIsTokenADouble( elementTokens[2] ) )
                            fittingPars.setAmplitude( new Double ( elementTokens[2] ) );
                        else
                           fittingPars.setAmplitude( 0.0 );

                        // test and set amplitude error
                        if ( testIsTokenADouble( elementTokens[9] ) )
                            fittingPars.setE_Amplitude( new Double ( elementTokens[9] ) );
                        else
                            fittingPars.setE_Amplitude( 0.0 );


                        // fittingPars.setAmplitudeModelReference( new Integer( elementTokens[4] ) );

                        // Offset related
                        // test and set offset
                        if ( testIsTokenADouble( elementTokens[5] ) )
                            fittingPars.setOffset( new Double ( elementTokens[5] ) );
                        else
                            fittingPars.setOffset( 0.0 );

//                        if ( testIsTokenADouble( elementTokens[6] ) )
//                            fittingPars.setE_Offset( new Double ( elementTokens[6] ) );
//                        else
//                            fittingPars.setE_Offset( 0.0 );

//                        fittingPars.setOffsetModelReference( new Integer( elementTokens[7] ) );

                        // gwidth related
                        if ( testIsTokenADouble( elementTokens[10] ) )
                        {
                            fittingPars.setGwidth( new Double ( elementTokens[10] ) );
                        }
                        else
                        {
                            fittingPars.setGwidth( new Double ( 0.0 ) );
                        }
//                        if ( testIsTokenADouble( elementTokens[9] ) )
//                            fittingPars.setE_Gwidth( new Double ( elementTokens[9] ) );
//                        else
//                            fittingPars.setE_Gwidth( 0.0 );

//                        fittingPars.setGWidthModelReference( new Integer( elementTokens[10] ) );

                        // phase related   
                        if ( testIsTokenADouble( elementTokens[8] ) )
                            fittingPars.setPhase( new Double ( elementTokens[8] ) );
                        else
                            fittingPars.setPhase( 0.0 );

//                        if ( testIsTokenADouble( elementTokens[8] ) )
//                            fittingPars.setE_Phase( new Double ( elementTokens[8] ) );
//                        else
//                            fittingPars.setE_Phase( 0.0 );

//                        fittingPars.setPhaseModelReference( new Integer( elementTokens[13] ) );

                        // lwidth related
                        if ( testIsTokenADouble( elementTokens[6] ) )
                            fittingPars.setLwidth( new Double ( elementTokens[6] ) );
                        else
                            fittingPars.setLwidth( 0.0 );

//                        if ( testIsTokenADouble( elementTokens[15] ) )
//                            fittingPars.setE_Lwidth( new Double ( elementTokens[15] ) );
//                        else
//                            fittingPars.setE_Lwidth( 0.0 );

//                        fittingPars.setLWidthModelReference( new Integer( elementTokens[16] ) );

                        // The 16th and 17th element are the same as 2nd and the 3rd element
                        // Add the object to the vector of TDFDFittingParameters
                        m_fittedElements.add( fittingPars );
                        spectralComponentCounter++;
                    }

                }
            }
           
        }
        return retVal;
    }
    
    /**
     * Reads the content of the .exl file and puts it into this object.
     * @return 
     */
    public boolean readFile( )
    {
        
        int spectralComponentCounter = 0;
        
        if ( m_fileReference != null )
        {
            // System.out.println( "File to read : " + m_fileReference );

            try ( BufferedReader br = new BufferedReader( new FileReader( m_fileReference ) ) ) 
            {
                
                int iLine = 0;
                String  line;
                String  command         = null;
                String  blank_line      = null;
                String  type_of_fit     = null;
                String  fitting_pars    = null;
                String  global_fitting_pars = null;
                
                while ( ( line = br.readLine( ) ) != null ) 
                {
                    if ( iLine == 0 )
                    {
                        m_commandObtainedResultsWith = line;
                    }

                    // line 1 is empty
                    if ( iLine == 2 )
                    {
                        m_typeOfFit = line; 
                    }

                    if ( iLine == 3 )
                    {
                        global_fitting_pars = line;
                        
                        String[] tokens = global_fitting_pars.split( "\\t" );
                        tokens = areTokensNumeric( tokens );
                        if ( testIsTokenADouble( tokens[0] ) == true )
                            m_fitQualityNumber  = new Double( tokens[0] );
                        else
                            m_fitQualityNumber = -1.0;
                        
                        if ( testIsTokenADouble( tokens[1] ) == true )
                            m_stDeviationReal   = new Double( tokens[1] );
                        else
                            m_stDeviationReal = -1.0;
                        
                        if ( testIsTokenADouble( tokens[2] ) == true )
                            m_stDeviationImag   = new Double( tokens[2] );
                        else
                            m_stDeviationImag = -1.0;
                        
                        if ( testIsTokenADouble( tokens[3] ) == true )
                            m_chiSqBefore       = new Double( tokens[3] );
                        else
                            m_chiSqBefore = -1.0;
                        
                        if ( testIsTokenADouble( tokens[4] ) == true )
                            m_chiSqAfter        = new Double( tokens[4] );
                        else
                            m_chiSqAfter = -1.0;
                        
                        if ( testIsTokenADouble( tokens[5] ) == true )
                            m_trueChiSq         = new Double( tokens[5] );
                        else
                            m_trueChiSq = 1.0;
                        
                        if ( testIsTokenADouble( tokens[6] ) == true )
                            m_lastIter          = new Integer( tokens[6] );
                        else
                            m_lastIter = -1;
                        
                        if ( testIsTokenADouble( tokens[6] ) == true )
                            m_lastWeightingType = new Integer( tokens[7] );
                        else
                            m_lastWeightingType = -1;
    
                    }
                    if ( iLine == 4 )
                    {
                        // System.out.println( "> line 4 is empty! " );
                    }
                    
                    if  ( ( iLine >= 5 ) && ( line.length( ) > 5 ) )
                    {
                        fitting_pars = line;
                        // Pars the line of text.
                        String[] elementTokens = fitting_pars.split( "\\t" );
                        elementTokens = areTokensNumeric( elementTokens );
                        
                        // Declare a new TDFDFittingParameters object
                        TDFDFittingParameters fittingPars = new TDFDFittingParameters( m_componentNames.get( spectralComponentCounter ) );
                        fittingPars.setNumber( new Integer( elementTokens[0] ) );
                        fittingPars.setElementType( elementTokens[1] ); 
                        
                        // test and set amplitude
                        if ( testIsTokenADouble( elementTokens[2] ) )
                            fittingPars.setAmplitude( new Double ( elementTokens[2] ) );
                        else
                           fittingPars.setAmplitude( 0.0 );
                        
                        // test and set amplitude error
                        if ( testIsTokenADouble( elementTokens[3] ) )
                            fittingPars.setE_Amplitude( new Double ( elementTokens[3] ) );
                        else
                            fittingPars.setE_Amplitude( 0.0 );
                        
                        
                        fittingPars.setAmplitudeModelReference( new Integer( elementTokens[4] ) );
                        
                        // offset related
                        // test and set offset
                        if ( testIsTokenADouble( elementTokens[5] ) )
                            fittingPars.setOffset( new Double ( elementTokens[5] ) );
                        else
                            fittingPars.setOffset( 0.0 );
                        
                        if ( testIsTokenADouble( elementTokens[6] ) )
                            fittingPars.setE_Offset( new Double ( elementTokens[6] ) );
                        else
                            fittingPars.setE_Offset( 0.0 );
                        
                        fittingPars.setOffsetModelReference( new Integer( elementTokens[7] ) );
                        
                        // gwidth related
                        if ( testIsTokenADouble( elementTokens[8] ) )
                            fittingPars.setGwidth( new Double ( elementTokens[8] ) );
                        else
                            fittingPars.setGwidth( new Double ( 0.0 ) );
                        
                        if ( testIsTokenADouble( elementTokens[9] ) )
                            fittingPars.setE_Gwidth( new Double ( elementTokens[9] ) );
                        else
                            fittingPars.setE_Gwidth( 0.0 );
                        
                        fittingPars.setGWidthModelReference( new Integer( elementTokens[10] ) );
                        
                        // phase related   
                        if ( testIsTokenADouble( elementTokens[11] ) )
                            fittingPars.setPhase( new Double ( elementTokens[11] ) );
                        else
                            fittingPars.setPhase( 0.0 );
                        
                        if ( testIsTokenADouble( elementTokens[11] ) )
                            fittingPars.setE_Phase( new Double ( elementTokens[11] ) );
                        else
                            fittingPars.setE_Phase( 0.0 );
                        
                        fittingPars.setPhaseModelReference( new Integer( elementTokens[13] ) );
                          
                        // lwidth related
                        if ( testIsTokenADouble( elementTokens[14] ) )
                            fittingPars.setLwidth( new Double ( elementTokens[14] ) );
                        else
                            fittingPars.setLwidth( 0.0 );
                        
                        if ( testIsTokenADouble( elementTokens[15] ) )
                            fittingPars.setE_Lwidth( new Double ( elementTokens[15] ) );
                        else
                            fittingPars.setE_Lwidth( 0.0 );
                        
                        fittingPars.setLWidthModelReference( new Integer( elementTokens[16] ) );
                        
                        // The 16th and 17th element are the same as 2nd and the 3rd element
                        // Add the object to the vector of TDFDFittingParameters
                        m_fittedElements.add( fittingPars );
                        spectralComponentCounter++;
                    }
                    
                    // System.out.println( iLine + "\t : " + line );                    
                    iLine++;
                }
                br.close( );
            }
            catch( IOException ioe )
            {
                System.out.println( "IOException Error in read exl-file " );
            }
        }
        else
        {
            return false;
        }
        return true;
    }
    
    
    /**
     * ¨Returns the fit quality number of the tdfdfit quantification.
     * @return 
     */
    public double getFitQualityNumber( )
    {
        return m_fitQualityNumber;
    }
    
    
    /**
     * Reads a TDFDFit file format binary file and stores its contents in the
     * appropriate member variables.
     * @param numberOfComplexPoints
     * @param fileRef
     * @return 
     */
    public boolean readTDFDFitModelOutputSpectrumFile( int numberOfComplexPoints, String fileRef )
    {
        
        if ( ( new File( fileRef ) ).exists( ) == true )
        {
            float[] absorptionChannelFD = new float[numberOfComplexPoints];
            float[] dispersionChannelFD = new float[numberOfComplexPoints];

            try
            {          
                RandomAccessFile reader = new RandomAccessFile( fileRef, "rw" );

                ByteBuffer buf;
                byte byte0;
                byte byte1;
                byte byte2;
                byte byte3;
                float readFloat;

                for ( int i = 0; i < numberOfComplexPoints; i++ )
                {

                    buf = ByteBuffer.allocate( 4 );

                    byte0 = reader.readByte( );
                    byte1 = reader.readByte( );
                    byte2 = reader.readByte( );
                    byte3 = reader.readByte( );

                    buf.put( byte3 );
                    buf.put( byte2 );
                    buf.put( byte1 );
                    buf.put( byte0 );

                    readFloat = buf.getFloat( 0 );
                    buf.clear( );

                    absorptionChannelFD[i] = readFloat;

                    buf = ByteBuffer.allocate( 4 );

                    byte0 = reader.readByte( );
                    byte1 = reader.readByte( );
                    byte2 = reader.readByte( );
                    byte3 = reader.readByte( );

                    buf.put( byte3 );
                    buf.put( byte2 );
                    buf.put( byte1 );
                    buf.put( byte0 );

                    readFloat = buf.getFloat( 0 );
                    buf.clear( );

                    dispersionChannelFD[i] = readFloat;

                }
                
                boolean noValidValues = false;
                for ( int i = 0; i < absorptionChannelFD.length; i++ )
                {
                    if ( new Double( dispersionChannelFD[i] ).isNaN( ) ) 
                    {
                        noValidValues = true;
                    }
                    if ( new Double( absorptionChannelFD[i] ).isNaN( ) ) 
                    {
                        noValidValues = true;
                    }
                }
                
                if ( noValidValues == true )
                {
                    for ( int i = 0; i < absorptionChannelFD.length; i++ )
                    {
                        absorptionChannelFD[i] = 0f;
                        dispersionChannelFD[i] = 0f;
                    }
                    
                }
                
                // the frequency domain response is read so generate the time 
                // domain response to get full representation
                calculateTDRepresentation( absorptionChannelFD, dispersionChannelFD );

                reader.close( );

            }
            catch ( IOException e )
            {
                e.printStackTrace( );
                System.out.println( "Cannot read tdfdfit-formatted model output datafile." );
                return false;
            }
        }
        else
        {
            return false;
        }
        return true;
    }
    
    /**
     * Reads a TDFDFit file format binary file and stores its contents in the
     * appropriate member variables.
     * @param numberOfComplexPoints
     * @param fileRef
     * @param timeDomain if true than return the time domain signal.
     * @return 
     */
    public float[] readTDFDFitModelOutputSpectrumFileAsFloatArray( int numberOfComplexPoints, 
                                                                   String fileRef, 
                                                                   boolean timeDomain )
    {
        
        if ( ( new File( fileRef ) ).exists( ) == true )
        {
            float[] absorptionChannelFD = new float[numberOfComplexPoints];
            float[] dispersionChannelFD = new float[numberOfComplexPoints];

            try
            {          
                RandomAccessFile reader = new RandomAccessFile( fileRef, "rw" );

                ByteBuffer buf;
                byte byte0;
                byte byte1;
                byte byte2;
                byte byte3;
                float readFloat;

                for ( int i = 0; i < numberOfComplexPoints; i++ )
                {

                    buf = ByteBuffer.allocate( 4 );

                    byte0 = reader.readByte( );
                    byte1 = reader.readByte( );
                    byte2 = reader.readByte( );
                    byte3 = reader.readByte( );

                    buf.put( byte3 );
                    buf.put( byte2 );
                    buf.put( byte1 );
                    buf.put( byte0 );

                    readFloat = buf.getFloat( 0 );
                    buf.clear( );

                    absorptionChannelFD[i] = readFloat;

                    buf = ByteBuffer.allocate( 4 );

                    byte0 = reader.readByte( );
                    byte1 = reader.readByte( );
                    byte2 = reader.readByte( );
                    byte3 = reader.readByte( );

                    buf.put( byte3 );
                    buf.put( byte2 );
                    buf.put( byte1 );
                    buf.put( byte0 );

                    readFloat = buf.getFloat( 0 );
                    buf.clear( );

                    dispersionChannelFD[i] = readFloat;

                }
                
                boolean noValidValues = false;
                for ( int i = 0; i < absorptionChannelFD.length; i++ )
                {
                    if ( new Double( dispersionChannelFD[i] ).isNaN( ) ) 
                    {
                        noValidValues = true;
                    }
                    if ( new Double( absorptionChannelFD[i] ).isNaN( ) ) 
                    {
                        noValidValues = true;
                    }
                }
                
                if ( noValidValues == true )
                {
                    for ( int i = 0; i < absorptionChannelFD.length; i++ )
                    {
                        absorptionChannelFD[i] = 0f;
                        dispersionChannelFD[i] = 0f;
                    }
                    
                }
                
                // the frequency domain response is read so generate the time 
                // domain response to get full representation
                calculateTDRepresentation( absorptionChannelFD, dispersionChannelFD );

                reader.close( );

            }
            catch ( IOException e )
            {
                e.printStackTrace( );
                System.out.println( "Cannot read tdfdfit-formatted model output datafile." );
                return null;
            }
        }
        else
        {
            return null;
        }
        
        float[] complexOutputSignal = new float[2*numberOfComplexPoints];
        
        if ( timeDomain == false )
        {    
            for ( int i = 0; i < numberOfComplexPoints; i++ )
            {
                complexOutputSignal[2*i]   = this.m_modelRealFDSignal[i];
                complexOutputSignal[2*i+1] = this.m_modelImagFDSignal[i];
            }
        }
        else
        {
            for ( int i = 0; i < numberOfComplexPoints; i++ )
            {
                complexOutputSignal[2*i]   = this.m_modelRealTDSignal[i];
                complexOutputSignal[2*i+1] = this.m_modelImagTDSignal[i];
            }
        }
        
        return complexOutputSignal;
    }
    
    /**
     * Calculates the frequency domain representation of 
     * the absorption and dispersion channels.
     */
    private void calculateTDRepresentation( float[] absorptionFD, 
                                            float[] dispersionFD )
    {
        
        m_numberOfPoints = absorptionFD.length;
        
        m_modelRealFDSignal = new float[ m_numberOfPoints ];
        m_modelImagFDSignal = new float[ m_numberOfPoints ];
        
        for ( int i = 0; i < absorptionFD.length; i++ )
        {
            m_modelRealFDSignal[i] = absorptionFD[i];
            m_modelImagFDSignal[i] = dispersionFD[i];
        }
        
        float[][] fftArray = new float[m_numberOfPoints][2];
        float[] bufAbs = new float[m_numberOfPoints];
        float[] bufDisp = new float[m_numberOfPoints];
               
        for ( int i = 0 ; i < m_numberOfPoints ; i++ )
        {
            fftArray[i][0] = m_modelRealFDSignal[ m_numberOfPoints - 1 - i ];
            fftArray[i][1] = m_modelImagFDSignal[ m_numberOfPoints - 1 - i ];
        }
        
        for ( int i = 0 ; i < m_numberOfPoints ; i++ )
        {
             bufAbs[ i ] = fftArray[i][0];
             bufDisp[ i ] = fftArray[i][1];           
        }
        
        for ( int i = 0 ; i < m_numberOfPoints ; i++ )
        {
            int d = ( i + 1 + m_numberOfPoints / 2 ) % m_numberOfPoints;
            fftArray[d][0] = bufAbs[i];
            fftArray[d][1] = bufDisp[i];
        }
        
        bufAbs = null;
        bufDisp = null;
        
        FFT fft = new FFT( );
        fftArray = fft.fft_1d( fftArray );
        
        m_modelRealTDSignal = new float[m_numberOfPoints];
        m_modelImagTDSignal = new float[m_numberOfPoints];

        for ( int i = 0 ; i < m_numberOfPoints ; i++ )
        {
            //int d = ( i + m_absorptionChannelTD.length / 2 ) % m_absorptionChannelFD.length;
            m_modelRealTDSignal[i] = fftArray[i][0] / m_numberOfPoints;
            m_modelImagTDSignal[i] = fftArray[i][1] / m_numberOfPoints;
        }
        
        fftArray = null;
    }
    
    /**
     * Returns the real part of the fitted model spectrum.
     * @return 
     */
    public float[] getFittedSpectrumRealTD( )
    {
        return m_modelRealTDSignal;
    }
    
     /**
     * Returns the imaginary part of the fitted time domain model spectrum.
     * @return 
     */
    public float[] getFittedSpectrumImagTD( )
    {
        return m_modelImagTDSignal;
    }
    
     /**
     * Returns the real part of the fitted model spectrum.
     * @return 
     */
    public float[] getFittedSpectrumRealFD( )
    {
        return m_modelRealFDSignal;
    }
    
    /**
     * Returns the imaginary part of the fitted model spectrum.
     * @return 
     */
    public float[] getFittedSpectrumImagFD( )
    {
        return m_modelImagFDSignal;
    }
    
     /**
     * Returns the real part of the fitted model spectrum.
     * @param realFittedTD 
     */
    public void setFittedSpectrumRealTD( float[] realFittedTD )
    {
        m_numberOfPoints = realFittedTD.length;
        m_modelRealTDSignal = new float[realFittedTD.length];
        System.arraycopy(realFittedTD, 0, m_modelRealTDSignal, 0, realFittedTD.length );
    }
    
     /**
     * Returns the imaginary part of the fitted time domain model spectrum.
     * @param imagFittedTD 
     */
    public void setFittedSpectrumImagTD( float[] imagFittedTD )
    {
        m_numberOfPoints = imagFittedTD.length;
        m_modelImagTDSignal = new float[imagFittedTD.length];
        System.arraycopy(imagFittedTD, 0, m_modelImagTDSignal, 0, imagFittedTD.length );
    }
    
     /**
     * Returns the real part of the fitted model spectrum.
     * @param realFittedFD 
     */
    public void setFittedSpectrumRealFD( float[] realFittedFD )
    {
        m_numberOfPoints = realFittedFD.length;
        m_modelRealFDSignal = new float[realFittedFD.length];
        System.arraycopy(realFittedFD, 0, m_modelRealFDSignal, 0, realFittedFD.length );
    }
    
    /**
     * Returns the imaginary part of the fitted model spectrum. 
     * @param imagFittedFD
     */
    public void setFittedSpectrumImagFD( float[] imagFittedFD  )
    {
        m_numberOfPoints = imagFittedFD.length;
        m_modelImagFDSignal = new float[imagFittedFD.length];
        System.arraycopy(imagFittedFD, 0, m_modelImagFDSignal, 0, imagFittedFD.length );
    }
       
    /**
     * Returns the fitted elements
     * @return 
     */
    public Vector<TDFDFittingParameters> getFittedParametersVector( )
    {
        return m_fittedElements;
    }
    
    public double getOffsetModelReferenceOf( String comp_name )
    {
        double frequency = 0.0; 
        for ( int i = 0; i < m_fittedElements.size( ); i++ )
        {
            TDFDFittingParameters pars = m_fittedElements.get( i );
            if ( pars.getComponentName( ).contains( comp_name ) )
            {
                frequency = pars.getOffset( );
                break;
            }
        }
        return frequency;           
    }
    
    /**
     * Method to write all additional attributes and fitting parameter attributes
     * to the file system.
     * @param dos
     * @throws IOException 
     */
    public void writeToDataOutputStream( DataOutputStream dos ) throws IOException 
    {
        // all additional String attributes
        dos.writeUTF( m_fileReference );
        dos.writeUTF( m_commandObtainedResultsWith );
        dos.writeUTF( m_typeOfFit );  
        dos.writeUTF( m_fittedModelOutputSpectrumFile );
        
        // all additional double attributes
        dos.writeDouble( m_fitQualityNumber );
        dos.writeDouble( m_stDeviationReal );
        dos.writeDouble( m_stDeviationImag );
        dos.writeDouble( m_chiSqBefore );
        dos.writeDouble( m_chiSqAfter );
        dos.writeDouble( m_trueChiSq );
        
        dos.writeInt( m_lastIter );
        dos.writeInt( m_lastWeightingType );        
        dos.writeInt( m_fittedElements.size( ) );
        
        for ( int i = 0; i < m_fittedElements.size( ); i++ )
        {
            TDFDFittingParameters par = m_fittedElements.get( i );
            par.writeToDataOutputStream( dos );
        }
        
        dos.flush( );
        
    }
    
     /**
     * Method to read all additional attributes and fitting parameter attributes
     * from the file system.
     * @param dis
     * @throws IOException 
     */
    public void readFromDataInputStream( DataInputStream dis ) throws IOException 
    {
        // All additional String attributes
        m_fileReference                 = dis.readUTF( );
        m_commandObtainedResultsWith    = dis.readUTF( );
        m_typeOfFit                     = dis.readUTF( );  
        m_fittedModelOutputSpectrumFile = dis.readUTF( );
        
        // All additional double attributes
        m_fitQualityNumber              = dis.readDouble( );
        m_stDeviationReal               = dis.readDouble( );
        m_stDeviationImag               = dis.readDouble( );
        m_chiSqBefore                   = dis.readDouble( );
        m_chiSqAfter                    = dis.readDouble( );
        m_trueChiSq                     = dis.readDouble( );
        
        // All additional integer attributes
        m_lastIter                      = dis.readInt( );
        m_lastWeightingType             = dis.readInt( );     
        
        int numberOfComponents = dis.readInt( );
        
        for ( int i = 0; i < numberOfComponents; i++ )
        {
            TDFDFittingParameters par = new TDFDFittingParameters( "" );      
            par.readFromDataInputStream( dis );
            m_componentNames.add( par.getComponentName( ) );
            m_fittedElements.add( par );
        }
        
        m_dataLoadedFromFile = true;
    }
    
    void setPreprocessedInputSignalRealTD( float[] absorptionTD ) 
    {   
        m_numberOfPoints = absorptionTD.length;
        m_preProcessedMeasuredTDRealSignal = new float[absorptionTD.length];
        System.arraycopy( absorptionTD, 0, m_preProcessedMeasuredTDRealSignal, 0, absorptionTD.length );
    }

    void setPreprocessedInputSignalImagTD( float[] dispersionTD ) 
    {
        m_numberOfPoints = dispersionTD.length;
        m_preProcessedMeasuredTDImagSignal = new float[dispersionTD.length];
        System.arraycopy( dispersionTD, 0, m_preProcessedMeasuredTDImagSignal, 0, dispersionTD.length );
    }
    
    
    void setPreprocessedInputSignalRealFD( float[] absorptionFD )
    {
        m_numberOfPoints = absorptionFD.length;
        m_preProcessedMeasuredFDRealSignal = new float[absorptionFD.length];
        System.arraycopy( absorptionFD, 0, m_preProcessedMeasuredFDRealSignal, 0, absorptionFD.length );
    }

    void setPreprocessedInputSignalImagFD( float[] dispersionFD ) 
    {
        m_numberOfPoints = dispersionFD.length;
        m_preProcessedMeasuredFDImagSignal = new float[dispersionFD.length];
        System.arraycopy( dispersionFD, 0, m_preProcessedMeasuredFDImagSignal, 0, dispersionFD.length );
    }
    
    
    // Attributes
    private boolean    m_dataLoadedFromFile         = false;
    
    private String     m_fileReference              = "Not defined!";
    private String     m_commandObtainedResultsWith = "Not defined!";
    private String     m_typeOfFit                  = "Not defined!";
    
    // General numeric attributes
    private double m_fitQualityNumber   = 0.0;
    private double m_stDeviationReal    = 0.0;
    private double m_stDeviationImag    = 0.0;
    private double m_chiSqBefore        = 0.0;
    private double m_chiSqAfter         = 0.0;
    private double m_trueChiSq          = 0.0;
    private int    m_lastIter           = -1;
    private int    m_lastWeightingType  = -1;
     
    // Attributes associated to fitted elements (lines, patterns)
    private Vector<String> m_componentNames                 = new Vector<>( );
    private Vector<TDFDFittingParameters> m_fittedElements  = new Vector<>( );
    
    // Some attributes to store the fitted spectrum (signals!) into.
    private float[] m_modelRealTDSignal;
    private float[] m_modelImagTDSignal;
    private float[] m_modelRealFDSignal;
    private float[] m_modelImagFDSignal;
    
    private float[] m_preProcessedMeasuredTDRealSignal;
    private float[] m_preProcessedMeasuredTDImagSignal;
    private float[] m_preProcessedMeasuredFDRealSignal;
    private float[] m_preProcessedMeasuredFDImagSignal;
     
    private String  m_fittedModelOutputSpectrumFile = "Not defined!";
    private int     m_numberOfPoints = 0;

    private String[] areTokensNumeric( String[] tokens ) 
    {
       for ( int i = 0; i < tokens.length; i++ )
       {
           if ( isNumeric( tokens[i] ) == false )
           {
               tokens[i] = "0.0000";
           }
       }
       return tokens;
    }

    /**
     *
     * @param strNum
     * @return
     */
    public static boolean isNumeric( String strNum ) 
    {
        if ( strNum == null ) 
        {
            return false;
        }
        try 
        {
            double d = Double.parseDouble(strNum);
        } 
        catch ( NumberFormatException nfe ) 
        {
            return false;
        }
        return true;
    }
    
}
