/*
 * PatternDefaultsPropertyPanel.java
 *
 * Created on December 3, 2003, 9:11 AM
 */

package org.spectrim.tdfdfit;

import java.awt.Dimension;
import java.awt.BorderLayout;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

import javax.swing.Box;
import javax.swing.JPanel;
import javax.swing.JCheckBox;
import javax.swing.BoxLayout;
import javax.swing.JLabel;
import javax.swing.BorderFactory;

import org.spectrim.lib.DecimalTextField;

/**
 * 
 * Define pattern defaults to be used by the model.
 * 
 * @author  johannes.slotboom@insel.ch
 */
public class PatternDefaultsPropertyPanel extends JPanel implements ActionListener 
{
    
    /** 
     * Creates a new instance of PatternDefaultsPropertyPanel. 
     */
    public PatternDefaultsPropertyPanel( ) 
    {
        
        setLayout( new BorderLayout( ) );
        
        m_contentPanel = new JPanel( );
        
        m_contentPanel.setLayout( new BoxLayout( m_contentPanel, BoxLayout.Y_AXIS ) );
        
        m_contentPanel.setPreferredSize( new Dimension( 600, 100 ) );
        m_contentPanel.setMinimumSize( new Dimension( 600, 100 ) );
        m_contentPanel.setMaximumSize( new Dimension( 600, 100 ) );
        
        m_contentPanel.setBorder( BorderFactory.createCompoundBorder( 
                                  BorderFactory.createTitledBorder( "Export model as pattern" ),
                                  BorderFactory.createEmptyBorder( 5, 5, 5, 5 ) ) );
        
        m_highpassFilterEnablePanel = new JPanel( );
        m_highpassFilterEnablePanel.setLayout( new BoxLayout( m_highpassFilterEnablePanel, BoxLayout.X_AXIS ) );
        
        m_applyLowPassFilterWhenExportModelToPattern = new JCheckBox( "Apply highpass filter when export as pattern" );
        m_applyLowPassFilterWhenExportModelToPattern.addActionListener( this );
        
        m_highpassFilterEnablePanel.add( m_applyLowPassFilterWhenExportModelToPattern );
        m_highpassFilterEnablePanel.add( Box.createHorizontalGlue( ) );
        
        m_highpassCutoffFrequency = new DecimalTextField( "#0.000E0", 5.0f, 0.1f, 0.5f, 1.0f );
        
        m_highpassPanel = new JPanel( );
        m_highpassPanel.setLayout( new BoxLayout(m_highpassPanel, BoxLayout.X_AXIS ) );
        
        m_highpassLabel = new JLabel( "Highpass cut-off frequency " );
        m_highpassCutoffFrequency .setMaximumSize( new Dimension( 100, 25 ) );
        m_highpassCutoffFrequency .setPreferredSize( new Dimension( 100, 25 ) );
        m_highpassCutoffFrequency .setMinimumSize( new Dimension( 100, 25 ) );
        m_highpassCutoffFrequency.addActionListener( this );
        
        m_highpassUnit = new JLabel( "  Hz" );
        
        m_highpassPanel.add( m_highpassLabel );
        m_highpassPanel.add( m_highpassCutoffFrequency );
        m_highpassPanel.add( m_highpassUnit );
        m_highpassPanel.add( Box.createHorizontalGlue( ) );
        
        m_contentPanel.add( m_highpassFilterEnablePanel );
        m_contentPanel.add( m_highpassPanel );
        
        add( m_contentPanel, BorderLayout.NORTH );
        
        TDFDFittingDefaults fdef = TDFDFittingDefaults.getInstance( );
        fdef.load( );
        
        if ( fdef.getProperty( TDFDFittingDefaults.EXPORT_MODEL_AS_PATTERN_APPLY_FILTER, TDFDFittingDefaults.YES ).compareTo( "yes" ) == 0 )
        {
            m_highpassLabel.setEnabled( true );
            m_highpassUnit.setEnabled( true );
            m_highpassCutoffFrequency.setEnabled( true );
            m_applyLowPassFilterWhenExportModelToPattern.setSelected( true );
        }
        else
        {
            m_applyLowPassFilterWhenExportModelToPattern.setSelected( false );
            m_highpassLabel.setEnabled( false );
            m_highpassUnit.setEnabled( false );
            m_highpassCutoffFrequency.setEnabled( false );
        }
        
        m_highpassCutoffFrequency.setFormattedText( fdef.getProperty( TDFDFittingDefaults.PATTERN_EXPORT_CUTOFF_FREQ, "5.000" ) );
        
        setVisible( true );
    }
    
    
    /**
     * Called whenever a registered component raises an ActionEvent.
     * @param actionEvent
     */
    @Override
    public void actionPerformed( ActionEvent actionEvent )
    {
        Object sender = actionEvent.getSource( );
        if ( sender == m_applyLowPassFilterWhenExportModelToPattern )
        {
            if ( m_applyLowPassFilterWhenExportModelToPattern.isSelected( ) == true )
            {
                m_highpassLabel.setEnabled( true );
                m_highpassUnit.setEnabled( true );
                m_highpassCutoffFrequency.setEnabled( true );
                TDFDFittingDefaults fdef = TDFDFittingDefaults.getInstance( );
                fdef.load( );
                
                fdef.setProperty( TDFDFittingDefaults.EXPORT_MODEL_AS_PATTERN_APPLY_FILTER, TDFDFittingDefaults.YES );
                
                fdef.save( );
            }
            else
            {
                m_highpassLabel.setEnabled( false );
                m_highpassUnit.setEnabled( false );
                m_highpassCutoffFrequency.setEnabled( false );
                TDFDFittingDefaults fdef = TDFDFittingDefaults.getInstance( );
                fdef.load( );
                fdef.setProperty( TDFDFittingDefaults.EXPORT_MODEL_AS_PATTERN_APPLY_FILTER, TDFDFittingDefaults.NO );
                fdef.save( );
            }
        }
        if ( sender == this.m_highpassCutoffFrequency )
        {
            TDFDFittingDefaults fdef = TDFDFittingDefaults.getInstance( );
            fdef.load( );
            fdef.setProperty( TDFDFittingDefaults.PATTERN_EXPORT_CUTOFF_FREQ, m_highpassCutoffFrequency.getText( ) );
            fdef.save( );
        }
    }
    
    // Attributes of this panel
    JPanel m_contentPanel;
    JCheckBox m_applyLowPassFilterWhenExportModelToPattern;
    DecimalTextField m_highpassCutoffFrequency;
    
    JPanel m_highpassFilterEnablePanel;
    
    JPanel m_highpassPanel;
    JLabel m_highpassLabel;
    JLabel m_highpassUnit;
    
    // For proper serialization
    private static final long serialVersionUID = 6482325916125123L;

}
