/*
 * File: $Id: QuickSort.java,v 1.1.1.1 2004/12/08 09:38:48 hans Exp $
 * History
 * Date: 08.12.2004  Authors: Margaret Lamb / Hans Slotboom
 *
 * Changes:
 *
 * Copyright (C) 2008 by
 *
 * Inselspital - Department of Neuroradiology
 * CH 3010 Bern Switzerland
 */

/*
 * QuickSort.java
 *
 * Created on December 8, 2004, 10:55 AM
 */

package org.spectrim.tdfdfit;


/************************************************************************
 * Demonstration of a quick sort of an array of ints.
 * This is very much like the code in the course ware.
 * Example for CISC 121A, Queens University, Kingston, Ontario
 * Fall 2002
 *
 * author: Margaret Lamb 
 ***********************************************************************/

public class QuickSort 
{
    /**********************************************************************
     * Swaps two elements of an array of ints.
     *
     * Parameters:
     *   array: the array
     *   index1: the index of the first element to swap
     *   index2: the index of the second element to swap
     *********************************************************************/
    static void swap( int array[], int index1, int index2 ) 
    {
        int temp = array[index1];
        array[index1] = array[index2];
        array[index2] = temp;
    } // end swap

    /**********************************************************************
     * Swaps two elements of an array of ints.
     *
     * Parameters:
     *   array: the float array
     *   index1: the index of the first element to swap
     *   index2: the index of the second element to swap
     *********************************************************************/
    static void swap( float array[], int index1, int index2 ) 
    {
        float temp = array[index1];
        array[index1] = array[index2];
        array[index2] = temp;
    } // end swap
    
    
    /**********************************************************************
     * Swaps two elements of an array of float.
     *
     * Parameters:
     *   array: the float array
     *   index1: the index of the first element to swap
     *   index2: the index of the second element to swap
     *********************************************************************/
    static void swap( int[] order, float array[], int index1, int index2 ) 
    {
        // define the order of the array
        int tempInt = order[index1];
        order[index1] = order[index2];
        order[index2] = tempInt;
        
        float temp = array[index1];
        array[index1] = array[index2];
        array[index2] = temp;
    } // end swap


    /**********************************************************************
     * Swaps two elements of an array of float.
     *
     * Parameters:
     *   array: the float array
     *   index1: the index of the first element to swap
     *   index2: the index of the second element to swap
     *********************************************************************/
    static void swap( int[] order, double array[], int index1, int index2 ) 
    {
        // define the order of the array
        int tempInt = order[index1];
        order[index1] = order[index2];
        order[index2] = tempInt;
        
        double temp = array[index1];
        array[index1] = array[index2];
        array[index2] = temp;
    } // end swap
    
    
    /**********************************************************************
     * Swaps two elements of an array of float.
     *
     * Parameters:
     *   array: the float array
     *   index1: the index of the first element to swap
     *   index2: the index of the second element to swap
     *********************************************************************/
    static void swap( int[] order, String[] array, int index1, int index2 )
    {
        // define the order of the array
        int tempInt = order[index1];
        order[index1] = order[index2];
        order[index2] = tempInt;

        String temp = array[index1];
        array[index1] = array[index2];
        array[index2] = temp;
    } // end swap


    
    /*********************************************************************
     * Recursive method to sort a section of an array of ints in non-decreasing 
     * order.  Uses the quicksort algorithm.  
     *
     * Parameters: 
     *   data: the array to be sorted
     *   first: the first index of the section of the array to sort
     *   last: the last element of the section of the array to sort
     *********************************************************************/
    static void quickSort( int data[], int first, int last ) 
    {
        // Base Case: if this array section has 0 or 1 element, it is already sorted.
        if ( first >= last )
        {
            return;
        }
        // swap the middle element into the first position for use as the pivot value
        swap( data, first, ( first + last ) / 2 );
        int pivot = data[first]; // the pivot value
   
        // Indexes for moving elements in the array.  
        int lower = first+1; 
        int upper = last;
   
        // Our goal is to move lower up and move upper down until they meet.
        // We want to swap elements around so that data[first..lower] <= pivot
        // and data[upper..last] > pivot
        while ( lower <= upper ) 
        {
            // move lower up until we find an element which is > pivot
            while ( data[lower] < pivot ) // correction: my original code had "<="
            {
                lower++;
            }
            // move lower down until we find an element which is <= pivot
            while ( data[upper] > pivot )
            {
                upper--;
            }
            // At this point one of two things have happend:
            //    1. data[lower] >= pivot >= data[upper].
            //       In this case, we swap elements lower and upper.
            //    2. We have run out of elements, so lower >= upper.
            //       In this case, our job is done and we increment
            //       lower to make sure the loop will end.
            if ( lower < upper ) 
            {
                swap( data, lower, upper );
                lower++;
                upper--;
            }
            else 
            {
                lower++;
            } // end if
        } // end while
   
        // Now upper is the index of the position *before* the first
        // element that is > pivot.  This is exactly where the pivot
        // value should go.
   
        // swap the pivot value into its proper place in the array
        swap( data, upper, first );
   
        // sort the two sub-arrays (elements before & after the pivot)
        quickSort( data, first, upper - 1 );
        quickSort( data, upper+1, last );
    } // end three-parameter recursive quickSort

    /*********************************************************************
     * Recursive method to sort a section of an array of ints in non-decreasing 
     * order.  Uses the quicksort algorithm.  
     *
     * Parameters: 
     *   data: the float array to be sorted
     *   first: the first index of the section of the array to sort
     *   last: the last element of the section of the array to sort
     *********************************************************************/
    static void quickSort( float data[], int first, int last ) 
    {
        // Base Case: if this array section has 0 or 1 element, it is already sorted.
        if ( first >= last )
        {
            return;
        }
        // swap the middle element into the first position for use as the pivot value
        swap( data, first, ( first + last ) / 2 );
        float pivot = data[first]; // the pivot value
   
        // Indexes for moving elements in the array.  
        int lower = first + 1; 
        int upper = last;
   
        // Our goal is to move lower up and move upper down until they meet.
        // We want to swap elements around so that data[first..lower] <= pivot
        // and data[upper..last] > pivot
        while ( lower <= upper ) 
        {
            // move lower up until we find an element which is > pivot
            while ( data[lower] < pivot ) // correction: my original code had "<="
            {
                lower++;
            }
            // move lower down until we find an element which is <= pivot
            while ( data[upper] > pivot )
            {
                upper--;
            }
            // At this point one of two things have happend:
            //    1. data[lower] >= pivot >= data[upper].
            //       In this case, we swap elements lower and upper.
            //    2. We have run out of elements, so lower >= upper.
            //       In this case, our job is done and we increment
            //       lower to make sure the loop will end.
            if ( lower < upper ) 
            {
                swap( data, lower, upper );
                lower++;
                upper--;
            }
            else 
            {
                lower++;
            } // end if
        } // end while
   
        // Now upper is the index of the position *before* the first
        // element that is > pivot.  This is exactly where the pivot
        // value should go.
   
        // swap the pivot value into its proper place in the array
        swap( data, upper, first );
   
        // sort the two sub-arrays (elements before & after the pivot)
        quickSort( data, first, upper - 1 );
        quickSort( data, upper+1, last );
    } // end three-parameter recursive quickSort

    /*********************************************************************
     * Recursive method to sort a section of an array of ints in non-decreasing 
     * order.  Uses the quicksort algorithm.  
     *
     * Parameters: 
     *   data: the float array to be sorted
     *   first: the first index of the section of the array to sort
     *   last: the last element of the section of the array to sort
     * 
     * Adapted by HS returns int[] containing new order of elements
     * 
     *********************************************************************/
    static void quickSort( int[] order, float[] data, int first, int last ) 
    {
        
        // Base Case: if this array section has 0 or 1 element, it is already sorted.
        if ( first >= last )
        {
            return;
        }
        // swap the middle element into the first position for use as the pivot value
        swap( order, data, first, ( first + last )/2 );
        float pivot = data[first]; // the pivot value
   
        // Indexes for moving elements in the array.  
        int lower = first + 1;
        int upper = last;
   
        // Our goal is to move lower up and move upper down until they meet.
        // We want to swap elements around so that data[first..lower] <= pivot
        // and data[upper..last] > pivot
        while ( lower <= upper ) 
        {
            // move lower up until we find an element which is > pivot
            while ( data[lower] < pivot ) // correction: my original code had "<="
            {
                lower++;
            }
            // move lower down until we find an element which is <= pivot
            while ( data[upper] > pivot )
            {
                upper--;
            }
            // At this point one of two things have happend:
            //    1. data[lower] >= pivot >= data[upper].
            //       In this case, we swap elements lower and upper.
            //    2. We have run out of elements, so lower >= upper.
            //       In this case, our job is done and we increment
            //       lower to make sure the loop will end.
            if ( lower < upper ) 
            {
                swap( order, data, lower, upper );
                lower++;
                upper--;
            }
            else 
            {
                lower++;
            } // end if
        } // end while
   
        // Now upper is the index of the position *before* the first
        // element that is > pivot.  This is exactly where the pivot
        // value should go.
   
        // swap the pivot value into its proper place in the array
        swap( order, data, upper, first );
   
        // sort the two sub-arrays (elements before & after the pivot)
        quickSort( order, data, first, upper - 1 );
        quickSort( order, data, upper + 1, last );
    } // end three-parameter recursive quickSort

/*********************************************************************
     * Recursive method to sort a section of an array of ints in non-decreasing 
     * order.  Uses the quicksort algorithm.  
     *
     * Parameters: 
     *   data: the float array to be sorted
     *   first: the first index of the section of the array to sort
     *   last: the last element of the section of the array to sort
     * 
     * Adapted by HS returns int[] containing new order of elements
     * 
     *********************************************************************/
    static void quickSort( int[] order, double[] data, int first, int last ) 
    {
        
        // Base Case: if this array section has 0 or 1 element, it is already sorted.
        if ( first >= last )
        {
            return;
        }
        // swap the middle element into the first position for use as the pivot value
        swap( order, data, first, ( first + last )/2 );
        double pivot = data[first]; // the pivot value
   
        // Indexes for moving elements in the array.  
        int lower = first + 1;
        int upper = last;
   
        // Our goal is to move lower up and move upper down until they meet.
        // We want to swap elements around so that data[first..lower] <= pivot
        // and data[upper..last] > pivot
        while ( lower <= upper ) 
        {
            // move lower up until we find an element which is > pivot
            while ( data[lower] < pivot ) // correction: my original code had "<="
            {
                lower++;
            }
            // move lower down until we find an element which is <= pivot
            while ( data[upper] > pivot )
            {
                upper--;
            }
            // At this point one of two things have happend:
            //    1. data[lower] >= pivot >= data[upper].
            //       In this case, we swap elements lower and upper.
            //    2. We have run out of elements, so lower >= upper.
            //       In this case, our job is done and we increment
            //       lower to make sure the loop will end.
            if ( lower < upper ) 
            {
                swap( order, data, lower, upper );
                lower++;
                upper--;
            }
            else 
            {
                lower++;
            } // end if
        } // end while
   
        // Now upper is the index of the position *before* the first
        // element that is > pivot.  This is exactly where the pivot
        // value should go.
   
        // swap the pivot value into its proper place in the array
        swap( order, data, upper, first );
   
        // sort the two sub-arrays (elements before & after the pivot)
        quickSort( order, data, first, upper - 1 );
        quickSort( order, data, upper + 1, last );
    } // end three-parameter recursive quickSort

    /*********************************************************************
     * Recursive method to sort a section of an array of String in 
     * non-decreasing order.  Uses the quicksort algorithm.
     *
     * Parameters:
     *   data: the float array to be sorted
     *   first: the first index of the section of the array to sort
     *   last: the last element of the section of the array to sort
     *
     * Adapted by HS returns int[] containing new order of elements
     *
     *********************************************************************/
    static void quickSort( int[] order, String[] data, int first, int last )
    {

        // Base Case: if this array section has 0 or 1 element, it is already sorted.
        if ( first >= last )
        {
            return;
        }
        // swap the middle element into the first position for use as the pivot value
        swap( order, data, first, ( first + last )/2 );
        String pivot = data[first]; // the pivot value

        // Indexes for moving elements in the array.
        int lower = first + 1;
        int upper = last;

        // Our goal is to move lower up and move upper down until they meet.
        // We want to swap elements around so that data[first..lower] <= pivot
        // and data[upper..last] > pivot
        while ( lower <= upper )
        {
            // move lower up until we find an element which is > pivot
            while ( data[lower].compareTo( pivot ) < 0 ) // correction: my original code had "<="
            {
                lower++;
            }
            // move lower down until we find an element which is <= pivot
            while ( data[upper].compareTo( pivot ) > 0 )
            {
                upper--;
            }
            // At this point one of two things have happend:
            //    1. data[lower] >= pivot >= data[upper].
            //       In this case, we swap elements lower and upper.
            //    2. We have run out of elements, so lower >= upper.
            //       In this case, our job is done and we increment
            //       lower to make sure the loop will end.
            if ( lower < upper )
            {
                swap( order, data, lower, upper );
                lower++;
                upper--;
            }
            else
            {
                lower++;
            } // end if
        } // end while

        // Now upper is the index of the position *before* the first
        // element that is > pivot.  This is exactly where the pivot
        // value should go.

        // swap the pivot value into its proper place in the array
        swap( order, data, upper, first );

        // sort the two sub-arrays (elements before & after the pivot)
        quickSort( order, data, first, upper - 1 );
        quickSort( order, data, upper + 1, last );
    } // end three-parameter recursive quickSort



   /*********************************************************************
    * Sorts an array of ints in non-decreasing order.  Uses the quicksort 
    * algorithm.  This is the top-level method: sets things up and
    * then calls the recursive method that sorts a section of an array.
    *
    * Parameter: the array to be sorted
     * @param data
     * @return 
    *********************************************************************/
    public static int[] quickSort( int data[] ) 
    {
        if ( data.length < 2 ) return data; // already sorted
   
        // Move the largest element of the array into the last position.
        // This saves lots of extra checks in the recursive version.
   
        int maxIndex = 0;
        for ( int i = 1; i < data.length; i++ )
        {
            if ( data[i] > data[maxIndex] )
            {
                maxIndex = i;
            }
        }
        
        swap( data, maxIndex, data.length - 1 );
   
        // Now use the recursive version to sort the rest of the array.
        quickSort( data, 0, data.length - 2 );
        
        return data;
    } // end one-parameter quickSort

    /*********************************************************************
    * Sorts an array of ints in non-decreasing order.  Uses the quicksort 
    * algorithm.  This is the top-level method: sets things up and
    * then calls the recursive method that sorts a section of an array.
    *
    * Parameter: the array to be sorted
     * @param data
     * @return 
    *********************************************************************/
    public static float[] quickSort( float data[] ) 
    {
        if ( data.length < 2 ) return data; // already sorted
   
        // Move the largest element of the array into the last position.
        // This saves lots of extra checks in the recursive version.
   
        int maxIndex = 0;
        for ( int i = 1; i < data.length; i++ )
        {
            if ( data[i] > data[maxIndex] )
            {
                maxIndex = i;
            }
        }
        
        swap( data, maxIndex, data.length - 1 );
   
        // Now use the recursive version to sort the rest of the array.
        quickSort( data, 0, data.length - 2 );
        
        return data;
    } // end one-parameter quickSort

    
    /*********************************************************************
    * Sorts an array of ints in non-decreasing order.  Uses the quicksort 
    * algorithm.  This is the top-level method: sets things up and
    * then calls the recursive method that sorts a section of an array.
    *
    * Parameter: the array to be sorted
     * @param order
     * @param data
     * @return 
    *********************************************************************/
    public static float[] quickSort( int[] order, float data[] ) 
    {
        
        for ( int i = 0; i < data.length; i++ )
        {
            order[i] = i;
        }
        
        if ( data.length < 2 ) return data; // already sorted
   
        // Move the largest element of the array into the last position.
        // This saves lots of extra checks in the recursive version.
   
        int maxIndex = 0;
        for ( int i = 1; i < data.length; i++ )
        {
            if ( data[i] > data[maxIndex] )
            {
                maxIndex = i;
            }
        }
        
        swap( order, data, maxIndex, data.length - 1 );
   
        // Now use the recursive version to sort the rest of the array.
        quickSort( order, data, 0, data.length - 2 );
        
        return data;
    } // end one-parameter quickSort
    

    /*********************************************************************
    * Sorts an array of ints in non-decreasing order.  Uses the quicksort 
    * algorithm.  This is the top-level method: sets things up and
    * then calls the recursive method that sorts a section of an array.
    *
    * Parameter: the array to be sorted
     * @param order
     * @param data
     * @return 
    *********************************************************************/
    public static double[] quickSort( int[] order, double data[] ) 
    {
        
        for ( int i = 0; i < data.length; i++ )
        {
            order[i] = i;
        }
        
        if ( data.length < 2 ) return data; // already sorted
   
        // Move the largest element of the array into the last position.
        // This saves lots of extra checks in the recursive version.
   
        int maxIndex = 0;
        for ( int i = 1; i < data.length; i++ )
        {
            if ( data[i] > data[maxIndex] )
            {
                maxIndex = i;
            }
        }
        
        swap( order, data, maxIndex, data.length - 1 );
   
        // Now use the recursive version to sort the rest of the array.
        quickSort( order, data, 0, data.length - 2 );
        
        return data;
    } // end one-parameter quickSort
    
    
    /*********************************************************************
    * Sorts an array of ints in non-decreasing order.  Uses the quicksort
    * algorithm.  This is the top-level method: sets things up and
    * then calls the recursive method that sorts a section of an array.
    *
    * Parameter: the array to be sorted
     * @param order
     * @param data
     * @return 
    *********************************************************************/
    public static String[] quickSort( int[] order, String[] data )
    {
        
        // System.out.println("calling .. quickSort( int[] order, String[] data )  " + data.length );

        for ( int i = 0; i < data.length; i++ )
        {
            order[i] = i;
        }

        if ( data.length < 2 ) return data; // already sorted

        // Move the largest element of the array into the last position.
        // This saves lots of extra checks in the recursive version.

        int maxIndex = 0;

        for ( int i = 1; i < data.length; i++ )
        {
            if ( data[i].compareTo( data[maxIndex] ) > 0 )
            {
                maxIndex = i;
            }
        }

        swap( order, data, maxIndex, data.length - 1 );

        // Now use the recursive version to sort the rest of the array.
        quickSort( order, data, 0, data.length - 2 );

        return data;
    } // end one-parameter quickSort


    /*********************************************************************
     * Main program for testing
     * @param args
     *********************************************************************/
    public static void main( String args[] ) 
    {
        float[] testArray = new float[10];
        int[] order = new int[10];
        System.out.println( "array before sorting : " );
        for ( int i = 0; i < testArray.length; i++ ) 
        {
            testArray[i] = (float)( 5 * Math.random( ) + 1 );
            System.out.print( testArray[i] + " " );
        }
        
        System.out.println( );
        quickSort( order, testArray );
        
        System.out.println( "array after sorting : " );
        
        for ( int i = 0; i < testArray.length; i++ )
        {
            System.out.println( "Element[" + i + "] = " + testArray[i] + " (" + order[i] + ")" );
        }
        
        System.out.println( );


        String[] testStr = new String[10];

        testStr[0] = "abcdgejewlwkssf";
        testStr[1] = "abcsgejewlwkssf";
        testStr[2] = "axcsgejewlwkssf";
        testStr[3] = "aacsgejewlwkssf";
        testStr[4] = "aaasgejewlwkssf";
        testStr[5] = "abcaaejewlwkssf";
        testStr[6] = "abcsgaaewlwkssf";
        testStr[7] = "abcsgejaalwkssf";
        testStr[8] = "abcsgejewlwassf";
        testStr[9] = "abcsgejewaakssf";


        System.out.println( "String array before sorting : " );
        for ( int i = 0; i < testStr.length; i++ )
        {

             System.out.println( "Element[" + i + "] = " + testStr[i] + " (" + i + ")" );
        }

        System.out.println( );

        testStr = quickSort( order, testStr );

        System.out.println( "String array after sorting : " );

        for ( int i = 0; i < testArray.length; i++ )
        {
            System.out.println( "Element[" + i + "] = " + testStr[i] + " (" + order[i] + ")" );
        }


        
    } // end main
    
} // end class QuickSort