/*
 * File: $Id: SingleVoxelSpectralDataClipboard.java,v 1.1.1.1 2006/04/19 09:38:51 hans Exp $
 * History
 * Date: 04.03.2003  Authors: Hans Slotboom  
 *
 * Changes:
 *
 * Copyright (C) 2003 by
 * 
 * Inselspital - Department of Neuroradiology
 * CH 3010 Bern Switzerland
 */

/*
 * SingleVoxelSpectralDataClipboard.java
 *
 * Created on April 5, 2006, 12:24 PM
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package org.spectrim.tdfdfit;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import java.util.Iterator;
import java.util.LinkedList;


/**
 *  Singleton class that contains one or more copies of a 
 *  SingleVoxelSpectralData object.
 *  This utitlity class allows the copying of polygons amongst
 *  different SingleVoxelSpectrumViewer or objects
 *
 *  @author  slotboom
 */


public class SingleVoxelSpectralDataClipboard
{
    
    /**
     * Returns the one and only instance of PolygonClipboard.
     * @return 
     */
    public static SingleVoxelSpectralDataClipboard getInstance( )
    {
        return theInstance;
    }
    
    /** 
     * Creates a new instance of SingleVoxelSpectralDataClipboard. 
     */
    protected SingleVoxelSpectralDataClipboard( ) 
    {
        // initializing instance fields
    }
    
    /**
     * Returns the clipboard SingleVoxelSpectralData.
     * @return 
     */
    public SingleVoxelSpectralData getSingleVoxelSpectralData( )
    {
        return m_singleVoxelSpectralData;
    }

  
    /**
     * Returns the clipboard SingleVoxelSpectralData-list.
     * @return 
     */
    public LinkedList<SingleVoxelMeasuredSpectralData> getSingleVoxelSpectralDataList( )
    {
        return m_singleVoxelSpectralDataList;
    }

    /**
     * Sets the clipboard SingleVoxelSpectralData object.
     * @param svsData
     */
    public void setSingleVoxelSpectralData( SingleVoxelSpectralData svsData )
    {
        m_singleVoxelSpectralData = new SingleVoxelSpectralData( svsData );
        
        if ( m_singleVoxelSpectralData != null )
        {
            notifyListeners( ADDED_SINGLEVOXELSPECTRALDATA_TO_CLIPBOARD );
        }
    }
    
    /**
     * Sets the clipboard SingleVoxelSpectralData-list.
     * @param svsDataList
     */
    public void setSingleVoxelSpectralDataList( LinkedList<SingleVoxelMeasuredSpectralData> svsDataList )
    {
        m_singleVoxelSpectralDataList = new LinkedList<SingleVoxelMeasuredSpectralData>( );
        
        Iterator it = svsDataList.iterator( );
        while ( it.hasNext( ) )
        {
            // SingleVoxelMeasuredSpectralData svsdata = new SingleVoxelMeasuredSpectralData( (SingleVoxelMeasuredSpectralData)it.next( ) );
            m_singleVoxelSpectralDataList.add( (SingleVoxelMeasuredSpectralData)it.next( ) );
        }
        
        if ( m_singleVoxelSpectralDataList.size( ) > 1 )
        {
            notifyListeners( ADDED_MULTIPLE_SINGLEVOXELSPECTRALDATA_TO_CLIPBOARD );
        }
    }
    
    /**
     * Notifies all registered listeners.
     */
    private void notifyListeners( int eventType  )
    {
        Iterator it = m_listeners.iterator( );
        while( it.hasNext( ) )
        {
            ( ( ActionListener )it.next( ) ).actionPerformed( new ActionEvent( this, eventType, "" ) );
        }
    }

    /**
     * Adds an action listener to the list of action listeners who are notified
     * when a SingleVoxelMeasuredSpectralData-object is added to the clipboard.
     * @param e
     */
    public void addActionListener( ActionListener e )
    {
	m_listeners.add( e );
    }
    
    /**
     * Returns true if the m_singleVoxelSpectralDataList.length >0 or
     * m_singleVoxelSpectralData unequal to null.
     * @return 
     */
    public boolean containsData( )
    {
        if ( ( m_singleVoxelSpectralData != null ) ||  ( m_singleVoxelSpectralDataList.size( ) > 0  ) )
        {
            return true;
        }
        else
        {
            return false;
        }
    }
    
    // static private attribute is the instance itself
    private static final SingleVoxelSpectralDataClipboard theInstance = new SingleVoxelSpectralDataClipboard( );
    
    // Members to add one polygon to the clipboard
    private SingleVoxelSpectralData m_singleVoxelSpectralData;
   
    // Members to add a list of polygons to the clipboard
    private LinkedList<SingleVoxelMeasuredSpectralData> m_singleVoxelSpectralDataList = new LinkedList<>( );
    
    // Listeners to changes in the segmentation gui control
    private LinkedList<ActionListener> m_listeners = new LinkedList<ActionListener>( );
    
    // Event typs of the clipboard object
    public final static int ADDED_SINGLEVOXELSPECTRALDATA_TO_CLIPBOARD = 0;
    public final static int ADDED_MULTIPLE_SINGLEVOXELSPECTRALDATA_TO_CLIPBOARD = 1;
     
}
