/*
 * File: $Id: SingleVoxelSpectrumViewer.java,v 1.2 2006/09/01 09:14:19 hans Exp $
 * History
 * Date: 09.05.2003, 12:06  Authors: Hans Slotboom  
 *
 * Changes:
 *
 * Copyright (C) 2003 by
 * 
 * Inselspital - Department of Neuroradiology
 * CH 3010 Bern Switzerland
 */


package org.spectrim.tdfdfit;

import java.awt.Color;
import org.spectrim.lib.Dialogs;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

import javax.swing.JMenuItem;
import javax.swing.JRadioButtonMenuItem;
import javax.swing.JMenu;
import javax.swing.ButtonGroup;
import javax.swing.JFrame;

import java.util.Iterator;
import java.util.LinkedList;

import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.Robot;
import java.awt.event.InputEvent;
import java.awt.geom.Point2D;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import javax.imageio.ImageIO;
import javax.swing.JFileChooser;
import org.spectrim.dicom.DicomDirectory;
import org.spectrim.hlsvd.HLSVDControlFrame;
import org.spectrim.hlsvd.HLSVDControlPanel;
import org.spectrim.waterremoval.SVDWaterLineRemoverGUI;
import org.spectrim.xyplot.Curve;

import org.spectrim.xyplot.XYPlot;
import org.spectrim.xyplot.XYPlotFrame;
import org.spectrim.xyplot.XYPlotRange;

/** 
 * Class allowing the viewing and manipulation of single voxel spectral data.
 * @author hans
 */
public class SingleVoxelSpectrumViewer extends XYPlot implements ActionListener, MouseListener, MouseMotionListener
{

    /**
     * Default constructor of a SingleVoxelSpectrumViewer.
     */
    public SingleVoxelSpectrumViewer( )
    {
        super( null , null );
        
        this.addMouseListener( this );
        this.addMouseMotionListener( this );

        m_svsDataClipboard.addActionListener( this );

        addPopupMenu( );

        if ( m_svsDataClipboard.containsData( ) == true )
        {
            m_pasteMenu.setEnabled( true );
            m_pasteSingleVoxelSpectrumDataMenuItem.setEnabled( true );
            m_pasteAndAddSingleVoxelSpectrumDataMenuItem.setEnabled( true );
            m_pasteAndSubtractSingleVoxelSpectrumDataMenuItem.setEnabled( true );
        }
    }
    
    /** 
     * Creates a new instance of SingleVoxelSpectrumViewer. 
     * @param title
     * @param parent
     * @param ddir
     */
    public SingleVoxelSpectrumViewer( String title, JFrame parent, DicomDirectory ddir )
    {
        super( title , parent );
        m_parent = parent;
        m_dicomDir = ddir;
        
        this.addMouseListener( this );
        this.addMouseMotionListener( this );
        
        m_svsDataClipboard.addActionListener( this );
        
        addPopupMenu( );
        
        if ( m_svsDataClipboard.containsData( ) == true )
        {
            m_pasteMenu.setEnabled( true );
            m_pasteSingleVoxelSpectrumDataMenuItem.setEnabled( true );
            m_pasteAndAddSingleVoxelSpectrumDataMenuItem.setEnabled( true );
            m_pasteAndSubtractSingleVoxelSpectrumDataMenuItem.setEnabled( true );
        }
        
    }
      
    /**
     * Enables or disables the appearance of the JPopupMenu.
     * @param enable 
     */
    public void enablePopupMenu( boolean enable )
    {
        if ( m_domainSelector != null )
        {
            m_domainSelector.setEnabled( enable );
        }

        if ( m_channelSelector != null )
        {
            m_channelSelector.setEnabled( enable );
        }

        if ( m_displayUnitSelector != null )
        {
            m_displayUnitSelector.setEnabled( enable );
        }

        if ( m_signalProcessing != null )
        {
            m_signalProcessing.setEnabled( enable );
        }

        if ( m_copySingleVoxelSpectrumDataMenuItem != null )
        {
            m_copySingleVoxelSpectrumDataMenuItem.setEnabled( enable );
        }

        if ( m_cloneSingleVoxelSpectrumDataMenuItem != null )
        {
            m_cloneSingleVoxelSpectrumDataMenuItem.setEnabled( enable );
        }
        
        if ( m_pasteMenu != null )
        {
            m_pasteMenu.setEnabled( enable );
        }

        if ( m_pasteSingleVoxelSpectrumDataMenuItem != null )
        {
            m_pasteSingleVoxelSpectrumDataMenuItem.setEnabled( enable );
        }

        if ( m_pasteAndAddSingleVoxelSpectrumDataMenuItem != null )
        {
            m_pasteAndAddSingleVoxelSpectrumDataMenuItem.setEnabled( enable );
        }

        if ( m_pasteAndSubtractSingleVoxelSpectrumDataMenuItem != null )
        {
            m_pasteAndSubtractSingleVoxelSpectrumDataMenuItem.setEnabled( enable );
        }
        // The displayer can show time or frequency domain data
        if ( m_timeDomain != null )
        {
            m_timeDomain.setEnabled( enable );
        }

        if ( m_frequencyDomain != null )
        {
            m_frequencyDomain.setEnabled( enable );
        }
        // The channel selector allows the user to select the following channels
        if ( m_absorption != null )
        {
            m_absorption.setEnabled( enable );
        }

        if ( m_dispersion != null )
        {
            m_dispersion.setEnabled( enable );
        }

        if ( m_magnitude != null )
        {
            m_magnitude.setEnabled( enable );
        }

        if ( m_power != null )
        {
            m_power.setEnabled( enable );
        }

        if ( m_phase != null )
        {
            m_phase.setEnabled( enable );
        }
        // Signal prosessing menu items
        if ( m_applyProcessing != null )
        {
            m_applyProcessing.setEnabled( enable );
        }

        if ( m_setApodizationParameters != null )
        {
            m_setApodizationParameters.setEnabled( enable );
        }

        if ( m_multiplyFirstTDPoint != null )
        {
            m_multiplyFirstTDPoint.setEnabled( enable );
        }

        if ( m_offsetCorrectionFD != null )
        {
            m_offsetCorrectionFD.setEnabled( enable );
        }

        if ( m_setZeroFillingParameters != null )
        {
            m_setZeroFillingParameters.setEnabled( enable );
        }

        if ( m_setPhasingParameters != null )
        {
            m_setPhasingParameters.setEnabled( enable );
        }

        if ( m_setShiftParameter != null )
        {
            m_setShiftParameter.setEnabled( enable );
        }

        if ( m_frequencyDependentBaselineElimination != null )
        {
            m_frequencyDependentBaselineElimination.setEnabled( enable );
        }

        if ( m_removeMacromolecularBaseline != null )
        {
            m_removeMacromolecularBaseline.setEnabled( enable );
        }

        if ( m_saveScreenShot != null )
        {
            m_saveScreenShot.setEnabled( enable );
        }

        if ( m_displayPointsOnlyUnit != null )
        {
            m_displayPointsOnlyUnit.setEnabled( enable );
        }

        if ( m_displaySecondsOrHertzUnit != null )
        {
            m_displaySecondsOrHertzUnit.setEnabled( enable );
        }

        if ( m_displaySecondsOrPPMUnit != null )
        {
            m_displaySecondsOrPPMUnit.setEnabled( enable );
        }

        if ( m_channelSelector != null )
        {
            m_channelSelector.setEnabled( enable );
        }
    }

    /**
     * Adds a context or popup menu to the spectrum viewer that allows the
     * manipulation of data.
     */
    private void addPopupMenu( )
    {
        m_popupmenu.addSeparator( );
        
        m_cloneSingleVoxelSpectrumDataMenuItem= new JMenuItem( "Clone data in new frame" );
        m_cloneSingleVoxelSpectrumDataMenuItem.addActionListener( this );
        m_cloneSingleVoxelSpectrumDataMenuItem.setEnabled( false );
        m_popupmenu.add( m_cloneSingleVoxelSpectrumDataMenuItem );
        
        
        m_copySingleVoxelSpectrumDataMenuItem = new JMenuItem( "Copy spectral data view" );
        m_copySingleVoxelSpectrumDataMenuItem.addActionListener( this );
        m_copySingleVoxelSpectrumDataMenuItem.setEnabled( false );
        m_popupmenu.add( m_copySingleVoxelSpectrumDataMenuItem );
        
        m_pasteMenu = new JMenu( "Paste" );
        m_pasteMenu.setToolTipText( "Allows advanced pasting of SVS Data sets" );
        m_pasteMenu.setEnabled( false );
        m_pasteSingleVoxelSpectrumDataMenuItem = new JMenuItem( "Paste spectral data in new frame" );
        m_pasteSingleVoxelSpectrumDataMenuItem.addActionListener( this );
        m_pasteSingleVoxelSpectrumDataMenuItem.setEnabled( false );
        m_pasteMenu.add( m_pasteSingleVoxelSpectrumDataMenuItem );
        
        m_pasteAndAddSingleVoxelSpectrumDataMenuItem = new JMenuItem( "Paste and add SVS-data");
        m_pasteAndAddSingleVoxelSpectrumDataMenuItem.addActionListener(this);
        m_pasteAndAddSingleVoxelSpectrumDataMenuItem.setEnabled( false ); 
        m_pasteMenu.add( m_pasteAndAddSingleVoxelSpectrumDataMenuItem );
        
        m_pasteAndSubtractSingleVoxelSpectrumDataMenuItem = new JMenuItem( "Paste and subtract SVS-data");
        m_pasteAndSubtractSingleVoxelSpectrumDataMenuItem.addActionListener(this);
        m_pasteAndSubtractSingleVoxelSpectrumDataMenuItem.setEnabled( false ); 
        m_pasteMenu.add( m_pasteAndSubtractSingleVoxelSpectrumDataMenuItem );
        
        m_popupmenu.add( m_pasteMenu );
        m_popupmenu.addSeparator( );
        
        // The popup menu is instantiated in the base class and is extended 
        // below
        TDFDFittingDefaults fitdef = TDFDFittingDefaults.getInstance( );
        fitdef.load( );
        String fittingDomain = fitdef.getProperty( TDFDFittingDefaults.DISPLAY_DOMAIN );
        String displayChannel = fitdef.getProperty( TDFDFittingDefaults.DISPLAY_CHANNEL );
        
        m_domainSelector = new JMenu( "Display domain" );
        m_domainSelector.setToolTipText( "Select time or frequency display domain" );
        m_domainSelector.addActionListener( this );
        
        m_timeDomain = new JRadioButtonMenuItem( "Time domain" );
        m_timeDomain.addActionListener( this );
        
        m_frequencyDomain = new JRadioButtonMenuItem( "Frequency domain" );
        m_frequencyDomain.addActionListener( this );
        
        m_domainButtonGroup = new ButtonGroup( );
        
        m_domainButtonGroup.add( m_timeDomain );
        m_domainButtonGroup.add( m_frequencyDomain );
        
        m_domainSelector.add( m_timeDomain );
        m_domainSelector.add( m_frequencyDomain );
        
        m_popupmenu.add( m_domainSelector );
        
        if ( fittingDomain.compareTo( TDFDFittingDefaults.FREQ_DOMAIN ) == 0 )
        {
            m_frequencyDomain.setSelected( true );
        }
        else
        {
            m_timeDomain.setSelected( true );
        }
        
        
        m_channelSelector = new JMenu( "Display channel" );
        m_channelSelector.setToolTipText( "Allows the selection of which channel has to be displayed" );
        
        m_absorption = new JRadioButtonMenuItem( "Absorption" );
        m_absorption.setToolTipText( "Displays the absorption (real) channel" );
        m_absorption.addActionListener( this );
        
        m_dispersion = new JRadioButtonMenuItem( "Dispersion" );
        m_dispersion.setToolTipText( "Displays the dispersion (imaginary) channel" );
        m_dispersion.addActionListener( this );
        
        m_magnitude = new JRadioButtonMenuItem( "Magnitude" );
        m_magnitude.setToolTipText( "Displays the magnitude representation of absorption and dispersion channel" );
        m_magnitude.addActionListener( this );    

        m_power = new JRadioButtonMenuItem( "Power" );
        m_power.setToolTipText( "Displays the power representation of absorption and dispersion channel" );
        m_power.addActionListener( this );          
        
        m_phase = new JRadioButtonMenuItem( "Phase" );
        m_phase.setToolTipText( "Displays the magnitude of absorption and dispersion channel" );
        m_phase.addActionListener( this );    
        
        m_channelSelector.add( m_absorption );
        m_channelSelector.add( m_dispersion );
        m_channelSelector.add( m_magnitude );
        m_channelSelector.add( m_power );
        m_channelSelector.add( m_phase );
         
        m_channelButtonGroup = new ButtonGroup( );
        
        m_channelButtonGroup.add( m_absorption );
        m_channelButtonGroup.add( m_dispersion );
        m_channelButtonGroup.add( m_magnitude );
        m_channelButtonGroup.add( m_power );
        m_channelButtonGroup.add( m_phase );
        
        if ( displayChannel.compareTo( TDFDFittingDefaults.ABSORPTION ) == 0 )
        {
            m_absorption.setSelected( true );
        }

        if ( displayChannel.compareTo( TDFDFittingDefaults.DISPERSION ) == 0 )
        {
            m_dispersion.setSelected( true );
        }

        if ( displayChannel.compareTo( TDFDFittingDefaults.MAGNITUDE ) == 0 )
        {
            m_magnitude.setSelected( true );
        }
        
        if ( displayChannel.compareTo( TDFDFittingDefaults.POWER ) == 0 )
        {
            m_power.setSelected( true );
        }

        if ( displayChannel.compareTo( TDFDFittingDefaults.PHASE ) == 0 )
        {
            m_phase.setSelected( true );
        }

        m_popupmenu.add( m_channelSelector );
        
        m_displayUnitSelector = new JMenu( "Display units" );
        
        m_displayPointsOnlyUnit = new JRadioButtonMenuItem( "Points - Points" );
        m_displayPointsOnlyUnit.addActionListener( this );
        m_displaySecondsOrHertzUnit = new JRadioButtonMenuItem( "Seconds - Hz" );
        m_displaySecondsOrHertzUnit.addActionListener( this );
        m_displaySecondsOrPPMUnit = new JRadioButtonMenuItem( "Seconds - ppm");
        m_displaySecondsOrPPMUnit.addActionListener( this );
        
        m_displayUnitSelector.add( m_displayPointsOnlyUnit );
        m_displayUnitSelector.add( m_displaySecondsOrHertzUnit );
        m_displayUnitSelector.add( m_displaySecondsOrPPMUnit );
        
        m_unitSelectionButtonGroup = new ButtonGroup( );
        m_unitSelectionButtonGroup.add( m_displayPointsOnlyUnit );
        m_unitSelectionButtonGroup.add( m_displaySecondsOrHertzUnit );
        m_unitSelectionButtonGroup.add( m_displaySecondsOrPPMUnit );
        
        String selectedUnit = fitdef.getProperty( TDFDFittingDefaults.DISPLAY_UNIT );

        if ( ( selectedUnit.compareTo(TDFDFittingDefaults.FREQUENCY_DOMAIN_POINTS ) == 0 ) ||
             ( selectedUnit.compareTo(TDFDFittingDefaults.TIME_DOMAIN_POINTS ) == 0 ) )
        {
            m_displayPointsOnlyUnit.setSelected( true );
        }

        if ( ( selectedUnit.compareTo(TDFDFittingDefaults.FREQUENCY_DOMAIN_HZ ) == 0 ) ||
             ( selectedUnit.compareTo(TDFDFittingDefaults.TIME_DOMAIN_SECONDS ) == 0 ) )
        {
            m_displaySecondsOrHertzUnit.setSelected( true );
        }
        
        if ( ( selectedUnit.compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_PPM ) == 0 ) ||
             ( selectedUnit.compareTo( TDFDFittingDefaults.TIME_DOMAIN_SECONDS ) == 0 ) )
        {
            m_displaySecondsOrPPMUnit.setSelected( true );
        }
        
        m_popupmenu.add( m_displayUnitSelector );
        
        m_signalProcessing = new JMenu( "Signal processing" );
        m_signalProcessing.setToolTipText( "Allows you to enable processing and define processing parameters" );
        
        m_applyProcessing= new JRadioButtonMenuItem( "Apply signal processing");
        m_applyProcessing.setToolTipText( "Apply zerofilling, apodization and phasing before FFT" );
        m_applyProcessing.addActionListener( this );
        
        m_applyProcessing.setSelected( true );
        
        m_signalProcessing.add( m_applyProcessing );
        
        // Spectrim SVD water removal
        m_setPerformSVDWaterRemoval = new JMenuItem( "SVD-water removal .." );
        m_setPerformSVDWaterRemoval.setToolTipText( "Removes residual water signal using simple SVD" );
        m_setPerformSVDWaterRemoval.addActionListener( this );
        m_signalProcessing.add( m_setPerformSVDWaterRemoval );
        
        // Spectrim SVD water removal
        m_setPerformHLSVDWaterRemoval = new JMenuItem( "HLSVD-water removal .." );
        m_setPerformHLSVDWaterRemoval.setToolTipText( "Launches HLSVD control panel." );
        m_setPerformHLSVDWaterRemoval.addActionListener( this );
        m_signalProcessing.add( m_setPerformHLSVDWaterRemoval );
        
        
        m_setApodizationParameters = new JMenuItem( "Apodization .." );
        m_setApodizationParameters.setToolTipText( "Allows you to set the apodization time constants" );
        m_setApodizationParameters.addActionListener( this );
        m_signalProcessing.add( m_setApodizationParameters );

        m_multiplyFirstTDPoint = new JMenuItem( "Multiply first TD samplepoint .." );
        m_multiplyFirstTDPoint.setToolTipText( "Allows for spectral offset removal" );
        m_multiplyFirstTDPoint.addActionListener( this );
        m_signalProcessing.add( m_multiplyFirstTDPoint );
   
        m_offsetCorrectionFD = new JMenuItem( "Offset correction FD .." );
        m_offsetCorrectionFD.setToolTipText( "Allows for frequency domain offset removal" );
        m_offsetCorrectionFD.addActionListener( this );
        m_signalProcessing.add( m_offsetCorrectionFD );
        
        m_setZeroFillingParameters = new JMenuItem( "Zero filling .." );
        m_setZeroFillingParameters.setToolTipText( "Allows you to set the zero filling factor" );
        m_setZeroFillingParameters.addActionListener( this );
        m_signalProcessing.add( m_setZeroFillingParameters );
        
        m_setPhasingParameters = new JMenuItem( "Phasing .." );
        m_setPhasingParameters.setToolTipText( "Allows you to set the frequency domain phasing parameters" );
        m_setPhasingParameters.addActionListener( this );
        
        m_signalProcessing.add( m_setPhasingParameters );
        
        m_setShiftParameter = new JMenuItem( "Shift .." );
        m_setShiftParameter.setToolTipText( "Allows you to set the amount of Hz the spectrum should be shifted" );
        m_setShiftParameter.addActionListener( this );
        
        m_signalProcessing.add( m_setShiftParameter );
        
        m_frequencyDependentBaselineElimination = new JMenuItem( "Define baseline for MM baseline elimination .." );
        
        m_frequencyDependentBaselineElimination.setToolTipText( "Allows for broad baseline elimination using splines" );
        m_frequencyDependentBaselineElimination.addActionListener( this );
        m_signalProcessing.add( m_frequencyDependentBaselineElimination );
        
        m_removeMacromolecularBaseline = new JMenuItem( "Remove broad baseline" );
        m_removeMacromolecularBaseline.addActionListener( this );
        m_removeMacromolecularBaseline.setEnabled( false );
        
        m_signalProcessing.add( m_removeMacromolecularBaseline );
        
        
        m_popupmenu.addSeparator( );
        
        m_saveScreenShot = new JMenuItem( "Save screenshot .." );
        m_saveScreenShot.setToolTipText( "Saves a copy of the currently displayed spectrum to a file" );
        m_saveScreenShot.addActionListener( this );
        
        m_popupmenu.add( m_saveScreenShot );
        
        m_popupmenu.add( m_signalProcessing );
        
        add( m_popupmenu );
    }
    
    /**
     * Creates a new instance of SingleVoxelSpectrumViewer.
     * @param title
     * @param specDataName
     * @param specData the spectral data object containing the complex data points
     * @param parent
     */
    public SingleVoxelSpectrumViewer( String title, 
                                      String specDataName, 
                                      SingleVoxelMeasuredSpectralData specData, 
                                      JFrame parent )
    {
        super( title , parent );
        
        m_parent = parent;
        
        this.addMouseListener( this );
        this.addMouseMotionListener( this );
        
        addPopupMenu( );
        
        m_responseList.add( specData );
        m_responseNameList.add( specDataName );
        
        if ( m_applyProcessing.isSelected( ) )
        {
            m_sdp = new SingleVoxelSpectralDataProcessing( m_responseList );
            m_responseList = m_sdp.zeroFill( 1.0f );
        }

        // get the default Hertz representation
        addCurve( specData.getAbsorptionCurveFDHertz( ), true );

    }

    
    /**
     * Sets the spectral data of the viewer and displays the absorption 
     * channel curve in the view by default.
     *
     * @param specDataName
     * @param specData the spectral data object containing the complex data points
     * @param refresh
     */
    public void addSpectralDataSet( String specDataName, 
                                    SingleVoxelMeasuredSpectralData specData, 
                                    boolean refresh )
    {
        m_responseNameList.add( specDataName );
        m_responseList.add( specData );
        m_responseListBuffer.add( specData );
        
        m_copySingleVoxelSpectrumDataMenuItem.setEnabled( true );
        m_cloneSingleVoxelSpectrumDataMenuItem.setEnabled( true );
        
        TDFDFittingDefaults def = TDFDFittingDefaults.getInstance( );
        def.load( );
        
        // Which unit type does the user want to be displayed is stored and updated
        // in the MRFittingDefault class
        String displayType = def.getProperty( TDFDFittingDefaults.DISPLAY_UNIT );
        String displayChannel = def.getProperty( TDFDFittingDefaults.DISPLAY_CHANNEL );
        String displayDomain = def.getProperty( TDFDFittingDefaults.DISPLAY_DOMAIN );
         
        if ( displayDomain.compareTo( TDFDFittingDefaults.FREQ_DOMAIN ) == 0 )
        {
            if ( displayChannel.compareTo( TDFDFittingDefaults.ABSORPTION ) == 0 )
            {
                if ( displayType.compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_POINTS ) == 0 )
                {
                    addCurve( specData.getAbsorptionCurveFDPoints( ) , refresh );
                }
                if ( displayType.compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_HZ ) == 0 )
                {
                    addCurve( specData.getAbsorptionCurveFDHertz( ) , refresh );
                }
        
                if ( displayType.compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_PPM ) == 0 )
                {
                    addCurve( specData.getAbsorptionCurveFDPPM( ) , refresh );
                }
            }

            if ( displayChannel.compareTo( TDFDFittingDefaults.DISPERSION ) == 0 )
            {
                if ( displayType.compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_POINTS ) == 0 )
                {
                    addCurve( specData.getDispersionCurveFDPoints( ) , refresh );
                }
                if ( displayType.compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_HZ ) == 0 )
                {
                    addCurve( specData.getDispersionCurveFDHertz( ) , refresh );
                }
        
                if ( displayType.compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_PPM ) == 0 )
                {
                    addCurve( specData.getDispersionCurveFDPPM( ) , refresh );
                }
            }

            if ( displayChannel.compareTo( TDFDFittingDefaults.MAGNITUDE ) == 0 )
            {
                if ( displayType.compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_POINTS ) == 0 )
                {
                    addCurve( specData.getMagnitudeCurveFDPoints( ) , refresh );
                }
                if ( displayType.compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_HZ ) == 0 )
                {
                    addCurve( specData.getMagnitudeCurveFDHertz( ) , refresh );
                }
        
                if ( displayType.compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_PPM ) == 0 )
                {
                    addCurve( specData.getMagnitudeCurveFDPPM( ) , refresh );
                }
            }

            if ( displayChannel.compareTo( TDFDFittingDefaults.POWER ) == 0 )
            {
                if ( displayType.compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_POINTS ) == 0 )
                {
                    addCurve( specData.getPowerCurveFDPoints( ) , refresh );
                }
                if ( displayType.compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_HZ ) == 0 )
                {
                    addCurve( specData.getPowerCurveFDHertz( ) , refresh );
                }
        
                if ( displayType.compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_PPM ) == 0 )
                {
                    addCurve( specData.getPowerCurveFDPPM( ) , refresh );
                }
            }

            if ( displayChannel.compareTo( TDFDFittingDefaults.PHASE ) == 0 )
            {
                if ( displayType.compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_POINTS ) == 0 )
                {
                    addCurve( specData.getPhaseCurveFDPoints( ) , refresh );
                }
                if ( displayType.compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_HZ ) == 0 )
                {
                    addCurve( specData.getPhaseCurveFDHertz( ) , refresh );
                }
        
                if ( displayType.compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_PPM ) == 0 )
                {
                    addCurve( specData.getPhaseCurveFDPPM( ) , refresh );
                }
            }
        }
        
        if ( displayDomain.compareTo( TDFDFittingDefaults.TIME_DOMAIN ) == 0 )
        {
            if ( displayChannel.compareTo( TDFDFittingDefaults.ABSORPTION ) == 0 )
            {
                if ( displayType.compareTo( TDFDFittingDefaults.TIME_DOMAIN_POINTS ) == 0 )
                {
                    addCurve( specData.getAbsorptionCurveTDPoints( ) , refresh );
                }
                if ( displayType.compareTo( TDFDFittingDefaults.TIME_DOMAIN_SECONDS ) == 0 )
                {
                    addCurve( specData.getAbsorptionCurveTDSeconds(), refresh );
                }
            }

            if ( displayChannel.compareTo( TDFDFittingDefaults.DISPERSION ) == 0 )
            {
                if ( displayType.compareTo( TDFDFittingDefaults.TIME_DOMAIN_POINTS ) == 0 )
                {
                    addCurve( specData.getDispersionCurveTDPoints( ) , refresh );
                }
                if ( displayType.compareTo( TDFDFittingDefaults.TIME_DOMAIN_SECONDS ) == 0 )
                {
                    addCurve( specData.getDispersionCurveTDSeconds( ), refresh );
                }
            }

            if ( displayChannel.compareTo( TDFDFittingDefaults.MAGNITUDE ) == 0 )
            {
                if ( displayType.compareTo( TDFDFittingDefaults.TIME_DOMAIN_POINTS ) == 0 )
                {
                    addCurve( specData.getMagnitudeCurveTDPoints( ) , refresh );
                }
                if ( displayType.compareTo( TDFDFittingDefaults.TIME_DOMAIN_SECONDS ) == 0 )
                {
                    addCurve( specData.getMagnitudeCurveTDSeconds( ), refresh );
                }
            }
            
            if ( displayChannel.compareTo( TDFDFittingDefaults.POWER ) == 0 )
            {
                if ( displayType.compareTo( TDFDFittingDefaults.TIME_DOMAIN_POINTS ) == 0 )
                {
                    addCurve( specData.getPowerCurveTDPoints( ) , refresh );
                }
                if ( displayType.compareTo( TDFDFittingDefaults.TIME_DOMAIN_SECONDS ) == 0 )
                {
                    addCurve( specData.getPowerCurveTDSeconds( ), refresh );
                }
            }

            if ( displayChannel.compareTo( TDFDFittingDefaults.PHASE ) == 0 )
            {
                if ( displayType.compareTo( TDFDFittingDefaults.TIME_DOMAIN_POINTS ) == 0 )
                {
                    addCurve( specData.getPhaseCurveTDPoints( ) , refresh );
                }
                if ( displayType.compareTo( TDFDFittingDefaults.TIME_DOMAIN_SECONDS ) == 0 )
                {
                    addCurve( specData.getPhaseCurveTDSeconds( ), refresh );
                }
            }
            
        }
        
        // After a response has been added the m_firstTimeChanged must be set
        // to true in order to be able to update the m_responseListBuffer correctly
        m_firstTimeChanged = true;
    }   
    
    /**
     * Called when the mouse button is clicked.
     */
    @Override
    public void mouseClicked( MouseEvent mouseEvent ) 
    {
    }
    
    /**
     * Called when the mouse is dragged.
     */
    @Override
    public void mouseDragged( MouseEvent mouseEvent )
    {
    }
    
    /**
     * Called when the mouse enters the control.
     */    
    @Override
    public void mouseEntered( MouseEvent mouseEvent )
    {
    }
    
    /**
     * Called when the mouse exits.
     */    
    @Override
    public void mouseExited( MouseEvent mouseEvent )
    {
    }
    
    /**
     * Called when the mouse button is released.
     */    
    @Override
    public void mouseMoved( MouseEvent mouseEvent )
    {
    }
    
    /**
     * Called when the mouse button is pressed.
     */    
    @Override
    public void mousePressed( MouseEvent mouseEvent )
    { 
        
        super.mousePressed( mouseEvent );
        
        // Check whether the GUI is in the mode where mouse clicks are awaited 
        if ( m_baseLineDefinitionModeActive == true )
        {
            // Is only of interest if left button is pressed
            if ( ( mouseEvent.getModifiers( ) & InputEvent.BUTTON1_MASK ) == InputEvent.BUTTON1_MASK )
            {
                int mouseButtonPressedX = mouseEvent.getX( );
                int mouseButtonPressedY = mouseEvent.getY( );
           
                determineValuesAtCursorPosition( new Point( mouseButtonPressedX, mouseButtonPressedY ) );            
                setCursor( Cursor.getPredefinedCursor(  Cursor.DEFAULT_CURSOR ) );
                notifyActionListeners( BASELINE_POINT_SELECTED_EVENT, "add user indicated line" );
            }
        }
        else
        {
            // super.mousePressed( mouseEvent );
            if ( mouseEvent.isPopupTrigger( ) )
            {
                m_popupmenu.show( mouseEvent.getComponent( ), 
                                  mouseEvent.getX( ), mouseEvent.getY( ) );
            }
        }
    }
    
    /**
     * Called when the mouse button is released.
     */
    @Override
    public void mouseReleased( MouseEvent mouseEvent )
    {
        if ( mouseEvent.isPopupTrigger( ) )
        {
            m_popupmenu.show( mouseEvent.getComponent( ), 
                              mouseEvent.getX( ), mouseEvent.getY( ) );
        }
    }
    
    /**
     * Changes the displayed signal based on the state members that
     * indicate channel type and domain type.
     */
    public void changeDisplayedSignal( )
    {
        
        // delete all curves from the vector of curves to display
        deleteAllCurves( );
        
        TDFDFittingDefaults fitdef = TDFDFittingDefaults.getInstance( );
        fitdef.load( );
        
        String displayDomain    = fitdef.getProperty( TDFDFittingDefaults.DISPLAY_DOMAIN );
        String displayChannel   = fitdef.getProperty( TDFDFittingDefaults.DISPLAY_CHANNEL );
        String displayUnits     = fitdef.getProperty( TDFDFittingDefaults.DISPLAY_UNIT );
        
        if ( displayDomain.compareTo( TDFDFittingDefaults.FREQ_DOMAIN ) == 0 )
        {
            if ( displayChannel.compareTo( TDFDFittingDefaults.ABSORPTION ) == 0 )
            {
                Iterator it = this.m_responseList.iterator( );
                while ( it.hasNext( ) )
                { 
                    SingleVoxelSpectralData specData = (SingleVoxelSpectralData)it.next( );
                    
                    // change the x-axis
                    specData.autoSetXaxis( );
                    
                    if ( displayUnits.compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_POINTS ) == 0 )
                    {
                        addCurve( specData.getAbsorptionCurveFDPoints( ), false );
                    }
                    if ( displayUnits.compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_HZ ) == 0 )
                    {
                        addCurve( specData.getAbsorptionCurveFDHertz( ), false );
                    }
                    if ( displayUnits.compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_PPM ) == 0 )
                    {
                        addCurve( specData.getAbsorptionCurveFDPPM( ), false );
                    }
                }
            }
            
            if ( displayChannel.compareTo( TDFDFittingDefaults.DISPERSION ) == 0 )
            {
                Iterator it = this.m_responseList.iterator( );
                while ( it.hasNext( ) )
                {
                    SingleVoxelSpectralData specData = (SingleVoxelSpectralData)it.next( );
                    
                    // change the x-axis
                    specData.autoSetXaxis( );
                    
                    if ( displayUnits.compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_POINTS ) == 0 )
                    {
                        addCurve( specData.getDispersionCurveFDPoints( ), false );
                    }
                    
                    if ( displayUnits.compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_HZ ) == 0 )
                    {
                        addCurve( specData.getDispersionCurveFDHertz( ), false );
                    }
                    
                    if ( displayUnits.compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_PPM ) == 0 )
                    {
                        addCurve( specData.getDispersionCurveFDPPM( ), false );
                    }
                }
            }
            
            if ( displayChannel.compareTo( TDFDFittingDefaults.MAGNITUDE ) == 0)
            {
               Iterator it = this.m_responseList.iterator( );
                while ( it.hasNext( ) )
                {
                    SingleVoxelSpectralData specData = (SingleVoxelSpectralData)it.next( );
                    
                    // change the x-axis
                    specData.autoSetXaxis( );
                    
                    if ( displayUnits.compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_POINTS ) == 0 )
                    { 
                        addCurve( specData.getMagnitudeCurveFDPoints( ), false );
                    }
                    
                    if ( displayUnits.compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_HZ ) == 0 )
                    { 
                        addCurve( specData.getMagnitudeCurveFDHertz( ), false );
                    }
                    
                    if ( displayUnits.compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_PPM ) == 0 )
                    { 
                        addCurve( specData.getMagnitudeCurveFDPPM( ), false );
                    }
                } 
            }
            
            if ( displayChannel.compareTo( TDFDFittingDefaults.POWER ) == 0 )
            {
                Iterator it = this.m_responseList.iterator( );
                while ( it.hasNext( ) )
                {
                    SingleVoxelSpectralData specData = (SingleVoxelSpectralData)it.next( );
                    
                    // change the x-axis
                    specData.autoSetXaxis( );
                    
                    if ( displayUnits.compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_POINTS ) == 0 )
                    { 
                        addCurve( specData.getPowerCurveFDPoints( ), false );
                    }
                    
                    if ( displayUnits.compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_HZ ) == 0 )
                    { 
                        addCurve( specData.getPowerCurveFDHertz( ), false );
                    }
                    
                    if ( displayUnits.compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_PPM ) == 0 )
                    { 
                        addCurve( specData.getPowerCurveFDPPM( ), false );
                    }
                }
            }

            if ( displayChannel.compareTo( TDFDFittingDefaults.PHASE ) == 0 )
            {
                Iterator it = this.m_responseList.iterator( );
                while ( it.hasNext( ) )
                {
                    SingleVoxelSpectralData specData = (SingleVoxelSpectralData)it.next( );
                    // change the x-axis
                    specData.autoSetXaxis( );
                    
                    if (  displayUnits.compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_POINTS ) == 0 )
                    {
                        addCurve( specData.getPhaseCurveFDPoints( ), false );
                    }
                    
                    if (  displayUnits.compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_HZ ) == 0)
                    {
                        addCurve( specData.getPhaseCurveFDHertz( ), false );
                    }

                    if ( displayUnits.compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_PPM ) == 0 )
                    {
                        addCurve( specData.getPhaseCurveFDPPM( ), false );
                    }
                }
            }
        }
        
        if ( displayDomain.compareTo( TDFDFittingDefaults.TIME_DOMAIN ) == 0 )
        {
            if ( displayChannel.compareTo( TDFDFittingDefaults.ABSORPTION ) == 0 )
            {
                Iterator it = this.m_responseList.iterator( );
                while ( it.hasNext( ) )
                {
                    
                    SingleVoxelSpectralData specData = (SingleVoxelSpectralData)it.next( );
                    // change the x-axis
                    specData.autoSetXaxis( );
                    
                    if ( displayUnits.compareTo( TDFDFittingDefaults.TIME_DOMAIN_POINTS ) == 0)
                    {
                        addCurve( specData.getAbsorptionCurveTDPoints( ), false );
                    }
                    
                    if ( displayUnits.compareTo( TDFDFittingDefaults.TIME_DOMAIN_SECONDS ) == 0)
                    {
                        addCurve( specData.getAbsorptionCurveTDPoints( ), false );
                    }
                    
                }
            }
            
            if ( displayChannel.compareTo( TDFDFittingDefaults.DISPERSION ) == 0 )
            {
                Iterator it = this.m_responseList.iterator( );
                while ( it.hasNext( ) )
                {
                    
                    SingleVoxelSpectralData specData = (SingleVoxelSpectralData)it.next( );
                    // change the x-axis
                    specData.autoSetXaxis( );
                    
                    if ( displayUnits.compareTo( TDFDFittingDefaults.TIME_DOMAIN_POINTS ) == 0 )
                    {
                        addCurve( specData.getDispersionCurveTDPoints( ), false );
                    }
                    
                    if ( displayUnits.compareTo( TDFDFittingDefaults.TIME_DOMAIN_SECONDS ) == 0 )
                    {
                        addCurve( specData.getDispersionCurveTDSeconds( ), false );
                    }
                }
            }
            
            if ( displayChannel.compareTo( TDFDFittingDefaults.MAGNITUDE ) == 0 )
            {
                Iterator it = this.m_responseList.iterator( );
                while ( it.hasNext( ) )
                {
                    SingleVoxelSpectralData specData = (SingleVoxelSpectralData)it.next( );
                    
                    // change the x-axis
                    specData.autoSetXaxis( );
                    
                    if ( displayUnits.compareTo( TDFDFittingDefaults.TIME_DOMAIN_POINTS ) == 0 )
                    {
                        addCurve( specData.getMagnitudeCurveTDPoints( ), false );
                    }

                    if ( displayUnits.compareTo( TDFDFittingDefaults.TIME_DOMAIN_SECONDS ) == 0)
                    {
                        addCurve( specData.getMagnitudeCurveTDSeconds( ), false );
                    }
                }
            }
            
            if ( displayChannel.compareTo( TDFDFittingDefaults.POWER ) == 0 )
            {
                Iterator it = this.m_responseList.iterator( );
                while ( it.hasNext( ) )
                {
                    
                    SingleVoxelSpectralData specData = (SingleVoxelSpectralData)it.next( );
                    
                    // change the x-axis
                    specData.autoSetXaxis( );
                    
                    if ( displayUnits.compareTo( TDFDFittingDefaults.TIME_DOMAIN_POINTS ) == 0 )
                    {
                        addCurve( specData.getPowerCurveTDPoints( ), false );
                    }
                    
                    if ( displayUnits.compareTo( TDFDFittingDefaults.TIME_DOMAIN_SECONDS ) == 0 )
                    {
                        addCurve( specData.getPowerCurveTDSeconds( ), false );
                    }
                }
            }

            if ( displayChannel.compareTo( TDFDFittingDefaults.PHASE ) == 0 )
            {
                Iterator it = this.m_responseList.iterator( );
                while ( it.hasNext( ) )
                { 
                    SingleVoxelSpectralData specData = (SingleVoxelSpectralData)it.next( );
                    // change the x-axis
                    specData.autoSetXaxis( );
                    
                    if ( displayUnits.compareTo( TDFDFittingDefaults.TIME_DOMAIN_SECONDS ) == 0 )
                    {
                        addCurve( specData.getPhaseCurveTDPoints( ), false );
                    }
                    
                    if ( displayUnits.compareTo( TDFDFittingDefaults.TIME_DOMAIN_SECONDS ) == 0 )
                    {
                        addCurve( specData.getPhaseCurveTDSeconds( ), false );
                    }
                }
            }
        }
        
        // Directly call refresh in order to minimize flickering
        autoDeterminePlotRanges( );  
        refresh( );
    }
    
    
    /**
     * This method copies the response list: this is necessary in case the user
     * cancels a post processing step without having applied it at least once.
     */
    private void copyResponseList( ) 
    {
        Iterator it = m_responseList.iterator( );
        while ( it.hasNext( ) )
        {
            SingleVoxelMeasuredSpectralData data = (SingleVoxelMeasuredSpectralData)it.next( );
            m_responseListBuffer.add( new SingleVoxelMeasuredSpectralData( data ) );
        }
    }
    
    
    /**
     * Called whenever a actionEvent occurs.
     */
    @Override
    public void actionPerformed( ActionEvent e )
    {
        // call base class
        super.actionPerformed( e );
        
        Object sender = e.getSource( );
        
        
        // Allows the user to define baseline ranges and subtraction of a spline approximation
        if ( sender ==  m_frequencyDependentBaselineElimination )
        {
            //set display to FD and total range
            
            m_baseLineDefinitionModeActive = true;
            
            m_baselineDefFrame = new BaseLineSplineSupportPointsDefinitionFrame( m_responseList.getFirst( ) );
            
            addActionListener( m_baselineDefFrame );
            
            m_baselineDefFrame.addActionListener( this );
             
        }
        
        
        if ( sender == m_removeMacromolecularBaseline )
        {
            removeBaseline( );
        }
        
        
        // called whenever the baseline definition frame raises an accept event
        if ( sender == m_baselineDefFrame )
        {
            
            if ( e.getID( ) == m_baselineDefFrame.ACCEPT_EVENT )
            {
                m_baseLineRangesDataModel = m_baselineDefFrame.getRangeModel( );
                
                m_baseLineDefinitionModeActive = false;
                
                if ( m_baseLineRangesDataModel.getSize( ) > 2 )
                {
                    // Based on the range information
                    determineBaseLineSpline( );
                    m_removeMacromolecularBaseline.setEnabled( true );
                }
                
                m_parent.update( m_parent.getGraphics( ) );
            }
            
            if ( e.getID( ) == m_baselineDefFrame.CANCEL_EVENT )
            {
                m_baseLineDefinitionModeActive = false;
                m_parent.update( m_parent.getGraphics( ) );
            }
            
            if ( e.getID( ) == m_baselineDefFrame.PREVIEW_EVENT )
            {
                m_parent.update( m_parent.getGraphics( ) );
            }
            
        }
        
        if ( sender == m_saveScreenShot )
        {
            storeScreenShotInFile( );   
        }
        
        if ( sender == m_svsDataClipboard )
        {
             if ( e.getID( ) ==  m_svsDataClipboard.ADDED_SINGLEVOXELSPECTRALDATA_TO_CLIPBOARD )
            {
                m_pasteMenu.setEnabled( true );
                m_pasteSingleVoxelSpectrumDataMenuItem.setEnabled( true );
                m_pasteAndAddSingleVoxelSpectrumDataMenuItem.setEnabled( true );
                m_pasteAndSubtractSingleVoxelSpectrumDataMenuItem.setEnabled( true );
            }
        }
        
        if ( sender == m_cloneSingleVoxelSpectrumDataMenuItem )
        {
            // data needs to be cloned: the clipboard has this functionality
            m_svsDataClipboard.setSingleVoxelSpectralDataList( m_responseList );
            // add this cloned date
            LinkedList<SingleVoxelMeasuredSpectralData> list = m_svsDataClipboard.getSingleVoxelSpectralDataList( );
            SingleVoxelSpectrumViewerFrame frame = new SingleVoxelSpectrumViewerFrame( list, m_dicomDir, null );
            
        }
        
        if ( sender == m_copySingleVoxelSpectrumDataMenuItem )
        {
            m_svsDataClipboard.setSingleVoxelSpectralDataList( m_responseList );
            m_pasteMenu.setEnabled( true );
            m_pasteSingleVoxelSpectrumDataMenuItem.setEnabled( true );
            m_pasteAndAddSingleVoxelSpectrumDataMenuItem.setEnabled( true );
            m_pasteAndSubtractSingleVoxelSpectrumDataMenuItem.setEnabled( true );
        }
        
        if ( sender == m_pasteSingleVoxelSpectrumDataMenuItem )
        {
            LinkedList<SingleVoxelMeasuredSpectralData> list = m_svsDataClipboard.getSingleVoxelSpectralDataList( );
            
            SingleVoxelSpectrumViewerFrame frame = new SingleVoxelSpectrumViewerFrame( list, m_dicomDir, null );
            
        }
        
        if ( sender == m_pasteAndAddSingleVoxelSpectrumDataMenuItem )
        {
            LinkedList<SingleVoxelMeasuredSpectralData> list = m_svsDataClipboard.getSingleVoxelSpectralDataList( );
            
            Iterator it = list.iterator( );
            
            Iterator it2 = this.m_responseList.iterator( );
            
            int i = 1;
            while ( it.hasNext( ) )
            {
                SingleVoxelSpectralData pasteSVSData = (SingleVoxelSpectralData)it.next( );
                SingleVoxelSpectralData thisSVSData = (SingleVoxelSpectralData)it2.next( );
                thisSVSData.addSingleVoxelSpectralDataObject( pasteSVSData );
            }
            
            m_responseListBuffer.clear();
            changeDisplayedSignal( );  
        }
        
        if ( sender == m_pasteAndSubtractSingleVoxelSpectrumDataMenuItem )
        {
            LinkedList<SingleVoxelMeasuredSpectralData> list = m_svsDataClipboard.getSingleVoxelSpectralDataList( );
            
            Iterator it = list.iterator( );
            
            Iterator it2 = this.m_responseList.iterator( );
            
            int i = 1;
            while ( it.hasNext( ) )
            {
                SingleVoxelSpectralData pasteSVSData = (SingleVoxelSpectralData)it.next( );
                SingleVoxelSpectralData thisSVSData = (SingleVoxelSpectralData)it2.next( );
                thisSVSData.subtractSingleVoxelSpectralDataObject( pasteSVSData );
            }
            m_responseListBuffer.clear( );
            changeDisplayedSignal( );  
        }
        
        
        // handles response user selects time domain representation
        if ( sender == m_timeDomain )
        {
            TDFDFittingDefaults fitdef = TDFDFittingDefaults.getInstance( );
            fitdef.load( );
            
            fitdef.setProperty( TDFDFittingDefaults.DISPLAY_DOMAIN, TDFDFittingDefaults.TIME_DOMAIN );
            fitdef.setProperty( TDFDFittingDefaults.DISPLAY_UNIT, TDFDFittingDefaults.TIME_DOMAIN_SECONDS );
            fitdef.save( );
            
            m_displayXReversed = false;
            
            // save the current range values
            if ( m_xyForcedPlotRange != null )
            {
                m_xyForcedPlotRangeFD = m_xyForcedPlotRange;
            }

            // get the old forced plotting range
            if ( m_xyForcedPlotRangeTD != null )
            {
                m_xyForcedPlotRange = m_xyForcedPlotRangeTD;
            }
            else
            {
                autoDeterminePlotRanges( );
                m_xyForcedPlotRange = this.m_xyAutoPlotRange;
            }
            m_responseListBuffer.clear( );
            changeDisplayedSignal( );            
        }
       
        // handles response user selects frequency domain representation
        if ( sender == m_frequencyDomain )
        {
                       
            TDFDFittingDefaults fitdef = TDFDFittingDefaults.getInstance( );
            fitdef.load( );
            
            fitdef.setProperty( TDFDFittingDefaults.DISPLAY_DOMAIN, TDFDFittingDefaults.FREQ_DOMAIN );
            fitdef.setProperty( TDFDFittingDefaults.DISPLAY_UNIT, TDFDFittingDefaults.FREQUENCY_DOMAIN_HZ );
            
            fitdef.save( );
            
            // save the current range values
            if ( m_xyForcedPlotRange != null )
            {
                m_xyForcedPlotRangeTD = m_xyForcedPlotRange;
            }
            
            // get the old forced plotting range
            if ( m_xyForcedPlotRangeFD != null )
            {
                m_xyForcedPlotRange = m_xyForcedPlotRangeFD;
            }
            else
            {
                autoDeterminePlotRanges( );
                m_xyForcedPlotRange = this.m_xyAutoPlotRange;
            }
            m_responseListBuffer.clear( );
            changeDisplayedSignal( );
        }
        
        // handles response user selects display of absorption channel
        if ( sender == m_absorption )
        {
            TDFDFittingDefaults fitdef = TDFDFittingDefaults.getInstance( );
            fitdef.load( );
            
            fitdef.setProperty( TDFDFittingDefaults.DISPLAY_CHANNEL, 
                                TDFDFittingDefaults.ABSORPTION );
            
            fitdef.save( );
            
            m_responseListBuffer.clear( );
            changeDisplayedSignal( );
        }

        // handles response user selects display of absorption channel
        if ( sender == m_dispersion )
        {
            TDFDFittingDefaults fitdef = TDFDFittingDefaults.getInstance( );
            fitdef.load( );
            
            fitdef.setProperty( TDFDFittingDefaults.DISPLAY_CHANNEL, 
                                TDFDFittingDefaults.DISPERSION );
            
            fitdef.save( );
            m_responseListBuffer.clear( );
            changeDisplayedSignal( );            
        }

        // handles response user selects display of the magnitude representation
        if ( sender == m_magnitude )
        {
            TDFDFittingDefaults fitdef = TDFDFittingDefaults.getInstance( );
            fitdef.load( );
            fitdef.setProperty( TDFDFittingDefaults.DISPLAY_CHANNEL, 
                                TDFDFittingDefaults.MAGNITUDE );
            fitdef.save( );
            m_responseListBuffer.clear( );
            changeDisplayedSignal( );
        }
        
        // handles response user selects display of the power representation
        if ( sender == m_power )
        {
            TDFDFittingDefaults fitdef = TDFDFittingDefaults.getInstance( );
            fitdef.load( );
            fitdef.setProperty( TDFDFittingDefaults.DISPLAY_CHANNEL, 
                                TDFDFittingDefaults.POWER );
            fitdef.save( );
            m_responseListBuffer.clear( );
            changeDisplayedSignal( );            
        }
        
        // Handles response user selects display of the phase representation
        if ( sender == m_phase )
        {
            TDFDFittingDefaults fitdef = TDFDFittingDefaults.getInstance( );
            fitdef.load( );
            fitdef.setProperty( TDFDFittingDefaults.DISPLAY_CHANNEL, 
                                TDFDFittingDefaults.PHASE );
            fitdef.save( );
            m_responseListBuffer.clear( );
            changeDisplayedSignal( );
        }
  
        if ( sender == m_setApodizationParameters )
        {
            launchApodization( );
        }
        
        if ( sender ==  this.m_multiplyFirstTDPoint )
        {
            launchFirstTDPointMultiplication( );
        }
        
        if ( sender ==  this.m_offsetCorrectionFD )
        {
            
            this.launchFDOffsetCorrection( );
           
        }
        
        if ( sender == m_setZeroFillingParameters )
        {
            
            launchZeroFilling( );
            
        }
        
        if ( sender == m_setPhasingParameters )
        {
            this.launchManualPhasing( );
        }
        
        if ( sender == this.m_setPerformHLSVDWaterRemoval )
        { 
           m_sdp = new SingleVoxelSpectralDataProcessing( m_responseList );
           copyResponseList( );
           
           m_hlsvdWaterRemovalGUI = new HLSVDControlFrame( m_responseList.getFirst( ), true );
           m_hlsvdWaterRemovalGUI.addActionListener( this );
           
        }
        
        if ( sender == this.m_hlsvdWaterRemovalGUI )
        {
            
            if ( e.getID( ) == HLSVDControlPanel.RETURN_HLSVD_FILTERED_SIGNAL_ID )
            {
                
                LinkedList<SingleVoxelMeasuredSpectralData> filteredData = m_hlsvdWaterRemovalGUI.getHLSVDFilteredSignalList( );
                
                m_hlsvdWaterRemovalGUI.getHLSVDFilteredSignalList( ).getFirst( ).doComplexConjugation( );
                
                m_responseListBuffer.clear( );
                m_firstTimeChanged = true;
                //m_responseList = m_sdp.getResponseList( );
                m_responseList = filteredData;
                // sent notification that measured spectrum has changed
                int index = -1;
                int cntr = 0;
                Iterator it = m_responseNameList.iterator( );
                while ( it.hasNext( ) )
                {
                    String str = (String)it.next( );
                    if ( str.compareTo( "measured" ) == 0 )
                    {
                        index = cntr;
                    }
                    cntr++;
                }
                
                if ( index != -1 )
                {
                    notifyMeasuredSpectrumChanged( m_responseList.get( index ) );
                }
                
                changeDisplayedSignal( );
            }
        }
        
        
        if ( sender == this.m_setPerformSVDWaterRemoval )
        {
            launchSVDInteractiveWaterProcessing( );            
        }    
            
            
        if ( sender == m_svdWaterRemovalGUI )
        {
            if ( e.getID( ) == m_svdWaterRemovalGUI.CANCEL_EVENT )
            {
               
                if ( !m_responseListBuffer.isEmpty( ) )
                {
                    // Restore old values
                    m_responseList.clear( );
                    // m_responseNameList.clear( );
                
                    Iterator it = this.m_responseListBuffer.iterator( );
                    while ( it.hasNext( ) )
                    {    
                        SingleVoxelMeasuredSpectralData specDataBuffer = (SingleVoxelMeasuredSpectralData)it.next( );
                        SingleVoxelMeasuredSpectralData specData = new SingleVoxelMeasuredSpectralData( specDataBuffer );
                        
                        m_responseList.add( specData );
                    }
                }
              
                m_firstTimeChanged = true;
                changeDisplayedSignal( );
            }
            
            if ( e.getID( ) == m_svdWaterRemovalGUI.ACCEPT_EVENT )
            {   
                m_responseListBuffer.clear( );
                m_firstTimeChanged = true;
                m_responseList = m_sdp.getResponseList( );
                
                // sent notification that measured spectrum has changed
                int index = -1;
                int cntr = 0;
                Iterator it = m_responseNameList.iterator( );
                while ( it.hasNext( ) )
                {
                    String str = (String)it.next( );
                    if ( str.compareTo( "measured" ) == 0 )
                    {
                        index = cntr;
                    }
                    cntr++;
                }
                
                if ( index != -1 )
                {
                    notifyMeasuredSpectrumChanged( m_responseList.get( index ) );
                }
                
                changeDisplayedSignal( );
            }
            
            if ( e.getID( ) == m_svdWaterRemovalGUI.CHANGE_EVENT )
            {
                
                m_responseList.clear( );
                m_responseList.add( m_svdWaterRemovalGUI.getFilteredOutputSpectrum( ) );
                changeDisplayedSignal( );
                
//                if ( m_firstTimeChanged == true )
//                {
//                    m_responseListBuffer.clear( );
//                    
//                    Iterator it = this.m_responseList.iterator( );
//                    while ( it.hasNext( ) )
//                    {
//                        SingleVoxelMeasuredSpectralData specData = (SingleVoxelMeasuredSpectralData)it.next( );                        
//                        SingleVoxelMeasuredSpectralData specDataBuffer = new SingleVoxelMeasuredSpectralData( specData );
//
//                        m_responseListBuffer.add( specDataBuffer );
//                        
//                    }
//                    m_firstTimeChanged = false;
//                }
//                
//                m_sdp = new SingleVoxelSpectralDataProcessing( m_responseListBuffer );
//                // m_responseList = m_sdp.performSimpleSVDBasedWaterRemoval( );
//                changeDisplayedSignal( );
            }
        }
        
        
        if ( sender == m_zerofillDefiner )
        {
            
            if ( e.getID( ) == m_zerofillDefiner.CANCEL_EVENT )
            {
               
                if ( !m_responseListBuffer.isEmpty( ) )
                {
                    // Restore old values
                    m_responseList.clear( );
                    // m_responseNameList.clear( );
                
                    Iterator it = this.m_responseListBuffer.iterator( );
                    while ( it.hasNext( ) )
                    {    
                        SingleVoxelMeasuredSpectralData specDataBuffer = (SingleVoxelMeasuredSpectralData)it.next( );
                        SingleVoxelMeasuredSpectralData specData = new SingleVoxelMeasuredSpectralData( specDataBuffer );
                        
                        m_responseList.add( specData );
                    }
                }
              
                m_firstTimeChanged = true;
                changeDisplayedSignal( );
            }
            
            if ( e.getID( ) == m_zerofillDefiner.ACCEPT_EVENT )
            {   
                m_responseListBuffer.clear( );
                m_firstTimeChanged = true;
                m_responseList = m_sdp.getResponseList( );
                
                // sent notification that measured spectrum has changed
                int index = -1;
                int cntr = 0;
                Iterator it = m_responseNameList.iterator( );
                while ( it.hasNext( ) )
                {
                    String str = (String)it.next( );
                    if ( str.compareTo( "measured" ) == 0 )
                    {
                        index = cntr;
                    }
                    cntr++;
                }
                
                if ( index != -1 )
                {
                    notifyMeasuredSpectrumChanged( m_responseList.get( index ) );
                }
                
                changeDisplayedSignal( );
            }
            
            if ( e.getID( ) == m_zerofillDefiner.CHANGE_EVENT )
            {
                if ( m_firstTimeChanged == true )
                {
                    m_responseListBuffer.clear( );
                    
                    Iterator it = this.m_responseList.iterator( );
                    while ( it.hasNext( ) )
                    {
                        SingleVoxelMeasuredSpectralData specData = (SingleVoxelMeasuredSpectralData)it.next( );                        
                        SingleVoxelMeasuredSpectralData specDataBuffer = new SingleVoxelMeasuredSpectralData( specData );

                        m_responseListBuffer.add( specDataBuffer );
                        
                    }
                    m_firstTimeChanged = false;
                }
                
                m_sdp = new SingleVoxelSpectralDataProcessing( m_responseListBuffer );
                m_responseList = m_sdp.zeroFill( m_zerofillDefiner.getSelectedZerofillfactor( ) );
                changeDisplayedSignal( );
            }
        }
        
        // Handles apodization control events ...
        if ( sender == m_apodizationDefiner )
        {
            if ( e.getID( ) == m_apodizationDefiner.CANCEL_EVENT )
            {
                if ( !m_responseListBuffer.isEmpty( ) )
                {
                    // Restore old values
                    m_responseList.clear( );
                    // m_responseNameList.clear( );
                
                    Iterator it = this.m_responseListBuffer.iterator( );
                    while ( it.hasNext( ) )
                    {    
                        SingleVoxelMeasuredSpectralData specDataBuffer = (SingleVoxelMeasuredSpectralData)it.next( );
                        SingleVoxelMeasuredSpectralData specData = new SingleVoxelMeasuredSpectralData( specDataBuffer );
                        
                        m_responseList.add( specData );
                    }
                }
              
                m_firstTimeChanged = true;
                changeDisplayedSignal( );
            }
            
            if ( e.getID( ) == ApodizationParameterDefinitionControl.ACCEPT_EVENT )
            {
                //////////////////////
                if ( m_firstTimeChanged == true )
                {
                    m_responseListBuffer.clear( );
                    
                    Iterator it = this.m_responseList.iterator( );
                    while ( it.hasNext( ) )
                    {
                        SingleVoxelMeasuredSpectralData specData = (SingleVoxelMeasuredSpectralData)it.next( );                        
                        SingleVoxelMeasuredSpectralData specDataBuffer = new SingleVoxelMeasuredSpectralData( specData );

                        m_responseListBuffer.add( specDataBuffer );
                        
                    }
                    m_firstTimeChanged = false;
                }
                
                m_sdp = new SingleVoxelSpectralDataProcessing( m_responseListBuffer );
                
                m_responseList = m_sdp.apodize( m_apodizationDefiner.getSelectedLorentzianTC( ), 
                                                m_apodizationDefiner.getSelectedGaussianTC( ) );
                changeDisplayedSignal( );
                //////////////////////
                m_responseListBuffer.clear( );
                m_firstTimeChanged = true;
                m_responseList = m_sdp.getResponseList( );
                
                int index = -1;
                int cntr = 0;
                Iterator it = m_responseNameList.iterator( );
                while ( it.hasNext( ) )
                {
                    String str = (String)it.next( );
                    if ( str.compareTo( "measured" ) == 0 )
                    {
                        index = cntr;
                    }
                    cntr++;
                }
                if ( index != -1 )
                {
                    notifyMeasuredSpectrumChanged( m_responseList.get( index ) );
                }
                
                changeDisplayedSignal( );
            }
            
            if ( e.getID( ) == ApodizationParameterDefinitionControl.GAUSSIAN_TIME_CONSTANT_CHANGE_EVENT  )
            {
                if ( m_firstTimeChanged == true )
                {
                    m_responseListBuffer.clear( );
                    
                    Iterator it = this.m_responseList.iterator( );
                    while ( it.hasNext( ) )
                    {
                        SingleVoxelMeasuredSpectralData specData = (SingleVoxelMeasuredSpectralData)it.next( );
                        SingleVoxelMeasuredSpectralData specDataBuffer = new SingleVoxelMeasuredSpectralData( );
                        
                        m_responseListBuffer.add( specDataBuffer );
                    }
                    m_firstTimeChanged = false;

                }
                
                m_sdp = new SingleVoxelSpectralDataProcessing( m_responseListBuffer );
                m_responseList = m_sdp.apodize( m_apodizationDefiner.getSelectedLorentzianTC( ), 
                                                m_apodizationDefiner.getSelectedGaussianTC( ) );
                changeDisplayedSignal( );                
                
            }
            
            if ( e.getID( ) == m_apodizationDefiner.LORENTZIAN_TIME_CONSTANT_CHANGED_EVENT  )
            {
                if ( m_firstTimeChanged == true )
                {
                    m_responseListBuffer.clear( );
                    
                    Iterator it = this.m_responseList.iterator( );
                    while ( it.hasNext( ) )
                    {
                        SingleVoxelMeasuredSpectralData specData = (SingleVoxelMeasuredSpectralData)it.next( );
                        
                        SingleVoxelMeasuredSpectralData specDataBuffer = new SingleVoxelMeasuredSpectralData( specData );
                        
                        m_responseListBuffer.add( specDataBuffer );
                    }
                    m_firstTimeChanged = false;
                }
                
                m_sdp = new SingleVoxelSpectralDataProcessing( m_responseListBuffer );
                m_responseList = m_sdp.apodize( m_apodizationDefiner.getSelectedLorentzianTC( ), 
                                                m_apodizationDefiner.getSelectedGaussianTC( ) );
                changeDisplayedSignal( );
            }
        }
        
        // Handles fd domain offset elimination events ...
       
        if ( sender ==  m_fdOffsetRemoverControl )
        {
            if ( e.getID( ) == m_fdOffsetRemoverControl.CANCEL_EVENT )
            {
                if ( !m_responseListBuffer.isEmpty( ) )
                {
                    // Restore old values
                    m_responseList.clear( );
                    // m_responseNameList.clear( );
                
                    Iterator it = this.m_responseListBuffer.iterator( );
                    while ( it.hasNext( ) )
                    {    
                        SingleVoxelMeasuredSpectralData specDataBuffer = (SingleVoxelMeasuredSpectralData)it.next( );
                        SingleVoxelMeasuredSpectralData specData = new SingleVoxelMeasuredSpectralData( specDataBuffer );
                        
                        m_responseList.add( specData );
                    }
                }
              
                m_firstTimeChanged = true;
                changeDisplayedSignal( );
            }
            
            if ( e.getID( ) == OffsetRemovalParameterControl.ACCEPT_EVENT )
            {
                //////////////////////
                if ( m_firstTimeChanged == true )
                {
                    m_responseListBuffer.clear( );
                    
                    Iterator it = this.m_responseList.iterator( );
                    while ( it.hasNext( ) )
                    {
                        SingleVoxelMeasuredSpectralData specData = (SingleVoxelMeasuredSpectralData)it.next( );                        
                        SingleVoxelMeasuredSpectralData specDataBuffer = new SingleVoxelMeasuredSpectralData( specData );

                        m_responseListBuffer.add( specDataBuffer );
                        
                    }
                    m_firstTimeChanged = false;
                }
                
                m_sdp = new SingleVoxelSpectralDataProcessing( m_responseListBuffer );
                
                m_responseList = m_sdp.dc_correct( (int)m_fdOffsetRemoverControl.getMinimumRangeValue( ), 
                                                   (int)m_fdOffsetRemoverControl.getMaximumRangeValue( ) );
                
                changeDisplayedSignal( );
                
                //////////////////////
                m_responseListBuffer.clear( );
                m_firstTimeChanged = true;
                m_responseList = m_sdp.getResponseList( );
                
                int index = -1;
                int cntr = 0;
                Iterator it = m_responseNameList.iterator( );
                while ( it.hasNext( ) )
                {
                    String str = (String)it.next( );
                    if ( str.compareTo( "measured" ) == 0 )
                    {
                        index = cntr;
                    }
                    cntr++;
                }
                if ( index != -1 )
                {
                    notifyMeasuredSpectrumChanged( m_responseList.get( index ) );
                }
                
                changeDisplayedSignal( );
            }
            
            if ( e.getID( ) == m_fdOffsetRemoverControl.MINIMUM_RANGE_VALUE_CHANGED_EVENT  )
            {
                if ( m_firstTimeChanged == true )
                {
                    m_responseListBuffer.clear( );
                    
                    Iterator it = this.m_responseList.iterator( );
                    while ( it.hasNext( ) )
                    {
                        SingleVoxelMeasuredSpectralData specData = (SingleVoxelMeasuredSpectralData)it.next( );                        
                        SingleVoxelMeasuredSpectralData specDataBuffer = new SingleVoxelMeasuredSpectralData( specData );

                        m_responseListBuffer.add( specDataBuffer );
                        
                    }
                    m_firstTimeChanged = false;
                }
                
                m_sdp = new SingleVoxelSpectralDataProcessing( m_responseListBuffer );
                
                m_responseList = m_sdp.dc_correct( (int)m_fdOffsetRemoverControl.getMinimumRangeValue( ), 
                                                   (int)m_fdOffsetRemoverControl.getMaximumRangeValue( ) );
                
                changeDisplayedSignal( );
                
            }
            
            if ( e.getID( ) == m_fdOffsetRemoverControl.MAXIMUM_RANGE_VALUE_CHANGED_EVENT  )
            {
                if ( m_firstTimeChanged == true )
                {
                    m_responseListBuffer.clear( );
                    
                    Iterator it = this.m_responseList.iterator( );
                    while ( it.hasNext( ) )
                    {
                        SingleVoxelMeasuredSpectralData specData = (SingleVoxelMeasuredSpectralData)it.next( );
                        SingleVoxelMeasuredSpectralData specDataBuffer = new SingleVoxelMeasuredSpectralData( specData );
                        
                        m_responseListBuffer.add( specDataBuffer );
                    }
                    m_firstTimeChanged = false;
                }
                
                m_sdp = new SingleVoxelSpectralDataProcessing( m_responseListBuffer );
                
                m_responseList = m_sdp.dc_correct( (int)m_fdOffsetRemoverControl.getMinimumRangeValue( ), 
                                                   (int)m_fdOffsetRemoverControl.getMaximumRangeValue( ) );
                changeDisplayedSignal( );
            }
        }
        
        if ( sender == m_phasingControl )
        {
            if ( e.getID( ) == m_phasingControl.ZERO_ORDER_PHASING_EVENT )
            {
                if ( m_firstTimeChanged == true )
                {

                    m_responseListBuffer.clear( );
                    
                    Iterator it = this.m_responseList.iterator( );
                    while ( it.hasNext( ) )
                    {
                        SingleVoxelMeasuredSpectralData specData = (SingleVoxelMeasuredSpectralData)it.next( );                        
                        SingleVoxelMeasuredSpectralData specDataBuffer = new SingleVoxelMeasuredSpectralData( specData );
                        
                        m_responseListBuffer.add( specDataBuffer );
                    }
                    m_firstTimeChanged = false;
                }
                
                m_sdp = new SingleVoxelSpectralDataProcessing( m_responseListBuffer );
                m_responseList = m_sdp.adjustZeroOrderPhase( m_phasingControl.getZeroOrderPhase( ) );
                changeDisplayedSignal( );
                
            }
            
            if ( e.getID( ) == m_phasingControl.FIRST_ORDER_PHASING_EVENT )
            {
                if ( m_firstTimeChanged == true )
                {
                    m_responseListBuffer.clear( );
                    
                    Iterator it = this.m_responseList.iterator( );
                    while ( it.hasNext( ) )
                    {
                        SingleVoxelMeasuredSpectralData specData = (SingleVoxelMeasuredSpectralData)it.next( );
                        SingleVoxelMeasuredSpectralData specDataBuffer = new SingleVoxelMeasuredSpectralData( specData );
                        
                        m_responseListBuffer.add( specDataBuffer );
                    }
                    m_firstTimeChanged = false;
                }
                
                m_sdp = new SingleVoxelSpectralDataProcessing( m_responseListBuffer );
                m_responseList = m_sdp.adjustFirstOrderPhase( m_phasingControl.getFirstOrderPhase( ) );
                changeDisplayedSignal( );
            }
            
            if ( e.getID( ) == m_phasingControl.CANCEL_EVENT )
            {   
                
               if ( !m_responseListBuffer.isEmpty( ) )
                {
                    // Restore old values
                    m_responseList.clear( );
                    // m_responseNameList.clear( );
                
                    Iterator it = this.m_responseListBuffer.iterator( );
                    while ( it.hasNext( ) )
                    {    
                        SingleVoxelMeasuredSpectralData specDataBuffer = (SingleVoxelMeasuredSpectralData)it.next( );
                        SingleVoxelMeasuredSpectralData specData = new SingleVoxelMeasuredSpectralData( specDataBuffer );
                        
                        m_responseList.add( specData );
                    }
                }
              
                m_firstTimeChanged = true;
                changeDisplayedSignal( );
            }
            
            if ( e.getID( ) == m_phasingControl.ACCEPT_EVENT )
            {
                m_responseListBuffer.clear( );
                m_firstTimeChanged = true;
                m_responseList = m_sdp.getResponseList( );
                
                int index = -1;
                int cntr = 0;
                Iterator it = m_responseNameList.iterator( );
                while ( it.hasNext( ) )
                {
                    String str = (String)it.next( );
                    System.out.println( "ResponseNameList String : " + str );
                    if ( str.compareTo( "measured" ) == 0 )
                    {
                        index = cntr;
                    }
                    cntr++;
                }
                
                if ( ( index != -1 ) && ( index < m_responseList.size( ) ) ) 
                {
                    notifyMeasuredSpectrumChanged( m_responseList.get( index ) );
                }
                
                changeDisplayedSignal( );
            }
        }
        
        if ( sender == m_setShiftParameter )
        {
            
            launchManualFrequencyShift( );
            
        }

        // Handle events from the spectral shift gui control
        
        if ( sender == m_shiftControl )
        {
            if ( e.getID( ) == m_shiftControl.CHANGE_EVENT )
            {
                System.out.println( "change event!!" );
                
                if ( m_firstTimeChanged == true )
                {
                    m_responseListBuffer.clear( );
                    
                    Iterator it = this.m_responseList.iterator( );
                    while ( it.hasNext( ) )
                    {
                        SingleVoxelMeasuredSpectralData specData = (SingleVoxelMeasuredSpectralData)it.next( );                   
                        SingleVoxelMeasuredSpectralData specDataBuffer = new SingleVoxelMeasuredSpectralData( specData );
                        
                   
                        m_responseListBuffer.add( specDataBuffer );
                    }
                    m_firstTimeChanged = false;
                }
                
                m_sdp = new SingleVoxelSpectralDataProcessing( m_responseListBuffer );
                
                m_responseList = m_sdp.frequencyShift( m_shiftControl.getShiftValue( ) );
                
                changeDisplayedSignal( );
            }
            
            if ( e.getID( ) == m_shiftControl.ACCEPT_EVENT )
            {
                System.out.println( "accept event!!" );
                               m_responseListBuffer.clear( );
                m_firstTimeChanged = true;
                m_responseList = m_sdp.getResponseList( );
                
                int index = -1;
                int cntr = 0;
                Iterator it = m_responseNameList.iterator( );
                while ( it.hasNext( ) )
                {
                    String str = (String)it.next( );
                    if ( str.compareTo( "measured" ) == 0 )
                    {
                        index = cntr;
                    }
                    cntr++;
                }
                
                if ( index != -1 )
                {
                    notifyMeasuredSpectrumChanged( m_responseList.get( index ) );
                }
                
                changeDisplayedSignal( );
            }
            
            if ( e.getID( ) == m_shiftControl.CANCEL_EVENT )
            {
                 if ( !m_responseListBuffer.isEmpty( ) )
                {
                    // Restore old values
                    m_responseList.clear( );
                    // m_responseNameList.clear( );
                
                    Iterator it = this.m_responseListBuffer.iterator( );
                    while ( it.hasNext( ) )
                    {    
                        SingleVoxelMeasuredSpectralData specDataBuffer = (SingleVoxelMeasuredSpectralData)it.next( );
                        SingleVoxelMeasuredSpectralData specData = new SingleVoxelMeasuredSpectralData( specDataBuffer );
                        
                        m_responseList.add( specData );
                    }
                }
              
                m_firstTimeChanged = true;
                changeDisplayedSignal( );
                
            }
        }
        
        // Handle events from the spectral multiply first sample point gui control
        
        if ( sender == m_firstPointMultiplicationControl )
        {
            if ( e.getID( ) == m_firstPointMultiplicationControl.CHANGE_EVENT )
            {
                
                if ( m_firstTimeChanged == true )
                {
                    m_responseListBuffer.clear( );
                    
                    Iterator it = this.m_responseList.iterator( );
                    while ( it.hasNext( ) )
                    {
                        SingleVoxelMeasuredSpectralData specData = (SingleVoxelMeasuredSpectralData)it.next( );
                        SingleVoxelMeasuredSpectralData specDataBuffer = new SingleVoxelMeasuredSpectralData( specData );
                                                
                        m_responseListBuffer.add( specDataBuffer );
                    }
                    m_firstTimeChanged = false;
                }
                
                m_sdp = new SingleVoxelSpectralDataProcessing( m_responseListBuffer );
                
                m_responseList = m_sdp.multiplyFirstTDPoint( m_firstPointMultiplicationControl.getMultiplicationFactor( ) );
                
                changeDisplayedSignal( );
            }
            
            if ( e.getID( ) == SingleVoxelFirstSamplePointMultiplicatorControl.ACCEPT_EVENT )
            {
                
                m_responseListBuffer.clear( );
                m_firstTimeChanged = true;
                m_responseList = m_sdp.getResponseList( );
                
                int index = -1;
                int cntr = 0;
                Iterator it = m_responseNameList.iterator( );
                while ( it.hasNext( ) )
                {
                    String str = (String)it.next( );
                    if ( str.compareTo( "measured" ) == 0 )
                    {
                        index = cntr;
                    }
                    cntr++;
                }
                
                if ( index != -1 )
                {
                    notifyMeasuredSpectrumChanged( m_responseList.get( index ) );
                }
                
                changeDisplayedSignal( );
            }
            
            if ( e.getID( ) == SingleVoxelFirstSamplePointMultiplicatorControl.CANCEL_EVENT )
            {
                if ( !m_responseListBuffer.isEmpty( ) )
                {
                    // Restore old values
                    m_responseList.clear( );
                    // m_responseNameList.clear( );
                
                    Iterator it = this.m_responseListBuffer.iterator( );
                    while ( it.hasNext( ) )
                    {    
                        SingleVoxelMeasuredSpectralData specDataBuffer = (SingleVoxelMeasuredSpectralData)it.next( );
                        SingleVoxelMeasuredSpectralData specData = new SingleVoxelMeasuredSpectralData( specDataBuffer );
                        
                        m_responseList.add( specData );
                    }
                }
              
                m_firstTimeChanged = true;
                changeDisplayedSignal( );
            }
        }
        
        if ( e.getSource( ) == m_plotRangeSelector )
        {
            m_responseListBuffer.clear( );
            m_firstTimeChanged = true;
            changeDisplayedSignal( );   
        }

        // handle the event that the user wants to display point units only
        if ( sender == m_displayPointsOnlyUnit )
        {
            
            // in this mode the scales are draw normally
            setDisplayXReversed( false );
            // save the new display settings
            TDFDFittingDefaults fitdef = TDFDFittingDefaults.getInstance( );
            fitdef.load( );
            String displayDomain = fitdef.getProperty( TDFDFittingDefaults.DISPLAY_DOMAIN );
            
            if ( displayDomain.compareTo( TDFDFittingDefaults.FREQ_DOMAIN ) == 0 )
            {
                fitdef.setProperty( TDFDFittingDefaults.DISPLAY_UNIT, 
                                    TDFDFittingDefaults.FREQUENCY_DOMAIN_POINTS );
            }
            else
            {
                fitdef.setProperty( TDFDFittingDefaults.DISPLAY_UNIT, 
                                    TDFDFittingDefaults.TIME_DOMAIN_POINTS );
            }
            
            fitdef.save( );
            
            // redraw the display with the new x-axis units
            m_responseListBuffer.clear( );
            m_firstTimeChanged = true;
            changeDisplayedSignal( );
        }

        // handle the event that the user wants to display seconds - Hz units
        if ( sender == m_displaySecondsOrHertzUnit )
        {
            // in this mode the scales are draw normally
            setDisplayXReversed( false );
            
            // save the new display settings
            TDFDFittingDefaults fitdef = TDFDFittingDefaults.getInstance( );
            fitdef.load( );
            
            String displayDomain = fitdef.getProperty( TDFDFittingDefaults.DISPLAY_DOMAIN );
            
            if ( displayDomain.compareTo( TDFDFittingDefaults.FREQ_DOMAIN ) == 0 )
            {
                fitdef.setProperty( TDFDFittingDefaults.DISPLAY_UNIT, 
                                    TDFDFittingDefaults.FREQUENCY_DOMAIN_HZ );
            }
            else
            {
                fitdef.setProperty( TDFDFittingDefaults.DISPLAY_UNIT, 
                                    TDFDFittingDefaults.TIME_DOMAIN_SECONDS );
            }            
            fitdef.save( );
            // redraw the display with the new x-axis units
            m_responseListBuffer.clear( );
            m_firstTimeChanged = true;
            changeDisplayedSignal( );

        }

        // handle the event that throws e user wants to display seconds ppm units
        if ( sender == m_displaySecondsOrPPMUnit )
        {
            // save the new display settings
            TDFDFittingDefaults fitdef = TDFDFittingDefaults.getInstance( );
            fitdef.load( );
            String displayDomain = fitdef.getProperty( TDFDFittingDefaults.DISPLAY_DOMAIN );
           
            if ( displayDomain.compareTo( TDFDFittingDefaults.FREQ_DOMAIN ) == 0 )
            {
                setDisplayXReversed( true );
                fitdef.setProperty( TDFDFittingDefaults.DISPLAY_UNIT, TDFDFittingDefaults.FREQUENCY_DOMAIN_PPM );
            }
            else
            {
                this.setDisplayXReversed( false );
                fitdef.setProperty( TDFDFittingDefaults.DISPLAY_UNIT, TDFDFittingDefaults.TIME_DOMAIN_SECONDS );
            }
            fitdef.save( );
            // redraw the display with the new x-axis units
            m_responseListBuffer.clear( );
            m_firstTimeChanged = true;
            changeDisplayedSignal( );
        }
    }

    /**
     * Method calling SVD typed processing.
     */
    public void launchSVDInteractiveWaterProcessing( )
    {
           m_sdp = new SingleVoxelSpectralDataProcessing( m_responseList );
           copyResponseList( );
           
           m_hlsvdWaterRemovalGUI = new HLSVDControlFrame( m_responseList.getFirst( ), true );
           m_hlsvdWaterRemovalGUI.addActionListener( this );
    }
    
    public void launchApodization( ) 
    {
        m_sdp = new SingleVoxelSpectralDataProcessing( m_responseList );
        copyResponseList( );

        m_apodizationDefiner = new ApodizationParameterDefinitionControl( "Define Apodization Parameters", 
                                                                          m_parent, 
                                                                          ApodizationParameterDefinitionControl.SINGLE_RESPONSE_DISPLAY );
        m_apodizationDefiner.addActionListener( this );
    }
    
    void launchFirstTDPointMultiplication( ) 
    {
        m_sdp = new SingleVoxelSpectralDataProcessing( m_responseList );
        copyResponseList( );
            
        m_firstPointMultiplicationControl = new SingleVoxelFirstSamplePointMultiplicatorControl( "Set TD first time sample point multiplication factor", 
                                                                                                 m_parent, SpectrumPhasingDefinitionControl.SINGLE_RESPONSE_DISPLAY );
        m_firstPointMultiplicationControl.addActionListener( this );
    }
    
    
    void launchFDOffsetCorrection( ) 
    {
        m_sdp = new SingleVoxelSpectralDataProcessing( m_responseList );
        copyResponseList( );
            
        m_fdOffsetRemoverControl = new OffsetRemovalParameterControl( "Frequency Domain Offset Removal", 
                                                                      m_parent, 
                                                                      SpectrumPhasingDefinitionControl.SINGLE_RESPONSE_DISPLAY,
                                                                      0.0f, 0.2f * m_sdp.getResponseList( ).getFirst( ).m_absorptionChannelFD.length );
        m_fdOffsetRemoverControl.addActionListener( this );
    }
    
    void launchZeroFilling( ) 
    {
        m_sdp = new SingleVoxelSpectralDataProcessing( m_responseList );
        copyResponseList( );
            
        m_zerofillDefiner = new ZeroFillParameterDefinitionControl( "Define Zerofilling factor", m_parent );
        m_zerofillDefiner.addActionListener( this );
    }
    
    void launchManualPhasing( ) 
    {
        m_sdp = new SingleVoxelSpectralDataProcessing( m_responseList );
        copyResponseList( );
            
        m_phasingControl = new SpectrumPhasingDefinitionControl( "Zero and First Order Phasing", m_parent, SpectrumPhasingDefinitionControl.SINGLE_RESPONSE_DISPLAY );
        m_phasingControl.addActionListener( this );
    }
    
    void launchManualFrequencyShift( )  
    {
        m_shiftControl = new SingleVoxelSpectrumShiftControl( "Define frequency shift [Hz]", m_parent, SingleVoxelSpectrumShiftControl.SINGLE_RESPONSE_DISPLAY );
        m_shiftControl.addActionListener( this );
    }
        
    void defineBaseLineNodes( )
    {
        BaseLineSplineSupportPointsDefinitionFrame frm = new BaseLineSplineSupportPointsDefinitionFrame( m_responseList.getFirst( ) );   
    }
            
    /**
     * Save a screen shot in a file; Action is performed in a separate 
     * thread on order to get the correct screen shots.
     */
    public void storeScreenShotInFile( )
    {
        final SwingWorker worker = new SwingWorker( )
        {
            @Override
            public Object construct( )
            {
                performScreenShotCreation( );          
                return null;
            }
                
            // runs in the event dispatching thread
            @Override
            public void finished( )
            {
                Dialogs d = new Dialogs( );
                d.messageBox( m_parent, "<HTML> Screenshot stored in image files </HTML>" );
            }

        };
        
        worker.start( );
    }
    
    
    
    /**
     * Removes the baseline by subtraction of a spline 
     * Spline is defined by the model or user defined points where the absorption
     * spectrum should dissappear.
     */
    private void removeBaseline( )
    {
        
        if ( m_sdp == null )
        {
            m_sdp = new SingleVoxelSpectralDataProcessing( m_responseList );
        }
        
        if ( this.m_baselineSpline != null )
        {
            m_responseListBuffer.clear( );
            m_firstTimeChanged = true;
            m_responseList = m_sdp.getResponseList( );
                
            int index = -1;
            int cntr = 0;
            Iterator it = m_responseNameList.iterator( );
            while ( it.hasNext( ) )
            {
                String str = (String)it.next( );
                if ( str.compareTo( "measured" ) == 0 )
                {
                    index = cntr;
                }
                cntr++;
            }
                
            if ( index != -1 )
            {
                notifyMeasuredSpectrumChanged( m_responseList.get( index ) );
            }
                
//            // All data of the series must be changed accordingly
//            // Iterate over all responses of the SingleVoxelSpectralDataSeries
//                
             LinkedList<SingleVoxelSpectralData> linkedList = new LinkedList<SingleVoxelSpectralData>( );
//               
//            for ( int i = 0; i < m_measuredPatientSpectrum.getNumberOfInstances( ); i++ )
//            {
//                SingleVoxelSpectralData sVoxData = new SingleVoxelSpectralData( m_measuredPatientSpectrumSeries.getAbsorptionChannelTD( i ),
//                                                                                m_measuredPatientSpectrumSeries.getDispersionChannelTD( i ),
//                                                                                SingleVoxelSpectralData.TIME_DOMAIN );
//                     
//                linkedList.add( sVoxData );
//            }
                
            m_sdp.setResponseList( m_responseList );
            
            // Subtract the baseline from all responses of this series
            m_responseList = m_sdp.removeBaseLineSpline( m_baseLineRangesDataModel );
                    
            changeDisplayedSignal( );
        }
    }
    
    
    /**
     * Method that allows user to save screen shots of "this" to be stored
     * in png-formatted file.
     */
    private void performScreenShotCreation( )
    {
        try
        {
            m_screenshotFileReference = getScreenShotFileReference( );
                    
            String fileNameBase = m_screenshotFileReference.getPath( );
  
            this.paintComponent(this.getGraphics( ) );
                
            try
            {
                Dimension dim = this.getSize( );
                Point pos = this.getLocationOnScreen( );
                
                Rectangle screenRect = new Rectangle( pos.x, pos.y, dim.width, dim.height );
                
                Robot robot = new Robot( );
            
                try
                {
                    BufferedImage image = robot.createScreenCapture(screenRect); 
                    // save an image to a PNG file
                   
                    ImageIO.write( image, "png", new File( fileNameBase + ".png" ) );
                   
                }
                catch( IOException ioe )
                {
                    System.err.println( "write: " + ioe.getMessage( ) );
                }
                       
            }
            catch ( Exception exception )
            { 
                System.out.println( exception );
            }
        }
        catch ( OutOfMemoryError e )
        {
             Dialogs d = new Dialogs( );
             d.messageBox( m_parent, "<HTML> An out of memory error occurred  <BR> Close unused windows to free memory </HTML>" );
        }
    }
    
    /**
     * Returns the file object of the screen shot selected by the user.
     */
    private File getScreenShotFileReference( )
    {   
        TDFDFittingDefaults fdef = TDFDFittingDefaults.getInstance( );
        fdef.load( );
        
        JFileChooser fileChooser = new JFileChooser( fdef.getProperty( "File name and path to store screenshot into" ) );
        
        int result = fileChooser.showOpenDialog( this );
        
        m_screenshotFileReference = new File( "" );
        
        switch ( result )
        {
            case JFileChooser.APPROVE_OPTION:   // Approve Open button was pressed
                                                m_screenshotFileReference = fileChooser.getSelectedFile( );
                                                m_screenshotFileReference = new File( m_screenshotFileReference.getPath( ) );
                                                break;
           case JFileChooser.CANCEL_OPTION:     // Cancel or close dialog was selected
                                                break;
           case JFileChooser.ERROR_OPTION:      // the selected process did not terminate properly
                                                break;
        }
        
        return m_screenshotFileReference;
    }
    
    
    
    /**
     * Adds a listener to the list of listeners to action events.
     * @param listener
     */
    public void addActionListener( ActionListener listener )
    {
        m_listeners.add( listener );
    }
    
    /**
     * Notifies all registered objects that a change has taken place.
     */
    private void notifyMeasuredSpectrumChanged( Object obj )
    {
        Iterator it = m_listeners.iterator( );
        while( it.hasNext( ) )
	{
            ( ( ActionListener )it.next( ) ).actionPerformed( new ActionEvent( obj, 0, "measured spectrum changed" ) );
        }
    }
    
    /**
     * Returns the response having name respName.
     * @param respName
     * @return 
     */
    public SingleVoxelMeasuredSpectralData getNamedResponse( String respName )
    {
        int index = 0;
        Iterator it = m_responseNameList.iterator( );
        SingleVoxelMeasuredSpectralData specData = new SingleVoxelMeasuredSpectralData( );
        
        while ( it.hasNext( ) )
        {
            if ( ( (String)it.next( ) ).compareTo( respName ) == 0 )
            {
                specData = (SingleVoxelMeasuredSpectralData)m_responseList.get( index );
            }
            index++;
        }
        
        return specData;
    }
    
   /**
    * Based on the private attribute:  
    *
    *      m_baseLineRangesDataModel
    *
    * this routine computes the baseline spline.
    */
    private void determineBaseLineSpline( )
    {
        
        // SingleVoxelSpectralData currentInstance = m_measuredPatientSpectrumSeries.getCurrentInstance( m_currentlyDisplayedInstance );
        
        SingleVoxelSpectralData currentInstance = m_responseList.getFirst( );
        
        float[] absCurve = currentInstance.getAbsorptionChannelFD( );
        float[] dispCurve = currentInstance.getDispersionChannelFD( );
        float[] magnitudeCurve = currentInstance.getMagnitudeCurveFDPoints( ).getYvalues( );
        float[] phase = currentInstance.getPhaseCurveFDPoints( ).getYvalues( );
        
        ///////////////////////////////////////////////////////////////////////
        // ABSORPTION PART
        ///////////////////////////////////////////////////////////////////////
        
        CatmullRomSpline b = new CatmullRomSpline( );
        
        // In order to guarantee continuity we have to preceed the interval 
        // model by two pre-points and two post points.
//        Point2D pnt = m_baseLineRangesDataModel.getPoint( 0 );
//        
        Point2D p_before = m_baseLineRangesDataModel.getPoint( 0 );
        
        // b.addPoint( (float)p_before.getX() - 200f, (float)p_before.getY( ) );
        b.addPoint( (float)p_before.getX() - 100f, (float)p_before.getY( ) );
        
        // Determine the points of the spline.
        for ( int i = 0 ; i < m_baseLineRangesDataModel.getSize( ); i++ )
        {
            Point2D pnt2 = m_baseLineRangesDataModel.getPoint( i );
            Point2D refineYValue = new Point2D.Double( (int)pnt2.getX( ), absCurve[(int)pnt2.getX( ) ] );
            b.addPoint( (float)refineYValue.getX( ), (float)refineYValue.getY( ) );
        }
        
        Point2D p_after = m_baseLineRangesDataModel.getPoint( m_baseLineRangesDataModel.getSize( ) - 1 );
        b.addPoint( (float)p_after.getX( ) + 100f, (float)p_after.getY( )  );
        b.addPoint( (float)p_after.getX( ) + 200f, (float)p_after.getY( )  );
        
        
        float[] baselineSplineAbs = b.computeSplineCurve( absCurve.length );
        // float[] baselineSplineDisp = new float[absCurve.length];
        
        
        float[] xaxis = new float[absCurve.length];
        
        for ( int kk = 0; kk < absCurve.length; kk++ )
        {
            xaxis[kk] = (float)kk;
        }
        XYPlotFrame xypf = new XYPlotFrame( "Plot spline (absorption)", null );
        xypf.addCurve( new Curve( xaxis, absCurve ), true );
        xypf.addCurve( new Curve( xaxis, baselineSplineAbs ), true );
        
        float[] difference = new float[xaxis.length];
        
        for ( int i = 0; i < xaxis.length; i++ )
        {
            difference[i] = absCurve[i] - baselineSplineAbs[i];
        }
        
        xypf.addCurve( new Curve( xaxis, difference, Color.red  ), true );
        
        
        ///////////////////////////////////////////////////////////////////////
        // Dispersion
        //////////////////////////////////////////////////////////////////////
        
        CatmullRomSpline b_disp = new CatmullRomSpline( );
        
        b_disp.addPoint( (int)p_before.getX(), dispCurve[ 0 ] );
        
        for ( int kk = 0; kk < m_baseLineRangesDataModel.getSize( ); kk++ )
        {
            Point2D p = m_baseLineRangesDataModel.getPoint( kk );
            Point2D refineYValue = new Point2D.Double( (int)p.getX( ), dispCurve[(int)p.getX( ) ] );
            b_disp.addPoint( (float)refineYValue.getX(), dispCurve[ (int)refineYValue.getX( ) ] );
        }
        
        b_disp.addPoint( (int)p_after.getX( )+100, dispCurve[ m_baseLineRangesDataModel.getSize( ) - 1 ] );
        b_disp.addPoint( (int)p_after.getX( )+200, dispCurve[ m_baseLineRangesDataModel.getSize( ) - 1 ] );
        
        float[] baselineSplineDisp = b_disp.computeSplineCurve( dispCurve.length );
        
        
        XYPlotFrame xypf_disp = new XYPlotFrame( "Plot spline (dispersion)", null );
        xypf_disp.addCurve( new Curve( xaxis, dispCurve ), true );
        xypf_disp.addCurve( new Curve( xaxis, baselineSplineDisp ), true );
        
        float[] difference_disp = new float[xaxis.length];
        
        for ( int i = 0; i < xaxis.length; i++ )
        {
            difference_disp[i] = dispCurve[i] - baselineSplineDisp[i];
        }
        
        xypf_disp.addCurve( new Curve( xaxis, difference_disp, Color.red  ), true );
        
        
        
        ///////////////////////////////////////////////////////////////////////
        // Magnitude
        //////////////////////////////////////////////////////////////////////
        
        CatmullRomSpline b_magnitude = new CatmullRomSpline( );
        
        b_magnitude.addPoint( (int)p_before.getX(), magnitudeCurve[ 0 ] );
        
        for ( int kk = 0; kk < m_baseLineRangesDataModel.getSize( ); kk++ )
        {
            Point2D p = m_baseLineRangesDataModel.getPoint( kk );
            Point2D refineYValue = new Point2D.Double( (int)p.getX( ), magnitudeCurve[(int)p.getX( ) ] );
            b_magnitude.addPoint( (float)refineYValue.getX(), magnitudeCurve[ (int)refineYValue.getX( ) ] );
        }
        
        b_magnitude.addPoint( (int)p_after.getX( )+100, magnitudeCurve[ m_baseLineRangesDataModel.getSize( ) - 1 ] );
        b_magnitude.addPoint( (int)p_after.getX( )+200, magnitudeCurve[ m_baseLineRangesDataModel.getSize( ) - 1 ] );
        
        float[] baselineSplineMagnitude = b_magnitude.computeSplineCurve( absCurve.length );
        
        XYPlotFrame xypf_magn = new XYPlotFrame( "Plot spline (magnitude)", null );
        xypf_magn.addCurve( new Curve( xaxis, magnitudeCurve ), true );
        xypf_magn.addCurve( new Curve( xaxis, baselineSplineMagnitude ), true );
        
        float[] difference_magn = new float[xaxis.length];
        
        for ( int i = 0; i < xaxis.length; i++ )
        {
            difference_magn[i] = magnitudeCurve[i] - baselineSplineMagnitude[i];
        }
        
        xypf_magn.addCurve( new Curve( xaxis, difference_magn, Color.red  ), true );
            
        
//        SingleVoxelSpectralData specData = new SingleVoxelSpectralData( );
//
//        Iterator it = this.m_responseList.iterator( );
//        
//        if ( it.hasNext( ) )
//        {
//            SingleVoxelSpectralData dummy =  (SingleVoxelSpectralData)it.next( );  
//        
//            specData.setSpectralDataPointsFD( baselineSplineAbs, baselineSplineDisp );
//            specData.setDicomHeaderInfo( dummy.getDicomHeaderInfo( ) );
//            specData.setSiemensSpecificDicomHeaderInfo( dummy.getSiemensSpecificDicomHeaderInfo( ) );
//        }
        
//        // define a member variable to model the baseline spline
//        m_baselineSpline = new SingleVoxelMeasuredSpectralData( );
//        
//        SingleVoxelMeasuredSpectralData dummy =  m_responseList.getFirst( );
//        
//        m_baselineSpline.setSpectralDataPointsFD( baselineSplineAbs, baselineSplineDisp );
//        m_baselineSpline.setDicomHeaderInfo( dummy.getDicomHeaderInfo( ) );
//        m_baselineSpline.setSiemensSpecificDicomHeaderInfo( dummy.getSiemensSpecificDicomHeaderInfo( ) );
//        
//        this.addSpectralDataSet( "baseline_spline", m_baselineSpline , true );
        
        ///////////////////////////////////////////////////////////////////////
        // Magnitude & Phase restored
        //////////////////////////////////////////////////////////////////////
        
        float[] difference_magn_phase_res_real = new float[ absCurve.length ];
        float[] difference_magn_phase_res_imag = new float[ absCurve.length ];
        
        CatmullRomSpline b_magnitude_phase_restored = new CatmullRomSpline( );
        
        b_magnitude_phase_restored.addPoint( (int)p_before.getX(), magnitudeCurve[ 0 ] );
        
        for ( int kk = 0; kk < m_baseLineRangesDataModel.getSize( ); kk++ )
        {
            Point2D p = m_baseLineRangesDataModel.getPoint( kk );
            Point2D refineYValue = new Point2D.Double( (int)p.getX( ), magnitudeCurve[(int)p.getX( ) ] );
            b_magnitude_phase_restored.addPoint( (float)refineYValue.getX(), magnitudeCurve[ (int)refineYValue.getX( ) ] );
        }
        
        b_magnitude.addPoint( (int)p_after.getX( ) + 100, magnitudeCurve[ m_baseLineRangesDataModel.getSize( ) - 1 ] );
        b_magnitude.addPoint( (int)p_after.getX( ) + 200, magnitudeCurve[ m_baseLineRangesDataModel.getSize( ) - 1 ] );
        
        float[] baselineSplineMagnitudePhaseRes = b_magnitude_phase_restored.computeSplineCurve( absCurve.length );
        
        XYPlotFrame xypf_magn_phase_res = new XYPlotFrame( "Plot spline (magnitude and phase restored)", null );
        xypf_magn_phase_res.addCurve( new Curve( xaxis, magnitudeCurve ), true );
        xypf_magn_phase_res.addCurve( new Curve( xaxis, baselineSplineMagnitude ), true );
        
        float[] difference_magn_2 = new float[xaxis.length];
        
        for ( int i = 0; i < xaxis.length; i++ )
        {
            difference_magn_2[i] = magnitudeCurve[i] - baselineSplineMagnitude[i];
            
            difference_magn_phase_res_real[i] = (float)difference_magn_2[i] * (float)Math.sin( phase[i] );
            difference_magn_phase_res_imag[i] = (float)difference_magn_2[i] * (float)Math.cos( phase[i] );
        }
        
        xypf_magn_phase_res.addCurve( new Curve( xaxis, difference_magn_phase_res_real, Color.red  ), true );
        xypf_magn_phase_res.addCurve( new Curve( xaxis, difference_magn_phase_res_imag, Color.green  ), true );    
        
//        SingleVoxelSpectralData specData = new SingleVoxelSpectralData( );
//
//        Iterator it = this.m_responseList.iterator( );
//        
//        if ( it.hasNext( ) )
//        {
//            SingleVoxelSpectralData dummy =  (SingleVoxelSpectralData)it.next( );  
//        
//            specData.setSpectralDataPointsFD( baselineSplineAbs, baselineSplineDisp );
//            specData.setDicomHeaderInfo( dummy.getDicomHeaderInfo( ) );
//            specData.setSiemensSpecificDicomHeaderInfo( dummy.getSiemensSpecificDicomHeaderInfo( ) );
//        }
        

        currentInstance.setSpectralDataPointsFD( difference_magn_phase_res_real, 
                                                 difference_magn_phase_res_imag );
        
        changeDisplayedSignal( );
        
        // Define a member variable to model the baseline spline
//        m_baselineSpline = new SingleVoxelMeasuredSpectralData( );
//        
//        SingleVoxelMeasuredSpectralData dummy =  m_responseList.getFirst( );
//        
//        m_baselineSpline.setSpectralDataPointsFD( baselineSplineAbs, baselineSplineDisp );
//        m_baselineSpline.setDicomHeaderInfo( dummy.getDicomHeaderInfo( ) );
//        m_baselineSpline.setSiemensSpecificDicomHeaderInfo( dummy.getSiemensSpecificDicomHeaderInfo( ) );
//        
//        this.addSpectralDataSet( "baseline_spline", m_baselineSpline , true );
        
        System.out.println( "spline computed" );

    }

    /**
     * All registered objects are notified that an event of type
     * actionEventType happened.
     */
    private void notifyActionListeners( int actionEventType, String actionCommand )
    {
        Iterator it = m_listeners.iterator( );
        while( it.hasNext( ) )
        {
            ( ( ActionListener )it.next( ) ).actionPerformed( new ActionEvent( this, actionEventType, actionCommand ) );
        }
    }
    
    /**
     * Determines the selected values at the position indicated by the
     * mouseEvent.
     */
    private Point2D.Float determineValuesAtCursorPosition( Point p )
    {
        
        if ( ( p.x > this.getSize( ).getWidth( ) / 10 ) && ( p.x < 9 * this.getSize( ).getWidth( ) / 10 ) )
        {   
            if ( ( p.y > this.getSize( ).getHeight( ) / 10 ) && ( p.y < 9 * this.getSize( ).getHeight( ) / 10 ) )
            {
                float xrange = 8.0f * (float)getSize( ).getWidth( ) / 10.0f;
                float yrange = 8.0f * (float)getSize( ).getHeight( ) / 10.0f;
                
                MRSignalProperties mrcond = MRSignalProperties.getInstance( );
                
                if ( m_curves != null )
                {
                    if ( m_curves.size( ) > 0 )
                    {
                        float xPosCorrected = ( ( (float)p.x - (float)this.getSize( ).getWidth( ) / 10.0f )  )  / xrange;
                        float yPosCorrected = ( ( (float)p.y - (float)this.getSize( ).getHeight( ) / 10.0f )  )  / yrange;
                        
                        if ( m_XYPlotRangeSelectorMenuItem.isSelected( ) == true )
                        {
                            if ( this.m_displayXReversed == false )
                            {
                                m_lastSelectedOffsetValue = m_xyForcedPlotRange.getMinimumX( ) + xPosCorrected * m_xyForcedPlotRange.getXRange( );
                            }
                            else
                            {
                                m_lastSelectedOffsetValue = m_xyForcedPlotRange.getMaximumX( ) + xPosCorrected * m_xyForcedPlotRange.getXRange( );
                            }
                            
                            m_lastSelectedAmplitudeValue = m_xyForcedPlotRange.getMaximumY( ) - yPosCorrected * m_xyForcedPlotRange.getYRange( );
                        }
                        else
                        {
                            if ( this.m_displayXReversed == false )
                            {
                                m_lastSelectedOffsetValue = m_xyAutoPlotRange.getMinimumX( ) + xPosCorrected * m_xyAutoPlotRange.getXRange( );                 
                            }
                            else
                            {   
                                m_lastSelectedOffsetValue = m_xyAutoPlotRange.getMinimumX( ) + xPosCorrected * m_xyAutoPlotRange.getXRange( );
                            }
                            m_lastSelectedAmplitudeValue = m_xyAutoPlotRange.getMaximumY( ) - yPosCorrected * m_xyAutoPlotRange.getYRange( );
                        }
                    }
                }
            }
        }
        
        return new Point2D.Float( m_lastSelectedOffsetValue, m_lastSelectedAmplitudeValue );
    }
    
    /**
     * Returns the Point2D.Float object of the last user selected point.
     * @return 
     */
    public Point2D.Double getLastSelectedPoint( )
    {
        return new Point2D.Double( m_lastSelectedOffsetValue, m_lastSelectedAmplitudeValue );
    }
    /**
     * Returns the number of sample points of the currently displayed signal.
     * @return 
     */
    public int getNumberOfSamplePoints( )
    {
        return ( m_responseList.getFirst( ) ).getAbsorptionChannelTD( ).length;
    }
    
    // Attributes
    
    private JFrame m_parent;
    
    private LinkedList<ActionListener> m_listeners = new LinkedList<>( );
        
    protected LinkedList<String> m_responseNameList = new LinkedList<>( );
    protected LinkedList<SingleVoxelMeasuredSpectralData> m_responseList = new LinkedList<>( );
    protected LinkedList<SingleVoxelMeasuredSpectralData> m_responseListBuffer = new LinkedList<>( );
       
    protected boolean m_firstTimeChanged = true;
    
    // Popup menu : derived classes should be able to extend the popup menu
    // protected JPopupMenu m_popupmenu;
    protected JMenu m_domainSelector;
    protected JMenu m_channelSelector;
    
    protected JMenu m_displayUnitSelector;
    
    protected JMenu m_signalProcessing;
    
    protected JMenuItem m_copySingleVoxelSpectrumDataMenuItem;
    protected JMenuItem m_cloneSingleVoxelSpectrumDataMenuItem;
    
    protected JMenu m_pasteMenu;
    
    protected JMenuItem m_pasteSingleVoxelSpectrumDataMenuItem;
    protected JMenuItem m_pasteAndAddSingleVoxelSpectrumDataMenuItem;
    protected JMenuItem m_pasteAndSubtractSingleVoxelSpectrumDataMenuItem;
    
    // The displayer can show time or frequency domain data
    protected JRadioButtonMenuItem m_timeDomain;
    protected JRadioButtonMenuItem m_frequencyDomain;
    
    // The channel selector allows the user to select the following channels
    protected JRadioButtonMenuItem m_absorption;
    protected JRadioButtonMenuItem m_dispersion;
    protected JRadioButtonMenuItem m_magnitude;
    protected JRadioButtonMenuItem m_power;
    protected JRadioButtonMenuItem m_phase;
     
    // Signal prosessing menu items
    protected JRadioButtonMenuItem m_applyProcessing;
    
    protected JMenuItem m_setPerformSVDWaterRemoval;
    protected JMenuItem m_setPerformHLSVDWaterRemoval;
    protected JMenuItem m_setApodizationParameters;
    protected JMenuItem m_multiplyFirstTDPoint;
    protected JMenuItem m_offsetCorrectionFD;
    protected JMenuItem m_setZeroFillingParameters;
    protected JMenuItem m_setPhasingParameters;
    protected JMenuItem m_setShiftParameter;
    protected JMenuItem m_frequencyDependentBaselineElimination;
    protected JMenuItem m_removeMacromolecularBaseline;
    
    protected JMenuItem m_saveScreenShot;
    
    protected JRadioButtonMenuItem m_displayPointsOnlyUnit;
    protected JRadioButtonMenuItem m_displaySecondsOrHertzUnit;
    protected JRadioButtonMenuItem m_displaySecondsOrPPMUnit;
    
    protected ButtonGroup m_unitSelectionButtonGroup;
    
    protected ButtonGroup m_domainButtonGroup;
    protected ButtonGroup m_channelButtonGroup;
    
    private ZeroFillParameterDefinitionControl m_zerofillDefiner;
    private ApodizationParameterDefinitionControl m_apodizationDefiner; 
    private SpectrumPhasingDefinitionControl m_phasingControl;
    private SVDWaterLineRemoverGUI m_svdWaterRemovalGUI;
    private HLSVDControlFrame m_hlsvdWaterRemovalGUI; 
            
    private SingleVoxelSpectrumShiftControl m_shiftControl;
    private SingleVoxelFirstSamplePointMultiplicatorControl m_firstPointMultiplicationControl;
    private OffsetRemovalParameterControl m_fdOffsetRemoverControl;
    
    // There is a separate plot range for time domain as well as for frequency domain
    private XYPlotRange m_xyAutoPlotRangeTD;
    private XYPlotRange m_xyForcedPlotRangeTD;

    private XYPlotRange m_xyAutoPlotRangeFD;
    private XYPlotRange m_xyForcedPlotRangeFD;
    
    private SingleVoxelSpectralDataProcessing m_sdp;
    
    private SingleVoxelSpectralDataClipboard m_svsDataClipboard = SingleVoxelSpectralDataClipboard.getInstance( );
    
    protected boolean m_baseLineDefinitionModeActive = false;
    protected float m_lastSelectedOffsetValue;
    protected float m_lastSelectedAmplitudeValue;
    
    public final static int BASELINE_POINT_SELECTED_EVENT = 500;
    protected SingleVoxelMeasuredSpectralData m_baselineSpline;
    
    protected BaseLineSplineSupportPointsDefinitionFrame m_baselineDefFrame;
    private OrderedBaseLinePointsDataModel m_baseLineRangesDataModel = OrderedBaseLinePointsDataModel.getInstance( );
    
    private File m_screenshotFileReference;
    // For proper serialization
    
    protected String m_measuredPatientSpectrumFileName;
    protected SingleVoxelMeasuredSpectralData m_measuredPatientSpectrum;
    
    protected String m_measuredPatientWaterReferenceSpectrumFileName;
    protected SingleVoxelMeasuredSpectralData m_measuredPatientWaterReferenceSpectrum;
    
    private static final long serialVersionUID = 1487225897889393L;
    
    protected DicomDirectory m_dicomDir;

    public void addUpperInfoDisplayLine( String string ) 
    {
       m_leftUpperInfoDisplayLines.add( string );
    }

    public void clearLeftUpperInfoDisplayLines( ) 
    {
        m_leftUpperInfoDisplayLines.clear( );
    }
}
