/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package org.spectrim.tdfdfit;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.nio.file.Files;
import java.util.Iterator;
import java.util.Vector;
import java.util.concurrent.TimeUnit;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.spectrim.main.ClinicalViewer;
import org.spectrim.tdfdfitNative.tdfdfitNative;

/**
 * This class performs the TDFDFitting of one single 1D spectrum 
 * in a SwingWorker thread.
 * @author Johannes Slotboom <johannes.slotboom at insel.ch>
 */
public class TDFDFitExecutorVariableModelSingleThreaded // extends javax.swing.SwingWorker<Integer, String> 
{
    // Declaration of class member variables.
    public final static int IS_PROCESSING = 0;
    public final static int IS_PERFORMED  = 1;
    public final static String IS_PERFORMED_CMD  = "TDFDFit computation performed.";
    
    private final int m_tdfdfitID;
    private final MRSignalProperties  m_MrSignalProperties = MRSignalProperties.getInstance( );
    private final SingleVoxelMeasuredSpectralData m_dataToFit;
    private String m_parameterFileName;
    private String m_verboseParameterFileName;
    private String m_dataFileToFit;
    private String m_dataFileToFitBin;
    private String m_tdfdfitExecutablePath;
    private String m_fileNameBase;
    
    private boolean m_displayIntermediateResults = false;
    // Listeners to textfield changes
    private final Vector<ActionListener> m_listeners = new Vector<>( );
    
    private Process m_process;
    private NumericTDFDFitResults m_tdfdfitResult;
    /**
     * 
     * Perform a fit of a 1D spectrum.
     * 
     * @param spectrumID
     * @param dataToFit
     * @param displayIntermediateResults
     * @param fileNameBase
     */
    public TDFDFitExecutorVariableModelSingleThreaded( int spectrumID, 
                                                       SingleVoxelMeasuredSpectralData dataToFit, 
                                                       boolean displayIntermediateResults,
                                                       String fileNameBase ) 
    {
        m_tdfdfitID = spectrumID;
        m_dataToFit = dataToFit;
        m_displayIntermediateResults = displayIntermediateResults;
        m_fileNameBase = fileNameBase;
    }
    
    /**
     * Returns the thread ID of this basic fit.
     * @return 
     */
    public int getTDFDFitSpectrumdID( )
    {
        return m_tdfdfitID;
    }
    
    public NumericTDFDFitResults getNumericTDFDFitResults( ) 
    {        
        return m_tdfdfitResult;
    }
    
    /**spectrum
     * 
     * Starts the fit of the 1D  stored in dataFile using 
     * model parameterFile.
     * 
     * @return 
     */
    public int startTDFDFitSingleThreaded( )
    {
        
        TDFDFitPrototypeModelSpectrumParameters protoTypeModel = TDFDFitPrototypeModelSpectrumParameters.getInstance( );
        
        m_parameterFileName = protoTypeModel.createTDFDFitStartingValueFiles( m_tdfdfitID );
        CharSequence target = ".par";
        CharSequence replacement = "_verbose.par";
        m_verboseParameterFileName = m_parameterFileName.replace( target, replacement );
        
        m_fileNameBase = m_fileNameBase + "_" + m_tdfdfitID;
        
        m_dataFileToFitBin = m_dataToFit.writeAsTdfdfitFileForFitting( ( m_fileNameBase + ".bin" )  );
        
        // m_dataFileToFit = protoTypeModel.getModelSpectrumName( ) + "_" + m_tdfdfitID;
        
        int     returnValue;
        
        String  systemCmdToStartFit;
        
        TDFDFittingDefaults fittingDefaults = TDFDFittingDefaults.getInstance( );
        fittingDefaults.load( );
        
        m_tdfdfitExecutablePath = fittingDefaults.getProperty( TDFDFittingDefaults.TDFDFIT_INSTALLATION_PATH );
        String tdfdfitExecutableName = fittingDefaults.getProperty( TDFDFittingDefaults.TDFDFIT_EXECUTABLE_NAME );      
        String TDFDFitCall;
        
        if ( m_tdfdfitExecutablePath.endsWith( System.getProperty( "file.separator" ) ) == false )
        {
            TDFDFitCall = m_tdfdfitExecutablePath + System.getProperty( "file.separator" ) + tdfdfitExecutableName;
        }
        else
        {
            TDFDFitCall = m_tdfdfitExecutablePath + tdfdfitExecutableName;
        }
        
        // TDFDFitCall = m_tdfdfitExecutablePath + System.getProperty( "file.separator" ) + tdfdfitExecutableName;
        systemCmdToStartFit = TDFDFitCall + " " + m_tdfdfitExecutablePath + " " + m_dataFileToFitBin + " " + m_parameterFileName;
        
        systemCmdToStartFit =  generateProperCommand( systemCmdToStartFit );
         
        try 
        {
            performTDFDFitFittingInSingleThreadMode( systemCmdToStartFit );
        } 
        catch ( Exception ex ) 
        {
            Logger.getLogger(TDFDFitExecutorVariableModelSingleThreaded.class.getName( ) ).log( Level.SEVERE, null, ex );
        }
        
        return 0;       
    }
     
    /**
     * Returns the process which is/was running i a separate thread.
     * @return 
     */
    public Process getProcess( )
    {
        return m_process;
    }
    
       
    /**
     * 
     * Replaces all blanks by "\  " in order to be interpreted 
     * correctly by the operating system.
     * 
     * @param cmd 
     * @return 
     */
    private String generateProperCommand( String cmd )
    {
        String out = "";
        for ( int i = 0; i < cmd.length( ); i++ )
        {
            if ( cmd.substring( i, i + 1 ).compareTo( "\\" ) == 0 )
            {
                out += "\\" + "\\";
            }
            else
            {
                out += cmd.substring( i, i + 1 );
            }
        }
        return out;
    }
    
    /**
     * Here the actual system call to TDFDFit is performed.
     * @param systemCmdToStartFit
     * @return
     * @throws Exception 
     */
    public int performTDFDFitFittingInSingleThreadMode( String systemCmdToStartFit ) throws Exception 
    {
        System.out.println( "Execution thread id : " + m_tdfdfitID + " started." );
        System.out.println( "Execution command   : " + systemCmdToStartFit );
        

        // System.out.println( "Trying following system command : " + systemCmdToStartFit );
        System.out.print( "." );
        try
        {   
//                ProcessBuilder builder = new ProcessBuilder( systemCmdToStartFit );
//                builder.redirectOutput( ProcessBuilder.Redirect.INHERIT );
//                Process process = builder.start( );
//                // boolean retBoolean = process.waitFor(3, TimeUnit.SECONDS);
//                int retVal = process.waitFor( );

//                return 0;
            Process p = Runtime.getRuntime( ).exec( systemCmdToStartFit ); 

            InputStream in = p.getInputStream( );
            int c;
            while ( ( c = in.read( ) ) != -1 )
            {
                System.out.print( (char)c ) ;
            }

            p.waitFor( 4, TimeUnit.SECONDS );

            in.close( );
        }
        catch ( IOException ioX )
        {
            // not handled yet
            System.out.println( "IOException occurred" );
            return -1;
        }
        catch ( InterruptedException intX )
        {
            // not handled yet
            System.out.println( "InterruptedException occurred" );
            return -1;
        }
    
        moveDeleteTDFDFitRelatedFiles( );
        // notifyActionListeners( IS_PERFORMED, IS_PERFORMED_CMD );
        return 0;
    }
    
    public synchronized void moveDeleteTDFDFitRelatedFiles( ) 
    {
         // Clean up the bin directory!
        TDFDFittingDefaults fitDef = TDFDFittingDefaults.getInstance( );
        fitDef.load( );
        
        String path = fitDef.getProperty( TDFDFittingDefaults.TDFDFIT_INSTALLATION_PATH );
        
        File fileDotBin = new File( path + System.getProperty( "file.separator" ) + m_fileNameBase  + ".bin" );
        File fileDotFit = new File( path + System.getProperty( "file.separator" ) + m_fileNameBase  + ".fit" );
        File fileDotFit0 = new File( path + System.getProperty( "file.separator" ) + m_fileNameBase  + ".fit0" );
        File fileDotFitRes = new File( path + System.getProperty( "file.separator" ) + m_fileNameBase  + ".fitres" );
        File fileDotOrg = new File( path + System.getProperty( "file.separator" ) + m_fileNameBase  + ".org" );
        File fileDotRes = new File( path + System.getProperty( "file.separator" ) + m_fileNameBase  + ".res" );
        File fileDotCorOrg = new File( path + System.getProperty( "file.separator" ) + m_fileNameBase  + ".cororg" );
        File fileDotExl = new File( path + System.getProperty( "file.separator" ) + m_fileNameBase  + ".exl" );
        File fileDotNln0 = new File( path + System.getProperty( "file.separator" ) + m_fileNameBase  + ".nln0" );
        File fileDotMat = new File( path + System.getProperty( "file.separator" ) + m_fileNameBase  + ".mat" );
        File fileDotErr = new File( path + System.getProperty( "file.separator" ) + m_fileNameBase  + ".err" );
        
        File parameterStartFile = new File( m_parameterFileName );
        File verboseParameterStartFile = new File( m_verboseParameterFileName );
        
        String outputFileNamePar = parameterStartFile.getName( );
        outputFileNamePar = outputFileNamePar.replace( ".par", "_" + m_fileNameBase + ".par" );
        String outputVerboseParameterStartFileName = verboseParameterStartFile.getName( );
        outputVerboseParameterStartFileName = outputVerboseParameterStartFileName.replace( ".par", "_" + m_fileNameBase + ".par" );
        // *********************************************************************    
        if ( path.endsWith( System.getProperty( "file.separator" ) ) == false )
        {
            path = path + System.getProperty( "file.separator" );
        }

//        String fileToOpen = path + m_fileNameBase  + ".bin.exl";
//        
//        m_tdfdfitResult = new NumericTDFDFitResults( fileToOpen );
//        boolean readOK = m_tdfdfitResult.readFile( );
//        if ( readOK == false )
//        {
//            System.out.println( "Error reading exl-file : " + fileToOpen );
//        }
//        TDFDFittingDefaults fittingDefaults = TDFDFittingDefaults.getInstance( );
//        fittingDefaults.load( );
//        
//        boolean readOK2 = m_tdfdfitResult.readFittedSpectrumFile( m_MrSignalProperties.getNumberOfSamplePoints( ) );
//        if ( readOK2 == false )
//        {
//            System.out.println( "Error reading fit-file : " + fileToOpen );
//        }
        
//        if ( readOK && readOK2 )
//        {
//            System.out.println( "Successfully read exl file" + fileToOpen );
//        }
//        else
//        {
//            System.out.println( "Errors in result reading!" + fileToOpen );
//        }
        
        // ********************************************************************* 
        
        
        File patDatabasePath = new File( fitDef.getProperty( TDFDFittingDefaults.PATIENT_DATABASE_ROOT_PATH ) );
        if ( parameterStartFile.exists( ) && patDatabasePath.exists( ) )
        {
            File outputFile = new File( patDatabasePath.getPath( ) + System.getProperty( "file.separator" ) + outputFileNamePar  ); 
            if ( outputFile.exists( ) )
            {
                outputFile.delete( );
            }
            
            if ( new File( outputFileNamePar ).exists( ) )
            {
//                System.out.println( "ParameterStartFile exists." );
                parameterStartFile.delete( );
            }
            
            if ( fitDef.getProperty( TDFDFittingDefaults.DELETE_TDFDFIT_OUTPUTFILE_DOT_PAR ) == null )
            {
                fitDef.setProperty( TDFDFittingDefaults.DELETE_TDFDFIT_OUTPUTFILE_DOT_PAR, "no" );
                fitDef.save( );
            }
            
            
            if ( fitDef.getProperty( TDFDFittingDefaults.DELETE_TDFDFIT_OUTPUTFILE_DOT_PAR ).compareTo( "yes" ) == 0 )
            {
                parameterStartFile.delete( );
            }
            else
            {
                try 
                {
                    Files.move( parameterStartFile.toPath( ), new File( patDatabasePath.getPath( ) + System.getProperty( "file.separator" ) + outputFileNamePar ).toPath( ), java.nio.file.StandardCopyOption.REPLACE_EXISTING);
//                    System.out.println( "Moved file : " + parameterStartFile.toPath( ) );
                } 
                catch ( IOException ex ) 
                {
                   System.out.println( "Error moving file " + parameterStartFile.toPath( ) );
                }
                
                // parameterStartFile.renameTo( new File ( patDatabasePath.getPath( ) + System.getProperty( "file.separator" ) + outputFileNamePar ) );
            }
            
            // ============================================================================================================================================
            
            File outputVerboseFile = new File ( patDatabasePath.getPath( ) + System.getProperty( "file.separator" ) + outputVerboseParameterStartFileName );
            if ( outputVerboseFile.exists( ) )
            {
                outputVerboseFile.delete( );
            }
            
            if ( fitDef.getProperty( TDFDFittingDefaults.DELETE_TDFDFIT_OUTPUTFILE_DOT_VERBOSE_PAR ) == null )
            {
                fitDef.setProperty( TDFDFittingDefaults.DELETE_TDFDFIT_OUTPUTFILE_DOT_VERBOSE_PAR, "no" );
                fitDef.save( );
            }
            
            if ( fitDef.getProperty( TDFDFittingDefaults.DELETE_TDFDFIT_OUTPUTFILE_DOT_VERBOSE_PAR ).compareTo( "yes" ) == 0 )
            {
                verboseParameterStartFile.delete( );
            }
            else
            {
                try 
                {
                    Files.move( verboseParameterStartFile.toPath( ), new File ( patDatabasePath.getPath( ) + System.getProperty( "file.separator" ) + outputVerboseParameterStartFileName ).toPath( ), java.nio.file.StandardCopyOption.REPLACE_EXISTING);
//                    System.out.println( "Moved file : " + verboseParameterStartFile.toPath( ) );
                } 
                catch ( IOException ex ) 
                {
                   System.out.println( "Error moving file : " + verboseParameterStartFile.toPath( ) );
                }
                
                // verboseParameterStartFile.renameTo( new File ( patDatabasePath.getPath( ) + System.getProperty( "file.separator" ) + outputVerboseParameterStartFileName ) );
            }
               
        }
        
        if ( fileDotBin.exists( ) && patDatabasePath.exists( ) )
        {
            
            File binfile = new File ( patDatabasePath.getPath( ) + System.getProperty( "file.separator" ) + m_fileNameBase  + ".bin"  );
            if ( binfile.exists( ) )
            {
                binfile.delete( );
            }
            
            if ( fitDef.getProperty( TDFDFittingDefaults.DELETE_TDFDFIT_OUTPUTFILE_BIN ) == null )
            {
                fitDef.setProperty( TDFDFittingDefaults.DELETE_TDFDFIT_OUTPUTFILE_BIN, "no" );
                fitDef.save( );
            }        
            
            if ( fitDef.getProperty( TDFDFittingDefaults.DELETE_TDFDFIT_OUTPUTFILE_BIN ).compareTo( "yes" ) == 0 )
            {
                fileDotBin.delete( );
            }
            else
            {
                try 
                {
                    Files.move( fileDotBin.toPath( ), new File ( patDatabasePath.getPath( ) + System.getProperty( "file.separator" ) + m_fileNameBase  + ".bin"  ).toPath( ), java.nio.file.StandardCopyOption.REPLACE_EXISTING);
//                    System.out.println( "Moved file : " + fileDotBin.toPath( ) );
                } 
                catch ( IOException ex ) 
                {
                   System.out.println( "Error moving file : " + fileDotBin.toPath( ) );
                }
                
                
//                fileDotBin.renameTo( new File ( patDatabasePath.getPath( ) + System.getProperty( "file.separator" ) + m_fileNameBase  + ".bin"  ) );
            }
            
            //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
            
            File fitfile = new File ( patDatabasePath.getPath( ) + System.getProperty( "file.separator" ) + m_fileNameBase  + ".fit"  );
            if ( fitfile.exists( ) )
            {
                fitfile.delete( );
            }
            
            
            if ( fitDef.getProperty( TDFDFittingDefaults.DELETE_TDFDFIT_OUTPUTFILE_BIN_FIT ) == null )
            {
                fitDef.setProperty( TDFDFittingDefaults.DELETE_TDFDFIT_OUTPUTFILE_BIN_FIT, "no" );
                fitDef.save( );
            }        
            
            if ( fitDef.getProperty( TDFDFittingDefaults.DELETE_TDFDFIT_OUTPUTFILE_BIN_FIT ).compareTo( "yes" ) == 0 )
            {
                fileDotFit.delete( );
            }
            else
            {
                try 
                {
                    Files.move( fileDotFit.toPath( ), new File ( patDatabasePath.getPath( ) + System.getProperty( "file.separator" ) + m_fileNameBase  + ".fit"  ).toPath( ), java.nio.file.StandardCopyOption.REPLACE_EXISTING);
//                    System.out.println( "Moved file : " + fileDotFit.toPath(  ) );
                } 
                catch ( IOException ex ) 
                {
                   System.out.println( "Error moving file : " + fileDotFit.toPath(  ) );
                }
                
                
//                fileDotFit.renameTo( new File ( patDatabasePath.getPath( ) + System.getProperty( "file.separator" ) + m_fileNameBase  + ".fit"  ) );
            }
            
            //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
            
            
            File fitfile0 = new File ( patDatabasePath.getPath( ) + System.getProperty( "file.separator" ) + m_fileNameBase  + ".fit0"  );
            if ( fitfile.exists( ) )
            {
                fitfile0.delete( );
            }
                                    
            if ( fitDef.getProperty( TDFDFittingDefaults.DELETE_TDFDFIT_OUTPUTFILE_BIN_FIT0 ) == null )
            {
                fitDef.setProperty( TDFDFittingDefaults.DELETE_TDFDFIT_OUTPUTFILE_BIN_FIT0, "no" );
            }
            
            if ( fitDef.getProperty( TDFDFittingDefaults.DELETE_TDFDFIT_OUTPUTFILE_BIN_FIT0 ).compareTo( "yes" ) == 0 )
            {
                fileDotFit0.delete( );
            }
            else
            {
                try 
                {
                    Files.move( fileDotFit0.toPath( ), new File ( patDatabasePath.getPath( ) + System.getProperty( "file.separator" ) + m_fileNameBase  + ".fit0" ).toPath( ), java.nio.file.StandardCopyOption.REPLACE_EXISTING);
//                    System.out.println( "Moved file : " + fileDotFit0.toPath( ) );
                } 
                catch ( IOException ex ) 
                {
                   System.out.println( "Error moving file : " + fileDotFit0.toPath( ) );
                }
                
//                fileDotFit0.renameTo( new File ( patDatabasePath.getPath( ) + System.getProperty( "file.separator" ) + m_fileNameBase  + ".fit0"  ) );
            }
            
            //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
            
            File filedotfitres = new File ( patDatabasePath.getPath( ) + System.getProperty( "file.separator" ) + m_fileNameBase  + ".fitres"  );
            if ( filedotfitres.exists( ) )
            {
                filedotfitres.delete( );
            }
            
            if ( fitDef.getProperty( TDFDFittingDefaults.DELETE_TDFDFIT_OUTPUTFILE_BIN_FITRES ) == null )
            {
                fitDef.setProperty( TDFDFittingDefaults.DELETE_TDFDFIT_OUTPUTFILE_BIN_FITRES, "no" );
                fitDef.save( );
            }        
            
            if ( fitDef.getProperty( TDFDFittingDefaults.DELETE_TDFDFIT_OUTPUTFILE_BIN_FITRES ).compareTo( "yes" ) == 0 )
            {
                fileDotFitRes.delete( );
            }
            else
            {
                try 
                {
                    Files.move( fileDotFitRes.toPath( ), new File ( patDatabasePath.getPath( ) + System.getProperty( "file.separator" ) + m_fileNameBase  + ".fitres"  ).toPath( ), java.nio.file.StandardCopyOption.REPLACE_EXISTING);
//                    System.out.println( "Moved file : " + fileDotFitRes.toPath( ) );
                } 
                catch ( IOException ex ) 
                {
                   System.out.println( "Error moving file : " + fileDotFitRes.toPath( ) );
                }
                
                
                // fileDotFitRes.renameTo( new File ( patDatabasePath.getPath( ) + System.getProperty( "file.separator" ) + m_fileNameBase  + ".fitres"  ) );
            }
            
            //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
            
            File file_dot_org =  new File ( patDatabasePath.getPath( ) + System.getProperty( "file.separator" ) + m_fileNameBase  + ".org"  );
            if ( file_dot_org.exists( ) )
            {
                file_dot_org.delete( );
            }
            
            if ( fitDef.getProperty( TDFDFittingDefaults.DELETE_TDFDFIT_OUTPUTFILE_BIN_ORG ) == null )
            {
                fitDef.setProperty( TDFDFittingDefaults.DELETE_TDFDFIT_OUTPUTFILE_BIN_ORG, "no" );
                fitDef.save( );
            }
            
            if ( fitDef.getProperty( TDFDFittingDefaults.DELETE_TDFDFIT_OUTPUTFILE_BIN_ORG ).compareTo( "yes" ) == 0 )
            {
                fileDotOrg.delete( );
            }
            else
            {
                
                try 
                {
                    Files.move( fileDotOrg.toPath( ), new File( patDatabasePath.getPath( ) + System.getProperty( "file.separator" ) + m_fileNameBase  + ".org"  ).toPath( ), java.nio.file.StandardCopyOption.REPLACE_EXISTING);
//                    System.out.println( "Moved file : " + fileDotOrg.toPath( ) );
                } 
                catch ( IOException ex ) 
                {
                   System.out.println( "Error moving file : " + fileDotOrg.toPath( ) );
                }
                
//                fileDotOrg.renameTo( new File ( patDatabasePath.getPath( ) + System.getProperty( "file.separator" ) + m_fileNameBase  + ".org"  ) );
            }
            
             //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
             
            File file_dot_res = new File ( patDatabasePath.getPath( ) + System.getProperty( "file.separator" ) + m_fileNameBase  + ".res"  );
            if ( file_dot_res.exists( ) )
            {
                file_dot_res.delete( );
            }
            
            if ( fitDef.getProperty( TDFDFittingDefaults.DELETE_TDFDFIT_OUTPUTFILE_BIN_RES ) == null )
            {
                fitDef.setProperty( TDFDFittingDefaults.DELETE_TDFDFIT_OUTPUTFILE_BIN_RES, "no" );
                fitDef.save( );
            }
            if ( fitDef.getProperty( TDFDFittingDefaults.DELETE_TDFDFIT_OUTPUTFILE_BIN_RES ).compareTo( "yes" ) == 0 )
            {
                fileDotRes.delete( );
            }
            else
            {
                try 
                {
                    Files.move( fileDotRes.toPath( ), new File ( patDatabasePath.getPath( ) + System.getProperty( "file.separator" ) + m_fileNameBase  + ".res"  ).toPath( ), java.nio.file.StandardCopyOption.REPLACE_EXISTING);
//                    System.out.println( "Moved file : " + fileDotRes.toPath( ) );
                } 
                catch ( IOException ex ) 
                {
                   System.out.println( "Error moving file : " + fileDotRes.toPath( ) );
                }
                
                
//                fileDotRes.renameTo( new File ( patDatabasePath.getPath( ) + System.getProperty( "file.separator" ) + m_fileNameBase  + ".res"  ) );
            }
            
            //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
            
            File file_dot_cororg = new File ( patDatabasePath.getPath( ) + System.getProperty( "file.separator" ) + m_fileNameBase  + ".cororg"  );
            if ( file_dot_cororg.exists( ) )
            {
                file_dot_cororg.delete( );
            }
            
            if ( fitDef.getProperty( TDFDFittingDefaults.DELETE_TDFDFIT_OUTPUTFILE_BIN_CORORG ) == null )
            {
                fitDef.setProperty( TDFDFittingDefaults.DELETE_TDFDFIT_OUTPUTFILE_BIN_CORORG, "no" );
                fitDef.save( );
            }
            if ( fitDef.getProperty( TDFDFittingDefaults.DELETE_TDFDFIT_OUTPUTFILE_BIN_CORORG ).compareTo( "yes" ) == 0 )
            {
                fileDotCorOrg.delete( );
            }
            else
            {
                try 
                {
                    Files.move( fileDotCorOrg.toPath( ), new File( patDatabasePath.getPath( ) + System.getProperty( "file.separator" ) + m_fileNameBase  + ".cororg"  ).toPath( ), java.nio.file.StandardCopyOption.REPLACE_EXISTING);
//                    System.out.println( "Moved file : " + fileDotCorOrg.toPath( ) );
                } 
                catch ( IOException ex ) 
                {
                   System.out.println( "Error moving file : " + fileDotCorOrg.toPath( ) );
                }
                
                
//                fileDotCorOrg.renameTo( new File ( patDatabasePath.getPath( ) + System.getProperty( "file.separator" ) + m_fileNameBase  + ".cororg"  ) );
            }
            
            //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
            
            File file_dot_exl = new File ( patDatabasePath.getPath( ) + System.getProperty( "file.separator" ) + m_fileNameBase  + ".exl"  );
            if ( file_dot_exl.exists( ) )
            {
                file_dot_exl.delete( );
            }
            if ( fitDef.getProperty( TDFDFittingDefaults.DELETE_TDFDFIT_OUTPUTFILE_BIN_EXL ) == null )
            {
                fitDef.setProperty( TDFDFittingDefaults.DELETE_TDFDFIT_OUTPUTFILE_BIN_EXL, "no" );
                fitDef.save( );
            }
            
            if ( fitDef.getProperty( TDFDFittingDefaults.DELETE_TDFDFIT_OUTPUTFILE_BIN_EXL ).compareTo( "yes" ) == 0 )
            {
                fileDotExl.delete( );
            }
            else
            {  
                try 
                {
                    Files.move( fileDotExl.toPath( ), new File( patDatabasePath.getPath( ) + System.getProperty( "file.separator" ) + m_fileNameBase  + ".exl"  ).toPath( ), java.nio.file.StandardCopyOption.REPLACE_EXISTING);
//                    System.out.println( "Moved file : " + fileDotExl.toPath( ) );
                } 
                catch ( IOException ex ) 
                {
                   System.out.println( "Error moving file : " + fileDotExl.toPath( ) );
                }
                
//                fileDotExl.renameTo( new File ( patDatabasePath.getPath( ) + System.getProperty( "file.separator" ) + m_fileNameBase  + ".exl"  ) );
            }
            
            //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
            
            
            File file_dot_nln_0 = new File ( patDatabasePath.getPath( ) + System.getProperty( "file.separator" ) + m_fileNameBase  + ".nln0"  );
            if ( file_dot_nln_0.exists( ) )
            {
                file_dot_nln_0.delete( );
            }
            
            if ( fitDef.getProperty( TDFDFittingDefaults.DELETE_TDFDFIT_OUTPUTFILE_BIN_NLN0 ) == null )
            {
                fitDef.setProperty( TDFDFittingDefaults.DELETE_TDFDFIT_OUTPUTFILE_BIN_NLN0, "no" );
                fitDef.save( );
            }
            if ( fitDef.getProperty( TDFDFittingDefaults.DELETE_TDFDFIT_OUTPUTFILE_BIN_NLN0 ).compareTo( "yes" ) == 0 )
            {
                fileDotNln0.delete( );
            }
            else
            {
                try 
                {
                    Files.move( fileDotNln0.toPath( ), new File ( patDatabasePath.getPath( ) + System.getProperty( "file.separator" ) + m_fileNameBase  + ".nln0"  ).toPath( ), java.nio.file.StandardCopyOption.REPLACE_EXISTING);
//                    System.out.println( "Moved file : " + fileDotNln0.toPath( ) );
                } 
                catch ( IOException ex ) 
                {
                   System.out.println( "Error moving file : " + fileDotNln0.toPath( ) );
                }
                
//                fileDotNln0.renameTo( new File ( patDatabasePath.getPath( ) + System.getProperty( "file.separator" ) + m_fileNameBase  + ".nln0"  ) );
            }
            
            //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
            
            File file_dot_mat = new File ( patDatabasePath.getPath( ) + System.getProperty( "file.separator" ) + m_fileNameBase  + ".mat"  );
            if ( file_dot_mat.exists( ) )
            {
                file_dot_mat.delete( );
            } 
            
            if ( fitDef.getProperty( TDFDFittingDefaults.DELETE_TDFDFIT_OUTPUTFILE_BIN_MAT ) == null )
            {
                fitDef.setProperty( TDFDFittingDefaults.DELETE_TDFDFIT_OUTPUTFILE_BIN_MAT, "no" );
                fitDef.save( );
            }
            if ( fitDef.getProperty( TDFDFittingDefaults.DELETE_TDFDFIT_OUTPUTFILE_BIN_MAT ).compareTo( "yes" ) == 0 )
            {
                fileDotMat.delete( );
            }
            else
            {
                try 
                {
                    Files.move( fileDotMat.toPath( ), new File ( patDatabasePath.getPath( ) + System.getProperty( "file.separator" ) + m_fileNameBase  + ".mat"  ).toPath( ), java.nio.file.StandardCopyOption.REPLACE_EXISTING);
//                    System.out.println( "Moved file : " + fileDotMat.toPath( ) );
                } 
                catch ( IOException ex ) 
                {
                   System.out.println( "Error moving file : " + fileDotMat.toPath( ) );
                }
                
//                fileDotMat.renameTo( new File ( patDatabasePath.getPath( ) + System.getProperty( "file.separator" ) + m_fileNameBase  + ".mat"  ) );
            }
            
            //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
            
            File file_dot_err = new File ( patDatabasePath.getPath( ) + System.getProperty( "file.separator" ) + m_fileNameBase  + ".err"  );
            if ( file_dot_err.exists( ) )
            {
                file_dot_err.delete( );
            }
            
            if ( fitDef.getProperty( TDFDFittingDefaults.DELETE_TDFDFIT_OUTPUTFILE_BIN_ERR ) == null )
            {
                fitDef.setProperty( TDFDFittingDefaults.DELETE_TDFDFIT_OUTPUTFILE_BIN_ERR, "no" );
                fitDef.save( );
            }
            if ( fitDef.getProperty( TDFDFittingDefaults.DELETE_TDFDFIT_OUTPUTFILE_BIN_ERR ).compareTo( "yes" ) == 0 )
            {
                fileDotErr.delete( );
            }
            else
            {
                try 
                {
                    Files.move( fileDotErr.toPath( ), new File ( patDatabasePath.getPath( ) + System.getProperty( "file.separator" ) + m_fileNameBase  + ".err"  ).toPath( ), java.nio.file.StandardCopyOption.REPLACE_EXISTING);
//                    System.out.println( "Moved file : " + fileDotErr.toPath( ) );
                } 
                catch ( IOException ex ) 
                {
                   System.out.println( "Error moving file : " + fileDotErr.toPath( ) );
                }
                
//                fileDotErr.renameTo( new File ( patDatabasePath.getPath( ) + System.getProperty( "file.separator" ) + m_fileNameBase  + ".err"  ) );
            }
           
        }
        
        // renameTo does not move, in fact it is a copy therefore we also need to delete the source files explicitly!
//        fileDotBin = new File( path + System.getProperty( "file.separator" ) + m_fileNameBase  + ".bin" );
//        if ( fileDotBin.exists( ) )
//        {
//            fileDotBin.delete( );
//        }
//        
//        fileDotFit = new File( path + System.getProperty( "file.separator" ) + m_fileNameBase  + ".bin.fit" );
//        if ( fileDotFit.exists( ) )
//        {
//            fileDotFit.delete( );
//        }
//        
//        fileDotFit0 = new File( path + System.getProperty( "file.separator" ) + m_fileNameBase  + ".bin.fit0" );
//        if ( fileDotFit0.exists( ) )
//        {
//            fileDotFit0.delete( );
//        }
//        
//        fileDotFitRes = new File( path + System.getProperty( "file.separator" ) + m_fileNameBase  + ".bin.fitres" );
//        if ( fileDotFitRes.exists( ) )
//        {
//            fileDotFitRes.delete( );
//        }
//        
//        fileDotOrg = new File( path + System.getProperty( "file.separator" ) + m_fileNameBase  + ".bin.org" );
//        if ( fileDotOrg.exists( ) )
//        {
//            fileDotOrg.delete( );
//        }
//        
//        fileDotRes = new File( path + System.getProperty( "file.separator" ) + m_fileNameBase  + ".bin.res" );
//        if ( fileDotRes.exists( ) )
//        {
//            fileDotRes.delete( );
//        }
//        
//        fileDotCorOrg = new File( path + System.getProperty( "file.separator" ) + m_fileNameBase  + ".bin.cororg" );
//        if ( fileDotCorOrg.exists( ) )
//        {
//            fileDotCorOrg.delete( );
//        }
//        
//        fileDotExl = new File( path + System.getProperty( "file.separator" ) + m_fileNameBase  + ".bin.exl" );
//        if ( fileDotExl.exists( ) )
//        {
//            fileDotExl.delete( );
//        }
//        
//        fileDotNln0 = new File( path + System.getProperty( "file.separator" ) + m_fileNameBase  + ".bin.nln0" );
//        if ( fileDotNln0.exists( ) )
//        {
//            fileDotNln0.delete( );
//        }
//        
//        fileDotMat = new File( path + System.getProperty( "file.separator" ) + m_fileNameBase  + ".bin.mat" );
//        if ( fileDotMat.exists( ) )
//        {
//            fileDotMat.delete( );
//        }
//        
//        fileDotErr = new File( path + System.getProperty( "file.separator" ) + m_fileNameBase  + ".bin.err" );
//        if ( fileDotErr.exists( ) )
//        {
//            fileDotErr.delete( );
//        }
    }
    
    
    
    /**
     * Called when doInBackgoundFunction is finished.
     */
    protected synchronized void done( )
    {   
//      System.out.println( "Done is called!" );
        if ( m_displayIntermediateResults == true )
        {
            TDFDFittingDefaults fittingDefaults = TDFDFittingDefaults.getInstance( );
            fittingDefaults.load( );
            String patientDir = fittingDefaults.getProperty( TDFDFittingDefaults.PATIENT_DATABASE_ROOT_PATH );
            FitResultViewerFrame fitViewer = new FitResultViewerFrame( m_MrSignalProperties.getNumberOfSamplePoints( ), 
                                                                       patientDir + System.getProperty( "file.separator" ) + m_dataFileToFitBin, 
                                                                       "", false  );
        }
    }
    
    /**
     * Adds an ActionListener to the list of action listeners who are notified
     * when something happens in this component.
     * @param e
     */
    public void addActionListener( ActionListener e ) 
    {
        m_listeners.add(e);
    }
    
    /**
     * Notifies registered ActionListeners.
     * @param eventType
     * @param actionCommand
     */
    private void notifyActionListeners( int eventType, String actionCommand )
    {
        Iterator it = m_listeners.iterator( );
        while( it.hasNext( ) )
        {
            ( ( ActionListener )it.next( ) ).actionPerformed( new ActionEvent( this, eventType, actionCommand ) );
        }
    }

}