/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package org.spectrim.tdfdfit.quantitativeresults;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.Iterator;
import java.util.LinkedList;
import javax.swing.JTable;
import javax.swing.ListSelectionModel;

/**
 * Container class that handles the definition and selection of normal value 
 * databases.
 * @author slotboom
 */
public class NormalDataBaseSelectionTable extends JTable implements ActionListener, MouseListener
{
    
    /**
     * Default constructor for the normal data base selection databases. 
     */
    public NormalDataBaseSelectionTable( )
    {
        super( );

        setModel( new NormalDataBaseSelectionTableModel( ) );

        setAutoCreateColumnsFromModel( false );
        setColumnSelectionAllowed (false );
        setRowSelectionAllowed( true );
        
        setSelectionMode( ListSelectionModel.SINGLE_SELECTION );

//        addPopupMenu( );

        m_selectionListener = new NormalDataBaseSelectionTableSelectionListener( this );

        getSelectionModel( ).addListSelectionListener( m_selectionListener );
        getColumnModel( ).getSelectionModel( ).addListSelectionListener( m_selectionListener );

        // listen to mouse events and react to them!
        addMouseListener( this );
    }

    /**
     * Adds an action listener to the list of action listeners who are notified.
     * @param e
     */
    public void addActionListener( ActionListener e )
    {
        m_listeners.add( e );
    }

    /**
     * Notifies all registered components interested in selection 
     * changes in the table.
     * @param eventType
     */
    public void notifyActionListeners( int eventType )
    {
        Iterator it = m_listeners.iterator( );

        while( it.hasNext( ) )
        {
            if ( eventType == NormalDataBaseSelectionTable.TABLE_SELECTED_EVENT )
            {
                ( ( ActionListener )it.next( ) ).actionPerformed( new ActionEvent( this, NormalDataBaseSelectionTable.TABLE_SELECTED_EVENT, "" ) );
            }
        }
    }

    /**
     * Called whenever a registered component raises an ActionEvent
     * @param arg0
     */
    @Override
    public void actionPerformed( ActionEvent arg0 )
    {
        // not implemented
    }


    /**
     * Is called whenever a mouse button is clicked
     * @param arg0
     */
    @Override
    public void mouseClicked( MouseEvent arg0 )
    {
        //System.out.println( "notifySelectionChange in mouseClicked" );
        //notifyActionListeners( TABLE_SELECTED_EVENT );
    }

    /**
     * The user pressed the table, this may be a change in the selected
     * row; Send a notification to the registered listeners of this table.
     * @param e
     */
    @Override
    public void mousePressed( MouseEvent e )
    {
//        if ( e.isPopupTrigger( ) )
//        {
//            m_popup.show( e.getComponent( ), e.getX( ), e.getY( ) );
//        }

//        notifySelectionChange( );
    }
    /**
     * The user released the mouse button within the table, 
     * this may be a change in the selected row; 
     * Send a notification to the registered listeners of this table.
     * @param e
     */
    @Override
    public void mouseReleased( MouseEvent e )
    {
//        if ( e.isPopupTrigger( ) )
//        {
//            m_popup.show( e.getComponent( ), e.getX( ), e.getY( ) );
//        }

        notifySelectionChange( );
    }

    @Override
    public void mouseEntered( MouseEvent e )
    {
        // not implemented
    }

    @Override
    public void mouseExited( MouseEvent e )
    {
        // not implemented
    }

    /**
     * Notifies listeners that a change in table selection was performed.
     */
    private void notifySelectionChange( )
    {
        
        // System.out.println( "notifySelectionChange" );

        notifyActionListeners( TABLE_SELECTED_EVENT );

//         if ( ( getSelectedRows( ).length == 0 ) || ( getSelectedRows( ).length > 2 ) )
//         {
//             m_applyLogic_AND_toImageSegments.setEnabled( false );
//             m_applyLogic_OR_toImageSegments.setEnabled( false );
//             m_applyLogic_EXOR_toImageSegments.setEnabled( false );
//             m_applyLogic_NOT_toImageSegments.setEnabled( false );
//
//         }
//
//         if ( getSelectedRows( ).length == 1 )
//         {
//             m_applyLogic_AND_toImageSegments.setEnabled( false );
//             m_applyLogic_OR_toImageSegments.setEnabled( false );
//             m_applyLogic_EXOR_toImageSegments.setEnabled( false );
//             m_applyLogic_NOT_toImageSegments.setEnabled( true );
//
//         }
//
//         if ( getSelectedRows( ).length == 2 )
//         {
//             m_applyLogic_AND_toImageSegments.setEnabled( true );
//             m_applyLogic_OR_toImageSegments.setEnabled( true );
//             m_applyLogic_EXOR_toImageSegments.setEnabled( true );
//             m_applyLogic_NOT_toImageSegments.setEnabled( false );
//
//         }

    }

    private final NormalDataBaseSelectionTableSelectionListener m_selectionListener;
    public final static int TABLE_SELECTED_EVENT = 0;

     // Listeners to action events in this class
    private final LinkedList<ActionListener> m_listeners = new LinkedList<>( );


}
