/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package org.spectrim.tdfdfit.quantitativeresults;

import java.io.IOException;
import java.io.RandomAccessFile;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * Container class that store the responses of the instances of the normal 
 * database spectrum response. 
 * @author slotboom
 */
public class NormalDatabaseSpectrumResponse
{

    /**
     * Default constructor.
     */
     public NormalDatabaseSpectrumResponse( )
     {

     }

    /**
     * Alternative constructor.
     * @param measuredReal
     * @param measuredImag
     * @param fittedReal
     * @param fittedImag
     */
    public NormalDatabaseSpectrumResponse( double[] measuredReal, double[] measuredImag,
                                           double[] fittedReal,   double[] fittedImag )
    {

        m_numberOfPoints = measuredReal.length;
        m_measuredReal = new double[measuredReal.length];
        m_measuredImag = new double[measuredReal.length];
        m_fittedReal   = new double[measuredReal.length];
        m_fittedImag   = new double[measuredReal.length];

        System.arraycopy( measuredReal, 0, m_measuredReal, 0, m_measuredReal.length );
        System.arraycopy( measuredImag, 0, m_measuredImag, 0, m_measuredImag.length );
        System.arraycopy( fittedReal,   0, m_fittedReal,   0, m_fittedReal.length );
        System.arraycopy( fittedImag,   0, m_fittedImag,   0, m_fittedImag.length );
    }

    /**
     * Reads and parses the attributes from a RandomAccesFile.
     * @param in
     */
    void read( RandomAccessFile in )
    {
        try
        {
            String readString = in.readUTF( );
            readString = in.readUTF( );
            m_numberOfPoints = in.readInt( );
           
            m_measuredReal = new double[m_numberOfPoints];
            m_measuredImag = new double[m_numberOfPoints];
            m_fittedReal   = new double[m_numberOfPoints];
            m_fittedImag   = new double[m_numberOfPoints];

            readString = in.readUTF( );
            
            for ( int i = 0; i < m_numberOfPoints; i++ )
            {
                m_measuredReal[i] = in.readDouble( );
            }
            in.readUTF( );

            in.readUTF( );
            for ( int i = 0; i < m_numberOfPoints; i++ )
            {
                m_measuredImag[i] = in.readDouble( );
            }
            in.readUTF( );

            in.readUTF( );
            for ( int i = 0; i < m_numberOfPoints; i++ )
            {
                m_fittedReal[i] = in.readDouble( );
            }
            in.readUTF( );

            in.readUTF( );
            for ( int i = 0; i < m_numberOfPoints; i++ )
            {
                m_fittedImag[i] = in.readDouble( );
            }
            in.readUTF( );
            
            in.readUTF( );

        }
        catch ( IOException ex )
        {
            Logger.getLogger( NormalDatabaseSpectrumResponse.class.getName( ) ).log( Level.SEVERE, null, ex );
        }

    }

    /**
     * Writes attributes to RandomAccessFile.
     * @param out
     */
    public void write( RandomAccessFile out )
    {
        try
        {
            out.writeUTF( "<NormalDatabaseSpectrumResponse>" );
            out.writeInt( m_measuredReal.length );
            
            
            //System.out.println( "writing m_measuredReal.length : " + m_measuredReal.length );


            out.writeUTF( "<MeasuredReal>" );
            for( int i = 0; i< m_measuredReal.length; i++)
            {
                out.writeDouble( m_measuredReal[i] );
            }
            out.writeUTF( "</MeasuredReal>" );

            out.writeUTF( "<MeasuredImag>" );
            for( int i = 0; i< m_measuredImag.length; i++)
            {
                out.writeDouble( m_measuredImag[i] );
            }
            out.writeUTF( "</MeasuredImag>" );

            out.writeUTF( "<FittedReal>" );
            for( int i = 0; i< m_fittedReal.length; i++)
            {
                out.writeDouble( m_fittedReal[i] );
            }
            out.writeUTF( "</FittedReal>" );

            out.writeUTF( "<FittedImag>" );
            for( int i = 0; i< m_fittedImag.length; i++)
            {
                out.writeDouble( m_fittedImag[i] );
            }
            out.writeUTF( "</FittedImag>" );

            out.writeUTF( "</NormalDatabaseSpectrumResponse>" );
        }
        catch ( IOException ex )
        {
            Logger.getLogger( NormalDatabaseSpectrumResponse.class.getName( ) ).log( Level.SEVERE, null, ex );
        }
    }

    /**
     * The measured real signal in the domain the user has chosen.
     * @return
     */
    public double[] getMeasuredReal( )
    {
       return m_measuredReal;
    }

    /**
     * The measured imaginary signal in the domain the user has chosen.
     * @return
     */
    public double[] getMeasuredImag( )
    {
       return m_measuredImag;
    }

      /**
     * The fitted real signal in the domain the user has chosen.
     * @return
     */
    public double[] getFittedReal( )
    {
       return m_fittedReal;
    }

    /**
     * The fitted imaginary signal in the domain the user has chosen.
     * @return
     */
    public double[] getFittedImag( )
    {
       return m_fittedImag;
    }


    // Attributes
    private double[] m_measuredReal;
    private double[] m_measuredImag;
    private double[] m_fittedReal;
    private double[] m_fittedImag;

    private int      m_numberOfPoints;
    
}
