/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

/*
 * NormalMetaboliteValueDatabaseBrowserFrame.java
 *
 * Created on 01-Apr-2011, 10:35:34
 */

package org.spectrim.tdfdfit.quantitativeresults;

import org.spectrim.lib.Dialogs;
//import scanalyze2.SingleVoxelSpectroscopy.SingleVoxelSpectralData;
import org.spectrim.xyplot.Curve;
import java.awt.Color;
import java.awt.Point;
import java.text.DecimalFormat;
import java.util.LinkedList;
import javax.swing.JFrame;
import javax.swing.table.DefaultTableModel;
import org.spectrim.tdfdfit.SingleVoxelMeasuredSpectralData;
import org.spectrim.tdfdfit.SingleVoxelSpectralData;

/**
 * Frame that allows to view the content of the normal value databases.
 * 
 * @author slotboom
 */
public class NormalMetaboliteValueDatabaseBrowserFrame extends javax.swing.JFrame
{

    /** 
     * Creates new form NormalMetaboliteValueDatabaseBrowserFrame.
     * @param parent
     * @param selectedDatabase
     */
    public NormalMetaboliteValueDatabaseBrowserFrame( JFrame parent, 
                                                      String selectedDatabase )
    {

        initComponents( );
        System.out.println( "Opening selectedDatabase : " + selectedDatabase );

        m_currentlySelectedNormalValuesDatabase = new NormalMetaboliteValuesAndSpectrumDatabase( selectedDatabase );

        if  ( m_currentlySelectedNormalValuesDatabase != null )
        {
            m_numberOfMetabolites = m_currentlySelectedNormalValuesDatabase.getMetaboliteNames( ).size( );
            
            m_metaboliteNames   = m_currentlySelectedNormalValuesDatabase.getMetaboliteNames( );
            m_currentResponse   = m_currentlySelectedNormalValuesDatabase.getResponse( m_currentlySelectedDatabaseElement );
            m_currentParameters = m_currentlySelectedNormalValuesDatabase.getNormalDatabaseMetaboliteParameters( m_currentlySelectedDatabaseElement );
            m_currentlySelectedLabel.setText( "" + ( m_currentlySelectedDatabaseElement + 1 ) );
            m_sizeOfDatabaseLabel.setText( "of  " + m_currentlySelectedNormalValuesDatabase.getNumberOfInstancesInDatabase( ) + "  instances." );

            // If everything is there: setup and display table.
            setUpResultsTable( );
            
            m_parent = parent;
            // Put the Dialog nicely in the middle of its parent
            Point parentLoc = parent.getLocation( );
            Point centerParent = new Point( parent.getX( ) + ( parent.getWidth( ) / 2 ) - this.getWidth( ) / 2,
                                            parent.getY( ) + ( parent.getHeight( ) / 2 ) - this.getHeight( ) / 2 );

            setTitle( "Normal metabolite parameter value database browser" );
            pack( );
            setLocation( centerParent );

            //frame.dispose( );

            setVisible( true );
            // draw it!
            m_normalValueDatabaseTable.updateUI( );

            m_singleVoxelSpectrumViewer.setDisplayXReversed( true );
            //m_singleVoxelSpectrumViewer.set_Default_1H_ForcedPlotRange( );

            m_singleVoxelSpectrumViewer.enablePopupMenu( false );

            drawSpectrum( );
        }
        else
        {
            int messageBox = Dialogs.messageBox( this, "Could not open database " + selectedDatabase + "." );
        }


    }

    /**
     * Draws the spectra of the selected database instance.
     */
    private void drawSpectrum( )
    {
        
        m_currentResponse   = m_currentlySelectedNormalValuesDatabase.getResponse( m_currentlySelectedDatabaseElement );

        double[] measRe     = m_currentResponse.getMeasuredReal( );
        double[] measIm     = m_currentResponse.getMeasuredImag( );

        double[] fittedRe   = m_currentResponse.getFittedReal( );
        double[] fittedIm   = m_currentResponse.getFittedImag( );

        SingleVoxelMeasuredSpectralData specData = new SingleVoxelMeasuredSpectralData( );

        specData.setSpectralDataPointsFD( measRe, measIm );
        specData.setSiemensSpecificDicomHeaderInfo( null ); 
        specData.setCurveDrawingStyle( Curve.FAT_SOLID_LINE );
        specData.setCurveColor( Color.RED );
        specData.setCurveLabel( "measured response" );

        m_singleVoxelSpectrumViewer.deleteAllCurves( );
        
        m_singleVoxelSpectrumViewer.setTitle( "Response from database" );

        m_singleVoxelSpectrumViewer.addSpectralDataSet( "measured response", 
                                                        specData, true );

        /// Measured Response

        SingleVoxelMeasuredSpectralData specData2 = new SingleVoxelMeasuredSpectralData( );

        specData2.setSpectralDataPointsFD( fittedRe, fittedIm );
        specData2.setSiemensSpecificDicomHeaderInfo( null );
        specData2.setCurveDrawingStyle( Curve.FAT_SOLID_LINE );
        specData2.setCurveColor( Color.BLACK );
        specData2.setCurveLabel( "fitted model" );

        m_singleVoxelSpectrumViewer.addSpectralDataSet( "fited model", specData2, true );

        if ( m_overlayWithAverageSpectrumCheckBox.isSelected( ) == true )
        {
            m_singleVoxelSpectrumViewer.addSpectralDataSet( "average spectrum", 
                                                            m_avSpecData, true );
        }

        m_singleVoxelSpectrumViewer.enablePopupMenu( false );
    }


     /**
     * Sets up the results table.
     */
    private void setUpResultsTable( )
    {

        DecimalFormat formatter  = new DecimalFormat(  "#########0.0" );
        
        m_currentParameters = m_currentlySelectedNormalValuesDatabase.getNormalDatabaseMetaboliteParameters( m_currentlySelectedDatabaseElement );

        // set the correct data object and size
        m_cellData = new Object[m_numberOfMetabolites][6];


        if ( m_displayAverageParametersCheckBox.isSelected( ) == false )
        {
            for ( int i = 0; i < m_numberOfMetabolites; i++ )
            {

                // names of metabs
                m_cellData[i][0] = m_metaboliteNames.get(i);

                // concentrations

                if ( m_preferredDisplayFormat.compareTo( METAB_DATA_RAW_UNCORRECTED_TYPE ) == 0 )
                {
                    m_cellData[i][1] = formatter.format( m_currentParameters.getRawPeakAreas( )[i] );   //  + " +/- " + formatter.format( m_errorConcentrations[m_currentlySelectedSignalNumber][i]  );
                }

                if ( m_preferredDisplayFormat.compareTo( METAB_DATA_RAW_COMPONENTS_ONLY_CORRECTED_TYPE ) == 0 )
                {
                    m_cellData[i][1] = formatter.format( m_currentParameters.getRawFractionCorrectedPeakAreas( )[i] );   //  + " +/- " + formatter.format( m_errorConcentrations[m_currentlySelectedSignalNumber][i]  );
                }

                if ( m_preferredDisplayFormat.compareTo( METAB_DATA_JMRUI_CONC_STYLE ) == 0 )
                {
                    m_cellData[i][1] = formatter.format( m_currentParameters.getJMRUITypeConcentrations( )[i] );   //  + " +/- " + formatter.format( m_errorConcentrations[m_currentlySelectedSignalNumber][i]  );
                }

                if ( m_preferredDisplayFormat.compareTo( METAB_DATA_TRUECONC_TWO_COMPONENTS_CORRECTED ) == 0 )
                {
                    m_cellData[i][1] = formatter.format( m_currentParameters.getTrueTwoCompartmentModelConcentrations( )[i] );  //  + " +/- " + formatter.format( m_errorConcentrations[m_currentlySelectedSignalNumber][i]  );
                }

                // chem. shift
                m_cellData[i][2] = formatter.format( m_currentParameters.getDFreqs( )[i] );             // + " +/- " + formatter.format( m_errorsChemShift[m_currentlySelectedSignalNumber][i] );

                // phase
                m_cellData[i][3] = formatter.format( m_currentParameters.getPhases( )[i] );             // + " +/- " + formatter.format( m_errorsPhase[m_currentlySelectedSignalNumber][i] );

                // lorentz width
                m_cellData[i][4] = formatter.format( m_currentParameters.getLorentzWidths( )[i] );      // + " +/- " + formatter.format( m_errorsLorentzWidth[m_currentlySelectedSignalNumber][i] );

                // lorentzR width
                m_cellData[i][5] = formatter.format( m_currentParameters.getLorentzRWidths( )[i] );     // + " +/- " + formatter.format( m_errorsLorentzWidthR[m_currentlySelectedSignalNumber][i] );
            }

            String[] columnNames = null;

            if ( m_preferredDisplayFormat.compareTo( METAB_DATA_RAW_UNCORRECTED_TYPE ) == 0 )
            {
                String[] columnNames1 = { "<HTML><B>Component name</B></HTML>",
                                          "<HTML><B>Peakareas raw [a.u]</B></HTML>",
                                          "<HTML><B>Chem. shift</B></HTML>",
                                          "<HTML><B>Phase</B><HTML>",
                                          "<HTML><B>Lorentzian damping</B><HTML>",
                                          "<HTML><B>Lorentzian damping R</B><HTML>"
                              };

                columnNames = columnNames1;
            }

            if ( m_preferredDisplayFormat.compareTo( METAB_DATA_RAW_COMPONENTS_ONLY_CORRECTED_TYPE ) == 0 )
            {
                String[] columnNames1 = { "<HTML><B>Component name</B></HTML>",
                                          "<HTML><B>Peakareas corr. [a.u]</B></HTML>",
                                          "<HTML><B>Chem. shift</B></HTML>",
                                          "<HTML><B>Phase</B><HTML>",
                                          "<HTML><B>Lorentzian damping</B><HTML>",
                                          "<HTML><B>Lorentzian damping R</B><HTML>"
                              };
                columnNames = columnNames1;
            }

            if ( m_preferredDisplayFormat.compareTo( METAB_DATA_JMRUI_CONC_STYLE ) == 0 )
            {
                String[] columnNames1 = { "<HTML><B>Component name</B></HTML>",
                                          "<HTML><B>Peakareas [mmol/l]</B></HTML>",
                                          "<HTML><B>Chem. shift</B></HTML>",
                                          "<HTML><B>Phase</B><HTML>",
                                          "<HTML><B>Lorentzian damping</B><HTML>",
                                          "<HTML><B>Lorentzian damping R</B><HTML>"
                              };
                columnNames = columnNames1;
            }


            if ( m_preferredDisplayFormat.compareTo( METAB_DATA_TRUECONC_TWO_COMPONENTS_CORRECTED ) == 0 )
            {
                String[] columnNames1 = { "<HTML><B>Component name</B></HTML>",
                                          "<HTML><B>Peakareas corr. [mmol/l]</B></HTML>",
                                          "<HTML><B>Chem. shift</B></HTML>",
                                          "<HTML><B>Phase</B><HTML>",
                                          "<HTML><B>Lorentzian damping</B><HTML>",
                                          "<HTML><B>Lorentzian damping R</B><HTML>"
                                        };
                
                columnNames = columnNames1;
            }


            m_tableModel = new DefaultTableModel( m_cellData, columnNames );

            m_normalValueDatabaseTable.setModel( m_tableModel );
            m_normalValueDatabaseTable.updateUI( );
        }
        else
        {
            for ( int i = 0; i < m_numberOfMetabolites; i++ )
            {

                // Names of metabolites
                m_cellData[i][0] = m_metaboliteNames.get(i);

                // concentrations

                if ( m_preferredDisplayFormat.compareTo( METAB_DATA_RAW_UNCORRECTED_TYPE ) == 0 )
                {
                    m_cellData[i][1] = formatter.format( m_currentParameters.getRawPeakAreas( )[i] ) + "  \t(" + formatter.format( m_averagesDatabaseParameters.getRawPeakAreas( )[i] ) + " +/- " + formatter.format( m_stDevsDatabaseParameters.getRawPeakAreas( )[i] ) + ")";
                }

                if ( m_preferredDisplayFormat.compareTo( METAB_DATA_RAW_COMPONENTS_ONLY_CORRECTED_TYPE ) == 0 )
                {
                    m_cellData[i][1] = formatter.format( m_currentParameters.getRawFractionCorrectedPeakAreas( )[i] ) + "  \t(" + formatter.format( m_averagesDatabaseParameters.getRawFractionCorrectedPeakAreas( )[i]  ) + " +/- " + formatter.format( m_stDevsDatabaseParameters.getRawFractionCorrectedPeakAreas( )[i] )+ ")";
                }

                if ( m_preferredDisplayFormat.compareTo( METAB_DATA_JMRUI_CONC_STYLE ) == 0 )
                {
                    m_cellData[i][1] = formatter.format( m_currentParameters.getJMRUITypeConcentrations( )[i] ) + "  \t(" + formatter.format( m_averagesDatabaseParameters.getJMRUITypeConcentrations( )[i]  ) + " +/- " + formatter.format( m_stDevsDatabaseParameters.getJMRUITypeConcentrations( )[i] ) + ")";
                }

                if ( m_preferredDisplayFormat.compareTo( METAB_DATA_TRUECONC_TWO_COMPONENTS_CORRECTED ) == 0 )
                {
                    m_cellData[i][1] = formatter.format( m_currentParameters.getTrueTwoCompartmentModelConcentrations( )[i] ) + "  \t(" + formatter.format( m_averagesDatabaseParameters.getTrueTwoCompartmentModelConcentrations( )[i] ) + " +/- " + formatter.format( m_stDevsDatabaseParameters.getTrueTwoCompartmentModelConcentrations( )[i] ) + ")";
                }

                // chem. shift
                m_cellData[i][2] = formatter.format( m_currentParameters.getDFreqs( )[i] ) + "  \t(" +   formatter.format( m_averagesDatabaseParameters.getDFreqs( )[i] ) + " +/- " + formatter.format( m_stDevsDatabaseParameters.getDFreqs( )[i] ) + ")";

                // phase
                m_cellData[i][3] = formatter.format( m_currentParameters.getPhases( )[i] ) + "  \t("  + formatter.format( m_averagesDatabaseParameters.getPhases( )[i] ) + " +/- " + formatter.format( m_stDevsDatabaseParameters.getPhases( )[i] ) + ")";

                // lorentz width
                m_cellData[i][4] = formatter.format( m_currentParameters.getLorentzWidths( )[i] ) + "  \t("  +  formatter.format( m_averagesDatabaseParameters.getLorentzWidths( )[i] ) + " +/- " + formatter.format( m_stDevsDatabaseParameters.getLorentzWidths( )[i]) + ")";

                // lorentzR width
                m_cellData[i][5] = formatter.format( m_currentParameters.getLorentzRWidths( )[i] ) + "  \t("  + formatter.format( m_stDevsDatabaseParameters.getLorentzRWidths( )[i] ) + " +/- " + formatter.format( m_stDevsDatabaseParameters.getLorentzRWidths( )[i] ) + ")";
            }

            String[] columnNames = null;

            if ( m_preferredDisplayFormat.compareTo( METAB_DATA_RAW_UNCORRECTED_TYPE ) == 0 )
            {
                String[] columnNames1 = { "<HTML><B>Component name</B></HTML>",
                                "<HTML><B>Peakareas raw [a.u]</B></HTML>",
                                "<HTML><B>Chem. shift</B></HTML>",
                                "<HTML><B>Phase</B><HTML>",
                                "<HTML><B>Lorentzian damping</B><HTML>",
                                "<HTML><B>Lorentzian damping R</B><HTML>"
                              };

                columnNames = columnNames1;
            }

            if ( m_preferredDisplayFormat.compareTo( METAB_DATA_RAW_COMPONENTS_ONLY_CORRECTED_TYPE ) == 0 )
            {
                String[] columnNames1 = { "<HTML><B>Component name</B></HTML>",
                                "<HTML><B>Peakareas corr. [a.u]</B></HTML>",
                                "<HTML><B>Chem. shift</B></HTML>",
                                "<HTML><B>Phase</B><HTML>",
                                "<HTML><B>Lorentzian damping</B><HTML>",
                                "<HTML><B>Lorentzian damping R</B><HTML>"
                              };
                columnNames = columnNames1;
            }

            if ( m_preferredDisplayFormat.compareTo( METAB_DATA_JMRUI_CONC_STYLE ) == 0 )
            {
                String[] columnNames1 = { "<HTML><B>Component name</B></HTML>",
                                "<HTML><B>Peakareas [mmol/l]</B></HTML>",
                                "<HTML><B>Chem. shift</B></HTML>",
                                "<HTML><B>Phase</B><HTML>",
                                "<HTML><B>Lorentzian damping</B><HTML>",
                                "<HTML><B>Lorentzian damping R</B><HTML>"
                              };
                columnNames = columnNames1;
            }


            if ( m_preferredDisplayFormat.compareTo( METAB_DATA_TRUECONC_TWO_COMPONENTS_CORRECTED ) == 0 )
            {
                String[] columnNames1 = { "<HTML><B>Component name</B></HTML>",
                                "<HTML><B>Peakareas corr. [mmol/l]</B></HTML>",
                                "<HTML><B>Chem. shift</B></HTML>",
                                "<HTML><B>Phase</B><HTML>",
                                "<HTML><B>Lorentzian damping</B><HTML>",
                                "<HTML><B>Lorentzian damping R</B><HTML>"
                              };
                columnNames = columnNames1;
            }


            m_tableModel = new DefaultTableModel( m_cellData, columnNames );

            m_normalValueDatabaseTable.setModel( m_tableModel );
            m_normalValueDatabaseTable.updateUI( );
        }

    }


    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jPanel1 = new javax.swing.JPanel();
        jScrollPane1 = new javax.swing.JScrollPane();
        m_normalValueDatabaseTable = new javax.swing.JTable();
        jPanel2 = new javax.swing.JPanel();
        m_showNextButton = new javax.swing.JButton();
        m_showPreviousButton = new javax.swing.JButton();
        m_selLabel = new javax.swing.JLabel();
        m_currentlySelectedLabel = new javax.swing.JLabel();
        m_sizeOfDatabaseLabel = new javax.swing.JLabel();
        m_deleteCurrentDBButton = new javax.swing.JButton();
        jPanel4 = new javax.swing.JPanel();
        m_overlayWithAverageSpectrumCheckBox = new javax.swing.JCheckBox();
        m_displayAverageParametersCheckBox = new javax.swing.JCheckBox();
        m_singleVoxelSpectrumViewer = new org.spectrim.tdfdfit.SingleVoxelSpectrumViewer();
        jPanel3 = new javax.swing.JPanel();
        m_closeFameButton = new javax.swing.JButton();

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);

        m_normalValueDatabaseTable.setModel(new javax.swing.table.DefaultTableModel(
            new Object [][] {
                {null, null, null, null},
                {null, null, null, null},
                {null, null, null, null},
                {null, null, null, null}
            },
            new String [] {
                "Title 1", "Title 2", "Title 3", "Title 4"
            }
        ));
        jScrollPane1.setViewportView(m_normalValueDatabaseTable);

        m_showNextButton.setText("next");
        m_showNextButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_showNextButtonActionPerformed(evt);
            }
        });

        m_showPreviousButton.setText("previous");
        m_showPreviousButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_showPreviousButtonActionPerformed(evt);
            }
        });

        m_selLabel.setText("Selected instance: ");

        m_currentlySelectedLabel.setText("0");

        m_sizeOfDatabaseLabel.setText("of  0 instances.");

        m_deleteCurrentDBButton.setText("Delete current DB-entry");
        m_deleteCurrentDBButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_deleteCurrentDBButtonActionPerformed(evt);
            }
        });

        m_overlayWithAverageSpectrumCheckBox.setText("Overlay with average spectrum");
        m_overlayWithAverageSpectrumCheckBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_overlayWithAverageSpectrumCheckBoxActionPerformed(evt);
            }
        });

        m_displayAverageParametersCheckBox.setText("show average parameters");
        m_displayAverageParametersCheckBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_displayAverageParametersCheckBoxActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout jPanel4Layout = new javax.swing.GroupLayout(jPanel4);
        jPanel4.setLayout(jPanel4Layout);
        jPanel4Layout.setHorizontalGroup(
            jPanel4Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel4Layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jPanel4Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(m_overlayWithAverageSpectrumCheckBox)
                    .addComponent(m_displayAverageParametersCheckBox))
                .addContainerGap(69, Short.MAX_VALUE))
        );
        jPanel4Layout.setVerticalGroup(
            jPanel4Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel4Layout.createSequentialGroup()
                .addComponent(m_overlayWithAverageSpectrumCheckBox)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(m_displayAverageParametersCheckBox)
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        javax.swing.GroupLayout jPanel2Layout = new javax.swing.GroupLayout(jPanel2);
        jPanel2.setLayout(jPanel2Layout);
        jPanel2Layout.setHorizontalGroup(
            jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel2Layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(m_showPreviousButton)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(m_showNextButton, javax.swing.GroupLayout.PREFERRED_SIZE, 75, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(m_selLabel)
                .addGap(18, 18, 18)
                .addComponent(m_currentlySelectedLabel, javax.swing.GroupLayout.PREFERRED_SIZE, 52, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(m_sizeOfDatabaseLabel, javax.swing.GroupLayout.PREFERRED_SIZE, 141, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addGap(18, 18, 18)
                .addComponent(m_deleteCurrentDBButton)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jPanel4, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addContainerGap())
        );
        jPanel2Layout.setVerticalGroup(
            jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel2Layout.createSequentialGroup()
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addGroup(jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, jPanel2Layout.createSequentialGroup()
                        .addGroup(jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(m_showPreviousButton)
                            .addComponent(m_showNextButton)
                            .addComponent(m_selLabel)
                            .addComponent(m_currentlySelectedLabel)
                            .addComponent(m_sizeOfDatabaseLabel)
                            .addComponent(m_deleteCurrentDBButton))
                        .addGap(33, 33, 33))
                    .addComponent(jPanel4, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)))
        );

        javax.swing.GroupLayout jPanel1Layout = new javax.swing.GroupLayout(jPanel1);
        jPanel1.setLayout(jPanel1Layout);
        jPanel1Layout.setHorizontalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(jScrollPane1, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, 1128, Short.MAX_VALUE)
            .addComponent(jPanel2, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
        );
        jPanel1Layout.setVerticalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel1Layout.createSequentialGroup()
                .addComponent(jScrollPane1, javax.swing.GroupLayout.PREFERRED_SIZE, 170, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jPanel2, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
        );

        m_closeFameButton.setText("Close");
        m_closeFameButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_closeFameButtonActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout jPanel3Layout = new javax.swing.GroupLayout(jPanel3);
        jPanel3.setLayout(jPanel3Layout);
        jPanel3Layout.setHorizontalGroup(
            jPanel3Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel3Layout.createSequentialGroup()
                .addGap(536, 536, 536)
                .addComponent(m_closeFameButton)
                .addContainerGap(538, Short.MAX_VALUE))
        );
        jPanel3Layout.setVerticalGroup(
            jPanel3Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel3Layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(m_closeFameButton)
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jPanel1, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addComponent(m_singleVoxelSpectrumViewer, javax.swing.GroupLayout.DEFAULT_SIZE, 1128, Short.MAX_VALUE)
                    .addComponent(jPanel3, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(m_singleVoxelSpectrumViewer, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jPanel1, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jPanel3, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addContainerGap())
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents
    
    /**
     * Displays those element that was higher in the list of databases.
     * @param evt 
     */
    private void m_showPreviousButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_showPreviousButtonActionPerformed
        if ( m_currentlySelectedDatabaseElement > 0 )
        {
            m_currentlySelectedDatabaseElement = m_currentlySelectedDatabaseElement - 1;
            m_currentlySelectedLabel.setText( "" + (m_currentlySelectedDatabaseElement + 1) );

            setUpResultsTable( );
            drawSpectrum( );
        }
    }//GEN-LAST:event_m_showPreviousButtonActionPerformed

    /**
     * Shows the next database element.
     * @param evt 
     */
    private void m_showNextButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_showNextButtonActionPerformed
        if ( m_currentlySelectedDatabaseElement < m_currentlySelectedNormalValuesDatabase.getNumberOfInstancesInDatabase( ) - 1 )
        {
            m_currentlySelectedDatabaseElement = m_currentlySelectedDatabaseElement + 1;
            m_currentlySelectedLabel.setText( "" + (m_currentlySelectedDatabaseElement + 1) );
            setUpResultsTable( );
            drawSpectrum( );
        }
    }//GEN-LAST:event_m_showNextButtonActionPerformed
    
    /**
     * Disposes this frame.
     * @param evt 
     */
    private void m_closeFameButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_closeFameButtonActionPerformed
        dispose( );
    }//GEN-LAST:event_m_closeFameButtonActionPerformed

    /**
     * Deletes the currently selected database.
     * @param evt 
     */
    private void m_deleteCurrentDBButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_deleteCurrentDBButtonActionPerformed

        m_currentlySelectedNormalValuesDatabase.deleteEntry( m_currentlySelectedDatabaseElement );

        // set the new selected database element to 0 and update the table
        if ( m_currentlySelectedNormalValuesDatabase.getNumberOfInstancesInDatabase( ) > -1 )
        {
            m_currentlySelectedDatabaseElement = 0;

            m_selLabel.setText( "Selected instance: " );
            m_currentlySelectedLabel.setText( "" + ( m_currentlySelectedDatabaseElement + 1 ) );
            m_sizeOfDatabaseLabel.setText( "of  " + m_currentlySelectedNormalValuesDatabase.getNumberOfInstancesInDatabase( ) + "  instances." );

            setUpResultsTable( );
            drawSpectrum( );
        }

    }//GEN-LAST:event_m_deleteCurrentDBButtonActionPerformed


    /**
     * Spectrum is displayed with an average normal spectrum overlay or not.
     * @param evt
     */
    private void m_overlayWithAverageSpectrumCheckBoxActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_overlayWithAverageSpectrumCheckBoxActionPerformed
        if ( m_overlayWithAverageSpectrumCheckBox.isSelected( ) == true )
        {
            m_avSpecData = m_currentlySelectedNormalValuesDatabase.getAverageSpectrum( );
            drawSpectrum( );
        }
        else
        {
            m_avSpecData = null;
            drawSpectrum( );
        }
}//GEN-LAST:event_m_overlayWithAverageSpectrumCheckBoxActionPerformed
    
    /**
     * Displays the average spectral parameters of the model.
     * @param evt 
     */
    private void m_displayAverageParametersCheckBoxActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_displayAverageParametersCheckBoxActionPerformed
        if ( m_displayAverageParametersCheckBox.isSelected( ) == true )
        {
            m_averagesDatabaseParameters = m_currentlySelectedNormalValuesDatabase.getAveragesDatabaseParameters( );
            m_stDevsDatabaseParameters = m_currentlySelectedNormalValuesDatabase.getStDevsDatabaseParameters( );
            setUpResultsTable( );
        }
        else
        {
            m_averagesDatabaseParameters = null;
            m_stDevsDatabaseParameters = null;
            setUpResultsTable( );
        }
    }//GEN-LAST:event_m_displayAverageParametersCheckBoxActionPerformed



    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JPanel jPanel1;
    private javax.swing.JPanel jPanel2;
    private javax.swing.JPanel jPanel3;
    private javax.swing.JPanel jPanel4;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JButton m_closeFameButton;
    private javax.swing.JLabel m_currentlySelectedLabel;
    private javax.swing.JButton m_deleteCurrentDBButton;
    private javax.swing.JCheckBox m_displayAverageParametersCheckBox;
    private javax.swing.JTable m_normalValueDatabaseTable;
    private javax.swing.JCheckBox m_overlayWithAverageSpectrumCheckBox;
    private javax.swing.JLabel m_selLabel;
    private javax.swing.JButton m_showNextButton;
    private javax.swing.JButton m_showPreviousButton;
    private org.spectrim.tdfdfit.SingleVoxelSpectrumViewer m_singleVoxelSpectrumViewer;
    private javax.swing.JLabel m_sizeOfDatabaseLabel;
    // End of variables declaration//GEN-END:variables

    // Further class attributes.
    private JFrame              m_parent;
    private DefaultTableModel   m_tableModel;
    private final NormalMetaboliteValuesAndSpectrumDatabase m_currentlySelectedNormalValuesDatabase;
    private Object[][]          m_cellData;

    private int                 m_numberOfMetabolites;
    private LinkedList<String>  m_metaboliteNames = new LinkedList<>();
    private int                 m_currentlySelectedDatabaseElement = 0;

    private NormalDatabaseSpectrumResponse      m_currentResponse;
    private NormalDatabaseMetaboliteParameters  m_currentParameters;

    private NormalDatabaseMetaboliteParameters  m_averagesDatabaseParameters;
    private NormalDatabaseMetaboliteParameters  m_stDevsDatabaseParameters;


    private SingleVoxelMeasuredSpectralData m_avSpecData;
    
    
    private final String m_preferredDisplayFormat = METAB_DATA_RAW_UNCORRECTED_TYPE;

    public final static String METAB_DATA_RAW_UNCORRECTED_TYPE                = "METAB_DATA_RAW_UNCORRECTED_TYPE";
    public final static String METAB_DATA_RAW_COMPONENTS_ONLY_CORRECTED_TYPE  = "METAB_DATA_RAW_COMPONENTS_ONLY_CORRECTED_TYPE";
    public final static String METAB_DATA_JMRUI_CONC_STYLE                    = "METAB_DATA_JMRUI_CONC_STYLE";
    public final static String METAB_DATA_TRUECONC_TWO_COMPONENTS_CORRECTED   = "METAB_DATA_TRUECONC_TWO_COMPONENTS_CORRECTED";
}
