/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package org.spectrim.tdfdfit.quantitativeresults;

import org.spectrim.dicom.DicomDecoder;
import org.spectrim.dicom.DicomFileOverNetworkTransferrer;
import org.spectrim.dicom.DicomImageData;
import org.spectrim.dicom.DicomNetworkNodeDefaults;
import org.spectrim.dicom.DicomNetworkNodes;
import org.spectrim.dicom.DicomTagAndValue;
import org.spectrim.dicom.FileInfo;
import org.spectrim.lib.Convert;
import org.spectrim.lib.Globals;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Point;
import java.awt.color.ColorSpace;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.image.BufferedImage;
import java.awt.image.BufferedImageOp;
import java.awt.image.ColorConvertOp;
import java.awt.image.ImageObserver;
import java.awt.image.PixelGrabber;
import java.awt.image.RenderedImage;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.RandomAccessFile;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.LinkedList;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.imageio.ImageIO;
import javax.swing.BoxLayout;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.SwingUtilities;
import org.spectrim.dicom.DicomDirectory;
import org.spectrim.model.GlobalModel;
import org.spectrim.report.ReportLocalizerPanels;

/**
 *
 * Handles the displaying of metabolite results in the frame,
 * also displaying three screen shots.
 *
 * @author slotboom
 */
public class SVSMetaboliteTextReportFrame extends JFrame
{


    /**
     * Creates a new instance of a SVSMetaboliteTextReportFrame for 
     * displaying spectroscopy results
     * @param parent
     * @param d
     * @param rawSpectrosopyDicomFileNames
     * @param waterReferenceFileName
     * @param numberOfMetabolites
     * @param metaboliteNames
     * @param concentrations
     * @param errorBars
     * @param normValues
     * @param errorsNormValues
     * @param referenceImage0
     * @param referenceImage1
     * @param referenceImage2
     * @param createDicomFile
     * @param seriesNumber
     * @param seriesDescription
     * @param SVSJMRUIDicomTag
     * @param outputStyle
     */
    public SVSMetaboliteTextReportFrame( JFrame parent,
                                         DicomDirectory dicomDir,
                                         Dimension d, 
                                         LinkedList<String> rawSpectrosopyDicomFileNames,
                                         String waterReferenceFileName,
                                         int numberOfMetabolites,
                                         LinkedList<String> metaboliteNames,
                                         double[] concentrations,
                                         double[] errorBars,
                                         double[] normValues,
                                         double[] errorsNormValues,
                                         String referenceImage0,
                                         String referenceImage1,
                                         String referenceImage2,
                                         boolean createDicomFile,
                                         int seriesNumber,
                                         String seriesDescription,
                                         SVS_JMRUI_DICOM_Tag_FAST SVSJMRUIDicomTag,
                                         int outputStyle,
                                         GlobalModel globalModel
                                       )

    {

        m_dicomDirectory = dicomDir;
        m_outputStyle = outputStyle;
        m_SVSJMRUIDicomTag = SVSJMRUIDicomTag;

        m_seriesNumber = seriesNumber;
        m_seriesDescription = seriesDescription;
        m_globalModel = globalModel;
        
        // System.out.println( "SVSMetaboliteTextReportFrame::constructor called!" );

        m_createDicomFile = createDicomFile;

        if ( referenceImage0 != null )
        {
            m_referenceImageReference = referenceImage0;
        }
        else if  ( referenceImage1 != null )
        {
            m_referenceImageReference = referenceImage1;
        }
        else if  ( referenceImage2 != null )
        {
            m_referenceImageReference = referenceImage2;
        }

        m_rawSpectrosopyDicomFileNames = rawSpectrosopyDicomFileNames;

        addWindowListener(
        new WindowAdapter( )
        {
            @Override
            public void windowClosing( WindowEvent event )
            {
               dispose( );
            }
        } );
        
        setLayout( new BoxLayout( getContentPane( ), BoxLayout.Y_AXIS ) );

        
        m_SVSMetaboliteTextReport = new SVSMetaboliteTextReport( d, (String)rawSpectrosopyDicomFileNames.get( 0 ),
                                                                 waterReferenceFileName, numberOfMetabolites,
                                                                 metaboliteNames, concentrations, errorBars,
                                                                 normValues, errorsNormValues, outputStyle,
                                                                 m_globalModel
                                                               );
        add( m_SVSMetaboliteTextReport );

//        m_SVSReferenceImagesPanel = new SVSReferenceImagesPanel( this,
//                                                                 referenceImage0,
//                                                                 referenceImage1,
//                                                                 referenceImage2,
//                                                                 m_dicomDirectory,
//                                                                 rawSpectrosopyDicomFileNames.get( 0 ) );
        
        ReportLocalizerPanels locPanels = new ReportLocalizerPanels( );
        locPanels.initPanels(globalModel);
        
        //add( m_SVSReferenceImagesPanel );
        add( locPanels );
        pack( );

        if ( m_parent != null )
        {
            // Put the Dialog nicely in the middle of its parent
            Point centerParent = new Point( m_parent.getX( ) + ( m_parent.getWidth( ) - getWidth( ) ) / 2,
                                            m_parent.getY( ) + ( m_parent.getHeight( ) - getHeight( ) ) / 2 );

            setLocation( centerParent );
        }

        this.getContentPane( ).setBackground( Color.BLACK );
        
        setVisible( true );
        // m_SVSReferenceImagesPanel.setImageMagnifications( );

        // System.out.println( "END OF CONSTRUCTOR SVSMetaboliteTextReportFrame" );

    }


    /**
     * Does the Dicom conversion and attempts to send it over the network.
     * @param coloured
     * @param compressed
     * @param addJMRUITAGToDICOM
     */
    public void dicomizeAndSent( boolean coloured, boolean compressed, boolean addJMRUITAGToDICOM )
    {

        //String dicom_rename_path = DicomNetworkNodeDefaults.getInstance( ).getProperty( DicomNetworkNodeDefaults.CURRENT_RENAME_TO_DICOM_DIRECTORY );

        if ( coloured == true )
        {
            if ( compressed == true )
            {
                writeAndSentColouredJPEGCompressedDICOMFile( addJMRUITAGToDICOM );
            }
            else
            {
                writeAndSentColouredNonCompressedDICOMFile( addJMRUITAGToDICOM );
            }
        }
        else
        {
            if ( compressed == true )
            {
                //System.out.println( "writeAndSentGrayValueCompressedDicomFile( ) .. " );
                writeAndSentGrayValueJPEGCompressedDicomFile( addJMRUITAGToDICOM );
            }
            else
            {
                //System.out.println( "writeAndSentGrayValueNonCompressedDicomFile( ) .. " );
                writeAndSentGrayValueNonCompressedDicomFile( addJMRUITAGToDICOM );
            }
        }
        // prepareGrayScaleDicomImage( );

        // m_fInfo = writeAsTrueDicom( dicom_rename_path );

        // createDICOMFileOfScreenShot( true );
        //System.out.println( "SVSFitResultGraphicsDisplayFrame-end::dicomizeAndSent called!" );
    }
    
    /**
     * Creates and sent a gray value non-compressed file to disk and 
     * selected node.
     */
    private void writeAndSentGrayValueNonCompressedDicomFile( boolean addJMRUIDICOMTag )
    {
        String dicom_rename_path = DicomNetworkNodeDefaults.getInstance( ).getProperty( DicomNetworkNodeDefaults.CURRENT_RENAME_TO_DICOM_DIRECTORY );
        prepareGrayScaleDicomImage( );
        m_fInfo = writeAsTrueDicom( dicom_rename_path, addJMRUIDICOMTag );
        sentConvertedNonCompressedDicomFileToCurrentlySelectedDicomNode( );
    }

    /**
     * Creates and sent a colored jpeg compressed file to disk and 
     * selected node.
     */
    private void writeAndSentColouredJPEGCompressedDICOMFile( boolean addJMRUIDICOMTag )
    {

        // prepare writing
        String dicom_rename_path = DicomNetworkNodeDefaults.getInstance( ).getProperty( DicomNetworkNodeDefaults.CURRENT_RENAME_TO_DICOM_DIRECTORY );
        prepareGrayScaleDicomImage( );
        m_fInfo = writeAsTrueDicom( dicom_rename_path, addJMRUIDICOMTag );

        createDICOMFileOfScreenShot( true, addJMRUIDICOMTag );
    }

    /**
     * Creates and sent a colored jpeg compressed file to disk and selected 
     * node.
     */
    private void writeAndSentColouredNonCompressedDICOMFile( boolean addJMRUIDicomTag )
    {
        String dicom_rename_path = DicomNetworkNodeDefaults.getInstance( ).getProperty( DicomNetworkNodeDefaults.CURRENT_RENAME_TO_DICOM_DIRECTORY );
        prepareGrayScaleDicomImage( );
        m_fInfo = writeAsTrueDicom( dicom_rename_path, addJMRUIDicomTag );
        sentConvertedNonCompressedDicomFileToCurrentlySelectedDicomNode( );
    }


     /**
     * Creates and sent a gray value jpeg compressed file to disk and 
     * selected node.
     */
    private void writeAndSentGrayValueJPEGCompressedDicomFile(  boolean addJMRUIDICOMTag )
    {

        String dicom_rename_path = DicomNetworkNodeDefaults.getInstance( ).getProperty( DicomNetworkNodeDefaults.CURRENT_RENAME_TO_DICOM_DIRECTORY );
        prepareGrayScaleDicomImage( );
        m_fInfo = writeAsTrueDicom( dicom_rename_path, addJMRUIDICOMTag );

        createDICOMFileOfScreenShot( false, addJMRUIDICOMTag );
    }

    /**
     * Sends the converted DICOM file into the network.
     */
    private void sentConvertedNonCompressedDicomFileToCurrentlySelectedDicomNode( )
    {
        int selectedRemoteDicomNodeNumber = DicomNetworkNodes.getInstance( ).getSelectedRemoteDicomNodeNumber( );

        // screenshot sent (compressed color DICOM screen shot image!
        DicomFileOverNetworkTransferrer dcmTransferrer = new DicomFileOverNetworkTransferrer( selectedRemoteDicomNodeNumber, m_outputLocalDicomFileReference );
        dcmTransferrer.performDicomSendOfLocalDicomFileToSelectedRemoteNode( );
    }

    /**
     * Creates a Dicom image.
     */
    public void prepareGrayScaleDicomImage( )
    {

        m_imageAsGrayscale = getBufferedGrayScaleImageInstance( );

        // set the pixel integervalues
        int w = getContentPane( ).getWidth( );
        int h = getContentPane( ).getHeight( );

        m_pixels = new int[ w * h ];

        PixelGrabber pg = new PixelGrabber( m_imageAsGrayscale, 0, 0, w, h, m_pixels, 0, w );
        try
        {
            //System.out.println( "Grabbing pixels!" );
            pg.grabPixels( );
        }
        catch ( InterruptedException e )
        {
            org.spectrim.lib.Dialogs.messageBox( this, "PixelGrabbing failed in main" );
        }
        if ( ( pg.getStatus( ) & ImageObserver.ABORT ) != 0 )
        {
            org.spectrim.lib.Dialogs.messageBox( this, "PixelGrabbing failed in main" );
        }
    }

    /**
     * 
     * Converts a BufferedImage to GrayScale image.
     * 
     * @param source
     * @return
     */
    public static BufferedImage convertToGrayscale( BufferedImage source )
    {
        BufferedImageOp op = new ColorConvertOp( ColorSpace.getInstance( ColorSpace.CS_GRAY ), null );
        return op.filter( source, null );
    }


    /**
     * Returns the screen as a BufferedImage (grayscale).
     * @return
     */
    public synchronized BufferedImage getBufferedGrayScaleImageInstance( )
    {

        BufferedImage bufferedImage = null;

        //===============================================================

        BufferedImage image = new BufferedImage( getContentPane( ).getWidth( ) ,
                                                 getContentPane( ).getHeight( ),
                                                 BufferedImage.TYPE_3BYTE_BGR );

        Graphics g = image.getGraphics( );
        
        g.translate( 0, 0 );

        getContentPane( ).printAll( g );
        
        if ( g != null )
        {
            g.dispose( );
        }

        try
        {
            String workingDir = System.getProperty( "user.dir" );
            ImageIO.write( image, "png", new File( workingDir + System.getProperty( "file.separator" ) + "test_original.png" ) );
        }
        catch ( IOException e )
        {
            System.out.print( "Problem writing local image file in DicomFormattedScreenShotCreator::createIntermediateImageOfScreenshot :\n" );
            e.printStackTrace( );
        }

        //===============================================================

        m_imageAsGrayscale = convertToGrayscale( image );
         
        try
        {
            String workingDir = System.getProperty( "user.dir" );
            ImageIO.write( (RenderedImage) m_imageAsGrayscale, "png", new File( workingDir + System.getProperty( "file.separator" ) + "test_gray.png" ) );
        }
        catch ( IOException ex )
        {
            org.spectrim.lib.Dialogs.messageBox( this, "PixelGrabbing failed in SVSMetaboliteTextReportFrame" );
        }

        return m_imageAsGrayscale;
    }


    /**
     *
     * Returns the image pixel values as 1D integer array.
     *
     * @return
     */
    private int[] getImageGreyValuePixels( )
    {
        if ( m_imageAsGrayscale != null )
        {
            // set the pixel integervalues
            int w = m_imageAsGrayscale.getWidth( );
            int h = m_imageAsGrayscale.getHeight( );

            m_pixels = new int[ w * h ];
            PixelGrabber pg = new PixelGrabber( m_imageAsGrayscale, 0, 0, w, h, m_pixels, 0, w );
            try
            {
                pg.grabPixels( );
            }
            catch ( InterruptedException e )
            {
                org.spectrim.lib.Dialogs.messageBox( this, "PixelGrabbing failed in main" );
            }
            if ( ( pg.getStatus( ) & ImageObserver.ABORT ) != 0 )
            {
                org.spectrim.lib.Dialogs.messageBox( this, "PixelGrabbing failed in main" );
            }

            int maxPixVal = org.spectrim.signalprocessing.SignalTools.getMaximum( m_pixels );

        }
        
        return m_pixels;
    }


    /**
     * Stores the image as true (Dicom) image.
     * @param outputPath
     * @param addJMRUIDicomTag
     * @return 
     */
    public FileInfo writeAsTrueDicom( String outputPath, boolean addJMRUIDicomTag )
    {

        System.out.println( "SVSMetaboliteTextReportFrame writing DICOM file path : " + outputPath + "  " + m_referenceImageReference );

        DicomImageData imaDat = new DicomImageData( m_referenceImageReference );
        FileInfo fileInfoImage = imaDat.getHeader( );

        fileInfoImage.setImageHeight( getContentPane( ).getHeight( ) );
        fileInfoImage.setImageWidth( getContentPane( ).getWidth( ) );

        DicomImageData spcDat = new DicomImageData( (String)m_rawSpectrosopyDicomFileNames.get( 0 ) );
        FileInfo fileInfoSpect = spcDat.getHeader( );

        m_outputLocalDicomFileReference =
                                          System.getProperty( "file.separator" ) +
                                          fileInfoSpect.getPatientName( )        + "_" +
                                          fileInfoSpect.getPatientBirthDate( )   + "_" +
                                          fileInfoSpect.getSeriesDescription( )  + "_" +
                                          fileInfoSpect.getSeriesNumber( )       + "_" +
                                          fileInfoSpect.getAccessionNumber( )    + ".dcm";

        System.out.println( "DICOM file name out: " + m_outputLocalDicomFileReference );

        m_outputLocalDicomFileReference = eliminateIllegalFileSystemCharacters( m_outputLocalDicomFileReference.trim( ) );

        m_outputLocalDicomFileReference = outputPath + m_outputLocalDicomFileReference;

        //=================================================================================
        // Prepare the DICOM header : change the Image header according the spectrum header
        //=================================================================================

        //fileInfoImage.setAcquisitionTime( fileInfoSpect.getAcquisitionTime( ) );
        //fileInfoImage.setManufacturer( "Neuroradiology Inselspital Bern SCAN" );

        fileInfoImage.setPixelDepth( "0" );
        fileInfoImage.setPixelHeight( "0" );
        fileInfoImage.setPixelWidth( "0" );
        fileInfoImage.setPatientName( fileInfoImage.getPatientName( ) );

        fileInfoImage.setImageOrientationPatient( "" );
        fileInfoImage.setImagePositionPatient( "" );


        //System.out.println( "<<****>> Reference SOP CLASS UID : " + fileInfoImage.getSOPClassUID( ) );

        fileInfoImage.setSOPClassUID( fileInfoImage.getSOPClassUID( ) );

        //fileInfoImage.setMediaStorageSOPClassUID( fileInfoImage.getSOPClassUID( ) );

        String uid_prefix = "1.3.12.2.1107.5.2";

        DicomNetworkNodeDefaults dnwnd = DicomNetworkNodeDefaults.getInstance( );
        String IP_address = dnwnd.getProperty( DicomNetworkNodeDefaults.LOCAL_HOST_IP_ADDRESS );

        long currentTime = System.currentTimeMillis( );
        String imageInstanceUID = uid_prefix + IP_address + currentTime;
        fileInfoImage.setSOPInstanceUID( imageInstanceUID );

        currentTime = System.currentTimeMillis( ) - 2;
        String seriesInstanceUID = uid_prefix + IP_address + currentTime;
        fileInfoImage.setSeriesInstanceUID( seriesInstanceUID );

        currentTime = System.currentTimeMillis( ) - 4;
        String mediaStorageInstanceUID = uid_prefix + IP_address + currentTime;
        fileInfoImage.setMediaStorageSOPInstanceUID( mediaStorageInstanceUID );


        fileInfoImage.setSliceLocation( "" );
        fileInfoImage.setSAR( "" );
        fileInfoImage.setContrastBolusVolume( "" );
        fileInfoImage.setContentTime( fileInfoSpect.getContentTime( ) );

        fileInfoImage.setSeriesDescription( fileInfoSpect.getSeriesDescription( ) + "_jMRUI" );

        String series_number = "" + ( 1000 + (int)(1000 * Math.random( ) ) );
        fileInfoImage.setSeriesNumber( series_number );

        fileInfoImage.setInstanceNumber( "1" );

        Calendar cal = Calendar.getInstance( );
        SimpleDateFormat sdf = new SimpleDateFormat( "yyyyMMdd" );
        String date = sdf.format( cal.getTime( ) );

        SimpleDateFormat sdf2 = new SimpleDateFormat( "Hmmss.SSS" );
        String time = sdf2.format( cal.getTime( ) );

        fileInfoImage.setContentDate( date );
        fileInfoImage.setContentTime(  time );
        
        Globals globs = Globals.getInstance( );

        fileInfoImage.setSoftwareVersion( globs.APP_NAME + globs.APP_VERSION );
        fileInfoImage.setManufacturer( "Support Center for Advanced Neuroimaging (SCAN)" );

        fileInfoImage.setPixelSpacing( "" );

        fileInfoImage.setSliceThickness( "1" );

        fileInfoImage.setSAR( fileInfoSpect.getSAR( ) );

        fileInfoImage.setNumberOfPhaseEncodes( fileInfoSpect.getNumberOfPhaseEncodes( ) );
        fileInfoImage.setPhaseEncodingDirection( fileInfoSpect.getPhaseEncodingDirection( ) );


        LinkedHashMap siemensSpecific = spcDat.getSiemensSpecificDataFromFile( );
        long TE = new Long( (String)siemensSpecific.get( "alTE[0]" ) ).longValue( ) / 1000;
        long TR = new Long( (String)siemensSpecific.get( "alTR[0]" ) ).longValue( ) / 1000;

        fileInfoImage.setTR( "" + TR );
        fileInfoImage.setEchoTime( "" + TE );

        fileInfoImage.setProtocolName( fileInfoSpect.getProtocolName( ) );
        fileInfoImage.setSequenceName( fileInfoSpect.getSequenceName( ) );

        fileInfoImage.setFlipAngle( fileInfoSpect.getFlipAngle( ) );
        fileInfoImage.setVariableFlipAngleFlag( fileInfoSpect.getVariableFlipAngleFlag( ) );

        fileInfoImage.setEchoTrainLength( fileInfoSpect.getEchoTrainLength( ) );
        fileInfoImage.setPercentSampling( fileInfoSpect.getPercentSampling( ) );
        fileInfoImage.setPercentPhaseFOV( fileInfoSpect.getPercentPhaseFOV( ) );
        fileInfoImage.setSamplesPerPixel( fileInfoSpect.getSamplesPerPixel( ) );
        fileInfoImage.setScanningSequence( fileInfoSpect.getScanningSequence( ) );
        fileInfoImage.setSequenceVariant( fileInfoSpect.getSequenceVariant( ) );
        fileInfoImage.setAcquisitionType( fileInfoSpect.getAcquisitionType( ) );
        fileInfoImage.setScanOptions( fileInfoSpect.getScanOptions( ) );

        if ( fileInfoImage.getSliceLocation( ).compareTo( "" ) == 0 )
        {
            fileInfoImage.setSliceLocation( "0.0" );
        }

        if ( fileInfoImage.getImagePositionPatient( ).compareTo( "" ) == 0 )
        {
            fileInfoImage.setImagePositionPatient( "0.0\\0.0\\0.0" );
        }
        if ( fileInfoImage.getSliceThickness( ).compareTo( "" ) == 0 )
        {
            fileInfoImage.setSliceThickness( "0" );
        }
        if ( fileInfoImage.getImageOrientationPatient( ).compareTo( "" ) == 0 )
        {
            fileInfoImage.setImageOrientationPatient( "1\\0\\0\\0\\1\\0" );
        }

        if ( fileInfoImage.getPixelSpacing( ).compareTo( "" ) == 0 )
        {
            fileInfoImage.setPixelSpacing( "1.0\\1.0" );
        }

        if ( m_SVSJMRUIDicomTag != null )
        {
            fileInfoImage.setJMRUIInfo( m_SVSJMRUIDicomTag.getJMRUIDICOMTag( ) );
        }

        try
        {
            RandomAccessFile fout = new RandomAccessFile( m_outputLocalDicomFileReference, "rw" );

            byte[] leadingZeroes = new byte[128];

            fout.write( leadingZeroes );

            String dicm = "DICM".trim( );

            fout.write( dicm.getBytes( ) );

            // dynamically write the DicomFileMetaInfo
            writeDICOMFileMetaInfo( fout );

            writeDICOMWithOrderDICOMTAGS_OF_JMRUI_GRAYSCALE_ScreenShot( fout, fileInfoImage );

            if ( addJMRUIDicomTag == true )
            {
                if ( fileInfoImage.isJMRUICreatedDICOMReportFile( ) == true )
                {
                    fileInfoImage.writeJMRUIDICOMTagsToFile( fout );
                }
            }

            writePixelInformation( fout );

            fout.close( );

            return fileInfoImage;
        }
        catch ( IOException ex )
        {
            org.spectrim.lib.Dialogs.messageBox( this, "Could not write Dicomfile" );
            return null;
        }
    }


    /**
     * Writes the DICOM file meta info part (kind of a header to file).
     */
    private void writeDICOMFileMetaInfo( RandomAccessFile fout )
    {

        int grouplength = 0;

        // META_INFO_VERSION
        grouplength = grouplength + 6 + 8;

        // MEDIA_STORAGE_SOP_CLASS_UID
        String mediaStorageClassUID   = "1.2.840.10008.5.1.4.1.1.4";
        grouplength = grouplength + mediaStorageClassUID.length( ) + 8;

        // MEDIA_STORAGE_SOP_INSTANCE_UID


        // ********** GENERATE A UNIQUE UID  *********************
        String uid_prefix = "1.3.12.2.1107.5.2";
        DicomNetworkNodeDefaults dnwnd = DicomNetworkNodeDefaults.getInstance( );
        String IP_address = dnwnd.getProperty( DicomNetworkNodeDefaults.LOCAL_HOST_IP_ADDRESS );
        long currentTime = System.currentTimeMillis( );
        String mediaStorageSOPInstanceUID = uid_prefix + IP_address + currentTime;
        // ********** GENERATE A UNIQUE UID  *********************

        grouplength = grouplength + mediaStorageSOPInstanceUID.length( ) + 8;

        // TRANSFER_SYNTAX_UID
        //inititial String testStr = "1.2.840.10008.1.2.1";
        //String transferSyntaxUID = "1.2.840.10008.1.2.4.50"; -> COMPRESSED!
        String transferSyntaxUID = "1.2.840.10008.1.2.1";
        grouplength = grouplength + transferSyntaxUID.length( ) + 8;

        // IMPLEMENTATION_CLASS_UID
        //String testStr2 = "1.3.12.2.1107.5.2";

        String implementationClassUID = "1.2.40.0.13.1.1";
        grouplength = grouplength + implementationClassUID.length( ) + 8;

        // IMPLEMENTATION_VERSION_NAME
        String implementationVersionName ="JMRUI_GENERATED";
        grouplength = grouplength + implementationVersionName.length( ) + 8;

//        System.out.println( "XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX" );
//        System.out.println( "Number of bytes of meta part is: " + grouplength );
//        System.out.println( "XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX" );

        byte[] groupLengthNameAndType = new byte[6];

        groupLengthNameAndType[0] = 0x02;
        groupLengthNameAndType[1] = 0x00;
        groupLengthNameAndType[2] = 0x00;
        groupLengthNameAndType[3] = 0x00;
        groupLengthNameAndType[4] = 0x55;
        groupLengthNameAndType[5] = 0x4c;

        byte[] grouplengthBytes = Convert.integer2ByteArray( grouplength );

//        for ( int i = 0; i < grouplengthBytes.length; i++)
//        {
//            System.out.println( "Byte[" + i + "] = " + "0x" + grouplengthBytes[i] );
//        }

        byte[] grouplengthBytesOut = new byte[6];

        grouplengthBytesOut[0] = 0x04;
        grouplengthBytesOut[1] = 0x00;
        grouplengthBytesOut[2] = grouplengthBytes[3];
        grouplengthBytesOut[3] = 0x00;
        grouplengthBytesOut[4] = 0x00;
        grouplengthBytesOut[5] = 0x00;


        try
        {
            fout.write( groupLengthNameAndType );
            fout.write( grouplengthBytesOut );
        }
        catch ( IOException ex )
        {
            Logger.getLogger( SVSMetaboliteTextReportFrame.class.getName( ) ).log( Level.SEVERE, null, ex );
        }

        LinkedHashMap<String, DicomTagAndValue> dicomFileMetaInfo2 = new LinkedHashMap<>( );

        // write hard coded META_INFO_VERSION tag!
        byte[] value = new byte[14];

        value[0] = 0x02;
        value[1] = 0x00;
        value[2] = 0x01;
        value[3] = 0x00;
        value[4] = 0x4f;
        value[5] = 0x42;
        value[6] = 0x00;
        value[7] = 0x00;
        value[8] = 0x02;
        value[9] = 0x00;
        value[10] = 0x00;
        value[11] = 0x00;
        value[12] = 0x00;
        value[13] = 0x01;

        try
        {
            fout.write( value );
        }
        catch ( IOException ex )
        {
            Logger.getLogger( SVSMetaboliteTextReportFrame.class.getName( ) ).log( Level.SEVERE, null, ex );
        }

        String tagName    = "MEDIA_STORAGE_SOP_CLASS_UID";
        int tagID         = DicomDecoder.MEDIA_STORAGE_SOP_CLASS_UID;
        int encodingID    = DicomDecoder.UI;
        String tagValue   = mediaStorageClassUID;

        DicomTagAndValue tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        dicomFileMetaInfo2.put( tagName, tag );

        tagName    = "MEDIA_STORAGE_SOP_INSTANCE_UID";
        tagID      = DicomDecoder.MEDIA_STORAGE_SOP_INSTANCE_UID;
        encodingID = DicomDecoder.UI;
        tagValue   = mediaStorageSOPInstanceUID;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        dicomFileMetaInfo2.put( tagName, tag );

        tagName    = "TRANSFER_SYNTAX_UID";
        tagID      = DicomDecoder.TRANSFER_SYNTAX_UID;
        encodingID = DicomDecoder.UI;
        tagValue   = transferSyntaxUID;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        dicomFileMetaInfo2.put( tagName, tag );

        tagName    = "IMPLEMENTATION_CLASS_UID";
        tagID      = DicomDecoder.IMPLEMENTATION_CLASS_UID;
        encodingID = DicomDecoder.UI;
        // original tagValue   = "1.3.12.2.1107.5.2";
        tagValue   = implementationClassUID;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        dicomFileMetaInfo2.put( tagName, tag );

        tagName    = "IMPLEMENTATION_VERSION_NAME";
        tagID      = DicomDecoder.IMPLEMENTATION_VERSION_NAME;
        encodingID = DicomDecoder.SH;
        tagValue   = implementationVersionName;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        dicomFileMetaInfo2.put( tagName, tag );

        try
        {
            writeAllOrderedDICOMTags( fout, dicomFileMetaInfo2 );
        }
        catch ( IOException ex )
        {
            Logger.getLogger(SVSMetaboliteTextReportFrame.class.getName()).log(Level.SEVERE, null, ex);
        }

    }


    /**
     *
     * Based on a DICOM file which gives all missing attributes for the association
     * a DICOM file is written.
     * @param fout
     * @param dicomAssociation
     */
    private void writeDICOMWithOrderDICOMTAGS_OF_JMRUI_GRAYSCALE_ScreenShot( RandomAccessFile fout, FileInfo dicomAssociation )
    {

        LinkedHashMap<String, DicomTagAndValue> allTags = new LinkedHashMap<>( );

        // 1. public static final int                SOP_CLASS_UID = 0x00080016;
        String tagName    = "SOP_CLASS_UID";
        int tagID      = DicomDecoder.SOP_CLASS_UID;
        int encodingID = DicomDecoder.UI;
        String tagValue;

        if ( dicomAssociation.getSOPClassUID( ) != null )
        {
            tagValue   = dicomAssociation.getSOPClassUID( );
        }
        else
        {
            tagValue = "1.2.840.10008.5.1.4.1.1.4";
        }

        DicomTagAndValue tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );

        allTags.put( tagName, tag );

        //2. public static final int              SOP_INSTANCE_UID = 0x00080018;

        // generate a new unique uid!
        String uid_prefix = "1.3.12.2.1107.5.2";

        DicomNetworkNodeDefaults dnwnd = DicomNetworkNodeDefaults.getInstance( );
        String IP_address = dnwnd.getProperty( DicomNetworkNodeDefaults.LOCAL_HOST_IP_ADDRESS );
        long currentTime = System.currentTimeMillis( );
        String sopInstanceUID = uid_prefix + IP_address + currentTime;

        sopInstanceUID = "1.3.12.2.1107.5.2.13.20608.30000006041006120356200001710";


        tagName    = "SOP_INSTANCE_UID";
        tagID      = DicomDecoder.SOP_INSTANCE_UID;
        encodingID = DicomDecoder.UI;
        tagValue   = sopInstanceUID;

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        allTags.put( tagName, tag );

        // 3. public static final int                   STUDY_DATE = 0x00080020;
        tagName    = "STUDY_DATE";
        tagID      = DicomDecoder.STUDY_DATE;
        encodingID = DicomDecoder.DA;
        tagValue   = dicomAssociation.getStudyDate( );

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        allTags.put( tagName, tag );

        // 4. public static final int                  SERIES_DATE = 0x00080021;
        tagName    = "SERIES_DATE";
        tagID      = DicomDecoder.SERIES_DATE;
        encodingID = DicomDecoder.DA;
        tagValue   = dicomAssociation.getSeriesDate( );

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        allTags.put( tagName, tag );

        // 5. public static final int             ACQUISITION_DATE = 0x00080022;
        tagName    = "ACQUISITION_DATE";
        tagID      = DicomDecoder.ACQUISITION_DATE;
        encodingID = DicomDecoder.DA;
        tagValue   = dicomAssociation.getAcquisitionDate( );

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        allTags.put( tagName, tag );

        // 6. public static final int               CONTENT_DATE   = 0x00080023;
        tagName    = "CONTENT_DATE";
        tagID      = DicomDecoder.CONTENT_DATE;
        encodingID = DicomDecoder.DA;
        tagValue   = dicomAssociation.getContentDate( );

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        allTags.put( tagName, tag );

        // 7. public static final int                 STUDY_TIME   = 0x00080030;
        tagName    = "STUDY_TIME";
        tagID      = DicomDecoder.STUDY_TIME;
        encodingID = DicomDecoder.TM;
        tagValue   = dicomAssociation.getStudyTime( );

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        allTags.put( tagName, tag );

        // 8. public static final int                 SERIES_TIME  = 0x00080031;
        tagName    = "SERIES_TIME";
        tagID      = DicomDecoder.SERIES_TIME;
        encodingID = DicomDecoder.TM;
        tagValue   = dicomAssociation.getSeriesTime( );

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        allTags.put( tagName, tag );

        // 9. public static final int            ACQUISITION_TIME  = 0x00080032;
        tagName    = "ACQUISITION_TIME";
        tagID      = DicomDecoder.ACQUISITION_TIME;
        encodingID = DicomDecoder.TM;
        tagValue   = dicomAssociation.getAcquisitionTime( );

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        allTags.put( tagName, tag );

        // 10. public static final int                CONTENT_TIME = 0x00080033;
        tagName    = "CONTENT_TIME";
        tagID      = DicomDecoder.CONTENT_TIME;
        encodingID = DicomDecoder.TM;
        tagValue   = dicomAssociation.getContentTime( );

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        allTags.put( tagName, tag );

        // 11. public static final int           ACCESSION_NUMBER  = 0x00080050;
        tagName    = "ACCESSION_NUMBER";
        tagID      = DicomDecoder.ACCESSION_NUMBER;
        encodingID = DicomDecoder.SH;
        tagValue   = dicomAssociation.getAccessionNumber( );

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        allTags.put( tagName, tag );

         // 12. public static final int             IMAGE_MODALITY = 0x00080060;
        tagName    = "IMAGE_MODALITY";
        tagID      = DicomDecoder.IMAGE_MODALITY;
        encodingID = DicomDecoder.CS;
        tagValue   = dicomAssociation.getImageModality( );

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        allTags.put( tagName, tag );


        // 13. public static final int                MANUFACTURER = 0x00080070;
        tagName    = "MANUFACTURER";
        tagID      = DicomDecoder.MANUFACTURER;
        encodingID = DicomDecoder.LO;
        tagValue   = dicomAssociation.getManufacturer( );

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        allTags.put( tagName, tag );

        // 14. public static final int           STUDY_DESCRIPTION = 0x00081030;
        tagName    = "STUDY_DESCRIPTION";
        tagID      = DicomDecoder.STUDY_DESCRIPTION;
        encodingID = DicomDecoder.LO;
        tagValue   = dicomAssociation.getStudyDescription( );

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        allTags.put( tagName, tag );

        // 15. public static final int          SERIES_DESCRIPTION = 0x0008103e;
        tagName    = "SERIES_DESCRIPTION";
        tagID      = DicomDecoder.SERIES_DESCRIPTION;
        encodingID = DicomDecoder.LO;
        tagValue   = dicomAssociation.getSeriesDescription( );

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        allTags.put( tagName, tag );

        // 16. public static final int               SCANNER_MODEL = 0x00081090;
        tagName    = "SCANNER_MODEL";
        tagID      = DicomDecoder.SCANNER_MODEL;
        encodingID = DicomDecoder.CS;
        tagValue = dicomAssociation.getScannerModelName( );

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        allTags.put( tagName, tag );

        // Patient related
        // 17. public static final int      PATIENT_NAME           = 0x00100010;
        tagName  = "PATIENT_NAME";

        tagID       = DicomDecoder.PATIENT_NAME;
        encodingID  = DicomDecoder.PN;
        tagValue = dicomAssociation.getPatientName( );
        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        
        allTags.put( tagName, tag );
        
        // 18. public static final int               PATIENT_ID    = 0x00100020;
        tagName    = "PATIENT_ID";
        tagID      = DicomDecoder.PATIENT_ID;
        encodingID = DicomDecoder.LO;
        tagValue   = dicomAssociation.getPatientID( );

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        allTags.put( tagName, tag );

        // 19. public static final int          PATIENT_BIRTHDATE  = 0x00100030;

        tagName    = "PATIENT_BIRTHDATE";
        tagID      = DicomDecoder.PATIENT_BIRTHDATE;
        encodingID = DicomDecoder.DA;
        tagValue   = dicomAssociation.getPatientBirthDate( );

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        allTags.put( tagName, tag );

        // 20. public static final int            PATIENT_SEX      = 0x00100040;
        tagName    = "PATIENT_SEX";
        tagID      = DicomDecoder.PATIENT_SEX;
        encodingID = DicomDecoder.CS;
        tagValue   = dicomAssociation.getPatientSex( );

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        allTags.put( tagName, tag );

        // 21. public static final int              PATIENT_AGE    = 0x00101010;
         tagName    = "PATIENT_AGE";
        tagID      = DicomDecoder.PATIENT_AGE;
        encodingID = DicomDecoder.AS;
        tagValue   = dicomAssociation.getPatientAge( );

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        allTags.put( tagName, tag );


        // 22. public static final int          PATIENT_WEIGHT     = 0x00101030;
        tagName    = "PATIENT_WEIGHT";
        tagID      = DicomDecoder.PATIENT_WEIGHT;
        encodingID = DicomDecoder.DS;
        tagValue   = dicomAssociation.getPatientWeight( );

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        allTags.put( tagName, tag );

        // 23. public static final int           SLICE_THICKNESS   = 0x00180050;
        tagName    = "SLICE_THICKNESS";
        tagID      = DicomDecoder.SLICE_THICKNESS;
        encodingID = DicomDecoder.DS;
        tagValue   = dicomAssociation.getSliceThickness( );

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        allTags.put( tagName, tag );

        // 24. public static final int            TR               = 0x00180080;
        tagName    = "TR";
        tagID      = DicomDecoder.TR;
        encodingID = DicomDecoder.DS;
        tagValue   = dicomAssociation.getTR( );

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        allTags.put( tagName, tag );

        // 25. public static final int         ECHO_TIME           = 0x00180081;
        tagName    = "ECHO_TIME";
        tagID      = DicomDecoder.ECHO_TIME;
        encodingID = DicomDecoder.DS;
        tagValue   = dicomAssociation.getEchoTime( );

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        allTags.put( tagName, tag );

        // 26. public static final int          IMAGE_FREQUENCY    = 0x00180084;

        tagName    = "IMAGE_FREQUENCY";
        tagID      = DicomDecoder.IMAGE_FREQUENCY;
        encodingID = DicomDecoder.DS;
        tagValue   = dicomAssociation.getImagingFrequency( );

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        allTags.put( tagName, tag );

        // 27. public static final int         SEQUENCE_NAME       = 0x00180024;
        tagName    = "SEQUENCE_NAME";
        tagID      = DicomDecoder.SEQUENCE_NAME;
        encodingID = DicomDecoder.SH;
        tagValue   = dicomAssociation.getSequenceName( );

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        allTags.put( tagName, tag );


        // 28. public static final int          SOFTWARE_VERSION   = 0x00181020;
        tagName    = "SOFTWARE_VERSION";
        tagID      = DicomDecoder.SOFTWARE_VERSION;
        encodingID = DicomDecoder.LO;
        tagValue   = dicomAssociation.getSoftwareVersion( );

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        allTags.put( tagName, tag );

        // 29. public static final int          PROTOCOL_NAME      = 0x00181030;
        tagName    = "PROTOCOL_NAME";
        tagID      = DicomDecoder.PROTOCOL_NAME;
        encodingID = DicomDecoder.LO;
        tagValue   = dicomAssociation.getProtocolName( );

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        allTags.put( tagName, tag );

        // 30. public static final int       TRANSMIT_COIL_NAME    = 0x00181251;
        
        tagName    = "TRANSMIT_COIL_NAME";
        tagID      = DicomDecoder.TRANSMIT_COIL_NAME;
        encodingID = DicomDecoder.DS;
        tagValue   = dicomAssociation.getTransmitCoilName( );

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        allTags.put( tagName, tag );


        // 31. public static final int            SAR              = 0x00181316;
        tagName    = "SAR";
        tagID      = DicomDecoder.SAR;
        encodingID = DicomDecoder.DS;
        tagValue   = dicomAssociation.getSAR( );

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        allTags.put( tagName, tag );


        // 32. public static final int PATIENT_POSITION            = 0x00185100;
        tagName    = "PATIENT_POSITION";
        tagID      = DicomDecoder.PATIENT_POSITION;
        encodingID = DicomDecoder.CS;
        tagValue   = dicomAssociation.getPatientPosition( );

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        allTags.put( tagName, tag );

        // 34. public static final  int STUDY_INSTANCE_UID         = 0x0020000D;

        tagName    = "STUDY_INSTANCE_UID";
        tagID      = DicomDecoder.STUDY_INSTANCE_UID;
        encodingID = DicomDecoder.UI;
        tagValue   = dicomAssociation.getStudyInstanceUID( );

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        allTags.put( tagName, tag );

        // 33. public static final  int  SERIES_INSTANCE_UID       = 0x0020000E;
        tagName    = "SERIES_INSTANCE_UID";
        tagID      = DicomDecoder.SERIES_INSTANCE_UID;
        encodingID = DicomDecoder.UI;
        tagValue   = dicomAssociation.getSeriesInstanceUID( );

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        allTags.put( tagName, tag );
        
        // 35. public static final int             STUDY_ID        = 0x00200010;
        tagName    = "STUDY_ID";
        tagID      = DicomDecoder.STUDY_ID;
        encodingID = DicomDecoder.SH;
        tagValue   = dicomAssociation.getStudyID( );

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        allTags.put( tagName, tag );


        // 36. public static final int          SERIES_NUMBER      = 0x00200011;
        tagName    = "SERIES_NUMBER";
        tagID      = DicomDecoder.SERIES_NUMBER;
        encodingID = DicomDecoder.IS;
        tagValue   = dicomAssociation.getSeriesNumber( );

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        allTags.put( tagName, tag );

        // 37. public static final int        ACQUISITION_NUMBER  =  0x00200012;
        tagName    = "ACQUISITION_NUMBER";
        tagID      = DicomDecoder.ACQUISITION_NUMBER;
        encodingID = DicomDecoder.IS;
        tagValue   = dicomAssociation.getAcquisitionNumber( );

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        allTags.put( tagName, tag );

        // 38. public static final int           INSTANCE_NUMBER   = 0x00200013;
        tagName    = "INSTANCE_NUMBER";
        tagID      = DicomDecoder.INSTANCE_NUMBER;
        encodingID = DicomDecoder.IS;
        tagValue   = dicomAssociation.getInstanceNumber( );

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        allTags.put( tagName, tag );

        // 39. public static final int          SLICE_LOCATION     = 0x00201041;
        tagName    = "SLICE_LOCATION";
        tagID      = DicomDecoder.SLICE_LOCATION;
        encodingID = DicomDecoder.DS;
        tagValue   = dicomAssociation.getSliceLocation( );

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        allTags.put( tagName, tag );


        // 40. public static final  int       SAMPLES_PER_PIXEL    = 0x00280002;
        tagName    = "SAMPLES_PER_PIXEL";
        tagID      = DicomDecoder.SAMPLES_PER_PIXEL;
        encodingID = DicomDecoder.US;
        tagValue   = dicomAssociation.getSamplesPerPixel( );

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        allTags.put( tagName, tag );


        // 41. public static final  int PHOTOMETRIC_INTERPRETATION = 0x00280004;
        tagName    = "PHOTOMETRIC_INTERPRETATION";
        tagID      = DicomDecoder.PHOTOMETRIC_INTERPRETATION;
        encodingID = DicomDecoder.CS;
        tagValue   = dicomAssociation.getPhotometricInterpretation( );

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        allTags.put( tagName, tag );


        //  42. public static final  int             ROWS          = 0x00280010;
        tagName    = "ROWS";
        tagID      = DicomDecoder.ROWS;
        encodingID = DicomDecoder.US;
        tagValue   = "" + dicomAssociation.getImageHeight( );

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        allTags.put( tagName, tag );

        // 43. public static final  int           COLUMNS          = 0x00280011;
        tagName    = "COLUMNS";
        tagID      = DicomDecoder.COLUMNS;
        encodingID = DicomDecoder.US;
        tagValue   = "" + dicomAssociation.getImageWidth( );

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        allTags.put( tagName, tag );


        // 44. public static final  int PIXEL_SPACING              = 0x00280030;
        tagName    = "PIXEL_SPACING";
        tagID      = DicomDecoder.PIXEL_SPACING;
        encodingID = DicomDecoder.DS;
        tagValue   = dicomAssociation.getPixelSpacing( );

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        allTags.put( tagName, tag );

        // 45. public static final  int BITS_ALLOCATED             = 0x00280100;
        tagName    = "BITS_ALLOCATED";
        tagID      = DicomDecoder.BITS_ALLOCATED;
        encodingID = DicomDecoder.US;
        tagValue   = dicomAssociation.getBitsAllocated( );

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        allTags.put( tagName, tag );


        // 46. public static final  int         BITS_STORED        = 0x00280101;
        tagName    = "BITS_STORED";
        tagID      = DicomDecoder.BITS_STORED;
        encodingID = DicomDecoder.US;
        tagValue   = dicomAssociation.getBitsStored( );

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        allTags.put( tagName, tag );

        // 47.  public static final  int           HIGH_BIT        = 0x00280102;
        tagName    = "HIGH_BIT";
        tagID      = DicomDecoder.HIGH_BIT;
        encodingID = DicomDecoder.US;
        tagValue   = dicomAssociation.getHighBit( );

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        allTags.put( tagName, tag );


        // 48. public static final  int       PIXEL_REPRESENTATION = 0x00280103;

        tagName    = "PIXEL_REPRESENTATION";
        tagID      = DicomDecoder.PIXEL_REPRESENTATION;
        encodingID = DicomDecoder.US;
        tagValue   = dicomAssociation.getPixelRepresentation( );

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        allTags.put( tagName, tag );


        // 49.  public static final  int SMALLEST_PIXEL_VALUE      = 0x00280106;
        tagName    = "SMALLEST_PIXEL_VALUE";
        tagID      = DicomDecoder.SMALLEST_PIXEL_VALUE;
        encodingID = DicomDecoder.SS;
        tagValue   = dicomAssociation.getSmallestPixelValue( );

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        allTags.put( tagName, tag );

        // 50. public static final  int   LARGEST_PIXEL_VALUE      = 0x00280107;
        tagName    = "LARGEST_PIXEL_VALUE";
        tagID      = DicomDecoder.LARGEST_PIXEL_VALUE;
        encodingID = DicomDecoder.SS;
        tagValue   = dicomAssociation.getLargestPixelValue( );

        tag = new DicomTagAndValue( tagName, tagID, encodingID, tagValue );
        allTags.put( tagName, tag );

        try
        {
            writeAllOrderedDICOMTags( fout, allTags );
        }
        catch ( IOException ex )
        {
            Logger.getLogger( SVSMetaboliteTextReportFrame.class.getName( ) ).log( Level.SEVERE, null, ex );
        }

    }



    /**
     * Writes all patient related tags to a random access output file "fout".
     * @param fout
     * @param allTagsLinkedHashMap
     * @throws java.io.IOException
     */
    public void writeAllOrderedDICOMTags( RandomAccessFile fout, 
                                          LinkedHashMap<String, DicomTagAndValue> allTagsLinkedHashMap ) 
                                          throws IOException
    {

        Iterator it = allTagsLinkedHashMap.keySet( ).iterator( );
        while ( it.hasNext( ) )
        {

            String tagName = (String)it.next( );
            DicomTagAndValue tag = allTagsLinkedHashMap.get( tagName );

//            System.out.println( "Study/Series/Instance related tagName           = " + tagName );
//            System.out.println( "Study/Series/Instance related tagID             = " + tag.getTagEcodingID( ) );
//            System.out.println( "Study/Series/Instance related tag description   = " + tag.getTagDescription( ) );
//            System.out.println( "Study/Series/Instance related tag value         = " + tag.getTagValueAsString( ) );

            if ( ( tag.getTagValueAsString( ) != null ) &&
                 ( tag.getTagValueAsString( ).compareTo( "N.A." ) != 0 ) )
            {
                switch ( tag.getTagEcodingID( ) )
                {
                    case DicomDecoder.AE:
                    case DicomDecoder.AS:
                    case DicomDecoder.AT:
                    case DicomDecoder.CS:
                    case DicomDecoder.DA:
                    case DicomDecoder.DS:
                    case DicomDecoder.DT:
                    case DicomDecoder.IS:
                    case DicomDecoder.LO:
                    case DicomDecoder.LT:
                    case DicomDecoder.PN:
                    case DicomDecoder.SH:
                    case DicomDecoder.ST:
                    case DicomDecoder.TM:
                    case DicomDecoder.UI:
                                            // here a test must be made whether the cc of 0xaabbccdd equals zero
                                            // if not, the rotateLeft must be different -> is to be found out qqqqqqqqqq

                                            byte[] bytes1 = Convert.integer2ByteArray( tag.getTagID( ) );


                                            if ( bytes1[2] == 0 )
                                            {
                                                int outp3 = Integer.rotateLeft( tag.getTagID( ), 8 );
                                                fout.writeInt( outp3 );
                                            }
                                            else
                                            {
                                                fout.write( bytes1[1] );
                                                fout.write( bytes1[0] );
                                                fout.write( bytes1[3] );
                                                fout.write( bytes1[2] );
                                            }

                                            fout.writeShort( tag.getTagEcodingID( ) );

                                            fout.writeShort( Short.reverseBytes( (short)tag.getTagValueAsString( ).getBytes( ).length ) );

                                            byte[] bytes = tag.getTagValueAsString( ).getBytes( );

                                            fout.write( bytes );
                                            break;

                    case DicomDecoder.US:
                    case DicomDecoder.SS:
                    case DicomDecoder.UL:

                                            byte[] bytes2 = Convert.integer2ByteArray( tag.getTagID( ) );

                                            if ( bytes2[2] == 0 )
                                            {
                                                int outp3 = Integer.rotateLeft( tag.getTagID( ), 8 );
                                                fout.writeInt( outp3 );
                                            }
                                            else
                                            {
                                                // System.out.println( ">> Writing bytes seperately!" );
                                                fout.write( bytes2[1] );
                                                fout.write( bytes2[0] );
                                                fout.write( bytes2[3] );
                                                fout.write( bytes2[2] );
                                            }

                                            fout.writeShort( tag.getTagEcodingID( ) );  // 2 bytes

                                            byte[] sab1= new byte[2];

                                            int valAsInt1 = 2;

                                            Convert.int2wordl( sab1, 0, valAsInt1 );

                                            fout.write( sab1 );     // 2 bytes

                                            byte[] sab2= new byte[2];

                                            int valAsInt2 = new Integer( tag.getTagValueAsString( ).trim( ) );

                                            Convert.int2wordl( sab2, 0, valAsInt2 );

                                            fout.write( sab2 );     // 2 bytes

                                            break;
                    default:
                                            System.out.println( "Non handled Tag Encoding ID ! >> " );
                                            System.out.println( "Description : " + tagName );
                                            System.out.println( "Tag ID      : " + tag.getTagEcodingID( ) );
                                            System.out.println( "Tag value   :" +  tag.getTagValueAsString( ) );
                }
            }
        }
    }


    /**
     * Depending on the type of the pixel information.
     * @param fout
     */
    private void writePixelInformation( RandomAccessFile fout ) throws IOException
    {

        fout.writeInt( Integer.rotateLeft( Integer.reverseBytes( DicomDecoder.PIXEL_DATA ), 16 ) );

        fout.writeShort( DicomDecoder.OW );

        byte[] pixels2write = new byte[ 2 * m_imageAsGrayscale.getWidth( ) * m_imageAsGrayscale.getHeight( ) ];

        for ( int i = 0; i < m_imageAsGrayscale.getWidth( ) * m_imageAsGrayscale.getHeight( ); i++ )
        {
            org.spectrim.lib.Convert.int2wordl( pixels2write, 2 * i, m_pixels[ i ] );
        }

        byte[] byteArr = new byte[2];

        fout.write( byteArr );

        fout.writeInt( Integer.reverseBytes( pixels2write.length ) );

        fout.write( pixels2write );
    }


    /**
     * Replaces all blanks by "\  " in order to be interpreted
     * correctly by the Windows operating system.
     * @param cmd
     * @return
     */
    private String eliminateIllegalFileSystemCharacters( String cmd )
    {

        // first get rid ofm blancs
        cmd = cmd.trim( );

        cmd = cmd.replace( ' ', '_' );
        cmd = cmd.replace( ',', '_' );
        cmd = cmd.replace( ';', '_' );
        cmd = cmd.replace( '%', '_' );
        //cmd.replace( '*', '_' );

        cmd = cmd.replace( 'é', 'e' ); // problematic on some linux systems
        cmd = cmd.replace( 'è', 'e' ); // problematic on some linux systems
        cmd = cmd.replace( 'ö', 'o' ); // problematic on some linux systems
        cmd = cmd.replace( 'ü', 'u' ); // problematic on some linux systems
        cmd = cmd.replace( 'à', 'a' ); // problematic on some linux systems
        cmd = cmd.replace( 'ä', 'a' ); // problematic on some linux systems
        cmd = cmd.replace( 'Ö', 'O' ); // problematic on some linux systems
        cmd = cmd.replace( 'Ü', 'U' ); // problematic on some linux systems

        cmd = cmd.replace( '@', '_' ); // problematic on some linux systems
        cmd = cmd.replace( '#', '_' ); // problematic on some linux systems

        cmd = cmd.replace( '?', '_' );  // problematic on some linux systems

        cmd = cmd.replace( '¢', 'c' ); // problematic on some linux systems
        cmd = cmd.replace( '!', '_' );  // problematic on some linux systems
        cmd = cmd.replace( '§', '_' );  // problematic on some linux systems
        cmd = cmd.replace( '¦', '_' );  // problematic on some linux systems

        cmd = cmd.replace( '$', '_' );  // problematic on some linux systems
        cmd = cmd.replace( '[', '_' );  // problematic on some linux systems
        cmd = cmd.replace( ']', '_' );  // problematic on some linux systems

        cmd = cmd.replace( '^', '_' );  // problematic on some linux systems
        cmd = cmd.replace( '~', '_' );  // problematic on some linux systems

        cmd = cmd.replace( 'ç', 'c' );  // problematic on some linux systems
        cmd = cmd.replace( '~', '_' );  // problematic on some linux systems
        cmd = cmd.replace( '"', '_' );  // problematic on some linux systems
        cmd = cmd.replace( "<", "_" );  // problematic on some linux systems
        cmd = cmd.replace( ">", "_" );  // problematic on some linux systems
        cmd = cmd.replace("____", "_" );
        cmd = cmd.replace("___", "_" );
        cmd = cmd.replace("__", "_" );

        return cmd;
    }


    //////////////////////////////////////////////////////////////////////////////////////


    /**
     * 
     * Creates of the current screen a DICOM file.
     * @param coloured
     * @param addJMRUIDICOMTag 
     */
    private void createDICOMFileOfScreenShot( boolean coloured, boolean addJMRUIDICOMTag )
    {
          createIntermediateImageOfScreenshot( coloured );

          performJPGTODicomConversion( addJMRUIDICOMTag );

          sentConvertedJPegCompressedDicomFileToCurrentlySelectedDicomNode( );
    }

    /**
     * Sends the converted DICOM file to a network node.
     */
    private void sentConvertedJPegCompressedDicomFileToCurrentlySelectedDicomNode( )
    {
        int selectedRemoteDicomNodeNumber = DicomNetworkNodes.getInstance( ).getSelectedRemoteDicomNodeNumber( );

        // screenshot sent (compressed color DICOM screen shot image!
        DicomFileOverNetworkTransferrer dcmTransferrer = new DicomFileOverNetworkTransferrer( selectedRemoteDicomNodeNumber, m_dicomOutputFileName.replaceAll( ".dcm", ".out.dcm" ) );
        dcmTransferrer.performDicomSendOfLocalDicomFileToSelectedRemoteNode( );

    }

    /**
     * Sends the converted DICOM file to the selected network node.
     */
    private void sentConvertedDicomFileToCurrentlySelectedDicomNode( )
    {
        int selectedRemoteDicomNodeNumber = DicomNetworkNodes.getInstance( ).getSelectedRemoteDicomNodeNumber( );

        // screenshot sent
        DicomFileOverNetworkTransferrer dcmTransferrer = new DicomFileOverNetworkTransferrer( selectedRemoteDicomNodeNumber, m_dicomOutputFileName );
        dcmTransferrer.performDicomSendOfLocalDicomFileToSelectedRemoteNode( );

        System.out.println( "Also sending gray value DICOM .. dicom file name  !! : " + m_outputLocalDicomFileReference );
        
        // true dicom sent
        DicomFileOverNetworkTransferrer dcmTransferrer2 = new DicomFileOverNetworkTransferrer( selectedRemoteDicomNodeNumber, m_outputLocalDicomFileReference );
        dcmTransferrer2.performDicomSendOfLocalDicomFileToSelectedRemoteNode( );

    }

    /**
     * Performs the jpg to Dicom conversion.
     * @param insertJMRUITAG 
     */
     private void performJPGTODicomConversion( boolean insertJMRUITAG )
     {
        String uid_prefix = "1.3.12.2.1107.5.2";

        DicomNetworkNodeDefaults dnwnd = DicomNetworkNodeDefaults.getInstance( );
        String IP_address = dnwnd.getProperty( DicomNetworkNodeDefaults.LOCAL_HOST_IP_ADDRESS );
        long currentTime = System.currentTimeMillis( );

        m_seriesInstanceUID = uid_prefix + IP_address + currentTime;
        m_instanceUID =   uid_prefix + IP_address + currentTime + 2;

        createJpg2DcmConfigFile( );
        doJPGTODicomConversion( );
        if ( insertJMRUITAG == true )
        {
            try
            {
                insertJMRUIDICOMTag( );

            } catch ( IOException ex )
            {
                Logger.getLogger( SVSFitResultGraphicsDisplayFrame.class.getName( ) ).log(Level.SEVERE, null, ex );
            }
        }
        else
        {
            // do the renaming to .out.dcm
            File file = new File( m_dicomOutputFileName );
            file.renameTo( new File( m_dicomOutputFileName.replaceAll( ".dcm", ".out.dcm" ) ) );
        }
     }


     /**
      * Does the insertion of the jMRUI-DICOM tag into the DICOM file.
      */
     private void insertJMRUIDICOMTag( ) throws IOException
     {
        byte[] hdr = readHeaderDICOM( m_dicomOutputFileName );

        byte[] hdrToWrite = new byte[hdr.length - 12];
        byte[] pixelDataTagInfoAsBytes = new byte[ 12 ];

        System.arraycopy( hdr, 0, hdrToWrite, 0, hdr.length - 12 );
        System.arraycopy( hdr, hdr.length - 12, pixelDataTagInfoAsBytes, 0, 12 );

        // try to write hdrToWrite and add the dicom types etc (these missing bytes before the data part!)

        try
        {
            RandomAccessFile fout = new RandomAccessFile( m_dicomOutputFileName.replaceAll( ".dcm", ".out.dcm" ), "rw" );
            fout.write( hdrToWrite );

            // write the tag to insert ..
            byte[] jmruiInfoAsBytes  = m_SVSJMRUIDicomTag.getJMRUIDICOMTag( ).getBytes( );

            fout.writeInt( Integer.rotateLeft( Integer.reverseBytes( DicomDecoder.JMRUI_INFO_TAG ), 16 ) );
            fout.writeShort( DicomDecoder.OB );
            byte[] byteArr = new byte[2];
            fout.write( byteArr );
            fout.writeInt( Integer.reverseBytes( jmruiInfoAsBytes.length ) );
            fout.write( jmruiInfoAsBytes );

            // write the data pixel information
            fout.write( pixelDataTagInfoAsBytes );
            fout.write( m_pixelDataAsBytes );

            fout.close( );

        }
        catch ( FileNotFoundException ex )
        {
            Logger.getLogger( SVSFitResultGraphicsDisplayFrame.class.getName( ) ).log( Level.SEVERE, null, ex );
        }
     }


     /**
      * Reads the complete Dicom header belonging to 'fileName'.
      * @param fileName
      * @return 
      */
    private byte[] readHeaderDICOM( String fileName )
    {
        byte[] hdr;

        try
        {
            DicomImageData ima = new DicomImageData( m_dicomOutputFileName );

            System.out.println( "Reading file : " + m_dicomOutputFileName );

            File fl = new File( m_dicomOutputFileName );

            int fSize = (int)fl.length( );
            int hdrSize = ima.getHeader( ).getDataOffsetPointer( );
            int bDataSize = fSize - hdrSize;


            m_pixelDataAsBytes = new byte[bDataSize];
            m_hdr = new byte[hdrSize];
            RandomAccessFile fin = new RandomAccessFile( m_dicomOutputFileName, "r" );

            fin.read( m_hdr );
            fin.read( m_pixelDataAsBytes );

            fin.close();

            System.out.println( "Read header!!" );
            System.out.println( "Filesize   : " + fSize );
            System.out.println( "Headersize : " + hdrSize );
            System.out.println( "Datasize   : " + bDataSize );

        }
        catch ( FileNotFoundException e )
        {
            System.out.println( "File " + fileName + " not found." );
            return null;
        }
        catch ( IOException e )
        {
            System.out.println( "IOException while reading " + fileName );
            return null;
        }

        return m_hdr;
    }

     /**
      * Does the actual jpg 2 Dicom conversion based on jpg file and 
      * configuration file.
      * @return
      */
     private boolean doJPGTODicomConversion( )
     {

        // String workingDir = DicomNetworkNodeDefaults.getInstance( ).getProperty( DicomNetworkNodeDefaults.CURRENT_INTERMEDIATE_DICOM_CREATION_DIRECTORY );

        String workingDir = System.getProperty( "user.dir" );
        int returnValue;
        String cmd = "";

        //DicomSliceCanvasPolyDraw draw = getCurrentSliceCanvasPolyDraw( );
        String patInfo = m_fInfo.getPatientName( );
        patInfo = patInfo + " (DOB:" + m_fInfo.getPatientBirthDate( ) + ")";
        String accessionNumber = "Accession number : " + m_fInfo.getAccessionNumber( );
        String studyDate = "Date " + m_fInfo.getStudyDate( );

        m_dicomOutputFileName = m_localImageFileNameBase + "_stamped.dcm";

        BufferedImage img = null;
        try
        {
            if ( m_localJPGFile != null )
            {
                img = ImageIO.read( new File( m_localImageFileNameBase + ".jpg" ) );
            }

            Graphics g = img.getGraphics( );

            g.setColor( Color.BLUE );
            g.drawString( patInfo + " / " + studyDate + " / " + accessionNumber, 1, img.getHeight( ) - 25 );
            g.drawString( "Secondary Capture by " + System.getProperty( "user.name" ) + " - For scientific purposes only - DIN-Bildlabor - Inselspital Bern", 1, img.getHeight( ) - 10 );

            ImageIO.write( img, "jpg", new File( ( m_localImageFileNameBase + "_stamped.jpg" ) ) );

        } catch ( IOException e )
        {
            // nop
            System.out.printf( "IOException occurred \n" );
            e.printStackTrace( );
            return false;
        }

        if ( System.getProperty( "os.name" ).toLowerCase( ).contains( "linux" ) == true )
        {
            cmd = workingDir + "/lib/jpg2dcm -c " + m_configFileName + " --uid-prefix " + " 1.3.12.2.1107.5.2 " + m_localImageFileNameBase + "_stamped.jpg" + " " + m_dicomOutputFileName + "\n";
        }

        if ( System.getProperty( "os.name" ).toLowerCase( ).contains( "windows" ) == true )
        {
            cmd = workingDir + "\\lib\\jpg2dcm.bat -c " + m_configFileName + " --uid-prefix " + " 1.3.12.2.1107.5.2 " + m_localImageFileNameBase + "_stamped.jpg" + " " + m_dicomOutputFileName + "\n";
            cmd = generateProperWindowsCommand( cmd );
        }

        //cmd = workingDir + "/lib/jpg2dcm.bat -c " + m_configFileName + " --uid-prefix " + " 1.3.12.2.1107.5.2 " + m_localImageFileNameBase + "_stamped.jpg" + " " + m_dicomFileName + "\n";

        System.out.println( "Processing following system command : " + cmd );

        try
        {
            Process p = Runtime.getRuntime( ).exec( cmd );

            InputStream in = p.getInputStream( );
            int c;
            while ( ( c = in.read( ) ) != -1 )
            {
                System.out.print( (char)c ) ;
            }

            returnValue = p.waitFor( );
            in.close( );
        }
        catch ( IOException ioX )
        {
            // not handled yet
            System.out.println( "IOException occurred" );
            return false;
        }
        catch ( InterruptedException intX )
        {
            // not handled yet
            System.out.println( "InterruptedException occurred" );
            return false;
        }

        return true;
    }

    /**
     * Creates a automatic configuration file which is used to perform the
     * the jpg -> Dicom conversion; This method creates all necessary tags
     * needed for a valid dcm-file.
     * @return 
     */
    public int createJpg2DcmConfigFile( )
    {

        m_configFileName = m_localImageFileNameBase + ".cfg";

        try
        {
            FileWriter fileWriter = new FileWriter( m_configFileName );
            fileWriter.write( "# jpg2dcm Default Configuration for encapsulating JPEG Baseline streams into\n" );
            fileWriter.write( "# DICOM Secondary Capture Image objects\n" );
            fileWriter.write( "# (s. DICOM Part 3, A.32.7 Video Photographic Image IOD)\n\n");
            fileWriter.write( "# Patient Module Attributes \n");
            fileWriter.write( "# Patient's Name\n");
            fileWriter.write( "00100010:" + m_fInfo.getPatientName( ) + "\n" );
            fileWriter.write( "# Patient ID\n");
            fileWriter.write( "00100020:" + m_fInfo.getPatientID( ) + "\n" );
            fileWriter.write( "# Issuer of Patient ID\n" );
            fileWriter.write( "#00100021:" );
            fileWriter.write( "# Patient's Birth Date\n" );
            fileWriter.write( "00100030:" + m_fInfo.getPatientBirthDate( ) + "\n" );
            fileWriter.write( "# Patient's Sex\n" );
            fileWriter.write( "00100040:" + m_fInfo.getPatientSex( ) + "\n\n" );
            fileWriter.write( "# General Study Module Attributes\n" );
            fileWriter.write( "# Study Instance UID\n" );
            fileWriter.write( "#0020000D:" + m_fInfo.getStudyInstanceUID( ) + "\n" );
            fileWriter.write( "# Study Date\n" );
            fileWriter.write( "00080020:" + m_fInfo.getStudyDate( ) + "\n" );
            fileWriter.write( "# Study Time\n" );
            fileWriter.write( "00080030:" + m_fInfo.getStudyTime( ) + "\n" );
            fileWriter.write( "# Referring Physician's Name\n" );
            fileWriter.write( "00080090:" + "DIN-Bildlabor\n" );
            fileWriter.write( "# Study ID\n" );
            fileWriter.write( "00200010:" + m_fInfo.getStudyID( ) + "\n" );
            fileWriter.write( "# Accession Number\n" );
            fileWriter.write( "00080050:" + m_fInfo.getAccessionNumber( ) + "\n" );
            fileWriter.write( "# Study Description\n" );
            fileWriter.write( "00081030:" + m_fInfo.getStudyDescription( ) + "\n" );
            fileWriter.write( "\n" );
            fileWriter.write( "# General Series Module Attributes\n" );
            fileWriter.write( "# Modality\n" );
            fileWriter.write( "00080060:MR\n" ); // for the time being keep this setting: most important is that it does display well
            fileWriter.write( "# Series Instance UID\n" );
            fileWriter.write( "0020000E:" + m_instanceUID + "\n" );
            fileWriter.write( "# Series description\n" );
            fileWriter.write( "0008103E:" + m_seriesDescription + "\n" );

            fileWriter.write( "# Series Number\n" );
            fileWriter.write( "00200011:" + m_seriesNumber + "\n" );


            fileWriter.write( "# General Equipment Module Attributes" + "\n" );
            fileWriter.write( "# Manufacturer" + "\n" );
            fileWriter.write( "00080070:" + "DRNN-SCAN  2002-2010" + "\n" );

            fileWriter.write( "# SC Equipment Module Attributes\n" );
            fileWriter.write( "# Conversion Type\n" );
            fileWriter.write( "00080064:SI" );

            fileWriter.write( "# General Image Module Attributes\n" );
            fileWriter.write( "# Instance Number\n" );
            fileWriter.write( "00200013:" + m_instanceNumber + "\n" );

//            fileWriter.write( "# Image Pixel Module Attributes (detected from JPEG file, if not specified)" + "\n" );
//            fileWriter.write( "# Samples per Pixel\n");
//            fileWriter.write( "#00280002:" + m_dicomInformationAssociation.getSamplesPerPixel( ) + "\n" );
//            fileWriter.write( "# Photometric Interpretation\n");
//            fileWriter.write( "#00280004:" + m_dicomInformationAssociation.getPhotometricInterpretation( ) + "\n" );
//            fileWriter.write( "# Planar Configuration\n" );
//            fileWriter.write( "#00280006:" + m_dicomInformationAssociation.getPlanarConfiguration() + "\n" );
//            fileWriter.write( "# Rows\n" );
//            fileWriter.write( "#00280010:" + m_dicomInformationAssociation.getImageHeight( ) + "\n" );
//            fileWriter.write( "# Columns\n" );
//            fileWriter.write( "#00280011:" + m_dicomInformationAssociation.getImageWidth( ) + "\n" );
//            fileWriter.write( "# Bits Allocated\n");
//            fileWriter.write( "#00280100:" + m_dicomInformationAssociation.getFileType( ) + "\n" );
//            fileWriter.write( "# Bits Stored\n" );
//            fileWriter.write( "#00280101:" + m_dicomInformationAssociation.getBitsStored( ) +"\n" );
//            fileWriter.write( "# High Bit\n" );
//            fileWriter.write( "#00280102:" + m_dicomInformationAssociation.getHighBit( ) + "\n" );
//            fileWriter.write( "# Pixel Representation\n" );
//            fileWriter.write( "#00280103:" + m_dicomInformationAssociation.getPixelRepresentation( ) + "\n\n" );
            fileWriter.write( "# SOP Common Module Attributes\n" );
            fileWriter.write( "# SOP Class UID\n" );
            fileWriter.write( "00080016:1.2.840.10008.5.1.4.1.1.7\n" ); // secondary capture
            fileWriter.write( "# SOP Instance UID\n");
            fileWriter.write( "00080018:" + m_instanceUID + "\n" );

            Globals globs = Globals.getInstance( );

            fileWriter.write( "# Software version" + "\n" );
            fileWriter.write( "00181020:" + globs.APP_NAME + " " + globs.APP_VERSION + "\n" );


//            fileWriter.write( "# jMRUITag" + "\n" );
//            fileWriter.write( "0x6FE00100:" + m_SVSJMRUIDicomTag.getJMRUIDICOMTag( ) + "\n\n" );


//            // user also wants the geometry information tags to be added
//            if ( m_includeGeometryTags == true )
//            {
//
//                String imagePosition = m_dicomInformationAssociation.getImagePositionPatient( ).replaceAll( "\\\\", "\\\\\\\\" );
//                String imageOrientation = m_dicomInformationAssociation.getImageOrientationPatient( ).replaceAll( "\\\\", "\\\\\\\\" );
//
//                fileWriter.write( "# Slice location" + "\n" );
//                fileWriter.write( "00201041:" + m_dicomInformationAssociation.getSliceLocation( ) + "\n" );
//                fileWriter.write( "# Image orientation" + "\n" );
//                fileWriter.write( "00200037:" + imageOrientation + "\n" );
//                fileWriter.write( "# Image position" + "\n" );
//                fileWriter.write( "00200032:" + imagePosition + "\n" );
//                fileWriter.write( "# Slice thickness" + "\n" );
//                fileWriter.write( "00180050:" + m_dicomInformationAssociation.getSliceThickness( ) + "\n" );
//
//                fileWriter.write( "# Image rows" + "\n" );
//                fileWriter.write( "00280010:" + m_dicomInformationAssociation.getImageHeight( ) + "\n" );
//                fileWriter.write( "# Image colums" + "\n" );
//                fileWriter.write( "00280011:" + m_dicomInformationAssociation.getImageWidth( ) + "\n" );
//
//                fileWriter.write( "# Pixel spacing" + "\n" );
//                fileWriter.write( "00280030:" + m_dicomInformationAssociation.getPixelSpacing( ) + "\n" );
//
//                fileWriter.write( "# Image modality" + "\n" );
//                fileWriter.write( "00080060:" + m_dicomInformationAssociation.getImageModality( ) + "\n" );
//
//                fileWriter.write( "# Sequence name" + "\n" );
//                fileWriter.write( "00180024:" + m_dicomInformationAssociation.getSequenceName( ) + "\n" );

//                if ( m_dicomInformationAssociation.getImageModality( ).compareTo( "MR" ) == 0 )
//                {
//
//                    fileWriter.write( "# Protocol name" + "\n" );
//                    fileWriter.write( "00181030:" + m_dicomInformationAssociation.getProtocolName( ) + "\n" );
//
//                    fileWriter.write( "# Echo time" + "\n" );
//                    fileWriter.write( "00180081:" + m_dicomInformationAssociation.getEchoTime( ) + "\n" );
//
//                    fileWriter.write( "# TR" + "\n" );
//                    fileWriter.write( "00180080:" + m_dicomInformationAssociation.getTR( ) + "\n" );
//
//                    fileWriter.write( "# Phase encoding direction" + "\n" );
//                    fileWriter.write( "00181312:" + m_dicomInformationAssociation.getPhaseEncodingDirection( ) + "\n" );
//
//                    fileWriter.write( "# Flip angle" + "\n" );
//                    fileWriter.write( "00181312:" + m_dicomInformationAssociation.getFlipAngle( ) + "\n" );
//
//                    fileWriter.write( "# Echo train length" + "\n" );
//                    fileWriter.write( "00180091:" + m_dicomInformationAssociation.getEchoTrainLength( ) + "\n" );
//
//                    fileWriter.write( "# Percent sampling" + "\n" );
//                    fileWriter.write( "00180093:" + m_dicomInformationAssociation.getPercentSampling( ) + "\n" );
//
//                    fileWriter.write( "# Percent FOV" + "\n" );
//                    fileWriter.write( "00180094:" + m_dicomInformationAssociation.getPercentPhaseFOV( ) + "\n" );
//
//                    fileWriter.write( "# Pixel bandwidth" + "\n" );
//                    fileWriter.write( "00180095:" + m_dicomInformationAssociation.getPixelBandwidth( ) + "\n" );
//
//                    fileWriter.write( "# Samples per pixel" + "\n" );
//                    fileWriter.write( "00280002:" + m_dicomInformationAssociation.getSamplesPerPixel( ) + "\n" );
//
//                    fileWriter.write( "# Transmit coil name" + "\n" );
//                    fileWriter.write( "00181251:" + m_dicomInformationAssociation.getTransmitCoilName( ) + "\n" );
//
//                    fileWriter.write( "# Acquisition type" + "\n" );
//                    fileWriter.write( "00180023:" + m_dicomInformationAssociation.getAcquisitionType( ) + "\n" );
//
//                    fileWriter.write( "# Scanning sequence" + "\n" );
//                    fileWriter.write( "00180020:" + m_dicomInformationAssociation.getScanningSequence( ) + "\n" );
//
//                    fileWriter.write( "# Sequence variant" + "\n" );
//                    fileWriter.write( "00180021:" + m_dicomInformationAssociation.getSequenceVariant( ) + "\n" );
//
//                    fileWriter.write( "# Scan options" + "\n" );
//                    fileWriter.write( "00180022:" + m_dicomInformationAssociation.getScanOptions( ) + "\n" );
//                }
//            }

            fileWriter.close( );

        }
        catch ( IOException e )
        {
            System.out.println( "Error creating jpg2dcm config file : " + m_configFileName );
            return -1;
        }
        return 0;
    }


    /**
     * Creates a screen shot of the contentPane of the frame.
     * @param coloured
     */
    public void createIntermediateImageOfScreenshot( boolean coloured )
    {
        m_component = (JComponent)getContentPane( );

        String patName = m_fInfo.getPatientName( );
        String accessionNumber = m_fInfo.getAccessionNumber( );
        String examDate = m_fInfo.getStudyDate( );
        String examTime = m_fInfo.getStudyTime( );
        int seriesNumber = Integer.parseInt( m_fInfo.getSeriesNumber( ).trim( ) );
        int currentInstanceNumber = 1;

        // String workingDir = System.getProperty( "user.dir" );
        String imageDir = DicomNetworkNodeDefaults.getInstance( ).getProperty( DicomNetworkNodeDefaults.CURRENT_INTERMEDIATE_DICOM_CREATION_DIRECTORY );

        String imageSubDirString  = patName + "_" + examDate + "_" + accessionNumber;

        imageSubDirString = imageDir + System.getProperty( "file.separator" ) + eliminateIllegalFileSystemCharacters( imageSubDirString );

        File imageSubDir = new File( imageSubDirString );

        if ( !imageSubDir.exists( ) )
        {
            imageSubDir.mkdir( );
        }

        m_localImageFileNameBase = imageSubDir + System.getProperty( "file.separator" ) +
                                   patName + "_" + accessionNumber + "_" + examDate +
                                   "_" + examTime + "_" + ( seriesNumber + 1000 ) + "_" + currentInstanceNumber ;

        //System.out.println( ">>>> Instance number = " + currentInstanceNumber );

        m_localImageFileNameBase = eliminateIllegalFileSystemCharacters( m_localImageFileNameBase );

        m_localJPGFile = new File( m_localImageFileNameBase + "." + "jpg" );

        System.out.println( "Local jpg filename : " + m_localJPGFile.getAbsolutePath( ) );

        BufferedImage image = null;

        if ( coloured == true )
        {
            image = new BufferedImage( m_component.getWidth( ) , m_component.getHeight( ),
                                       BufferedImage.TYPE_3BYTE_BGR );
        }
        else
        {
            image = new BufferedImage( m_component.getWidth( ) , m_component.getHeight( ),
                                       BufferedImage.TYPE_BYTE_GRAY );
        }

        Graphics g = image.createGraphics( );
        g.translate( 0, 0 );
        m_component.printAll( g );

        if ( g != null )
        {
            g.dispose( );
        }

        try
        {
            ImageIO.write( image, "jpg", m_localJPGFile );
        }
        catch ( IOException e )
        {
            System.out.print( "Problem writing local image file in DicomFormattedScreenShotCreator::createIntermediateImageOfScreenshot :\n" );
            e.printStackTrace( );
        }
    }


    /**
     * Replaces all blanks by "\  " in order to be interpreted
     * correctly by the Windows operating system
     * @param cmd
     * @return
     */
    private String generateProperWindowsCommand( String cmd )
    {
        String out = "";
        for ( int i = 0; i < cmd.length( ); i++ )
        {
            if ( cmd.substring( i, i + 1 ).compareTo( "\\" ) == 0 )
            {
                out += "\\" + "\\";
            }
            else
            {
                out += cmd.substring( i, i + 1 );
            }
        }
        return out;
    }

//    /**
//     * 
//     * @param args 
//     */
//    public static void main( String[] args )
//    {
//        //Schedule a job for the event dispatch thread:
//        //creating and showing this application's GUI.
//        SwingUtilities.invokeLater(new Runnable( )
//        {
//            @Override
//            public void run( )
//            {
//
//                int numberOfMetabolites = 10;
//                Dimension d = new Dimension( 1024, 270 + numberOfMetabolites * 25 );
//
//                LinkedList<String> metabNames = new LinkedList<String>( );
//
//                double[] concentrations = new double[numberOfMetabolites];
//                double[] errorBars = new double[numberOfMetabolites];
//                double[] normValues = new double[numberOfMetabolites];
//                double[] errorsNormValues = new double[numberOfMetabolites];
//                
//                String refImage0 = "/home/slotboom/Desktop/TESTDATA-FAST/DICOM-NETWORK/ORG_Becher_Sandro_20090226/ep2d_diff_3scan_trace_p3/Becher_Sandro_ep2d_diff_3scan_trace_p3_2_31.dcm";
//                String refImage1 = "/home/slotboom/Desktop/TESTDATA-FAST/DICOM-NETWORK/ORG_Becher_Sandro_20090226/ep2d_diff_3scan_trace_p3/Becher_Sandro_ep2d_diff_3scan_trace_p3_2_31.dcm";
//                String refImage2 = "/home/slotboom/Desktop/TESTDATA-FAST/DICOM-NETWORK/ORG_Becher_Sandro_20090226/ep2d_diff_3scan_trace_p3/Becher_Sandro_ep2d_diff_3scan_trace_p3_2_31.dcm";
//
//                String rawSpectroscopyFileName = "/home/slotboom/Desktop/TESTDATA-FAST/DICOM-NETWORK/ORG_Becher_Sandro_20090226/svs_12ch_135_sep_8av_20m_CS/Becher_Sandro_svs_12ch_135_sep_8av_20m_CS_12_1.dcm";
//                String waterReferenceFileName  = "/home/slotboom/Desktop/TESTDATA-FAST/DICOM-NETWORK/ORG_Becher_Sandro_20090226/svs_12ch_135_nws_CS/ORG_Becher_Sandro_20090226/svs_12ch_135_sep_8av_20m_CS/Becher_Sandro_svs_12ch_135_nws_CS_13_1.dcm";
//
//                for ( int i = 0; i < numberOfMetabolites; i++ )
//                {
//                    metabNames.add( "Metab" + i );
//                    concentrations[i] = 20.0 * (double)(i);
//                    errorBars[i] = (double)(i);
//                    normValues[i] = 25.0 * (double)(i);
//                    errorsNormValues[i] = (double)(i);
//                }
//
//                LinkedList<String> dicomFileRefList = new LinkedList<>( );
//
//                dicomFileRefList.add( rawSpectroscopyFileName );
//
//                SVS_JMRUI_DICOM_Tag_FAST SVSJMRUIDicomTag = new SVS_JMRUI_DICOM_Tag_FAST( );
//
//                SVSMetaboliteTextReportFrame reportFrame =
//                        new SVSMetaboliteTextReportFrame( null, 
//                                                          d, dicomFileRefList,
//                                                          "testWaterRefernceFileName", 5,
//                                                          metabNames,
//                                                          concentrations, errorBars,
//                                                          normValues, errorsNormValues,
//                                                          refImage0, refImage1, refImage2,
//                                                          true,
//                                                          1000,
//                                                          "Screenshot by jMRUI",
//                                                          SVSJMRUIDicomTag,
//                                                          0
//                                                        );
//
//                reportFrame.dicomizeAndSent( true, true, true );
//            }
//        });
//    }
    
    // Attributes
    private final SVSMetaboliteTextReport m_SVSMetaboliteTextReport;
    // private final SVSReferenceImagesPanel m_SVSReferenceImagesPanel;

    private JFrame                  m_parent;
    private BufferedImage           m_bufferedImage;
    private BufferedImage           m_imageAsGrayscale;
    private int[]                   m_pixels;

    private String                      m_referenceImageReference;
    private final LinkedList<String>    m_rawSpectrosopyDicomFileNames;
    private final boolean          m_createDicomFile;

    private String                 m_outputLocalDicomFileReference;

    // added attributes

    private FileInfo       m_fInfo;
    private JComponent     m_component;

    private final int      m_instanceNumber = 1;
    private int            m_seriesNumber   = 1000;

    private String         m_imageDescription;
    private boolean        m_includeGeometryTags;
    private String         m_localImageFileNameBase;
    private File           m_localJPGFile;
    private String         m_configFileName;

    private String         m_instanceUID;
    private String         m_seriesInstanceUID;
    private String         m_seriesDescription = "SCAN-Spectroscopy";
    private String         m_dicomOutputFileName;

    private final SVS_JMRUI_DICOM_Tag_FAST m_SVSJMRUIDicomTag;
    private byte[]         m_pixelDataAsBytes;
    private byte[]         m_hdr;
    private final int      m_outputStyle;
    private GlobalModel    m_globalModel;
    private final DicomDirectory m_dicomDirectory;
    
}
