/*
 * File: $Id: TwoCompartmentModelFittingFrame.java,v 1.2 2006/04/20 09:32:58 hans Exp $
 * History
 * Date: 17.03.2006, 12:06  Authors: Hans Slotboom  
 *
 * Changes:
 *
 * Copyright (C) 2006 by
 * 
 * Inselspital - Department of Neuroradiology
 * CH 3010 Bern Switzerland
 */


/*
 * TwoCompartmentModelFittingFrame.java
 *
 * Created on April 12, 2006, 9:11 AM
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package org.spectrim.tdfdfit.waterreferencemodelling;

import org.spectrim.dicom.DicomDirectory;
import org.spectrim.dicom.DicomImageData;
import org.spectrim.dicom.DicomImageSeriesData;
import org.spectrim.dicom.FileInfo;
import org.spectrim.lib.Dialogs;
import org.spectrim.lib.DoubleRange;
import org.spectrim.lib.DualProgressBar;
import org.spectrim.xyplot.Curve;

import java.awt.BorderLayout;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;

import javax.swing.JFrame;
import javax.swing.JPanel;

import org.spectrim.xyplot.XYPlot;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Toolkit;
import java.text.DecimalFormat;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.LinkedList;
import javax.swing.JOptionPane;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTabbedPane;
import org.dcm4che2.data.DicomObject;
import org.spectrim.data.CSIData;
import org.spectrim.io.dicom.DicomHandler;
import org.spectrim.io.dicom.DicomHeader;
import org.spectrim.lib.DicomStringReformatter;
import org.spectrim.spectra.Spectra;
import org.spectrim.tdfdfit.CopyAndPastableResultAsTextDisplayer;
import org.spectrim.tdfdfit.SingleVoxelMeasuredSpectralData;
import org.spectrim.tdfdfit.TDFDFittingDefaults;
import org.spectrim.tdfdfit.TwoExponentialsModelRandomMininizer;
import org.spectrim.tdfdfit.quantitativeresults.QuantitativeSVSContextDataContainer;

/**
 *
 * Frame containing graphic steering components to perform a bi-exponential fit
 * on water signals that are recorded in separate pulse sequences.
 *
 * @author hans
 */
public class TwoCompartmentWaterReferenceModelFittingFrame extends JFrame implements ActionListener
{

    /** 
     * Creates a new instance of TwoCompartmentModelFittingFrame. 
     * @param parent
     * @param dicomDir
     */
    public TwoCompartmentWaterReferenceModelFittingFrame( JFrame parent, DicomDirectory dicomDir )
    {

        m_parent = parent;

        m_contentPanel = new JPanel( );
        m_contentPanel.setLayout( new BorderLayout( ) );
        
        m_seriesSelectorPanel = new JPanel( );
        m_seriesSelectorPanel.setLayout( new BoxLayout( m_seriesSelectorPanel, BoxLayout.Y_AXIS ) );
    
        m_dicomFileSelector = new TwoCompartmentWaterReferenceModelFittingMeasuredSpectrumSelectionControl( m_parent );

        m_dicomFileSelector.setDicomDir( dicomDir );

        m_dicomDir = dicomDir;

        m_dicomFileSelector.setTreeMultiSelectable( true );

        m_dicomFileSelector.addActionListener( this );

        m_seriesSelectorPanel.add( m_dicomFileSelector );
        
        m_contentPanel.add( m_seriesSelectorPanel, BorderLayout.CENTER );
        
        m_steeringPanel = new JPanel( );
        m_steeringPanel.setLayout( new BoxLayout( m_steeringPanel, BoxLayout.X_AXIS ) );
        m_steeringPanel.setBorder( BorderFactory.createEmptyBorder(5,5,5,5) );
        
        m_loadSelectedSignalsButton = new JButton( "Load selected signals" );
        m_loadSelectedSignalsButton.addActionListener( this );
        
        m_fitAcceptButton = new JButton( "Accept & Quit" );
        m_fitAcceptButton.addActionListener( this );
        
        m_fitCancelButton = new JButton( "Cancel & Quit" );
        m_fitCancelButton.addActionListener( this );
        
        m_steeringPanel.add( Box.createHorizontalGlue( ) );
        m_steeringPanel.add( m_loadSelectedSignalsButton );
        m_steeringPanel.add( Box.createHorizontalGlue( ) );
        m_steeringPanel.add( m_fitAcceptButton );
        m_steeringPanel.add( Box.createHorizontalGlue( ) );
        m_steeringPanel.add( m_fitCancelButton );
        m_steeringPanel.add( Box.createHorizontalGlue( ) );
        
        m_steeringPanel.setMaximumSize( new Dimension( 1024, 100 ) );
        
        m_contentPanel.add( m_steeringPanel, BorderLayout.SOUTH );
        
        // Panel that contains starting values and fitting conditions
        
        m_fittingConditionsPanel = new TwoCompartmentWaterReferenceModelFittingConditionsPanel( m_parent );
        m_fittingConditionsPanel.addActionListener( this );
        addActionListener( m_fittingConditionsPanel );
        
        m_xySignalsPlot = new XYPlot( "Selected Signals (Frequency Domain)", this );
        
        TDFDFittingDefaults defs = TDFDFittingDefaults.getInstance( );
        defs.load( );
        
        if ( defs.getProperty( TDFDFittingDefaults.DISPLAY_DOMAIN ).compareTo( TDFDFittingDefaults.FREQ_DOMAIN ) == 0 )
        { 
            if ( defs.getProperty( TDFDFittingDefaults.DISPLAY_UNIT ).compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_PPM ) == 0 )
            {
                m_xySignalsPlot.setDisplayXReversed( true );
            }
            else
            {
                m_xySignalsPlot.setDisplayXReversed( false );
            }
        }
        m_xySignalsPlot.setDisplayXReversed( true );
        
        m_xyDecayPlot = new XYPlot( "Exponential Decay of Selected Frequency Range", this );
        m_xyDecayPlot.setDisplayXReversed(false);
        
        m_waterReferencePlot = new XYPlot( "Water reference signal", this );
        if ( defs.getProperty( TDFDFittingDefaults.DISPLAY_DOMAIN ).compareTo( TDFDFittingDefaults.FREQ_DOMAIN) == 0 )
        { 
            if ( defs.getProperty( TDFDFittingDefaults.DISPLAY_UNIT ).compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_PPM ) == 0 )
            {
                m_waterReferencePlot.setDisplayXReversed( true );
            }
            else
            {
                m_waterReferencePlot.setDisplayXReversed( false );
            }
        }
        
        m_signalsTabbedPane = new JTabbedPane( );
         
        m_signalsTabbedPane.addTab( "Signals", m_xySignalsPlot );
        m_signalsTabbedPane.addTab( "Decay", m_xyDecayPlot );
        m_signalsTabbedPane.addTab( "Water reference signal", m_waterReferencePlot );
        
        m_horizontalSplitPane = new JSplitPane( JSplitPane.HORIZONTAL_SPLIT, m_fittingConditionsPanel, m_signalsTabbedPane );
        m_verticalSplitPane = new JSplitPane( JSplitPane.VERTICAL_SPLIT, m_horizontalSplitPane, m_dicomFileSelector );
        
        m_signalsTabbedPane.setSelectedIndex( 0 );

        m_contentPanel.add( m_verticalSplitPane, BorderLayout.NORTH  );
        
        m_mainScrollPane = new JScrollPane( m_contentPanel,
                                            JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
                                            JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED );

        setContentPane( m_mainScrollPane );
        
        // add window listener necessary to handle the window closing 
        // event properly
        addWindowListener( 
            new WindowAdapter( )
            {
                @Override
                public void windowClosing( WindowEvent event )
                {
                   dispose( );
                }
            }
        );
        
        pack( );
        setTitle( "Water Reference Signal(s) Definition" );
        
        // Put the Dialog nicely in the middle of its parent
        if ( parent != null )
        {
            Point parentLoc = parent.getLocation( );
            Point centerLocation;
            if ( ( parentLoc.x != 0 ) && ( parentLoc.x != 0) )
            {
                centerLocation = new Point( parent.getX( ) + ( parent.getWidth( ) / 2 ) - this.getWidth( ) / 2,
                                            parent.getY( ) + ( parent.getHeight( ) / 2 ) - this.getHeight( ) / 2 );
            }
            else
            {
                Dimension screenSize = Toolkit.getDefaultToolkit( ).getScreenSize( );
                int width  = (int)screenSize.getWidth( )/2 - this.getWidth( ) / 2;
                int height = (int)screenSize.getHeight( )/2 - this.getHeight( ) / 2;
                if ( width < 0 )
                {
                    width = 0;
                }
                if ( height < 0 )
                {
                    height = 0;
                }
                centerLocation = new Point( width, height );
            }
            
            setLocation( centerLocation );
        }
        setVisible( true );
        
        JOptionPane.showMessageDialog( m_parent,
                                       "Please define: \n 1. Reference water signal (same TE) \n 2. The 2 compartment reference water signals.\n Then press fit signals button. \n Finally close dialog by pressing 'Accept and Quit'.", 
                                       "No water reference signals defined yet!",
                                       JOptionPane.INFORMATION_MESSAGE );
    }

    /**
     *
     * Returns the progress in % of the fitting
     * @return
     */
    public int getFittingProgress( )
    {
        return m_fittingProgressPercent;
    }

    /**
     * Called when registered object raises an ActionEvent
     * @param e
     */
    @Override
    public void actionPerformed( ActionEvent e ) 
    {    
        Object sender = e.getSource( );


        if ( sender == m_dicomFileSelector )
        {
            if ( e.getID( ) == TwoCompartmentWaterReferenceModelFittingMeasuredSpectrumSelectionControl.IMPORT_2COMP_MODEL_WATER_REFERENCE_FILES_EVENT )
            {
                // old style (before spectrim) setSignalsFor2CompartmentControl( );
                setSpectrImSignalsFor2CompartmentControl( );
            }

            if ( e.getID( ) == TwoCompartmentWaterReferenceModelFittingMeasuredSpectrumSelectionControl.IMPORT_WATER_REFERENCE_FILE_EVENT )
            {
                // old style is to be replaced with SpectrIm setReferenceWaterSignal( );
                setSpectrImReferenceWaterSignal( );
            }

        }

        if ( sender == m_fittingConditionsPanel )
        {
            if ( e.getID( ) ==  TwoCompartmentWaterReferenceModelFittingConditionsPanel.FIT_EVENT )
            {
                // System.out.println( "fit event !" );
                
                if ( this.m_fittingConditionsPanel.getFittingMethod( ) == TwoCompartmentWaterReferenceModelFittingConditionsPanel.LEVENBERG_MARQUARDT_METHOD )
                {
                   //TO DO ;
                }
                else
                {
                    performRandomFittingInThread( );
                }
            }
        }
        
        if ( e.getSource( ) == m_loadSelectedSignalsButton )
        { 
            int[] selectedSeries = m_dicomFileSelector.getSelectedSeriesNumbers( );
            if ( selectedSeries.length == 1 )
            {
                // old style; has to be replaced by setReferenceWaterSignal( );
                setSpectrImReferenceWaterSignal( );
            }
            else
            {
                // old style (before spectrim)setSignalsFor2CompartmentControl( );
                setSpectrImSignalsFor2CompartmentControl( );
            }
        }

        if ( e.getSource( ) == m_fitAcceptButton )
        {

            if ( m_compartmentModelFittedSignalStatus == COMPARTMENTS_NOT_FITTED_ID )
            {
                int selection = JOptionPane.showConfirmDialog( m_parent,
                "Two compartment reference signals not fitted: Continue accept and quit?", "No 2 compartment fit available!",
                JOptionPane.YES_NO_OPTION, JOptionPane.INFORMATION_MESSAGE );

                if ( selection == JOptionPane.YES_OPTION )
                {

                    if ( m_referenceSignalStatus == REFERENCE_WATER_STATUS_OK_ID )
                    {
                        notifyActionListeners( REFERENCE_WATER_STATUS_OK_ID, REFERENCE_WATER_STATUS_OK_CMD );
                      
                        setVisible( false );
                    }

                    if ( m_referenceSignalStatus == REFERENCE_WATER_STATUS_INITIAL_ID )
                    {
                        int selection2 = JOptionPane.showConfirmDialog( m_parent,
                        "No reference water and 2 compartment reference signals set: Continue accept and quit?", "No water reference signals defined at all!",
                        JOptionPane.YES_NO_OPTION, JOptionPane.INFORMATION_MESSAGE );
                        if ( selection2 == JOptionPane.YES_OPTION )
                        {
                            notifyActionListeners( REFERENCE_WATER_STATUS_INITIAL_ID, REFERENCE_WATER_STATUS_INITIAL_CMD );
                            setVisible( false );
                        }
                    }

                    if ( m_referenceSignalStatus == REFERENCE_WATER_STATUS_2COMP_SET_ONLY_ID  )
                    {
                        int selection3 = JOptionPane.showConfirmDialog( m_parent,
                        "Reference water signal not set: Continue accept and quit?", "Reference water signal not defined!",
                        JOptionPane.YES_NO_OPTION, JOptionPane.INFORMATION_MESSAGE );
                        if ( selection3 == JOptionPane.YES_OPTION )
                        {
                            notifyActionListeners( REFERENCE_WATER_STATUS_2COMP_SET_ONLY_ID, REFERENCE_WATER_STATUS_2COMP_SET_ONLY_CMD );
                            setVisible( false );
                        }
                    }

                    if ( m_referenceSignalStatus == REFERENCE_WATER_STATUS_WATER_REF_SET_ONLY_ID  )
                    {
                        int selection4 = JOptionPane.showConfirmDialog( m_parent,
                                                                       "Two compartment reference signals not set: Continue Accept and Quit?",
                                                                       "Two compartment reference signals not defined!",
                                                                       JOptionPane.YES_NO_OPTION, JOptionPane.INFORMATION_MESSAGE );
                        if ( selection4 == JOptionPane.YES_OPTION )
                        {                            
                            
                            // Only set the water reference signal.
                            QuantitativeSVSContextDataContainer waterRefData = QuantitativeSVSContextDataContainer.getInstance( );

                            waterRefData.setTwoCompartmentModelSet( false );
                            waterRefData.setSimpleWaterRefefenceSignalSet( true );
                            waterRefData.setWaterReferenceSignalReal( getReferenceWaterReal( ) ); 
                            waterRefData.setWaterReferenceSignalImag( getReferenceWaterImag( ) );
                            
                            notifyActionListeners( REFERENCE_WATER_STATUS_WATER_REF_SET_ONLY_ID, 
                                                   REFERENCE_WATER_STATUS_WATER_REF_SET_ONLY_CMD );
                            setVisible( false );
                        }
                    }
                }
                else
                {
                    // System.out.println( "Accept & Quit stopped by user." );
                }
            }
            else
            {
                if ( m_referenceSignalStatus == REFERENCE_WATER_STATUS_OK_ID )
                {
                    
                    // Is the case where both reference signals are set. 
                    QuantitativeSVSContextDataContainer waterRefData = QuantitativeSVSContextDataContainer.getInstance( );

                    waterRefData.setSpectroscopyDicomWaterFileReferences( m_dicomFileSelector.getSelectedSeriesFileReferences( ) );
                    
                    waterRefData.setTwoCompartmentModelSet( true );
                    waterRefData.setSimpleWaterRefefenceSignalSet( true );
                    
                    waterRefData.setAmp1_TCM( getAmplitude_A1( ) );
                    waterRefData.setAmp2_TCM( getAmplitude_A2( ) );
                    waterRefData.setTC1_TCM( getTimeConstant_TC1( ) );
                    waterRefData.setTC2_TCM( getTimeConstant_TC2( ) );

                    waterRefData.setErrAmp1_TCM( getErrorAmplitude_A1( ) );
                    waterRefData.setErrAmp1_TCM( getErrorAmplitude_A2( ) );
                    waterRefData.setErrTC1_TCM( getErrorTimeConstant_TC1( ) );
                    waterRefData.setErrTC1_TCM( getErrorTimeConstant_TC2( ) );

                    // Set the simple water reference signal.
                    waterRefData.setWaterReferenceSignalReal( getReferenceWaterReal( ) ); 
                    waterRefData.setWaterReferenceSignalImag( getReferenceWaterImag( ) );

                    // Set the water 2 component reference signals. 
                    waterRefData.set2CompartmentModelSignals( getNumberOf2ComponentWaterReferences( ), 
                                                              getReference2CompartmentModelWaterReal( ) , 
                                                              getReference2CompartmentModelWaterImag( ) );
                    
                    waterRefData.setFittedSpectrumFileNames( m_dicomFileSelector.getSelected2CompartmentWaterReferences( ) );
                    waterRefData.setReferenceWaterDefinedFileName( m_dicomFileSelector.getSelectedSingleWaterReference( ) );
                    
                    notifyActionListeners( REFERENCE_WATER_STATUS_OK_ID, REFERENCE_WATER_STATUS_OK_CMD );
                    setVisible( false );
                }

                if ( m_referenceSignalStatus == REFERENCE_WATER_STATUS_INITIAL_ID )
                {
                    
                    // This if statement handles the case where the user opened
                    // the water reference control, but did not do further 
                    // anything else
                    int selection2 = JOptionPane.showConfirmDialog( m_parent,
                    "No reference water and 2 compartment reference signals set: Continue accept and quit?", "No water reference signals defined at all!",
                    JOptionPane.YES_NO_OPTION, JOptionPane.INFORMATION_MESSAGE );
                    if ( selection2 == JOptionPane.YES_OPTION )
                    {
                        notifyActionListeners( REFERENCE_WATER_STATUS_INITIAL_ID, REFERENCE_WATER_STATUS_INITIAL_CMD );
                        setVisible( false );
                    }
                }

                if ( m_referenceSignalStatus == REFERENCE_WATER_STATUS_2COMP_SET_ONLY_ID  )
                {
                    
                    // The user only defined the 2 compartment model signals 
                    int selection3 = JOptionPane.showConfirmDialog( m_parent,
                    "Reference water signal not set: Continue accept and quit?", "Reference water signal not defined!",
                    JOptionPane.YES_NO_OPTION, JOptionPane.INFORMATION_MESSAGE );
                    if ( selection3 == JOptionPane.YES_OPTION )
                    {
                        // The user only wants to set the 2 compartment 
                        // water references. 
                        
                        QuantitativeSVSContextDataContainer waterRefData = QuantitativeSVSContextDataContainer.getInstance( );

                        waterRefData.setTwoCompartmentModelSet( true );
                        waterRefData.setSimpleWaterRefefenceSignalSet( false );
                        
                        waterRefData.setAmp1_TCM( getAmplitude_A1( ) );
                        waterRefData.setAmp2_TCM( getAmplitude_A2( ) );
                        waterRefData.setTC1_TCM( getTimeConstant_TC1( ) );
                        waterRefData.setTC2_TCM( getTimeConstant_TC2( ) );

                        waterRefData.setErrAmp1_TCM( getErrorAmplitude_A1( ) );
                        waterRefData.setErrAmp1_TCM( getErrorAmplitude_A2( ) );
                        waterRefData.setErrTC1_TCM( getErrorTimeConstant_TC1( ) );
                        waterRefData.setErrTC1_TCM( getErrorTimeConstant_TC2( ) );

                        // Set the water 2 component reference signals 
                        waterRefData.set2CompartmentModelSignals( getNumberOf2ComponentWaterReferences( ), 
                                                                  getReference2CompartmentModelWaterReal( ) , 
                                                                  getReference2CompartmentModelWaterImag( ) );
                
                        waterRefData.setFittedSpectrumFileNames( m_dicomFileSelector.getSelected2CompartmentWaterReferences( ) );
                       
                        notifyActionListeners( REFERENCE_WATER_STATUS_2COMP_SET_ONLY_ID, REFERENCE_WATER_STATUS_2COMP_SET_ONLY_CMD );
                        setVisible( false );
                    }
                }

                if ( m_referenceSignalStatus == REFERENCE_WATER_STATUS_WATER_REF_SET_ONLY_ID  )
                {
                    int selection4 = JOptionPane.showConfirmDialog( m_parent,
                                                                   "Two compartment reference signals not set: Continue Accept and Quit?",
                                                                   "Two compartment reference signals not defined!",
                                                                   JOptionPane.YES_NO_OPTION, JOptionPane.INFORMATION_MESSAGE );
                    if ( selection4 == JOptionPane.YES_OPTION )
                    {
                        QuantitativeSVSContextDataContainer waterRefData = QuantitativeSVSContextDataContainer.getInstance( );

                        waterRefData.setTwoCompartmentModelSet( false );
                        waterRefData.setSimpleWaterRefefenceSignalSet( true );
                        waterRefData.setReferenceWaterDefinedFileName( m_dicomFileSelector.getSelectedSingleWaterReference( ) );
  
                        // Set the simple water reference signal.
                        waterRefData.setWaterReferenceSignalReal( getReferenceWaterReal( ) ); 
                        waterRefData.setWaterReferenceSignalImag( getReferenceWaterImag( ) );
                        
                        notifyActionListeners( REFERENCE_WATER_STATUS_WATER_REF_SET_ONLY_ID, REFERENCE_WATER_STATUS_WATER_REF_SET_ONLY_CMD );
                        setVisible( false );
                    }
                }
            }
        }

        if ( e.getSource( ) == m_fitCancelButton )
        {
            dispose( );
        }
    }
    
    
    /**
     * Does the actual fitting work!
     */
    public void performRandomFitting( )
    {
        int numberOfFittingRepetitions = m_fittingConditionsPanel.getNumberOfFittingRepetitions( );
        int numberOfIterationsInRandomFit = m_fittingConditionsPanel.getNumberOfIterationsInRandomFit( );
        int numberOfFunctionEvaluationsInRandomFit = m_fittingConditionsPanel.getNumberOfFunctionEvaluationsInRandomFit( );

        double[][] allMinPars = new double[numberOfFittingRepetitions][4];
        double[] minFncVals = new double[numberOfFittingRepetitions];

        DoubleRange[] parameterRanges = new DoubleRange[4];


        for ( int i = 0; i < numberOfFittingRepetitions; i++ )
        {
            double[] startingPars = new double[4];

            startingPars[0] = m_fittingConditionsPanel.getTC_Amplitude1_StartingValue( ) * m_signalDecaySignal[0];
            startingPars[1] = 1.0 / ( m_fittingConditionsPanel.getTC_TimeConstant1_StartingValue( ) / 1000.0 );

            startingPars[2] = m_fittingConditionsPanel.getTC_Amplitude2_StartingValue( ) * m_signalDecaySignal[0];
            startingPars[3] = 1.0 / ( m_fittingConditionsPanel.getTC_TimeConstant2_StartingValue( ) / 1000.0 );

            ////////////////////////////////////////////////////////////////////
            double ampValMin = m_fittingConditionsPanel.getMinimumAllowedTC_Amplitude_1( ) * m_signalDecaySignal[0];
            double ampValMax = m_fittingConditionsPanel.getMaximumAllowedTC_Amplitude_1( ) * m_signalDecaySignal[0];

            parameterRanges[0] = new DoubleRange( ampValMin, ampValMax );

            ////////////////////////////////////////////////////////////////////
            double tcValMin = 1.0 / ( m_fittingConditionsPanel.getMaximumAllowed_TC1_TimeConstant( ) / 1000.0 );
            double tcValMax = 1.0 / ( m_fittingConditionsPanel.getMinimumAllowed_TC1_TimeConstant( ) / 1000.0 );
            
            parameterRanges[1] = new DoubleRange( tcValMin, tcValMax);


            ////////////////////////////////////////////////////////////////////
            ampValMin = m_fittingConditionsPanel.getMinimum_Allowed_TC_Amplitude_2( ) * m_signalDecaySignal[0];
            ampValMax = m_fittingConditionsPanel.getMaximum_Allowed_TC_Amplitude_2( ) * m_signalDecaySignal[0];

            parameterRanges[2] = new DoubleRange( ampValMin, ampValMax );

            ////////////////////////////////////////////////////////////////////
            tcValMax = 1.0 / ( m_fittingConditionsPanel.getMinimumAllowed_TC2_TimeConstant( ) / 1000.0 );
            tcValMin = 1.0 / ( m_fittingConditionsPanel.getMaximumAllowed_TC2_TimeConstant( ) / 1000.0 );

            parameterRanges[3] = new DoubleRange( tcValMin, tcValMax );

            TwoExponentialsModelRandomMininizer rndMin = new TwoExponentialsModelRandomMininizer( );

            rndMin.setXValues( m_svsDataSeriesTEs );

            double[] signal = new double[m_svsDataSeriesTEs.length];

            signal = rndMin.getSignal( startingPars );


//            Curve curve = new Curve( m_svsDataSeriesTEs, signal );
//            curve.setColor( Color.BLUE );
//            curve.setStyle( Curve.FAT_SOLID_LINE );
//
//            m_xyDecayPlot.addCurve( new Curve( m_svsDataSeriesTEs, signal ), true );

            rndMin.setYValues( m_signalDecaySignal );

            rndMin.setStartParameters( startingPars );

            rndMin.setParameterRanges( parameterRanges );

            m_initialFunctionValue = rndMin.fnc( startingPars );

            rndMin.performRandomMinimization( numberOfFunctionEvaluationsInRandomFit, numberOfIterationsInRandomFit, false );

            double[] minPars = new double[4];

            minPars = rndMin.getMinimizingParameters( );

            allMinPars[i][0] = minPars[0];
            allMinPars[i][1] = minPars[1];
            allMinPars[i][2] = minPars[2];
            allMinPars[i][3] = minPars[3];

            minFncVals[i] = rndMin.getMinimumFucntionValue( );

            DecimalFormat formatter  = new DecimalFormat( "0.00E00" );

            String info =  "Initial ChiSq = " + formatter.format( m_initialFunctionValue ) + "\t" +
                           "ChiSq         = " + formatter.format( minFncVals[i] ) + "\t" +
                           "A[1]          = " + formatter.format( allMinPars[i][0] ) + "\t" +
                           "TC[1]         = " + formatter.format( 1000.0/allMinPars[i][1] ) + "\t" +
                           "A[2]          = " + formatter.format( allMinPars[i][2] ) + "\t" +
                           "TC[2]         = " + formatter.format( 1000.0/allMinPars[i][3] );

            // System.out.println( "Fit number" + i + " : " + info );

            signal  = rndMin.getSignal( minPars );


            // compute residue signal
            double[] residue = new double[signal.length];
            double[] zeroLine = new double[signal.length];

            for ( int k = 0; k < signal.length; k++ )
            {
                residue[k] = signal[k] - m_signalDecaySignal[k] - m_signalDecaySignal[0] * 0.1;
                zeroLine[k] = 0.0;
            }

            Curve curveModelSignal = new Curve( m_svsDataSeriesTEs, signal );
            curveModelSignal.setColor( Color.RED );
            curveModelSignal.setStyle( Curve.FAT_SOLID_LINE );
            
            // delete all curves
            m_xyDecayPlot.deleteAllCurves( );

            Curve curveMeasuredSignal = new Curve( m_svsDataSeriesTEs, m_signalDecaySignal );
            curveMeasuredSignal.setColor( Color.BLACK );
            curveMeasuredSignal.setStyle( Curve.FAT_SOLID_LINE );

            // Add the measured curve (which is fitted)
            m_xyDecayPlot.addCurve( curveMeasuredSignal, true );

            // set the 2 component model parameters to zero
            m_minPars = new double[4];
            m_errorMinPars= new double[4];

            m_xyDecayPlot.addCurve( curveModelSignal, true );

            Curve zeroCurve = new Curve( m_svsDataSeriesTEs, zeroLine );
            zeroCurve.setColor( Color.ORANGE );
            //zeroCurve.setStyle( Curve.SOLID_LINE );
         
            m_xyDecayPlot.addCurve( zeroCurve, true );
            
            
            Curve curveResidue = new Curve( m_svsDataSeriesTEs, residue );
            curveResidue.setColor( Color.GREEN );
            curveResidue.setStyle( Curve.FAT_SOLID_LINE );

            m_xyDecayPlot.addCurve( curveResidue, true );
            
            m_fittingProgressPercent = (int)( 100.0 * (double)i / numberOfFittingRepetitions );

            notifyActionListeners( FIT_STEP_PROGRESS_EVENT_ID, FIT_STEP_PROGRESS_EVENT_CMD );
            m_fittingProgressPercent = (int)( 100.0 * (double)i / numberOfFittingRepetitions );

        }

        notifyActionListeners( FIT_FINISHED_EVENT_ID, FIT_FINISHED_EVENT_CMD );

        double  a1  = 0.0;
        double  a2  = 0.0;
        double  tc1 = 0.0;
        double  tc2 = 0.0;
        double  chisq = 0.0;

        for ( int i = 0 ; i < numberOfFittingRepetitions ; i++ )
        {
            a1 = a1 + allMinPars[i][0];
            a2 = a2 + allMinPars[i][2];

            tc1 = tc1 + allMinPars[i][1];
            tc2 = tc2 + allMinPars[i][3];
            chisq = chisq + minFncVals[i];
        }

        double avgA1  = a1 / numberOfFittingRepetitions;
        double avgA2  = a2 / numberOfFittingRepetitions;
        double avgTC1 = tc1 / numberOfFittingRepetitions;
        double avgTC2 = tc2 / numberOfFittingRepetitions;
        double avgChisq = chisq / numberOfFittingRepetitions;

        double stDevA1  = 0.0;
        double stDevA2  = 0.0;
        double stDevTC1 = 0.0;
        double stDevTC2 = 0.0;
        double stDevChisq = 0.0;

        for ( int i = 0 ; i < numberOfFittingRepetitions ; i++ )
        {
            stDevA1  = stDevA1 + ( allMinPars[i][0] - avgA1 ) * ( allMinPars[i][0] - avgA1 );
            stDevA2  = stDevA2 + ( allMinPars[i][2] - avgA2 ) * ( allMinPars[i][2] - avgA2 );

            stDevTC1 = stDevTC1 + ( allMinPars[i][1] - avgTC1 ) * ( allMinPars[i][1] - avgTC1 );
            stDevTC2 = stDevTC2 + ( allMinPars[i][3] - avgTC2 ) * ( allMinPars[i][3] - avgTC2 );

            stDevChisq = stDevChisq + ( minFncVals[i] - avgChisq ) * ( minFncVals[i] - avgChisq );
        }


        stDevA1 = Math.sqrt( stDevA1 / numberOfFittingRepetitions );
        stDevA2 = Math.sqrt( stDevA2 / numberOfFittingRepetitions );

        stDevTC1 = Math.sqrt( stDevTC1 / numberOfFittingRepetitions );
        stDevTC2 = Math.sqrt( stDevTC2 / numberOfFittingRepetitions );

        stDevChisq = Math.sqrt( stDevChisq / numberOfFittingRepetitions );

        DecimalFormat formatter  = new DecimalFormat( "0.0E00" );
        DecimalFormat formatter2 = new DecimalFormat( "0.0000E00" );
        DecimalFormat formatter3 = new DecimalFormat( "0.00E00" );

        String info =  "A[1]= " + formatter.format( avgA1 ) + " " +
                       "TC[1]= " + formatter.format( 1000.0/avgTC1 ) + " " +
                       "A[2]=  " + formatter.format( avgA2 ) + " " +
                       "TC[2]= " + formatter.format( 1000.0/avgTC2 ) +
                       "Chisq= " + formatter.format( avgChisq );

        String info2 = "Initial Chisq\t" + formatter2.format( m_initialFunctionValue  ) + "\n" +
                       "Chisq\t" + formatter2.format( avgChisq  ) + " +/- " + formatter2.format( stDevChisq ) + "\n\n" +
                       "A[1 ]\t" + formatter2.format( avgA1  ) + " +/- " + formatter2.format( stDevA1  ) + " on value interval [" + formatter3.format( ( (DoubleRange)parameterRanges[0]).getMinimum( ) ) + " , " + formatter3.format( ( (DoubleRange)parameterRanges[0]).getMaximum( ) ) + " ] \n" +
                       "A[2 ]\t" + formatter2.format( avgA2 )  + " +/- " + formatter2.format( stDevA2  ) + " on value interval [" + formatter3.format( ( (DoubleRange)parameterRanges[2]).getMinimum( ) ) + " , " + formatter3.format( ( (DoubleRange)parameterRanges[2]).getMaximum( ) ) + " ] \n" +
                       "TC[1]\t" + formatter2.format( 1000.0 / avgTC1 ) + " +/- " + formatter2.format( 1000.0 * stDevTC1 / ( avgTC1 * avgTC1 ) ) + " on value interval [" + formatter3.format( 1000.0/( (DoubleRange)parameterRanges[1]).getMaximum( ) ) + " , " + formatter3.format( 1000.0/( (DoubleRange)parameterRanges[1]).getMinimum( ) ) + "] \n" +
                       "TC[2]\t" + formatter2.format( 1000.0 / avgTC2 ) + " +/- " + formatter2.format( 1000.0 * stDevTC2 / ( avgTC2 * avgTC2 ) ) + " on value interval [" + formatter3.format( 1000.0/( (DoubleRange)parameterRanges[3]).getMaximum( ) )+ " , " + formatter3.format( 1000.0/( (DoubleRange)parameterRanges[3]).getMinimum( ) ) + "] \n";

//                       formatter2.format( avgA1 )           + "\n" +
//                       formatter2.format( avgA2 )           + "\n" +
//                       formatter2.format( 1000.0/avgTC1 )   + "\n" +
//                       formatter2.format( 1000.0/avgTC2 )   + "\n" +
//
//                       formatter2.format( stDevA1 )         + "\n" +
//                       formatter2.format( stDevA2 )         + "\n" +
//                       formatter2.format( 1000.0 * stDevTC1 / ( avgTC1 * avgTC1 ) ) + "\n" +
//                       formatter2.format( 1000.0 * stDevTC2 / ( avgTC2 * avgTC2 ) ) + "\n";


        m_minPars[0] = avgA1;
        m_minPars[2] = avgA2;
        m_minPars[1] = 1000.0/avgTC1;
        m_minPars[3] = 1000.0/avgTC2;

        m_errorMinPars[0] = stDevA1;
        m_errorMinPars[2] = stDevA2;
        m_errorMinPars[1] = 1000.0 * stDevTC1 / ( avgTC1 * avgTC1 );
        m_errorMinPars[3] = 1000.0 * stDevTC2 / ( avgTC2 * avgTC2 );


        m_xyDecayPlot.setInfoString( info, Color.BLUE );

        CopyAndPastableResultAsTextDisplayer displayer = new CopyAndPastableResultAsTextDisplayer( m_parent, info2, "Fitting parameters Result Frame" );
        
        // Update the waterreference status
        QuantitativeSVSContextDataContainer waterRefData = QuantitativeSVSContextDataContainer.getInstance( );
        waterRefData.setTwoCompartmentModelFitted( true );
    }

    /**
     *
     * Performs the renaming in a separate thread to keep the GUI responsive.
     *
     */
    private void performRandomFittingInThread( )
    {

        if ( m_svsDataSeries != null )
        {
        
            final org.spectrim.lib.SwingWorker worker = new org.spectrim.lib.SwingWorker( )
            {

                 @Override
                 public Object construct( )
                 {
                     performRandomFitting( );
                     return null;
                 }

                 // runs in the event dispatching thread
                 @Override
                 public void finished( )
                 {
                    m_compartmentModelFittedSignalStatus = COMPARTMENTS_FITTED_ID;
                    //Dialogs.messageBox( m_parent, "Fitting finished!" );
                 }
             };
             worker.start( );
        }
        else
        {
            JOptionPane.showMessageDialog( rootPane, "No two component water reference spectra are set!" );
        }
    }

    /**
     * Sets the water reference signal.
     */
    private void setReferenceWaterSignal( )
    {
        int[] selectedSeries = m_dicomFileSelector.getSelectedSeriesNumbers( );

        LinkedList<String> selectedSeriesFileReferences;
        selectedSeriesFileReferences = m_dicomDir.getImageFileReferencesOfSeriesNumber( selectedSeries[0] );

        if ( selectedSeries.length > 1 )
        {
            Dialogs d = new Dialogs( );
            Dialogs.messageBox( m_parent, "<HTML> Please select only one series <BR> that contains the unwatersuppressed response</HTML>" );
        }
        else
        {
            if ( m_dicomFileSelector.isImageSeries( ) == true )
            {
                Dialogs d = new Dialogs( );
                Dialogs.messageBox( m_parent, "<HTML> Series contains image data <BR> Please select only series containing spectroscopic data <BR> that contains the unwatersuppressed response</HTML>" );
            }
            else
            {

                DicomImageData d_ima_data = new DicomImageData( selectedSeriesFileReferences.getFirst( ) );

                LinkedHashMap ssd = d_ima_data.getSiemensSpecificDataFromFile( );

                int dim1, dim2, dim3;
                dim1 = dim2 = dim3 = 1;

                if ( (String)ssd.get( "sSpecPara.lFinalMatrixSizePhase" ) != null )
                {
                    dim1 = ( new Integer( (String)ssd.get( "sSpecPara.lFinalMatrixSizePhase" ) ) ).intValue( );
                }

                if ( (String)ssd.get( "sSpecPara.lFinalMatrixSizePhase" ) != null )
                {
                    dim2 = ( new Integer( (String)ssd.get( "sSpecPara.lFinalMatrixSizePhase" ) ) ).intValue( );
                }

                // 3D data set not yet available, so set the value to one
                dim3 = 1;

                if ( ( dim1 == 1 ) && ( dim2 == 1 ) && ( dim3 == 1 ) )
                {
                    // we are dealing with an Single Voxel Spectroscopy measurement
                    FileInfo fi = d_ima_data.getHeader( );
                    SingleVoxelMeasuredSpectralData specData =  new SingleVoxelMeasuredSpectralData( fi.getNonImageData( ) );
                    specData.setDicomHeaderInfo( fi );
                    specData.setSiemensSpecificDicomHeaderInfo( d_ima_data.getSiemensSpecificDataFromFile( ) );

                    m_svsReferenceWaterData = specData;
                    m_svsReferenceWaterTE = (double)specData.getTE( ) / 1000000.0;
                    m_signalsTabbedPane.setSelectedIndex( 2 );
                    
                    m_waterReferencePlot.deleteAllCurves( );
                    m_waterReferencePlot.addCurve( specData.getAbsorptionCurveFDPoints( ), true );

                    if ( m_referenceSignalStatus == REFERENCE_WATER_STATUS_2COMP_SET_ONLY_ID )
                    {
                        m_referenceSignalStatus = REFERENCE_WATER_STATUS_OK_ID;
                    }
                    else
                    {
                        m_referenceSignalStatus = REFERENCE_WATER_STATUS_WATER_REF_SET_ONLY_ID;
                    }

                }
            }
        }
    }

    /**
     * Sets the water reference signal from within SpectrIm.
     */
    private void setSpectrImReferenceWaterSignal( )
    {
        int[] selectedSeries = m_dicomFileSelector.getSelectedSeriesNumbers( );

        LinkedList<String> selectedSeriesFileReferences;
        selectedSeriesFileReferences = m_dicomDir.getImageFileReferencesOfSeriesNumber( selectedSeries[0] );

        if ( selectedSeries.length > 1 )
        {
            Dialogs d = new Dialogs( );
            Dialogs.messageBox( m_parent, "<HTML> Please select only one series <BR> that contains the unwatersuppressed response</HTML>" );
        }
        else
        {   
            // Try to load the SVS dataset
            SingleVoxelMeasuredSpectralData specData = getSVSData( selectedSeriesFileReferences.getFirst( ) );
            
            m_svsReferenceWaterData = specData;
            m_svsReferenceWaterTE = (double)specData.getTE( ) / 1000.0;
            m_signalsTabbedPane.setSelectedIndex( 2 );

            m_waterReferencePlot.deleteAllCurves( );
            m_waterReferencePlot.addCurve( specData.getAbsorptionCurveFDPoints( ), true );

            if ( m_referenceSignalStatus == REFERENCE_WATER_STATUS_2COMP_SET_ONLY_ID )
            {
                m_referenceSignalStatus = REFERENCE_WATER_STATUS_OK_ID;
            }
            else
            {
                m_referenceSignalStatus = REFERENCE_WATER_STATUS_WATER_REF_SET_ONLY_ID;
            }
        }
    } 
    
    /**
     * Returns the ID whether the 2 compartment signals were fitted
     * @return 
     */
    public int getTwoCompartmentModelFittingStatus(  )
    {
        return m_compartmentModelFittedSignalStatus;
    }

    /**
     * Sets the signals for the 2 compartment model.
     */
    private void setSignalsFor2CompartmentControl( )
    {
        m_xySignalsPlot.deleteAllCurves( );

        int[] selectedSeries = m_dicomFileSelector.getSelectedSeriesNumbers( );

        m_svsDataSeries = new SingleVoxelMeasuredSpectralData[ selectedSeries.length ];
        m_svsDataSeriesTEs = new double[ selectedSeries.length ];
        
        m_numberOf2ComponentWaterReferences = selectedSeries.length;

        for ( int i = 0 ; i < selectedSeries.length; i++ )
        {
            LinkedList<String> selectedSeriesFileReferences;
            selectedSeriesFileReferences = m_dicomDir.getImageFileReferencesOfSeriesNumber( selectedSeries[i] );

            DicomImageSeriesData dicomData = new DicomImageSeriesData( selectedSeriesFileReferences );
            dicomData.orderBySlicePosition( );

            if ( m_dicomFileSelector.isImageSeries( ) == true )
            {
                 Dialogs d = new Dialogs( );
                 Dialogs.messageBox( m_parent, "<HTML> Cannot start fitting  <BR> Image data selected; SVS-data expected!</HTML>" );
            }
            else
            {
                DicomImageData d_ima_data = new DicomImageData( selectedSeriesFileReferences.getFirst( ) );

                LinkedHashMap ssd = d_ima_data.getSiemensSpecificDataFromFile( );

                int dim1, dim2, dim3;
                dim1=dim2=dim3=1;

                if ( (String)ssd.get( "sSpecPara.lFinalMatrixSizePhase" ) != null )
                {
                    dim1 = ( new Integer( (String)ssd.get( "sSpecPara.lFinalMatrixSizePhase" ) ) ).intValue( );
                }

                if ( (String)ssd.get( "sSpecPara.lFinalMatrixSizePhase" ) != null )
                {
                    dim2 = ( new Integer( (String)ssd.get( "sSpecPara.lFinalMatrixSizePhase" ) ) ).intValue( );
                }

                // 3D data set not yet available, so set the value to one
                dim3 = 1;

                if ( ( dim1 == 1 ) && ( dim2 == 1 ) && ( dim3 == 1 ) )
                {
                    // we are dealing with an Single Voxel Spectroscopy measurement
                    FileInfo fi = d_ima_data.getHeader( );
                    SingleVoxelMeasuredSpectralData specData =  new SingleVoxelMeasuredSpectralData( fi.getNonImageData( ) );
                    specData.setDicomHeaderInfo( fi );
                    specData.setSiemensSpecificDicomHeaderInfo( d_ima_data.getSiemensSpecificDataFromFile( ) );

                    m_svsDataSeries[ i ] = specData;
                    m_svsDataSeriesTEs[ i ] = (double)specData.getTE( )/1000000.0;

                    // System.out.println( "TE = " + m_svsDataSeriesTEs[ i ] );

                    m_signalsTabbedPane.setSelectedIndex( 1 );
                    m_xySignalsPlot.addCurve( specData.getMagnitudeCurveFDPoints( ), true );                    
                }

                if ( ( dim1 != 1 ) || ( dim2 != 1 ) || ( dim3 != 1 ) )
                {
                    Dialogs d = new Dialogs( );
                    Dialogs.messageBox( m_parent, "<HTML> Cannot start fitting  <BR> CSI data selected; SVS-data expected!</HTML>" );
                }
            }
        }

        m_xySignalsPlot.addTwoHorizontalXYPlotSliders( );
        m_xySignalsPlot.addMouseListener( m_xySignalsPlot );
        m_xySignalsPlot.addMouseMotionListener( m_xySignalsPlot );

        int[] positions = m_xySignalsPlot.getSliderPositions( );

        m_minimumSignalArrayIndex = positions[0];
        m_maximumSignalArrayIndex = positions[1];

        computeAndSetDecay( );

        Curve curve = new Curve( m_svsDataSeriesTEs, m_signalDecaySignal );
        curve.setColor( Color.BLACK );
        curve.setStyle( Curve.FAT_SOLID_LINE );

        m_xyDecayPlot.deleteAllCurves( );
        m_xyDecayPlot.addCurve( curve, true );

        if ( m_referenceSignalStatus == REFERENCE_WATER_STATUS_WATER_REF_SET_ONLY_ID )
        {
            m_referenceSignalStatus = REFERENCE_WATER_STATUS_OK_ID;
        }
        else
        {
            m_referenceSignalStatus = REFERENCE_WATER_STATUS_2COMP_SET_ONLY_ID;
        }
    }

    /**
     * Sets the signals for the 2 compartment model in SpectrIm.
     */
    private void setSpectrImSignalsFor2CompartmentControl( )
    {
        m_xySignalsPlot.deleteAllCurves( );

        int[] selectedSeries = m_dicomFileSelector.getSelectedSeriesNumbers( );

        m_svsDataSeries = new SingleVoxelMeasuredSpectralData[ selectedSeries.length ];
        m_svsDataSeriesTEs = new double[ selectedSeries.length ];
        
        m_numberOf2ComponentWaterReferences = selectedSeries.length;

        for ( int i = 0 ; i < selectedSeries.length; i++ )
        {
            LinkedList<String> selectedSeriesFileReferences;
            selectedSeriesFileReferences = m_dicomDir.getImageFileReferencesOfSeriesNumber( selectedSeries[i] );
                
            SingleVoxelMeasuredSpectralData specData =  getSVSData( selectedSeriesFileReferences.getFirst( ) );
            
            m_svsDataSeries[ i ] = specData;
            
            // whether this is correct from Philips data has to be checked!
            m_svsDataSeriesTEs[ i ] = (double)specData.getTE( )/1000.0;

            // System.out.println( "TE = " + m_svsDataSeriesTEs[ i ] );

            m_signalsTabbedPane.setSelectedIndex( 1 );
            m_xySignalsPlot.addCurve( specData.getMagnitudeCurveFDPoints( ), true );                    
         
        }

        m_xySignalsPlot.addTwoHorizontalXYPlotSliders( );
        m_xySignalsPlot.addMouseListener( m_xySignalsPlot );
        m_xySignalsPlot.addMouseMotionListener( m_xySignalsPlot );

        int[] positions = m_xySignalsPlot.getSliderPositions( );

        m_minimumSignalArrayIndex = positions[0];
        m_maximumSignalArrayIndex = positions[1];

        computeAndSetDecay( );

        Curve curve = new Curve( m_svsDataSeriesTEs, m_signalDecaySignal );
        curve.setColor( Color.BLACK );
        curve.setStyle( Curve.FAT_SOLID_LINE );

        m_xyDecayPlot.deleteAllCurves( );
        m_xyDecayPlot.addCurve( curve, true );

        if ( m_referenceSignalStatus == REFERENCE_WATER_STATUS_WATER_REF_SET_ONLY_ID )
        {
            m_referenceSignalStatus = REFERENCE_WATER_STATUS_OK_ID;
        }
        else
        {
            m_referenceSignalStatus = REFERENCE_WATER_STATUS_2COMP_SET_ONLY_ID;
        }
    }
    
    
    /**
     * This method is the interface between the old loading of data as was done
     * on Siemens data of SCANalyze and SpectrIm way of loading Siemens or 
     * Philips data; It returns a SingleVoxelMeasuredSpectralData object.
     * @param selectedSVSFileReference
     * @return 
     */
     public SingleVoxelMeasuredSpectralData getSVSData( String selectedSVSFileReference )
     {
        SingleVoxelMeasuredSpectralData svsData = null;
        
        CSIData csidata;
        DicomObject object;
        String filetype;
        String imagetype;
        String classUID;
        boolean canLoadSVS       = false;
        boolean isSiemensMRSData = false;
        
        if ( DicomHandler.isDicom( selectedSVSFileReference ) ==  true ) 
        {
            object = DicomHandler.getDicomObject(selectedSVSFileReference);
            imagetype = DicomHandler.getImageType(object);
            classUID = DicomHandler.getSOPClassUID(object);
            switch ( classUID ) 
            {
                case "1.2.840.10008.5.1.4.1.1.4": // Image
                    canLoadSVS = false;
                    break;
                case "1.2.840.10008.5.1.4.1.1.4.2": // Spectro std
                    canLoadSVS = true;
                    isSiemensMRSData = false; // this is the case of Philips data and Siemens data follwoing the Dicom standard.
                    break;
                case "1.3.12.2.1107.5.9.1": // Spectro/Tensor Siemens
                    if ( imagetype.indexOf("TENSOR") >= 0 ) 
                    {
                        canLoadSVS = false;
                    } 
                    else 
                    {
                        // we are dealing with Siemens spectroscopy data
                        canLoadSVS = true;
                        isSiemensMRSData = true;
                    }
                    break;
                case "1.2.840.10008.5.1.4.1.1.7": // Perfusion derived maps
                    canLoadSVS = false;
                    break;
                case "1.2.840.10008.5.1.4.1.1.4.1":
                    //enhanced MR image
                    canLoadSVS = false;
                    break;                        
                default: // Unknown format
                    canLoadSVS = false;
                    break;
            }
            
            DicomHeader header = DicomHandler.getHeader( object );
                
            if ( isSiemensMRSData == true ) 
            {
                DicomHandler.readSiemensData( object, header );
            }
            
            if ( canLoadSVS ) 
            {
               	csidata = new CSIData( header, selectedSVSFileReference );
                
                if ( csidata != null )
                {
            
                    Spectra spectra = csidata.getSpectra( );
                    String key = spectra.getData( ).keySet( ).iterator( ).next( );
                    
                    float[] absTD  = csidata.getSpectra( ).getSpectrumData( key, Spectra.Domain.TD, Spectra.Mode.ABS );
                    float[] dispTD = csidata.getSpectra( ).getSpectrumData( key, Spectra.Domain.TD, Spectra.Mode.DISP );
           
        
                    // Read the single voxel spectral data response
                    svsData = new SingleVoxelMeasuredSpectralData( absTD, dispTD );
                    DicomStringReformatter dformat = new DicomStringReformatter( );
            
                    svsData.setPatientName( dformat.reformatPatientName( csidata.getPatientName( ).replaceAll( "�", "_" ) ) );
                    svsData.setPatientBirthDate( dformat.getDateMonthYear( csidata.getPatientDOB( ) ) );
                    svsData.setSeriesDescription( csidata.getSeriesDescription( ) );
                    svsData.setSeriesNumber( csidata.getSeriesNumber( ) );
                    svsData.setInstanceNumber( csidata.getInstanceNumber( ) );
                    svsData.setAcquisitionDate( dformat.getDateMonthYear( csidata.getStudyDate( ) ) );
                    svsData.setTE( csidata.getTE( ) );
                    svsData.setTR( csidata.getTR( ) ); 
            
                    double[] excVoxelDimensions = csidata.csigrid.getDimsExcitedVolume( );
            
                    svsData.set_sVOIThickness( excVoxelDimensions[0] );
                    svsData.set_sVoIPhaseFOV( excVoxelDimensions[1] );
                    svsData.set_sVoIReadoutFOV( excVoxelDimensions[2] );
            
                    // m_MRSpectrumModellerPanel.setMeasuredClinicalSVSSpectrum( clinicalSpectrumData.getFile( ), m_singleVoxelClinicalData );             
           
                }
            }
        }
        
        return svsData;
     }
    
    /**
     * Computes and sets decay.
     */
    private void computeAndSetDecay( )
    {
         SingleVoxelMeasuredSpectralData svsDat = m_svsDataSeries[0];
        int numberOfPoints = svsDat.getAbsorptionChannelFD( ).length;
        
//        int minimum = (int)((float)( m_minimumSignalArrayIndex * numberOfPoints ) / (float)m_xySignalsPlot.getWidth( ) );
//        int maximum = (int)((float)( m_maximumSignalArrayIndex * numberOfPoints ) / (float)m_xySignalsPlot.getWidth( ) );
        
        m_signalDecaySignal = new double[ m_svsDataSeries.length ];
        
        for ( int i = 0; i < m_svsDataSeries.length; i++ )
        {
            svsDat = m_svsDataSeries[i];
            Curve absSignal = svsDat.getMagnitudeCurveFDPoints( );
            float[] magn = new float[ absSignal.getSize( ) ];
            
            magn = absSignal.getYvalues( );
            
            for ( int j = 0; j < absSignal.getSize( );  j++ )
            {
                m_signalDecaySignal[i] += magn[j] / numberOfPoints;
            }
        }      
    }
    
    /**
     * Adds an action listener to the list of listeners to this object.
     * @param listener
     */
    public void addActionListener( ActionListener listener )
    {
        m_actionListeners.add( listener );
    }
    
    /**
     * Notifies registered ActionListeners.
     */
    private void notifyActionListeners( int eventType, String actionCommand )
    {
        Iterator it = m_actionListeners.iterator( );
        while( it.hasNext( ) )
        {
            ( ( ActionListener )it.next( ) ).actionPerformed( new ActionEvent( this, eventType, actionCommand ) );
        }
    }

    /**
     * Returns the fit value of the long time constant.
     * @return
     */
    public float getAmplitude_A1( )
    {
        return (float)m_minPars[0];
    }

    /**
     * Returns the fit value of the long time constant.
     * @return
     */
    public float getTimeConstant_TC1( )
    {
        return (float)m_minPars[1];
    }

    /**
     * Returns the fit value of the long time constant.
     * @return
     */
    public float getAmplitude_A2( )
    {
        return (float)m_minPars[2];
    }

    /**
     * Returns the fit value of the long time constant.
     * @return
     */
    public float getTimeConstant_TC2( )
    {
        return (float)m_minPars[3];
    }

    /**
     * Returns the fit value of the long time constant.
     * @return
     */
    public double getErrorAmplitude_A1( )
    {
        return m_errorMinPars[0];
    }

    /**
     * Returns the fit value of the long time constant.
     * @return
     */
    public double getErrorTimeConstant_TC1( )
    {
        return m_errorMinPars[1];
    }

    /**
     * Returns the fit value of the long time constant.
     * @return
     */
    public double getErrorAmplitude_A2( )
    {
        return m_errorMinPars[2];
    }

    /**
     * Returns the fit value of the long time constant.
     * @return
     */
    public double getErrorTimeConstant_TC2( )
    {
        return m_errorMinPars[3];
    }


    /**
     * Returns the reference signal status.
     * @return 
     */
    public int getReferenceSignalStatus( )
    {
        return m_referenceSignalStatus;
    }

    /**
     * Returns the real part of water reference water signal
     * @return
     */
    public double[] getReferenceWaterReal( )
    {
        double[] retSig = new double[ m_svsReferenceWaterData.getAbsorptionChannelTD( ).length ];
        float[] sig = m_svsReferenceWaterData.getAbsorptionChannelTD( ); 
        
        for ( int i = 0; i < retSig.length;i++ )
        {
             retSig[i]= (double)sig[i];
        }
        
        return retSig;
    }

    /**
     * Returns the real part of water reference water signal
     * @return
     */
    public double[] getReferenceWaterImag( )
    {

        double[] retSig = new double[ m_svsReferenceWaterData.getDispersionChannelTD( ).length ];
        float[] sig = m_svsReferenceWaterData.getDispersionChannelTD( );

        for ( int i = 0; i < retSig.length;i++ )
        {
             retSig[i]= (double)sig[i];
        }

        return retSig;
    }

    /**
     * Returns the real part of water reference water signal
     * @return
     */
    public double[][] getReference2CompartmentModelWaterReal( )
    {

        double[][] returnSig = new double[m_svsDataSeries.length][m_svsDataSeries[0].getAbsorptionChannelTD( ).length];

        for ( int i = 0; i < m_svsDataSeries.length; i++ )
        {

            float[] sig = m_svsDataSeries[i].getAbsorptionChannelTD( );
            for ( int j = 0; j < sig.length; j++ )
            {
                returnSig[i][j] = (double)sig[j];
            }
            
        }

        return returnSig;
    }

    /**
     * Returns the real part of water reference water signal
     * @return
     */
    public double[][] getReference2CompartmentModelWaterImag( )
    {
        double[][] returnSig = new double[m_svsDataSeries.length][m_svsDataSeries[0].getDispersionChannelTD( ).length];

        for ( int i = 0; i < m_svsDataSeries.length; i++ )
        {

            float[] sig = m_svsDataSeries[i].getDispersionChannelTD( );
            for ( int j = 0; j < sig.length; j++ )
            {
                returnSig[i][j] = (double)sig[j];
            }

        }

        return returnSig;
    }

    /**
     *
     * Returns the number of 2 component signal files.
     *
     * @return
     */
    public int getNumberOf2ComponentWaterReferences( )
    {
        return m_numberOf2ComponentWaterReferences;
    }

    // Attributes
    private final JFrame      m_parent;
    private final JPanel      m_contentPanel;
    private final JPanel      m_seriesSelectorPanel;
    private final JPanel      m_steeringPanel;
    
    private final JSplitPane  m_horizontalSplitPane;
    private final JSplitPane  m_verticalSplitPane;
    
    private final JButton     m_loadSelectedSignalsButton;
    private final JButton     m_fitAcceptButton;
    private final JButton     m_fitCancelButton;
    
    private final XYPlot      m_xySignalsPlot;
    private final XYPlot      m_xyDecayPlot;
    private final XYPlot      m_waterReferencePlot;

    private final JTabbedPane m_signalsTabbedPane;
    
    private int         m_minimumSignalArrayIndex;
    private int         m_maximumSignalArrayIndex;
    
    private SingleVoxelMeasuredSpectralData m_svsReferenceWaterData;

    private SingleVoxelMeasuredSpectralData[]  m_svsDataSeries;
    private double      m_svsReferenceWaterTE;
    
    private double[]    m_svsDataSeriesTEs;
    private double[]    m_signalDecaySignal;
    private final TwoCompartmentWaterReferenceModelFittingMeasuredSpectrumSelectionControl m_dicomFileSelector;
    private final TwoCompartmentWaterReferenceModelFittingConditionsPanel m_fittingConditionsPanel;
    private JPanel      m_fittingConditionPanel;


    private final DicomDirectory m_dicomDir;

    // For proper serialization
    private static final long   serialVersionUID = 6482325998121123L;

    private LinkedList<ActionListener> m_actionListeners = new LinkedList<>( );

    public final static int     USE_2_COMPARTMENT_VALUES = 1;
    public final static String  USE_2COMPARTMENT_VALUES_STRING = "Use 2 compartment values";

    public final static int     FIT_STEP_PROGRESS_EVENT_ID  = 100;
    public final static String  FIT_STEP_PROGRESS_EVENT_CMD = "FIT_STEP_PROGRESS_EVENT_ID";

    public final static int     FIT_FINISHED_EVENT_ID  = 101;
    public final static String  FIT_FINISHED_EVENT_CMD = "FIT_FINISHED_EVENT_ID";

    private int m_referenceSignalStatus                     =  REFERENCE_WATER_STATUS_INITIAL_ID;
    private int m_compartmentModelFittedSignalStatus        =  COMPARTMENTS_NOT_FITTED_ID;

    public final static int     REFERENCE_WATER_STATUS_OK_ID                    = 1000;

    public final static int     REFERENCE_WATER_STATUS_INITIAL_ID               = 1001;
    public final static int     REFERENCE_WATER_STATUS_2COMP_SET_ONLY_ID        = 1002;
    public final static int     REFERENCE_WATER_STATUS_WATER_REF_SET_ONLY_ID    = 1003;

    public final static int     COMPARTMENTS_FITTED_ID                          = 2000;
    public final static int     COMPARTMENTS_NOT_FITTED_ID                      = 2001;

    public final static String  REFERENCE_WATER_STATUS_OK_CMD                   = "STATUS_OK";
    public final static String  REFERENCE_WATER_STATUS_INITIAL_CMD              = "STATUS_INITIAL";
    public final static String  REFERENCE_WATER_STATUS_2COMP_SET_ONLY_CMD       = "STATUS_2COMP_SET_ONLY";
    public final static String  REFERENCE_WATER_STATUS_WATER_REF_SET_ONLY_CMD   = "STATUS_WATER_REF_SET_ONLY";

    public final static String  COMPARTMENTS_FITTED_CMD                         = "STATUS_COMPARMENT_MODEL_FITTED";
    public final static String  COMPARTMENTS_NOT_FITTED_CMD                     = "STATUS_COMPARMENT_MODEL_NOT_FITTED";

    public double[]             m_minPars;
    public double[]             m_errorMinPars;

    private DualProgressBar     m_dualProgressBar;
    public int                  m_fittingProgressPercent = 0;
    private double              m_initialFunctionValue;
    private int                 m_numberOf2ComponentWaterReferences = 0;

    // added for small screen resolution PCs
    private final JScrollPane m_mainScrollPane;
}
