/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package org.spectrim.texturemaps;

import org.spectrim.dicom.DicomImageSeriesData;
import org.spectrim.signalprocessing.SignalTools;


/**
 *
 * @author slotboom
 */
public class ImageTextureCalculator
{
    /**
     * - noiseThresshold: level in percent below which the signal is regarded
     *                    to be pure noise
     * - maximumValThresshold: level in percent above which signal is cut
     * - xRegionSize is size over which the local texture is computed (2*xRegionSize + 1)
     * - yRegionSize is size over which the local texture is computed (2*yRegionSize + 1)
     * Computes an image texture
     */
    public ImageTextureCalculator( DicomImageSeriesData imageSeriesData, 
                                   int noiseThresshold, int maximumValThresshold,
                                   int xRegionSize, int yRegionSize )
    {
        m_imageSeriesData   = imageSeriesData;
        m_numberOfImages    = m_imageSeriesData.getNumberOfImages( );
        m_imageWidth        = m_imageSeriesData.getFileInfo( 0 ).getImageWidth( );
        m_imageHeight       = m_imageSeriesData.getFileInfo( 0 ).getImageWidth( );

        m_originalPixelData = new int[ m_numberOfImages * m_imageWidth * m_imageHeight ];
        m_texturePixelData  = new int[ m_numberOfImages * m_imageWidth * m_imageHeight ];
        m_rawImagePixelData = new int[ m_numberOfImages * m_imageWidth * m_imageHeight ];
        
        System.arraycopy( imageSeriesData.getSeriesPixelData( ), 0, m_originalPixelData,
                          0, imageSeriesData.getSeriesPixelData( ).length );
        
        System.arraycopy( imageSeriesData.getSeriesPixelData( ), 0, m_rawImagePixelData,
                          0, imageSeriesData.getSeriesPixelData( ).length );
        
        applyMinimumValueThresshold( noiseThresshold );
        applyMaximumValueThresshold( maximumValThresshold );

        m_rectHorizontalRegionSize = xRegionSize;
        m_rectVerticalRegionSize   = yRegionSize;
        
        
        int grey_levels = 16;
        m_cooc = new CooccurenceMatrix( m_numberOfImages, 
                                        m_imageWidth, m_imageHeight,
                                        m_originalPixelData, 
                                        m_rectHorizontalRegionSize, m_rectVerticalRegionSize,
                                        grey_levels );
        
        // Compute all cooccurence matrices and 
        long compTime = System.currentTimeMillis();
        //m_cooc.computeAllCooccurrenceMatrices( );
        m_cooc.computeCooccurrenceMatrices( );
        System.out.println( "Time required for computation: "+ ( System.currentTimeMillis( ) - compTime ) );

    }

    /**
     * - noiseThresshold: level in percent below which the signal is regarded
     *                    to be pure noise
     * - maximumValThresshold: level in percent above which signal is cut
     * - xRegionSize is size over which the local texture is computed (2*xRegionSize + 1)
     * - yRegionSize is size over which the local texture is computed (2*yRegionSize + 1)
     * Computes an image textures
     * 
     * Meant for time series of images.
     * 
     */
    public ImageTextureCalculator( float pixelScalingFactor, float pixelDifferenceImageScaleFactor,
                                   DicomImageSeriesData imageSeriesData,
                                   int noiseThresshold, int maximumValThresshold,
                                   int xRegionSize, int yRegionSize, 
                                   boolean computeCoocAlso, boolean computeDifferenceImages,
                                   int baseline_begin, int baseline_end, int inflow_max, int reperfusion_time )
    {
        
        m_imageSeriesData   = imageSeriesData;
        m_numberOfImages    = m_imageSeriesData.getNumberOfImages( );
        m_imageWidth        = m_imageSeriesData.getFileInfo( 0 ).getImageWidth( );
        m_imageHeight       = m_imageSeriesData.getFileInfo( 0 ).getImageWidth( );
        
        m_computeDifferenceImages = computeDifferenceImages;
        
        m_originalPixelData = new int[ m_numberOfImages * m_imageWidth * m_imageHeight ];
        
        m_pixelScalingFactor = pixelScalingFactor;
        m_pixelDifferenceImageScaleFactor = pixelDifferenceImageScaleFactor;
        
        
        m_texturePixelData  = new int[ m_numberOfImages * m_imageWidth * m_imageHeight ];
        m_rawImagePixelData = new int[ m_numberOfImages * m_imageWidth * m_imageHeight ];

        System.arraycopy( imageSeriesData.getSeriesPixelData( ), 0, m_originalPixelData,
                          0, imageSeriesData.getSeriesPixelData( ).length );
        
        System.arraycopy( imageSeriesData.getSeriesPixelData( ), 0, m_rawImagePixelData,
                          0, imageSeriesData.getSeriesPixelData( ).length );
        
        if ( m_computeDifferenceImages == true )
        {
            m_differencePixelData = new int[ m_numberOfImages * m_imageWidth * m_imageHeight ];
            m_baseLineMean = new float[ m_imageWidth * m_imageHeight ];
            int counter = 0;
            for ( int i = baseline_begin; i <= baseline_end; i++ )
            {
                for ( int j = 0; j < m_imageWidth; j++ )
                {
                    for ( int k = 0; k < m_imageHeight; k++ )
                    {
                        m_baseLineMean[ j* m_imageHeight + k ] = m_baseLineMean[ j* m_imageHeight + k ] + m_rawImagePixelData[ i * m_imageWidth * m_imageHeight + j* m_imageHeight + k ];
                    }
                }
                counter++;
            }
            
            for ( int j = 0; j < m_imageWidth; j++ )
            {
                for ( int k = 0; k < m_imageHeight; k++ )
                {
                    m_baseLineMean[ j* m_imageHeight + k ] = m_baseLineMean[ j* m_imageHeight + k ]/(float)counter;
                }
            }
        }
        
        if ( pixelScalingFactor != -1.0f )
        {
            for ( int i = 0; i < m_originalPixelData.length; i++ )
            {
                m_originalPixelData[ i ] = (int)( m_originalPixelData[ i ] );
                m_rawImagePixelData[ i ] = (int)( m_rawImagePixelData[ i ] * pixelScalingFactor );
            }
        }
        
        if ( m_computeDifferenceImages == true )
        {    
            computeDifferenceImages( pixelDifferenceImageScaleFactor );
        }
        
        applyMinimumValueThresshold( noiseThresshold );
        applyMaximumValueThresshold( maximumValThresshold );
        // applyRangeShift( );
        
        m_rectHorizontalRegionSize = xRegionSize;
        m_rectVerticalRegionSize   = yRegionSize;

        
        int grey_levels = 16;
        if ( computeDifferenceImages == false )
        {
            m_cooc = new CooccurenceMatrix( m_numberOfImages, 
                                            m_imageWidth, m_imageHeight,
                                            m_originalPixelData, 
                                            m_rectHorizontalRegionSize, m_rectVerticalRegionSize,
                                            grey_levels );
        }
        else
        {
            m_cooc = new CooccurenceMatrix( m_numberOfImages, 
                                            m_imageWidth, m_imageHeight,
                                            // m_differencePixelData, 
                                            m_originalPixelData,
                                            m_rectHorizontalRegionSize, m_rectVerticalRegionSize,
                                            grey_levels );
        }
        // Compute all cooccurence matrices and 
        // m_cooc.computeAllCooccurrenceMatrices( );
        if ( computeCoocAlso == true )
        {
            m_cooc.computeCooccurrenceMatrices( );
        }
        
    }

    /**
     * Use this method if you want to determine difference images with the
     * first image.
     */
    public void computeDifferenceImages( float scalingFactorDifferenceImageTimeSeries )
    {

        System.out.println( "computeDifferenceImages( ) called" );
        
//        int[] firstImage = new int[m_imageWidth * m_imageHeight];
//        int cntr = 0;
//
//        for ( int i = 0; i < m_imageWidth; i++ )
//        {
//            for ( int j = 0; j < m_imageHeight; j++ )
//            {
//                firstImage[cntr] = m_originalPixelData[ i * m_imageHeight + j ];
//                cntr++;
//            }
//        }

        for ( int i = 0 ; i < m_numberOfImages; i++ )
        {
            for ( int j = 0; j < m_imageWidth; j++ )
            {
                for ( int k = 0; k < m_imageHeight; k++ )
                {
                    //m_differencePixelData[ i * m_imageWidth * m_imageHeight + j* m_imageHeight + k ] = (int)( ( scalingFactorDifferenceImageTimeSeries * ( m_baseLineMean[ j * m_imageHeight + k ] - (float)m_originalPixelData[ i * m_imageWidth * m_imageHeight + j* m_imageHeight + k ] ) ) );
                    //m_differencePixelData[ i * m_imageWidth * m_imageHeight + j* m_imageHeight + k ] = m_originalPixelData[ i * m_imageWidth * m_imageHeight + j* m_imageHeight + k ];
                    if ( scalingFactorDifferenceImageTimeSeries == 1.0 )
                    {
                        m_differencePixelData[ i * m_imageWidth * m_imageHeight + j* m_imageHeight + k ] = -(int)( 1000.0 * Math.log((float)m_originalPixelData[ i * m_imageWidth * m_imageHeight + j* m_imageHeight + k ] / m_baseLineMean[ j * m_imageHeight + k ] ) );
                    }
                    else
                    {
                        m_differencePixelData[ i * m_imageWidth * m_imageHeight + j* m_imageHeight + k ] = -(int)( scalingFactorDifferenceImageTimeSeries * 1000.0 * Math.log((float)m_originalPixelData[ i * m_imageWidth * m_imageHeight + j* m_imageHeight + k ] / m_baseLineMean[ j * m_imageHeight + k ] ) );
                    }
                }
            }
        }
    }

     /**
     * Use this method if you want to determine difference images with the
     * first image.
     */
    public void computeDifferenceImages( )
    {

        System.out.println( "computeDifferenceImages( ) called" );

        int[] firstImage = new int[m_imageWidth * m_imageHeight];
        int cntr = 0;

        for ( int i = 0; i < m_imageWidth; i++ )
        {
            for ( int j = 0; j < m_imageHeight; j++ )
            {
                firstImage[cntr] = m_rawImagePixelData[ i * m_imageHeight + j ];
                cntr++;
            }
        }

        for ( int i = 0 ; i < m_numberOfImages; i++ )
        {
            for ( int j = 0; j < m_imageWidth; j++ )
            {
                for ( int k = 0; k < m_imageHeight; k++ )
                {
                    m_differencePixelData[ i * m_imageWidth * m_imageHeight + j * m_imageHeight + k ] = ( firstImage[ j* m_imageHeight + k ] - m_rawImagePixelData[ i * m_imageWidth * m_imageHeight + j * m_imageHeight + k ] );
                }
            }
        }
    }


    /**
     * Noise level in percent of maximum value
     * @param noiseLevel
     */
    private void applyMinimumValueThresshold( int noiseLevel )
    {
        int maxVal = SignalTools.getMaximum( m_rawImagePixelData );
        for( int i = 0; i < m_rawImagePixelData.length; i++ )
        {
            if ( m_rawImagePixelData[i] < noiseLevel * maxVal / 100  )
            {
                m_originalPixelData[i] = 0;
                m_rawImagePixelData[i] = 0;
                if ( m_computeDifferenceImages == true )
                {
                    m_differencePixelData[i] = 0;
                }
            }
        }
    }

    /**
     * Noise level in percent of maximum value
     * @param noiseLevel
     */
    private void applyMaximumValueThresshold( int maxPixelValueLevel )
    {
        int maxVal = SignalTools.getMaximum( m_originalPixelData );
        for ( int i = 0; i < m_originalPixelData.length; i++ )
        {
            if ( m_originalPixelData[i] > maxPixelValueLevel * maxVal / 100  )
            {
                m_originalPixelData[i] = 0;
                m_rawImagePixelData[i] = 0;
                if ( m_computeDifferenceImages == true )
                {
                    m_differencePixelData[i] = 0;
                }
            }
        }
    }

    /**
     * Applies a range shift, i.e. the minimum data value of all image pixels
     * are set to 0.
     * @param noiseLevel
     */
    private void applyRangeShift( )
    {
        
        int maxVal = SignalTools.getMaximum( m_originalPixelData );
        
        for ( int i = 0; i < m_originalPixelData.length; i++ )
        {
            m_originalPixelData[i] = -1 * ( m_originalPixelData[i] - (int)Math.abs( maxVal ) );
        }
        
    }

    /**
     * Computes the texture belonging to texture type
     * @return
     */
    public int[] computeTexture( String textureType )
    {
        int[] zeroes = new int[m_rawImagePixelData.length];
        
        m_texturePixelData = null;
        
        // the HISTOGRAM base ones.
        
        if ( textureType.compareTo( ORIGINAL_IMAGE_DATA_NAME ) == 0 )
        {
            return m_rawImagePixelData;
        }

        if ( textureType.compareTo( RAW_DIFFERENCE_NAME ) == 0 )
        {
            m_texturePixelData = computeTextureRaw( );
            return m_texturePixelData;
        }

        if ( textureType.compareTo( RECT_REGION_AVERAGE_NAME ) == 0 )
        {
            computeTextureRectRegionAverage( );
        }

        if ( textureType.compareTo( RECT_REGION_VARIANCE_NAME ) == 0 )
        {
            computeTextureRectRegionVariance( );
        }

        if ( textureType.compareTo( RECT_REGION_STANDARD_DEVIATION_NAME ) == 0 )
        {
            computeTextureRectRegionStandardDeviation( );
        }

        if ( textureType.compareTo( RECT_REGION_SKEWNESS_NAME ) == 0 )
        {
            computeTextureRectRegionSkewness( );
        }

        if ( textureType.compareTo( RECT_REGION_KURTOSIS_NAME ) == 0 )
        {
            computeTextureRectRegionKurtosis( );
        }

        if ( textureType.compareTo( RECT_REGION_VARIANCEOFVARIANCE_NAME ) == 0 )
        {
            computeTextureRectRegionVarianceOfVariance( );
        }
        
        // the GLCM based texture maps
        if ( m_cooc.isSuccessfullyComputed( ) == true )
        {
            
            if ( textureType.compareTo( RECT_GLCM_CONTRAST_NAME ) == 0 )
            {
                return m_cooc.getGLCM_CONtrastAsScaledIntArray( );   
            }
        
            if ( textureType.compareTo( RECT_GLCM_DISSIMILARITY_NAME ) == 0 )
            {
                return m_cooc.getGLCM_DISsimilarityAsScaledIntArray( );
            }
        
            if ( textureType.compareTo( RECT_GLCM_HOMOGENEITY_NAME ) == 0 )
            {
                return m_cooc.getGLCM_HOMogeneityAsScaledIntArray( );
            }
        
            if ( textureType.compareTo( RECT_GLCM_ANGULAR_SECOND_MOMENT_ASM_NAME ) == 0 )
            {
                return m_cooc.getGLCM_AngularSecondMoment_ASM_AsScaledIntArray( );
            }
        
            if ( textureType.compareTo( RECT_GLCM_ENERGY_ENE_NAME ) == 0 )
            {
                return m_cooc.getGLCM_Energy_AsScaledIntArray( );
            }
        
            if ( textureType.compareTo( RECT_GLCM_MAXIMUM_PROBABILITY_MAX_NAME ) == 0 )
            {
                return m_cooc.getGLCM_MAXimumProbability_AsScaledIntArray( );
            }
        
            if ( textureType.compareTo( RECT_GLCM_ENTROPY_EN_NAME ) == 0 )
            {
                return m_cooc.getGLCM_ENTropy_AsScaledIntArray( );
            }
        
            if ( textureType.compareTo( RECT_GLCM_MEAN_NAME ) == 0 )
            {
                return m_cooc.getGLCM_Mean_AsScaledIntArray( );
            }
        
            if ( textureType.compareTo( RECT_GLCM_VARIANCE_NAME ) == 0 )
            {
                return m_cooc.getGLCM_Variance_AsScaledIntArray( );
            }
        
            if ( textureType.compareTo( RECT_GLCM_CORRELATION_NAME ) == 0 )
            {
                return m_cooc.getGLCM_Correlation_AsScaledIntArray( );
            }
        }
        else
        {
            if ( m_texturePixelData == null )
            {
                m_texturePixelData = new int[m_rawImagePixelData.length];
                System.arraycopy( zeroes, 0, m_texturePixelData,
                                  0, m_texturePixelData.length );
            }
        }
        
        return m_texturePixelData;

    }

    /**
     *
     * Computes the raw data texture map: trivial case
     * Raw data is for perfusiuon (or time series case) the difference image
     * series.
     * @param imagePixelData
     * @return
     */
    public int[] computeTextureRaw( )
    {
        m_texturePixelData = new int[m_originalPixelData.length];
        
        if ( m_computeDifferenceImages == false )
        {
            System.arraycopy( m_originalPixelData, 0, m_texturePixelData,
                              0, m_originalPixelData.length );
        }
        else
        {
            System.arraycopy( m_differencePixelData, 0, m_texturePixelData,
                              0, m_originalPixelData.length );
        }
        
        return m_texturePixelData;
    }

    /**
     *
     * Computes the local mean of a rectangular region texture
     *
     * @param pixelData
     * @return
     */
    public int[] computeTextureRectRegionAverage( )
    {

        m_texturePixelData = new int[m_originalPixelData.length];

        if ( m_localStatisticsIsComputed == false )
        {
            computeLocalStatisics( m_rectHorizontalRegionSize, m_rectVerticalRegionSize );
        }

        for ( int i = 0; i < m_localMeanValues.length; i++ )
        {
            m_texturePixelData[i] = (int)( m_localMeanValues[i] + 0.5f );
        }

        return m_texturePixelData;

    }

    /**
     *
     * Computes the local variance of a rectangular region texture
     *
     * @param pixelData
     * @return
     */
    public int[] computeTextureRectRegionVariance( )
    {

        m_texturePixelData = new int[m_originalPixelData.length];

        if ( m_localStatisticsIsComputed == false )
        {
            computeLocalStatisics( m_rectHorizontalRegionSize, m_rectVerticalRegionSize );
        }

        for ( int i = 0; i < m_localMeanValues.length; i++ )
        {
            m_texturePixelData[i] = (int)( m_localVarianceValues[i] + 0.5f );
        }

        return m_texturePixelData;

    }


    /**
     *
     * Computes the local variance of a rectangular region texture
     *
     * @param pixelData
     * @return
     */
    public int[] computeTextureRectRegionStandardDeviation( )
    {

        m_texturePixelData = new int[m_originalPixelData.length];

        if ( m_localStatisticsIsComputed == false )
        {
            computeLocalStatisics( m_rectHorizontalRegionSize, m_rectVerticalRegionSize );
        }

        for ( int i = 0; i < m_localMeanValues.length; i++ )
        {
            m_texturePixelData[i] = (int)( Math.sqrt( m_localVarianceValues[i] ) + 0.5f );
        }

        return m_texturePixelData;

    }


    /**
     *
     * Computes the local variance of a rectangular region texture
     *
     * @param pixelData
     * @return
     */
    public int[] computeTextureRectRegionVarianceOfVariance( )
    {

        m_texturePixelData = new int[m_originalPixelData.length];

        if ( m_localStatisticsIsComputed == false )
        {
            computeLocalStatisics( m_rectHorizontalRegionSize, m_rectVerticalRegionSize );
        }

        for ( int i = 0; i < m_localMeanValues.length; i++ )
        {
            m_texturePixelData[i] = (int)( m_localVarianceOfVarianceValues[i] + 0.5f );
        }

        return m_texturePixelData;

    }


    /**
     *
     * Computes the local skewness of a rectangular region texture
     *
     * @param pixelData
     * @return
     */
    public int[] computeTextureRectRegionSkewness( )
    {

        m_texturePixelData = new int[m_originalPixelData.length];

        if ( m_localStatisticsIsComputed == false )
        {
            computeLocalStatisics( m_rectHorizontalRegionSize, m_rectVerticalRegionSize );
        }

        for ( int i = 0; i < m_localMeanValues.length; i++ )
        {
            m_texturePixelData[i] = (int)(10000.0 * ( m_localSkewnessValues[i] ) );
        }

        return m_texturePixelData;

    }

    /**
     *
     * Computes the local skewness of a rectangular region texture
     *
     * @param pixelData
     * @return
     */
    public int[] computeTextureRectRegionKurtosis( )
    {

        m_texturePixelData = new int[m_originalPixelData.length];

        if ( m_localStatisticsIsComputed == false )
        {
            computeLocalStatisics( m_rectHorizontalRegionSize, m_rectVerticalRegionSize );
        }

        for ( int i = 0; i < m_localMeanValues.length; i++ )
        {
            m_texturePixelData[i] = (int)( 1000.0f * m_localKurtosisValues[i] + 0.5f );
        }

        return m_texturePixelData;

    }


    /**
     * Computes the local statistics of the image stack!
     */
    public void computeLocalStatisics( int xRegionSize, int yRegionSize )
    {
            
        m_localMeanValues               = new float[m_numberOfImages * m_imageWidth * m_imageHeight];
        m_localVarianceValues           = new float[m_numberOfImages * m_imageWidth * m_imageHeight];
        m_localSkewnessValues           = new float[m_numberOfImages * m_imageWidth * m_imageHeight];
        m_localKurtosisValues           = new float[m_numberOfImages * m_imageWidth * m_imageHeight];
        m_localVarianceOfVarianceValues = new float[m_numberOfImages * m_imageWidth * m_imageHeight];
        
        if ( this.m_computeDifferenceImages == false )
        {
            
            for ( int ima = 0; ima < m_numberOfImages; ima++ )
            {
                for ( int i = 0 ; i < m_imageHeight ; i++ )
                {
                    for ( int j = 0 ; j < m_imageWidth; j++ )
                    {

                        // current coordinate
                        int coord = i * m_imageWidth + j;
                        float[] localImageValues = new float[ ( 2 * xRegionSize + 1 ) * ( 2 * yRegionSize + 1 ) ];
                        int cntr = 0;
                        for ( int k = j - yRegionSize ; k <= j + yRegionSize; k++ )
                        {
                            for ( int l = i - xRegionSize ; l <= i + xRegionSize; l++ )
                            {
                                int ii = ( k + m_imageWidth ) % m_imageWidth;
                                int jj = ( l + m_imageHeight ) % m_imageHeight;
                                localImageValues[cntr] = m_originalPixelData[  ima * m_imageWidth * m_imageHeight + ii + m_imageWidth * jj ];
                                cntr++;
                            }
                        }

                        MomentAboutTheMeanOfArrayCalculator momCalc = new MomentAboutTheMeanOfArrayCalculator( localImageValues );
                        MomentsAboutTheMean moments = momCalc.getMomentsAboutTheMean( );

                        m_localMeanValues[ ima * m_imageWidth * m_imageHeight + m_imageWidth * i + j ] = (float)moments.getMean( );

                        if ( (float)moments.getMean( ) > m_localMeanMaximumValue )
                        {
                            m_localMeanMaximumValue = (float)moments.getMean( );
                        }
                        if ( (float)moments.getMean( ) < m_localMeanMinimumValue )
                        {
                            m_localMeanMinimumValue = (float)moments.getMean( );
                        }

                        m_localVarianceValues[ ima * m_imageWidth * m_imageHeight + m_imageWidth * i + j ] = (float)moments.getVariance( );

                        if ( (float)moments.getVariance( ) > m_localVarianceMaximumValue )
                        {
                            m_localVarianceMaximumValue = (float)moments.getVariance( );
                        }
                        if ( (float)moments.getVariance( ) < m_localVarianceMinimumValue )
                        {
                            m_localVarianceMinimumValue = (float)moments.getVariance( );
                        }

                        m_localSkewnessValues[ ima * m_imageWidth * m_imageHeight + m_imageWidth * i + j ] = (float)moments.getSkewness( );

                        if ( (float)moments.getSkewness( ) > m_localSkewnessMaximumValue )
                        {
                            m_localSkewnessMaximumValue = (float)moments.getSkewness( );
                        }
                        if ( (float)moments.getSkewness( ) < m_localSkewnessMinimumValue )
                        {
                            m_localSkewnessMinimumValue = (float)moments.getSkewness( );
                        }

                        m_localKurtosisValues[ ima * m_imageWidth * m_imageHeight + m_imageWidth * i + j ] = (float)moments.getKurtosis( );

                        if ( (float)moments.getKurtosis( ) > m_localKurtosisMaximumValue )
                        {
                            m_localKurtosisMaximumValue = (float)moments.getKurtosis( );
                        }
                        if ( (float)moments.getKurtosis( ) < m_localKurtosisMinimumValue )
                        {
                            m_localKurtosisMinimumValue = (float)moments.getKurtosis( );
                        }

                        m_localVarianceOfVarianceValues[ ima * m_imageWidth * m_imageHeight + m_imageWidth * i + j ] = (float)moments.getVarianceOfVariance( );

                        if ( (float)moments.getVariance( ) > m_localVarianceOfVarianceMaximumValue )
                        {
                            m_localVarianceOfVarianceMaximumValue = (float)moments.getVarianceOfVariance( );
                        }

                        if ( (float)moments.getVariance( ) < m_localVarianceOfVarianceMinimumValue )
                        {
                            m_localVarianceOfVarianceMinimumValue = (float)moments.getVarianceOfVariance( );
                        }
                    }
                }
            }
        }
        else
        {
            for ( int ima = 0; ima < m_numberOfImages; ima++ )
            {
                for ( int i = 0 ; i < m_imageHeight ; i++ )
                {
                    for ( int j = 0 ; j < m_imageWidth; j++ )
                    {

                        // current coordinate
                        int coord = i * m_imageWidth + j;
                        float[] localImageValues = new float[ ( 2 * xRegionSize + 1 ) * ( 2 * yRegionSize + 1 ) ];
                        int cntr = 0;
                        for ( int k = j - yRegionSize ; k <= j + yRegionSize; k++ )
                        {
                            for ( int l = i - xRegionSize ; l <= i + xRegionSize; l++ )
                            {
                                int ii = ( k + m_imageWidth ) % m_imageWidth;
                                int jj = ( l + m_imageHeight ) % m_imageHeight;
                                localImageValues[cntr] = m_differencePixelData[  ima * m_imageWidth * m_imageHeight + ii + m_imageWidth * jj ];
                                cntr++;
                            }
                        }

                        MomentAboutTheMeanOfArrayCalculator momCalc = new MomentAboutTheMeanOfArrayCalculator( localImageValues );
                        MomentsAboutTheMean moments = momCalc.getMomentsAboutTheMean( );

                        m_localMeanValues[ ima * m_imageWidth * m_imageHeight + m_imageWidth * i + j ] = (float)moments.getMean( );

                        if ( (float)moments.getMean( ) > m_localMeanMaximumValue )
                        {
                            m_localMeanMaximumValue = (float)moments.getMean( );
                        }
                        if ( (float)moments.getMean( ) < m_localMeanMinimumValue )
                        {
                            m_localMeanMinimumValue = (float)moments.getMean( );
                        }

                        m_localVarianceValues[ ima * m_imageWidth * m_imageHeight + m_imageWidth * i + j ] = (float)moments.getVariance( );

                        if ( (float)moments.getVariance( ) > m_localVarianceMaximumValue )
                        {
                            m_localVarianceMaximumValue = (float)moments.getVariance( );
                        }
                        if ( (float)moments.getVariance( ) < m_localVarianceMinimumValue )
                        {
                            m_localVarianceMinimumValue = (float)moments.getVariance( );
                        }

                        m_localSkewnessValues[ ima * m_imageWidth * m_imageHeight + m_imageWidth * i + j ] = (float)moments.getSkewness( );

                        if ( (float)moments.getSkewness( ) > m_localSkewnessMaximumValue )
                        {
                            m_localSkewnessMaximumValue = (float)moments.getSkewness( );
                        }
                        if ( (float)moments.getSkewness( ) < m_localSkewnessMinimumValue )
                        {
                            m_localSkewnessMinimumValue = (float)moments.getSkewness( );
                        }

                        m_localKurtosisValues[ ima * m_imageWidth * m_imageHeight + m_imageWidth * i + j ] = (float)moments.getKurtosis( );

                        if ( (float)moments.getKurtosis( ) > m_localKurtosisMaximumValue )
                        {
                            m_localKurtosisMaximumValue = (float)moments.getKurtosis( );
                        }
                        if ( (float)moments.getKurtosis( ) < m_localKurtosisMinimumValue )
                        {
                            m_localKurtosisMinimumValue = (float)moments.getKurtosis( );
                        }

                        m_localVarianceOfVarianceValues[ ima * m_imageWidth * m_imageHeight + m_imageWidth * i + j ] = (float)moments.getVarianceOfVariance( );

                        if ( (float)moments.getVariance( ) > m_localVarianceOfVarianceMaximumValue )
                        {
                            m_localVarianceOfVarianceMaximumValue = (float)moments.getVarianceOfVariance( );
                        }

                        if ( (float)moments.getVariance( ) < m_localVarianceOfVarianceMinimumValue )
                        {
                            m_localVarianceOfVarianceMinimumValue = (float)moments.getVarianceOfVariance( );
                        }
                    }
                }
            }
        }
        
        m_localStatisticsIsComputed = true;
        
    }


    // Attributes basic
    private DicomImageSeriesData m_imageSeriesData;
    private int[] m_rawImagePixelData;

    private int[] m_originalPixelData;
    private int[] m_differencePixelData;
    
    private float[] m_baseLineMean;
    
    private int[] m_texturePixelData;

    private int m_numberOfImages ;
    private int m_imageWidth;
    private int m_imageHeight;

    private int m_rectHorizontalRegionSize = 2;
    private int m_rectVerticalRegionSize   = 2;

    private float[] m_localMeanValues;
    private float[] m_localVarianceValues;
    private float[] m_localSkewnessValues;
    private float[] m_localKurtosisValues;
    private float[] m_localVarianceOfVarianceValues;

    private boolean m_localStatisticsIsComputed = false;
     // Texture types - subtype statistical
    public final static int ORIGINAL_IMAGE_DATA             = 0;
    public final static int RAW_DIFFERENCE                  = 1;
    public final static int RECT_REGION_AVERAGE             = 2;
    public final static int RECT_REGION_VARIANCE            = 3;
    public final static int RECT_REGION_STDEV               = 4;
    public final static int RECT_REGION_SKEWNESS            = 5;
    public final static int RECT_REGION_KURTOSIS            = 6;
    public final static int RECT_REGION_VARIANCEOFVARIANCE  = 7;

    // Names of statistical parameters
    public final static String ORIGINAL_IMAGE_DATA_NAME             = "Original image data";
    public final static String RAW_DIFFERENCE_NAME                  = "Raw difference";
    public final static String RECT_REGION_AVERAGE_NAME             = "Region average";
    public final static String RECT_REGION_VARIANCE_NAME            = "Region variance";
    public final static String RECT_REGION_STANDARD_DEVIATION_NAME  = "Region standard deviation";
    public final static String RECT_REGION_SKEWNESS_NAME            = "Region skewness";
    public final static String RECT_REGION_KURTOSIS_NAME            = "Region kurtosis";
    public final static String RECT_REGION_VARIANCEOFVARIANCE_NAME  = "Region variance of variance";

    public final static String RECT_GLCM_CONTRAST_NAME              = "GLCM CON contrast";
    public final static String RECT_GLCM_DISSIMILARITY_NAME         = "GLCM DIS dissimilarity";
    public final static String RECT_GLCM_HOMOGENEITY_NAME           = "GLCM HOM homogeneity";
    public final static String RECT_GLCM_ANGULAR_SECOND_MOMENT_ASM_NAME = "GLCM ASM angular second moment";
    public final static String RECT_GLCM_ENERGY_ENE_NAME            = "GLCM ENE energy";
    public final static String RECT_GLCM_MAXIMUM_PROBABILITY_MAX_NAME = "GLCM MAX maximum probability";
    public final static String RECT_GLCM_ENTROPY_EN_NAME              = "GLCM ENT entropy";
    public final static String RECT_GLCM_MEAN_NAME                    = "GLCM MEAN";
    public final static String RECT_GLCM_VARIANCE_NAME                = "GLCM VARIANCE";
    public final static String RECT_GLCM_CORRELATION_NAME             = "GLCM CORRELATION";
    
    
    private float m_localMeanMinimumValue = Float.MAX_VALUE;
    private float m_localMeanMaximumValue = -Float.MAX_VALUE;


    private float m_localVarianceMinimumValue = Float.MAX_VALUE;
    private float m_localVarianceMaximumValue = -Float.MAX_VALUE;

    private float m_localSkewnessMinimumValue = Float.MAX_VALUE;
    private float m_localSkewnessMaximumValue = -Float.MAX_VALUE;

    private float m_localKurtosisMinimumValue = Float.MAX_VALUE;
    private float m_localKurtosisMaximumValue = -Float.MAX_VALUE;

    private float m_localVarianceOfVarianceMinimumValue = Float.MAX_VALUE;
    private float m_localVarianceOfVarianceMaximumValue = -Float.MAX_VALUE;

    private float m_pixelScalingFactor = 1f;
    private float m_pixelDifferenceImageScaleFactor = 1f;
    
    private CooccurenceMatrix m_cooc = null;
    
    private boolean m_computeDifferenceImages = false;
    
}
