/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package org.spectrim.toolbars;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.util.Iterator;
import java.util.LinkedList;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JToolBar;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;



/**
 *
 * Tool bar for basic DICOM file oriented operations.
 *
 * @author slotboom
 */
public class DicomToolBar extends JPanel implements ActionListener
{

   
    public DicomToolBar( )
    {
        super( new BorderLayout( ) );

        //Create the toolbar.
        JToolBar toolBar = new JToolBar( "Dicom ToolBar" );
        addButtons( toolBar );

        add( toolBar, BorderLayout.PAGE_START );
        
        setVisible( true );
    }

    /**
     * Add the buttons to the tool bar.
     * @param toolBar
     */
    protected void addButtons( JToolBar toolBar )
    {
        JButton button = null;

        //first button
        button = makeDicomToolBarButton( "MountDicomDirectory24", MOUNT_DICOM_DIRECTORY,
                                         "Mount DICOM file system directory",
                                         "MountDicomDir" );
        toolBar.add( button );
       
        //second button
        button = makeDicomToolBarButton( "DicomNetworkNodes24", DICOM_NETWORK_NODE_PROPERTIES,
                                         "Define DICOM network node properties",
                                         "DefineDicomNetworkNodes");
        toolBar.add( button );
        button.setEnabled( false );
        //third button
        button = makeDicomToolBarButton( "DicomListener24", DICOM_NETWORK_LISTENER,
                                         "Configure and start DICOM listener",
                                         "ConfigureAndStartDicomListener" );
        toolBar.add( button );
        button.setEnabled( false );

        //fourth button
        button = makeDicomToolBarButton( "DicomizeScreenshot24", DICOMIZE_SCREENSHOT,
                                         "DICOM-ize screenshot of current display",
                                         "DICOMIZE_SCREENSHOT" );
        button.setEnabled( false );
        toolBar.add( button );

        //fifth button
        button = makeDicomToolBarButton( "DicomizeNonDicomImage24", DICOMIZE_NON_DICOM_IMAGE,
                                         "DICOM-ize non-dicom image",
                                         "DICOMIZE_NON_DICOM_IMAGE" );
        button.setEnabled( false );
        toolBar.add( button );
    }


    /**
     *
     * Creates all buttons of the tool bar.
     *
     * @param imageName
     * @param actionCommand
     * @param toolTipText
     * @param altText
     * @return the button to add
     */
    protected JButton makeDicomToolBarButton( String imageName,
                                              String actionCommand,
                                              String toolTipText,
                                              String altText )
    {
        // Look for the image.
        String workingDir = System.getProperty( "user.dir" );
        String imgLocation = workingDir + System.getProperty( "file.separator" )
                                        + "images"
                                        + System.getProperty( "file.separator" )
                                        + imageName
                                        + ".gif";
        File imageFile = new File( imgLocation );

        //Create and initialize the button.
        JButton button = new JButton( );
        button.setActionCommand( actionCommand );
        button.setToolTipText( toolTipText );
        button.addActionListener( this );

        if ( imageFile.exists( ) )
        {                      //image found
            button.setIcon( new ImageIcon( imgLocation ) );
        }
        else
        {                        //no image found
            button.setText( altText );
            System.err.println( "Resource not found: "
                               + imgLocation );
        }

        return button;
    }


    /**
     *
     * Is called whenever a registered component raises an ActionEvent
     *
     * @param arg0
     */
    public void actionPerformed( ActionEvent e )
    {
        String cmd = e.getActionCommand( );
        String description = null;

        // Handle each button.
        if ( DICOM_NETWORK_LISTENER.equals( cmd ) )
        { //first button clicked
           notifyActionListeners( DICOM_NETWORK_LISTENER, DICOM_NETWORK_LISTENER_ID );
        }
        else if ( MOUNT_DICOM_DIRECTORY.equals( cmd ) )
        {   // second button clicked
            notifyActionListeners( MOUNT_DICOM_DIRECTORY, MOUNT_DICOM_DIRECTORY_ID );
        }
        else if ( DICOM_NETWORK_NODE_PROPERTIES.equals( cmd ) )
        {   // third button clicked
            notifyActionListeners( DICOM_NETWORK_NODE_PROPERTIES, DICOM_NETWORK_NODE_PROPERTIES_ID );
        }
        else if ( DICOMIZE_SCREENSHOT.equals( cmd ) )
        {   // fourth button clicked
            notifyActionListeners( DICOMIZE_SCREENSHOT, DICOMIZE_SCREENSHOT_ID );
        }
        else if ( DICOMIZE_NON_DICOM_IMAGE.equals( cmd ) )
        {   // fifth button clicked
            notifyActionListeners( DICOMIZE_NON_DICOM_IMAGE, DICOMIZE_NON_DICOM_IMAGE_ID );
        }
    }

    /**
     * Create the GUI and show it.  For thread safety,
     * this method should be invoked from the
     * event dispatch thread.
     */
    private static void createAndShowGUI( )
    {
        //Create and set up the window.
        JFrame frame = new JFrame( "DicomToolBar" );
        frame.setDefaultCloseOperation( JFrame.EXIT_ON_CLOSE );

        //Add content to the window.
        frame.add( new DicomToolBar( ) );

        //Display the window.
        frame.pack( );
        frame.setVisible( true );
    }

        /**
     * Notifies actionEvent to all registered listeners
     */
    private void notifyActionListeners( String actionCommand, int actionID )
    {
        Iterator it = m_actionListeners.iterator( );
        while( it.hasNext( ) )
        {
            ( ( ActionListener )it.next( ) ).actionPerformed( new ActionEvent( this, actionID, actionCommand ) );
        }
    }

    /**
     * Adds an actionlistener to the list of listeners
     */
    public void addActionListener( ActionListener e )
    {
        m_actionListeners.add( e );
    }


    public static void main( String[] args )
    {
        //Schedule a job for the event dispatch thread:
        //creating and showing this application's GUI.
        SwingUtilities.invokeLater( new Runnable( )
        {
            public void run( )
            {
                //Turn off metal's use of bold fonts
                UIManager.put( "swing.boldMetal", Boolean.FALSE );
                createAndShowGUI( );
            }
        });
    }

    static final public String MOUNT_DICOM_DIRECTORY     = "Mount DICOM file system directory";
    static final public String DICOM_NETWORK_NODE_PROPERTIES    = "Define DICOM network node properties";
    static final public String DICOM_NETWORK_LISTENER = "Configure and start DICOM listener";
    static final public String DICOMIZE_SCREENSHOT    = "DICOM-ize screenshot of current display";
    static final public String DICOMIZE_NON_DICOM_IMAGE = "DICOM-ize non-dicom image";

    static final public int MOUNT_DICOM_DIRECTORY_ID            = 0;
    static final public int DICOM_NETWORK_NODE_PROPERTIES_ID    = 1;
    static final public int DICOM_NETWORK_LISTENER_ID           = 2;
    static final public int DICOMIZE_SCREENSHOT_ID              = 3;
    static final public int DICOMIZE_NON_DICOM_IMAGE_ID         = 4;

    LinkedList<ActionListener> m_actionListeners = new LinkedList<ActionListener>( );
}