/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package org.spectrim.view;

import org.spectrim.imagegraphicobjects.Point3D;
import java.awt.Image;
import java.awt.Point;
import java.awt.Polygon;
import java.awt.image.BufferedImage;
import java.awt.image.Raster;
import java.awt.image.WritableRaster;
import java.io.IOException;
import java.util.HashMap;
import org.dcm4che2.data.Tag;
import org.dcm4che2.util.TagUtils;
import org.spectrim.grid.PrecisionPolygon;
import org.spectrim.io.dicom.DicomHeader;
import org.spectrim.io.dicom.DicomHeaderLine;

/**
 *
 * @author nunopbarros
 */
public class ImageSlice {

    public Raster image;
    public Point3D imagePosition;
    public Point3D imageOrientationX;
    public Point3D imageOrientationY;
    public float pixelSpacingX;
    public float pixelSpacingY;
    public float sliceThickness;
    public int sizeX;
    public int sizeY;
    private double[] maxIntensity = new double[]{Double.NEGATIVE_INFINITY, Double.NEGATIVE_INFINITY, Double.NEGATIVE_INFINITY};
    private double[] minIntensity = new double[]{Double.POSITIVE_INFINITY, Double.POSITIVE_INFINITY, Double.POSITIVE_INFINITY};
    public String file;
    public enum Orientation {TRANS,COR,SAG};

    public ImageSlice(Raster image, DicomHeader header, String file) {
        DicomHeaderLine headerLine;
        this.image = image;
        headerLine = (DicomHeaderLine) header.get(TagUtils.toString(Tag.ImageOrientationPatient));
        if (headerLine == null) {
            headerLine = (DicomHeaderLine) header.get("(0020,9116)->(0020,0037)");
        }
        String[] parts = headerLine.Value.split("\\\\");
        imageOrientationY = new Point3D();
        imageOrientationX = new Point3D();
        imageOrientationY.m_x3 = Double.valueOf(parts[5]);
        imageOrientationY.m_x2 = Double.valueOf(parts[4]);
        imageOrientationY.m_x1 = Double.valueOf(parts[3]);
        imageOrientationX.m_x3 = Double.valueOf(parts[2]);
        imageOrientationX.m_x2 = Double.valueOf(parts[1]);
        imageOrientationX.m_x1 = Double.valueOf(parts[0]);
        headerLine = (DicomHeaderLine) header.get(TagUtils.toString(Tag.ImagePositionPatient));
        if (headerLine == null) {
            headerLine = (DicomHeaderLine) header.get("(0020,9113)->(0020,0032)");
        }        
        parts = headerLine.Value.split("\\\\");
        imagePosition = new Point3D(Double.valueOf(parts[0]), Double.valueOf(parts[1]), Double.valueOf(parts[2]));
        headerLine = (DicomHeaderLine) header.get(TagUtils.toString(Tag.Columns));
        sizeX = Integer.valueOf(headerLine.Value);
        headerLine = (DicomHeaderLine) header.get(TagUtils.toString(Tag.Rows));
        sizeY = Integer.valueOf(headerLine.Value);
        headerLine = (DicomHeaderLine) header.get(TagUtils.toString(Tag.SliceThickness));
        if (headerLine == null) {
            headerLine = (DicomHeaderLine) header.get(TagUtils.toString(Tag.SpacingBetweenSlices));
        }
        sliceThickness = Float.valueOf(headerLine.Value);
        headerLine = (DicomHeaderLine) header.get(TagUtils.toString(Tag.PixelSpacing));
        if (headerLine == null) {
            headerLine = (DicomHeaderLine) header.get("(0028,9110)->(0028,0030)");
        }        
        parts = headerLine.Value.split("\\\\");
        pixelSpacingX = Float.valueOf(parts[0]);
        pixelSpacingY = Float.valueOf(parts[1]);
        this.file = file;
    }

    public ImageSlice(Point3D dirX, Point3D dirY, Point3D pos, int sizex, int sizey, float spacingx, float spacingy) {
//        Image = new BufferedImage(sizex,sizey,BufferedImage.TYPE_INT_RGB);
//        Image = Raster.createPackedRaster( java.awt.image.DataBuffer.TYPE_INT, sizex, sizey, 1, null);
//        Image = Raster.createBandedRaster(java.awt.image.DataBuffer.TYPE_INT, sizex, sizey, 1, null);
        image = new BufferedImage(sizex, sizey, BufferedImage.TYPE_USHORT_GRAY).getRaster();
        imageOrientationY = dirY;
        imageOrientationX = dirX;
        imagePosition = pos;
        sizeX = sizex;
        sizeY = sizey;
        pixelSpacingX = spacingx;
        pixelSpacingY = spacingy;
    }

    public ImageSlice(Point3D dirX, Point3D dirY, Point3D pos, int sizex, int sizey, float spacingx, float spacingy, Raster image, String file) {
        this.image = image;
        imageOrientationY = dirY;
        imageOrientationX = dirX;
        imagePosition = pos;
        sizeX = sizex;
        sizeY = sizey;
        pixelSpacingX = spacingx;
        pixelSpacingY = spacingy;
    }

    public ImageSlice(Raster image, MultiFrameParameters mfp, String file, int index) throws IOException {
        //image
        this.image = image;

        //orientation
        String orientation = mfp.getImageOrientationPatient(index);
        String[] parts = orientation.split("\\\\");
        imageOrientationY = new Point3D();
        imageOrientationX = new Point3D();
        imageOrientationY.m_x3 = Double.valueOf(parts[5]);
        imageOrientationY.m_x2 = Double.valueOf(parts[4]);
        imageOrientationY.m_x1 = Double.valueOf(parts[3]);
        imageOrientationX.m_x3 = Double.valueOf(parts[2]);
        imageOrientationX.m_x2 = Double.valueOf(parts[1]);
        imageOrientationX.m_x1 = Double.valueOf(parts[0]);

        // image position patient
        String position = mfp.getImagePosition(index);
        parts = position.split("\\\\");
        imagePosition = new Point3D(Double.valueOf(parts[0]), Double.valueOf(parts[1]), Double.valueOf(parts[2]));

        //rows and columns
        sizeX = mfp.getColumns();
        sizeY = mfp.getRows();

        // slice thickness
        sliceThickness = Float.valueOf(mfp.getSliceThickness(index));

        //pixel spacing
        String spacing = mfp.getPixelSpacing(index);
        parts = spacing.split("\\\\");
        pixelSpacingX = Float.valueOf(parts[0]);
        pixelSpacingY = Float.valueOf(parts[1]);

        // file
        this.file = file;
    }
    
    
    public ImageSlice clone() {
        ImageSlice copiedSlice = new ImageSlice(this.imageOrientationX, this.imageOrientationY, this.imagePosition, this.sizeX, this.sizeY, this.pixelSpacingX, this.pixelSpacingY, this.image, this.file);
        return copiedSlice;
    }

    public ImageSlice clone(Raster image) {
        ImageSlice copiedSlice = new ImageSlice(this.imageOrientationX, this.imageOrientationY, this.imagePosition, this.sizeX, this.sizeY, this.pixelSpacingX, this.pixelSpacingY, image, this.file);
        return copiedSlice;
    }

    public Raster getImage() {
        return image;
    }

    public int[] getImageAsInt(){
        int[] imagePixels = new int[image.getWidth()*image.getHeight()];
        int[] values = new int[image.getNumBands()];
        for (int i = 0; i < image.getWidth(); i++) {
            for (int j = 0; j < image.getHeight(); j++) {
                imagePixels[i * image.getWidth() + j] = this.image.getPixel(i, j, values)[0];
            }
        }
        return imagePixels;
    }

    public boolean hasSameDimensions(ImageSlice slice2) {
        return (this.imageOrientationX.m_x1 == slice2.imageOrientationX.m_x1
                && this.imageOrientationX.m_x2 == slice2.imageOrientationX.m_x2
                && this.imageOrientationX.m_x3 == slice2.imageOrientationX.m_x3
                && this.imageOrientationY.m_x1 == slice2.imageOrientationY.m_x1
                && this.imageOrientationY.m_x2 == slice2.imageOrientationY.m_x2
                && this.imageOrientationY.m_x3 == slice2.imageOrientationY.m_x3
                && this.pixelSpacingX == slice2.pixelSpacingX
                && this.pixelSpacingY == slice2.pixelSpacingY
                && this.sizeX == slice2.sizeX
                && this.sizeY == slice2.sizeY
                && this.image.getHeight() == slice2.image.getHeight()
                && this.image.getWidth() == slice2.image.getWidth());
    }

    public void clearMinMax() {
        maxIntensity = new double[]{Double.NEGATIVE_INFINITY, Double.NEGATIVE_INFINITY, Double.NEGATIVE_INFINITY};
        minIntensity = new double[]{Double.POSITIVE_INFINITY, Double.POSITIVE_INFINITY, Double.POSITIVE_INFINITY};
    }

    public double[] getMaxVal() {
        if (image != null) {
            if (maxIntensity[0] == Double.NEGATIVE_INFINITY) {
                int[] val = new int[3];
                for (int i = image.getMinX(); i < image.getWidth(); i++) {
                    for (int j = image.getMinY(); j < image.getHeight(); j++) {
                        image.getPixel(i, j, val);
                        if (val[0] > maxIntensity[0]) {
                            maxIntensity[0] = val[0];
                        };
                        if (val[1] > maxIntensity[1]) {
                            maxIntensity[1] = val[1];
                        };
                        if (val[2] > maxIntensity[2]) {
                            maxIntensity[2] = val[2];
                        };
                    }
                }
                return maxIntensity;
            }
            return maxIntensity;
        } else {
            return new double[]{0, 0, 0};
        }
    }

    public double[] getMinVal() {
        if (image != null) {
            if (minIntensity[0] == Double.POSITIVE_INFINITY) {
                int[] val = new int[3];
                for (int i = image.getMinX(); i < image.getWidth(); i++) {
                    for (int j = image.getMinY(); j < image.getHeight(); j++) {
                        image.getPixel(i, j, val);
                        if (val[0] < minIntensity[0]) {
                            minIntensity[0] = val[0];
                        };
                        if (val[1] < minIntensity[1]) {
                            minIntensity[1] = val[1];
                        };
                        if (val[2] < minIntensity[2]) {
                            minIntensity[2] = val[2];
                        };
                    }
                }
            }
            return minIntensity;
        } else {
            return new double[]{0, 0, 0};
        }
    }

    public ImageSlice getDifference(Raster raster) {
        WritableRaster newraster = this.image.createCompatibleWritableRaster();
        double[] val1 = {0, 0, 0};
        double[] val2 = {0, 0, 0};
        double mean1 = 0;
        double mean2 = 0;
        double var1 = 0;
        double var2 = 0;
        int n = 0;
        if (raster.getWidth() == this.image.getWidth() && raster.getHeight() == this.image.getHeight()) {
            for (int i = 0; i < this.image.getWidth(); i++) {
                for (int j = 0; j < this.image.getHeight(); j++) {
                    mean1 += image.getPixel(i, j, val1)[0];
                    mean2 += raster.getPixel(i, j, val2)[0];
                    n++;
                }
            }
            mean1 = mean1 / n;
            mean2 = mean2 / n;
            for (int i = 0; i < this.image.getWidth(); i++) {
                for (int j = 0; j < this.image.getHeight(); j++) {
                    var1 += Math.pow(image.getPixel(i, j, val1)[0] - mean1, 2);
                    var2 += Math.pow(raster.getPixel(i, j, val2)[0] - mean2, 2);
                }
            }
            var1 = var1 / n;
            var2 = var2 / n;
            double value;

            for (int i = 0; i < this.image.getWidth(); i++) {
                for (int j = 0; j < this.image.getHeight(); j++) {
//                        val1 = Image.getPixel(i, j, val1);
                    value = Math.abs(((image.getPixel(i, j, val1)[0] - mean1) * 128 / Math.sqrt(var1) + 128)
                            - ((raster.getPixel(i, j, val2)[0] - mean2) * 128 / Math.sqrt(var2) + 128));
//                        val2 = raster.getPixel(i, j, val2);
//                        newraster.setPixel(i, j, new int[]{(int) Math.abs((val1[0] - val2[0]) / 2), (int) Math.abs((val1[1] - val2[1]) / 2), (int) Math.abs((val1[2] - val2[2]) / 2)});
                    newraster.setPixel(i, j, new int[]{(int) value, (int) value, (int) value});
                }
            }
            return new ImageSlice(imageOrientationX, imageOrientationY, imagePosition, sizeX, sizeY, pixelSpacingX, pixelSpacingY, newraster, file);
        } else {
            System.out.println("Different Image Dimensions!");
            return null;
        }
    }

    public ImageSlice getCheckboard(Raster raster) {
        WritableRaster newraster = this.image.createCompatibleWritableRaster();
        double[] val = {0, 0, 0};
        double mean1 = 0;
        double mean2 = 0;
        double var1 = 0;
        double var2 = 0;
        int n = 0;
        if (raster.getWidth() == this.image.getWidth() && raster.getHeight() == this.image.getHeight()) {
            for (int i = 0; i < this.image.getWidth(); i++) {
                for (int j = 0; j < this.image.getHeight(); j++) {
                    mean1 += image.getPixel(i, j, val)[0];
                    mean2 += raster.getPixel(i, j, val)[0];
                    n++;
                }
            }
            mean1 = mean1 / n;
            mean2 = mean2 / n;
            for (int i = 0; i < this.image.getWidth(); i++) {
                for (int j = 0; j < this.image.getHeight(); j++) {
                    var1 += Math.pow(image.getPixel(i, j, val)[0] - mean1, 2);
                    var2 += Math.pow(raster.getPixel(i, j, val)[0] - mean2, 2);
                }
            }
            var1 = var1 / n;
            var2 = var2 / n;
            double value;
            for (int i = 0; i < this.image.getWidth(); i++) {
                for (int j = 0; j < this.image.getHeight(); j++) {
                    if (((Math.round((double) i / 16) + Math.round((double) j / 16)) & 1) == 0) {
                        value = Math.abs((image.getPixel(i, j, val)[0] - mean1) * 128 / Math.sqrt(var1) + 128);
                    } else {
                        value = Math.abs((raster.getPixel(i, j, val)[0] - mean2) * 128 / Math.sqrt(var2) + 128);
                    }
                    newraster.setPixel(i, j, new int[]{(int) value, (int) value, (int) value});
                }
            }
            return new ImageSlice(imageOrientationX, imageOrientationY, imagePosition, sizeX, sizeY, pixelSpacingX, pixelSpacingY, newraster, file);
        } else {
            System.out.println("Different Image Dimensions!");
            return null;
        }
    }
    public void scale(double s)
    {
        for (int i = 0; i < this.image.getWidth(); i++) {
            for (int j = 0; j < this.image.getHeight(); j++) {
                double[] val = new double[this.image.getNumBands()];
                this.image.getPixel(i, j, val);
                for (int k = 0; k < val.length; k++) {
                    val[k] *= s;
                }
                ((WritableRaster)this.image).setPixel(i, j, val);
            }
        }
        clearMinMax();
        getMaxVal();
        getMinVal();
    }

    public ImageSlice getScaledInstance(int width, int height)
    {
        WritableRaster scaledInstance = null;
        try{
            BufferedImage img = new BufferedImage(sizeX, sizeY, BufferedImage.TYPE_USHORT_GRAY);
            img.setData(image);
            Image im = img.getScaledInstance(width, height,  BufferedImage.SCALE_DEFAULT);
            BufferedImage bufferedImage = new BufferedImage(width, height, BufferedImage.TYPE_USHORT_GRAY);
            bufferedImage.getGraphics().drawImage(im, 0, 0, null);
            scaledInstance = bufferedImage.getRaster();
        }catch(Exception e){System.out.println("exception: "+e.toString());}
        return new ImageSlice(imageOrientationX, imageOrientationY, imagePosition, width, height, (pixelSpacingX * (float)sizeX / (float)width), (pixelSpacingY * (float)sizeY / (float)height), scaledInstance ,file) ;
    }

    public HashMap<Point,Integer> getPixelsInside(PrecisionPolygon plg)
    {
        int minx;
        int maxx;
        int miny;
        int maxy;
        int[] xvals;
        int[] yvals;
        Polygon pol;
        HashMap<Point,Integer> pixelMap = new HashMap<>();

        if (plg != null) {
            int precision = 10;
            pol = plg.getRescaledPolygon(precision, precision);
            xvals = pol.xpoints;
            yvals = pol.ypoints;
            minx = xvals[0];
            maxx = xvals[0];
            for (int i = 1; i < xvals.length; i++) {
                if (xvals[i] > maxx) {
                    maxx = xvals[i];
                }
                if (xvals[i] < minx) {
                    minx = xvals[i];
                }
            }
            miny = yvals[0];
            maxy = yvals[0];
            for (int i = 1; i < yvals.length; i++) {
                if (yvals[i] > maxy) {
                    maxy = yvals[i];
                }
                if (yvals[i] < miny) {
                    miny = yvals[i];
                }
            }
            for (int x = minx; x <= maxx; x++) {
                for (int y = miny; y <= maxy; y++) {
                    //System.out.print(".");
                    if (pol.contains(x, y)) {
//                        System.out.println("got inside: "+x/precision / pixelSpacingX+","+y/precision / pixelSpacingY);
                        try{
                            int[] value = new int[this.image.getNumBands()];
                            this.image.getPixel((int)((float)x / (float)precision / pixelSpacingX), (int)((float)y / (float)precision / pixelSpacingY), value);
//                            ((WritableRaster) this.image).setPixel((int)((float)x / (float)precision / pixelSpacingX), (int)((float)y / (float)precision / pixelSpacingY), new int[] {5000,5000,5000}); // To test which part of the image is being included
                            pixelMap.put(new Point((int)((float)x / (float)precision / pixelSpacingX), (int)((float)y / (float)precision / pixelSpacingY)), value[0]);
                        }catch(ArrayIndexOutOfBoundsException e){System.out.println("Not in image.");}
                    }
                }
            }
        }
        return pixelMap;
    }   
    public Orientation getOrientation(){
        Point3D normalVect = Point3D.crossProduct(this.imageOrientationX,this.imageOrientationY);
        if(Math.abs(normalVect.m_x1)>Math.abs(normalVect.m_x2)&&Math.abs(normalVect.m_x1)>Math.abs(normalVect.m_x3)){
            return Orientation.SAG;
        }else if(Math.abs(normalVect.m_x2)>Math.abs(normalVect.m_x3)){
            return Orientation.COR;
        }else{
            return Orientation.TRANS;
        }
    }
}
