/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package org.spectrim.view;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.Iterator;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 *
 * @author nunopbarros
 */
public class OverlayStack {

    private float minThresh = 0;
    private float maxThresh = 0;
    private ArrayList<OverlaySlice> images = new ArrayList<>();
    private double[] maxIntensity = new double[]{Double.NEGATIVE_INFINITY, Double.NEGATIVE_INFINITY, Double.NEGATIVE_INFINITY};
    private double[] minIntensity = new double[]{Double.POSITIVE_INFINITY, Double.POSITIVE_INFINITY, Double.POSITIVE_INFINITY};

    public void add(OverlaySlice slice) {
        if (size() == 0 || slice.hasSameDimensions(images.get(images.size() - 1))) {
            images.add(slice);
            if (slice.getMaxVal()[0] > this.maxIntensity[0]) {
                maxIntensity[0] = slice.getMaxVal()[0];
            }
            if (slice.getMinVal()[0] < this.minIntensity[0]) {
                minIntensity[0] = slice.getMinVal()[0];
            }
            if (slice.getMaxVal()[1] > this.maxIntensity[1]) {
                maxIntensity[1] = slice.getMaxVal()[1];
            }
            if (slice.getMinVal()[1] < this.minIntensity[1]) {
                minIntensity[1] = slice.getMinVal()[1];
            }
            if (slice.getMaxVal()[2] > this.maxIntensity[2]) {
                maxIntensity[2] = slice.getMaxVal()[2];
            }
            if (slice.getMinVal()[2] < this.minIntensity[2]) {
                minIntensity[2] = slice.getMinVal()[2];
            }
        } else {
            System.out.println("Slice with different dimensions! Not added to Image Stack");
        }
    }

    public ArrayList<String> getFiles() {
        HashSet<String> hashSet = new HashSet<>();
        ArrayList<String> res = new ArrayList<>(hashSet);
        for (int i = 0; i < images.size(); i++) {
            if (!hashSet.contains(images.get(i).file)) {
                hashSet.add(images.get(i).file);
                res.add(images.get(i).file);
            }
        }
        return res;
    }

    public static OverlayStack getMeanStack(ArrayList<OverlayStack> stacks) {
        OverlayStack res = new OverlayStack();
        int size = stacks.get(0).size();
        for (int j = 0; j < size; j++) {
            ArrayList<OverlaySlice> slices = new ArrayList<>();
            for (int i = 0; i < stacks.size(); i++) {
                slices.add(stacks.get(i).get(j));
            }
            res.add(OverlaySlice.getMeanSlice(slices));
        }
        return res;
    }

    public static OverlayStack getStdDevStack(ArrayList<OverlayStack> stacks) {
        OverlayStack res = new OverlayStack();
        int size = stacks.get(0).size();
        for (int j = 0; j < size; j++) {
            ArrayList<OverlaySlice> slices = new ArrayList<>();
            for (int i = 0; i < stacks.size(); i++) {
                slices.add(stacks.get(i).get(j));
            }
            res.add(OverlaySlice.getStdDevSlice(slices));
        }
        return res;
    }

    public void addAll(Collection<OverlaySlice> col) {
        Iterator<OverlaySlice> iter = col.iterator();
        while (iter.hasNext()) {
            add(iter.next());
        }
    }

    public OverlaySlice get(int nr) {
        return images.get(nr);
    }

    public void set(int nr, OverlaySlice slice) {
        images.set(nr, slice);
    }

    public int[] getDimensions() {
        return new int[]{images.get(0).sizeX, images.get(0).sizeY, images.size()};
    }

    public void clear() {
        images.clear();
        maxIntensity = new double[]{Double.NEGATIVE_INFINITY, Double.NEGATIVE_INFINITY, Double.NEGATIVE_INFINITY};
        minIntensity = new double[]{Double.POSITIVE_INFINITY, Double.POSITIVE_INFINITY, Double.POSITIVE_INFINITY};
    }

    public int size() {
        return images.size();
    }

    public double[] getMaxIntensity() {
        return maxIntensity;
    }

    public double[] getMinIntensity() {
        return minIntensity;
    }

    public void setMinThreshold(float val, boolean update) {
        minThresh = val;
        if (update) {
            updateImages();
        }
    }

    public void setMaxThreshold(float val, boolean update) {
        maxThresh = val;
        if (update) {
            updateImages();
        }
    }

    public void updateImages() {
        for (int i = 0; i < this.size(); i++) {
            this.get(i).updateImage(minThresh, maxThresh, OverlaySlice.colorMode.warm);
        }
    }
}
