/*
 * File: $Id: XYPlotFrame.java,v 1.1.1.1 2006/04/19 09:38:51 hans Exp $
 * History
 * Date: 01.04.2003, 17:09 Hans Slotboom  Changes:
 *
 * Copyright (C) 2003 by
 * 
 * Inselspital - Department of Neuroradiology
 * CH 3010 Bern Switzerland
 */

/*
 * XYPlotFrame.java
 *
 * Displays an XYPlot object within a Frame
 *
 * Created on April 18, 2005, 11:20 AM
 */

package org.spectrim.xyplot;

import java.awt.Dimension;

import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.JFrame;
import javax.swing.BoxLayout;


/**
 * 
 * Simple XYplot drawn within most simple JFrame.
 * 
 * @author  hans
 */
public class XYPlotFrame extends JFrame
{
    
    /** 
     * Creates a new instance of XYPlotFrame. 
     * @param title
     * @param parent
     */
    public XYPlotFrame( String title, JFrame parent )
    {
        m_parent = parent;
        
        m_plotArea = new XYPlot( title, m_parent );
        
        this.setMinimumSize( new Dimension( 256, 256 ) );
        this.setPreferredSize( new Dimension( 256, 256 ) );
        
        setLayout( new BoxLayout( this.getContentPane( ), BoxLayout.Y_AXIS ) );
        
        this.setContentPane( m_plotArea );
        
        addWindowListener( 
            new WindowAdapter( )
            {
                @Override
                public void windowClosing( WindowEvent event )
                {
                    dispose( );
                }
            }
        );
        
        this.pack( );
        setVisible( true );
    }
    
    
    /**
     * Defines whether the plot should be displayed x-reversed
     * @param displayReversed 
     */
    public void setDisplayXReversed( boolean displayReversed )
    {
        this.m_plotArea.setDisplayXReversed( displayReversed );
    }
    
    
    /**
     * Adds a signal to display to the XYPlot object and displays it.
     * @param cur
     * @param refresh
     */
    public void addCurve( Curve cur, boolean refresh )
    {
        m_plotArea.addCurve( cur, refresh );
    }
    
    public void deleteAllCurves( )
    {
        m_plotArea.deleteAllCurves( );
    }
    
    /**
     * Returns a reference to the plot area.
     * @return 
     */
    public XYPlot getPlotArea( )
    {
        return m_plotArea;
    }
    
    
    // Attributes of XYPlotFrame
    private final JFrame m_parent;
    private final XYPlot m_plotArea;
    
    // For proper serialization
    private static final long serialVersionUID = 1482454591289213L;
}
